@@ -168,14 +168,14 @@ inline void Structure::transitionTableAdd(const StructureTransitionTableHash::Ke
     if (!specificValue) {
         TransitionTable::iterator find = transitionTable()->find(key);
         if (find == transitionTable()->end())
-            transitionTable()->add(key, Transition(structure, 0));
+            transitionTable()->add(key, Transition(structure, static_cast<Structure*>(0)));
         else
             find->second.first = structure;
     } else {
         // If we're adding a transition to a specific value, then there cannot be
         // an existing transition
         ASSERT(!transitionTable()->contains(key));
-        transitionTable()->add(key, Transition(0, structure));
+        transitionTable()->add(key, Transition(static_cast<Structure*>(0), structure));
     }
 }
 
@@ -233,6 +233,7 @@ Structure::Structure(JSValue prototype, const TypeInfo& typeInfo, unsigned anony
     , m_dictionaryKind(NoneDictionaryKind)
     , m_isPinnedPropertyTable(false)
     , m_hasGetterSetterProperties(false)
+    , m_hasNonEnumerableProperties(false)
     , m_attributesInPrevious(0)
     , m_specificFunctionThrashCount(0)
     , m_anonymousSlotCount(anonymousSlotCount)
@@ -270,7 +271,7 @@ Structure::~Structure()
     if (m_propertyTable) {
         unsigned entryCount = m_propertyTable->keyCount + m_propertyTable->deletedSentinelCount;
         for (unsigned i = 1; i <= entryCount; i++) {
-            if (UString::Rep* key = m_propertyTable->entries()[i].key)
+            if (StringImpl* key = m_propertyTable->entries()[i].key)
                 key->deref();
         }
 
@@ -395,7 +396,7 @@ void Structure::growPropertyStorageCapacity()
 
 void Structure::despecifyDictionaryFunction(const Identifier& propertyName)
 {
-    const UString::Rep* rep = propertyName._ustring.rep();
+    const StringImpl* rep = propertyName.impl();
 
     materializePropertyMapIfNecessary();
 
@@ -444,7 +445,7 @@ PassRefPtr<Structure> Structure::addPropertyTransitionToExistingStructure(Struct
     ASSERT(!structure->isDictionary());
     ASSERT(structure->typeInfo().type() == ObjectType);
 
-    if (Structure* existingTransition = structure->transitionTableGet(make_pair(propertyName.ustring().rep(), attributes), specificValue)) {
+    if (Structure* existingTransition = structure->transitionTableGet(make_pair(propertyName.impl(), attributes), specificValue)) {
         ASSERT(existingTransition->m_offset != noOffset);
         offset = existingTransition->m_offset + existingTransition->m_anonymousSlotCount;
         ASSERT(offset >= structure->m_anonymousSlotCount);
@@ -479,7 +480,7 @@ PassRefPtr<Structure> Structure::addPropertyTransition(Structure* structure, con
 
     transition->m_cachedPrototypeChain = structure->m_cachedPrototypeChain;
     transition->m_previous = structure;
-    transition->m_nameInPrevious = propertyName.ustring().rep();
+    transition->m_nameInPrevious = propertyName.impl();
     transition->m_attributesInPrevious = attributes;
     transition->m_specificValueInPrevious = specificValue;
     transition->m_propertyStorageCapacity = structure->m_propertyStorageCapacity;
@@ -509,7 +510,7 @@ PassRefPtr<Structure> Structure::addPropertyTransition(Structure* structure, con
 
     transition->m_offset = offset - structure->m_anonymousSlotCount;
     ASSERT(structure->anonymousSlotCount() == transition->anonymousSlotCount());
-    structure->transitionTableAdd(make_pair(propertyName.ustring().rep(), attributes), transition.get(), specificValue);
+    structure->transitionTableAdd(make_pair(propertyName.impl(), attributes), transition.get(), specificValue);
     return transition.release();
 }
 
@@ -738,7 +739,7 @@ PropertyMapHashTable* Structure::copyPropertyTable()
 
     unsigned entryCount = m_propertyTable->keyCount + m_propertyTable->deletedSentinelCount;
     for (unsigned i = 1; i <= entryCount; ++i) {
-        if (UString::Rep* key = newTable->entries()[i].key)
+        if (StringImpl* key = newTable->entries()[i].key)
             key->ref();
     }
 
@@ -749,7 +750,7 @@ PropertyMapHashTable* Structure::copyPropertyTable()
     return newTable;
 }
 
-size_t Structure::get(const UString::Rep* rep, unsigned& attributes, JSCell*& specificValue)
+size_t Structure::get(const StringImpl* rep, unsigned& attributes, JSCell*& specificValue)
 {
     materializePropertyMapIfNecessary();
     if (!m_propertyTable)
@@ -806,7 +807,7 @@ bool Structure::despecifyFunction(const Identifier& propertyName)
     if (!m_propertyTable)
         return false;
 
-    UString::Rep* rep = propertyName._ustring.rep();
+    StringImpl* rep = propertyName.impl();
 
     unsigned i = rep->existingHash();
 
@@ -870,7 +871,7 @@ size_t Structure::put(const Identifier& propertyName, unsigned attributes, JSCel
     if (attributes & DontEnum)
         m_hasNonEnumerableProperties = true;
 
-    UString::Rep* rep = propertyName._ustring.rep();
+    StringImpl* rep = propertyName.impl();
 
     if (!m_propertyTable)
         createPropertyMapHashTable();
@@ -954,7 +955,7 @@ size_t Structure::put(const Identifier& propertyName, unsigned attributes, JSCel
     return newOffset;
 }
 
-bool Structure::hasTransition(UString::Rep* rep, unsigned attributes)
+bool Structure::hasTransition(StringImpl* rep, unsigned attributes)
 {
     return transitionTableHasTransition(make_pair(rep, attributes));
 }
@@ -965,7 +966,7 @@ size_t Structure::remove(const Identifier& propertyName)
 
     checkConsistency();
 
-    UString::Rep* rep = propertyName._ustring.rep();
+    StringImpl* rep = propertyName.impl();
 
     if (!m_propertyTable)
         return notFound;
@@ -979,7 +980,7 @@ size_t Structure::remove(const Identifier& propertyName)
     unsigned i = rep->existingHash();
     unsigned k = 0;
     unsigned entryIndex;
-    UString::Rep* key = 0;
+    StringImpl* key = 0;
     while (1) {
         entryIndex = m_propertyTable->entryIndices[i & m_propertyTable->sizeMask];
         if (entryIndex == emptyEntryIndex)
@@ -1241,7 +1242,7 @@ void Structure::checkConsistency()
     unsigned nonEmptyEntryCount = 0;
     for (unsigned c = 1; c <= m_propertyTable->keyCount + m_propertyTable->deletedSentinelCount; ++c) {
         ASSERT(m_hasNonEnumerableProperties || !(m_propertyTable->entries()[c].attributes & DontEnum));
-        UString::Rep* rep = m_propertyTable->entries()[c].key;
+        StringImpl* rep = m_propertyTable->entries()[c].key;
         ASSERT(m_propertyTable->entries()[c].offset >= m_anonymousSlotCount);
         if (!rep)
             continue;
