@@ -38,8 +38,9 @@
 #include "AccessibilityListBoxOption.h"
 #include "AccessibilityMediaControls.h"
 #include "AccessibilityMenuList.h"
-#include "AccessibilityMenuListPopup.h"
 #include "AccessibilityMenuListOption.h"
+#include "AccessibilityMenuListPopup.h"
+#include "AccessibilityProgressIndicator.h"
 #include "AccessibilityRenderObject.h"
 #include "AccessibilityScrollbar.h"
 #include "AccessibilitySlider.h"
@@ -58,7 +59,13 @@
 #endif
 #include "InputElement.h"
 #include "Page.h"
-#include "RenderObject.h"
+#include "RenderListBox.h"
+#include "RenderMenuList.h"
+#include "RenderTable.h"
+#include "RenderTableCell.h"
+#include "RenderTableRow.h"
+#include "RenderProgress.h"
+#include "RenderSlider.h"
 #include "RenderView.h"
 
 #include <wtf/PassRefPtr.h>
@@ -158,74 +165,87 @@ AccessibilityObject* AXObjectCache::get(RenderObject* renderer)
     
     return obj;
 }
-    
-bool AXObjectCache::nodeIsAriaType(Node* node, String role)
+
+// FIXME: This probably belongs on Node.
+// FIXME: This should take a const char*, but one caller passes nullAtom.
+bool nodeHasRole(Node* node, const String& role)
 {
     if (!node || !node->isElementNode())
         return false;
-    
+
     return equalIgnoringCase(static_cast<Element*>(node)->getAttribute(roleAttr), role);
 }
 
+static PassRefPtr<AccessibilityObject> createFromRenderer(RenderObject* renderer)
+{
+    // FIXME: How could renderer->node() ever not be an Element?
+    Node* node = renderer->node();
+
+    // If the node is aria role="list" or the aria role is empty and its a
+    // ul/ol/dl type (it shouldn't be a list if aria says otherwise).
+    if (node && ((nodeHasRole(node, "list") || nodeHasRole(node, "directory"))
+                      || (nodeHasRole(node, nullAtom) && (node->hasTagName(ulTag) || node->hasTagName(olTag) || node->hasTagName(dlTag)))))
+        return AccessibilityList::create(renderer);
+
+    // aria tables
+    if (nodeHasRole(node, "grid") || nodeHasRole(node, "treegrid"))
+        return AccessibilityARIAGrid::create(renderer);
+    if (nodeHasRole(node, "row"))
+        return AccessibilityARIAGridRow::create(renderer);
+    if (nodeHasRole(node, "gridcell") || nodeHasRole(node, "columnheader") || nodeHasRole(node, "rowheader"))
+        return AccessibilityARIAGridCell::create(renderer);
+
+#if ENABLE(VIDEO)
+    // media controls
+    if (node && node->isMediaControlElement())
+        return AccessibilityMediaControl::create(renderer);
+#endif
+
+    if (renderer->isBoxModelObject()) {
+        RenderBoxModelObject* cssBox = toRenderBoxModelObject(renderer);
+        if (cssBox->isListBox())
+            return AccessibilityListBox::create(toRenderListBox(cssBox));
+        if (cssBox->isMenuList())
+            return AccessibilityMenuList::create(toRenderMenuList(cssBox));
+
+        // standard tables
+        if (cssBox->isTable())
+            return AccessibilityTable::create(toRenderTable(cssBox));
+        if (cssBox->isTableRow())
+            return AccessibilityTableRow::create(toRenderTableRow(cssBox));
+        if (cssBox->isTableCell())
+            return AccessibilityTableCell::create(toRenderTableCell(cssBox));
+
+#if ENABLE(PROGRESS_TAG)
+        // progress bar
+        if (cssBox->isProgress())
+            return AccessibilityProgressIndicator::create(toRenderProgress(cssBox));
+#endif
+
+        // input type=range
+        if (cssBox->isSlider())
+            return AccessibilitySlider::create(toRenderSlider(cssBox));
+    }
+
+    return AccessibilityRenderObject::create(renderer);
+}
+
 AccessibilityObject* AXObjectCache::getOrCreate(RenderObject* renderer)
 {
     if (!renderer)
         return 0;
-    
-    AccessibilityObject* obj = get(renderer);
 
-    if (!obj) {
-        Node* node = renderer->node();
-        RefPtr<AccessibilityObject> newObj = 0;
-        if (renderer->isListBox())
-            newObj = AccessibilityListBox::create(renderer);
-        else if (renderer->isMenuList())
-            newObj = AccessibilityMenuList::create(renderer);
+    if (AccessibilityObject* obj = get(renderer))
+        return obj;
 
-        // If the node is aria role="list" or the aria role is empty and its a ul/ol/dl type (it shouldn't be a list if aria says otherwise). 
-        else if (node && ((nodeIsAriaType(node, "list") || nodeIsAriaType(node, "directory"))
-                          || (nodeIsAriaType(node, nullAtom) && (node->hasTagName(ulTag) || node->hasTagName(olTag) || node->hasTagName(dlTag)))))
-            newObj = AccessibilityList::create(renderer);
-        
-        // aria tables
-        else if (nodeIsAriaType(node, "grid") || nodeIsAriaType(node, "treegrid"))
-            newObj = AccessibilityARIAGrid::create(renderer);
-        else if (nodeIsAriaType(node, "row"))
-            newObj = AccessibilityARIAGridRow::create(renderer);
-        else if (nodeIsAriaType(node, "gridcell") || nodeIsAriaType(node, "columnheader") || nodeIsAriaType(node, "rowheader"))
-            newObj = AccessibilityARIAGridCell::create(renderer);
+    RefPtr<AccessibilityObject> newObj = createFromRenderer(renderer);
 
-        // standard tables
-        else if (renderer->isTable())
-            newObj = AccessibilityTable::create(renderer);
-        else if (renderer->isTableRow())
-            newObj = AccessibilityTableRow::create(renderer);
-        else if (renderer->isTableCell())
-            newObj = AccessibilityTableCell::create(renderer);
+    getAXID(newObj.get());
 
-#if ENABLE(VIDEO)
-        // media controls
-        else if (renderer->node() && renderer->node()->isMediaControlElement())
-            newObj = AccessibilityMediaControl::create(renderer);
-#endif
-
-        // input type=range
-        else if (renderer->isSlider())
-            newObj = AccessibilitySlider::create(renderer);
-
-        else
-            newObj = AccessibilityRenderObject::create(renderer);
-        
-        obj = newObj.get();
-        
-        getAXID(obj);
-        
-        m_renderObjectMapping.set(renderer, obj->axObjectID());
-        m_objects.set(obj->axObjectID(), obj);    
-        attachWrapper(obj);
-    }
-    
-    return obj;
+    m_renderObjectMapping.set(renderer, newObj->axObjectID());
+    m_objects.set(newObj->axObjectID(), newObj);
+    attachWrapper(newObj.get());
+    return newObj.get();
 }
 
 AccessibilityObject* AXObjectCache::getOrCreate(AccessibilityRole role)
@@ -303,7 +323,7 @@ void AXObjectCache::remove(RenderObject* renderer)
     m_renderObjectMapping.remove(renderer);
 }
 
-#if !PLATFORM(WIN)
+#if !PLATFORM(WIN) || OS(WINCE)
 AXID AXObjectCache::platformGenerateAXID() const
 {
     static AXID lastUsedID = 0;
@@ -441,11 +461,32 @@ void AXObjectCache::postNotification(AccessibilityObject* object, Document* docu
 
 void AXObjectCache::selectedChildrenChanged(RenderObject* renderer)
 {
-    postNotification(renderer, AXSelectedChildrenChanged, true);
+    // postToElement is false so that you can pass in any child of an element and it will go up the parent tree
+    // to find the container which should send out the notification.
+    postNotification(renderer, AXSelectedChildrenChanged, false);
+}
+
+void AXObjectCache::nodeTextChangeNotification(RenderObject* renderer, AXTextChange textChange, unsigned offset, unsigned count)
+{
+    if (!renderer)
+        return;
+
+    // Delegate on the right platform
+    AccessibilityObject* obj = getOrCreate(renderer);
+    nodeTextChangePlatformNotification(obj, textChange, offset, count);
 }
 #endif
 
 #if HAVE(ACCESSIBILITY)
+void AXObjectCache::handleAriaExpandedChange(RenderObject *renderer)
+{
+    if (!renderer)
+        return;
+    AccessibilityObject* obj = getOrCreate(renderer);
+    if (obj)
+        obj->handleAriaExpandedChanged();
+}
+    
 void AXObjectCache::handleActiveDescendantChanged(RenderObject* renderer)
 {
     if (!renderer)
@@ -464,9 +505,12 @@ void AXObjectCache::handleAriaRoleChanged(RenderObject* renderer)
         static_cast<AccessibilityRenderObject*>(obj)->updateAccessibilityRole();
 }
 #endif
-    
+
 VisiblePosition AXObjectCache::visiblePositionForTextMarkerData(TextMarkerData& textMarkerData)
 {
+    if (!isNodeInUse(textMarkerData.node))
+        return VisiblePosition();
+    
     VisiblePosition visiblePos = VisiblePosition(textMarkerData.node, textMarkerData.offset, textMarkerData.affinity);
     Position deepPos = visiblePos.deepEquivalent();
     if (deepPos.isNull())
@@ -518,7 +562,9 @@ void AXObjectCache::textMarkerDataForVisiblePosition(TextMarkerData& textMarkerD
     textMarkerData.axID = obj.get()->axObjectID();
     textMarkerData.node = domNode;
     textMarkerData.offset = deepPos.deprecatedEditingOffset();
-    textMarkerData.affinity = visiblePos.affinity();    
+    textMarkerData.affinity = visiblePos.affinity();   
+    
+    cache->setNodeInUse(domNode);
 }
     
 } // namespace WebCore
