@@ -24,7 +24,6 @@
 #include "debugger/DebuggerCallFrame.h"
 
 #include "ActiveDOMObject.h"
-#include "CSSHelper.h"
 #include "DOMCoreException.h"
 #include "DOMObjectHashTableMap.h"
 #include "Document.h"
@@ -36,18 +35,24 @@
 #include "HTMLCanvasElement.h"
 #include "HTMLFrameElementBase.h"
 #include "HTMLImageElement.h"
+#include "HTMLLinkElement.h"
 #include "HTMLNames.h"
 #include "HTMLScriptElement.h"
+#include "HTMLStyleElement.h"
+#include "JSBinding.h"
+#include "JSBindingState.h"
 #include "JSDOMCoreException.h"
 #include "JSDOMWindowCustom.h"
 #include "JSDebugWrapperSet.h"
 #include "JSEventException.h"
 #include "JSExceptionBase.h"
+#include "JSMainThreadExecState.h"
 #include "JSNode.h"
 #include "JSRangeException.h"
 #include "JSXMLHttpRequestException.h"
 #include "KURL.h"
 #include "MessagePort.h"
+#include "ProcessingInstruction.h"
 #include "RangeException.h"
 #include "ScriptCachedFrameData.h"
 #include "ScriptController.h"
@@ -71,6 +76,21 @@
 #include "XPathException.h"
 #endif
 
+#if ENABLE(DATABASE)
+#include "JSSQLException.h"
+#include "SQLException.h"
+#endif
+
+#if ENABLE(BLOB) || ENABLE(FILE_SYSTEM)
+#include "FileException.h"
+#include "JSFileException.h"
+#endif
+
+#if ENABLE(INDEXED_DATABASE)
+#include "IDBDatabaseException.h"
+#include "JSIDBDatabaseException.h"
+#endif
+
 using namespace JSC;
 
 namespace WebCore {
@@ -217,20 +237,9 @@ void cacheDOMNodeWrapper(JSC::ExecState* exec, Document* document, Node* node, J
         node->setWrapper(wrapper);
 }
 
-void forgetAllDOMNodesForDocument(Document* document)
-{
-    ASSERT(document);
-    JSWrapperCacheMap& wrapperCacheMap = document->wrapperCacheMap();
-    JSWrapperCacheMap::const_iterator wrappersMapEnd = wrapperCacheMap.end();
-    for (JSWrapperCacheMap::const_iterator wrappersMapIter = wrapperCacheMap.begin(); wrappersMapIter != wrappersMapEnd; ++wrappersMapIter) {
-        delete wrappersMapIter->second;
-        wrappersMapIter->first->forgetDocument(document);
-    }
-}
-
 static inline bool isObservableThroughDOM(JSNode* jsNode, DOMWrapperWorld* world)
 {
-    // Certain conditions implicitly make a JS DOM node wrapper observable
+    // Certain conditions implicitly make existence of a JS DOM node wrapper observable
     // through the DOM, even if no explicit reference to it remains.
 
     Node* node = jsNode->impl();
@@ -252,19 +261,22 @@ static inline bool isObservableThroughDOM(JSNode* jsNode, DOMWrapperWorld* world
         // those objects through the DOM must reflect those properties.
         // FIXME: It would be better if this logic could be in the node next to
         // the custom markChildren functions rather than here.
+        // Note that for some compound objects like stylesheets and CSSStyleDeclarations,
+        // we don't descend to check children for custom properties, and just conservatively
+        // keep the node wrappers protecting them alive.
         if (node->isElementNode()) {
             if (NamedNodeMap* attributes = static_cast<Element*>(node)->attributeMap()) {
                 if (DOMObject* wrapper = world->m_wrappers.uncheckedGet(attributes)) {
+                    // FIXME: This check seems insufficient, because NamedNodeMap items can have custom properties themselves.
+                    // Maybe it would be OK to just keep the wrapper alive, as it is done for CSSOM objects below.
                     if (wrapper->hasCustomProperties())
                         return true;
                 }
             }
             if (node->isStyledElement()) {
                 if (CSSMutableStyleDeclaration* style = static_cast<StyledElement*>(node)->inlineStyleDecl()) {
-                    if (DOMObject* wrapper = world->m_wrappers.uncheckedGet(style)) {
-                        if (wrapper->hasCustomProperties())
-                            return true;
-                    }
+                    if (world->m_wrappers.uncheckedGet(style))
+                        return true;
                 }
             }
             if (static_cast<Element*>(node)->hasTagName(canvasTag)) {
@@ -274,6 +286,21 @@ static inline bool isObservableThroughDOM(JSNode* jsNode, DOMWrapperWorld* world
                             return true;
                     }
                 }
+            } else if (static_cast<Element*>(node)->hasTagName(linkTag)) {
+                if (StyleSheet* sheet = static_cast<HTMLLinkElement*>(node)->sheet()) {
+                    if (world->m_wrappers.uncheckedGet(sheet))
+                        return true;
+                }
+            } else if (static_cast<Element*>(node)->hasTagName(styleTag)) {
+                if (StyleSheet* sheet = static_cast<HTMLStyleElement*>(node)->sheet()) {
+                    if (world->m_wrappers.uncheckedGet(sheet))
+                        return true;
+                }
+            }
+        } else if (node->nodeType() == Node::PROCESSING_INSTRUCTION_NODE) {
+            if (StyleSheet* sheet = static_cast<ProcessingInstruction*>(node)->sheet()) {
+                if (world->m_wrappers.uncheckedGet(sheet))
+                    return true;
             }
         }
     } else {
@@ -441,7 +468,7 @@ JSValue jsStringSlowCase(ExecState* exec, JSStringCache& stringCache, StringImpl
     if (JSString* wrapper = stringCache.uncheckedGet(stringImpl))
         stringCache.uncheckedRemove(stringImpl, wrapper);
 
-    JSString* wrapper = jsStringWithFinalizer(exec, stringImpl->ustring(), stringWrapperDestroyed, stringImpl);
+    JSString* wrapper = jsStringWithFinalizer(exec, UString(stringImpl), stringWrapperDestroyed, stringImpl);
     stringCache.set(stringImpl, wrapper);
     // ref explicitly instead of using a RefPtr-keyed hashtable because the wrapper can
     // outlive the cache, so the stringImpl has to match the wrapper's lifetime.
@@ -456,6 +483,13 @@ JSValue jsStringOrNull(ExecState* exec, const String& s)
     return jsString(exec, s);
 }
 
+JSValue jsOwnedStringOrNull(ExecState* exec, const String& s)
+{
+    if (s.isNull())
+        return jsNull();
+    return jsOwnedString(exec, stringToUString(s));
+}
+
 JSValue jsOwnedStringOrNull(ExecState* exec, const UString& s)
 {
     if (s.isNull())
@@ -503,18 +537,27 @@ JSValue jsStringOrFalse(ExecState* exec, const KURL& url)
     return jsString(exec, url.string());
 }
 
-UString valueToStringWithNullCheck(ExecState* exec, JSValue value)
+AtomicStringImpl* findAtomicString(const Identifier& identifier)
 {
-    if (value.isNull())
-        return UString();
-    return value.toString(exec);
+    if (identifier.isNull())
+        return 0;
+    StringImpl* impl = identifier.impl();
+    ASSERT(impl->existingHash());
+    return AtomicString::find(impl->characters(), impl->length(), impl->existingHash());
 }
 
-UString valueToStringWithUndefinedOrNullCheck(ExecState* exec, JSValue value)
+String valueToStringWithNullCheck(ExecState* exec, JSValue value)
+{
+    if (value.isNull())
+        return String();
+    return ustringToString(value.toString(exec));
+}
+
+String valueToStringWithUndefinedOrNullCheck(ExecState* exec, JSValue value)
 {
     if (value.isUndefinedOrNull())
-        return UString();
-    return value.toString(exec);
+        return String();
+    return ustringToString(value.toString(exec));
 }
 
 JSValue jsDateOrNull(ExecState* exec, double value)
@@ -535,6 +578,9 @@ double valueToDate(ExecState* exec, JSValue value)
 
 void reportException(ExecState* exec, JSValue exception)
 {
+    if (exception.isObject() && asObject(exception)->exceptionType() == Terminated)
+        return;
+
     UString errorMessage = exception.toString(exec);
     JSObject* exceptionObject = exception.toObject(exec);
     int lineNumber = exceptionObject->get(exec, Identifier(exec, "line")).toInt32(exec);
@@ -542,7 +588,7 @@ void reportException(ExecState* exec, JSValue exception)
     exec->clearException();
 
     if (ExceptionBase* exceptionBase = toExceptionBase(exception))
-        errorMessage = exceptionBase->message() + ": "  + exceptionBase->description();
+        errorMessage = stringToUString(exceptionBase->message() + ": "  + exceptionBase->description());
 
     ScriptExecutionContext* scriptExecutionContext = static_cast<JSDOMGlobalObject*>(exec->lexicalGlobalObject())->scriptExecutionContext();
     ASSERT(scriptExecutionContext);
@@ -552,7 +598,7 @@ void reportException(ExecState* exec, JSValue exception)
     if (!scriptExecutionContext)
         return;
 
-    scriptExecutionContext->reportException(errorMessage, lineNumber, exceptionSourceURL);
+    scriptExecutionContext->reportException(ustringToString(errorMessage), lineNumber, ustringToString(exceptionSourceURL));
 }
 
 void reportCurrentException(ExecState* exec)
@@ -591,18 +637,33 @@ void setDOMException(ExecState* exec, ExceptionCode ec)
             break;
 #if ENABLE(SVG)
         case SVGExceptionType:
-            errorObject = toJS(exec, globalObject, SVGException::create(description).get(), 0 /* no context on purpose */);
+            errorObject = toJS(exec, globalObject, SVGException::create(description).get());
             break;
 #endif
 #if ENABLE(XPATH)
         case XPathExceptionType:
             errorObject = toJS(exec, globalObject, XPathException::create(description));
             break;
+#endif
+#if ENABLE(DATABASE)
+        case SQLExceptionType:
+            errorObject = toJS(exec, globalObject, SQLException::create(description));
+            break;
+#endif
+#if ENABLE(BLOB) || ENABLE(FILE_SYSTEM)
+        case FileExceptionType:
+            errorObject = toJS(exec, globalObject, FileException::create(description));
+            break;
+#endif
+#if ENABLE(INDEXED_DATABASE)
+        case IDBDatabaseExceptionType:
+            errorObject = toJS(exec, globalObject, IDBDatabaseException::create(description));
+            break;
 #endif
     }
 
     ASSERT(errorObject);
-    exec->setException(errorObject);
+    throwError(exec, errorObject);
 }
 
 bool checkNodeSecurity(ExecState* exec, Node* node)
@@ -628,13 +689,13 @@ bool allowsAccessFromFrame(ExecState* exec, Frame* frame, String& message)
 
 bool shouldAllowNavigation(ExecState* exec, Frame* frame)
 {
-    Frame* lexicalFrame = toLexicalFrame(exec);
-    return lexicalFrame && lexicalFrame->loader()->shouldAllowNavigation(frame);
+    JSBindingState state(exec);
+    return JSBindingSecurity::shouldAllowNavigation(&state, frame);
 }
 
 bool allowSettingSrcToJavascriptURL(ExecState* exec, Element* element, const String& name, const String& value)
 {
-    if ((element->hasTagName(iframeTag) || element->hasTagName(frameTag)) && equalIgnoringCase(name, "src") && protocolIsJavaScript(deprecatedParseURL(value))) {
+    if ((element->hasTagName(iframeTag) || element->hasTagName(frameTag)) && equalIgnoringCase(name, "src") && protocolIsJavaScript(stripLeadingAndTrailingHTMLSpaces(value))) {
           Document* contentDocument = static_cast<HTMLFrameElementBase*>(element)->contentDocument();
           if (contentDocument && !checkNodeSecurity(exec, contentDocument))
               return false;
@@ -646,46 +707,33 @@ void printErrorMessageForFrame(Frame* frame, const String& message)
 {
     if (!frame)
         return;
-    if (message.isEmpty())
-        return;
-
-    Settings* settings = frame->settings();
-    if (!settings)
-        return;
-    if (settings->privateBrowsingEnabled())
-        return;
-
-    frame->domWindow()->console()->addMessage(JSMessageSource, LogMessageType, ErrorMessageLevel, message, 1, String()); // FIXME: provide a real line number and source URL.
+    frame->domWindow()->printErrorMessage(message);
 }
 
 Frame* toLexicalFrame(ExecState* exec)
 {
-    return asJSDOMWindow(exec->lexicalGlobalObject())->impl()->frame();
+    return JSBindingState(exec).getActiveFrame();
 }
 
 Frame* toDynamicFrame(ExecState* exec)
 {
-    return asJSDOMWindow(exec->dynamicGlobalObject())->impl()->frame();
+    return JSBindingState(exec).getFirstFrame();
 }
 
-bool processingUserGesture(ExecState* exec)
+bool processingUserGesture()
 {
-    Frame* frame = toDynamicFrame(exec);
-    return frame && frame->script()->processingUserGesture(currentWorld(exec));
+    return JSBindingState(JSMainThreadExecState::currentState()).processingUserGesture();
 }
 
 KURL completeURL(ExecState* exec, const String& relativeURL)
 {
-    // For historical reasons, we need to complete the URL using the dynamic frame.
-    Frame* frame = toDynamicFrame(exec);
-    if (!frame)
-        return KURL();
-    return frame->loader()->completeURL(relativeURL);
+    JSBindingState state(exec);
+    return completeURL(&state, relativeURL);
 }
 
 JSValue objectToStringFunctionGetter(ExecState* exec, JSValue, const Identifier& propertyName)
 {
-    return new (exec) NativeFunctionWrapper(exec, exec->lexicalGlobalObject()->prototypeFunctionStructure(), 0, propertyName, objectProtoFuncToString);
+    return new (exec) NativeFunctionWrapper(exec, exec->lexicalGlobalObject(), exec->lexicalGlobalObject()->prototypeFunctionStructure(), 0, propertyName, objectProtoFuncToString);
 }
 
 Structure* getCachedDOMStructure(JSDOMGlobalObject* globalObject, const ClassInfo* classInfo)
@@ -728,7 +776,7 @@ JSC::JSObject* toJSSequence(ExecState* exec, JSValue value, unsigned& length)
 {
     JSObject* object = value.getObject();
     if (!object) {
-        throwError(exec, TypeError);
+        throwTypeError(exec);
         return 0;
     }
     JSValue lengthValue = object->get(exec, exec->propertyNames().length);
@@ -736,7 +784,7 @@ JSC::JSObject* toJSSequence(ExecState* exec, JSValue value, unsigned& length)
         return 0;
 
     if (lengthValue.isUndefinedOrNull()) {
-        throwError(exec, TypeError);
+        throwTypeError(exec);
         return 0;
     }
 
