@@ -25,11 +25,11 @@
 #include "JSDOMGlobalObject.h"
 #include "JSDOMWrapper.h"
 #include "DOMWrapperWorld.h"
-#include "JSSVGContextCache.h"
 #include "Document.h"
 #include <runtime/Completion.h>
 #include <runtime/Lookup.h>
 #include <runtime/WeakGCMap.h>
+#include <wtf/Forward.h>
 #include <wtf/Noncopyable.h>
 
 namespace JSC {
@@ -44,21 +44,19 @@ namespace WebCore {
     class JSNode;
     class KURL;
     class Node;
-    class String;
     class ScriptController;
     class ScriptCachedFrameData;
 
     typedef int ExceptionCode;
 
-#if ENABLE(SVG)
-    class SVGElement;
-#endif
-
     // FIXME: This class should collapse into DOMObject once all DOMObjects are
     // updated to store a globalObject pointer.
     class DOMObjectWithGlobalPointer : public DOMObject {
     public:
-        JSDOMGlobalObject* globalObject() const { return static_cast<JSDOMGlobalObject*>(getAnonymousValue(GlobalObjectSlot).asCell()); }
+        JSDOMGlobalObject* globalObject() const
+        {
+            return static_cast<JSDOMGlobalObject*>(DOMObject::globalObject());
+        }
 
         ScriptExecutionContext* scriptExecutionContext() const
         {
@@ -72,19 +70,14 @@ namespace WebCore {
         }
 
     protected:
-        static const unsigned AnonymousSlotCount = 1 + DOMObject::AnonymousSlotCount;
-        static const unsigned GlobalObjectSlot = AnonymousSlotCount - 1;
-
         DOMObjectWithGlobalPointer(NonNullPassRefPtr<JSC::Structure> structure, JSDOMGlobalObject* globalObject)
-            : DOMObject(structure)
+            : DOMObject(globalObject, structure)
         {
             // FIXME: This ASSERT is valid, but fires in fast/dom/gc-6.html when trying to create
             // new JavaScript objects on detached windows due to DOMWindow::document()
             // needing to reach through the frame to get to the Document*.  See bug 27640.
             // ASSERT(globalObject->scriptExecutionContext());
-            putAnonymousValue(GlobalObjectSlot, globalObject);
         }
-        virtual ~DOMObjectWithGlobalPointer() { }
     };
 
     // Base class for all constructor objects in the JSC bindings.
@@ -128,7 +121,6 @@ namespace WebCore {
 
     JSNode* getCachedDOMNodeWrapper(JSC::ExecState*, Document*, Node*);
     void cacheDOMNodeWrapper(JSC::ExecState*, Document*, Node*, JSNode* wrapper);
-    void forgetAllDOMNodesForDocument(Document*);
     void updateDOMNodeDocument(Node*, Document* oldDocument, Document* newDocument);
 
     void markDOMNodesForDocument(JSC::MarkStack&, Document*);
@@ -188,28 +180,6 @@ namespace WebCore {
         return createDOMObjectWrapper<WrapperClass>(exec, globalObject, object);
     }
 
-#if ENABLE(SVG)
-    #define CREATE_SVG_OBJECT_WRAPPER(exec, globalObject, className, object, context) createDOMObjectWrapper<JS##className>(exec, globalObject, static_cast<className*>(object), context)
-    template<class WrapperClass, class DOMClass> inline DOMObject* createDOMObjectWrapper(JSC::ExecState* exec, JSDOMGlobalObject* globalObject, DOMClass* object, SVGElement* context)
-    {
-        DOMObject* wrapper = createDOMObjectWrapper<WrapperClass, DOMClass>(exec, globalObject, object);
-        ASSERT(wrapper);
-        if (context)
-            JSSVGContextCache::addWrapper(wrapper, context);
-        return wrapper;
-    }
-    template<class WrapperClass, class DOMClass> inline JSC::JSValue getDOMObjectWrapper(JSC::ExecState* exec, JSDOMGlobalObject* globalObject, DOMClass* object, SVGElement* context)
-    {
-        if (!object)
-            return JSC::jsNull();
-        if (DOMObject* wrapper = getCachedDOMObjectWrapper(exec, object)) {
-            ASSERT(JSSVGContextCache::svgContextForDOMObject(wrapper) == context);
-            return wrapper;
-        }
-        return createDOMObjectWrapper<WrapperClass, DOMClass>(exec, globalObject, object, context);
-    }
-#endif
-
     #define CREATE_DOM_NODE_WRAPPER(exec, globalObject, className, object) createDOMNodeWrapper<JS##className>(exec, globalObject, static_cast<className*>(object))
     template<class WrapperClass, class DOMClass> inline JSNode* createDOMNodeWrapper(JSC::ExecState* exec, JSDOMGlobalObject* globalObject, DOMClass* node)
     {
@@ -225,7 +195,7 @@ namespace WebCore {
     {
         if (!node)
             return JSC::jsNull();
-        if (JSNode* wrapper = getCachedDOMNodeWrapper(exec, node->document(), node))
+        if (JSC::JSCell* wrapper = getCachedDOMNodeWrapper(exec, node->document(), node))
             return wrapper;
         return createDOMNodeWrapper<WrapperClass>(exec, globalObject, node);
     }
@@ -257,10 +227,26 @@ namespace WebCore {
 
     // See JavaScriptCore for explanation: Should be used for any UString that is already owned by another
     // object, to let the engine know that collecting the JSString wrapper is unlikely to save memory.
+    JSC::JSValue jsOwnedStringOrNull(JSC::ExecState*, const String&); 
     JSC::JSValue jsOwnedStringOrNull(JSC::ExecState*, const JSC::UString&); 
 
-    JSC::UString valueToStringWithNullCheck(JSC::ExecState*, JSC::JSValue); // null if the value is null
-    JSC::UString valueToStringWithUndefinedOrNullCheck(JSC::ExecState*, JSC::JSValue); // null if the value is null or undefined
+    String identifierToString(const JSC::Identifier&);
+    String ustringToString(const JSC::UString&);
+    JSC::UString stringToUString(const String&);
+
+    AtomicString identifierToAtomicString(const JSC::Identifier&);
+    AtomicString ustringToAtomicString(const JSC::UString&);
+    AtomicStringImpl* findAtomicString(const JSC::Identifier&);
+
+    String valueToStringWithNullCheck(JSC::ExecState*, JSC::JSValue); // null if the value is null
+    String valueToStringWithUndefinedOrNullCheck(JSC::ExecState*, JSC::JSValue); // null if the value is null or undefined
+
+    inline int32_t finiteInt32Value(JSC::JSValue value, JSC::ExecState* exec, bool& okay)
+    {
+        double number = value.toNumber(exec);
+        okay = isfinite(number);
+        return JSC::toInt32(number);
+    }
 
     // Returns a Date instance for the specified value, or null if the value is NaN or infinity.
     JSC::JSValue jsDateOrNull(JSC::ExecState*, double);
@@ -301,12 +287,14 @@ namespace WebCore {
     bool allowsAccessFromFrame(JSC::ExecState*, Frame*);
     bool allowsAccessFromFrame(JSC::ExecState*, Frame*, String& message);
     bool shouldAllowNavigation(JSC::ExecState*, Frame*);
+    bool allowSettingSrcToJavascriptURL(JSC::ExecState*, Element*, const String&, const String&);
+
     void printErrorMessageForFrame(Frame*, const String& message);
     JSC::JSValue objectToStringFunctionGetter(JSC::ExecState*, JSC::JSValue, const JSC::Identifier& propertyName);
 
     Frame* toLexicalFrame(JSC::ExecState*);
     Frame* toDynamicFrame(JSC::ExecState*);
-    bool processingUserGesture(JSC::ExecState*);
+    bool processingUserGesture();
     KURL completeURL(JSC::ExecState*, const String& relativeURL);
     
     inline JSC::JSValue jsString(JSC::ExecState* exec, const String& s)
@@ -316,7 +304,7 @@ namespace WebCore {
             return jsEmptyString(exec);
 
         if (stringImpl->length() == 1 && stringImpl->characters()[0] <= 0xFF)
-            return jsString(exec, stringImpl->ustring());
+            return jsString(exec, stringToUString(s));
 
         JSStringCache& stringCache = currentWorld(exec)->m_stringCache;
         if (JSC::JSString* wrapper = stringCache.get(stringImpl))
@@ -330,6 +318,31 @@ namespace WebCore {
         return currentWorld(exec)->m_wrappers;
     }
 
+    inline String ustringToString(const JSC::UString& u)
+    {
+        return u.impl();
+    }
+
+    inline JSC::UString stringToUString(const String& s)
+    {
+        return JSC::UString(s.impl());
+    }
+
+    inline String identifierToString(const JSC::Identifier& i)
+    {
+        return i.impl();
+    }
+
+    inline AtomicString ustringToAtomicString(const JSC::UString& u)
+    {
+        return AtomicString(u.impl());
+    }
+
+    inline AtomicString identifierToAtomicString(const JSC::Identifier& identifier)
+    {
+        return AtomicString(identifier.impl());
+    }
+
 } // namespace WebCore
 
 #endif // JSDOMBinding_h
