@@ -78,11 +78,26 @@
 #include "Notification.h"
 #endif
 
+#if ENABLE(INDEXED_DATABASE)
+#include "IDBRequest.h"
+#include "JSIDBRequest.h"
+#endif
+
+#if ENABLE(WEB_AUDIO)
+#include "JSJavaScriptAudioNode.h"
+#include "JavaScriptAudioNode.h"
+#endif
+
 #if ENABLE(WEB_SOCKETS)
 #include "JSWebSocket.h"
 #include "WebSocket.h"
 #endif
 
+#if ENABLE(BLOB)
+#include "JSFileReader.h"
+#include "FileReader.h"
+#endif
+
 using namespace JSC;
 
 namespace WebCore {
@@ -128,7 +143,7 @@ JSValue toJS(ExecState* exec, JSDOMGlobalObject* globalObject, EventTarget* targ
         return toJS(exec, globalObject, worker);
 
     if (DedicatedWorkerContext* workerContext = target->toDedicatedWorkerContext())
-        return toJSDOMGlobalObject(workerContext);
+        return toJSDOMGlobalObject(workerContext, exec);
 #endif
 
 #if ENABLE(SHARED_WORKERS)
@@ -136,7 +151,7 @@ JSValue toJS(ExecState* exec, JSDOMGlobalObject* globalObject, EventTarget* targ
         return toJS(exec, globalObject, sharedWorker);
 
     if (SharedWorkerContext* workerContext = target->toSharedWorkerContext())
-        return toJSDOMGlobalObject(workerContext);
+        return toJSDOMGlobalObject(workerContext, exec);
 #endif
 
 #if ENABLE(NOTIFICATIONS)
@@ -144,11 +159,26 @@ JSValue toJS(ExecState* exec, JSDOMGlobalObject* globalObject, EventTarget* targ
         return toJS(exec, notification);
 #endif
 
+#if ENABLE(INDEXED_DATABASE)
+    if (IDBRequest* idbRequest = target->toIDBRequest())
+        return toJS(exec, idbRequest);
+#endif
+
+#if ENABLE(WEB_AUDIO)
+    if (JavaScriptAudioNode* jsAudioNode = target->toJavaScriptAudioNode())
+        return toJS(exec, globalObject, jsAudioNode);
+#endif
+
 #if ENABLE(WEB_SOCKETS)
     if (WebSocket* webSocket = target->toWebSocket())
         return toJS(exec, webSocket);
 #endif
 
+#if ENABLE(BLOB)
+    if (FileReader* fileReader = target->toFileReader())
+        return toJS(exec, globalObject, fileReader);
+#endif
+
     ASSERT_NOT_REACHED();
     return jsNull();
 }
