@@ -34,6 +34,7 @@
 #include "DocumentType.h"
 #include "Entity.h"
 #include "EntityReference.h"
+#include "ExceptionCode.h"
 #include "HTMLElement.h"
 #include "JSAttr.h"
 #include "JSCDATASection.h"
@@ -67,127 +68,50 @@ using namespace JSC;
 
 namespace WebCore {
 
-static inline bool isAttrFrameSrc(Element *element, const String& name)
-{
-    return element && (element->hasTagName(HTMLNames::iframeTag) || element->hasTagName(HTMLNames::frameTag)) && equalIgnoringCase(name, "src");
-}
-
-void JSNode::setNodeValue(JSC::ExecState* exec, JSC::JSValue value)
+JSValue JSNode::insertBefore(ExecState* exec)
 {
     Node* imp = static_cast<Node*>(impl());
-    String nodeValue = valueToStringWithNullCheck(exec, value);
-
-    if (imp->nodeType() == Node::ATTRIBUTE_NODE) {
-        Element* ownerElement = static_cast<Attr*>(impl())->ownerElement();
-        if (ownerElement && !allowSettingSrcToJavascriptURL(exec, ownerElement, imp->nodeName(), nodeValue))
-            return;
-    }
-
     ExceptionCode ec = 0;
-    imp->setNodeValue(nodeValue, ec);
-    setDOMException(exec, ec);
-}
-
-void JSNode::setTextContent(JSC::ExecState* exec, JSC::JSValue value)
-{
-    Node* imp = static_cast<Node*>(impl());
-    String nodeValue = valueToStringWithNullCheck(exec, value);
-
-    if (imp->nodeType() == Node::ATTRIBUTE_NODE) {
-        Element* ownerElement = static_cast<Attr*>(impl())->ownerElement();
-        if (ownerElement && !allowSettingSrcToJavascriptURL(exec, ownerElement, imp->nodeName(), nodeValue))
-            return;
-    }
-
-    ExceptionCode ec = 0;
-    imp->setTextContent(nodeValue, ec);
-    setDOMException(exec, ec);
-}
-
-JSValue JSNode::insertBefore(ExecState* exec, const ArgList& args)
-{
-    Node* imp = static_cast<Node*>(impl());
-    if (imp->nodeType() == Node::ATTRIBUTE_NODE && isAttrFrameSrc(static_cast<Attr*>(impl())->ownerElement(), imp->nodeName())) {
-        setDOMException(exec, NOT_SUPPORTED_ERR);
-        return jsNull();
-    }
-
-    ExceptionCode ec = 0;
-    bool ok = imp->insertBefore(toNode(args.at(0)), toNode(args.at(1)), ec, true);
+    bool ok = imp->insertBefore(toNode(exec->argument(0)), toNode(exec->argument(1)), ec, true);
     setDOMException(exec, ec);
     if (ok)
-        return args.at(0);
+        return exec->argument(0);
     return jsNull();
 }
 
-JSValue JSNode::replaceChild(ExecState* exec, const ArgList& args)
+JSValue JSNode::replaceChild(ExecState* exec)
 {
     Node* imp = static_cast<Node*>(impl());
-    if (imp->nodeType() == Node::ATTRIBUTE_NODE && isAttrFrameSrc(static_cast<Attr*>(impl())->ownerElement(), imp->nodeName())) {
-        setDOMException(exec, NOT_SUPPORTED_ERR);
-        return jsNull();
-    }
-
     ExceptionCode ec = 0;
-    bool ok = imp->replaceChild(toNode(args.at(0)), toNode(args.at(1)), ec, true);
+    bool ok = imp->replaceChild(toNode(exec->argument(0)), toNode(exec->argument(1)), ec, true);
     setDOMException(exec, ec);
     if (ok)
-        return args.at(1);
+        return exec->argument(1);
     return jsNull();
 }
 
-JSValue JSNode::removeChild(ExecState* exec, const ArgList& args)
+JSValue JSNode::removeChild(ExecState* exec)
 {
     Node* imp = static_cast<Node*>(impl());
-    if (imp->nodeType() == Node::ATTRIBUTE_NODE && isAttrFrameSrc(static_cast<Attr*>(impl())->ownerElement(), imp->nodeName())) {
-        setDOMException(exec, NOT_SUPPORTED_ERR);
-        return jsNull();
-    }
-
     ExceptionCode ec = 0;
-    bool ok = imp->removeChild(toNode(args.at(0)), ec);
+    bool ok = imp->removeChild(toNode(exec->argument(0)), ec);
     setDOMException(exec, ec);
     if (ok)
-        return args.at(0);
+        return exec->argument(0);
     return jsNull();
 }
 
-JSValue JSNode::appendChild(ExecState* exec, const ArgList& args)
+JSValue JSNode::appendChild(ExecState* exec)
 {
     Node* imp = static_cast<Node*>(impl());
-    if (imp->nodeType() == Node::ATTRIBUTE_NODE && isAttrFrameSrc(static_cast<Attr*>(impl())->ownerElement(), imp->nodeName())) {
-        setDOMException(exec, NOT_SUPPORTED_ERR);
-        return jsNull();
-    }
-
     ExceptionCode ec = 0;
-    bool ok = imp->appendChild(toNode(args.at(0)), ec, true);
+    bool ok = imp->appendChild(toNode(exec->argument(0)), ec, true);
     setDOMException(exec, ec);
     if (ok)
-        return args.at(0);
+        return exec->argument(0);
     return jsNull();
 }
 
-JSValue JSNode::addEventListener(ExecState* exec, const ArgList& args)
-{
-    JSValue listener = args.at(1);
-    if (!listener.isObject())
-        return jsUndefined();
-
-    impl()->addEventListener(args.at(0).toString(exec), JSEventListener::create(asObject(listener), this, false, currentWorld(exec)), args.at(2).toBoolean(exec));
-    return jsUndefined();
-}
-
-JSValue JSNode::removeEventListener(ExecState* exec, const ArgList& args)
-{
-    JSValue listener = args.at(1);
-    if (!listener.isObject())
-        return jsUndefined();
-
-    impl()->removeEventListener(args.at(0).toString(exec), JSEventListener::create(asObject(listener), this, false, currentWorld(exec)).get(), args.at(2).toBoolean(exec));
-    return jsUndefined();
-}
-
 void JSNode::pushEventHandlerScope(ExecState*, ScopeChain&) const
 {
 }
@@ -203,6 +127,8 @@ void JSNode::markChildren(MarkStack& markStack)
     // the document, we need to mark the document, but we don't need to explicitly
     // mark any other nodes.
     if (node->inDocument()) {
+        // FIXME: Do we really want to call a virtual function, ownerDocument here,
+        // when the non-virtual inline function, document, is so much faster?!
         if (Document* doc = node->ownerDocument())
             markDOMNodeWrapper(markStack, doc, doc);
         return;
