@@ -21,15 +21,16 @@
 #include "config.h"
 #include "ScriptController.h"
 
-#include "CString.h"
+#include "ScriptableDocumentParser.h"
 #include "Event.h"
 #include "EventNames.h"
 #include "Frame.h"
 #include "FrameLoaderClient.h"
 #include "GCController.h"
 #include "HTMLPlugInElement.h"
-#include "InspectorTimelineAgent.h"
+#include "InspectorInstrumentation.h"
 #include "JSDocument.h"
+#include "JSMainThreadExecState.h"
 #include "NP_jsobject.h"
 #include "Page.h"
 #include "PageGroup.h"
@@ -37,12 +38,15 @@
 #include "ScriptValue.h"
 #include "Settings.h"
 #include "StorageNamespace.h"
+#include "UserGestureIndicator.h"
+#include "WebCoreJSClientData.h"
 #include "XSSAuditor.h"
 #include "npruntime_impl.h"
 #include "runtime_root.h"
 #include <debugger/Debugger.h>
 #include <runtime/InitializeThreading.h>
 #include <runtime/JSLock.h>
+#include <wtf/Threading.h>
 
 using namespace JSC;
 using namespace std;
@@ -52,11 +56,11 @@ namespace WebCore {
 void ScriptController::initializeThreading()
 {
     JSC::initializeThreading();
+    WTF::initializeMainThread();
 }
 
 ScriptController::ScriptController(Frame* frame)
     : m_frame(frame)
-    , m_handlerLineNumber(0)
     , m_sourceURL(0)
     , m_inExecuteScript(false)
     , m_processingTimerCallback(false)
@@ -70,7 +74,7 @@ ScriptController::ScriptController(Frame* frame)
 #endif
     , m_XSSAuditor(new XSSAuditor(frame))
 {
-#if PLATFORM(MAC) && ENABLE(MAC_JAVA_BRIDGE)
+#if PLATFORM(MAC) && ENABLE(JAVA_BRIDGE)
     static bool initializedJavaJSBindings;
     if (!initializedJavaJSBindings) {
         initializedJavaJSBindings = true;
@@ -81,22 +85,43 @@ ScriptController::ScriptController(Frame* frame)
 
 ScriptController::~ScriptController()
 {
-    if (!m_windowShells.isEmpty()) {
-        m_windowShells.clear();
-    
-        // It's likely that releasing the global object has created a lot of garbage.
-        gcController().garbageCollectSoon();
+    disconnectPlatformScriptObjects();
+
+    if (m_cacheableBindingRootObject) {
+        m_cacheableBindingRootObject->invalidate();
+        m_cacheableBindingRootObject = 0;
     }
 
-    disconnectPlatformScriptObjects();
+    // It's likely that destroying m_windowShells will create a lot of garbage.
+    if (!m_windowShells.isEmpty()) {
+        while (!m_windowShells.isEmpty())
+            destroyWindowShell(m_windowShells.begin()->first.get());
+        gcController().garbageCollectSoon();
+    }
 }
 
-ScriptValue ScriptController::evaluateInWorld(const ScriptSourceCode& sourceCode, DOMWrapperWorld* world)
+void ScriptController::destroyWindowShell(DOMWrapperWorld* world)
+{
+    ASSERT(m_windowShells.contains(world));
+    m_windowShells.remove(world);
+    world->didDestroyWindowShell(this);
+}
+
+JSDOMWindowShell* ScriptController::createWindowShell(DOMWrapperWorld* world)
+{
+    ASSERT(!m_windowShells.contains(world));
+    JSDOMWindowShell* windowShell = new JSDOMWindowShell(m_frame->domWindow(), world);
+    m_windowShells.add(world, windowShell);
+    world->didCreateWindowShell(this);
+    return windowShell;
+}
+
+ScriptValue ScriptController::evaluateInWorld(const ScriptSourceCode& sourceCode, DOMWrapperWorld* world, ShouldAllowXSS shouldAllowXSS)
 {
     const SourceCode& jsSourceCode = sourceCode.jsSourceCode();
-    String sourceURL = jsSourceCode.provider()->url();
+    String sourceURL = ustringToString(jsSourceCode.provider()->url());
 
-    if (!m_XSSAuditor->canEvaluate(sourceCode.source())) {
+    if (shouldAllowXSS == DoNotAllowXSS && !m_XSSAuditor->canEvaluate(sourceCode.source())) {
         // This script is not safe to be evaluated.
         return JSValue();
     }
@@ -117,19 +142,13 @@ ScriptValue ScriptController::evaluateInWorld(const ScriptSourceCode& sourceCode
 
     RefPtr<Frame> protect = m_frame;
 
-#if ENABLE(INSPECTOR)
-    if (InspectorTimelineAgent* timelineAgent = m_frame->page() ? m_frame->page()->inspectorTimelineAgent() : 0)
-        timelineAgent->willEvaluateScript(sourceURL, sourceCode.startLine());
-#endif
+    InspectorInstrumentationCookie cookie = InspectorInstrumentation::willEvaluateScript(m_frame, sourceURL, sourceCode.startLine());
 
     exec->globalData().timeoutChecker.start();
-    Completion comp = JSC::evaluate(exec, exec->dynamicGlobalObject()->globalScopeChain(), jsSourceCode, shell);
+    Completion comp = JSMainThreadExecState::evaluate(exec, exec->dynamicGlobalObject()->globalScopeChain(), jsSourceCode, shell);
     exec->globalData().timeoutChecker.stop();
 
-#if ENABLE(INSPECTOR)
-    if (InspectorTimelineAgent* timelineAgent = m_frame->page() ? m_frame->page()->inspectorTimelineAgent() : 0)
-        timelineAgent->didEvaluateScript();
-#endif
+    InspectorInstrumentation::didEvaluateScript(cookie);
 
     // Evaluating the JavaScript could cause the frame to be deallocated
     // so we start the keep alive timer here.
@@ -147,28 +166,14 @@ ScriptValue ScriptController::evaluateInWorld(const ScriptSourceCode& sourceCode
     return JSValue();
 }
 
-ScriptValue ScriptController::evaluate(const ScriptSourceCode& sourceCode) 
+ScriptValue ScriptController::evaluate(const ScriptSourceCode& sourceCode, ShouldAllowXSS shouldAllowXSS) 
 {
-    return evaluateInWorld(sourceCode, mainThreadNormalWorld());
+    return evaluateInWorld(sourceCode, mainThreadNormalWorld(), shouldAllowXSS);
 }
 
-// An DOMWrapperWorld other than the thread's normal world.
-class IsolatedWorld : public DOMWrapperWorld {
-public:
-    IsolatedWorld(JSGlobalData* globalData)
-        : DOMWrapperWorld(globalData, false)
-    {
-        JSGlobalData::ClientData* clientData = globalData->clientData;
-        ASSERT(clientData);
-        static_cast<WebCoreJSClientData*>(clientData)->rememberWorld(this);
-    }
-
-    static PassRefPtr<IsolatedWorld> create(JSGlobalData* globalData) { return adoptRef(new IsolatedWorld(globalData)); }
-};
-
 PassRefPtr<DOMWrapperWorld> ScriptController::createWorld()
 {
-    return IsolatedWorld::create(JSDOMWindow::commonJSGlobalData());
+    return DOMWrapperWorld::create(JSDOMWindow::commonJSGlobalData());
 }
 
 void ScriptController::getAllWorlds(Vector<DOMWrapperWorld*>& worlds)
@@ -176,7 +181,7 @@ void ScriptController::getAllWorlds(Vector<DOMWrapperWorld*>& worlds)
     static_cast<WebCoreJSClientData*>(JSDOMWindow::commonJSGlobalData()->clientData)->getAllWorlds(worlds);
 }
 
-void ScriptController::clearWindowShell()
+void ScriptController::clearWindowShell(bool goingIntoPageCache)
 {
     if (m_windowShells.isEmpty())
         return;
@@ -192,14 +197,21 @@ void ScriptController::clearWindowShell()
         windowShell->window()->willRemoveFromWindowShell();
         windowShell->setWindow(m_frame->domWindow());
 
+        // An m_cacheableBindingRootObject persists between page navigations
+        // so needs to know about the new JSDOMWindow.
+        if (m_cacheableBindingRootObject)
+            m_cacheableBindingRootObject->updateGlobalObject(windowShell->window());
+
         if (Page* page = m_frame->page()) {
             attachDebugger(windowShell, page->debugger());
             windowShell->window()->setProfileGroup(page->group().identifier());
         }
     }
 
-    // There is likely to be a lot of garbage now.
-    gcController().garbageCollectSoon();
+    // It's likely that resetting our windows created a lot of garbage, unless
+    // it went in a back/forward cache.
+    if (!goingIntoPageCache)
+        gcController().garbageCollectSoon();
 }
 
 JSDOMWindowShell* ScriptController::initScript(DOMWrapperWorld* world)
@@ -208,8 +220,8 @@ JSDOMWindowShell* ScriptController::initScript(DOMWrapperWorld* world)
 
     JSLock lock(SilenceAssertionsOnly);
 
-    JSDOMWindowShell* windowShell = new JSDOMWindowShell(m_frame->domWindow(), world);
-    m_windowShells.add(world, windowShell);
+    JSDOMWindowShell* windowShell = createWindowShell(world);
+
     windowShell->window()->updateDocument();
 
     if (Page* page = m_frame->page()) {
@@ -222,21 +234,39 @@ JSDOMWindowShell* ScriptController::initScript(DOMWrapperWorld* world)
     return windowShell;
 }
 
-bool ScriptController::processingUserGesture(DOMWrapperWorld* world) const
+int ScriptController::eventHandlerLineNumber() const
 {
-    return m_allowPopupsFromPlugin || processingUserGestureEvent(world) || isJavaScriptAnchorNavigation();
+    // JSC expects 1-based line numbers, so we must add one here to get it right.
+    ScriptableDocumentParser* parser = m_frame->document()->scriptableDocumentParser();
+    if (parser)
+        return parser->lineNumber() + 1;
+    return 0;
 }
 
-bool ScriptController::processingUserGestureEvent(DOMWrapperWorld* world) const
+bool ScriptController::processingUserGesture()
 {
-    JSDOMWindowShell* shell = existingWindowShell(world);
-    if (!shell)
-        return false;
+    ExecState* exec = JSMainThreadExecState::currentState();
+    Frame* frame = exec ? toDynamicFrame(exec) : 0;
+    // No script is running, so it is user-initiated unless the gesture stack
+    // explicitly says it is not.
+    if (!frame)
+        return UserGestureIndicator::getUserGestureState() != DefinitelyNotProcessingUserGesture;
 
-    if (Event* event = shell->window()->currentEvent())
-        return event->fromUserGesture();
-    
-    return false;
+    // FIXME: We check the plugin popup flag and javascript anchor navigation
+    // from the dynamic frame becuase they should only be initiated on the
+    // dynamic frame in which execution began if they do happen.
+    ScriptController* scriptController = frame->script();
+    ASSERT(scriptController);
+    if (scriptController->allowPopupsFromPlugin() || scriptController->isJavaScriptAnchorNavigation())
+        return true;
+
+    // If a DOM event is being processed, check that it was initiated by the user
+    // and that it is in the whitelist of event types allowed to generate pop-ups.
+    if (JSDOMWindowShell* shell = scriptController->existingWindowShell(currentWorld(exec)))
+        if (Event* event = shell->window()->currentEvent())
+            return event->fromUserGesture();
+
+    return UserGestureIndicator::processingUserGesture();
 }
 
 // FIXME: This seems like an insufficient check to verify a click on a javascript: anchor.
@@ -281,6 +311,16 @@ bool ScriptController::anyPageIsProcessingUserGesture() const
     return false;
 }
 
+bool ScriptController::canAccessFromCurrentOrigin(Frame *frame)
+{
+    ExecState* exec = JSMainThreadExecState::currentState();
+    if (exec)
+        return allowsAccessFromFrame(exec, frame);
+    // If the current state is 0 we're in a call path where the DOM security 
+    // check doesn't apply (eg. parser).
+    return true;
+}
+
 void ScriptController::attachDebugger(JSC::Debugger* debugger)
 {
     for (ShellMap::iterator iter = m_windowShells.begin(); iter != m_windowShells.end(); ++iter)
@@ -314,9 +354,21 @@ void ScriptController::updateSecurityOrigin()
     // Our bindings do not do anything in this case.
 }
 
+Bindings::RootObject* ScriptController::cacheableBindingRootObject()
+{
+    if (!canExecuteScripts(NotAboutToExecuteScript))
+        return 0;
+
+    if (!m_cacheableBindingRootObject) {
+        JSLock lock(SilenceAssertionsOnly);
+        m_cacheableBindingRootObject = Bindings::RootObject::create(0, globalObject(pluginWorld()));
+    }
+    return m_cacheableBindingRootObject.get();
+}
+
 Bindings::RootObject* ScriptController::bindingRootObject()
 {
-    if (!canExecuteScripts())
+    if (!canExecuteScripts(NotAboutToExecuteScript))
         return 0;
 
     if (!m_bindingRootObject) {
@@ -338,12 +390,18 @@ PassRefPtr<Bindings::RootObject> ScriptController::createRootObject(void* native
     return rootObject.release();
 }
 
+#if ENABLE(INSPECTOR)
+void ScriptController::setCaptureCallStackForUncaughtExceptions(bool)
+{
+}
+#endif
+
 #if ENABLE(NETSCAPE_PLUGIN_API)
 
 NPObject* ScriptController::windowScriptNPObject()
 {
     if (!m_windowScriptNPObject) {
-        if (canExecuteScripts()) {
+        if (canExecuteScripts(NotAboutToExecuteScript)) {
             // JavaScript is enabled, so there is a JavaScript window object.
             // Return an NPObject bound to the window object.
             JSC::JSLock lock(SilenceAssertionsOnly);
@@ -376,7 +434,7 @@ NPObject* ScriptController::createScriptObjectForPluginElement(HTMLPlugInElement
 JSObject* ScriptController::jsObjectForPluginElement(HTMLPlugInElement* plugin)
 {
     // Can't create JSObjects when JavaScript is disabled
-    if (!canExecuteScripts())
+    if (!canExecuteScripts(NotAboutToExecuteScript))
         return 0;
 
     // Create a JSObject bound to this element
@@ -439,17 +497,17 @@ void ScriptController::clearScriptObjects()
 #endif
 }
 
-ScriptValue ScriptController::executeScriptInWorld(DOMWrapperWorld* world, const String& script, bool forceUserGesture)
+ScriptValue ScriptController::executeScriptInWorld(DOMWrapperWorld* world, const String& script, bool forceUserGesture, ShouldAllowXSS shouldAllowXSS)
 {
-    ScriptSourceCode sourceCode(script, forceUserGesture ? KURL() : m_frame->loader()->url());
+    ScriptSourceCode sourceCode(script, forceUserGesture ? KURL() : m_frame->document()->url());
 
-    if (!canExecuteScripts() || isPaused())
+    if (!canExecuteScripts(AboutToExecuteScript) || isPaused())
         return ScriptValue();
 
     bool wasInExecuteScript = m_inExecuteScript;
     m_inExecuteScript = true;
 
-    ScriptValue result = evaluateInWorld(sourceCode, world);
+    ScriptValue result = evaluateInWorld(sourceCode, world, shouldAllowXSS);
 
     if (!wasInExecuteScript) {
         m_inExecuteScript = false;
