@@ -32,10 +32,14 @@
 #include "ScriptObject.h"
 
 #include "JSDOMBinding.h"
-#include "JSInspectorBackend.h"
 
 #include <runtime/JSLock.h>
 
+#if ENABLE(INSPECTOR)
+#include "JSInjectedScriptHost.h"
+#include "JSInspectorFrontendHost.h"
+#endif
+
 using namespace JSC;
 
 namespace WebCore {
@@ -55,68 +59,6 @@ static bool handleException(ScriptState* scriptState)
     return false;
 }
 
-bool ScriptObject::set(const String& name, const String& value)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    PutPropertySlot slot;
-    jsObject()->put(m_scriptState, Identifier(m_scriptState, name), jsString(m_scriptState, value), slot);
-    return handleException(m_scriptState);
-}
-
-bool ScriptObject::set(const char* name, const ScriptObject& value)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    PutPropertySlot slot;
-    jsObject()->put(m_scriptState, Identifier(m_scriptState, name), value.jsObject(), slot);
-    return handleException(m_scriptState);
-}
-
-bool ScriptObject::set(const char* name, const String& value)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    PutPropertySlot slot;
-    jsObject()->put(m_scriptState, Identifier(m_scriptState, name), jsString(m_scriptState, value), slot);
-    return handleException(m_scriptState);
-}
-
-bool ScriptObject::set(const char* name, double value)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    PutPropertySlot slot;
-    jsObject()->put(m_scriptState, Identifier(m_scriptState, name), jsNumber(m_scriptState, value), slot);
-    return handleException(m_scriptState);
-}
-
-bool ScriptObject::set(const char* name, long long value)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    PutPropertySlot slot;
-    jsObject()->put(m_scriptState, Identifier(m_scriptState, name), jsNumber(m_scriptState, value), slot);
-    return handleException(m_scriptState);
-}
-
-bool ScriptObject::set(const char* name, int value)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    PutPropertySlot slot;
-    jsObject()->put(m_scriptState, Identifier(m_scriptState, name), jsNumber(m_scriptState, value), slot);
-    return handleException(m_scriptState);
-}
-
-bool ScriptObject::set(const char* name, bool value)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    PutPropertySlot slot;
-    jsObject()->put(m_scriptState, Identifier(m_scriptState, name), jsBoolean(value), slot);
-    return handleException(m_scriptState);
-}
-
-ScriptObject ScriptObject::createNew(ScriptState* scriptState)
-{
-    JSLock lock(SilenceAssertionsOnly);
-    return ScriptObject(scriptState, constructEmptyObject(scriptState));
-}
-
 bool ScriptGlobalObject::set(ScriptState* scriptState, const char* name, const ScriptObject& value)
 {
     JSLock lock(SilenceAssertionsOnly);
@@ -125,7 +67,15 @@ bool ScriptGlobalObject::set(ScriptState* scriptState, const char* name, const S
 }
 
 #if ENABLE(INSPECTOR)
-bool ScriptGlobalObject::set(ScriptState* scriptState, const char* name, InspectorBackend* value)
+bool ScriptGlobalObject::set(ScriptState* scriptState, const char* name, InspectorFrontendHost* value)
+{
+    JSLock lock(SilenceAssertionsOnly);
+    JSDOMGlobalObject* globalObject = static_cast<JSDOMGlobalObject*>(scriptState->lexicalGlobalObject());
+    globalObject->putDirect(Identifier(scriptState, name), toJS(scriptState, globalObject, value));
+    return handleException(scriptState);
+}
+
+bool ScriptGlobalObject::set(ScriptState* scriptState, const char* name, InjectedScriptHost* value)
 {
     JSLock lock(SilenceAssertionsOnly);
     JSDOMGlobalObject* globalObject = static_cast<JSDOMGlobalObject*>(scriptState->lexicalGlobalObject());
