@@ -17,12 +17,12 @@
  * along with this library; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
- */
+ */ 
 
 #include "config.h"
 #include "CSSCursorImageValue.h"
 
-#include "DocLoader.h"
+#include "CachedResourceLoader.h"
 #include "Document.h"
 #include "PlatformString.h"
 #include <wtf/MathExtras.h>
@@ -30,6 +30,7 @@
 
 #if ENABLE(SVG)
 #include "SVGCursorElement.h"
+#include "SVGNames.h"
 #include "SVGURIReference.h"
 #endif
 
@@ -52,9 +53,9 @@ static inline SVGCursorElement* resourceReferencedByCursorElement(const String&
 }
 #endif
 
-CSSCursorImageValue::CSSCursorImageValue(const String& url, const IntPoint& hotspot)
+CSSCursorImageValue::CSSCursorImageValue(const String& url, const IntPoint& hotSpot)
     : CSSImageValue(url)
-    , m_hotspot(hotspot)
+    , m_hotSpot(hotSpot)
 {
 }
 
@@ -70,7 +71,7 @@ CSSCursorImageValue::~CSSCursorImageValue()
 
     for (; it != end; ++it) {
         SVGElement* referencedElement = *it;
-        referencedElement->cursorImageValueRemoved(); 
+        referencedElement->cursorElementRemoved();
         if (SVGCursorElement* cursorElement = resourceReferencedByCursorElement(url, referencedElement->document()))
             cursorElement->removeClient(referencedElement);
     }
@@ -90,11 +91,12 @@ bool CSSCursorImageValue::updateIfSVGCursorIsUsed(Element* element)
         return false;
 
     if (SVGCursorElement* cursorElement = resourceReferencedByCursorElement(url, element->document())) {
+        // FIXME: This will override hot spot specified in CSS, which is probably incorrect.
         float x = roundf(cursorElement->x().value(0));
-        m_hotspot.setX(static_cast<int>(x));
+        m_hotSpot.setX(static_cast<int>(x));
 
         float y = roundf(cursorElement->y().value(0));
-        m_hotspot.setY(static_cast<int>(y));
+        m_hotSpot.setY(static_cast<int>(y));
 
         if (cachedImageURL() != element->document()->completeURL(cursorElement->href()))
             clearCachedImage();
@@ -110,13 +112,13 @@ bool CSSCursorImageValue::updateIfSVGCursorIsUsed(Element* element)
     return false;
 }
 
-StyleCachedImage* CSSCursorImageValue::cachedImage(DocLoader* loader)
+StyleCachedImage* CSSCursorImageValue::cachedImage(CachedResourceLoader* loader)
 {
     String url = getStringValue();
 
 #if ENABLE(SVG) 
-    if (isSVGCursorIdentifier(url) && loader && loader->doc()) {
-        if (SVGCursorElement* cursorElement = resourceReferencedByCursorElement(url, loader->doc()))
+    if (isSVGCursorIdentifier(url) && loader && loader->document()) {
+        if (SVGCursorElement* cursorElement = resourceReferencedByCursorElement(url, loader->document()))
             url = cursorElement->href();
     }
 #endif
