@@ -1,6 +1,6 @@
 /*
  * (C) 1999-2003 Lars Knoll (knoll@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,7 +39,6 @@ namespace WebCore {
 
 CSSMutableStyleDeclaration::CSSMutableStyleDeclaration()
     : m_node(0)
-    , m_variableDependentValueCount(0)
     , m_strictParsing(false)
 #ifndef NDEBUG
     , m_iteratorCount(0)
@@ -50,7 +49,6 @@ CSSMutableStyleDeclaration::CSSMutableStyleDeclaration()
 CSSMutableStyleDeclaration::CSSMutableStyleDeclaration(CSSRule* parent)
     : CSSStyleDeclaration(parent)
     , m_node(0)
-    , m_variableDependentValueCount(0)
     , m_strictParsing(!parent || parent->useStrictParsing())
 #ifndef NDEBUG
     , m_iteratorCount(0)
@@ -58,11 +56,10 @@ CSSMutableStyleDeclaration::CSSMutableStyleDeclaration(CSSRule* parent)
 {
 }
 
-CSSMutableStyleDeclaration::CSSMutableStyleDeclaration(CSSRule* parent, const Vector<CSSProperty>& properties, unsigned variableDependentValueCount)
+CSSMutableStyleDeclaration::CSSMutableStyleDeclaration(CSSRule* parent, const Vector<CSSProperty>& properties)
     : CSSStyleDeclaration(parent)
     , m_properties(properties)
     , m_node(0)
-    , m_variableDependentValueCount(variableDependentValueCount)
     , m_strictParsing(!parent || parent->useStrictParsing())
 #ifndef NDEBUG
     , m_iteratorCount(0)
@@ -75,20 +72,22 @@ CSSMutableStyleDeclaration::CSSMutableStyleDeclaration(CSSRule* parent, const Ve
 CSSMutableStyleDeclaration::CSSMutableStyleDeclaration(CSSRule* parent, const CSSProperty* const * properties, int numProperties)
     : CSSStyleDeclaration(parent)
     , m_node(0)
-    , m_variableDependentValueCount(0)
     , m_strictParsing(!parent || parent->useStrictParsing())
 #ifndef NDEBUG
     , m_iteratorCount(0)
 #endif
 {
     m_properties.reserveInitialCapacity(numProperties);
+    HashSet<int> candidates;
     for (int i = 0; i < numProperties; ++i) {
-        ASSERT(properties[i]);
-        m_properties.append(*properties[i]);
-        if (properties[i]->value()->isVariableDependentValue())
-            m_variableDependentValueCount++;
+        const CSSProperty *property = properties[i];
+        ASSERT(property);
+        if (candidates.contains(property->id()))
+            removeProperty(properties[i]->id(), false);
+        m_properties.append(*property);
+        if (!getPropertyPriority(property->id()) && !property->isImportant())
+            candidates.add(property->id());
     }
-    // FIXME: This allows duplicate properties.
 }
 
 CSSMutableStyleDeclaration& CSSMutableStyleDeclaration::operator=(const CSSMutableStyleDeclaration& other)
@@ -143,8 +142,7 @@ String CSSMutableStyleDeclaration::getPropertyValue(int propertyID) const
                                             CSSPropertyBorderBottomColor,
                                             CSSPropertyBorderLeftColor }};
             String res;
-            const int nrprops = sizeof(properties) / sizeof(properties[0]);
-            for (int i = 0; i < nrprops; ++i) {
+            for (size_t i = 0; i < WTF_ARRAY_LENGTH(properties); ++i) {
                 String value = getCommonValue(properties[i], 4);
                 if (!value.isNull()) {
                     if (!res.isNull())
@@ -240,10 +238,11 @@ String CSSMutableStyleDeclaration::getPropertyValue(int propertyID) const
             return getLayeredShorthandValue(properties, 4);
         }
         case CSSPropertyWebkitAnimation: {
-            const int properties[6] = { CSSPropertyWebkitAnimationName, CSSPropertyWebkitAnimationDuration,
+            const int properties[7] = { CSSPropertyWebkitAnimationName, CSSPropertyWebkitAnimationDuration,
                                         CSSPropertyWebkitAnimationTimingFunction, CSSPropertyWebkitAnimationDelay,
-                                        CSSPropertyWebkitAnimationIterationCount, CSSPropertyWebkitAnimationDirection };
-            return getLayeredShorthandValue(properties, 6);
+                                        CSSPropertyWebkitAnimationIterationCount, CSSPropertyWebkitAnimationDirection,
+                                        CSSPropertyWebkitAnimationFillMode };
+            return getLayeredShorthandValue(properties, 7);
         }
 #if ENABLE(SVG)
         case CSSPropertyMarker: {
@@ -330,7 +329,9 @@ String CSSMutableStyleDeclaration::getLayeredShorthandValue(const int* propertie
             // then it was written with only one value. Here we figure out which value that was so we can
             // report back correctly. 
             if (properties[j] == CSSPropertyBackgroundRepeatX && isPropertyImplicit(properties[j])) {
-                if (j < number - 1 && properties[j + 1] == CSSPropertyBackgroundRepeatY) {
+
+                // BUG 49055: make sure the value was not reset in the layer check just above.
+                if (j < number - 1 && properties[j + 1] == CSSPropertyBackgroundRepeatY && value) {
                     RefPtr<CSSValue> yValue;
                     RefPtr<CSSValue> nextValue = values[j + 1];
                     if (nextValue->isValueList())
@@ -404,18 +405,16 @@ String CSSMutableStyleDeclaration::getCommonValue(const int* properties, int num
 {
     String res;
     for (int i = 0; i < number; ++i) {
-        if (!isPropertyImplicit(properties[i])) {
-            RefPtr<CSSValue> value = getPropertyCSSValue(properties[i]);
-            if (!value)
-                return String();
-            String text = value->cssText();
-            if (text.isNull())
-                return String();
-            if (res.isNull())
-                res = text;
-            else if (res != text)
-                return String();
-        }
+        RefPtr<CSSValue> value = getPropertyCSSValue(properties[i]);
+        if (!value)
+            return String();
+        String text = value->cssText();
+        if (text.isNull())
+            return String();
+        if (res.isNull())
+            res = text;
+        else if (res != text)
+            return String();
     }
     return res;
 }
@@ -451,9 +450,6 @@ String CSSMutableStyleDeclaration::removeProperty(int propertyID, bool notifyCha
     
     String value = returnText ? foundProperty->value()->cssText() : String();
 
-    if (foundProperty->value()->isVariableDependentValue())
-        m_variableDependentValueCount--;
-
     // A more efficient removal strategy would involve marking entries as empty
     // and sweeping them when the vector grows too big.
     m_properties.remove(foundProperty - m_properties.data());
@@ -479,12 +475,13 @@ void CSSMutableStyleDeclaration::setNeedsStyleRecalc()
         return;
     }
 
-    // FIXME: quick&dirty hack for KDE 3.0... make this MUCH better! (Dirk)
     StyleBase* root = this;
     while (StyleBase* parent = root->parent())
         root = parent;
-    if (root->isCSSStyleSheet())
-        static_cast<CSSStyleSheet*>(root)->doc()->updateStyleSelector();
+    if (root->isCSSStyleSheet()) {
+        if (Document* document = static_cast<CSSStyleSheet*>(root)->document())
+            document->styleSelectorChanged(DeferRecalcStyle);
+    }
 }
 
 bool CSSMutableStyleDeclaration::getPropertyPriority(int propertyID) const
@@ -602,8 +599,6 @@ void CSSMutableStyleDeclaration::addParsedProperties(const CSSProperty* const* p
             removeProperty(properties[i]->id(), false);
             ASSERT(properties[i]);
             m_properties.append(*properties[i]);
-            if (properties[i]->value()->isVariableDependentValue())
-                m_variableDependentValueCount++;
         }
     }
     // FIXME: This probably should have a call to setNeedsStyleRecalc() if something changed. We may also wish to add
@@ -713,13 +708,13 @@ void CSSMutableStyleDeclaration::setCssText(const String& text, ExceptionCode& e
     setNeedsStyleRecalc();
 }
 
-void CSSMutableStyleDeclaration::merge(CSSMutableStyleDeclaration* other, bool argOverridesOnConflict)
+void CSSMutableStyleDeclaration::merge(const CSSMutableStyleDeclaration* other, bool argOverridesOnConflict)
 {
     ASSERT(!m_iteratorCount);
 
     unsigned size = other->m_properties.size();
     for (unsigned n = 0; n < size; ++n) {
-        CSSProperty& toMerge = other->m_properties[n];
+        const CSSProperty& toMerge = other->m_properties[n];
         CSSProperty* old = findPropertyWithId(toMerge.id());
         if (old) {
             if (!argOverridesOnConflict && old->value())
@@ -762,7 +757,7 @@ static const int blockProperties[] = {
     CSSPropertyWidows
 };
 
-const unsigned numBlockProperties = sizeof(blockProperties) / sizeof(blockProperties[0]);
+const unsigned numBlockProperties = WTF_ARRAY_LENGTH(blockProperties);
 
 PassRefPtr<CSSMutableStyleDeclaration> CSSMutableStyleDeclaration::copyBlockProperties() const
 {
@@ -814,7 +809,7 @@ PassRefPtr<CSSMutableStyleDeclaration> CSSMutableStyleDeclaration::makeMutable()
 
 PassRefPtr<CSSMutableStyleDeclaration> CSSMutableStyleDeclaration::copy() const
 {
-    return adoptRef(new CSSMutableStyleDeclaration(0, m_properties, m_variableDependentValueCount));
+    return adoptRef(new CSSMutableStyleDeclaration(0, m_properties));
 }
 
 const CSSProperty* CSSMutableStyleDeclaration::findPropertyWithId(int propertyID) const
