@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003 Lars Knoll (knoll@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,7 +21,7 @@
 #ifndef CSSParserValues_h
 #define CSSParserValues_h
 
-#include "AtomicString.h"
+#include <wtf/text/AtomicString.h>
 
 namespace WebCore {
 
@@ -55,7 +55,6 @@ struct CSSParserValue {
     };
     int unit;
     
-    bool isVariable() const;
     
     PassRefPtr<CSSValue> createCSSValue();
 };
@@ -64,7 +63,6 @@ class CSSParserValueList : public FastAllocBase {
 public:
     CSSParserValueList()
         : m_current(0)
-        , m_variablesCount(0)
     {
     }
     ~CSSParserValueList();
@@ -80,19 +78,14 @@ public:
         
     void clear() { m_values.clear(); }
 
-    bool containsVariables() const { return m_variablesCount; }
-
 private:
-    Vector<CSSParserValue, 4> m_values;
     unsigned m_current;
-    unsigned m_variablesCount;
+    Vector<CSSParserValue, 4> m_values;
 };
 
 struct CSSParserFunction : FastAllocBase {
     CSSParserString name;
-    CSSParserValueList* args;
-
-    ~CSSParserFunction() { delete args; }
+    OwnPtr<CSSParserValueList> args;
 };
 
 }
