@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,22 +23,25 @@
 #define CSSStyleSelector_h
 
 #include "CSSFontSelector.h"
+#include "CSSRule.h"
 #include "LinkHash.h"
 #include "MediaQueryExp.h"
 #include "RenderStyle.h"
-#include "StringHash.h"
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/RefPtr.h>
 #include <wtf/Vector.h>
+#include <wtf/text/StringHash.h>
 
 namespace WebCore {
 
 class CSSMutableStyleDeclaration;
+class CSSPageRule;
 class CSSPrimitiveValue;
 class CSSProperty;
 class CSSFontFace;
 class CSSFontFaceRule;
+class CSSImageValue;
 class CSSRuleData;
 class CSSRuleDataList;
 class CSSRuleList;
@@ -47,8 +50,7 @@ class CSSSelector;
 class CSSStyleRule;
 class CSSStyleSheet;
 class CSSValue;
-class CSSVariableDependentValue;
-class CSSVariablesRule;
+class ContainerNode;
 class DataGridColumn;
 class Document;
 class Element;
@@ -56,6 +58,7 @@ class Frame;
 class FrameView;
 class KURL;
 class KeyframeList;
+class KeyframeValue;
 class MediaQueryEvaluator;
 class Node;
 class Settings;
@@ -63,6 +66,7 @@ class StyleImage;
 class StyleSheet;
 class StyleSheetList;
 class StyledElement;
+class WebKitCSSKeyframeRule;
 class WebKitCSSKeyframesRule;
 
 class MediaQueryResult : public Noncopyable {
@@ -85,11 +89,13 @@ public:
                          bool strictParsing, bool matchAuthorAndUserStyles);
         ~CSSStyleSelector();
 
-        void initForStyleResolve(Element*, RenderStyle* parentStyle = 0, PseudoId = NOPSEUDO);
-        PassRefPtr<RenderStyle> styleForElement(Element*, RenderStyle* parentStyle = 0, bool allowSharing = true, bool resolveForRootDefault = false, bool matchVisitedLinks = false);
+        PassRefPtr<RenderStyle> styleForElement(Element* e, RenderStyle* parentStyle = 0, bool allowSharing = true, bool resolveForRootDefault = false, bool matchVisitedPseudoClass = false);
+        
         void keyframeStylesForAnimation(Element*, const RenderStyle*, KeyframeList& list);
 
-        PassRefPtr<RenderStyle> pseudoStyleForElement(PseudoId, Element*, RenderStyle* parentStyle = 0, bool matchVisitedLinks = false);
+        PassRefPtr<RenderStyle> pseudoStyleForElement(PseudoId pseudo, Element* e, RenderStyle* parentStyle = 0, bool matchVisitedPseudoClass = false);
+
+        PassRefPtr<RenderStyle> styleForPage(int pageIndex);
 
         static PassRefPtr<RenderStyle> styleForDocument(Document*);
 
@@ -100,23 +106,30 @@ public:
 #endif
 
     private:
+        void initForStyleResolve(Element*, RenderStyle* parentStyle = 0, PseudoId = NOPSEUDO);
         void initElement(Element*);
-        RenderStyle* locateSharedStyle();
+        ALWAYS_INLINE RenderStyle* locateSharedStyle();
         Node* locateCousinList(Element* parent, unsigned depth = 1);
         bool canShareStyleWithElement(Node*);
 
         RenderStyle* style() const { return m_style.get(); }
 
+        PassRefPtr<RenderStyle> styleForKeyframe(const RenderStyle*, const WebKitCSSKeyframeRule*, KeyframeValue&);
+
     public:
         // These methods will give back the set of rules that matched for a given element (or a pseudo-element).
-        PassRefPtr<CSSRuleList> styleRulesForElement(Element*, bool authorOnly);
-        PassRefPtr<CSSRuleList> pseudoStyleRulesForElement(Element*, const String& pseudoStyle, bool authorOnly);
+        PassRefPtr<CSSRuleList> styleRulesForElement(Element*, bool authorOnly, bool includeEmptyRules = false, CSSRuleFilter filter = AllCSSRules);
+        PassRefPtr<CSSRuleList> pseudoStyleRulesForElement(Element*, PseudoId, bool authorOnly, bool includeEmptyRules = false, CSSRuleFilter filter = AllCSSRules);
 
         // Given a CSS keyword in the range (xx-small to -webkit-xxx-large), this function will return
         // the correct font size scaled relative to the user's default (medium).
-        static float fontSizeForKeyword(Document*, int keyword, bool monospace);
-        
+        static float fontSizeForKeyword(Document*, int keyword, bool shouldUseFixedDefaultSize);
+
+        // Given a font size in pixel, this function will return legacy font size between 1 and 7.
+        static int legacyFontSize(Document*, int pixelFontSize, bool shouldUseFixedDefaultSize);
+
     private:
+
         // When the CSS keyword "larger" is used, this function will attempt to match within the keyword
         // table, and failing that, will simply multiply by 1.2.
         float largerFontSize(float size, bool quirksMode) const;
@@ -126,11 +139,11 @@ public:
 
     public:
         void setStyle(PassRefPtr<RenderStyle> s) { m_style = s; } // Used by the document when setting up its root style.
-        void setFontSize(FontDescription&, float size);
 
         void applyPropertyToStyle(int id, CSSValue*, RenderStyle*);
 
     private:
+        void setFontSize(FontDescription&, float size);
         static float getComputedSizeFromSpecifiedSize(Document*, RenderStyle*, bool isAbsoluteSize, float specifiedSize, bool useSVGZoomRules);
 
     public:
@@ -150,11 +163,8 @@ public:
         void allVisitedStateChanged() { m_checker.allVisitedStateChanged(); }
         void visitedStateChanged(LinkHash visitedHash) { m_checker.visitedStateChanged(visitedHash); }
 
-        void addVariables(CSSVariablesRule* variables);
-        CSSValue* resolveVariableDependentValue(CSSVariableDependentValue*);
-        void resolveVariablesForDeclaration(CSSMutableStyleDeclaration* decl, CSSMutableStyleDeclaration* newDecl, HashSet<String>& usedBlockVariables);
-
         void addKeyframeStyle(PassRefPtr<WebKitCSSKeyframesRule> rule);
+        void addPageStyle(PassRefPtr<CSSPageRule>);
 
         static bool createTransformOperations(CSSValue* inValue, RenderStyle* inStyle, RenderStyle* rootStyle, TransformOperations& outOperations);
 
@@ -166,19 +176,27 @@ public:
         void checkForZoomChange(RenderStyle*, RenderStyle* parentStyle);
         void checkForTextSizeAdjust();
 
-        void adjustRenderStyle(RenderStyle*, Element*);
+        void adjustRenderStyle(RenderStyle* styleToAdjust, RenderStyle* parentStyle, Element*);
 
         void addMatchedRule(CSSRuleData* rule) { m_matchedRules.append(rule); }
         void addMatchedDeclaration(CSSMutableStyleDeclaration* decl);
 
-        void matchRules(CSSRuleSet*, int& firstRuleIndex, int& lastRuleIndex);
-        void matchRulesForList(CSSRuleDataList*, int& firstRuleIndex, int& lastRuleIndex);
+        void matchRules(CSSRuleSet*, int& firstRuleIndex, int& lastRuleIndex, bool includeEmptyRules);
+        void matchRulesForList(CSSRuleDataList*, int& firstRuleIndex, int& lastRuleIndex, bool includeEmptyRules);
         void sortMatchedRules(unsigned start, unsigned end);
 
-        void applyDeclarations(bool firstPass, bool important, int startIndex, int endIndex);
+        template <bool firstPass>
+        void applyDeclarations(bool important, int startIndex, int endIndex);
+
+        void matchPageRules(CSSRuleSet*, bool isLeftPage, bool isFirstPage, const String& pageName);
+        void matchPageRulesForList(CSSRuleDataList*, bool isLeftPage, bool isFirstPage, const String& pageName);
+        bool isLeftPage(int pageIndex) const;
+        bool isRightPage(int pageIndex) const { return !isLeftPage(pageIndex); }
+        bool isFirstPage(int pageIndex) const;
+        String pageName(int pageIndex) const;
         
-        CSSRuleSet* m_authorStyle;
-        CSSRuleSet* m_userStyle;
+        OwnPtr<CSSRuleSet> m_authorStyle;
+        OwnPtr<CSSRuleSet> m_userStyle;
 
         bool m_hasUAAppearance;
         BorderData m_borderData;
@@ -208,6 +226,7 @@ public:
             Document* m_document;
             bool m_strictParsing;
             bool m_collectRulesOnly;
+            bool m_sameOriginOnly;
             PseudoId m_pseudoStyle;
             bool m_documentIsHTML;
             mutable bool m_matchVisitedPseudoClass;
@@ -217,22 +236,20 @@ public:
     private:
         static RenderStyle* s_styleNotYetAvailable;
 
-        void init();
-
         void matchUARules(int& firstUARule, int& lastUARule);
         void updateFont();
         void cacheBorderAndBackground();
 
-        void mapFillAttachment(FillLayer*, CSSValue*);
-        void mapFillClip(FillLayer*, CSSValue*);
-        void mapFillComposite(FillLayer*, CSSValue*);
-        void mapFillOrigin(FillLayer*, CSSValue*);
-        void mapFillImage(FillLayer*, CSSValue*);
-        void mapFillRepeatX(FillLayer*, CSSValue*);
-        void mapFillRepeatY(FillLayer*, CSSValue*);
-        void mapFillSize(FillLayer*, CSSValue*);
-        void mapFillXPosition(FillLayer*, CSSValue*);
-        void mapFillYPosition(FillLayer*, CSSValue*);
+        void mapFillAttachment(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillClip(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillComposite(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillOrigin(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillImage(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillRepeatX(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillRepeatY(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillSize(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillXPosition(CSSPropertyID, FillLayer*, CSSValue*);
+        void mapFillYPosition(CSSPropertyID, FillLayer*, CSSValue*);
 
         void mapAnimationDelay(Animation*, CSSValue*);
         void mapAnimationDirection(Animation*, CSSValue*);
@@ -244,27 +261,21 @@ public:
         void mapAnimationProperty(Animation*, CSSValue*);
         void mapAnimationTimingFunction(Animation*, CSSValue*);
 
-        void mapNinePieceImage(CSSValue*, NinePieceImage&);
+        void mapNinePieceImage(CSSPropertyID, CSSValue*, NinePieceImage&);
 
         void applyProperty(int id, CSSValue*);
+        void applyPageSizeProperty(CSSValue*);
+        bool pageSizeFromName(CSSPrimitiveValue*, CSSPrimitiveValue*, Length& width, Length& height);
+        Length mmLength(double mm);
+        Length inchLength(double inch);
 #if ENABLE(SVG)
         void applySVGProperty(int id, CSSValue*);
 #endif
 
-        StyleImage* styleImage(CSSValue* value);
-
-
-        EInsideLink currentElementLinkState() const
-        {
-            if (!m_haveCachedLinkState) {
-                m_cachedLinkState = m_checker.determineLinkState(m_element);
-                m_haveCachedLinkState = true;
-            }
-            return m_cachedLinkState;
-        }
-
-        mutable EInsideLink m_cachedLinkState;
-        mutable bool m_haveCachedLinkState;
+        void loadPendingImages();
+        
+        StyleImage* styleImage(CSSPropertyID, CSSValue* value);
+        StyleImage* cachedOrPendingFromValue(CSSPropertyID property, CSSImageValue* value);
 
         // We collect the set of decls that match in |m_matchedDecls|.  We then walk the
         // set of matched decls four times, once for those properties that others depend on (like font-size),
@@ -277,8 +288,10 @@ public:
         Vector<CSSRuleData*, 32> m_matchedRules;
 
         RefPtr<CSSRuleList> m_ruleList;
+        
+        HashSet<int> m_pendingImageProperties; // Hash of CSSPropertyIDs
 
-        MediaQueryEvaluator* m_medium;
+        OwnPtr<MediaQueryEvaluator> m_medium;
         RefPtr<RenderStyle> m_rootDefaultStyle;
 
         PseudoId m_dynamicPseudo;
@@ -290,7 +303,8 @@ public:
         RenderStyle* m_rootElementStyle;
         Element* m_element;
         StyledElement* m_styledElement;
-        Node* m_parentNode;
+        EInsideLink m_elementLinkState;
+        ContainerNode* m_parentNode;
         CSSValue* m_lineHeightValue;
         bool m_fontDirty;
         bool m_matchAuthorAndUserStyles;
@@ -299,9 +313,6 @@ public:
         HashSet<AtomicStringImpl*> m_selectorAttrs;
         Vector<CSSMutableStyleDeclaration*> m_additionalAttributeStyleDecls;
         Vector<MediaQueryResult*> m_viewportDependentMediaQueryResults;
-        
-        HashMap<String, CSSVariablesRule*> m_variablesMap;
-        HashMap<CSSMutableStyleDeclaration*, RefPtr<CSSMutableStyleDeclaration> > m_resolvedVariablesDeclarations;
     };
 
     class CSSRuleData : public Noncopyable {
