@@ -27,17 +27,33 @@
 #include "CSSRuleList.h"
 #include "Document.h"
 #include "ExceptionCode.h"
+#include "HTMLNames.h"
 #include "Node.h"
+#include "SVGNames.h"
 #include "SecurityOrigin.h"
 #include "TextEncoding.h"
 #include <wtf/Deque.h>
 
 namespace WebCore {
 
+#if !ASSERT_DISABLED
+static bool isAcceptableCSSStyleSheetParent(Node* parentNode)
+{
+    // Only these nodes can be parents of StyleSheets, and they need to call clearOwnerNode() when moved out of document.
+    return !parentNode
+        || parentNode->isDocumentNode()
+        || parentNode->hasTagName(HTMLNames::linkTag)
+        || parentNode->hasTagName(HTMLNames::styleTag)
+#if ENABLE(SVG)
+        || parentNode->hasTagName(SVGNames::styleTag)
+#endif    
+        || parentNode->nodeType() == Node::PROCESSING_INSTRUCTION_NODE;
+}
+#endif
+
 CSSStyleSheet::CSSStyleSheet(CSSStyleSheet* parentSheet, const String& href, const KURL& baseURL, const String& charset)
     : StyleSheet(parentSheet, href, baseURL)
-    , m_doc(parentSheet ? parentSheet->doc() : 0)
-    , m_namespaces(0)
+    , m_document(parentSheet ? parentSheet->document() : 0)
     , m_charset(charset)
     , m_loadCompleted(false)
     , m_strictParsing(!parentSheet || parentSheet->useStrictParsing())
@@ -48,32 +64,30 @@ CSSStyleSheet::CSSStyleSheet(CSSStyleSheet* parentSheet, const String& href, con
 
 CSSStyleSheet::CSSStyleSheet(Node* parentNode, const String& href, const KURL& baseURL, const String& charset)
     : StyleSheet(parentNode, href, baseURL)
-    , m_doc(parentNode->document())
-    , m_namespaces(0)
+    , m_document(parentNode->document())
     , m_charset(charset)
     , m_loadCompleted(false)
     , m_strictParsing(false)
     , m_isUserStyleSheet(false)
     , m_hasSyntacticallyValidCSSHeader(true)
 {
+    ASSERT(isAcceptableCSSStyleSheetParent(parentNode));
 }
 
 CSSStyleSheet::CSSStyleSheet(CSSRule* ownerRule, const String& href, const KURL& baseURL, const String& charset)
     : StyleSheet(ownerRule, href, baseURL)
-    , m_namespaces(0)
     , m_charset(charset)
     , m_loadCompleted(false)
     , m_strictParsing(!ownerRule || ownerRule->useStrictParsing())
     , m_hasSyntacticallyValidCSSHeader(true)
 {
     CSSStyleSheet* parentSheet = ownerRule ? ownerRule->parentStyleSheet() : 0;
-    m_doc = parentSheet ? parentSheet->doc() : 0;
+    m_document = parentSheet ? parentSheet->document() : 0;
     m_isUserStyleSheet = parentSheet ? parentSheet->isUserStyleSheet() : false;
 }
 
 CSSStyleSheet::~CSSStyleSheet()
 {
-    delete m_namespaces;
 }
 
 CSSRule *CSSStyleSheet::ownerRule() const
@@ -96,9 +110,24 @@ unsigned CSSStyleSheet::insertRule(const String& rule, unsigned index, Exception
         return 0;
     }
 
-    // ###
-    // HIERARCHY_REQUEST_ERR: Raised if the rule cannot be inserted at the specified index e.g. if an
-    //@import rule is inserted after a standard rule set or other at-rule.
+    // Throw a HIERARCHY_REQUEST_ERR exception if the rule cannot be inserted at the specified index.  The best
+    // example of this is an @import rule inserted after regular rules.
+    if (index > 0) {
+        if (r->isImportRule()) {
+            // Check all the rules that come before this one to make sure they are only @charset and @import rules.
+            for (unsigned i = 0; i < index; ++i) {
+                if (!item(i)->isCharsetRule() && !item(i)->isImportRule()) {
+                    ec = HIERARCHY_REQUEST_ERR;
+                    return 0;
+                }
+            }
+        } else if (r->isCharsetRule()) {
+            // The @charset rule has to come first and there can be only one.
+            ec = HIERARCHY_REQUEST_ERR;
+            return 0;
+        }
+    }
+
     insert(index, r.release());
     
     styleSheetChanged();
@@ -119,10 +148,10 @@ int CSSStyleSheet::addRule(const String& selector, const String& style, Exceptio
     return addRule(selector, style, length(), ec);
 }
 
-
 PassRefPtr<CSSRuleList> CSSStyleSheet::cssRules(bool omitCharsetRules)
 {
-    if (doc() && !doc()->securityOrigin()->canRequest(baseURL()))
+    KURL url = finalURL();
+    if (!url.isEmpty() && document() && !document()->securityOrigin()->canRequest(url))
         return 0;
     return CSSRuleList::create(this, omitCharsetRules);
 }
@@ -135,7 +164,6 @@ void CSSStyleSheet::deleteRule(unsigned index, ExceptionCode& ec)
     }
 
     ec = 0;
-    item(index)->setParent(0); 
     remove(index);
     styleSheetChanged();
 }
@@ -145,7 +173,7 @@ void CSSStyleSheet::addNamespace(CSSParser* p, const AtomicString& prefix, const
     if (uri.isNull())
         return;
 
-    m_namespaces = new CSSNamespace(prefix, uri, m_namespaces);
+    m_namespaces = adoptPtr(new CSSNamespace(prefix, uri, m_namespaces.release()));
     
     if (prefix.isEmpty())
         // Set the default namespace on the parser so that selectors that omit namespace info will
@@ -160,18 +188,22 @@ const AtomicString& CSSStyleSheet::determineNamespace(const AtomicString& prefix
     if (prefix == starAtom)
         return starAtom; // We'll match any namespace.
     if (m_namespaces) {
-        CSSNamespace* ns = m_namespaces->namespaceForPrefix(prefix);
-        if (ns)
-            return ns->uri();
+        if (CSSNamespace* namespaceForPrefix = m_namespaces->namespaceForPrefix(prefix))
+            return namespaceForPrefix->uri;
     }
-    return nullAtom; // Assume we wont match any namespaces.
+    return nullAtom; // Assume we won't match any namespaces.
 }
 
 bool CSSStyleSheet::parseString(const String &string, bool strict)
+{
+    return parseStringAtLine(string, strict, 0);
+}
+
+bool CSSStyleSheet::parseStringAtLine(const String& string, bool strict, int startLineNumber)
 {
     setStrictParsing(strict);
     CSSParser p(strict);
-    p.parseSheet(this, string);
+    p.parseSheet(this, string, startLineNumber);
     return true;
 }
 
@@ -193,7 +225,8 @@ void CSSStyleSheet::checkLoaded()
     if (parent())
         parent()->checkLoaded();
 
-    // Avoid |this| being deleted by scripts that run via HTMLTokenizer::executeScriptsWaitingForStylesheets().
+    // Avoid |this| being deleted by scripts that run via
+    // ScriptableDocumentParser::executeScriptsWaitingForStylesheets().
     // See <rdar://problem/6622300>.
     RefPtr<CSSStyleSheet> protector(this);
     m_loadCompleted = ownerNode() ? ownerNode()->sheetLoaded() : true;
@@ -204,14 +237,14 @@ void CSSStyleSheet::styleSheetChanged()
     StyleBase* root = this;
     while (StyleBase* parent = root->parent())
         root = parent;
-    Document* documentToUpdate = root->isCSSStyleSheet() ? static_cast<CSSStyleSheet*>(root)->doc() : 0;
+    Document* documentToUpdate = root->isCSSStyleSheet() ? static_cast<CSSStyleSheet*>(root)->document() : 0;
     
     /* FIXME: We don't need to do everything updateStyleSelector does,
      * basically we just need to recreate the document's selector with the
      * already existing style sheets.
      */
     if (documentToUpdate)
-        documentToUpdate->updateStyleSelector();
+        documentToUpdate->styleSelectorChanged(DeferRecalcStyle);
 }
 
 KURL CSSStyleSheet::completeURL(const String& url) const
@@ -231,8 +264,7 @@ void CSSStyleSheet::addSubresourceStyleURLs(ListHashSet<KURL>& urls)
     styleSheetQueue.append(this);
 
     while (!styleSheetQueue.isEmpty()) {
-        CSSStyleSheet* styleSheet = styleSheetQueue.first();
-        styleSheetQueue.removeFirst();
+        CSSStyleSheet* styleSheet = styleSheetQueue.takeFirst();
 
         for (unsigned i = 0; i < styleSheet->length(); ++i) {
             StyleBase* styleBase = styleSheet->item(i);
