@@ -1,6 +1,6 @@
 /*
  * (C) 1999-2003 Lars Knoll (knoll@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -54,10 +54,11 @@ public:
     void prepend(PassRefPtr<CSSValue>);
     bool removeAll(CSSValue*);
     bool hasValue(CSSValue*);
+    PassRefPtr<CSSValueList> copy();
 
     virtual String cssText() const;
 
-    CSSParserValueList* createParserValueList() const;
+    PassOwnPtr<CSSParserValueList> createParserValueList() const;
 
     virtual void addSubresourceStyleURLs(ListHashSet<KURL>&, const CSSStyleSheet*);
 
