@@ -23,8 +23,9 @@
 #include "config.h"
 #include "Event.h"
 
-#include "AtomicString.h"
+#include "UserGestureIndicator.h"
 #include <wtf/CurrentTime.h>
+#include <wtf/text/AtomicString.h>
 
 namespace WebCore {
 
@@ -32,13 +33,13 @@ Event::Event()
     : m_canBubble(false)
     , m_cancelable(false)
     , m_propagationStopped(false)
+    , m_immediatePropagationStopped(false)
     , m_defaultPrevented(false)
     , m_defaultHandled(false)
     , m_cancelBubble(false)
-    , m_createdByDOM(false)
     , m_eventPhase(0)
     , m_currentTarget(0)
-    , m_createTime(static_cast<DOMTimeStamp>(currentTime() * 1000.0))
+    , m_createTime(convertSecondsToDOMTimeStamp(currentTime()))
 {
 }
 
@@ -47,13 +48,13 @@ Event::Event(const AtomicString& eventType, bool canBubbleArg, bool cancelableAr
     , m_canBubble(canBubbleArg)
     , m_cancelable(cancelableArg)
     , m_propagationStopped(false)
+    , m_immediatePropagationStopped(false)
     , m_defaultPrevented(false)
     , m_defaultHandled(false)
     , m_cancelBubble(false)
-    , m_createdByDOM(false)
     , m_eventPhase(0)
     , m_currentTarget(0)
-    , m_createTime(static_cast<DOMTimeStamp>(currentTime() * 1000.0))
+    , m_createTime(convertSecondsToDOMTimeStamp(currentTime()))
 {
 }
 
@@ -71,6 +72,11 @@ void Event::initEvent(const AtomicString& eventTypeArg, bool canBubbleArg, bool
     m_cancelable = cancelableArg;
 }
 
+bool Event::isCustomEvent() const
+{
+    return false;
+}
+
 bool Event::isUIEvent() const
 {
     return false;
@@ -166,6 +172,11 @@ bool Event::isBeforeLoadEvent() const
     return false;
 }
 
+bool Event::isHashChangeEvent() const
+{
+    return false;
+}
+
 #if ENABLE(SVG)
 bool Event::isSVGZoomEvent() const
 {
@@ -180,6 +191,18 @@ bool Event::isStorageEvent() const
 }
 #endif
 
+#if ENABLE(INDEXED_DATABASE)
+bool Event::isIDBErrorEvent() const
+{
+    return false;
+}
+
+bool Event::isIDBSuccessEvent() const
+{
+    return false;
+}
+#endif
+
 #if ENABLE(WORKERS)
 bool Event::isErrorEvent() const
 {
@@ -194,9 +217,35 @@ bool Event::isTouchEvent() const
 }
 #endif
 
+#if ENABLE(DEVICE_ORIENTATION)
+bool Event::isDeviceMotionEvent() const
+{
+    return false;
+}
+
+bool Event::isDeviceOrientationEvent() const
+{
+    return false;
+}
+#endif
+
+#if ENABLE(WEB_AUDIO)
+bool Event::isAudioProcessingEvent() const
+{
+    return false;
+}
+#endif
+
+#if ENABLE(INPUT_SPEECH)
+bool Event::isSpeechInputEvent() const
+{
+    return false;
+}
+#endif
+
 bool Event::fromUserGesture()
 {
-    if (createdByDOM())
+    if (!UserGestureIndicator::processingUserGesture())
         return false;
 
     const AtomicString& type = this->type();
