@@ -24,17 +24,15 @@
 #ifndef Event_h
 #define Event_h
 
-#include "AtomicString.h"
+#include "DOMTimeStamp.h"
 #include "EventTarget.h"
 #include <wtf/RefCounted.h>
+#include <wtf/text/AtomicString.h>
 
 namespace WebCore {
 
     class Clipboard;
 
-    // FIXME: this should probably defined elsewhere.
-    typedef unsigned long long DOMTimeStamp;
-
     class Event : public RefCounted<Event> {
     public:
         enum PhaseType { 
@@ -75,7 +73,7 @@ namespace WebCore {
         void initEvent(const AtomicString& type, bool canBubble, bool cancelable);
 
         const AtomicString& type() const { return m_type; }
-
+        
         EventTarget* target() const { return m_target.get(); }
         void setTarget(PassRefPtr<EventTarget>);
 
@@ -88,8 +86,10 @@ namespace WebCore {
         bool bubbles() const { return m_canBubble; }
         bool cancelable() const { return m_cancelable; }
         DOMTimeStamp timeStamp() const { return m_createTime; }
-        void stopPropagation() { m_propagationStopped = true; }
 
+        void stopPropagation() { m_propagationStopped = true; }
+        void stopImmediatePropagation() { m_immediatePropagationStopped = true; }
+        
         // IE Extensions
         EventTarget* srcElement() const { return target(); } // MSIE extension - "the object that fired the event"
 
@@ -98,6 +98,7 @@ namespace WebCore {
 
         Clipboard* clipboardData() const { return isClipboardEvent() ? clipboard() : 0; }
 
+        virtual bool isCustomEvent() const;
         virtual bool isUIEvent() const;
         virtual bool isMouseEvent() const;
         virtual bool isMutationEvent() const;
@@ -117,21 +118,37 @@ namespace WebCore {
         virtual bool isWebKitAnimationEvent() const;
         virtual bool isWebKitTransitionEvent() const;
         virtual bool isBeforeLoadEvent() const;
+        virtual bool isHashChangeEvent() const;
 #if ENABLE(SVG)
         virtual bool isSVGZoomEvent() const;
 #endif
 #if ENABLE(DOM_STORAGE)
         virtual bool isStorageEvent() const;
 #endif
+#if ENABLE(INDEXED_DATABASE)
+        virtual bool isIDBErrorEvent() const;
+        virtual bool isIDBSuccessEvent() const;
+#endif
+#if ENABLE(WEB_AUDIO)
+        virtual bool isAudioProcessingEvent() const;
+#endif
 #if ENABLE(WORKERS)
         virtual bool isErrorEvent() const;
 #endif
 #if ENABLE(TOUCH_EVENTS)
         virtual bool isTouchEvent() const;
+#endif
+#if ENABLE(DEVICE_ORIENTATION)
+        virtual bool isDeviceMotionEvent() const;
+        virtual bool isDeviceOrientationEvent() const;
+#endif
+#if ENABLE(INPUT_SPEECH)
+        virtual bool isSpeechInputEvent() const;
 #endif
         bool fromUserGesture();
         
-        bool propagationStopped() const { return m_propagationStopped; }
+        bool propagationStopped() const { return m_propagationStopped || m_immediatePropagationStopped; }
+        bool immediatePropagationStopped() const { return m_immediatePropagationStopped; }
 
         bool defaultPrevented() const { return m_defaultPrevented; }
         void preventDefault() { if (m_cancelable) m_defaultPrevented = true; }
@@ -151,9 +168,6 @@ namespace WebCore {
 
         virtual Clipboard* clipboard() const { return 0; }
 
-        bool createdByDOM() const { return m_createdByDOM; }
-        void setCreatedByDOM(bool createdByDOM) { m_createdByDOM = createdByDOM; }
-
     protected:
         Event();
         Event(const AtomicString& type, bool canBubble, bool cancelable);
@@ -167,13 +181,11 @@ namespace WebCore {
         bool m_cancelable;
 
         bool m_propagationStopped;
+        bool m_immediatePropagationStopped;
         bool m_defaultPrevented;
         bool m_defaultHandled;
         bool m_cancelBubble;
 
-        // Whether this event was created by document.createEvent().
-        bool m_createdByDOM;
-
         unsigned short m_eventPhase;
         EventTarget* m_currentTarget;
         RefPtr<EventTarget> m_target;
