@@ -21,7 +21,6 @@
 #ifndef EventListener_h
 #define EventListener_h
 
-#include "PlatformString.h"
 #include <wtf/RefCounted.h>
 
 namespace JSC {
@@ -31,26 +30,31 @@ namespace JSC {
 
 namespace WebCore {
 
+    class ScriptExecutionContext;
     class Event;
 
     class EventListener : public RefCounted<EventListener> {
     public:
-        enum Type { JSEventListenerType, 
-                    ImageEventListenerType, 
-                    InspectorDOMAgentType, 
-                    ObjCEventListenerType, 
-                    ConditionEventListenerType };
-                    
+        enum Type {
+            JSEventListenerType, 
+            ImageEventListenerType, 
+            InspectorDOMAgentType,
+            InspectorDOMStorageResourceType,
+            ObjCEventListenerType,
+            CPPEventListenerType,
+            ConditionEventListenerType,
+            GObjectEventListenerType,
+            NativeEventListenerType
+        };
+
         virtual ~EventListener() { }
         virtual bool operator==(const EventListener&) = 0;
-        virtual void handleEvent(Event*, bool isWindowEvent = false) = 0;
-        // Return true to indicate that the error is handled.
-        virtual bool reportError(const String& /*message*/, const String& /*url*/, int /*lineNumber*/) { return false; }
+        virtual void handleEvent(ScriptExecutionContext*, Event*) = 0;
         virtual bool wasCreatedFromMarkup() const { return false; }
 
 #if USE(JSC)
-        virtual JSC::JSObject* jsFunction() const { return 0; }
         virtual void markJSFunction(JSC::MarkStack&) { }
+        virtual void invalidateJSFunction(JSC::JSObject*) { }
 #endif
 
         bool isAttribute() const { return virtualisAttribute(); }
@@ -68,10 +72,6 @@ namespace WebCore {
         Type m_type;
     };
 
-#if USE(JSC)
-    inline void markIfNotNull(JSC::MarkStack& markStack, EventListener* listener) { if (listener) listener->markJSFunction(markStack); }
-#endif
-
 }
 
 #endif
