@@ -32,22 +32,26 @@
 #ifndef EventTarget_h
 #define EventTarget_h
 
-#include "AtomicStringHash.h"
 #include "EventNames.h"
 #include "RegisteredEventListener.h"
 #include <wtf/Forward.h>
 #include <wtf/HashMap.h>
+#include <wtf/text/AtomicStringHash.h>
 
 namespace WebCore {
 
     class AbstractWorker;
-    class AtomicString;
     class DedicatedWorkerContext;
     class DOMApplicationCache;
     class DOMWindow;
     class Event;
     class EventListener;
     class EventSource;
+    class FileReader;
+    class FileWriter;
+    class IDBRequest;
+    class IDBTransaction;
+    class JavaScriptAudioNode;
     class MessagePort;
     class Node;
     class Notification;
@@ -80,6 +84,7 @@ namespace WebCore {
     typedef HashMap<AtomicString, EventListenerVector*> EventListenerMap;
 
     struct EventTargetData : Noncopyable {
+        EventTargetData();
         ~EventTargetData();
 
         EventListenerMap eventListenerMap;
@@ -111,6 +116,11 @@ namespace WebCore {
         virtual SharedWorker* toSharedWorker();
         virtual SharedWorkerContext* toSharedWorkerContext();
 #endif
+
+#if ENABLE(WEB_AUDIO)
+        virtual JavaScriptAudioNode* toJavaScriptAudioNode();
+#endif
+
 #if ENABLE(WEB_SOCKETS)
         virtual WebSocket* toWebSocket();
 #endif
@@ -118,6 +128,17 @@ namespace WebCore {
 #if ENABLE(NOTIFICATIONS)
         virtual Notification* toNotification();
 #endif
+#if ENABLE(BLOB)
+        virtual FileReader* toFileReader();
+#endif
+#if ENABLE(FILE_SYSTEM)
+        virtual FileWriter* toFileWriter();
+#endif
+
+#if ENABLE(INDEXED_DATABASE)
+        virtual IDBRequest* toIDBRequest();
+        virtual IDBTransaction* toIDBTransaction();
+#endif
 
         virtual ScriptExecutionContext* scriptExecutionContext() const = 0;
 
@@ -153,15 +174,23 @@ namespace WebCore {
     private:
         virtual void refEventTarget() = 0;
         virtual void derefEventTarget() = 0;
+        
+        void fireEventListeners(Event*, EventTargetData*, EventListenerVector&);
     };
 
+    // FIXME: These macros should be split into separate DEFINE and DECLARE
+    // macros to avoid causing so many header includes.
     #define DEFINE_ATTRIBUTE_EVENT_LISTENER(attribute) \
         EventListener* on##attribute() { return getAttributeEventListener(eventNames().attribute##Event); } \
         void setOn##attribute(PassRefPtr<EventListener> listener) { setAttributeEventListener(eventNames().attribute##Event, listener); } \
 
-    #define DEFINE_VIRTUAL_ATTRIBUTE_EVENT_LISTENER(attribute) \
-        virtual EventListener* on##attribute() { return getAttributeEventListener(eventNames().attribute##Event); } \
-        virtual void setOn##attribute(PassRefPtr<EventListener> listener) { setAttributeEventListener(eventNames().attribute##Event, listener); } \
+    #define DECLARE_VIRTUAL_ATTRIBUTE_EVENT_LISTENER(attribute) \
+        virtual EventListener* on##attribute(); \
+        virtual void setOn##attribute(PassRefPtr<EventListener> listener); \
+
+    #define DEFINE_VIRTUAL_ATTRIBUTE_EVENT_LISTENER(type, attribute) \
+        EventListener* type::on##attribute() { return getAttributeEventListener(eventNames().attribute##Event); } \
+        void type::setOn##attribute(PassRefPtr<EventListener> listener) { setAttributeEventListener(eventNames().attribute##Event, listener); } \
 
     #define DEFINE_WINDOW_ATTRIBUTE_EVENT_LISTENER(attribute) \
         EventListener* on##attribute() { return document()->getWindowAttributeEventListener(eventNames().attribute##Event); } \
