@@ -90,10 +90,15 @@ void InsertLineBreakCommand::doApply()
 {
     deleteSelection();
     VisibleSelection selection = endingSelection();
-    if (selection.isNone())
+    if (!selection.isNonOrphanedCaretOrRange())
         return;
     
     VisiblePosition caret(selection.visibleStart());
+    // FIXME: If the node is hidden, we should still be able to insert text. 
+    // For now, we return to avoid a crash.  https://bugs.webkit.org/show_bug.cgi?id=40342
+    if (caret.isNull())
+        return;
+
     Position pos(caret.deepEquivalent());
 
     pos = positionAvoidingSpecialElementBoundary(pos);
@@ -159,15 +164,15 @@ void InsertLineBreakCommand::doApply()
     }
 
     // Handle the case where there is a typing style.
-    
-    CSSMutableStyleDeclaration* typingStyle = document()->frame()->typingStyle();
-    
-    if (typingStyle && typingStyle->length() > 0) {
+
+    RefPtr<EditingStyle> typingStyle = document()->frame()->selection()->typingStyle();
+
+    if (typingStyle && !typingStyle->isEmpty()) {
         // Apply the typing style to the inserted line break, so that if the selection
         // leaves and then comes back, new input will have the right style.
         // FIXME: We shouldn't always apply the typing style to the line break here,
         // see <rdar://problem/5794462>.
-        applyStyle(typingStyle, firstDeepEditingPositionForNode(nodeToInsert.get()), lastDeepEditingPositionForNode(nodeToInsert.get()));
+        applyStyle(typingStyle.get(), firstDeepEditingPositionForNode(nodeToInsert.get()), lastDeepEditingPositionForNode(nodeToInsert.get()));
         // Even though this applyStyle operates on a Range, it still sets an endingSelection().
         // It tries to set a VisibleSelection around the content it operated on. So, that VisibleSelection
         // will either (a) select the line break we inserted, or it will (b) be a caret just 
