@@ -26,12 +26,25 @@
 #ifndef TextIterator_h
 #define TextIterator_h
 
+#include "FindOptions.h"
 #include "InlineTextBox.h"
 #include "Range.h"
 #include <wtf/Vector.h>
 
 namespace WebCore {
 
+class RenderText;
+class RenderTextFragment;
+
+enum TextIteratorBehavior {
+    TextIteratorDefaultBehavior = 0,
+    TextIteratorEmitsCharactersBetweenAllVisiblePositions = 1 << 0,
+    TextIteratorEntersTextControls = 1 << 1,
+    TextIteratorEmitsTextsWithoutTranscoding = 1 << 2,
+    TextIteratorEndsAtEditingBoundary = 1 << 3,
+    TextIteratorIgnoresStyleVisibility = 1 << 4
+};
+    
 // FIXME: Can't really answer this question correctly without knowing the white-space mode.
 // FIXME: Move this somewhere else in the editing directory. It doesn't belong here.
 inline bool isCollapsibleWhitespace(UChar c)
@@ -45,13 +58,16 @@ inline bool isCollapsibleWhitespace(UChar c)
     }
 }
 
-String plainText(const Range*);
-UChar* plainTextToMallocAllocatedBuffer(const Range*, unsigned& bufferLength, bool isDisplayString);
+String plainText(const Range*, TextIteratorBehavior defaultBehavior = TextIteratorDefaultBehavior);
+UChar* plainTextToMallocAllocatedBuffer(const Range*, unsigned& bufferLength, bool isDisplayString, TextIteratorBehavior = TextIteratorDefaultBehavior);
+PassRefPtr<Range> findPlainText(const Range*, const String&, FindOptions);
+// FIXME: Switch callers over to the FindOptions version and retire this one.
 PassRefPtr<Range> findPlainText(const Range*, const String&, bool forward, bool caseSensitive);
 
 class BitStack {
 public:
     BitStack();
+    ~BitStack();
 
     void push(bool);
     void pop();
@@ -71,8 +87,9 @@ private:
 class TextIterator {
 public:
     TextIterator();
-    explicit TextIterator(const Range*, bool emitCharactersBetweenAllVisiblePositions = false, bool enterTextControls = false);
-    
+    ~TextIterator();
+    explicit TextIterator(const Range*, TextIteratorBehavior = TextIteratorDefaultBehavior);
+
     bool atEnd() const { return !m_positionNode; }
     void advance();
     
@@ -95,7 +112,10 @@ private:
     bool handleReplacedElement();
     bool handleNonTextNode();
     void handleTextBox();
+    void handleTextNodeFirstLetter(RenderTextFragment*);
+    bool hasVisibleTextNode(RenderText*);
     void emitCharacter(UChar, Node* textNode, Node* offsetBaseNode, int textStartOffset, int textEndOffset);
+    void emitText(Node* textNode, RenderObject* renderObject, int textStartOffset, int textEndOffset);
     void emitText(Node* textNode, int textStartOffset, int textEndOffset);
     
     // Current position, not necessarily of the text being returned, but position
@@ -120,11 +140,18 @@ private:
     mutable int m_positionEndOffset;
     const UChar* m_textCharacters;
     int m_textLength;
-    
+    // Hold string m_textCharacters points to so we ensure it won't be deleted.
+    String m_text;
+
     // Used when there is still some pending text from the current node; when these
     // are false and 0, we go back to normal iterating.
-    bool m_needAnotherNewline;
+    bool m_needsAnotherNewline;
     InlineTextBox* m_textBox;
+    // Used when iteration over :first-letter text to save pointer to
+    // remaining text box.
+    InlineTextBox* m_remainingTextBox;
+    // Used to point to RenderText object for :first-letter.
+    RenderText *m_firstLetterText;
     
     // Used to do the whitespace collapsing logic.
     Node* m_lastTextNode;    
@@ -139,14 +166,21 @@ private:
     size_t m_sortedTextBoxesPosition;
     
     // Used when deciding whether to emit a "positioning" (e.g. newline) before any other content
-    bool m_haveEmitted;
+    bool m_hasEmitted;
     
     // Used by selection preservation code.  There should be one character emitted between every VisiblePosition
     // in the Range used to create the TextIterator.
     // FIXME <rdar://problem/6028818>: This functionality should eventually be phased out when we rewrite 
     // moveParagraphs to not clone/destroy moved content.
-    bool m_emitCharactersBetweenAllVisiblePositions;
-    bool m_enterTextControls;
+    bool m_emitsCharactersBetweenAllVisiblePositions;
+    bool m_entersTextControls;
+
+    // Used when we want texts for copying, pasting, and transposing.
+    bool m_emitsTextWithoutTranscoding;
+    // Used when deciding text fragment created by :first-letter should be looked into.
+    bool m_handledFirstLetter;
+    // Used when the visibility of the style should not affect text gathering.
+    bool m_ignoresStyleVisibility;
 };
 
 // Iterates through the DOM range, returning all the text, and 0-length boundaries
@@ -155,7 +189,7 @@ private:
 class SimplifiedBackwardsTextIterator {
 public:
     SimplifiedBackwardsTextIterator();
-    explicit SimplifiedBackwardsTextIterator(const Range*);
+    explicit SimplifiedBackwardsTextIterator(const Range*, TextIteratorBehavior = TextIteratorDefaultBehavior);
     
     bool atEnd() const { return !m_positionNode; }
     void advance();
@@ -171,7 +205,11 @@ private:
     bool handleReplacedElement();
     bool handleNonTextNode();
     void emitCharacter(UChar, Node*, int startOffset, int endOffset);
-    
+    bool crossesEditingBoundary(Node*) const;
+    bool setCurrentNode(Node*);
+    void clearCurrentNode();
+
+    TextIteratorBehavior m_behavior;
     // Current position, not necessarily of the text being returned, but position
     // as we walk through the DOM tree.
     Node* m_node;
@@ -210,7 +248,7 @@ private:
 class CharacterIterator {
 public:
     CharacterIterator();
-    explicit CharacterIterator(const Range*, bool emitCharactersBetweenAllVisiblePositions = false, bool enterTextControls = false);
+    explicit CharacterIterator(const Range*, TextIteratorBehavior = TextIteratorDefaultBehavior);
     
     void advance(int numCharacters);
     
@@ -235,7 +273,7 @@ private:
 class BackwardsCharacterIterator {
 public:
     BackwardsCharacterIterator();
-    explicit BackwardsCharacterIterator(const Range*);
+    explicit BackwardsCharacterIterator(const Range*, TextIteratorBehavior = TextIteratorDefaultBehavior);
 
     void advance(int);
 
@@ -244,6 +282,7 @@ public:
     PassRefPtr<Range> range() const;
 
 private:
+    TextIteratorBehavior m_behavior;
     int m_offset;
     int m_runOffset;
     bool m_atBreak;
@@ -257,6 +296,7 @@ class WordAwareIterator {
 public:
     WordAwareIterator();
     explicit WordAwareIterator(const Range*);
+    ~WordAwareIterator();
 
     bool atEnd() const { return !m_didLookAhead && m_textIterator.atEnd(); }
     void advance();
