@@ -86,11 +86,12 @@ void TypingCommand::deleteKeyPressed(Document *document, bool smartDelete, TextG
 {
     ASSERT(document);
     
-    Frame *frame = document->frame();
+    Frame* frame = document->frame();
     ASSERT(frame);
     
     EditCommand* lastEditCommand = frame->editor()->lastEditCommand();
     if (granularity == CharacterGranularity && isOpenForMoreTypingCommand(lastEditCommand)) {
+        updateSelectionIfDifferentFromCurrentSelection(static_cast<TypingCommand*>(lastEditCommand), frame);
         static_cast<TypingCommand*>(lastEditCommand)->deleteKeyPressed(granularity, killRing);
         return;
     }
@@ -105,11 +106,12 @@ void TypingCommand::forwardDeleteKeyPressed(Document *document, bool smartDelete
     // FIXME: Forward delete in TextEdit appears to open and close a new typing command.
     ASSERT(document);
     
-    Frame *frame = document->frame();
+    Frame* frame = document->frame();
     ASSERT(frame);
     
     EditCommand* lastEditCommand = frame->editor()->lastEditCommand();
     if (granularity == CharacterGranularity && isOpenForMoreTypingCommand(lastEditCommand)) {
+        updateSelectionIfDifferentFromCurrentSelection(static_cast<TypingCommand*>(lastEditCommand), frame);
         static_cast<TypingCommand*>(lastEditCommand)->forwardDeleteKeyPressed(granularity, killRing);
         return;
     }
@@ -119,6 +121,18 @@ void TypingCommand::forwardDeleteKeyPressed(Document *document, bool smartDelete
     typingCommand->apply();
 }
 
+void TypingCommand::updateSelectionIfDifferentFromCurrentSelection(TypingCommand* typingCommand, Frame* frame)
+{
+    ASSERT(frame);
+    VisibleSelection currentSelection = frame->selection()->selection();
+    if (currentSelection == typingCommand->endingSelection())
+        return;
+    
+    typingCommand->setStartingSelection(currentSelection);
+    typingCommand->setEndingSelection(currentSelection);
+}
+    
+
 void TypingCommand::insertText(Document* document, const String& text, bool selectInsertedText, bool insertedTextIsComposition)
 {
     ASSERT(document);
@@ -129,16 +143,21 @@ void TypingCommand::insertText(Document* document, const String& text, bool sele
     insertText(document, text, frame->selection()->selection(), selectInsertedText, insertedTextIsComposition);
 }
 
+// FIXME: We shouldn't need to take selectionForInsertion. It should be identical to SelectionController's current selection.
 void TypingCommand::insertText(Document* document, const String& text, const VisibleSelection& selectionForInsertion, bool selectInsertedText, bool insertedTextIsComposition)
 {
+#if REMOVE_MARKERS_UPON_EDITING
+    if (!text.isEmpty())
+        document->frame()->editor()->removeSpellAndCorrectionMarkersFromWordsToBeEdited(isSpaceOrNewline(text.characters()[0]));
+#endif
+
     ASSERT(document);
-    
+
     RefPtr<Frame> frame = document->frame();
     ASSERT(frame);
-    
+
     VisibleSelection currentSelection = frame->selection()->selection();
     bool changeSelection = currentSelection != selectionForInsertion;
-    
     String newText = text;
     Node* startNode = selectionForInsertion.start().node();
     
@@ -159,15 +178,11 @@ void TypingCommand::insertText(Document* document, const String& text, const Vis
     RefPtr<EditCommand> lastEditCommand = frame->editor()->lastEditCommand();
     if (isOpenForMoreTypingCommand(lastEditCommand.get())) {
         TypingCommand* lastTypingCommand = static_cast<TypingCommand*>(lastEditCommand.get());
-        if (changeSelection) {
+        if (lastTypingCommand->endingSelection() != selectionForInsertion) {
             lastTypingCommand->setStartingSelection(selectionForInsertion);
             lastTypingCommand->setEndingSelection(selectionForInsertion);
         }
         lastTypingCommand->insertText(newText, selectInsertedText);
-        if (changeSelection) {
-            lastTypingCommand->setEndingSelection(currentSelection);
-            frame->selection()->setSelection(currentSelection);
-        }
         return;
     }
 
@@ -187,7 +202,7 @@ void TypingCommand::insertLineBreak(Document *document)
 {
     ASSERT(document);
     
-    Frame *frame = document->frame();
+    Frame* frame = document->frame();
     ASSERT(frame);
     
     EditCommand* lastEditCommand = frame->editor()->lastEditCommand();
@@ -203,7 +218,7 @@ void TypingCommand::insertParagraphSeparatorInQuotedContent(Document *document)
 {
     ASSERT(document);
     
-    Frame *frame = document->frame();
+    Frame* frame = document->frame();
     ASSERT(frame);
     
     EditCommand* lastEditCommand = frame->editor()->lastEditCommand();
@@ -219,7 +234,7 @@ void TypingCommand::insertParagraphSeparator(Document *document)
 {
     ASSERT(document);
     
-    Frame *frame = document->frame();
+    Frame* frame = document->frame();
     ASSERT(frame);
     
     EditCommand* lastEditCommand = frame->editor()->lastEditCommand();
@@ -244,7 +259,7 @@ void TypingCommand::closeTyping(EditCommand* cmd)
 
 void TypingCommand::doApply()
 {
-    if (endingSelection().isNone())
+    if (!endingSelection().isNonOrphanedCaretOrRange())
         return;
         
     if (m_commandType == DeleteKey)
@@ -252,27 +267,27 @@ void TypingCommand::doApply()
             m_openedByBackwardDelete = true;
 
     switch (m_commandType) {
-        case DeleteSelection:
-            deleteSelection(m_smartDelete);
-            return;
-        case DeleteKey:
-            deleteKeyPressed(m_granularity, m_killRing);
-            return;
-        case ForwardDeleteKey:
-            forwardDeleteKeyPressed(m_granularity, m_killRing);
-            return;
-        case InsertLineBreak:
-            insertLineBreak();
-            return;
-        case InsertParagraphSeparator:
-            insertParagraphSeparator();
-            return;
-        case InsertParagraphSeparatorInQuotedContent:
-            insertParagraphSeparatorInQuotedContent();
-            return;
-        case InsertText:
-            insertText(m_textToInsert, m_selectInsertedText);
-            return;
+    case DeleteSelection:
+        deleteSelection(m_smartDelete);
+        return;
+    case DeleteKey:
+        deleteKeyPressed(m_granularity, m_killRing);
+        return;
+    case ForwardDeleteKey:
+        forwardDeleteKeyPressed(m_granularity, m_killRing);
+        return;
+    case InsertLineBreak:
+        insertLineBreak();
+        return;
+    case InsertParagraphSeparator:
+        insertParagraphSeparator();
+        return;
+    case InsertParagraphSeparatorInQuotedContent:
+        insertParagraphSeparatorInQuotedContent();
+        return;
+    case InsertText:
+        insertText(m_textToInsert, m_selectInsertedText);
+        return;
     }
 
     ASSERT_NOT_REACHED();
@@ -283,7 +298,7 @@ EditAction TypingCommand::editingAction() const
     return EditActionTyping;
 }
 
-void TypingCommand::markMisspellingsAfterTyping()
+void TypingCommand::markMisspellingsAfterTyping(ETypingCommand commandType)
 {
 #if PLATFORM(MAC) && !defined(BUILDING_ON_TIGER) && !defined(BUILDING_ON_LEOPARD)
     if (!document()->frame()->editor()->isContinuousSpellCheckingEnabled()
@@ -306,21 +321,27 @@ void TypingCommand::markMisspellingsAfterTyping()
         VisiblePosition p1 = startOfWord(previous, LeftWordIfOnBoundary);
         VisiblePosition p2 = startOfWord(start, LeftWordIfOnBoundary);
         if (p1 != p2)
-            document()->frame()->editor()->markMisspellingsAfterTypingToPosition(p1);
+            document()->frame()->editor()->markMisspellingsAfterTypingToWord(p1, endingSelection());
+#if SUPPORT_AUTOCORRECTION_PANEL
+        else if (commandType == TypingCommand::InsertText)
+            document()->frame()->editor()->startCorrectionPanelTimer(CorrectionPanelInfo::PanelTypeCorrection);
+#else
+    UNUSED_PARAM(commandType);
+#endif
     }
 }
 
 void TypingCommand::typingAddedToOpenCommand(ETypingCommand commandTypeForAddedTyping)
 {
     updatePreservesTypingStyle(commandTypeForAddedTyping);
-    
+
 #if PLATFORM(MAC) && !defined(BUILDING_ON_TIGER) && !defined(BUILDING_ON_LEOPARD)
     document()->frame()->editor()->appliedEditing(this);
     // Since the spellchecking code may also perform corrections and other replacements, it should happen after the typing changes.
-    markMisspellingsAfterTyping();
+    markMisspellingsAfterTyping(commandTypeForAddedTyping);
 #else
     // The old spellchecking code requires that checking be done first, to prevent issues like that in 6864072, where <doesn't> is marked as misspelled.
-    markMisspellingsAfterTyping();
+    markMisspellingsAfterTyping(commandTypeForAddedTyping);
     document()->frame()->editor()->appliedEditing(this);
 #endif
 }
@@ -332,28 +353,27 @@ void TypingCommand::insertText(const String &text, bool selectInsertedText)
     // an existing selection; at the moment they can either put the caret after what's inserted or
     // select what's inserted, but there's no way to "extend selection" to include both an old selection
     // that ends just before where we want to insert text and the newly inserted text.
-    int offset = 0;
-    int newline;
-    while ((newline = text.find('\n', offset)) != -1) {
+    unsigned offset = 0;
+    size_t newline;
+    while ((newline = text.find('\n', offset)) != notFound) {
         if (newline != offset)
             insertTextRunWithoutNewlines(text.substring(offset, newline - offset), false);
         insertParagraphSeparator();
         offset = newline + 1;
     }
-    if (offset == 0)
+    if (!offset)
         insertTextRunWithoutNewlines(text, selectInsertedText);
     else {
-        int length = text.length();
-        if (length != offset) {
+        unsigned length = text.length();
+        if (length != offset)
             insertTextRunWithoutNewlines(text.substring(offset, length - offset), selectInsertedText);
-        }
     }
 }
 
 void TypingCommand::insertTextRunWithoutNewlines(const String &text, bool selectInsertedText)
 {
     RefPtr<InsertTextCommand> command;
-    if (!document()->frame()->typingStyle() && !m_commands.isEmpty()) {
+    if (!document()->frame()->selection()->typingStyle() && !m_commands.isEmpty()) {
         EditCommand* lastCommand = m_commands.last().get();
         if (lastCommand->isInsertTextCommand())
             command = static_cast<InsertTextCommand*>(lastCommand);
@@ -362,6 +382,10 @@ void TypingCommand::insertTextRunWithoutNewlines(const String &text, bool select
         command = InsertTextCommand::create(document());
         applyCommandToComposite(command);
     }
+    if (endingSelection() != command->endingSelection()) {
+        command->setStartingSelection(endingSelection());
+        command->setEndingSelection(endingSelection());
+    }
     command->input(text, selectInsertedText);
     typingAddedToOpenCommand(InsertText);
 }
@@ -414,86 +438,89 @@ bool TypingCommand::makeEditableRootEmpty()
 
 void TypingCommand::deleteKeyPressed(TextGranularity granularity, bool killRing)
 {
+#if REMOVE_MARKERS_UPON_EDITING
+    document()->frame()->editor()->removeSpellAndCorrectionMarkersFromWordsToBeEdited(false);
+#endif
     VisibleSelection selectionToDelete;
     VisibleSelection selectionAfterUndo;
-    
+
     switch (endingSelection().selectionType()) {
-        case VisibleSelection::RangeSelection:
-            selectionToDelete = endingSelection();
-            selectionAfterUndo = selectionToDelete;
-            break;
-        case VisibleSelection::CaretSelection: {
-            // After breaking out of an empty mail blockquote, we still want continue with the deletion
-            // so actual content will get deleted, and not just the quote style.
-            if (breakOutOfEmptyMailBlockquotedParagraph())
-                typingAddedToOpenCommand(DeleteKey);
-        
-            m_smartDelete = false;
+    case VisibleSelection::RangeSelection:
+        selectionToDelete = endingSelection();
+        selectionAfterUndo = selectionToDelete;
+        break;
+    case VisibleSelection::CaretSelection: {
+        // After breaking out of an empty mail blockquote, we still want continue with the deletion
+        // so actual content will get deleted, and not just the quote style.
+        if (breakOutOfEmptyMailBlockquotedParagraph())
+            typingAddedToOpenCommand(DeleteKey);
 
-            SelectionController selection;
-            selection.setSelection(endingSelection());
-            selection.modify(SelectionController::EXTEND, SelectionController::BACKWARD, granularity);
-            if (killRing && selection.isCaret() && granularity != CharacterGranularity) 
-                selection.modify(SelectionController::EXTEND, SelectionController::BACKWARD, CharacterGranularity); 
-            
-            if (endingSelection().visibleStart().previous(true).isNull()) {
-                // When the caret is at the start of the editable area in an empty list item, break out of the list item.
-                if (breakOutOfEmptyListItem()) {
-                    typingAddedToOpenCommand(DeleteKey);
-                    return;
-                }
-                // When there are no visible positions in the editing root, delete its entire contents.
-                if (endingSelection().visibleStart().next(true).isNull() && makeEditableRootEmpty()) {
-                    typingAddedToOpenCommand(DeleteKey);
-                    return;
-                }
-            }
-            
-            VisiblePosition visibleStart(endingSelection().visibleStart());
-            // If we have a caret selection on an empty cell, we have nothing to do.
-            if (isEmptyTableCell(visibleStart.deepEquivalent().node()))
-                return;
+        m_smartDelete = false;
 
-            // If the caret is at the start of a paragraph after a table, move content into the last table cell.
-            if (isStartOfParagraph(visibleStart) && isFirstPositionAfterTable(visibleStart.previous(true))) {
-                // Unless the caret is just before a table.  We don't want to move a table into the last table cell.
-                if (isLastPositionBeforeTable(visibleStart))
-                    return;
-                // Extend the selection backward into the last cell, then deletion will handle the move.
-                selection.modify(SelectionController::EXTEND, SelectionController::BACKWARD, granularity);
-            // If the caret is just after a table, select the table and don't delete anything.
-            } else if (Node* table = isFirstPositionAfterTable(visibleStart)) {
-                setEndingSelection(VisibleSelection(Position(table, 0), endingSelection().start(), DOWNSTREAM));
+        SelectionController selection;
+        selection.setSelection(endingSelection());
+        selection.modify(SelectionController::AlterationExtend, SelectionController::DirectionBackward, granularity);
+        if (killRing && selection.isCaret() && granularity != CharacterGranularity)
+            selection.modify(SelectionController::AlterationExtend, SelectionController::DirectionBackward, CharacterGranularity);
+
+        if (endingSelection().visibleStart().previous(true).isNull()) {
+            // When the caret is at the start of the editable area in an empty list item, break out of the list item.
+            if (breakOutOfEmptyListItem()) {
                 typingAddedToOpenCommand(DeleteKey);
                 return;
             }
-
-            selectionToDelete = selection.selection();
-
-            if (granularity == CharacterGranularity && selectionToDelete.end().node() == selectionToDelete.start().node() && selectionToDelete.end().deprecatedEditingOffset() - selectionToDelete.start().deprecatedEditingOffset() > 1) {
-                // If there are multiple Unicode code points to be deleted, adjust the range to match platform conventions.
-                selectionToDelete.setWithoutValidation(selectionToDelete.end(), selectionToDelete.end().previous(BackwardDeletion));
+            // When there are no visible positions in the editing root, delete its entire contents.
+            if (endingSelection().visibleStart().next(true).isNull() && makeEditableRootEmpty()) {
+                typingAddedToOpenCommand(DeleteKey);
+                return;
             }
-
-            if (!startingSelection().isRange() || selectionToDelete.base() != startingSelection().start())
-                selectionAfterUndo = selectionToDelete;
-            else
-                // It's a little tricky to compute what the starting selection would have been in the original document.
-                // We can't let the VisibleSelection class's validation kick in or it'll adjust for us based on
-                // the current state of the document and we'll get the wrong result.
-                selectionAfterUndo.setWithoutValidation(startingSelection().end(), selectionToDelete.extent());
-            break;
         }
-        case VisibleSelection::NoSelection:
-            ASSERT_NOT_REACHED();
-            break;
+
+        VisiblePosition visibleStart(endingSelection().visibleStart());
+        // If we have a caret selection on an empty cell, we have nothing to do.
+        if (isEmptyTableCell(visibleStart.deepEquivalent().node()))
+            return;
+
+        // If the caret is at the start of a paragraph after a table, move content into the last table cell.
+        if (isStartOfParagraph(visibleStart) && isFirstPositionAfterTable(visibleStart.previous(true))) {
+            // Unless the caret is just before a table.  We don't want to move a table into the last table cell.
+            if (isLastPositionBeforeTable(visibleStart))
+                return;
+            // Extend the selection backward into the last cell, then deletion will handle the move.
+            selection.modify(SelectionController::AlterationExtend, SelectionController::DirectionBackward, granularity);
+        // If the caret is just after a table, select the table and don't delete anything.
+        } else if (Node* table = isFirstPositionAfterTable(visibleStart)) {
+            setEndingSelection(VisibleSelection(Position(table, 0), endingSelection().start(), DOWNSTREAM));
+            typingAddedToOpenCommand(DeleteKey);
+            return;
+        }
+
+        selectionToDelete = selection.selection();
+
+        if (granularity == CharacterGranularity && selectionToDelete.end().node() == selectionToDelete.start().node() && selectionToDelete.end().deprecatedEditingOffset() - selectionToDelete.start().deprecatedEditingOffset() > 1) {
+            // If there are multiple Unicode code points to be deleted, adjust the range to match platform conventions.
+            selectionToDelete.setWithoutValidation(selectionToDelete.end(), selectionToDelete.end().previous(BackwardDeletion));
+        }
+
+        if (!startingSelection().isRange() || selectionToDelete.base() != startingSelection().start())
+            selectionAfterUndo = selectionToDelete;
+        else
+            // It's a little tricky to compute what the starting selection would have been in the original document.
+            // We can't let the VisibleSelection class's validation kick in or it'll adjust for us based on
+            // the current state of the document and we'll get the wrong result.
+            selectionAfterUndo.setWithoutValidation(startingSelection().end(), selectionToDelete.extent());
+        break;
+    }
+    case VisibleSelection::NoSelection:
+        ASSERT_NOT_REACHED();
+        break;
     }
     
     ASSERT(!selectionToDelete.isNone());
     if (selectionToDelete.isNone())
         return;
     
-    if (selectionToDelete.isCaret() || !document()->frame()->shouldDeleteSelection(selectionToDelete))
+    if (selectionToDelete.isCaret() || !document()->frame()->selection()->shouldDeleteSelection(selectionToDelete))
         return;
     
     if (killRing)
@@ -510,73 +537,76 @@ void TypingCommand::deleteKeyPressed(TextGranularity granularity, bool killRing)
 
 void TypingCommand::forwardDeleteKeyPressed(TextGranularity granularity, bool killRing)
 {
+#if REMOVE_MARKERS_UPON_EDITING
+    document()->frame()->editor()->removeSpellAndCorrectionMarkersFromWordsToBeEdited(false);
+#endif
     VisibleSelection selectionToDelete;
     VisibleSelection selectionAfterUndo;
 
     switch (endingSelection().selectionType()) {
-        case VisibleSelection::RangeSelection:
-            selectionToDelete = endingSelection();
-            selectionAfterUndo = selectionToDelete;
-            break;
-        case VisibleSelection::CaretSelection: {
-            m_smartDelete = false;
+    case VisibleSelection::RangeSelection:
+        selectionToDelete = endingSelection();
+        selectionAfterUndo = selectionToDelete;
+        break;
+    case VisibleSelection::CaretSelection: {
+        m_smartDelete = false;
 
-            // Handle delete at beginning-of-block case.
-            // Do nothing in the case that the caret is at the start of a
-            // root editable element or at the start of a document.
-            SelectionController selection;
-            selection.setSelection(endingSelection());
-            selection.modify(SelectionController::EXTEND, SelectionController::FORWARD, granularity);
-            if (killRing && selection.isCaret() && granularity != CharacterGranularity) 
-                selection.modify(SelectionController::EXTEND, SelectionController::FORWARD, CharacterGranularity); 
-            
-            Position downstreamEnd = endingSelection().end().downstream();
-            VisiblePosition visibleEnd = endingSelection().visibleEnd();
-            if (visibleEnd == endOfParagraph(visibleEnd))
-                downstreamEnd = visibleEnd.next(true).deepEquivalent().downstream();
-            // When deleting tables: Select the table first, then perform the deletion
-            if (downstreamEnd.node() && downstreamEnd.node()->renderer() && downstreamEnd.node()->renderer()->isTable() && downstreamEnd.deprecatedEditingOffset() == 0) {
-                setEndingSelection(VisibleSelection(endingSelection().end(), lastDeepEditingPositionForNode(downstreamEnd.node()), DOWNSTREAM));
-                typingAddedToOpenCommand(ForwardDeleteKey);
-                return;
-            }
+        // Handle delete at beginning-of-block case.
+        // Do nothing in the case that the caret is at the start of a
+        // root editable element or at the start of a document.
+        SelectionController selection;
+        selection.setSelection(endingSelection());
+        selection.modify(SelectionController::AlterationExtend, SelectionController::DirectionForward, granularity);
+        if (killRing && selection.isCaret() && granularity != CharacterGranularity)
+            selection.modify(SelectionController::AlterationExtend, SelectionController::DirectionForward, CharacterGranularity);
 
-            // deleting to end of paragraph when at end of paragraph needs to merge the next paragraph (if any)
-            if (granularity == ParagraphBoundary && selection.selection().isCaret() && isEndOfParagraph(selection.selection().visibleEnd()))
-                selection.modify(SelectionController::EXTEND, SelectionController::FORWARD, CharacterGranularity);
-
-            selectionToDelete = selection.selection();
-            if (!startingSelection().isRange() || selectionToDelete.base() != startingSelection().start())
-                selectionAfterUndo = selectionToDelete;
-            else {
-                // It's a little tricky to compute what the starting selection would have been in the original document.
-                // We can't let the VisibleSelection class's validation kick in or it'll adjust for us based on
-                // the current state of the document and we'll get the wrong result.
-                Position extent = startingSelection().end();
-                if (extent.node() != selectionToDelete.end().node())
-                    extent = selectionToDelete.extent();
-                else {
-                    int extraCharacters;
-                    if (selectionToDelete.start().node() == selectionToDelete.end().node())
-                        extraCharacters = selectionToDelete.end().deprecatedEditingOffset() - selectionToDelete.start().deprecatedEditingOffset();
-                    else
-                        extraCharacters = selectionToDelete.end().deprecatedEditingOffset();
-                    extent = Position(extent.node(), extent.deprecatedEditingOffset() + extraCharacters);
-                }
-                selectionAfterUndo.setWithoutValidation(startingSelection().start(), extent);
-            }
-            break;
+        Position downstreamEnd = endingSelection().end().downstream();
+        VisiblePosition visibleEnd = endingSelection().visibleEnd();
+        if (visibleEnd == endOfParagraph(visibleEnd))
+            downstreamEnd = visibleEnd.next(true).deepEquivalent().downstream();
+        // When deleting tables: Select the table first, then perform the deletion
+        if (downstreamEnd.node() && downstreamEnd.node()->renderer() && downstreamEnd.node()->renderer()->isTable() && !downstreamEnd.deprecatedEditingOffset()) {
+            setEndingSelection(VisibleSelection(endingSelection().end(), lastDeepEditingPositionForNode(downstreamEnd.node()), DOWNSTREAM));
+            typingAddedToOpenCommand(ForwardDeleteKey);
+            return;
         }
-        case VisibleSelection::NoSelection:
-            ASSERT_NOT_REACHED();
-            break;
+
+        // deleting to end of paragraph when at end of paragraph needs to merge the next paragraph (if any)
+        if (granularity == ParagraphBoundary && selection.selection().isCaret() && isEndOfParagraph(selection.selection().visibleEnd()))
+            selection.modify(SelectionController::AlterationExtend, SelectionController::DirectionForward, CharacterGranularity);
+
+        selectionToDelete = selection.selection();
+        if (!startingSelection().isRange() || selectionToDelete.base() != startingSelection().start())
+            selectionAfterUndo = selectionToDelete;
+        else {
+            // It's a little tricky to compute what the starting selection would have been in the original document.
+            // We can't let the VisibleSelection class's validation kick in or it'll adjust for us based on
+            // the current state of the document and we'll get the wrong result.
+            Position extent = startingSelection().end();
+            if (extent.node() != selectionToDelete.end().node())
+                extent = selectionToDelete.extent();
+            else {
+                int extraCharacters;
+                if (selectionToDelete.start().node() == selectionToDelete.end().node())
+                    extraCharacters = selectionToDelete.end().deprecatedEditingOffset() - selectionToDelete.start().deprecatedEditingOffset();
+                else
+                    extraCharacters = selectionToDelete.end().deprecatedEditingOffset();
+                extent = Position(extent.node(), extent.deprecatedEditingOffset() + extraCharacters);
+            }
+            selectionAfterUndo.setWithoutValidation(startingSelection().start(), extent);
+        }
+        break;
+    }
+    case VisibleSelection::NoSelection:
+        ASSERT_NOT_REACHED();
+        break;
     }
     
     ASSERT(!selectionToDelete.isNone());
     if (selectionToDelete.isNone())
         return;
     
-    if (selectionToDelete.isCaret() || !document()->frame()->shouldDeleteSelection(selectionToDelete))
+    if (selectionToDelete.isCaret() || !document()->frame()->selection()->shouldDeleteSelection(selectionToDelete))
         return;
         
     if (killRing)
@@ -597,17 +627,17 @@ void TypingCommand::deleteSelection(bool smartDelete)
 void TypingCommand::updatePreservesTypingStyle(ETypingCommand commandType)
 {
     switch (commandType) {
-        case DeleteSelection:
-        case DeleteKey:
-        case ForwardDeleteKey:
-        case InsertParagraphSeparator:
-        case InsertLineBreak:
-            m_preservesTypingStyle = true;
-            return;
-        case InsertParagraphSeparatorInQuotedContent:
-        case InsertText:
-            m_preservesTypingStyle = false;
-            return;
+    case DeleteSelection:
+    case DeleteKey:
+    case ForwardDeleteKey:
+    case InsertParagraphSeparator:
+    case InsertLineBreak:
+        m_preservesTypingStyle = true;
+        return;
+    case InsertParagraphSeparatorInQuotedContent:
+    case InsertText:
+        m_preservesTypingStyle = false;
+        return;
     }
     ASSERT_NOT_REACHED();
     m_preservesTypingStyle = false;
