@@ -2,7 +2,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2007, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,14 +30,20 @@ namespace WebCore {
 
 class HTMLButtonElement : public HTMLFormControlElement {
 public:
-    HTMLButtonElement(const QualifiedName&, Document*, HTMLFormElement* = 0);
-    virtual ~HTMLButtonElement();
+    static PassRefPtr<HTMLButtonElement> create(const QualifiedName&, Document*, HTMLFormElement*);
+
+    String value() const;
+
+private:
+    HTMLButtonElement(const QualifiedName& tagName, Document*, HTMLFormElement*);
+
+    enum Type { SUBMIT, RESET, BUTTON };
 
     virtual const AtomicString& formControlType() const;
         
     virtual RenderObject* createRenderer(RenderArena*, RenderStyle*);
 
-    virtual void parseMappedAttribute(MappedAttribute*);
+    virtual void parseMappedAttribute(Attribute*);
     virtual void defaultEventHandler(Event*);
     virtual bool appendFormData(FormDataList&, bool);
 
@@ -48,20 +54,12 @@ public:
     virtual void setActivatedSubmit(bool flag);
 
     virtual void accessKeyAction(bool sendToAnyElement);
+    virtual bool isURLAttribute(Attribute*) const;
 
     virtual bool canStartSelection() const { return false; }
 
-    String accessKey() const;
-    void setAccessKey(const String&);
-
-    String value() const;
-    void setValue(const String&);
-
-    virtual bool willValidate() const { return false; }
-    
-private:
-    enum Type { SUBMIT, RESET, BUTTON };
     virtual bool isOptionalFormControl() const { return true; }
+    virtual bool recalcWillValidate() const;
 
     Type m_type;
     bool m_activeSubmit;
