@@ -25,9 +25,13 @@
 #include "config.h"
 #include "HTMLFormControlElement.h"
 
+#include "Attribute.h"
+#include "CharacterNames.h"
 #include "Chrome.h"
 #include "ChromeClient.h"
 #include "Document.h"
+#include "DocumentParser.h"
+#include "ElementRareData.h"
 #include "Event.h"
 #include "EventHandler.h"
 #include "EventNames.h"
@@ -35,65 +39,59 @@
 #include "HTMLFormElement.h"
 #include "HTMLInputElement.h"
 #include "HTMLNames.h"
-#include "HTMLParser.h"
-#include "HTMLTokenizer.h"
-#include "MappedAttribute.h"
+#include "LabelsNodeList.h"
 #include "Page.h"
 #include "RenderBox.h"
 #include "RenderTextControl.h"
 #include "RenderTheme.h"
 #include "ScriptEventListener.h"
+#include "ValidationMessage.h"
 #include "ValidityState.h"
+#include <limits>
+#include <wtf/Vector.h>
 
 namespace WebCore {
 
 using namespace HTMLNames;
+using namespace std;
 
-HTMLFormControlElement::HTMLFormControlElement(const QualifiedName& tagName, Document* doc, HTMLFormElement* f, ConstructionType constructionType)
-    : HTMLElement(tagName, doc, constructionType)
-    , m_form(f)
-    , m_hasName(false)
+HTMLFormControlElement::HTMLFormControlElement(const QualifiedName& tagName, Document* document, HTMLFormElement* form)
+    : HTMLElement(tagName, document)
+    , FormAssociatedElement(form)
     , m_disabled(false)
     , m_readOnly(false)
     , m_required(false)
     , m_valueMatchesRenderer(false)
+    , m_willValidateInitialized(false)
+    , m_willValidate(true)
+    , m_isValid(true)
 {
-    if (!m_form)
-        m_form = findFormAncestor();
-    if (m_form)
-        m_form->registerFormElement(this);
+    if (!this->form())
+        setForm(findFormAncestor());
+    if (this->form())
+        this->form()->registerFormElement(this);
 }
 
 HTMLFormControlElement::~HTMLFormControlElement()
 {
-    if (m_form)
-        m_form->removeFormElement(this);
+    if (form())
+        form()->removeFormElement(this);
+}
+
+void HTMLFormControlElement::detach()
+{
+    hideVisibleValidationMessage();
+    HTMLElement::detach();
 }
 
 bool HTMLFormControlElement::formNoValidate() const
 {
-    return !getAttribute(formnovalidateAttr).isNull();
+    return fastHasAttribute(formnovalidateAttr);
 }
 
-void HTMLFormControlElement::setFormNoValidate(bool formnovalidate)
+void HTMLFormControlElement::parseMappedAttribute(Attribute* attr)
 {
-    setAttribute(formnovalidateAttr, formnovalidate ? "" : 0);
-}
-
-ValidityState* HTMLFormControlElement::validity()
-{
-    if (!m_validityState)
-        m_validityState = ValidityState::create(this);
-
-    return m_validityState.get();
-}
-
-void HTMLFormControlElement::parseMappedAttribute(MappedAttribute *attr)
-{
-    bool oldWillValidate = willValidate();
-    if (attr->name() == nameAttr)
-        m_hasName = !attr->isEmpty();
-    else if (attr->name() == disabledAttr) {
+    if (attr->name() == disabledAttr) {
         bool oldDisabled = m_disabled;
         m_disabled = !attr->isNull();
         if (oldDisabled != m_disabled) {
@@ -112,12 +110,13 @@ void HTMLFormControlElement::parseMappedAttribute(MappedAttribute *attr)
     } else if (attr->name() == requiredAttr) {
         bool oldRequired = m_required;
         m_required = !attr->isNull();
-        if (oldRequired != m_required)
-            setNeedsStyleRecalc();
+        if (oldRequired != m_required) {
+            setNeedsValidityCheck();
+            setNeedsStyleRecalc(); // Updates for :required :optional classes.
+        }
     } else
         HTMLElement::parseMappedAttribute(attr);
-    if (oldWillValidate != willValidate())
-        setNeedsWillValidateCheck();
+    setNeedsWillValidateCheck();
 }
 
 void HTMLFormControlElement::attach()
@@ -147,62 +146,26 @@ void HTMLFormControlElement::attach()
 
 void HTMLFormControlElement::insertedIntoTree(bool deep)
 {
-    if (!m_form) {
-        // This handles the case of a new form element being created by
-        // JavaScript and inserted inside a form.  In the case of the parser
-        // setting a form, we will already have a non-null value for m_form, 
-        // and so we don't need to do anything.
-        m_form = findFormAncestor();
-        if (m_form) {
-            m_form->registerFormElement(this);
-            setNeedsWillValidateCheck();
-        } else
-            document()->checkedRadioButtons().addButton(this);
-    }
+    FormAssociatedElement::insertedIntoTree();
+    if (!form())
+        document()->checkedRadioButtons().addButton(this);
 
     HTMLElement::insertedIntoTree(deep);
 }
 
-static inline Node* findRoot(Node* n)
-{
-    Node* root = n;
-    for (; n; n = n->parentNode())
-        root = n;
-    return root;
-}
-
 void HTMLFormControlElement::removedFromTree(bool deep)
 {
-    // If the form and element are both in the same tree, preserve the connection to the form.
-    // Otherwise, null out our form and remove ourselves from the form's list of elements.
-    HTMLParser* parser = 0;
-    if (Tokenizer* tokenizer = document()->tokenizer())
-        if (tokenizer->isHTMLTokenizer())
-            parser = static_cast<HTMLTokenizer*>(tokenizer)->htmlParser();
-    
-    if (m_form && !(parser && parser->isHandlingResidualStyleAcrossBlocks()) && findRoot(this) != findRoot(m_form)) {
-        m_form->removeFormElement(this);
-        m_form = 0;
-        setNeedsWillValidateCheck();
-    }
-
+    FormAssociatedElement::removedFromTree();
     HTMLElement::removedFromTree(deep);
 }
 
-void HTMLFormControlElement::formDestroyed()
-{
-    if (m_form)
-        setNeedsWillValidateCheck();
-    m_form = 0;
-}
-
 const AtomicString& HTMLFormControlElement::formControlName() const
 {
-    const AtomicString& n = getAttribute(nameAttr);
-    return n.isNull() ? emptyAtom : n;
+    const AtomicString& name = fastGetAttribute(nameAttr);
+    return name.isNull() ? emptyAtom : name;
 }
 
-void HTMLFormControlElement::setName(const AtomicString &value)
+void HTMLFormControlElement::setName(const AtomicString& value)
 {
     setAttribute(nameAttr, value);
 }
@@ -217,31 +180,16 @@ void HTMLFormControlElement::setDisabled(bool b)
     setAttribute(disabledAttr, b ? "" : 0);
 }
 
-void HTMLFormControlElement::setReadOnly(bool b)
-{
-    setAttribute(readonlyAttr, b ? "" : 0);
-}
-
 bool HTMLFormControlElement::autofocus() const
 {
     return hasAttribute(autofocusAttr);
 }
 
-void HTMLFormControlElement::setAutofocus(bool b)
-{
-    setAttribute(autofocusAttr, b ? "autofocus" : 0);
-}
-
 bool HTMLFormControlElement::required() const
 {
     return m_required;
 }
 
-void HTMLFormControlElement::setRequired(bool b)
-{
-    setAttribute(requiredAttr, b ? "required" : 0);
-}
-
 static void updateFromElementCallback(Node* node)
 {
     ASSERT_ARG(node, node->isElementNode());
@@ -263,7 +211,7 @@ void HTMLFormControlElement::recalcStyle(StyleChange change)
 
 bool HTMLFormControlElement::supportsFocus() const
 {
-    return !disabled();
+    return !m_disabled;
 }
 
 bool HTMLFormControlElement::isFocusable() const
@@ -286,7 +234,7 @@ bool HTMLFormControlElement::isKeyboardFocusable(KeyboardEvent* event) const
 
 bool HTMLFormControlElement::isMouseFocusable() const
 {
-#if PLATFORM(GTK)
+#if PLATFORM(GTK) || PLATFORM(QT)
     return HTMLElement::isMouseFocusable();
 #else
     return false;
@@ -299,13 +247,39 @@ short HTMLFormControlElement::tabIndex() const
     return Element::tabIndex();
 }
 
+bool HTMLFormControlElement::recalcWillValidate() const
+{
+    // FIXME: Should return false if this element has a datalist element as an
+    // ancestor. See HTML5 4.10.10 'The datalist element.'
+    return !m_disabled && !m_readOnly;
+}
+
 bool HTMLFormControlElement::willValidate() const
 {
-    // FIXME: Implementation shall be completed with these checks:
-    //      The control does not have a repetition template as an ancestor.
-    //      The control does not have a datalist element as an ancestor.
-    //      The control is not an output element.
-    return m_form && m_hasName && !m_disabled && !m_readOnly;
+    if (!m_willValidateInitialized) {
+        m_willValidateInitialized = true;
+        m_willValidate = recalcWillValidate();
+    } else {
+        // If the following assertion fails, setNeedsWillValidateCheck() is not
+        // called correctly when something which changes recalcWillValidate() result
+        // is updated.
+        ASSERT(m_willValidate == recalcWillValidate());
+    }
+    return m_willValidate;
+}
+
+void HTMLFormControlElement::setNeedsWillValidateCheck()
+{
+    // We need to recalculate willValidte immediately because willValidate
+    // change can causes style change.
+    bool newWillValidate = recalcWillValidate();
+    if (m_willValidateInitialized && m_willValidate == newWillValidate)
+        return;
+    m_willValidateInitialized = true;
+    m_willValidate = newWillValidate;
+    setNeedsStyleRecalc();
+    if (!m_willValidate)
+        hideVisibleValidationMessage();
 }
 
 String HTMLFormControlElement::validationMessage()
@@ -313,73 +287,149 @@ String HTMLFormControlElement::validationMessage()
     return validity()->validationMessage();
 }
 
-void HTMLFormControlElement::setNeedsWillValidateCheck()
+void HTMLFormControlElement::updateVisibleValidationMessage()
 {
-    setNeedsStyleRecalc();
-    // FIXME: Show/hide a validation message.
+    Page* page = document()->page();
+    if (!page)
+        return;
+    String message;
+    if (renderer() && willValidate()) {
+        message = validationMessage().stripWhiteSpace();
+        // HTML5 specification doesn't ask UA to show the title attribute value
+        // with the validationMessage.  However, this behavior is same as Opera
+        // and the specification describes such behavior as an example.
+        const AtomicString& title = getAttribute(titleAttr);
+        if (!message.isEmpty() && !title.isEmpty()) {
+            message.append('\n');
+            message.append(title);
+        }
+    }
+    if (!m_validationMessage) {
+        m_validationMessage = ValidationMessage::create(this);
+        m_validationMessage->setMessage(message);
+    } else if (message.isEmpty())
+        hideVisibleValidationMessage();
+    else if (m_validationMessage->message() != message)
+        m_validationMessage->setMessage(message);
 }
 
-bool HTMLFormControlElement::checkValidity()
+void HTMLFormControlElement::hideVisibleValidationMessage()
 {
-    if (willValidate() && !isValidFormControlElement()) {
-        dispatchEvent(Event::create(eventNames().invalidEvent, false, true));
-        return false;
-    }
+    m_validationMessage = 0;
+}
 
-    return true;
+String HTMLFormControlElement::visibleValidationMessage() const
+{
+    return m_validationMessage ? m_validationMessage->message() : String();
+}
+
+bool HTMLFormControlElement::checkValidity(Vector<RefPtr<FormAssociatedElement> >* unhandledInvalidControls)
+{
+    if (!willValidate() || isValidFormControlElement())
+        return true;
+    // An event handler can deref this object.
+    RefPtr<HTMLFormControlElement> protector(this);
+    RefPtr<Document> originalDocument(document());
+    bool needsDefaultAction = dispatchEvent(Event::create(eventNames().invalidEvent, false, true));
+    if (needsDefaultAction && unhandledInvalidControls && inDocument() && originalDocument == document())
+        unhandledInvalidControls->append(this);
+    return false;
+}
+
+bool HTMLFormControlElement::isValidFormControlElement()
+{
+    // If the following assertion fails, setNeedsValidityCheck() is not called
+    // correctly when something which changes validity is updated.
+    ASSERT(m_isValid == validity()->valid());
+    return m_isValid;
 }
 
 void HTMLFormControlElement::setNeedsValidityCheck()
 {
-    if (willValidate()) {
+    bool newIsValid = validity()->valid();
+    if (willValidate() && newIsValid != m_isValid) {
         // Update style for pseudo classes such as :valid :invalid.
         setNeedsStyleRecalc();
     }
-    // FIXME: show/hide a validation message.
+    m_isValid = newIsValid;
+
+    // Updates only if this control already has a validtion message.
+    if (!visibleValidationMessage().isEmpty()) {
+        // Calls updateVisibleValidationMessage() even if m_isValid is not
+        // changed because a validation message can be chagned.
+        updateVisibleValidationMessage();
+    }
 }
 
 void HTMLFormControlElement::setCustomValidity(const String& error)
 {
     validity()->setCustomErrorMessage(error);
 }
-    
+
 void HTMLFormControlElement::dispatchFocusEvent()
 {
-    if (document()->frame() && document()->frame()->page())
-        document()->frame()->page()->chrome()->client()->formDidFocus(this);
+    if (document()->page())
+        document()->page()->chrome()->client()->formDidFocus(this);
 
     HTMLElement::dispatchFocusEvent();
 }
 
 void HTMLFormControlElement::dispatchBlurEvent()
 {
-    if (document()->frame() && document()->frame()->page())
-        document()->frame()->page()->chrome()->client()->formDidBlur(this);
+    if (document()->page())
+        document()->page()->chrome()->client()->formDidBlur(this);
 
     HTMLElement::dispatchBlurEvent();
+    hideVisibleValidationMessage();
 }
 
 HTMLFormElement* HTMLFormControlElement::virtualForm() const
 {
-    return m_form;
+    return FormAssociatedElement::form();
 }
 
 bool HTMLFormControlElement::isDefaultButtonForForm() const
 {
-    return isSuccessfulSubmitButton() && m_form && m_form->defaultButton() == this;
+    return isSuccessfulSubmitButton() && form() && form()->defaultButton() == this;
 }
 
-bool HTMLFormControlElement::isValidFormControlElement()
+void HTMLFormControlElement::attributeChanged(Attribute* attr, bool preserveDecls)
 {
-    return validity()->valid();
+    if (attr->name() == formAttr) {
+        formAttributeChanged();
+        if (!form())
+            document()->checkedRadioButtons().addButton(this);
+    } else
+        HTMLElement::attributeChanged(attr, preserveDecls);
 }
 
-void HTMLFormControlElement::removeFromForm()
+bool HTMLFormControlElement::isLabelable() const
 {
-    if (!m_form)
-        return;
-    m_form->removeFormElement(this);
-    m_form = 0;
+    // FIXME: Add meterTag and outputTag to the list once we support them.
+    return hasTagName(buttonTag) || hasTagName(inputTag) || hasTagName(keygenTag)
+#if ENABLE(METER_TAG)
+        || hasTagName(meterTag)
+#endif
+#if ENABLE(PROGRESS_TAG)
+        || hasTagName(progressTag)
+#endif
+        || hasTagName(selectTag) || hasTagName(textareaTag);
+}
+
+PassRefPtr<NodeList> HTMLFormControlElement::labels()
+{
+    if (!isLabelable())
+        return 0;
+    if (!document())
+        return 0;
+    
+    NodeRareData* data = Node::ensureRareData();
+    if (!data->nodeLists()) {
+        data->setNodeLists(NodeListsNodeData::create());
+        document()->addNodeListCache();
+    }
+    
+    return LabelsNodeList::create(this);
 }
 
 HTMLFormControlElementWithState::HTMLFormControlElementWithState(const QualifiedName& tagName, Document* doc, HTMLFormElement* f)
@@ -405,9 +455,29 @@ void HTMLFormControlElementWithState::didMoveToNewOwnerDocument()
     HTMLFormControlElement::didMoveToNewOwnerDocument();
 }
 
+bool HTMLFormControlElementWithState::autoComplete() const
+{
+    if (!form())
+        return true;
+    return form()->autoComplete();
+}
+
+bool HTMLFormControlElementWithState::shouldSaveAndRestoreFormControlState() const
+{
+    // We don't save/restore control state in a form with autocomplete=off.
+    return attached() && autoComplete();
+}
+
 void HTMLFormControlElementWithState::finishParsingChildren()
 {
     HTMLFormControlElement::finishParsingChildren();
+
+    // We don't save state of a control with shouldSaveAndRestoreFormControlState()=false.
+    // But we need to skip restoring process too because a control in another
+    // form might have the same pair of name and type and saved its state.
+    if (!shouldSaveAndRestoreFormControlState())
+        return;
+
     Document* doc = document();
     if (doc->hasStateForNewFormElements()) {
         String state;
@@ -416,6 +486,16 @@ void HTMLFormControlElementWithState::finishParsingChildren()
     }
 }
 
+void HTMLFormControlElementWithState::defaultEventHandler(Event* event)
+{
+    if (event->type() == eventNames().webkitEditableContentChangedEvent && renderer() && renderer()->isTextControl()) {
+        toRenderTextControl(renderer())->subtreeHasChanged();
+        return;
+    }
+
+    HTMLFormControlElement::defaultEventHandler(event);
+}
+
 HTMLTextFormControlElement::HTMLTextFormControlElement(const QualifiedName& tagName, Document* doc, HTMLFormElement* form)
     : HTMLFormControlElementWithState(tagName, doc, form)
 {
@@ -441,12 +521,40 @@ void HTMLTextFormControlElement::dispatchBlurEvent()
     HTMLFormControlElementWithState::dispatchBlurEvent();
 }
 
+String HTMLTextFormControlElement::strippedPlaceholder() const
+{
+    // According to the HTML5 specification, we need to remove CR and LF from
+    // the attribute value.
+    const AtomicString& attributeValue = getAttribute(placeholderAttr);
+    if (!attributeValue.contains(newlineCharacter) && !attributeValue.contains(carriageReturn))
+        return attributeValue;
+
+    Vector<UChar> stripped;
+    unsigned length = attributeValue.length();
+    stripped.reserveCapacity(length);
+    for (unsigned i = 0; i < length; ++i) {
+        UChar character = attributeValue[i];
+        if (character == newlineCharacter || character == carriageReturn)
+            continue;
+        stripped.append(character);
+    }
+    return String::adopt(stripped);
+}
+
+static bool isNotLineBreak(UChar ch) { return ch != newlineCharacter && ch != carriageReturn; }
+
+bool HTMLTextFormControlElement::isPlaceholderEmpty() const
+{
+    const AtomicString& attributeValue = getAttribute(placeholderAttr);
+    return attributeValue.string().find(isNotLineBreak) == notFound;
+}
+
 bool HTMLTextFormControlElement::placeholderShouldBeVisible() const
 {
     return supportsPlaceholder()
         && isEmptyValue()
         && document()->focusedNode() != this
-        && !getAttribute(placeholderAttr).isEmpty();
+        && !isPlaceholderEmpty();
 }
 
 void HTMLTextFormControlElement::updatePlaceholderVisibility(bool placeholderValueChanged)
@@ -465,29 +573,25 @@ RenderTextControl* HTMLTextFormControlElement::textRendererAfterUpdateLayout()
 
 void HTMLTextFormControlElement::setSelectionStart(int start)
 {
-    if (RenderTextControl* renderer = textRendererAfterUpdateLayout())
-        renderer->setSelectionStart(start);
+    setSelectionRange(start, max(start, selectionEnd()));
 }
 
 void HTMLTextFormControlElement::setSelectionEnd(int end)
 {
-    if (RenderTextControl* renderer = textRendererAfterUpdateLayout())
-        renderer->setSelectionEnd(end);
+    setSelectionRange(min(end, selectionStart()), end);
 }
 
 void HTMLTextFormControlElement::select()
 {
-    if (RenderTextControl* renderer = textRendererAfterUpdateLayout())
-        renderer->select();
+    setSelectionRange(0, numeric_limits<int>::max());
 }
 
 void HTMLTextFormControlElement::setSelectionRange(int start, int end)
 {
-    if (RenderTextControl* renderer = textRendererAfterUpdateLayout())
-        renderer->setSelectionRange(start, end);
+    WebCore::setSelectionRange(this, start, end);
 }
 
-int HTMLTextFormControlElement::selectionStart()
+int HTMLTextFormControlElement::selectionStart() const
 {
     if (!isTextFormControl())
         return 0;
@@ -498,7 +602,7 @@ int HTMLTextFormControlElement::selectionStart()
     return toRenderTextControl(renderer())->selectionStart();
 }
 
-int HTMLTextFormControlElement::selectionEnd()
+int HTMLTextFormControlElement::selectionEnd() const
 {
     if (!isTextFormControl())
         return 0;
@@ -509,21 +613,17 @@ int HTMLTextFormControlElement::selectionEnd()
     return toRenderTextControl(renderer())->selectionEnd();
 }
 
-VisibleSelection HTMLTextFormControlElement::selection() const
+PassRefPtr<Range> HTMLTextFormControlElement::selection() const
 {
     if (!renderer() || !isTextFormControl() || cachedSelectionStart() < 0 || cachedSelectionEnd() < 0)
-        return VisibleSelection();
+        return 0;
     return toRenderTextControl(renderer())->selection(cachedSelectionStart(), cachedSelectionEnd());
 }
 
-void HTMLTextFormControlElement::parseMappedAttribute(MappedAttribute* attr)
+void HTMLTextFormControlElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == placeholderAttr)
         updatePlaceholderVisibility(true);
-    else if (attr->name() == onfocusAttr)
-        setAttributeEventListener(eventNames().focusEvent, createAttributeEventListener(this, attr));
-    else if (attr->name() == onblurAttr)
-        setAttributeEventListener(eventNames().blurEvent, createAttributeEventListener(this, attr));
     else if (attr->name() == onselectAttr)
         setAttributeEventListener(eventNames().selectEvent, createAttributeEventListener(this, attr));
     else if (attr->name() == onchangeAttr)
