@@ -2,7 +2,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,13 +25,15 @@
 #define HTMLFormElement_h
 
 #include "CheckedRadioButtons.h"
-#include "FormDataBuilder.h"
+#include "FormState.h"
+#include "FormSubmission.h"
 #include "HTMLElement.h"
 #include <wtf/OwnPtr.h>
 
 namespace WebCore {
 
 class Event;
+class FormAssociatedElement;
 class FormData;
 class HTMLFormControlElement;
 class HTMLImageElement;
@@ -41,44 +43,35 @@ class TextEncoding;
 
 struct CollectionCache;
 
-class HTMLFormElement : public HTMLElement { 
+class HTMLFormElement : public HTMLElement {
 public:
-    HTMLFormElement(const QualifiedName&, Document*);
+    static PassRefPtr<HTMLFormElement> create(Document*);
+    static PassRefPtr<HTMLFormElement> create(const QualifiedName&, Document*);
     virtual ~HTMLFormElement();
 
-    virtual HTMLTagStatus endTagRequirement() const { return TagStatusRequired; }
-    virtual int tagPriority() const { return 3; }
-
-    virtual void attach();
-    virtual bool rendererIsNeeded(RenderStyle*);
-    virtual void insertedIntoDocument();
-    virtual void removedFromDocument();
- 
-    virtual void handleLocalEvents(Event*, bool useCapture);
-     
     PassRefPtr<HTMLCollection> elements();
     void getNamedElements(const AtomicString&, Vector<RefPtr<Node> >&);
-    
+
     unsigned length() const;
     Node* item(unsigned index);
 
-    String enctype() const { return m_formDataBuilder.encodingType(); }
+    String enctype() const { return m_attributes.encodingType(); }
     void setEnctype(const String&);
 
-    String encoding() const { return m_formDataBuilder.encodingType(); }
+    String encoding() const { return m_attributes.encodingType(); }
     void setEncoding(const String& value) { setEnctype(value); }
 
     bool autoComplete() const { return m_autocomplete; }
 
-    virtual void parseMappedAttribute(MappedAttribute*);
-
-    void registerFormElement(HTMLFormControlElement*);
-    void removeFormElement(HTMLFormControlElement*);
+    // FIXME: Should rename these two functions to say "form control"
+    // or "form-associated element" instead of "form element".
+    void registerFormElement(FormAssociatedElement*);
+    void removeFormElement(FormAssociatedElement*);
     void registerImgElement(HTMLImageElement*);
     void removeImgElement(HTMLImageElement*);
 
     bool prepareSubmit(Event*);
-    void submit(Event* = 0, bool activateSubmitButton = false, bool lockHistory = false);
+    void submit(Frame* javaScriptActiveFrame = 0);
     void reset();
 
     // Used to indicate a malformed state to keep from applying the bottom margin of the form.
@@ -88,18 +81,14 @@ public:
     void setDemoted(bool demoted) { m_demoted = demoted; }
     bool isDemoted() const { return m_demoted; }
 
-    virtual bool isURLAttribute(Attribute*) const;
-    
-    void submitClick(Event*);
+    void submitImplicitly(Event*, bool fromImplicitSubmissionTrigger);
     bool formWouldHaveSecureSubmission(const String& url);
 
     String name() const;
-    void setName(const String&);
 
     bool noValidate() const;
-    void setNoValidate(bool);
 
-    String acceptCharset() const { return m_formDataBuilder.acceptCharset(); }
+    String acceptCharset() const { return m_attributes.acceptCharset(); }
     void setAcceptCharset(const String&);
 
     String action() const;
@@ -109,7 +98,8 @@ public:
     void setMethod(const String&);
 
     virtual String target() const;
-    void setTarget(const String&);
+
+    bool wasUserSubmitted() const;
 
     HTMLFormControlElement* defaultButton() const;
 
@@ -118,36 +108,55 @@ public:
     PassRefPtr<HTMLFormControlElement> elementForAlias(const AtomicString&);
     void addElementAlias(HTMLFormControlElement*, const AtomicString& alias);
 
-    // FIXME: Change this to be private after getting rid of all the clients.
-    Vector<HTMLFormControlElement*> formElements;
-
     CheckedRadioButtons& checkedRadioButtons() { return m_checkedRadioButtons; }
-    
+
+    const Vector<FormAssociatedElement*>& associatedElements() const { return m_associatedElements; }
+
+private:
+    HTMLFormElement(const QualifiedName&, Document*);
+
+    virtual bool rendererIsNeeded(RenderStyle*);
+    virtual void insertedIntoDocument();
+    virtual void removedFromDocument();
+
+    virtual void handleLocalEvents(Event*);
+
+    virtual void parseMappedAttribute(Attribute*);
+
+    virtual bool isURLAttribute(Attribute*) const;
+
     virtual void documentDidBecomeActive();
 
-protected:
     virtual void willMoveToNewOwnerDocument();
     virtual void didMoveToNewOwnerDocument();
 
-private:
-    bool isMailtoForm() const;
-    TextEncoding dataEncoding() const;
-    PassRefPtr<FormData> createFormData(const CString& boundary);
-    unsigned formElementIndex(HTMLFormControlElement*);
+    void submit(Event*, bool activateSubmitButton, bool processingUserGesture, FormSubmissionTrigger);
+
+    unsigned formElementIndexWithFormAttribute(Element*);
+    unsigned formElementIndex(FormAssociatedElement*);
+    // Returns true if the submission should be proceeded.
+    bool validateInteractively(Event*);
+    // Validates each of the controls, and stores controls of which 'invalid'
+    // event was not canceled to the specified vector.
+    void collectUnhandledInvalidControls(Vector<RefPtr<FormAssociatedElement> >&);
 
     friend class HTMLFormCollection;
 
     typedef HashMap<RefPtr<AtomicStringImpl>, RefPtr<HTMLFormControlElement> > AliasMap;
 
-    FormDataBuilder m_formDataBuilder;
-    AliasMap* m_elementAliases;
-    CollectionCache* collectionInfo;
+    FormSubmission::Attributes m_attributes;
+    OwnPtr<AliasMap> m_elementAliases;
+    OwnPtr<CollectionCache> m_collectionCache;
 
     CheckedRadioButtons m_checkedRadioButtons;
-    
-    Vector<HTMLImageElement*> imgElements;
-    String m_url;
-    String m_target;
+
+    unsigned m_associatedElementsBeforeIndex;
+    unsigned m_associatedElementsAfterIndex;
+    Vector<FormAssociatedElement*> m_associatedElements;
+    Vector<HTMLImageElement*> m_imageElements;
+
+    bool m_wasUserSubmitted;
+
     bool m_autocomplete : 1;
     bool m_insubmit : 1;
     bool m_doingsubmit : 1;
