@@ -2,7 +2,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2003, 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2003, 2006, 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
  * Copyright (C) 2009 Rob Buis (rwlbuis@gmail.com)
  *
  * This library is free software; you can redistribute it and/or
@@ -24,20 +24,20 @@
 #include "config.h"
 #include "HTMLLinkElement.h"
 
-#include "CSSHelper.h"
+#include "Attribute.h"
 #include "CachedCSSStyleSheet.h"
-#include "DNS.h"
-#include "DocLoader.h"
+#include "CachedResourceLoader.h"
 #include "Document.h"
 #include "Frame.h"
 #include "FrameLoader.h"
 #include "FrameLoaderClient.h"
 #include "FrameTree.h"
 #include "HTMLNames.h"
-#include "MappedAttribute.h"
+#include "HTMLParserIdioms.h"
 #include "MediaList.h"
 #include "MediaQueryEvaluator.h"
 #include "Page.h"
+#include "ResourceHandle.h"
 #include "ScriptEventListener.h"
 #include "Settings.h"
 #include <wtf/StdLibExtras.h>
@@ -46,33 +46,44 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
-HTMLLinkElement::HTMLLinkElement(const QualifiedName& qName, Document *doc, bool createdByParser)
-    : HTMLElement(qName, doc)
-    , m_cachedSheet(0)
-    , m_disabledState(0)
+inline HTMLLinkElement::HTMLLinkElement(const QualifiedName& tagName, Document* document, bool createdByParser)
+    : HTMLElement(tagName, document)
+#if ENABLE(LINK_PREFETCH)
+    , m_onloadTimer(this, &HTMLLinkElement::onloadTimerFired)
+#endif
+    , m_disabledState(Unset)
     , m_loading(false)
-    , m_alternate(false)
-    , m_isStyleSheet(false)
-    , m_isIcon(false)
-    , m_isDNSPrefetch(false)
     , m_createdByParser(createdByParser)
 {
     ASSERT(hasTagName(linkTag));
 }
 
+PassRefPtr<HTMLLinkElement> HTMLLinkElement::create(const QualifiedName& tagName, Document* document, bool createdByParser)
+{
+    return adoptRef(new HTMLLinkElement(tagName, document, createdByParser));
+}
+
 HTMLLinkElement::~HTMLLinkElement()
 {
+    if (m_sheet)
+        m_sheet->clearOwnerNode();
+
     if (m_cachedSheet) {
         m_cachedSheet->removeClient(this);
         if (m_loading && !isDisabled() && !isAlternate())
             document()->removePendingSheet();
     }
+    
+#if ENABLE(LINK_PREFETCH)
+    if (m_cachedLinkPrefetch)
+        m_cachedLinkPrefetch->removeClient(this);
+#endif
 }
 
 void HTMLLinkElement::setDisabledState(bool _disabled)
 {
-    int oldDisabledState = m_disabledState;
-    m_disabledState = _disabled ? 2 : 1;
+    DisabledState oldDisabledState = m_disabledState;
+    m_disabledState = _disabled ? Disabled : EnabledViaScript;
     if (oldDisabledState != m_disabledState) {
         // If we change the disabled state while the sheet is still loading, then we have to
         // perform three checks:
@@ -80,11 +91,11 @@ void HTMLLinkElement::setDisabledState(bool _disabled)
             // Check #1: If the sheet becomes disabled while it was loading, and if it was either
             // a main sheet or a sheet that was previously enabled via script, then we need
             // to remove it from the list of pending sheets.
-            if (m_disabledState == 2 && (!m_alternate || oldDisabledState == 1))
+            if (m_disabledState == Disabled && (!m_relAttribute.m_isAlternate || oldDisabledState == EnabledViaScript))
                 document()->removePendingSheet();
 
             // Check #2: An alternate sheet becomes enabled while it is still loading.
-            if (m_alternate && m_disabledState == 1)
+            if (m_relAttribute.m_isAlternate && m_disabledState == EnabledViaScript)
                 document()->addPendingSheet();
 
             // Check #3: A main sheet becomes enabled while it was still loading and
@@ -92,7 +103,7 @@ void HTMLLinkElement::setDisabledState(bool _disabled)
             // happen (a double toggle for no reason essentially).  This happens on
             // virtualplastic.net, which manages to do about 12 enable/disables on only 3
             // sheets. :)
-            if (!m_alternate && m_disabledState == 1 && oldDisabledState == 2)
+            if (!m_relAttribute.m_isAlternate && m_disabledState == EnabledViaScript && oldDisabledState == Disabled)
                 document()->addPendingSheet();
 
             // If the sheet is already loading just bail.
@@ -100,10 +111,10 @@ void HTMLLinkElement::setDisabledState(bool _disabled)
         }
 
         // Load the sheet, since it's never been loaded before.
-        if (!m_sheet && m_disabledState == 1)
+        if (!m_sheet && m_disabledState == EnabledViaScript)
             process();
         else
-            document()->updateStyleSelector(); // Update the style selector.
+            document()->styleSelectorChanged(DeferRecalcStyle); // Update the style selector.
     }
 }
 
@@ -112,13 +123,13 @@ StyleSheet* HTMLLinkElement::sheet() const
     return m_sheet.get();
 }
 
-void HTMLLinkElement::parseMappedAttribute(MappedAttribute *attr)
+void HTMLLinkElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == relAttr) {
-        tokenizeRelAttribute(attr->value(), m_isStyleSheet, m_alternate, m_isIcon, m_isDNSPrefetch);
+        tokenizeRelAttribute(attr->value(), m_relAttribute);
         process();
     } else if (attr->name() == hrefAttr) {
-        m_url = document()->completeURL(deprecatedParseURL(attr->value()));
+        m_url = document()->completeURL(stripLeadingAndTrailingHTMLSpaces(attr->value()));
         process();
     } else if (attr->name() == typeAttr) {
         m_type = attr->value();
@@ -126,10 +137,14 @@ void HTMLLinkElement::parseMappedAttribute(MappedAttribute *attr)
     } else if (attr->name() == mediaAttr) {
         m_media = attr->value().string().lower();
         process();
-    } else if (attr->name() == disabledAttr) {
+    } else if (attr->name() == disabledAttr)
         setDisabledState(!attr->isNull());
-    } else if (attr->name() == onbeforeloadAttr)
+    else if (attr->name() == onbeforeloadAttr)
         setAttributeEventListener(eventNames().beforeloadEvent, createAttributeEventListener(this, attr));
+#if ENABLE(LINK_PREFETCH)
+    else if (attr->name() == onloadAttr)
+        setAttributeEventListener(eventNames().loadEvent, createAttributeEventListener(this, attr));
+#endif
     else {
         if (attr->name() == titleAttr && m_sheet)
             m_sheet->setTitle(attr->value());
@@ -137,21 +152,28 @@ void HTMLLinkElement::parseMappedAttribute(MappedAttribute *attr)
     }
 }
 
-void HTMLLinkElement::tokenizeRelAttribute(const AtomicString& rel, bool& styleSheet, bool& alternate, bool& icon, bool& dnsPrefetch)
+void HTMLLinkElement::tokenizeRelAttribute(const AtomicString& rel, RelAttribute& relAttribute)
 {
-    styleSheet = false;
-    icon = false; 
-    alternate = false;
-    dnsPrefetch = false;
+    relAttribute.m_isStyleSheet = false;
+    relAttribute.m_isIcon = false;
+    relAttribute.m_isAlternate = false;
+    relAttribute.m_isDNSPrefetch = false;
+#if ENABLE(LINK_PREFETCH)
+    relAttribute.m_isLinkPrefetch = false;
+#endif
     if (equalIgnoringCase(rel, "stylesheet"))
-        styleSheet = true;
+        relAttribute.m_isStyleSheet = true;
     else if (equalIgnoringCase(rel, "icon") || equalIgnoringCase(rel, "shortcut icon"))
-        icon = true;
+        relAttribute.m_isIcon = true;
     else if (equalIgnoringCase(rel, "dns-prefetch"))
-        dnsPrefetch = true;
+        relAttribute.m_isDNSPrefetch = true;
+#if ENABLE(LINK_PREFETCH)
+    else if (equalIgnoringCase(rel, "prefetch"))
+        relAttribute.m_isLinkPrefetch = true;
+#endif
     else if (equalIgnoringCase(rel, "alternate stylesheet") || equalIgnoringCase(rel, "stylesheet alternate")) {
-        styleSheet = true;
-        alternate = true;
+        relAttribute.m_isStyleSheet = true;
+        relAttribute.m_isAlternate = true;
     } else {
         // Tokenize the rel attribute and set bits based on specific keywords that we find.
         String relString = rel.string();
@@ -161,40 +183,60 @@ void HTMLLinkElement::tokenizeRelAttribute(const AtomicString& rel, bool& styleS
         Vector<String>::const_iterator end = list.end();
         for (Vector<String>::const_iterator it = list.begin(); it != end; ++it) {
             if (equalIgnoringCase(*it, "stylesheet"))
-                styleSheet = true;
+                relAttribute.m_isStyleSheet = true;
             else if (equalIgnoringCase(*it, "alternate"))
-                alternate = true;
+                relAttribute.m_isAlternate = true;
             else if (equalIgnoringCase(*it, "icon"))
-                icon = true;
+                relAttribute.m_isIcon = true;
         }
     }
 }
 
 void HTMLLinkElement::process()
 {
-    if (!inDocument())
+    if (!inDocument()) {
+        ASSERT(!m_sheet);
         return;
+    }
 
     String type = m_type.lower();
 
     // IE extension: location of small icon for locationbar / bookmarks
     // We'll record this URL per document, even if we later only use it in top level frames
-    if (m_isIcon && m_url.isValid() && !m_url.isEmpty())
+    if (m_relAttribute.m_isIcon && m_url.isValid() && !m_url.isEmpty())
         document()->setIconURL(m_url.string(), type);
 
-    if (m_isDNSPrefetch && document()->isDNSPrefetchEnabled() && m_url.isValid() && !m_url.isEmpty())
-        prefetchDNS(m_url.host());
+    if (m_relAttribute.m_isDNSPrefetch) {
+        Settings* settings = document()->settings();
+        // FIXME: The href attribute of the link element can be in "//hostname" form, and we shouldn't attempt
+        // to complete that as URL <https://bugs.webkit.org/show_bug.cgi?id=48857>.
+        if (settings && settings->dnsPrefetchingEnabled() && m_url.isValid() && !m_url.isEmpty())
+            ResourceHandle::prepareForURL(m_url);
+    }
+
+#if ENABLE(LINK_PREFETCH)
+    if (m_relAttribute.m_isLinkPrefetch && m_url.isValid() && document()->frame()) {
+        m_cachedLinkPrefetch = document()->cachedResourceLoader()->requestLinkPrefetch(m_url);
+        if (m_cachedLinkPrefetch)
+            m_cachedLinkPrefetch->addClient(this);
+    }
+#endif
 
     bool acceptIfTypeContainsTextCSS = document()->page() && document()->page()->settings() && document()->page()->settings()->treatsAnyTextCSSLinkAsStylesheet();
 
-    // Stylesheet
-    // This was buggy and would incorrectly match <link rel="alternate">, which has a different specified meaning. -dwh
-    if (m_disabledState != 2 && (m_isStyleSheet || (acceptIfTypeContainsTextCSS && type.contains("text/css"))) && document()->frame() && m_url.isValid()) {
-        // also, don't load style sheets for standalone documents
+    bool mediaIsScreen = true;
+    if (!m_media.isEmpty()) {
+        RefPtr<MediaList> media = MediaList::createAllowingDescriptionSyntax(m_media);
+        MediaQueryEvaluator screenEvaluator("screen", true);
+        mediaIsScreen = screenEvaluator.eval(media.get());
+    }
+    
+    if (m_disabledState != Disabled && (m_relAttribute.m_isStyleSheet || (acceptIfTypeContainsTextCSS && type.contains("text/css"))) 
+        && document()->frame() && m_url.isValid()) {
         
         String charset = getAttribute(charsetAttr);
         if (charset.isEmpty() && document()->frame())
-            charset = document()->frame()->loader()->encoding();
+            charset = document()->frame()->loader()->writer()->encoding();
 
         if (m_cachedSheet) {
             if (m_loading)
@@ -213,7 +255,9 @@ void HTMLLinkElement::process()
         if (!isAlternate())
             document()->addPendingSheet();
 
-        m_cachedSheet = document()->docLoader()->requestCSSStyleSheet(m_url, charset);
+        // Load non-screen stylesheets with low priority so they don't affect normal page loading.
+        ResourceLoadPriority priority = mediaIsScreen ? ResourceLoadPriorityUnresolved : ResourceLoadPriorityVeryLow;
+        m_cachedSheet = document()->cachedResourceLoader()->requestCSSStyleSheet(m_url, charset, priority);
         
         if (m_cachedSheet)
             m_cachedSheet->addClient(this);
@@ -226,7 +270,7 @@ void HTMLLinkElement::process()
     } else if (m_sheet) {
         // we no longer contain a stylesheet, e.g. perhaps rel or type was changed
         m_sheet = 0;
-        document()->updateStyleSelector();
+        document()->styleSelectorChanged(DeferRecalcStyle);
     }
 }
 
@@ -234,6 +278,7 @@ void HTMLLinkElement::insertedIntoDocument()
 {
     HTMLElement::insertedIntoDocument();
     document()->addStyleSheetCandidateNode(this, m_createdByParser);
+
     process();
 }
 
@@ -243,9 +288,14 @@ void HTMLLinkElement::removedFromDocument()
 
     document()->removeStyleSheetCandidateNode(this);
 
-    // FIXME: It's terrible to do a synchronous update of the style selector just because a <style> or <link> element got removed.
+    if (m_sheet) {
+        ASSERT(m_sheet->ownerNode() == this);
+        m_sheet->clearOwnerNode();
+        m_sheet = 0;
+    }
+
     if (document()->renderer())
-        document()->updateStyleSelector();
+        document()->styleSelectorChanged(DeferRecalcStyle);
 }
 
 void HTMLLinkElement::finishParsingChildren()
@@ -256,9 +306,14 @@ void HTMLLinkElement::finishParsingChildren()
 
 void HTMLLinkElement::setCSSStyleSheet(const String& href, const KURL& baseURL, const String& charset, const CachedCSSStyleSheet* sheet)
 {
+    if (!inDocument()) {
+        ASSERT(!m_sheet);
+        return;
+    }
+
     m_sheet = CSSStyleSheet::create(this, href, baseURL, charset);
 
-    bool strictParsing = !document()->inCompatMode();
+    bool strictParsing = !document()->inQuirksMode();
     bool enforceMIMEType = strictParsing;
     bool crossOriginCSS = false;
     bool validMIMEType = false;
@@ -266,7 +321,7 @@ void HTMLLinkElement::setCSSStyleSheet(const String& href, const KURL& baseURL,
 
     // Check to see if we should enforce the MIME type of the CSS resource in strict mode.
     // Running in iWeb 2 is one example of where we don't want to - <rdar://problem/6099748>
-    if (enforceMIMEType && document()->page() && !document()->page()->settings()->enforceCSSMIMETypeInStrictMode())
+    if (enforceMIMEType && document()->page() && !document()->page()->settings()->enforceCSSMIMETypeInNoQuirksMode())
         enforceMIMEType = false;
 
 #if defined(BUILDING_ON_TIGER) || defined(BUILDING_ON_LEOPARD)
@@ -323,6 +378,23 @@ bool HTMLLinkElement::isLoading() const
     return static_cast<CSSStyleSheet *>(m_sheet.get())->isLoading();
 }
 
+#if ENABLE(LINK_PREFETCH)
+void HTMLLinkElement::onloadTimerFired(Timer<HTMLLinkElement>* timer)
+{
+    ASSERT_UNUSED(timer, timer == &m_onloadTimer);
+    dispatchEvent(Event::create(eventNames().loadEvent, false, false));
+}
+
+void HTMLLinkElement::notifyFinished(CachedResource* resource)
+{
+    m_onloadTimer.startOneShot(0);
+    if (m_cachedLinkPrefetch.get() == resource) {
+        m_cachedLinkPrefetch->removeClient(this);
+        m_cachedLinkPrefetch = 0;
+    }
+}
+#endif
+
 bool HTMLLinkElement::sheetLoaded()
 {
     if (!isLoading() && !isDisabled() && !isAlternate()) {
@@ -337,105 +409,35 @@ bool HTMLLinkElement::isURLAttribute(Attribute *attr) const
     return attr->name() == hrefAttr;
 }
 
-bool HTMLLinkElement::disabled() const
-{
-    return !getAttribute(disabledAttr).isNull();
-}
-
-void HTMLLinkElement::setDisabled(bool disabled)
-{
-    setAttribute(disabledAttr, disabled ? "" : 0);
-}
-
-String HTMLLinkElement::charset() const
-{
-    return getAttribute(charsetAttr);
-}
-
-void HTMLLinkElement::setCharset(const String& value)
-{
-    setAttribute(charsetAttr, value);
-}
-
 KURL HTMLLinkElement::href() const
 {
     return document()->completeURL(getAttribute(hrefAttr));
 }
 
-void HTMLLinkElement::setHref(const String& value)
-{
-    setAttribute(hrefAttr, value);
-}
-
-String HTMLLinkElement::hreflang() const
-{
-    return getAttribute(hreflangAttr);
-}
-
-void HTMLLinkElement::setHreflang(const String& value)
-{
-    setAttribute(hreflangAttr, value);
-}
-
-String HTMLLinkElement::media() const
-{
-    return getAttribute(mediaAttr);
-}
-
-void HTMLLinkElement::setMedia(const String& value)
-{
-    setAttribute(mediaAttr, value);
-}
-
 String HTMLLinkElement::rel() const
 {
     return getAttribute(relAttr);
 }
 
-void HTMLLinkElement::setRel(const String& value)
-{
-    setAttribute(relAttr, value);
-}
-
-String HTMLLinkElement::rev() const
-{
-    return getAttribute(revAttr);
-}
-
-void HTMLLinkElement::setRev(const String& value)
-{
-    setAttribute(revAttr, value);
-}
-
 String HTMLLinkElement::target() const
 {
     return getAttribute(targetAttr);
 }
 
-void HTMLLinkElement::setTarget(const String& value)
-{
-    setAttribute(targetAttr, value);
-}
-
 String HTMLLinkElement::type() const
 {
     return getAttribute(typeAttr);
 }
 
-void HTMLLinkElement::setType(const String& value)
-{
-    setAttribute(typeAttr, value);
-}
-
 void HTMLLinkElement::addSubresourceAttributeURLs(ListHashSet<KURL>& urls) const
 {
     HTMLElement::addSubresourceAttributeURLs(urls);
 
     // Favicons are handled by a special case in LegacyWebArchive::create()
-    if (m_isIcon)
+    if (m_relAttribute.m_isIcon)
         return;
 
-    if (!m_isStyleSheet)
+    if (!m_relAttribute.m_isStyleSheet)
         return;
     
     // Append the URL of this link element.
