@@ -29,9 +29,14 @@
 #if ENABLE(VIDEO)
 
 #include "HTMLElement.h"
+#include "ActiveDOMObject.h"
 #include "MediaCanStartListener.h"
 #include "MediaPlayer.h"
 
+#if ENABLE(PLUGIN_PROXY_FOR_VIDEO)
+#include "MediaPlayerProxy.h"
+#endif
+
 namespace WebCore {
 
 class Event;
@@ -39,12 +44,15 @@ class HTMLSourceElement;
 class MediaError;
 class KURL;
 class TimeRanges;
+#if ENABLE(PLUGIN_PROXY_FOR_VIDEO)
+class Widget;
+#endif
 
 // FIXME: The inheritance from MediaPlayerClient here should be private inheritance.
 // But it can't be until the Chromium WebMediaPlayerClientImpl class is fixed so it
 // no longer depends on typecasting a MediaPlayerClient to an HTMLMediaElement.
 
-class HTMLMediaElement : public HTMLElement, public MediaPlayerClient, private MediaCanStartListener {
+class HTMLMediaElement : public HTMLElement, public MediaPlayerClient, private MediaCanStartListener, private ActiveDOMObject {
 public:
     MediaPlayer* player() const { return m_player.get(); }
     
@@ -57,6 +65,7 @@ public:
 
     // Eventually overloaded in HTMLVideoElement
     virtual bool supportsFullscreen() const { return false; };
+
     virtual bool supportsSave() const;
     
     PlatformMedia platformMedia() const;
@@ -75,11 +84,10 @@ public:
     PassRefPtr<MediaError> error() const;
 
 // network state
-    KURL src() const;
     void setSrc(const String&);
     String currentSrc() const;
 
-    enum NetworkState { NETWORK_EMPTY, NETWORK_IDLE, NETWORK_LOADING, NETWORK_LOADED, NETWORK_NO_SOURCE };
+    enum NetworkState { NETWORK_EMPTY, NETWORK_IDLE, NETWORK_LOADING, NETWORK_NO_SOURCE };
     NetworkState networkState() const;
     
     String preload() const;    
@@ -139,11 +147,14 @@ public:
     float percentLoaded() const;
 
 #if ENABLE(PLUGIN_PROXY_FOR_VIDEO)
+    void allocateMediaPlayerIfNecessary();
     void setNeedWidgetUpdate(bool needWidgetUpdate) { m_needWidgetUpdate = needWidgetUpdate; }
-    void deliverNotification(MediaPlayerProxyNotificationType);
-    void setMediaPlayerProxy(WebMediaPlayerProxy*);
-    String initialURL();
+    void deliverNotification(MediaPlayerProxyNotificationType notification);
+    void setMediaPlayerProxy(WebMediaPlayerProxy* proxy);
+    void getPluginProxyParams(KURL& url, Vector<String>& names, Vector<String>& values);
     virtual void finishParsingChildren();
+    void createMediaPlayerProxy();
+    void updateWidget(bool onlyCreateNonNetscapePlugins);
 #endif
 
     bool hasSingleSecurityOrigin() const { return !m_player || m_player->hasSingleSecurityOrigin(); }
@@ -158,15 +169,25 @@ public:
 
     bool processingUserGesture() const;
 
+    void sourceWillBeRemoved(HTMLSourceElement*);
+    void sourceWasAdded(HTMLSourceElement*);
+
 protected:
     HTMLMediaElement(const QualifiedName&, Document*);
     virtual ~HTMLMediaElement();
 
-    virtual void parseMappedAttribute(MappedAttribute *);
+    virtual void parseMappedAttribute(Attribute*);
+    virtual bool isURLAttribute(Attribute*) const;
     virtual void attach();
 
+    virtual void willMoveToNewOwnerDocument();
+    virtual void didMoveToNewOwnerDocument();
+
+    enum DisplayMode { Unknown, None, Poster, Video };
+    DisplayMode displayMode() const { return m_displayMode; }
+    virtual void setDisplayMode(DisplayMode mode) { m_displayMode = mode; }
+
 private:
-    virtual bool checkDTD(const Node* newChild);    
     virtual void attributeChanged(Attribute*, bool preserveDecls);
     virtual bool rendererIsNeeded(RenderStyle*);
     virtual RenderObject* createRenderer(RenderArena*, RenderStyle*);
@@ -176,20 +197,20 @@ private:
     
     virtual void defaultEventHandler(Event*);
     
-    float getTimeOffsetAttribute(const QualifiedName&, float valueOnError) const;
-    void setTimeOffsetAttribute(const QualifiedName&, float value);
+    // ActiveDOMObject functions.
+    virtual bool canSuspend() const;
+    virtual void suspend(ReasonForSuspension);
+    virtual void resume();
+    virtual void stop();
+    virtual bool hasPendingActivity() const;
     
-    virtual void documentWillBecomeInactive();
-    virtual void documentDidBecomeActive();
     virtual void mediaVolumeDidChange();
-    virtual void updatePosterImage() { }
+
+    virtual void updateDisplayState() { }
     
     void setReadyState(MediaPlayer::ReadyState);
     void setNetworkState(MediaPlayer::NetworkState);
 
-    virtual void willMoveToNewOwnerDocument();
-    virtual void didMoveToNewOwnerDocument();
-
     virtual Document* mediaPlayerOwningDocument();
     virtual void mediaPlayerNetworkStateChanged(MediaPlayer*);
     virtual void mediaPlayerReadyStateChanged(MediaPlayer*);
@@ -198,6 +219,7 @@ private:
     virtual void mediaPlayerMuteChanged(MediaPlayer*);
     virtual void mediaPlayerDurationChanged(MediaPlayer*);
     virtual void mediaPlayerRateChanged(MediaPlayer*);
+    virtual void mediaPlayerPlaybackStateChanged(MediaPlayer*);
     virtual void mediaPlayerSawUnsupportedTracks(MediaPlayer*);
     virtual void mediaPlayerRepaint(MediaPlayer*);
     virtual void mediaPlayerSizeChanged(MediaPlayer*);
@@ -205,6 +227,7 @@ private:
     virtual bool mediaPlayerRenderingCanBeAccelerated(MediaPlayer*);
     virtual void mediaPlayerRenderingModeChanged(MediaPlayer*);
 #endif
+    virtual void mediaPlayerEngineUpdated(MediaPlayer*);
 
     void loadTimerFired(Timer<HTMLMediaElement>*);
     void asyncEventTimerFired(Timer<HTMLMediaElement>*);
@@ -244,7 +267,8 @@ private:
     void pauseInternal();
 
     void prepareForLoad();
-    
+    void allowVideoRendering();
+
     bool processingMediaPlayerCallback() const { return m_processingMediaPlayerCallback > 0; }
     void beginProcessingMediaPlayerCallback() { ++m_processingMediaPlayerCallback; }
     void endProcessingMediaPlayerCallback() { ASSERT(m_processingMediaPlayerCallback); --m_processingMediaPlayerCallback; }
@@ -265,6 +289,11 @@ private:
 
     virtual void mediaCanStart();
 
+    void setShouldDelayLoadEvent(bool);
+
+    void invalidateCachedTime();
+    void refreshCachedTime() const;
+
     // Restrictions to change default behaviors. This is effectively a compile time choice at the moment
     // because there are no accessor functions.
     enum BehaviorRestrictions {
@@ -285,6 +314,7 @@ private:
     bool m_webkitPreservesPitch;
     NetworkState m_networkState;
     ReadyState m_readyState;
+    ReadyState m_readyStateMaximum;
     String m_currentSrc;
     
     RefPtr<MediaError> m_error;
@@ -305,23 +335,31 @@ private:
     enum LoadState { WaitingForSource, LoadingFromSrcAttr, LoadingFromSourceElement };
     LoadState m_loadState;
     HTMLSourceElement* m_currentSourceNode;
-    
+    Node* m_nextChildNodeToConsider;
+    Node* sourceChildEndOfListValue() { return static_cast<Node*>(this); }
+
     OwnPtr<MediaPlayer> m_player;
+#if ENABLE(PLUGIN_PROXY_FOR_VIDEO)
+    RefPtr<Widget> m_proxyWidget;
+#endif
 
     BehaviorRestrictions m_restrictions;
     
     MediaPlayer::Preload m_preload;
 
-    bool m_playing;
+    DisplayMode m_displayMode;
 
     // Counter incremented while processing a callback from the media player, so we can avoid
     // calling the media engine recursively.
     int m_processingMediaPlayerCallback;
 
-    bool m_isWaitingUntilMediaCanStart;
+    mutable float m_cachedTime;
+    mutable double m_cachedTimeWallClockUpdateTime;
+    mutable double m_minimumWallClockTimeToCacheMediaTime;
 
-    bool m_processingLoad : 1;
-    bool m_delayingTheLoadEvent : 1;
+    bool m_playing : 1;
+    bool m_isWaitingUntilMediaCanStart : 1;
+    bool m_shouldDelayLoadEvent : 1;
     bool m_haveFiredLoadedData : 1;
     bool m_inActiveDocument : 1;
     bool m_autoplaying : 1;
@@ -349,6 +387,8 @@ private:
 #endif
 
     bool m_dispatchingCanPlayEvent : 1;
+    bool m_loadInitiatedByUserGesture : 1;
+    bool m_completelyLoaded : 1;
 };
 
 } //namespace
