@@ -2,8 +2,9 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2004, 2005, 2006 Apple Computer, Inc.
  *           (C) 2006 Alexey Proskuryakov (ap@nypop.com)
+ * Copyright (C) 2004, 2005, 2006, 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2010 Google Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,12 +26,12 @@
 #include "config.h"
 #include "HTMLOptionElement.h"
 
+#include "Attribute.h"
 #include "CSSStyleSelector.h"
 #include "Document.h"
 #include "ExceptionCode.h"
 #include "HTMLNames.h"
 #include "HTMLSelectElement.h"
-#include "MappedAttribute.h"
 #include "NodeRenderStyle.h"
 #include "RenderMenuList.h"
 #include "Text.h"
@@ -41,16 +42,40 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
-HTMLOptionElement::HTMLOptionElement(const QualifiedName& tagName, Document* doc, HTMLFormElement* f)
-    : HTMLFormControlElement(tagName, doc, f)
-    , m_style(0)
+HTMLOptionElement::HTMLOptionElement(const QualifiedName& tagName, Document* document, HTMLFormElement* form)
+    : HTMLFormControlElement(tagName, document, form)
 {
     ASSERT(hasTagName(optionTag));
 }
 
-bool HTMLOptionElement::checkDTD(const Node* newChild)
+PassRefPtr<HTMLOptionElement> HTMLOptionElement::create(Document* document, HTMLFormElement* form)
 {
-    return newChild->isTextNode() || newChild->hasTagName(scriptTag);
+    return adoptRef(new HTMLOptionElement(optionTag, document, form));
+}
+
+PassRefPtr<HTMLOptionElement> HTMLOptionElement::create(const QualifiedName& tagName, Document* document, HTMLFormElement* form)
+{
+    return adoptRef(new HTMLOptionElement(tagName, document, form));
+}
+
+PassRefPtr<HTMLOptionElement> HTMLOptionElement::createForJSConstructor(Document* document, const String& data, const String& value,
+        bool defaultSelected, bool selected, ExceptionCode& ec)
+{
+    RefPtr<HTMLOptionElement> element = adoptRef(new HTMLOptionElement(optionTag, document));
+
+    RefPtr<Text> text = Text::create(document, data.isNull() ? "" : data);
+
+    ec = 0;
+    element->appendChild(text.release(), ec);
+    if (ec)
+        return 0;
+
+    if (!value.isNull())
+        element->setValue(value);
+    element->setDefaultSelected(defaultSelected);
+    element->setSelected(selected);
+
+    return element.release();
 }
 
 void HTMLOptionElement::attach()
@@ -113,7 +138,7 @@ int HTMLOptionElement::index() const
     return OptionElement::optionIndex(ownerSelectElement(), this);
 }
 
-void HTMLOptionElement::parseMappedAttribute(MappedAttribute *attr)
+void HTMLOptionElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == selectedAttr)
         m_data.setSelected(!attr->isNull());
@@ -168,19 +193,19 @@ void HTMLOptionElement::childrenChanged(bool changedByParser, Node* beforeChange
 
 HTMLSelectElement* HTMLOptionElement::ownerSelectElement() const
 {
-    Node* select = parentNode();
-    while (select && !select->hasTagName(selectTag))
+    ContainerNode* select = parentNode();
+    while (select && !(select->hasTagName(selectTag) || select->hasTagName(keygenTag)))
         select = select->parentNode();
 
     if (!select)
         return 0;
-    
+
     return static_cast<HTMLSelectElement*>(select);
 }
 
 bool HTMLOptionElement::defaultSelected() const
 {
-    return !getAttribute(selectedAttr).isNull();
+    return fastHasAttribute(selectedAttr);
 }
 
 void HTMLOptionElement::setDefaultSelected(bool b)
@@ -193,19 +218,14 @@ String HTMLOptionElement::label() const
     return m_data.label();
 }
 
-void HTMLOptionElement::setLabel(const String& value)
-{
-    setAttribute(labelAttr, value);
-}
-
 void HTMLOptionElement::setRenderStyle(PassRefPtr<RenderStyle> newStyle)
 {
     m_style = newStyle;
 }
 
-RenderStyle* HTMLOptionElement::nonRendererRenderStyle() const 
-{ 
-    return m_style.get(); 
+RenderStyle* HTMLOptionElement::nonRendererRenderStyle() const
+{
+    return m_style.get();
 }
 
 String HTMLOptionElement::textIndentedToRespectGroupLabel() const
@@ -214,8 +234,8 @@ String HTMLOptionElement::textIndentedToRespectGroupLabel() const
 }
 
 bool HTMLOptionElement::disabled() const
-{ 
-    return ownElementDisabled() || (parentNode() && static_cast<HTMLFormControlElement*>(parentNode())->disabled()); 
+{
+    return ownElementDisabled() || (parentNode() && static_cast<HTMLFormControlElement*>(parentNode())->disabled());
 }
 
 void HTMLOptionElement::insertedIntoTree(bool deep)
@@ -227,7 +247,7 @@ void HTMLOptionElement::insertedIntoTree(bool deep)
             select->setSelectedIndex(index(), false);
         select->scrollToSelection();
     }
-    
+
     HTMLFormControlElement::insertedIntoTree(deep);
 }
 
