@@ -1,8 +1,9 @@
 /*
+ * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2007, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 Apple Inc. All rights reserved.
  * Copyright (C) 2010 Google Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,15 +34,17 @@ namespace WebCore {
 
 class HTMLOptionElement;
 class HTMLOptionsCollection;
-class KeyboardEvent;
 
 class HTMLSelectElement : public HTMLFormControlElementWithState, public SelectElement {
 public:
-    HTMLSelectElement(const QualifiedName&, Document*, HTMLFormElement* = 0);
+    static PassRefPtr<HTMLSelectElement> create(const QualifiedName&, Document*, HTMLFormElement*);
 
     virtual int selectedIndex() const;
     virtual void setSelectedIndex(int index, bool deselect = true);
-    virtual void setSelectedIndexByUser(int index, bool deselect = true, bool fireOnChangeNow = false);
+    virtual void setSelectedIndexByUser(int index, bool deselect = true, bool fireOnChangeNow = false, bool allowMultipleSelection = false);
+
+    // For ValidityState
+    bool valueMissing() const;
 
     unsigned length() const;
 
@@ -80,10 +83,14 @@ public:
 
     void scrollToSelection();
 
-private:
-    virtual int tagPriority() const { return 6; }
-    virtual bool checkDTD(const Node* newChild);
+    void listBoxSelectItem(int listIndex, bool allowMultiplySelections, bool shift, bool fireOnChangeNow = true);
 
+    virtual void updateValidity() { setNeedsValidityCheck(); }
+
+protected:
+    HTMLSelectElement(const QualifiedName&, Document*, HTMLFormElement*);
+
+private:
     virtual const AtomicString& formControlType() const;
     
     virtual bool isKeyboardFocusable(KeyboardEvent*) const;
@@ -103,7 +110,7 @@ private:
     virtual bool saveFormControlState(String& value) const;
     virtual void restoreFormControlState(const String&);
 
-    virtual void parseMappedAttribute(MappedAttribute*);
+    virtual void parseMappedAttribute(Attribute*);
 
     virtual RenderObject* createRenderer(RenderArena*, RenderStyle *);
     virtual bool appendFormData(FormDataList&, bool);
@@ -132,7 +139,10 @@ private:
 
     virtual void insertedIntoTree(bool);
 
-    virtual bool isOptionalFormControl() const { return true; }
+    virtual bool isOptionalFormControl() const { return !isRequiredFormControl(); }
+    virtual bool isRequiredFormControl() const;
+
+    bool hasPlaceholderLabelOption() const;
 
     SelectElementData m_data;
     CollectionCache m_collectionInfo;
