@@ -1,10 +1,10 @@
-/**
+/*
  * Copyright (C) 1997 Martin Jones (mjones@kde.org)
  *           (C) 1997 Torben Weis (weis@kde.org)
  *           (C) 1998 Waldo Bastian (bastian@kde.org)
  *           (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
- * Copyright (C) 2003, 2004, 2005, 2006 Apple Computer, Inc.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,11 +25,11 @@
 #include "config.h"
 #include "HTMLTableCellElement.h"
 
+#include "Attribute.h"
 #include "CSSPropertyNames.h"
 #include "CSSValueKeywords.h"
 #include "HTMLNames.h"
 #include "HTMLTableElement.h"
-#include "MappedAttribute.h"
 #include "RenderTableCell.h"
 
 using std::max;
@@ -42,19 +42,18 @@ static const int maxRowspan = 8190;
 
 using namespace HTMLNames;
 
-HTMLTableCellElement::HTMLTableCellElement(const QualifiedName& tagName, Document *doc)
-    : HTMLTablePartElement(tagName, doc)
-    , _row(-1)
-    , _col(-1)
-    , rSpan(1)
-    , cSpan(1)
-    , rowHeight(0)
-    , m_solid(false)
+inline HTMLTableCellElement::HTMLTableCellElement(const QualifiedName& tagName, Document* document)
+    : HTMLTablePartElement(tagName, document)
+    , m_row(-1)
+    , m_col(-1)
+    , m_rowSpan(1)
+    , m_colSpan(1)
 {
 }
 
-HTMLTableCellElement::~HTMLTableCellElement()
+PassRefPtr<HTMLTableCellElement> HTMLTableCellElement::create(const QualifiedName& tagName, Document* document)
 {
+    return adoptRef(new HTMLTableCellElement(tagName, document));
 }
 
 int HTMLTableCellElement::cellIndex() const
@@ -84,16 +83,14 @@ bool HTMLTableCellElement::mapToEntry(const QualifiedName& attrName, MappedAttri
     return HTMLTablePartElement::mapToEntry(attrName, result);
 }
 
-void HTMLTableCellElement::parseMappedAttribute(MappedAttribute *attr)
+void HTMLTableCellElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == rowspanAttr) {
-        rSpan = !attr->isNull() ? attr->value().toInt() : 1;
-        rSpan = max(1, min(rSpan, maxRowspan));
+        m_rowSpan = max(1, min(attr->value().toInt(), maxRowspan));
         if (renderer() && renderer()->isTableCell())
             toRenderTableCell(renderer())->updateFromElement();
     } else if (attr->name() == colspanAttr) {
-        cSpan = !attr->isNull() ? attr->value().toInt() : 1;
-        cSpan = max(1, cSpan);
+        m_colSpan = max(1, attr->value().toInt());
         if (renderer() && renderer()->isTableCell())
             toRenderTableCell(renderer())->updateFromElement();
     } else if (attr->name() == nowrapAttr) {
@@ -118,7 +115,7 @@ void HTMLTableCellElement::parseMappedAttribute(MappedAttribute *attr)
 // used by table cells to share style decls created by the enclosing table.
 void HTMLTableCellElement::additionalAttributeStyleDecls(Vector<CSSMutableStyleDeclaration*>& results)
 {
-    Node* p = parentNode();
+    ContainerNode* p = parentNode();
     while (p && !p->hasTagName(tableTag))
         p = p->parentNode();
     if (!p)
@@ -136,61 +133,11 @@ String HTMLTableCellElement::abbr() const
     return getAttribute(abbrAttr);
 }
 
-void HTMLTableCellElement::setAbbr(const String &value)
-{
-    setAttribute(abbrAttr, value);
-}
-
-String HTMLTableCellElement::align() const
-{
-    return getAttribute(alignAttr);
-}
-
-void HTMLTableCellElement::setAlign(const String &value)
-{
-    setAttribute(alignAttr, value);
-}
-
 String HTMLTableCellElement::axis() const
 {
     return getAttribute(axisAttr);
 }
 
-void HTMLTableCellElement::setAxis(const String &value)
-{
-    setAttribute(axisAttr, value);
-}
-
-String HTMLTableCellElement::bgColor() const
-{
-    return getAttribute(bgcolorAttr);
-}
-
-void HTMLTableCellElement::setBgColor(const String &value)
-{
-    setAttribute(bgcolorAttr, value);
-}
-
-String HTMLTableCellElement::ch() const
-{
-    return getAttribute(charAttr);
-}
-
-void HTMLTableCellElement::setCh(const String &value)
-{
-    setAttribute(charAttr, value);
-}
-
-String HTMLTableCellElement::chOff() const
-{
-    return getAttribute(charoffAttr);
-}
-
-void HTMLTableCellElement::setChOff(const String &value)
-{
-    setAttribute(charoffAttr, value);
-}
-
 void HTMLTableCellElement::setColSpan(int n)
 {
     setAttribute(colspanAttr, String::number(n));
@@ -201,31 +148,6 @@ String HTMLTableCellElement::headers() const
     return getAttribute(headersAttr);
 }
 
-void HTMLTableCellElement::setHeaders(const String &value)
-{
-    setAttribute(headersAttr, value);
-}
-
-String HTMLTableCellElement::height() const
-{
-    return getAttribute(heightAttr);
-}
-
-void HTMLTableCellElement::setHeight(const String &value)
-{
-    setAttribute(heightAttr, value);
-}
-
-bool HTMLTableCellElement::noWrap() const
-{
-    return !getAttribute(nowrapAttr).isNull();
-}
-
-void HTMLTableCellElement::setNoWrap(bool b)
-{
-    setAttribute(nowrapAttr, b ? "" : 0);
-}
-
 void HTMLTableCellElement::setRowSpan(int n)
 {
     setAttribute(rowspanAttr, String::number(n));
@@ -236,31 +158,6 @@ String HTMLTableCellElement::scope() const
     return getAttribute(scopeAttr);
 }
 
-void HTMLTableCellElement::setScope(const String &value)
-{
-    setAttribute(scopeAttr, value);
-}
-
-String HTMLTableCellElement::vAlign() const
-{
-    return getAttribute(valignAttr);
-}
-
-void HTMLTableCellElement::setVAlign(const String &value)
-{
-    setAttribute(valignAttr, value);
-}
-
-String HTMLTableCellElement::width() const
-{
-    return getAttribute(widthAttr);
-}
-
-void HTMLTableCellElement::setWidth(const String &value)
-{
-    setAttribute(widthAttr, value);
-}
-
 void HTMLTableCellElement::addSubresourceAttributeURLs(ListHashSet<KURL>& urls) const
 {
     HTMLTablePartElement::addSubresourceAttributeURLs(urls);
@@ -268,4 +165,20 @@ void HTMLTableCellElement::addSubresourceAttributeURLs(ListHashSet<KURL>& urls)
     addSubresourceURL(urls, document()->completeURL(getAttribute(backgroundAttr)));
 }
 
+HTMLTableCellElement* HTMLTableCellElement::cellAbove() const
+{
+    RenderObject* cellRenderer = renderer();
+    if (!cellRenderer)
+        return 0;
+    if (!cellRenderer->isTableCell())
+        return 0;
+
+    RenderTableCell* tableCellRenderer = toRenderTableCell(cellRenderer);
+    RenderTableCell* cellAboveRenderer = tableCellRenderer->table()->cellAbove(tableCellRenderer);
+    if (!cellAboveRenderer)
+        return 0;
+
+    return static_cast<HTMLTableCellElement*>(cellAboveRenderer->node());
 }
+
+} // namespace WebCore
