@@ -2,7 +2,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2007, 2010 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,68 +33,41 @@ class VisibleSelection;
 
 class HTMLTextAreaElement : public HTMLTextFormControlElement {
 public:
-    HTMLTextAreaElement(const QualifiedName&, Document*, HTMLFormElement* = 0);
-
-    virtual bool checkDTD(const Node* newChild) { return newChild->isTextNode(); }
+    static PassRefPtr<HTMLTextAreaElement> create(const QualifiedName&, Document*, HTMLFormElement*);
 
     int cols() const { return m_cols; }
     int rows() const { return m_rows; }
 
     bool shouldWrapText() const { return m_wrap != NoWrap; }
 
-    virtual bool isEnumeratable() const { return true; }
-
-    virtual const AtomicString& formControlType() const;
-
-    virtual bool saveFormControlState(String& value) const;
-    virtual void restoreFormControlState(const String&);
-
-    bool readOnly() const { return isReadOnlyFormControl(); }
-
-    virtual bool isTextFormControl() const { return true; }
-
-    virtual bool valueMissing() const { return isRequiredFormControl() && !disabled() && !readOnly() && value().isEmpty(); }
-
-    virtual void childrenChanged(bool changedByParser = false, Node* beforeChange = 0, Node* afterChange = 0, int childCountDelta = 0);
-    virtual void parseMappedAttribute(MappedAttribute*);
-    virtual RenderObject* createRenderer(RenderArena*, RenderStyle*);
-    virtual bool appendFormData(FormDataList&, bool);
-    virtual void reset();
-    virtual void defaultEventHandler(Event*);
-    virtual bool isMouseFocusable() const;
-    virtual bool isKeyboardFocusable(KeyboardEvent*) const;
-    virtual void updateFocusAppearance(bool restorePreviousSelection);
-
-    String value() const;
+    virtual String value() const;
     void setValue(const String&);
     String defaultValue() const;
     void setDefaultValue(const String&);
     int textLength() const { return value().length(); }
-    int maxLength() const;
+    virtual int maxLength() const;
     void setMaxLength(int, ExceptionCode&);
-    virtual bool tooLong() const;
+    bool valueMissing(const String& value) const { return isRequiredFormControl() && !disabled() && !readOnly() && value.isEmpty(); }
+    bool tooLong(const String&, NeedsToCheckDirtyFlag) const;
+    bool isValidValue(const String&) const;
     
     void rendererWillBeDestroyed();
     
-    virtual void accessKeyAction(bool sendToAnyElement);
-    
-    const AtomicString& accessKey() const;
-    void setAccessKey(const String&);
-
     void setCols(int);
     void setRows(int);
     
     void cacheSelection(int s, int e) { m_cachedSelectionStart = s; m_cachedSelectionEnd = e; };
 
-    virtual bool shouldUseInputMethod() const;
-
 private:
+    HTMLTextAreaElement(const QualifiedName&, Document*, HTMLFormElement*);
+
     enum WrapMethod { NoWrap, SoftWrap, HardWrap };
 
     void handleBeforeTextInsertedEvent(BeforeTextInsertedEvent*) const;
     static String sanitizeUserInputValue(const String&, unsigned maxLength);
     void updateValue() const;
     void setNonDirtyValue(const String&);
+    void setValueCommon(const String&);
 
     virtual bool supportsPlaceholder() const { return true; }
     virtual bool isEmptyValue() const { return value().isEmpty(); }
@@ -104,6 +77,30 @@ private:
     virtual bool isOptionalFormControl() const { return !isRequiredFormControl(); }
     virtual bool isRequiredFormControl() const { return required(); }
 
+    virtual void defaultEventHandler(Event*);
+
+    virtual bool isEnumeratable() const { return true; }
+
+    virtual const AtomicString& formControlType() const;
+
+    virtual bool saveFormControlState(String& value) const;
+    virtual void restoreFormControlState(const String&);
+
+    virtual bool isTextFormControl() const { return true; }
+
+    virtual void childrenChanged(bool changedByParser = false, Node* beforeChange = 0, Node* afterChange = 0, int childCountDelta = 0);
+    virtual void parseMappedAttribute(Attribute*);
+    virtual RenderObject* createRenderer(RenderArena*, RenderStyle*);
+    virtual bool appendFormData(FormDataList&, bool);
+    virtual void reset();
+    virtual bool isMouseFocusable() const;
+    virtual bool isKeyboardFocusable(KeyboardEvent*) const;
+    virtual void updateFocusAppearance(bool restorePreviousSelection);
+
+    virtual void accessKeyAction(bool sendToAnyElement);
+
+    virtual bool shouldUseInputMethod() const;
+
     int m_rows;
     int m_cols;
     WrapMethod m_wrap;
