@@ -30,33 +30,40 @@
 #ifndef InspectorDOMAgent_h
 #define InspectorDOMAgent_h
 
-#include "AtomicString.h"
+#include "Document.h"
 #include "EventListener.h"
 #include "EventTarget.h"
-#include "ScriptArray.h"
-#include "ScriptObject.h"
-#include "ScriptState.h"
+#include "InspectorValues.h"
+#include "NodeList.h"
+#include "Timer.h"
 
+#include <wtf/Deque.h>
 #include <wtf/ListHashSet.h>
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/PassRefPtr.h>
 #include <wtf/RefPtr.h>
+#include <wtf/text/AtomicString.h>
 
 namespace WebCore {
     class ContainerNode;
     class CSSRule;
+    class CSSRuleList;
     class CSSStyleDeclaration;
     class CSSStyleRule;
     class CSSStyleSheet;
+    class CharacterData;
     class Element;
     class Event;
-    class Document;
+    class InspectorDOMAgent;
     class InspectorFrontend;
+    class MatchJob;
     class NameNodeMap;
     class Node;
     class Page;
 
+#if ENABLE(INSPECTOR)
+
     struct EventListenerInfo {
         EventListenerInfo(Node* node, const AtomicString& eventType, const EventListenerVector& eventListenerVector)
             : node(node)
@@ -72,6 +79,14 @@ namespace WebCore {
 
     class InspectorDOMAgent : public EventListener {
     public:
+        struct DOMListener {
+            virtual ~DOMListener()
+            {
+            }
+            virtual void didRemoveDocument(Document*) = 0;
+            virtual void didRemoveDOMNode(Node*) = 0;
+        };
+
         static PassRefPtr<InspectorDOMAgent> create(InspectorFrontend* frontend)
         {
             return adoptRef(new InspectorDOMAgent(frontend));
@@ -92,22 +107,21 @@ namespace WebCore {
         virtual bool operator==(const EventListener& other);
 
         // Methods called from the frontend for DOM nodes inspection.
-        void getChildNodes(long callId, long nodeId);
-        void setAttribute(long callId, long elementId, const String& name, const String& value);
-        void removeAttribute(long callId, long elementId, const String& name);
-        void setTextNodeValue(long callId, long nodeId, const String& value);
-        void getEventListenersForNode(long callId, long nodeId);
-
-        // Methods called from the frontend for CSS styles inspection.
-        void getStyles(long callId, long nodeId, bool authorOnly);
-        void getInlineStyle(long callId, long nodeId);
-        void getComputedStyle(long callId, long nodeId);
-        void applyStyleText(long callId, long styleId, const String& styleText, const String& propertyName);
-        void setStyleText(long callId, long styleId, const String& cssText);
-        void setStyleProperty(long callId, long styleId, const String& name, const String& value);
-        void toggleStyleEnabled(long callId, long styleId, const String& propertyName, bool disabled);
-        void setRuleSelector(long callId, long ruleId, const String& selector, long selectedNodeId);
-        void addRule(long callId, const String& selector, long selectedNodeId);
+        void getChildNodes(long nodeId);
+        void setAttribute(long elementId, const String& name, const String& value, bool* success);
+        void removeAttribute(long elementId, const String& name, bool* success);
+        void removeNode(long nodeId, long* outNodeId);
+        void changeTagName(long nodeId, const String& tagName, long* newId);
+        void getOuterHTML(long nodeId, WTF::String* outerHTML);
+        void setOuterHTML(long nodeId, const String& outerHTML, long* newId);
+        void setTextNodeValue(long nodeId, const String& value, bool* success);
+        void getEventListenersForNode(long nodeId, long* outNodeId, RefPtr<InspectorArray>* listenersArray);
+        void addInspectedNode(long nodeId);
+        void performSearch(const String& whitespaceTrimmedQuery, bool runSynchronously);
+        void searchCanceled();
+        bool shouldBreakOnNodeInsertion(Node* node, Node* parent, PassRefPtr<InspectorObject> details);
+        bool shouldBreakOnNodeRemoval(Node* node, PassRefPtr<InspectorObject> details);
+        bool shouldBreakOnAttributeModification(Element* element, PassRefPtr<InspectorObject> details);
 
         // Methods called from the InspectorController.
         void setDocument(Document* document);
@@ -116,13 +130,23 @@ namespace WebCore {
         void didInsertDOMNode(Node*);
         void didRemoveDOMNode(Node*);
         void didModifyDOMAttr(Element*);
+        void characterDataModified(CharacterData*);
 
         Node* nodeForId(long nodeId);
-        Node* nodeForPath(const String& path);
         long pushNodePathToFrontend(Node* node);
         void pushChildNodesToFrontend(long nodeId);
+        void pushNodeByPathToFrontend(const String& path, long* nodeId);
+        long inspectedNode(unsigned long num);
+        void copyNode(long nodeId);
+        const ListHashSet<RefPtr<Document> >& documents() { return m_documents; }
+        void setDOMListener(DOMListener*);
 
-   private:
+        String documentURLString(Document* document) const;
+
+        void setDOMBreakpoint(long nodeId, long type);
+        void removeDOMBreakpoint(long nodeId, long type);
+
+    private:
         void startListening(Document* document);
         void stopListening(Document* document);
 
@@ -135,12 +159,14 @@ namespace WebCore {
 
         bool pushDocumentToFrontend();
 
-        ScriptArray getMatchedCSSRules(Element* element, bool authorOnly);
-        ScriptObject getAttributeStyles(Element* element);
-        ScriptObject buildObjectForNode(Node* node, int depth, NodeToIdMap* nodesMap);
-        ScriptArray buildArrayForElementAttributes(Element* element);
-        ScriptArray buildArrayForContainerChildren(Node* container, int depth, NodeToIdMap* nodesMap);
-        ScriptObject buildObjectForEventListener(const RegisteredEventListener& registeredEventListener, const AtomicString& eventType, Node* node);
+        bool hasBreakpoint(Node* node, long type);
+        void updateSubtreeBreakpoints(Node* root, uint32_t rootMask, bool value);
+        void descriptionForDOMEvent(Node* target, long breakpointType, bool insertion, PassRefPtr<InspectorObject> description);
+
+        PassRefPtr<InspectorObject> buildObjectForNode(Node* node, int depth, NodeToIdMap* nodesMap);
+        PassRefPtr<InspectorArray> buildArrayForElementAttributes(Element* element);
+        PassRefPtr<InspectorArray> buildArrayForContainerChildren(Node* container, int depth, NodeToIdMap* nodesMap);
+        PassRefPtr<InspectorObject> buildObjectForEventListener(const RegisteredEventListener& registeredEventListener, const AtomicString& eventType, Node* node);
 
         // We represent embedded doms as a part of the same hierarchy. Hence we treat children of frame owners differently.
         // We also skip whitespace text nodes conditionally. Following methods encapsulate these specifics.
@@ -152,22 +178,17 @@ namespace WebCore {
         bool isWhitespace(Node* node);
 
         Document* mainFrameDocument() const;
-        String documentURLString(Document* document) const;
 
-        long bindStyle(CSSStyleDeclaration*);
-        long bindRule(CSSStyleRule*);
-        ScriptObject buildObjectForStyle(CSSStyleDeclaration*, bool bind);
-        void populateObjectWithStyleProperties(CSSStyleDeclaration*, ScriptObject& result);
-        ScriptObject buildObjectForRule(CSSStyleRule*);
-        Vector<String> uniqueStyleProperties(CSSStyleDeclaration*);
-        Vector<String> longhandProperties(CSSStyleDeclaration*, const String& shorthandProperty);
-        String shorthandPriority(CSSStyleDeclaration*, const String& shorthandProperty);
-        bool ruleAffectsNode(CSSStyleRule*, Node*);
-        ScriptArray toArray(const Vector<String>& data);
+        void onMatchJobsTimer(Timer<InspectorDOMAgent>*);
+        void reportNodesAsSearchResults(ListHashSet<Node*>& resultCollector);
+
+        Node* nodeForPath(const String& path);
+        PassRefPtr<InspectorArray> toArray(const Vector<String>& data);
 
         void discardBindings();
 
         InspectorFrontend* m_frontend;
+        DOMListener* m_domListener;
         NodeToIdMap m_documentNodeToIdMap;
         // Owns node mappings for dangling nodes.
         Vector<NodeToIdMap*> m_danglingNodeToIdMaps;
@@ -175,23 +196,15 @@ namespace WebCore {
         HashMap<long, NodeToIdMap*> m_idToNodesMap;
         HashSet<long> m_childrenRequested;
         long m_lastNodeId;
-
-        typedef HashMap<CSSStyleDeclaration*, long> StyleToIdMap;
-        typedef HashMap<long, RefPtr<CSSStyleDeclaration> > IdToStyleMap;
-        StyleToIdMap m_styleToId;
-        IdToStyleMap m_idToStyle;
-        typedef HashMap<CSSStyleRule*, long> RuleToIdMap;
-        typedef HashMap<long, RefPtr<CSSStyleRule> > IdToRuleMap;
-        RuleToIdMap m_ruleToId;
-        IdToRuleMap m_idToRule;
-        IdToStyleMap m_idToDisabledStyle;
-        RefPtr<CSSStyleSheet> m_inspectorStyleSheet;
-
-        long m_lastStyleId;
-        long m_lastRuleId;
         ListHashSet<RefPtr<Document> > m_documents;
+        Deque<MatchJob*> m_pendingMatchJobs;
+        Timer<InspectorDOMAgent> m_matchJobsTimer;
+        HashSet<RefPtr<Node> > m_searchResults;
+        Vector<long> m_inspectedNodes;
+        HashMap<Node*, uint32_t> m_breakpoints;
     };
 
+#endif
 
 } // namespace WebCore
 
