@@ -42,17 +42,17 @@ WebInspector.AuditRules.CacheableResponseCodes =
     304: true // Underlying resource is cacheable
 }
 
-WebInspector.AuditRules.getDomainToResourcesMap = function(resources, types, regexp, needFullResources)
+WebInspector.AuditRules.getDomainToResourcesMap = function(resources, types, needFullResources)
 {
     var domainToResourcesMap = {};
     for (var i = 0, size = resources.length; i < size; ++i) {
         var resource = resources[i];
         if (types && types.indexOf(resource.type) === -1)
             continue;
-        var match = resource.url.match(regexp);
-        if (!match)
+        var parsedURL = resource.url.asParsedURL();
+        if (!parsedURL)
             continue;
-        var domain = match[2];
+        var domain = parsedURL.host;
         var domainResources = domainToResourcesMap[domain];
         if (domainResources === undefined) {
           domainResources = [];
@@ -63,9 +63,9 @@ WebInspector.AuditRules.getDomainToResourcesMap = function(resources, types, reg
     return domainToResourcesMap;
 }
 
-WebInspector.AuditRules.evaluateInTargetWindow = function(func, callback)
+WebInspector.AuditRules.evaluateInTargetWindow = function(func, args, callback)
 {
-    InjectedScriptAccess.getDefault().evaluateOnSelf(func.toString(), callback);
+    InjectedScriptAccess.getDefault().evaluateOnSelf(func.toString(), args, callback);
 }
 
 
@@ -92,13 +92,13 @@ WebInspector.AuditRules.GzipRule.prototype = {
                 }
                 var savings = 2 * size / 3;
                 totalSavings += savings;
-                summary.addChild(String.sprintf("%s could save ~%s", WebInspector.linkifyURL(resource.url), Number.bytesToString(savings)));
+                summary.addChild(String.sprintf("%s could save ~%s", WebInspector.AuditRuleResult.linkifyDisplayName(resource.url), Number.bytesToString(savings, WebInspector.UIString)));
                 result.violationCount++;
             }
         }
         if (!totalSavings)
             return callback(null);
-        summary.value = String.sprintf("Compressing the following resources with gzip could reduce their transfer size by about two thirds (~%s):", Number.bytesToString(totalSavings));
+        summary.value = String.sprintf("Compressing the following resources with gzip could reduce their transfer size by about two thirds (~%s):", Number.bytesToString(totalSavings, WebInspector.UIString));
         callback(result);
     },
 
@@ -128,7 +128,7 @@ WebInspector.AuditRules.CombineExternalResourcesRule = function(id, name, type,
 WebInspector.AuditRules.CombineExternalResourcesRule.prototype = {
     doRun: function(resources, result, callback)
     {
-        var domainToResourcesMap = WebInspector.AuditRules.getDomainToResourcesMap(resources, [this._type], WebInspector.URLRegExp);
+        var domainToResourcesMap = WebInspector.AuditRules.getDomainToResourcesMap(resources, [this._type]);
         var penalizedResourceCount = 0;
         // TODO: refactor according to the chosen i18n approach
         var summary = result.addChild("", true);
@@ -138,7 +138,7 @@ WebInspector.AuditRules.CombineExternalResourcesRule.prototype = {
             if (extraResourceCount <= 0)
                 continue;
             penalizedResourceCount += extraResourceCount - 1;
-            summary.addChild(String.sprintf("%d %s resources served from %s.", domainResources.length, this._resourceTypeName, domain));
+            summary.addChild(String.sprintf("%d %s resources served from %s.", domainResources.length, this._resourceTypeName, WebInspector.AuditRuleResult.resourceDomain(domain)));
             result.violationCount += domainResources.length;
         }
         if (!penalizedResourceCount)
@@ -175,14 +175,14 @@ WebInspector.AuditRules.MinimizeDnsLookupsRule.prototype = {
     doRun: function(resources, result, callback)
     {
         var summary = result.addChild("");
-        var domainToResourcesMap = WebInspector.AuditRules.getDomainToResourcesMap(resources, undefined, WebInspector.URLRegExp);
+        var domainToResourcesMap = WebInspector.AuditRules.getDomainToResourcesMap(resources, undefined);
         for (var domain in domainToResourcesMap) {
             if (domainToResourcesMap[domain].length > 1)
                 continue;
-            var match = domain.match(WebInspector.URLRegExp);
-            if (!match)
+            var parsedURL = domain.asParsedURL();
+            if (!parsedURL)
                 continue;
-            if (!match[2].search(WebInspector.AuditRules.IPAddressRegexp))
+            if (!parsedURL.host.search(WebInspector.AuditRules.IPAddressRegexp))
                 continue; // an IP address
             summary.addSnippet(match[2]);
             result.violationCount++;
@@ -220,7 +220,6 @@ WebInspector.AuditRules.ParallelizeDownloadRule.prototype = {
         var domainToResourcesMap = WebInspector.AuditRules.getDomainToResourcesMap(
             resources,
             [WebInspector.Resource.Type.Stylesheet, WebInspector.Resource.Type.Image],
-            WebInspector.URLRegExp,
             true);
 
         var hosts = [];
@@ -278,76 +277,113 @@ WebInspector.AuditRules.UnusedCssRule.prototype = {
     doRun: function(resources, result, callback)
     {
         var self = this;
-        function evalCallback(routineResult, isException) {
-            if (isException || !routineResult || !routineResult.styleSheets.length)
+
+        function evalCallback(styleSheets) {
+            if (!styleSheets.length)
                 return callback(null);
 
-            var totalUnusedPercent = Math.round(100 * routineResult.unusedSize / routineResult.totalSize);
-            var summary = result.addChild(String.sprintf("%d%% of CSS (estimated) is not used by the current page.", totalUnusedPercent), true);
-
-            for (var i = 0; i < routineResult.styleSheets.length; ++i) {
-                var stylesheet = routineResult.styleSheets[i];
-
-                var url = stylesheet.type === "href" ? WebInspector.linkifyURL(stylesheet.location) : String.sprintf("Inline block #%s", stylesheet.location);
-                var pctUnused = Math.round(100 * stylesheet.unusedSize / stylesheet.totalSize);
-                var entry = summary.addChild(String.sprintf("%s: %d%% (estimated) is not used by the current page.", url, pctUnused));
-
-                for (var j = 0; j < stylesheet.unusedRules.length; ++j)
-                    entry.addSnippet(stylesheet.unusedRules[j]);
-
-                result.violationCount += stylesheet.unusedRules.length;
-            }
-
-            callback(result);
-        }
-
-        function routine()
-        {
-            var styleSheets = document.styleSheets;
-            if (!styleSheets)
-                return false;
-            var routineResult = { styleSheets: [] };
-            var inlineBlockOrdinal = 0;
-            var totalStylesheetSize = 0;
-            var totalUnusedStylesheetSize = 0;
-            var pseudoSelectorRegexp = /:hover|:link|:active|:visited|:focus/;
+            var pseudoSelectorRegexp = /:hover|:link|:active|:visited|:focus|:before|:after/;
+            var selectors = [];
+            var testedSelectors = {};
             for (var i = 0; i < styleSheets.length; ++i) {
                 var styleSheet = styleSheets[i];
-                if (!styleSheet.cssRules)
-                    continue;
-                var stylesheetSize = 0;
-                var unusedStylesheetSize = 0;
-                var unusedRules = [];
-                for (var curRule = 0; curRule < styleSheet.cssRules.length; ++curRule) {
-                    var rule = styleSheet.cssRules[curRule];
-                    var textLength = rule.cssText ? rule.cssText.length : 0;
-                    stylesheetSize += textLength;
-                    if (rule.type !== 1 || rule.selectorText.match(pseudoSelectorRegexp))
+                for (var curRule = 0; curRule < styleSheet.rules.length; ++curRule) {
+                    var selectorText = styleSheet.rules[curRule].selectorText;
+                    if (selectorText.match(pseudoSelectorRegexp) || testedSelectors[selectorText])
                         continue;
-                    var nodes = document.querySelectorAll(rule.selectorText);
-                    if (nodes && nodes.length)
-                        continue;
-                    unusedStylesheetSize += textLength;
-                    unusedRules.push(rule.selectorText);
-                }
-                totalStylesheetSize += stylesheetSize;
-                totalUnusedStylesheetSize += unusedStylesheetSize;
-
-                if (unusedRules.length) {
-                    var entry = { type: styleSheet.href ? "href" : "inline",
-                                  totalSize: stylesheetSize,
-                                  unusedSize: unusedStylesheetSize,
-                                  location: styleSheet.href ? styleSheet.href : ++inlineBlockOrdinal,
-                                  unusedRules: unusedRules };
-                    routineResult.styleSheets.push(entry);
+                    selectors.push(selectorText);
+                    testedSelectors[selectorText] = 1;
                 }
             }
-            routineResult.totalSize = totalStylesheetSize;
-            routineResult.unusedSize = totalUnusedStylesheetSize;
-            return routineResult;
+
+            function selectorsCallback(callback, styleSheets, testedSelectors, foundSelectors)
+            {
+                var inlineBlockOrdinal = 0;
+                var totalStylesheetSize = 0;
+                var totalUnusedStylesheetSize = 0;
+                var summary;
+
+                for (var i = 0; i < styleSheets.length; ++i) {
+                    var styleSheet = styleSheets[i];
+                    var stylesheetSize = 0;
+                    var unusedStylesheetSize = 0;
+                    var unusedRules = [];
+                    for (var curRule = 0; curRule < styleSheet.rules.length; ++curRule) {
+                        var rule = styleSheet.rules[curRule];
+                        // Exact computation whenever source ranges are available.
+                        var textLength = (rule.selectorRange && rule.style.properties.endOffset) ? rule.style.properties.endOffset - rule.selectorRange.start + 1 : 0;
+                        if (!textLength && rule.style.cssText)
+                            textLength = rule.style.cssText.length + rule.selectorText.length;
+                        stylesheetSize += textLength;
+                        if (!testedSelectors[rule.selectorText] || foundSelectors[rule.selectorText])
+                            continue;
+                        unusedStylesheetSize += textLength;
+                        unusedRules.push(rule.selectorText);
+                    }
+                    totalStylesheetSize += stylesheetSize;
+                    totalUnusedStylesheetSize += unusedStylesheetSize;
+
+                    if (!unusedRules.length)
+                        continue;
+
+                    var resource = WebInspector.resourceManager.resourceForURL(styleSheet.sourceURL);
+                    var isInlineBlock = resource && resource.type == WebInspector.Resource.Type.Document;
+                    var url = !isInlineBlock ? WebInspector.AuditRuleResult.linkifyDisplayName(styleSheet.sourceURL) : String.sprintf("Inline block #%d", ++inlineBlockOrdinal);
+                    var pctUnused = Math.round(100 * unusedStylesheetSize / stylesheetSize);
+                    if (!summary)
+                        summary = result.addChild("", true);
+                    var entry = summary.addChild(String.sprintf("%s: %s (%d%%) is not used by the current page.", url, Number.bytesToString(unusedStylesheetSize), pctUnused));
+
+                    for (var j = 0; j < unusedRules.length; ++j)
+                        entry.addSnippet(unusedRules[j]);
+
+                    result.violationCount += unusedRules.length;
+                }
+
+                if (!totalUnusedStylesheetSize)
+                    return callback(null);
+
+                var totalUnusedPercent = Math.round(100 * totalUnusedStylesheetSize / totalStylesheetSize);
+                summary.value = String.sprintf("%s (%d%%) of CSS is not used by the current page.", Number.bytesToString(totalUnusedStylesheetSize), totalUnusedPercent);
+
+                callback(result);
+            }
+
+            function routine(selectorArray)
+            {
+                var result = {};
+                for (var i = 0; i < selectorArray.length; ++i) {
+                    try {
+                        if (document.querySelector(selectorArray[i]))
+                            result[selectorArray[i]] = true;
+                    } catch(e) {
+                        // Ignore and mark as unused.
+                    }
+                }
+                return result;
+            }
+
+            WebInspector.AuditRules.evaluateInTargetWindow(routine, [selectors], selectorsCallback.bind(null, callback, styleSheets, testedSelectors));
         }
 
-        WebInspector.AuditRules.evaluateInTargetWindow(routine, evalCallback);
+        function styleSheetCallback(styleSheets, continuation, styleSheet)
+        {
+            if (styleSheet)
+                styleSheets.push(styleSheet);
+            if (continuation)
+                continuation(styleSheets);
+        }
+
+        function allStylesCallback(styleSheetIds)
+        {
+            if (!styleSheetIds || !styleSheetIds.length)
+                return evalCallback([]);
+            var styleSheets = [];
+            for (var i = 0; i < styleSheetIds.length; ++i)
+                WebInspector.CSSStyleSheet.createForId(styleSheetIds[i], styleSheetCallback.bind(null, styleSheets, i == styleSheetIds.length - 1 ? evalCallback : null));
+        }
+
+        InspectorBackend.getAllStyles2(allStylesCallback);
     }
 }
 
@@ -603,73 +639,89 @@ WebInspector.AuditRules.ImageDimensionsRule = function()
 WebInspector.AuditRules.ImageDimensionsRule.prototype = {
     doRun: function(resources, result, callback)
     {
-        function evalCallback(evalResult, isException)
+        function doneCallback(context)
         {
-            if (isException || !evalResult || !evalResult.totalImages)
-                return callback(null);
-
-            var entry = result.addChild("A width and height should be specified for all images in order to speed up page display. The following image(s) are missing a width and/or height:", true);
-            var map = evalResult.map;
+            var map = context.urlToNoDimensionCount;
             for (var url in map) {
-                var value = WebInspector.linkifyURL(url);
+                var entry = entry || result.addChild("A width and height should be specified for all images in order to speed up page display. The following image(s) are missing a width and/or height:", true);
+                var value = WebInspector.AuditRuleResult.linkifyDisplayName(url);
                 if (map[url] > 1)
-                    value += " (" + map[url] + " uses)";
+                    value += String.sprintf(" (%d uses)", map[url]);
                 entry.addChild(value);
                 result.violationCount++;
             }
-            callback(result);
+            callback(entry ? result : null);
         }
 
-        function routine()
+        function imageStylesReady(imageId, context, styles)
         {
-            var images = document.getElementsByTagName("img");
-            const widthRegExp = /width[^:;]*:/gim;
-            const heightRegExp = /height[^:;]*:/gim;
+            --context.imagesLeft;
 
-            function hasDimension(element, cssText, rules, regexp, attributeName) {
-                if (element.attributes.getNamedItem(attributeName) != null || (cssText && cssText.match(regexp)))
-                    return true;
-
-                if (!rules)
-                    return false;
-                for (var i = 0; i < rules.length; ++i) {
-                    if (rules.item(i).style.cssText.match(regexp))
-                        return true;
-                }
-                return false;
-            }
-
-            function hasWidth(element, cssText, rules) {
-                return hasDimension(element, cssText, rules, widthRegExp, "width");
-            }
-
-            function hasHeight(element, cssText, rules) {
-                return hasDimension(element, cssText, rules, heightRegExp, "height");
-            }
-
-            var urlToNoDimensionCount = {};
-            var found = false;
-            for (var i = 0; i < images.length; ++i) {
-                var image = images[i];
-                if (!image.src)
-                    continue;
-                var position = document.defaultView.getComputedStyle(image).getPropertyValue("position");
-                if (position === "absolute")
-                    continue;
-                var cssText = (image.style && image.style.cssText) ? image.style.cssText : "";
-                var rules = document.defaultView.getMatchedCSSRules(image, "", true);
-                if (!hasWidth(image, cssText, rules) || !hasHeight(image, cssText, rules)) {
-                    found = true;
-                    if (urlToNoDimensionCount.hasOwnProperty(image.src))
-                        ++urlToNoDimensionCount[image.src];
-                    else
-                        urlToNoDimensionCount[image.src] = 1;
+            const node = WebInspector.domAgent.nodeForId(imageId);
+            var src = node.getAttribute("src");
+            if (!src.asParsedURL()) {
+                for (var frameOwnerCandidate = node; frameOwnerCandidate; frameOwnerCandidate = frameOwnerCandidate.parentNode) {
+                    if (frameOwnerCandidate.documentURL) {
+                        var completeSrc = WebInspector.completeURL(frameOwnerCandidate.documentURL, src);
+                        break;
+                    }
                 }
             }
-            return found ? {totalImages: images.length, map: urlToNoDimensionCount} : null;
+            if (completeSrc)
+                src = completeSrc;
+
+            const computedStyle = styles.computedStyle;
+            if (computedStyle.getPropertyValue("position") === "absolute") {
+                if (!context.imagesLeft)
+                    doneCallback(context);
+                return;
+            }
+
+            var widthFound = "width" in styles.styleAttributes;
+            var heightFound = "height" in styles.styleAttributes;
+
+            for (var i = styles.matchedCSSRules.length - 1; i >= 0 && !(widthFound && heightFound); --i) {
+                var style = styles.matchedCSSRules[i].style;
+                if (style.getPropertyValue("width") !== "")
+                    widthFound = true;
+                if (style.getPropertyValue("height") !== "")
+                    heightFound = true;
+            }
+            
+            if (!widthFound || !heightFound) {
+                if (src in context.urlToNoDimensionCount)
+                    ++context.urlToNoDimensionCount[src];
+                else
+                    context.urlToNoDimensionCount[src] = 1;
+            }
+
+            if (!context.imagesLeft)
+                doneCallback(context);
         }
 
-        WebInspector.AuditRules.evaluateInTargetWindow(routine, evalCallback.bind(this));
+        function receivedImages(imageIds)
+        {
+            if (!imageIds || !imageIds.length)
+                return callback(null);
+            var context = {imagesLeft: imageIds.length, urlToNoDimensionCount: {}};
+            for (var i = imageIds.length - 1; i >= 0; --i)
+                WebInspector.cssModel.getStylesAsync(imageIds[i], imageStylesReady.bind(this, imageIds[i], context));
+        }
+
+        function pushImageNodes()
+        {
+            const nodeIds = [];
+            var nodes = document.getElementsByTagName("img");
+            for (var i = 0; i < nodes.length; ++i) {
+                if (!nodes[i].src)
+                    continue;
+                var nodeId = this.getNodeId(nodes[i]);
+                nodeIds.push(nodeId);
+            }
+            return nodeIds;
+        }
+
+        WebInspector.AuditRules.evaluateInTargetWindow(pushImageNodes, null, receivedImages);
     }
 }
 
@@ -684,9 +736,9 @@ WebInspector.AuditRules.CssInHeadRule = function()
 WebInspector.AuditRules.CssInHeadRule.prototype = {
     doRun: function(resources, result, callback)
     {
-        function evalCallback(evalResult, isException)
+        function evalCallback(evalResult)
         {
-            if (isException || !evalResult)
+            if (!evalResult)
                 return callback(null);
 
             var summary = result.addChild("");
@@ -694,11 +746,13 @@ WebInspector.AuditRules.CssInHeadRule.prototype = {
             var outputMessages = [];
             for (var url in evalResult) {
                 var urlViolations = evalResult[url];
-                if (urlViolations[0])
-                    result.addChild(String.sprintf("%s style block(s) in the %s body should be moved to the document head.", urlViolations[0], WebInspector.linkifyURL(url)));
+                if (urlViolations[0]) {
+                    result.addChild(String.sprintf("%s style block(s) in the %s body should be moved to the document head.", urlViolations[0], WebInspector.AuditRuleResult.linkifyDisplayName(url)));
+                    result.violationCount += urlViolations[0];
+                }
                 for (var i = 0; i < urlViolations[1].length; ++i)
-                    result.addChild(String.sprintf("Link node %s should be moved to the document head in %s", WebInspector.linkifyURL(urlViolations[1])), WebInspector.linkifyURL(url));
-                result.violationCount += urlViolations.length;
+                    result.addChild(String.sprintf("Link node %s should be moved to the document head in %s", WebInspector.AuditRuleResult.linkifyDisplayName(urlViolations[1][i]), WebInspector.AuditRuleResult.linkifyDisplayName(url)));
+                result.violationCount += urlViolations[1].length;
             }
             summary.value = String.sprintf("CSS in the document body adversely impacts rendering performance.");
             callback(result);
@@ -743,7 +797,7 @@ WebInspector.AuditRules.CssInHeadRule.prototype = {
             return found ? urlToViolationsArray : null;
         }
 
-        WebInspector.AuditRules.evaluateInTargetWindow(routine, evalCallback);
+        WebInspector.AuditRules.evaluateInTargetWindow(routine, null, evalCallback);
     }
 }
 
@@ -758,9 +812,9 @@ WebInspector.AuditRules.StylesScriptsOrderRule = function()
 WebInspector.AuditRules.StylesScriptsOrderRule.prototype = {
     doRun: function(resources, result, callback)
     {
-        function evalCallback(resultValue, isException)
+        function evalCallback(resultValue)
         {
-            if (isException || !resultValue)
+            if (!resultValue)
                 return callback(null);
 
             var lateCssUrls = resultValue[0];
@@ -790,7 +844,7 @@ WebInspector.AuditRules.StylesScriptsOrderRule.prototype = {
             return [ lateStyleUrls, cssBeforeInlineCount ];
         }
 
-        WebInspector.AuditRules.evaluateInTargetWindow(routine, evalCallback.bind(this));
+        WebInspector.AuditRules.evaluateInTargetWindow(routine, null, evalCallback.bind(this));
     }
 }
 
@@ -892,7 +946,6 @@ WebInspector.AuditRules.CookieSizeRule.prototype = {
 
         var domainToResourcesMap = WebInspector.AuditRules.getDomainToResourcesMap(resources,
                 null,
-                WebInspector.URLRegExp,
                 true);
         var matchingResourceData = {};
         this.mapResourceCookies(domainToResourcesMap, allCookies, collectorCallback.bind(this));
@@ -913,7 +966,7 @@ WebInspector.AuditRules.CookieSizeRule.prototype = {
         for (var i = 0, len = sortedCookieSizes.length; i < len; ++i) {
             var maxCookieSize = sortedCookieSizes[i].maxCookieSize;
             if (maxCookieSize > this._maxBytesThreshold)
-                hugeCookieDomains.push(sortedCookieSizes[i].domain + ": " + Number.bytesToString(maxCookieSize));
+                hugeCookieDomains.push(WebInspector.AuditRuleResult.resourceDomain(sortedCookieSizes[i].domain) + ": " + Number.bytesToString(maxCookieSize, WebInspector.UIString));
         }
 
         var bigAvgCookieDomains = [];
@@ -922,9 +975,9 @@ WebInspector.AuditRules.CookieSizeRule.prototype = {
             var domain = sortedCookieSizes[i].domain;
             var avgCookieSize = sortedCookieSizes[i].avgCookieSize;
             if (avgCookieSize > this._avgBytesThreshold && avgCookieSize < this._maxBytesThreshold)
-                bigAvgCookieDomains.push(domain + ": " + Number.bytesToString(avgCookieSize));
+                bigAvgCookieDomains.push(WebInspector.AuditRuleResult.resourceDomain(domain) + ": " + Number.bytesToString(avgCookieSize, WebInspector.UIString));
         }
-        result.addChild(String.sprintf("The average cookie size for all requests on this page is %s", Number.bytesToString(avgAllCookiesSize)));
+        result.addChild(String.sprintf("The average cookie size for all requests on this page is %s", Number.bytesToString(avgAllCookiesSize, WebInspector.UIString)));
 
         var message;
         if (hugeCookieDomains.length) {
@@ -935,7 +988,7 @@ WebInspector.AuditRules.CookieSizeRule.prototype = {
 
         if (bigAvgCookieDomains.length) {
             var entry = result.addChild(String.sprintf("The following domains have an average cookie size in excess of %d bytes. Reducing the size of cookies for these domains can reduce the time it takes to send requests.", this._avgBytesThreshold), true);
-            entry.addURLs(bigAvgCookieDomains.length);
+            entry.addURLs(bigAvgCookieDomains);
             result.violationCount += bigAvgCookieDomains.length;
         }
     }
@@ -956,7 +1009,6 @@ WebInspector.AuditRules.StaticCookielessRule.prototype = {
         var domainToResourcesMap = WebInspector.AuditRules.getDomainToResourcesMap(resources,
                 [WebInspector.Resource.Type.Stylesheet,
                  WebInspector.Resource.Type.Image],
-                WebInspector.URLRegExp,
                 true);
         var totalStaticResources = 0;
         for (var domain in domainToResourcesMap)
@@ -975,7 +1027,7 @@ WebInspector.AuditRules.StaticCookielessRule.prototype = {
         if (badUrls.length < this._minResources)
             return;
 
-        var entry = result.addChild(String.sprintf("%s of cookies were sent with the following static resources. Serve these static resources from a domain that does not set cookies:", Number.bytesToString(cookieBytes)), true);
+        var entry = result.addChild(String.sprintf("%s of cookies were sent with the following static resources. Serve these static resources from a domain that does not set cookies:", Number.bytesToString(cookieBytes, WebInspector.UIString)), true);
         entry.addURLs(badUrls);
         result.violationCount = badUrls.length;
     },
