@@ -90,11 +90,10 @@ WebInspector.ProfileType.prototype = {
 
 WebInspector.ProfilesPanel = function()
 {
-    WebInspector.Panel.call(this);
+    WebInspector.Panel.call(this, "profiles");
 
     this.createSidebar();
 
-    this.element.addStyleClass("profiles");
     this._profileTypesByIdMap = {};
     this._profileTypeButtonsByIdMap = {};
 
@@ -113,20 +112,22 @@ WebInspector.ProfilesPanel = function()
     this.enableToggleButton = new WebInspector.StatusBarButton("", "enable-toggle-status-bar-item");
     this.enableToggleButton.addEventListener("click", this._toggleProfiling.bind(this), false);
 
+    this.clearResultsButton = new WebInspector.StatusBarButton(WebInspector.UIString("Clear CPU profiles."), "clear-status-bar-item");
+    this.clearResultsButton.addEventListener("click", this._clearProfiles.bind(this), false);
+
     this.profileViewStatusBarItemsContainer = document.createElement("div");
-    this.profileViewStatusBarItemsContainer.id = "profile-view-status-bar-items";
+    this.profileViewStatusBarItemsContainer.className = "status-bar-items";
 
     this.welcomeView = new WebInspector.WelcomeView("profiles", WebInspector.UIString("Welcome to the Profiles panel"));
     this.element.appendChild(this.welcomeView.element);
 
     this._profiles = [];
     this._profilerEnabled = Preferences.profilerAlwaysEnabled;
-    this.reset();
+    this._reset();
+    InspectorBackend.registerDomainDispatcher("Profiler", this);
 }
 
 WebInspector.ProfilesPanel.prototype = {
-    toolbarItemClass: "profiles",
-
     get toolbarItemLabel()
     {
         return WebInspector.UIString("Profiles");
@@ -151,23 +152,14 @@ WebInspector.ProfilesPanel.prototype = {
                 items.push(button.element);
             }
         }
-        items.push(this.profileViewStatusBarItemsContainer);
+        items.push(this.clearResultsButton.element, this.profileViewStatusBarItemsContainer);
         return items;
     },
 
     show: function()
     {
         WebInspector.Panel.prototype.show.call(this);
-        if (this._shouldPopulateProfiles)
-            this._populateProfiles();
-    },
-
-    populateInterface: function()
-    {
-        if (this.visible)
-            this._populateProfiles();
-        else
-            this._shouldPopulateProfiles = true;
+        this._populateProfiles();
     },
 
     profilerWasEnabled: function()
@@ -176,8 +168,10 @@ WebInspector.ProfilesPanel.prototype = {
             return;
 
         this._profilerEnabled = true;
-        this.reset();
-        this.populateInterface();
+
+        this._reset();
+        if (this.visible)
+            this._populateProfiles();
     },
 
     profilerWasDisabled: function()
@@ -186,10 +180,15 @@ WebInspector.ProfilesPanel.prototype = {
             return;
 
         this._profilerEnabled = false;
-        this.reset();
+        this._reset();
     },
 
-    reset: function()
+    resetProfiles: function()
+    {
+        this._reset();
+    },
+
+    _reset: function()
     {
         for (var i = 0; i < this._profiles.length; ++i)
             delete this._profiles[i]._profileView;
@@ -202,6 +201,7 @@ WebInspector.ProfilesPanel.prototype = {
         this._profilesIdMap = {};
         this._profileGroups = {};
         this._profileGroupsForLinks = {}
+        this._profilesWereRequested = false;
 
         this.sidebarTreeElement.removeStyleClass("some-expandable");
 
@@ -216,6 +216,12 @@ WebInspector.ProfilesPanel.prototype = {
         this.welcomeView.show();
     },
 
+    _clearProfiles: function()
+    {
+        InspectorBackend.clearProfiles();
+        this._reset();
+    },
+
     registerProfileType: function(profileType)
     {
         this._profileTypesByIdMap[profileType.id] = profileType;
@@ -239,7 +245,6 @@ WebInspector.ProfilesPanel.prototype = {
             container.appendChild(part1);
      
             var button = new WebInspector.StatusBarButton(profileType.buttonTooltip, profileType.buttonStyle, profileType.buttonCaption);
-            button.element.addEventListener("click", profileType.buttonClicked.bind(profileType), false);
             container.appendChild(button.element);
        
             var part2 = document.createElement("span");
@@ -312,14 +317,46 @@ WebInspector.ProfilesPanel.prototype = {
         profile._profilesTreeElement = profileTreeElement;
 
         sidebarParent.appendChild(profileTreeElement);
-        this.welcomeView.hide();
-        if (!this.visibleView)
-            this.showProfile(profile);
+        if (!profile.isTemporary) {
+            this.welcomeView.hide();
+            if (!this.visibleView)
+                this.showProfile(profile);
+            this.dispatchEventToListeners("profile added");
+        }
+    },
+
+    removeProfileHeader: function(profile)
+    {
+        var typeId = profile.typeId;
+        var profileType = this.getProfileType(typeId);
+        var sidebarParent = profileType.treeElement;
+
+        for (var i = 0; i < this._profiles.length; ++i) {
+            if (this._profiles[i].uid === profile.uid) {
+                profile = this._profiles[i];
+                this._profiles.splice(i, 1);
+                break;
+            }
+        }
+        delete this._profilesIdMap[this._makeKey(profile.uid, typeId)];
+
+        var profileTitleKey = this._makeKey(profile.title, typeId);
+        delete this._profileGroups[profileTitleKey];
+
+        sidebarParent.removeChild(profile._profilesTreeElement);
+
+        if (!profile.isTemporary)
+            InspectorBackend.removeProfile(profile.typeId, profile.uid);
+
+        // No other item will be selected if there aren't any other profiles, so
+        // make sure that view gets cleared when the last profile is removed.
+        if (!this._profiles.length)
+            this.closeVisibleView();
     },
 
     showProfile: function(profile)
     {
-        if (!profile)
+        if (!profile || profile.isTemporary)
             return;
 
         this.closeVisibleView();
@@ -340,6 +377,85 @@ WebInspector.ProfilesPanel.prototype = {
             this.profileViewStatusBarItemsContainer.appendChild(statusBarItems[i]);
     },
 
+    getProfiles: function(typeId)
+    {
+        var result = [];
+        var profilesCount = this._profiles.length;
+        for (var i = 0; i < profilesCount; ++i)
+            if (this._profiles[i].typeId === typeId)
+                result.push(this._profiles[i]);
+        return result;
+    },
+
+    hasTemporaryProfile: function(typeId)
+    {
+        var profilesCount = this._profiles.length;
+        for (var i = 0; i < profilesCount; ++i)
+            if (this._profiles[i].typeId === typeId && this._profiles[i].isTemporary)
+                return true;
+        return false;
+    },
+
+    hasProfile: function(profile)
+    {
+        return !!this._profilesIdMap[this._makeKey(profile.uid, profile.typeId)];
+    },
+
+    updateProfile: function(profile)
+    {
+        var profilesCount = this._profiles.length;
+        for (var i = 0; i < profilesCount; ++i)
+            if (this._profiles[i].typeId === profile.typeId
+                && this._profiles[i].uid === profile.uid) {
+                this._profiles[i] = profile;
+                break;
+            }
+    },
+
+    loadHeapSnapshot: function(uid, callback)
+    {
+        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.HeapSnapshotProfileType.TypeId)];
+        if (!profile)
+            return;
+
+        if (profile._loaded)
+            callback(profile);
+        else if (profile._is_loading)
+            profile._callbacks.push(callback);
+        else {
+            profile._is_loading = true;
+            profile._callbacks = [callback];
+            profile._json = "";
+            InspectorBackend.getProfile(profile.typeId, profile.uid);
+        }
+    },
+
+    addHeapSnapshotChunk: function(uid, chunk)
+    {
+        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.HeapSnapshotProfileType.TypeId)];
+        if (!profile || profile._loaded || !profile._is_loading)
+            return;
+
+        profile._json += chunk;
+    },
+
+    finishHeapSnapshot: function(uid)
+    {
+        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.HeapSnapshotProfileType.TypeId)];
+        if (!profile || profile._loaded || !profile._is_loading)
+            return;
+
+        var callbacks = profile._callbacks;
+        delete profile._callbacks;
+        var loadedSnapshot = JSON.parse(profile._json);
+        delete profile._json;
+        delete profile._is_loading;
+        profile._loaded = true;
+        WebInspector.HeapSnapshotView.prototype.processLoadedSnapshot(profile, loadedSnapshot);
+        for (var i = 0; i < callbacks.length; ++i)
+            callbacks[i](profile);
+    },
+
     showView: function(view)
     {
         this.showProfile(view.profile);
@@ -386,12 +502,12 @@ WebInspector.ProfilesPanel.prototype = {
             if (!(titleKey in this._profileGroupsForLinks))
                 this._profileGroupsForLinks[titleKey] = 0;
 
-            groupNumber = ++this._profileGroupsForLinks[titleKey];
+            var groupNumber = ++this._profileGroupsForLinks[titleKey];
 
             if (groupNumber > 2)
                 // The title is used in the console message announcing that a profile has started so it gets
                 // incremented twice as often as it's displayed
-                title += " " + WebInspector.UIString("Run %d", groupNumber / 2);
+                title += " " + WebInspector.UIString("Run %d", (groupNumber + 1) / 2);
         }
         
         return title;
@@ -444,6 +560,7 @@ WebInspector.ProfilesPanel.prototype = {
             for (var typeId in this._profileTypeButtonsByIdMap)
                 this._profileTypeButtonsByIdMap[typeId].removeStyleClass("hidden");
             this.profileViewStatusBarItemsContainer.removeStyleClass("hidden");
+            this.clearResultsButton.element.removeStyleClass("hidden");
             this.panelEnablerView.visible = false;
         } else {
             this.enableToggleButton.title = WebInspector.UIString("Profiling disabled. Click to enable.");
@@ -451,6 +568,7 @@ WebInspector.ProfilesPanel.prototype = {
             for (var typeId in this._profileTypeButtonsByIdMap)
                 this._profileTypeButtonsByIdMap[typeId].addStyleClass("hidden");
             this.profileViewStatusBarItemsContainer.addStyleClass("hidden");
+            this.clearResultsButton.element.addStyleClass("hidden");
             this.panelEnablerView.visible = true;
         }
     },
@@ -472,45 +590,62 @@ WebInspector.ProfilesPanel.prototype = {
 
     _populateProfiles: function()
     {
-        var sidebarTreeChildrenCount = this.sidebarTree.children.length;
-        for (var i = 0; i < sidebarTreeChildrenCount; ++i) {
-            var treeElement = this.sidebarTree.children[i];
-            if (treeElement.children.length)
-                return;
-        }
+        if (!this._profilerEnabled || this._profilesWereRequested)
+            return;
 
         function populateCallback(profileHeaders) {
             profileHeaders.sort(function(a, b) { return a.uid - b.uid; });
             var profileHeadersLength = profileHeaders.length;
             for (var i = 0; i < profileHeadersLength; ++i)
-                WebInspector.addProfileHeader(profileHeaders[i]);
+                if (!this.hasProfile(profileHeaders[i]))
+                    WebInspector.panels.profiles.addProfileHeader(profileHeaders[i]);
         }
 
-        var callId = WebInspector.Callback.wrap(populateCallback);
-        InspectorBackend.getProfileHeaders(callId);
+        InspectorBackend.getProfileHeaders(populateCallback.bind(this));
 
-        delete this._shouldPopulateProfiles;
+        this._profilesWereRequested = true;
     },
 
     updateMainViewWidth: function(width)
     {
         this.welcomeView.element.style.left = width + "px";
         this.profileViews.style.left = width + "px";
-        this.profileViewStatusBarItemsContainer.style.left = width + "px";
+        this.profileViewStatusBarItemsContainer.style.left = Math.max(155, width) + "px";
         this.resize();
+    },
+
+    setRecordingProfile: function(isProfiling)
+    {
+        this.getProfileType(WebInspector.CPUProfileType.TypeId).setRecordingProfile(isProfiling);
+        if (this.hasTemporaryProfile(WebInspector.CPUProfileType.TypeId) !== isProfiling) {
+            if (!this._temporaryRecordingProfile) {
+                this._temporaryRecordingProfile = {
+                    typeId: WebInspector.CPUProfileType.TypeId,
+                    title: WebInspector.UIString("Recording"),
+                    uid: -1,
+                    isTemporary: true
+                };
+            }
+            if (isProfiling)
+                this.addProfileHeader(this._temporaryRecordingProfile);
+            else
+                this.removeProfileHeader(this._temporaryRecordingProfile);
+        }
+        this.updateProfileTypeButtons();
     }
 }
 
 WebInspector.ProfilesPanel.prototype.__proto__ = WebInspector.Panel.prototype;
 
-WebInspector.ProfileSidebarTreeElement = function(profile)
+WebInspector.ProfileSidebarTreeElement = function(profile, titleFormat, className)
 {
     this.profile = profile;
+    this._titleFormat = titleFormat;
 
     if (this.profile.title.indexOf(UserInitiatedProfileName) === 0)
         this._profileNumber = this.profile.title.substring(UserInitiatedProfileName.length + 1);
 
-    WebInspector.SidebarTreeElement.call(this, "profile-sidebar-tree-item", "", "", profile, false);
+    WebInspector.SidebarTreeElement.call(this, className, "", "", profile, false);
 
     this.refreshTitles();
 }
@@ -518,7 +653,13 @@ WebInspector.ProfileSidebarTreeElement = function(profile)
 WebInspector.ProfileSidebarTreeElement.prototype = {
     onselect: function()
     {
-        WebInspector.panels.profiles.showProfile(this.profile);
+        this.treeOutline.panel.showProfile(this.profile);
+    },
+
+    ondelete: function()
+    {
+        this.treeOutline.panel.removeProfileHeader(this.profile);
+        return true;
     },
 
     get mainTitle()
@@ -526,7 +667,7 @@ WebInspector.ProfileSidebarTreeElement.prototype = {
         if (this._mainTitle)
             return this._mainTitle;
         if (this.profile.title.indexOf(UserInitiatedProfileName) === 0)
-            return WebInspector.UIString("Profile %d", this._profileNumber);
+            return WebInspector.UIString(this._titleFormat, this._profileNumber);
         return this.profile.title;
     },
 
@@ -571,11 +712,10 @@ WebInspector.ProfileGroupSidebarTreeElement = function(title, subtitle)
 WebInspector.ProfileGroupSidebarTreeElement.prototype = {
     onselect: function()
     {
-        WebInspector.panels.profiles.showProfile(this.children[this.children.length - 1].profile);
+        if (this.children.length > 0)
+            WebInspector.panels.profiles.showProfile(this.children[this.children.length - 1].profile);
     }
 }
 
 WebInspector.ProfileGroupSidebarTreeElement.prototype.__proto__ = WebInspector.SidebarTreeElement.prototype;
 
-WebInspector.didGetProfileHeaders = WebInspector.Callback.processCallback;
-WebInspector.didGetProfile = WebInspector.Callback.processCallback;
