@@ -28,9 +28,13 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-WebInspector.SourceFrame = function(parentElement, addBreakpointDelegate, removeBreakpointDelegate)
+WebInspector.SourceFrame = function(parentElement, scripts, canEditScripts)
 {
     this._parentElement = parentElement;
+    this._scripts = {};
+    for (var i = 0; i < scripts.length; ++i)
+        this._scripts[scripts[i].sourceID] = scripts[i];
+    this._canEditScripts = canEditScripts;
 
     this._textModel = new WebInspector.TextEditorModel();
     this._textModel.replaceTabsWithSpaces = true;
@@ -38,13 +42,8 @@ WebInspector.SourceFrame = function(parentElement, addBreakpointDelegate, remove
     this._messages = [];
     this._rowMessages = {};
     this._messageBubbles = {};
-    this.breakpoints = [];
-    this._shortcuts = {};
 
     this._loaded = false;
-
-    this._addBreakpointDelegate = addBreakpointDelegate;
-    this._removeBreakpointDelegate = removeBreakpointDelegate;
     this._popoverObjectGroup = "popover";
 }
 
@@ -54,9 +53,20 @@ WebInspector.SourceFrame.prototype = {
     {
         this._visible = visible;
         this._createViewerIfNeeded();
-        if (!visible && this._textViewer)
-            this._textViewer.freeCachedElements();
-            
+
+        if (visible) {
+            if (this._textViewer && this._scrollTop)
+                this._textViewer.element.scrollTop = this._scrollTop;
+            if (this._textViewer && this._scrollLeft)
+                this._textViewer.element.scrollLeft = this._scrollLeft;
+        } else {
+            this._hidePopup();
+            if (this._textViewer) {
+                this._scrollTop = this._textViewer.element.scrollTop;
+                this._scrollLeft = this._textViewer.element.scrollLeft;
+                this._textViewer.freeCachedElements();
+            }
+        }
     },
 
     get executionLine()
@@ -76,6 +86,16 @@ WebInspector.SourceFrame.prototype = {
             this._updateExecutionLine(previousLine);
     },
 
+    markDiff: function(diffData)
+    {
+        if (this._diffLines && this._textViewer)
+            this._removeDiffDecorations();
+
+        this._diffLines = diffData;
+        if (this._textViewer)
+            this._updateDiffDecorations();
+    },
+
     revealLine: function(lineNumber)
     {
         if (this._textViewer)
@@ -84,26 +104,6 @@ WebInspector.SourceFrame.prototype = {
             this._lineNumberToReveal = lineNumber;
     },
 
-    addBreakpoint: function(breakpoint)
-    {
-        this.breakpoints.push(breakpoint);
-        breakpoint.addEventListener("enabled", this._breakpointChanged, this);
-        breakpoint.addEventListener("disabled", this._breakpointChanged, this);
-        breakpoint.addEventListener("condition-changed", this._breakpointChanged, this);
-        if (this._textViewer)
-            this._addBreakpointToSource(breakpoint);
-    },
-
-    removeBreakpoint: function(breakpoint)
-    {
-        this.breakpoints.remove(breakpoint);
-        breakpoint.removeEventListener("enabled", null, this);
-        breakpoint.removeEventListener("disabled", null, this);
-        breakpoint.removeEventListener("condition-changed", null, this);
-        if (this._textViewer)
-            this._removeBreakpointFromSource(breakpoint);
-    },
-
     addMessage: function(msg)
     {
         // Don't add the message if there is no message or valid line or if the msg isn't an error or warning.
@@ -114,6 +114,11 @@ WebInspector.SourceFrame.prototype = {
             this._addMessageToSource(msg);
     },
 
+    addScript: function(script)
+    {
+        this._scripts[script.sourceID] = script;
+    },
+
     clearMessages: function()
     {
         for (var line in this._messageBubbles) {
@@ -143,6 +148,27 @@ WebInspector.SourceFrame.prototype = {
         this._createViewerIfNeeded();
     },
 
+    updateContent: function(content)
+    {
+        this._textModel.setText(null, content);
+    },
+
+    get textModel()
+    {
+        return this._textModel;
+    },
+
+    get scrollTop()
+    {
+        return this._textViewer ? this._textViewer.element.scrollTop : 0;
+    },
+
+    set scrollTop(scrollTop)
+    {
+        if (this._textViewer)
+            this._textViewer.element.scrollTop = scrollTop;
+    },
+
     highlightLine: function(line)
     {
         if (this._textViewer)
@@ -151,6 +177,14 @@ WebInspector.SourceFrame.prototype = {
             this._lineToHighlight = line;
     },
 
+    clearLineHighlight: function()
+    {
+        if (this._textViewer)
+            this._textViewer.clearLineHighlight();
+        else
+            delete this._lineToHighlight;
+    },
+
     _createViewerIfNeeded: function()
     {
         if (!this._visible || !this._loaded || this._textViewer)
@@ -158,22 +192,18 @@ WebInspector.SourceFrame.prototype = {
 
         this._textViewer = new WebInspector.TextViewer(this._textModel, WebInspector.platform, this._url);
         var element = this._textViewer.element;
-        element.addEventListener("keydown", this._keyDown.bind(this), true);
         element.addEventListener("contextmenu", this._contextMenu.bind(this), true);
         element.addEventListener("mousedown", this._mouseDown.bind(this), true);
         element.addEventListener("mousemove", this._mouseMove.bind(this), true);
         element.addEventListener("scroll", this._scroll.bind(this), true);
         this._parentElement.appendChild(element);
 
-        this._needsProgramCounterImage = true;
-        this._needsBreakpointImages = true;
-
         this._textViewer.beginUpdates();
 
         this._textViewer.mimeType = this._mimeType;
         this._addExistingMessagesToSource();
-        this._addExistingBreakpointsToSource();
         this._updateExecutionLine();
+        this._updateDiffDecorations();
         this._textViewer.resize();
 
         if (this._lineNumberToReveal) {
@@ -191,7 +221,16 @@ WebInspector.SourceFrame.prototype = {
             this.highlightLine(this._lineToHighlight);
             delete this._lineToHighlight;
         }
+
+        var breakpoints = this._breakpoints();
+        for (var i = 0; i < breakpoints.length; ++i)
+            this._addBreakpoint(breakpoints[i]);
+        WebInspector.debuggerModel.addEventListener(WebInspector.DebuggerModel.Events.BreakpointAdded, this._breakpointAdded, this);
+
         this._textViewer.endUpdates();
+
+        if (this._canEditScripts)
+            this._textViewer.editCallback = this._editLine.bind(this);
     },
 
     findSearchMatches: function(query)
@@ -260,24 +299,6 @@ WebInspector.SourceFrame.prototype = {
         msg._resourceMessageRepeatCountElement.textContent = WebInspector.UIString(" (repeated %d times)", msg.repeatCount);
     },
 
-    _breakpointChanged: function(event)
-    {
-        var breakpoint = event.target;
-        var lineNumber = breakpoint.line - 1;
-        if (lineNumber >= this._textModel.linesCount)
-            return;
-
-        if (breakpoint.enabled)
-            this._textViewer.removeDecoration(lineNumber, "webkit-breakpoint-disabled");
-        else
-            this._textViewer.addDecoration(lineNumber, "webkit-breakpoint-disabled");
-
-        if (breakpoint.condition)
-            this._textViewer.addDecoration(lineNumber, "webkit-breakpoint-conditional");
-        else
-            this._textViewer.removeDecoration(lineNumber, "webkit-breakpoint-conditional");
-    },
-
     _updateExecutionLine: function(previousLine)
     {
         if (previousLine) {
@@ -288,12 +309,37 @@ WebInspector.SourceFrame.prototype = {
         if (!this._executionLine)
             return;
 
-        this._drawProgramCounterImageIfNeeded();
-
         if (this._executionLine < this._textModel.linesCount)
             this._textViewer.addDecoration(this._executionLine - 1, "webkit-execution-line");
     },
 
+    _updateDiffDecorations: function()
+    {
+        if (!this._diffLines)
+            return;
+
+        function addDecorations(textViewer, lines, className)
+        {
+            for (var i = 0; i < lines.length; ++i)
+                textViewer.addDecoration(lines[i], className);
+        }
+        addDecorations(this._textViewer, this._diffLines.added, "webkit-added-line");
+        addDecorations(this._textViewer, this._diffLines.removed, "webkit-removed-line");
+        addDecorations(this._textViewer, this._diffLines.changed, "webkit-changed-line");
+    },
+
+    _removeDiffDecorations: function()
+    {
+        function removeDecorations(textViewer, lines, className)
+        {
+            for (var i = 0; i < lines.length; ++i)
+                textViewer.removeDecoration(lines[i], className);
+        }
+        removeDecorations(this._textViewer, this._diffLines.added, "webkit-added-line");
+        removeDecorations(this._textViewer, this._diffLines.removed, "webkit-removed-line");
+        removeDecorations(this._textViewer, this._diffLines.changed, "webkit-changed-line");
+    },
+
     _addExistingMessagesToSource: function()
     {
         var length = this._messages.length;
@@ -321,7 +367,7 @@ WebInspector.SourceFrame.prototype = {
         }
 
         for (var i = 0; i < rowMessages.length; ++i) {
-            if (rowMessages[i].isEqual(msg, true)) {
+            if (rowMessages[i].isEqual(msg)) {
                 this._incrementMessageRepeatCount(rowMessages[i], msg.repeatDelta);
                 return;
             }
@@ -355,36 +401,64 @@ WebInspector.SourceFrame.prototype = {
         msg._resourceMessageLineElement = messageLineElement;
     },
 
-    _addExistingBreakpointsToSource: function()
+    _breakpointAdded: function(event)
     {
-        for (var i = 0; i < this.breakpoints.length; ++i)
-            this._addBreakpointToSource(this.breakpoints[i]);
+        var breakpoint = event.data;
+
+        if (breakpoint.sourceID in this._scripts)
+            this._addBreakpoint(breakpoint);
     },
 
-    _addBreakpointToSource: function(breakpoint)
+    _addBreakpoint: function(breakpoint)
     {
-        var lineNumber = breakpoint.line - 1;
-        if (lineNumber >= this._textModel.linesCount)
+        if (breakpoint.line > this._textModel.linesCount)
             return;
 
-        this._textModel.setAttribute(lineNumber, "breakpoint", breakpoint);
-        breakpoint.sourceText = this._textModel.line(breakpoint.line - 1);
-        this._drawBreakpointImagesIfNeeded();
+        breakpoint.addEventListener("enable-changed", this._breakpointChanged, this);
+        breakpoint.addEventListener("condition-changed", this._breakpointChanged, this);
+        breakpoint.addEventListener("removed", this._breakpointRemoved, this);
 
+        breakpoint.sourceText = this._textModel.line(breakpoint.line - 1);
+        this._setBreakpointDecoration(breakpoint.line, breakpoint.enabled, !!breakpoint.condition);
+    },
+
+    _breakpointRemoved: function(event)
+    {
+        var breakpoint = event.target;
+
+        breakpoint.removeEventListener("enable-changed", null, this);
+        breakpoint.removeEventListener("condition-changed", null, this);
+        breakpoint.removeEventListener("removed", null, this);
+
+        this._removeBreakpointDecoration(breakpoint.line);
+    },
+
+    _breakpointChanged: function(event)
+    {
+        var breakpoint = event.target;
+        this._setBreakpointDecoration(breakpoint.line, breakpoint.enabled, !!breakpoint.condition);
+    },
+
+    _setBreakpointDecoration: function(lineNumber, enabled, hasCondition)
+    {
+        lineNumber -= 1;
         this._textViewer.beginUpdates();
         this._textViewer.addDecoration(lineNumber, "webkit-breakpoint");
-        if (!breakpoint.enabled)
+        if (enabled)
+            this._textViewer.removeDecoration(lineNumber, "webkit-breakpoint-disabled");
+        else
             this._textViewer.addDecoration(lineNumber, "webkit-breakpoint-disabled");
-        if (breakpoint.condition)
+        if (hasCondition)
             this._textViewer.addDecoration(lineNumber, "webkit-breakpoint-conditional");
+        else
+            this._textViewer.removeDecoration(lineNumber, "webkit-breakpoint-conditional");
         this._textViewer.endUpdates();
     },
 
-    _removeBreakpointFromSource: function(breakpoint)
+    _removeBreakpointDecoration: function(lineNumber)
     {
-        var lineNumber = breakpoint.line - 1;
+        lineNumber -= 1;
         this._textViewer.beginUpdates();
-        this._textModel.removeAttribute(lineNumber, "breakpoint");
         this._textViewer.removeDecoration(lineNumber, "webkit-breakpoint");
         this._textViewer.removeDecoration(lineNumber, "webkit-breakpoint-disabled");
         this._textViewer.removeDecoration(lineNumber, "webkit-breakpoint-conditional");
@@ -393,31 +467,48 @@ WebInspector.SourceFrame.prototype = {
 
     _contextMenu: function(event)
     {
-        if (event.target.className !== "webkit-line-number")
+        if (!WebInspector.panels.scripts)
             return;
-        var row = event.target.parentElement;
 
-        var lineNumber = row.lineNumber;
+        var target = event.target.enclosingNodeOrSelfWithClass("webkit-line-number");
+        if (!target)
+            return;
+        var lineNumber = target.parentElement.lineNumber + 1;
+
         var contextMenu = new WebInspector.ContextMenu();
 
-        var breakpoint = this._textModel.getAttribute(lineNumber, "breakpoint");
+        contextMenu.appendItem(WebInspector.UIString("Continue to Here"), this._continueToLine.bind(this, lineNumber));
+
+        var breakpoint = this._findBreakpoint(lineNumber);
         if (!breakpoint) {
             // This row doesn't have a breakpoint: We want to show Add Breakpoint and Add and Edit Breakpoint.
-            contextMenu.appendItem(WebInspector.UIString("Add Breakpoint"), this._addBreakpointDelegate.bind(this, lineNumber + 1));
+            contextMenu.appendItem(WebInspector.UIString("Add Breakpoint"), this._setBreakpoint.bind(this, lineNumber));
 
-            function addConditionalBreakpoint() 
+            function addConditionalBreakpoint()
             {
-                this._addBreakpointDelegate(lineNumber + 1);
-                var breakpoint = this._textModel.getAttribute(lineNumber, "breakpoint");
-                if (breakpoint)
-                    this._editBreakpointCondition(breakpoint);
+                this._setBreakpointDecoration(lineNumber, true, true);
+                function didEditBreakpointCondition(committed, condition)
+                {
+                    this._removeBreakpointDecoration(lineNumber);
+                    if (committed)
+                        this._setBreakpoint(lineNumber, condition);
+                }
+                this._editBreakpointCondition(lineNumber, "", didEditBreakpointCondition.bind(this));
             }
-
-            contextMenu.appendItem(WebInspector.UIString("Add Conditional Breakpoint..."), addConditionalBreakpoint.bind(this));
+            contextMenu.appendItem(WebInspector.UIString("Add Conditional Breakpoint…"), addConditionalBreakpoint.bind(this));
         } else {
             // This row has a breakpoint, we want to show edit and remove breakpoint, and either disable or enable.
-            contextMenu.appendItem(WebInspector.UIString("Remove Breakpoint"), WebInspector.panels.scripts.removeBreakpoint.bind(WebInspector.panels.scripts, breakpoint));
-            contextMenu.appendItem(WebInspector.UIString("Edit Breakpoint..."), this._editBreakpointCondition.bind(this, breakpoint));
+            contextMenu.appendItem(WebInspector.UIString("Remove Breakpoint"), breakpoint.remove.bind(breakpoint));
+            function editBreakpointCondition()
+            {
+                function didEditBreakpointCondition(committed, condition)
+                {
+                    if (committed)
+                        breakpoint.condition = condition;
+                }
+                this._editBreakpointCondition(lineNumber, breakpoint.condition, didEditBreakpointCondition);
+            }
+            contextMenu.appendItem(WebInspector.UIString("Edit Breakpoint…"), editBreakpointCondition.bind(this));
             if (breakpoint.enabled)
                 contextMenu.appendItem(WebInspector.UIString("Disable Breakpoint"), function() { breakpoint.enabled = false; });
             else
@@ -435,19 +526,21 @@ WebInspector.SourceFrame.prototype = {
     {
         this._resetHoverTimer();
         this._hidePopup();
-        if (event.button != 0 || event.altKey || event.ctrlKey || event.metaKey || event.shiftKey)
+        if (event.button != 0 || event.altKey || event.ctrlKey || event.metaKey)
             return;
-        if (event.target.className !== "webkit-line-number")
+        var target = event.target.enclosingNodeOrSelfWithClass("webkit-line-number");
+        if (!target)
             return;
-        var row = event.target.parentElement;
+        var lineNumber = target.parentElement.lineNumber + 1;
 
-        var lineNumber = row.lineNumber;
-
-        var breakpoint = this._textModel.getAttribute(lineNumber, "breakpoint");
-        if (breakpoint)
-            this._removeBreakpointDelegate(breakpoint);
-        else if (this._addBreakpointDelegate)
-            this._addBreakpointDelegate(lineNumber + 1);
+        var breakpoint = this._findBreakpoint(lineNumber);
+        if (breakpoint) {
+            if (event.shiftKey)
+                breakpoint.enabled = !breakpoint.enabled;
+            else
+                breakpoint.remove();
+        } else
+            this._setBreakpoint(lineNumber);
         event.preventDefault();
     },
 
@@ -466,7 +559,8 @@ WebInspector.SourceFrame.prototype = {
                 self._hidePopup();
                 delete self._hidePopupTimer;
             }
-            this._hidePopupTimer = setTimeout(doHide, 500);
+            if (!("_hidePopupTimer" in this))
+                this._hidePopupTimer = setTimeout(doHide, 500);
         }
 
         this._hoverElement = event.target;
@@ -482,7 +576,7 @@ WebInspector.SourceFrame.prototype = {
         } else if (!this._hoverElement.hasStyleClass("webkit-javascript-ident"))
             return;
 
-        const toolTipDelay = this._popup ? 600 : 1500;
+        const toolTipDelay = this._popup ? 600 : 1000;
         this._hoverTimer = setTimeout(this._mouseHover.bind(this, this._hoverElement), toolTipDelay);
     },
 
@@ -566,9 +660,11 @@ WebInspector.SourceFrame.prototype = {
             var popupContentElement = null;
             if (result.type !== "object" && result.type !== "node" && result.type !== "array") {
                 popupContentElement = document.createElement("span");
-                popupContentElement.className = "monospace";
+                popupContentElement.className = "monospace console-formatted-" + result.type;
                 popupContentElement.style.whiteSpace = "pre";
                 popupContentElement.textContent = result.description;
+                if (result.type === "string")
+                    popupContentElement.textContent = "\"" + popupContentElement.textContent + "\"";
                 this._popup = new WebInspector.Popover(popupContentElement);
                 this._popup.show(element);
             } else {
@@ -595,9 +691,9 @@ WebInspector.SourceFrame.prototype = {
             popupContentElement.addEventListener("mousemove", killHidePopupTimer.bind(this), true);
         }
 
-        function evaluateCallback(result, exception)
+        function evaluateCallback(result)
         {
-            if (exception)
+            if (result.isError())
                 return;
             if (!WebInspector.panels.scripts.paused)
                 return;
@@ -606,38 +702,29 @@ WebInspector.SourceFrame.prototype = {
         WebInspector.panels.scripts.evaluateInSelectedCallFrame(element.textContent, false, this._popoverObjectGroup, evaluateCallback.bind(this));
     },
 
-    _editBreakpointCondition: function(breakpoint)
+    _editBreakpointCondition: function(lineNumber, condition, callback)
     {
-        this._showBreakpointConditionPopup(breakpoint.line);
+        lineNumber -= 1;
+        this._conditionElement = this._createConditionElement(lineNumber);
+        this._textViewer.addDecoration(lineNumber, this._conditionElement);
 
-        function committed(element, newText)
+        function finishEditing(committed, element, newText)
         {
-            breakpoint.condition = newText;
-            dismissed.call(this);
-        }
-
-        function dismissed()
-        {
-            if (this._conditionElement)
-                this._textViewer.removeDecoration(breakpoint.line - 1, this._conditionElement);
+            this._textViewer.removeDecoration(lineNumber, this._conditionElement);
             delete this._conditionEditorElement;
             delete this._conditionElement;
+            callback(committed, newText);
         }
 
-        var dismissedHandler = dismissed.bind(this);
-        this._conditionEditorElement.addEventListener("blur", dismissedHandler, false);
-
-        WebInspector.startEditing(this._conditionEditorElement, committed.bind(this), dismissedHandler);
-        this._conditionEditorElement.value = breakpoint.condition;
+        WebInspector.startEditing(this._conditionEditorElement, {
+            context: null,
+            commitHandler: finishEditing.bind(this, true),
+            cancelHandler: finishEditing.bind(this, false)
+        });
+        this._conditionEditorElement.value = condition;
         this._conditionEditorElement.select();
     },
 
-    _showBreakpointConditionPopup: function(lineNumber)
-    {
-        this._conditionElement = this._createConditionElement(lineNumber);
-        this._textViewer.addDecoration(lineNumber - 1, this._conditionElement);
-    },
-
     _createConditionElement: function(lineNumber)
     {
         var conditionElement = document.createElement("div");
@@ -659,17 +746,6 @@ WebInspector.SourceFrame.prototype = {
         return conditionElement;
     },
 
-    _keyDown: function(event)
-    {
-        var shortcut = WebInspector.KeyboardShortcut.makeKeyFromEvent(event);
-        var handler = this._shortcuts[shortcut];
-        if (handler) {
-            handler(event);
-            event.preventDefault();
-        } else
-            WebInspector.documentKeyDown(event);
-    },
-
     _evalSelectionInCallFrame: function(event)
     {
         if (!WebInspector.panels.scripts || !WebInspector.panels.scripts.paused)
@@ -679,12 +755,12 @@ WebInspector.SourceFrame.prototype = {
         if (!selection.rangeCount)
             return;
 
-        var expression = selection.getRangeAt(0).toString().trimWhitespace();
-        WebInspector.panels.scripts.evaluateInSelectedCallFrame(expression, false, "console", function(result, exception) {
+        var expression = selection.getRangeAt(0).toString().trim();
+        WebInspector.panels.scripts.evaluateInSelectedCallFrame(expression, false, "console", function(result) {
             WebInspector.showConsole();
             var commandMessage = new WebInspector.ConsoleCommand(expression);
             WebInspector.console.addMessage(commandMessage);
-            WebInspector.console.addMessage(new WebInspector.ConsoleCommandResult(result, exception, commandMessage));
+            WebInspector.console.addMessage(new WebInspector.ConsoleCommandResult(result, commandMessage));
         });
     },
 
@@ -694,134 +770,94 @@ WebInspector.SourceFrame.prototype = {
             this._textViewer.resize();
     },
 
-    _drawProgramCounterInContext: function(ctx, glow)
+    _continueToLine: function(lineNumber)
     {
-        if (glow)
-            ctx.save();
-
-        ctx.beginPath();
-        ctx.moveTo(17, 2);
-        ctx.lineTo(19, 2);
-        ctx.lineTo(19, 0);
-        ctx.lineTo(21, 0);
-        ctx.lineTo(26, 5.5);
-        ctx.lineTo(21, 11);
-        ctx.lineTo(19, 11);
-        ctx.lineTo(19, 9);
-        ctx.lineTo(17, 9);
-        ctx.closePath();
-        ctx.fillStyle = "rgb(142, 5, 4)";
-
-        if (glow) {
-            ctx.shadowBlur = 4;
-            ctx.shadowColor = "rgb(255, 255, 255)";
-            ctx.shadowOffsetX = -1;
-            ctx.shadowOffsetY = 0;
-        }
-
-        ctx.fill();
-        ctx.fill(); // Fill twice to get a good shadow and darker anti-aliased pixels.
-
-        if (glow)
-            ctx.restore();
+        var sourceID = this._sourceIDForLine(lineNumber);
+        if (!sourceID)
+            return;
+        WebInspector.panels.scripts.continueToLine(sourceID, lineNumber);
     },
 
-    _drawProgramCounterImageIfNeeded: function()
+    _editLine: function(lineNumber, newContent, cancelEditingCallback)
     {
-        if (!this._needsProgramCounterImage)
-            return;
+        lineNumber += 1;
 
-        var ctx = document.getCSSCanvasContext("2d", "program-counter", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        this._drawProgramCounterInContext(ctx, true);
-
-        delete this._needsProgramCounterImage;
-    },
-
-    _drawBreakpointImagesIfNeeded: function(conditional)
-    {
-        if (!this._needsBreakpointImages)
-            return;
-
-        function drawBreakpoint(ctx, disabled, conditional)
-        {
-            ctx.beginPath();
-            ctx.moveTo(0, 2);
-            ctx.lineTo(2, 0);
-            ctx.lineTo(21, 0);
-            ctx.lineTo(26, 5.5);
-            ctx.lineTo(21, 11);
-            ctx.lineTo(2, 11);
-            ctx.lineTo(0, 9);
-            ctx.closePath();
-            ctx.fillStyle = conditional ? "rgb(217, 142, 1)" : "rgb(1, 142, 217)";
-            ctx.strokeStyle = conditional ? "rgb(205, 103, 0)" : "rgb(0, 103, 205)";
-            ctx.lineWidth = 3;
-            ctx.fill();
-            ctx.save();
-            ctx.clip();
-            ctx.stroke();
-            ctx.restore();
-
-            if (!disabled)
-                return;
-
-            ctx.save();
-            ctx.globalCompositeOperation = "destination-out";
-            ctx.fillStyle = "rgba(0, 0, 0, 0.5)";
-            ctx.fillRect(0, 0, 26, 11);
-            ctx.restore();
+        var lines = [];
+        for (var i = 0; i < this._textModel.linesCount; ++i) {
+            if (i === lineNumber - 1)
+                lines.push(newContent);
+            else
+                lines.push(this._textModel.line(i));
         }
 
+        var editData = {};
+        editData.sourceID = this._sourceIDForLine(lineNumber);
+        editData.content = lines.join("\n");
+        editData.line = lineNumber;
+        editData.linesCountToShift = newContent.split("\n").length - 1;
+        this._doEditLine(editData, cancelEditingCallback);
+    },
 
-        // Unconditional breakpoints.
+    _revertEditLine: function(editData, contentToRevertTo)
+    {
+        var newEditData = {};
+        newEditData.sourceID = editData.sourceID;
+        newEditData.content = contentToRevertTo;
+        newEditData.line = editData.line;
+        newEditData.linesCountToShift = -editData.linesCountToShift;
+        this._doEditLine(newEditData);
+    },
 
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx);
+    _doEditLine: function(editData, cancelEditingCallback)
+    {
+        var revertEditingCallback = this._revertEditLine.bind(this, editData);
+        var commitEditingCallback = this._commitEditLine.bind(this, editData, revertEditingCallback);
+        WebInspector.panels.scripts.editScriptSource(editData, commitEditingCallback, cancelEditingCallback);
+    },
 
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint-program-counter", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx);
-        ctx.clearRect(20, 0, 6, 11);
-        this._drawProgramCounterInContext(ctx, true);
+    _commitEditLine: function(editData, revertEditLineCallback, newContent)
+    {
+        var script = this._scripts[editData.sourceID];
+        script.source = newContent;
+        if (script.resource)
+            script.resource.setContent(newContent, revertEditLineCallback);
+    },
 
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint-disabled", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx, true);
+    _setBreakpoint: function(lineNumber, condition)
+    {
+        var sourceID = this._sourceIDForLine(lineNumber);
+        if (!sourceID)
+            return;
+        WebInspector.debuggerModel.setBreakpoint(sourceID, lineNumber, true, condition);
+        if (!WebInspector.panels.scripts.breakpointsActivated)
+            WebInspector.panels.scripts.toggleBreakpointsClicked();
+    },
 
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint-disabled-program-counter", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx, true);
-        ctx.clearRect(20, 0, 6, 11);
-        this._drawProgramCounterInContext(ctx, true);
+    _breakpoints: function()
+    {
+        var scripts = this._scripts;
+        return WebInspector.debuggerModel.queryBreakpoints(function(b) { return b.sourceID in scripts; });
+    },
 
+    _findBreakpoint: function(lineNumber)
+    {
+        var sourceID = this._sourceIDForLine(lineNumber);
+        return WebInspector.debuggerModel.findBreakpoint(sourceID, lineNumber);
+    },
 
-        // Conditional breakpoints.
-
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint-conditional", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx, false, true);
-
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint-conditional-program-counter", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx, false, true);
-        ctx.clearRect(20, 0, 6, 11);
-        this._drawProgramCounterInContext(ctx, true);
-
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint-disabled-conditional", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx, true, true);
-
-        var ctx = document.getCSSCanvasContext("2d", "breakpoint-disabled-conditional-program-counter", 26, 11);
-        ctx.clearRect(0, 0, 26, 11);
-        drawBreakpoint(ctx, true, true);
-        ctx.clearRect(20, 0, 6, 11);
-        this._drawProgramCounterInContext(ctx, true);
-
-        delete this._needsBreakpointImages;
+    _sourceIDForLine: function(lineNumber)
+    {
+        var sourceIDForLine = null;
+        var closestStartingLine = 0;
+        for (var sourceID in this._scripts) {
+            var script = this._scripts[sourceID];
+            if (script.startingLine <= lineNumber && script.startingLine >= closestStartingLine) {
+                closestStartingLine = script.startingLine;
+                sourceIDForLine = sourceID;
+            }
+        }
+        return sourceIDForLine;
     }
 }
 
-
 WebInspector.SourceFrame.prototype.__proto__ = WebInspector.Object.prototype;
