@@ -32,23 +32,36 @@ WebInspector.SourceView = function(resource)
 
     this.element.addStyleClass("source");
 
-    this.sourceFrame = new WebInspector.SourceFrame(this.contentElement, this._addBreakpoint.bind(this));
+    var scripts = WebInspector.debuggerModel.scriptsForURL(resource.url);
+    var canEditScripts = WebInspector.panels.scripts.canEditScripts() && resource.type === WebInspector.Resource.Type.Script;
+    this.sourceFrame = new WebInspector.SourceFrame(this.element, scripts, canEditScripts);
     resource.addEventListener("finished", this._resourceLoadingFinished, this);
     this._frameNeedsSetup = true;
 }
 
+// This is a map from resource.type to mime types
+// found in WebInspector.SourceTokenizer.Registry.
+WebInspector.SourceView.DefaultMIMETypeForResourceType = {
+    0: "text/html",
+    1: "text/css",
+    4: "text/javascript"
+}
+
 WebInspector.SourceView.prototype = {
     show: function(parentElement)
     {
         WebInspector.ResourceView.prototype.show.call(this, parentElement);
+        this.setupSourceFrameIfNeeded();
         this.sourceFrame.visible = true;
         this.resize();
     },
 
     hide: function()
     {
-        WebInspector.View.prototype.hide.call(this);
         this.sourceFrame.visible = false;
+        if (!this._frameNeedsSetup)
+            this.sourceFrame.clearLineHighlight();
+        WebInspector.View.prototype.hide.call(this);
         this._currentSearchResultIndex = -1;
     },
 
@@ -58,38 +71,43 @@ WebInspector.SourceView.prototype = {
             this.sourceFrame.resize();
     },
 
-    detach: function()
+    get scrollTop()
     {
-        WebInspector.ResourceView.prototype.detach.call(this);
-
-        // FIXME: We need to mark the frame for setup on detach because the frame DOM is cleared
-        // when it is removed from the document. Is this a bug?
-        this._frameNeedsSetup = true;
-        this._sourceFrameSetup = false;
+        return this.sourceFrame.scrollTop;
     },
 
+    set scrollTop(scrollTop)
+    {
+        this.sourceFrame.scrollTop = scrollTop;
+    },
+
+
     setupSourceFrameIfNeeded: function()
     {
         if (!this._frameNeedsSetup)
             return;
 
-        this.attach();
-
         delete this._frameNeedsSetup;
-        WebInspector.getResourceContent(this.resource.identifier, this._contentLoaded.bind(this));
+        this.resource.requestContent(this._contentLoaded.bind(this));
     },
 
-    contentTabSelected: function()
+    hasContent: function()
     {
-        this.setupSourceFrameIfNeeded();
+        return true;
     },
 
     _contentLoaded: function(content)
     {
-        this.sourceFrame.setContent(this.resource.mimeType, content, this.resource.url);
+        var mimeType = this._canonicalMimeType(this.resource);
+        this.sourceFrame.setContent(mimeType, content, this.resource.url);
         this._sourceFrameSetupFinished();
     },
 
+    _canonicalMimeType: function(resource)
+    {
+        return WebInspector.SourceView.DefaultMIMETypeForResourceType[resource.type] || resource.mimeType;
+    },
+
     _resourceLoadingFinished: function(event)
     {
         this._frameNeedsSetup = true;
@@ -99,25 +117,6 @@ WebInspector.SourceView.prototype = {
         this.resource.removeEventListener("finished", this._resourceLoadingFinished, this);
     },
 
-    _addBreakpoint: function(line)
-    {
-        var sourceID = null;
-        var closestStartingLine = 0;
-        var scripts = this.resource.scripts;
-        for (var i = 0; i < scripts.length; ++i) {
-            var script = scripts[i];
-            if (script.startingLine <= line && script.startingLine >= closestStartingLine) {
-                closestStartingLine = script.startingLine;
-                sourceID = script.sourceID;
-            }
-        }
-
-        if (WebInspector.panels.scripts) {
-            var breakpoint = new WebInspector.Breakpoint(this.resource.url, line, sourceID);
-            WebInspector.panels.scripts.addBreakpoint(breakpoint);
-        }
-    },
-
     // The rest of the methods in this prototype need to be generic enough to work with a ScriptView.
     // The ScriptView prototype pulls these methods into it's prototype to avoid duplicate code.
 
@@ -125,7 +124,7 @@ WebInspector.SourceView.prototype = {
     {
         this._currentSearchResultIndex = -1;
         this._searchResults = [];
-        this.sourceFrame.clearSelection();
+        this.sourceFrame.clearMarkedRange();
         delete this._delayedFindSearchMatches;
     },
 
@@ -225,7 +224,7 @@ WebInspector.SourceView.prototype = {
         if (!foundRange)
             return;
 
-        this.sourceFrame.setSelection(foundRange);
+        this.sourceFrame.markAndRevealRange(foundRange);
     },
 
     _sourceFrameSetupFinished: function()
