@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2009 Google Inc. All rights reserved.
+ * Copyright (C) 2010 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,6 +40,10 @@ WebInspector.TextViewer = function(textModel, platform, url)
     this.element.tabIndex = 0;
 
     this.element.addEventListener("scroll", this._scroll.bind(this), false);
+    this.element.addEventListener("keydown", this._handleKeyDown.bind(this), false);
+    this.element.addEventListener("beforecopy", this._beforeCopy.bind(this), false);
+    this.element.addEventListener("copy", this._copy.bind(this), false);
+    this.element.addEventListener("dblclick", this._handleDoubleClick.bind(this), false);
 
     this._url = url;
 
@@ -75,6 +80,11 @@ WebInspector.TextViewer.prototype = {
         chunk.element.scrollIntoViewIfNeeded();
     },
 
+    set editCallback(editCallback)
+    {
+        this._editCallback = editCallback;
+    },
+
     addDecoration: function(lineNumber, decoration)
     {
         var chunk = this._makeLineAChunk(lineNumber);
@@ -107,16 +117,22 @@ WebInspector.TextViewer.prototype = {
 
     highlightLine: function(lineNumber)
     {
-        if (typeof this._highlightedLine === "number") {
-            var chunk = this._makeLineAChunk(this._highlightedLine);
-            chunk.removeDecoration("webkit-highlighted-line");
-        }
+        this.clearLineHighlight();
         this._highlightedLine = lineNumber;
         this.revealLine(lineNumber);
         var chunk = this._makeLineAChunk(lineNumber);
         chunk.addDecoration("webkit-highlighted-line");
     },
 
+    clearLineHighlight: function()
+    {
+        if (typeof this._highlightedLine === "number") {
+            var chunk = this._makeLineAChunk(this._highlightedLine);
+            chunk.removeDecoration("webkit-highlighted-line");
+            delete this._highlightedLine;
+        }
+    },
+
     freeCachedElements: function()
     {
         this._cachedSpans = [];
@@ -134,7 +150,9 @@ WebInspector.TextViewer.prototype = {
             this._textChunks.push(chunk);
             this._linesContainerElement.appendChild(chunk.element);
         }
+
         this._indexChunks();
+        this._highlighter.reset();
         this._repaintAll();
     },
 
@@ -203,6 +221,86 @@ WebInspector.TextViewer.prototype = {
         }.bind(this), 50);
     },
 
+    _handleKeyDown: function()
+    {
+        if (this._editingLine || event.metaKey || event.shiftKey || event.ctrlKey || event.altKey)
+            return;
+
+        var scrollValue = 0;
+        if (event.keyCode === WebInspector.KeyboardShortcut.Keys.Up.code)
+            scrollValue = -1;
+        else if (event.keyCode == WebInspector.KeyboardShortcut.Keys.Down.code)
+            scrollValue = 1;
+        
+        if (scrollValue) {
+            event.preventDefault();
+            event.stopPropagation();
+            this.element.scrollByLines(scrollValue);
+            return;
+        }
+        
+        scrollValue = 0;
+        if (event.keyCode === WebInspector.KeyboardShortcut.Keys.Left.code)
+            scrollValue = -40;
+        else if (event.keyCode == WebInspector.KeyboardShortcut.Keys.Right.code)
+            scrollValue = 40;
+        
+        if (scrollValue) {
+            event.preventDefault();
+            event.stopPropagation();
+            this.element.scrollLeft += scrollValue;
+        }
+    },
+
+    _handleDoubleClick: function(e)
+    {
+        if (!this._editCallback)
+            return;
+
+        var cell = e.target.enclosingNodeOrSelfWithNodeName("TD");
+        if (!cell)
+            return;
+
+        var lineRow = cell.parentElement;
+        if (lineRow.firstChild === cell)
+            return;  // Do not trigger editing from line numbers.
+
+        var oldContent = lineRow.lastChild.innerHTML;
+        var cancelEditingCallback = this._cancelEditingLine.bind(this, lineRow.lastChild, oldContent);
+        var commitEditingCallback = this._commitEditingLine.bind(this, lineRow.lineNumber, lineRow.lastChild, cancelEditingCallback);
+        this._editingLine = WebInspector.startEditing(lineRow.lastChild, {
+            context: null,
+            commitHandler: commitEditingCallback,
+            cancelHandler: cancelEditingCallback,
+            multiline: true
+        });
+    },
+
+    _commitEditingLine: function(lineNumber, element, cancelEditingCallback)
+    {
+        this._editCallback(lineNumber, element.textContent, cancelEditingCallback);
+        delete this._editingLine;
+    },
+
+    _cancelEditingLine: function(element, oldContent, e)
+    {
+        element.innerHTML = oldContent;
+        delete this._editingLine;
+    },
+
+    _beforeCopy: function(e)
+    {
+        e.preventDefault();
+    },
+
+    _copy: function(e)
+    {
+        var range = this._getSelection();
+        var text = this._textModel.copyRange(range);
+        InspectorFrontendHost.copyText(text);
+        e.preventDefault();
+    },
+
     beginUpdates: function(enabled)
     {
         this._paintCoalescingLevel++;
@@ -330,25 +428,25 @@ WebInspector.TextViewer.prototype = {
     _paintLine: function(lineRow, lineNumber)
     {
         var element = lineRow.lastChild;
-        var highlighterState = this._textModel.getAttribute(lineNumber, "highlighter-state");
-        var line = this._textModel.line(lineNumber);
-
-        if (!highlighterState) {
+        var highlight = this._textModel.getAttribute(lineNumber, "highlight");
+        if (!highlight) {
             if (this._rangeToMark && this._rangeToMark.startLine === lineNumber)
                 this._markedRangeElement = highlightSearchResult(element, this._rangeToMark.startColumn, this._rangeToMark.endColumn - this._rangeToMark.startColumn);
             return;
         }
 
         element.removeChildren();
+        var line = this._textModel.line(lineNumber);
 
         var plainTextStart = -1;
         for (var j = 0; j < line.length;) {
             if (j > 1000) {
                 // This line is too long - do not waste cycles on minified js highlighting.
-                plainTextStart = j;
+                if (plainTextStart === -1)
+                    plainTextStart = j;
                 break;
             }
-            var attribute = highlighterState && highlighterState.attributes[j];
+            var attribute = highlight[j];
             if (!attribute || !attribute.tokenType) {
                 if (plainTextStart === -1)
                     plainTextStart = j;
@@ -398,6 +496,9 @@ WebInspector.TextViewer.prototype = {
         if (selection.isCollapsed)
             return null;
         var selectionRange = selection.getRangeAt(0);
+        // Selection may be outside of the viewer.
+        if (!this.element.isAncestor(selectionRange.startContainer) || !this.element.isAncestor(selectionRange.endContainer))
+            return null;
         var start = this._selectionToPosition(selectionRange.startContainer, selectionRange.startOffset);
         var end = this._selectionToPosition(selectionRange.endContainer, selectionRange.endOffset);
         return new WebInspector.TextRange(start.line, start.column, end.line, end.column);
@@ -460,29 +561,25 @@ WebInspector.TextViewer.prototype = {
         if (container.nodeName === "TD" && offset === 1)
             return { line: lineNumber, column: this._textModel.lineLength(lineNumber) };
 
-        var column = 0;
-        if (lineRow.chunk) {
-            // This is chunk.
-            var text = lineRow.lastChild.textContent;
-            for (var i = 0; i < offset; ++i) {
-                if (text.charAt(i) === "\n") {
-                    lineNumber++;
-                    column = 0;
-                } else
-                    column++; 
-            }
-            return { line: lineNumber, column: column };
-        }
-
-        // This is individul line.
         var column = 0;
         var node = lineRow.lastChild.traverseNextTextNode(lineRow.lastChild);
         while (node && node !== container) {
             column += node.textContent.length;
             node = node.traverseNextTextNode(lineRow.lastChild);
         }
-        column += offset;
-        return { line: lineRow.lineNumber, column: column };
+
+        // This may be chunk and chunks may contain \n.
+        if (node === container && offset) {
+            var text = node.textContent;
+            for (var i = 0; i < offset; ++i) {
+                if (text.charAt(i) === "\n") {
+                    lineNumber++;
+                    column = 0;
+                } else
+                    column++;
+            }
+        }
+        return { line: lineNumber, column: column };
     },
 
     _appendSpan: function(element, content, className)
@@ -576,7 +673,18 @@ WebInspector.TextChunk = function(textViewer, startLine, endLine)
         lineNumbers.push(i + 1);
         lines.push(this._textModel.line(i));
     }
-    this._lineNumberElement.textContent = lineNumbers.join("\n");
+    if (this.linesCount === 1) {
+        // Single line chunks are typically created for decorations. Host line number in
+        // the sub-element in order to allow flexible border / margin management.
+        var innerSpan = document.createElement("span");
+        innerSpan.className = "webkit-line-number-inner";
+        innerSpan.textContent = startLine + 1;
+        var outerSpan = document.createElement("div");
+        outerSpan.className = "webkit-line-number-outer";
+        outerSpan.appendChild(innerSpan);
+        this._lineNumberElement.appendChild(outerSpan);
+    } else
+        this._lineNumberElement.textContent = lineNumbers.join("\n");
     this._lineContentElement.textContent = lines.join("\n");
 }
 
