@@ -29,19 +29,18 @@
 #include "config.h"
 #include "DocumentLoader.h"
 
-#if ENABLE(OFFLINE_WEB_APPLICATIONS)
-#include "ApplicationCache.h"
-#include "ApplicationCacheGroup.h"
-#include "ApplicationCacheResource.h"
-#endif
+#include "ApplicationCacheHost.h"
 #include "ArchiveFactory.h"
 #include "ArchiveResourceCollection.h"
 #include "CachedPage.h"
-#include "DocLoader.h"
+#include "CachedResourceLoader.h"
+#include "DOMWindow.h"
 #include "Document.h"
+#include "DocumentParser.h"
 #include "Event.h"
 #include "Frame.h"
 #include "FrameLoader.h"
+#include "FrameLoaderClient.h"
 #include "FrameTree.h"
 #include "HistoryItem.h"
 #include "Logging.h"
@@ -50,88 +49,29 @@
 #include "PlatformString.h"
 #include "Settings.h"
 #include "SharedBuffer.h"
-#include "StringBuffer.h"
-#include "XMLTokenizer.h"
 
 #include <wtf/Assertions.h>
+#include <wtf/text/CString.h>
 #include <wtf/unicode/Unicode.h>
 
 namespace WebCore {
 
-/*
- * Performs four operations:
- *  1. Convert backslashes to currency symbols
- *  2. Convert control characters to spaces
- *  3. Trim leading and trailing spaces
- *  4. Collapse internal whitespace.
- */
-static inline String canonicalizedTitle(const String& title, Frame* frame)
-{
-    ASSERT(!title.isEmpty());
-
-    const UChar* characters = title.characters();
-    unsigned length = title.length();
-    unsigned i;
-
-    StringBuffer buffer(length);
-    unsigned builderIndex = 0;
-
-    // Skip leading spaces and leading characters that would convert to spaces
-    for (i = 0; i < length; ++i) {
-        UChar c = characters[i];
-        if (!(c <= 0x20 || c == 0x7F))
-            break;
-    }
-
-    if (i == length)
-        return "";
-
-    // Replace control characters with spaces, and backslashes with currency symbols, and collapse whitespace.
-    bool previousCharWasWS = false;
-    for (; i < length; ++i) {
-        UChar c = characters[i];
-        if (c <= 0x20 || c == 0x7F || (WTF::Unicode::category(c) & (WTF::Unicode::Separator_Line | WTF::Unicode::Separator_Paragraph))) {
-            if (previousCharWasWS)
-                continue;
-            buffer[builderIndex++] = ' ';
-            previousCharWasWS = true;
-        } else if (c == '\\') {
-            buffer[builderIndex++] = frame->backslashAsCurrencySymbol();
-            previousCharWasWS = false;
-        } else {
-            buffer[builderIndex++] = c;
-            previousCharWasWS = false;
-        }
-    }
-
-    // Strip trailing spaces
-    while (builderIndex > 0) {
-        --builderIndex;
-        if (buffer[builderIndex] != ' ')
-            break;
-    }
-
-    if (!builderIndex && buffer[builderIndex] == ' ')
-        return "";
-
-    buffer.shrink(builderIndex + 1);
-    return String::adopt(buffer);
-}
-
 static void cancelAll(const ResourceLoaderSet& loaders)
 {
-    const ResourceLoaderSet copy = loaders;
-    ResourceLoaderSet::const_iterator end = copy.end();
-    for (ResourceLoaderSet::const_iterator it = copy.begin(); it != end; ++it)
-        (*it)->cancel();
+    Vector<RefPtr<ResourceLoader> > loadersCopy;
+    copyToVector(loaders, loadersCopy);
+    size_t size = loadersCopy.size();
+    for (size_t i = 0; i < size; ++i)
+        loadersCopy[i]->cancel();
 }
 
 static void setAllDefersLoading(const ResourceLoaderSet& loaders, bool defers)
 {
-    const ResourceLoaderSet copy = loaders;
-    ResourceLoaderSet::const_iterator end = copy.end();
-    for (ResourceLoaderSet::const_iterator it = copy.begin(); it != end; ++it)
-        (*it)->setDefersLoading(defers);
+    Vector<RefPtr<ResourceLoader> > loadersCopy;
+    copyToVector(loaders, loadersCopy);
+    size_t size = loadersCopy.size();
+    for (size_t i = 0; i < size; ++i)
+        loadersCopy[i]->setDefersLoading(defers);
 }
 
 DocumentLoader::DocumentLoader(const ResourceRequest& req, const SubstituteData& substituteData)
@@ -147,11 +87,12 @@ DocumentLoader::DocumentLoader(const ResourceRequest& req, const SubstituteData&
     , m_gotFirstByte(false)
     , m_primaryLoadComplete(false)
     , m_isClientRedirect(false)
-    , m_loadingFromCachedPage(false)
+    , m_wasOnloadHandled(false)
     , m_stopRecordingResponses(false)
     , m_substituteResourceDeliveryTimer(this, &DocumentLoader::substituteResourceDeliveryTimerFired)
+    , m_didCreateGlobalHistoryEntry(false)
 #if ENABLE(OFFLINE_WEB_APPLICATIONS)
-    , m_candidateApplicationCacheGroup(0)
+    , m_applicationCacheHost(adoptPtr(new ApplicationCacheHost(this)))
 #endif
 {
 }
@@ -166,13 +107,6 @@ FrameLoader* DocumentLoader::frameLoader() const
 DocumentLoader::~DocumentLoader()
 {
     ASSERT(!m_frame || frameLoader()->activeDocumentLoader() != this || !frameLoader()->isLoading());
-    
-#if ENABLE(OFFLINE_WEB_APPLICATIONS)
-    if (m_applicationCache)
-        m_applicationCache->group()->documentLoaderDestroyed(this);
-    else if (m_candidateApplicationCacheGroup)
-        m_candidateApplicationCacheGroup->documentLoaderDestroyed(this);
-#endif
 }
 
 PassRefPtr<SharedBuffer> DocumentLoader::mainResourceData() const
@@ -209,7 +143,7 @@ const KURL& DocumentLoader::url() const
     return request().url();
 }
 
-void DocumentLoader::replaceRequestURLForAnchorScroll(const KURL& url)
+void DocumentLoader::replaceRequestURLForSameDocumentNavigation(const KURL& url)
 {
     m_originalRequestCopy.setURL(url);
     m_request.setURL(url);
@@ -253,13 +187,10 @@ void DocumentLoader::clearErrors()
 
 void DocumentLoader::mainReceivedError(const ResourceError& error, bool isComplete)
 {
+    ASSERT(!error.isNull());
+
 #if ENABLE(OFFLINE_WEB_APPLICATIONS)
-    ApplicationCacheGroup* group = m_candidateApplicationCacheGroup;
-    if (!group && m_applicationCache && !mainResourceApplicationCache())
-        group = m_applicationCache->group();
-    
-    if (group)
-        group->failedLoadingMainResource(this);
+    m_applicationCacheHost->failedLoadingMainResource();
 #endif
     
     if (!frameLoader())
@@ -273,7 +204,7 @@ void DocumentLoader::mainReceivedError(const ResourceError& error, bool isComple
 // one document at a time, but one document may have many related resources. 
 // stopLoading will stop all loads initiated by the data source, 
 // but not loads initiated by child frames' data sources -- that's the WebFrame's job.
-void DocumentLoader::stopLoading()
+void DocumentLoader::stopLoading(DatabasePolicy databasePolicy)
 {
     // In some rare cases, calling FrameLoader::stopLoading could set m_loading to false.
     // (This can happen when there's a single XMLHttpRequest currently loading and stopLoading causes it
@@ -285,13 +216,18 @@ void DocumentLoader::stopLoading()
         // still  parsing. Failure to do so can cause a world leak.
         Document* doc = m_frame->document();
         
-        if (loading || (doc && doc->parsing()))
-            m_frame->loader()->stopLoading(false);
+        if (loading || doc->parsing())
+            m_frame->loader()->stopLoading(UnloadEventPolicyNone, databasePolicy);
     }
 
     // Always cancel multipart loaders
     cancelAll(m_multipartSubresourceLoaders);
 
+    // Appcache uses ResourceHandle directly, DocumentLoader doesn't count these loads.
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    m_applicationCacheHost->stopLoadingInFrame(m_frame);
+#endif
+
     if (!loading)
         return;
     
@@ -330,7 +266,7 @@ void DocumentLoader::commitIfReady()
 {
     if (m_gotFirstByte && !m_committed) {
         m_committed = true;
-        frameLoader()->commitProvisionalLoad(0);
+        frameLoader()->commitProvisionalLoad();
     }
 }
 
@@ -340,7 +276,7 @@ void DocumentLoader::finishedLoading()
     commitIfReady();
     if (FrameLoader* loader = frameLoader()) {
         loader->finishedLoadingDocument(this);
-        loader->end();
+        loader->writer()->end();
     }
 }
 
@@ -351,8 +287,27 @@ void DocumentLoader::commitLoad(const char* data, int length)
     RefPtr<DocumentLoader> protect(this);
 
     commitIfReady();
-    if (FrameLoader* frameLoader = DocumentLoader::frameLoader())
-        frameLoader->committedLoad(this, data, length);
+    FrameLoader* frameLoader = DocumentLoader::frameLoader();
+    if (!frameLoader)
+        return;
+    if (ArchiveFactory::isArchiveMimeType(response().mimeType()))
+        return;
+    frameLoader->client()->committedLoad(this, data, length);
+}
+
+void DocumentLoader::commitData(const char* bytes, int length)
+{
+    // Set the text encoding.  This is safe to call multiple times.
+    bool userChosen = true;
+    String encoding = overrideEncoding();
+    if (encoding.isNull()) {
+        userChosen = false;
+        encoding = response().textEncodingName();
+    }
+    // FIXME: DocumentWriter should be owned by DocumentLoader.
+    m_frame->loader()->writer()->setEncoding(encoding, userChosen);
+    ASSERT(m_frame->document()->parsing());
+    m_frame->loader()->writer()->addData(bytes, length);
 }
 
 bool DocumentLoader::doesProgressiveLoad(const String& MIMEType) const
@@ -382,7 +337,7 @@ void DocumentLoader::setupForReplaceByMIMEType(const String& newMIMEType)
     }
     
     frameLoader()->finishedLoadingDocument(this);
-    m_frame->loader()->end();
+    m_frame->loader()->writer()->end();
     
     frameLoader()->setReplacing();
     m_gotFirstByte = false;
@@ -399,8 +354,18 @@ void DocumentLoader::setupForReplaceByMIMEType(const String& newMIMEType)
 
 void DocumentLoader::updateLoading()
 {
+    if (!m_frame) {
+        setLoading(false);
+        return;
+    }
     ASSERT(this == frameLoader()->activeDocumentLoader());
+    bool wasLoading = m_loading;
     setLoading(frameLoader()->isLoading());
+
+    if (wasLoading && !m_loading) {
+        if (DOMWindow* window = m_frame->existingDOMWindow())
+            window->finishedLoading();
+    }
 }
 
 void DocumentLoader::setFrame(Frame* frame)
@@ -420,6 +385,9 @@ void DocumentLoader::attachToFrame()
 void DocumentLoader::detachFromFrame()
 {
     ASSERT(m_frame);
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    m_applicationCacheHost->setDOMApplicationCache(0);
+#endif
     m_frame = 0;
 }
 
@@ -441,12 +409,11 @@ void DocumentLoader::setPrimaryLoadComplete(bool flag)
     if (flag) {
         if (m_mainResourceLoader) {
             m_mainResourceData = m_mainResourceLoader->resourceData();
-#if ENABLE(OFFLINE_WEB_APPLICATIONS)
-            m_mainResourceApplicationCache = m_mainResourceLoader->applicationCache();
-#endif
             m_mainResourceLoader = 0;
         }
-        updateLoading();
+
+        if (this == frameLoader()->activeDocumentLoader())
+            updateLoading();
     }
 }
 
@@ -459,13 +426,12 @@ bool DocumentLoader::isLoadingInAPISense() const
             return true;
         if (!m_subresourceLoaders.isEmpty())
             return true;
-        if (Document* doc = m_frame->document()) {
-            if (doc->docLoader()->requestCount())
+        Document* doc = m_frame->document();
+        if (doc->cachedResourceLoader()->requestCount())
+            return true;
+        if (DocumentParser* parser = doc->parser())
+            if (parser->processingData())
                 return true;
-            if (Tokenizer* tok = doc->tokenizer())
-                if (tok->processingData())
-                    return true;
-        }
     }
     return frameLoader()->subframeIsLoading();
 }
@@ -473,7 +439,7 @@ bool DocumentLoader::isLoadingInAPISense() const
 void DocumentLoader::addAllArchiveResources(Archive* archive)
 {
     if (!m_archiveResourceCollection)
-        m_archiveResourceCollection.set(new ArchiveResourceCollection);
+        m_archiveResourceCollection = adoptPtr(new ArchiveResourceCollection);
         
     ASSERT(archive);
     if (!archive)
@@ -487,7 +453,7 @@ void DocumentLoader::addAllArchiveResources(Archive* archive)
 void DocumentLoader::addArchiveResource(PassRefPtr<ArchiveResource> resource)
 {
     if (!m_archiveResourceCollection)
-        m_archiveResourceCollection.set(new ArchiveResourceCollection);
+        m_archiveResourceCollection = adoptPtr(new ArchiveResourceCollection);
         
     ASSERT(resource);
     if (!resource)
@@ -530,7 +496,11 @@ SharedBuffer* DocumentLoader::parsedArchiveData() const
 PassRefPtr<ArchiveResource> DocumentLoader::mainResource() const
 {
     const ResourceResponse& r = response();
-    return ArchiveResource::create(mainResourceData(), r.url(), r.mimeType(), r.textEncodingName(), frame()->tree()->name());
+    RefPtr<SharedBuffer> mainResourceBuffer = mainResourceData();
+    if (!mainResourceBuffer)
+        mainResourceBuffer = SharedBuffer::create();
+        
+    return ArchiveResource::create(mainResourceBuffer, r.url(), r.mimeType(), r.textEncodingName(), frame()->tree()->uniqueName());
 }
 
 PassRefPtr<ArchiveResource> DocumentLoader::subresource(const KURL& url) const
@@ -538,15 +508,20 @@ PassRefPtr<ArchiveResource> DocumentLoader::subresource(const KURL& url) const
     if (!isCommitted())
         return 0;
     
-    Document* doc = m_frame->document();
-    if (!doc)
+    CachedResource* resource = m_frame->document()->cachedResourceLoader()->cachedResource(url);
+    if (!resource || !resource->isLoaded())
         return archiveResourceForURL(url);
-        
-    CachedResource* resource = doc->docLoader()->cachedResource(url);
-    if (!resource)
-        return archiveResourceForURL(url);
-        
-    return ArchiveResource::create(resource->data(), url, resource->response());
+
+    // FIXME: This has the side effect of making the resource non-purgeable.
+    // It would be better if it didn't have this permanent effect.
+    if (!resource->makePurgeable(false))
+        return 0;
+
+    RefPtr<SharedBuffer> data = resource->data();
+    if (!data)
+        return 0;
+
+    return ArchiveResource::create(data.release(), url, resource->response());
 }
 
 void DocumentLoader::getSubresources(Vector<PassRefPtr<ArchiveResource> >& subresources) const
@@ -555,13 +530,11 @@ void DocumentLoader::getSubresources(Vector<PassRefPtr<ArchiveResource> >& subre
         return;
 
     Document* document = m_frame->document();
-    if (!document)
-        return;
 
-    const HashMap<String, CachedResource*>& allResources = document->docLoader()->allCachedResources();
-    HashMap<String, CachedResource*>::const_iterator end = allResources.end();
-    for (HashMap<String, CachedResource*>::const_iterator it = allResources.begin(); it != end; ++it) {
-        RefPtr<ArchiveResource> subresource = this->subresource(KURL(it->second->url()));
+    const CachedResourceLoader::DocumentResourceMap& allResources = document->cachedResourceLoader()->allCachedResources();
+    CachedResourceLoader::DocumentResourceMap::const_iterator end = allResources.end();
+    for (CachedResourceLoader::DocumentResourceMap::const_iterator it = allResources.begin(); it != end; ++it) {
+        RefPtr<ArchiveResource> subresource = this->subresource(KURL(ParsedURLString, it->second->url()));
         if (subresource)
             subresources.append(subresource.release());
     }
@@ -601,7 +574,7 @@ void DocumentLoader::substituteResourceDeliveryTimerFired(Timer<DocumentLoader>*
         
             loader->didReceiveResponse(resource->response());
             loader->didReceiveData(data->data(), data->size(), data->size(), true);
-            loader->didFinishLoading();
+            loader->didFinishLoading(0);
         } else {
             // A null resource means that we should fail the load.
             // FIXME: Maybe we should use another error here - something like "not in cache".
@@ -663,14 +636,24 @@ void DocumentLoader::setTitle(const String& title)
     if (title.isEmpty())
         return;
 
-    String trimmed = canonicalizedTitle(title, m_frame);
-    if (!trimmed.isEmpty() && m_pageTitle != trimmed) {
+    if (m_pageTitle != title) {
         frameLoader()->willChangeTitle(this);
-        m_pageTitle = trimmed;
+        m_pageTitle = title;
         frameLoader()->didChangeTitle(this);
     }
 }
 
+void DocumentLoader::setIconURL(const String& iconURL)
+{
+    if (iconURL.isEmpty())
+        return;
+
+    if (m_pageIconURL != iconURL) {
+        m_pageIconURL = iconURL;
+        frameLoader()->didChangeIcons(this);
+    }
+}
+
 KURL DocumentLoader::urlForHistory() const
 {
     // Return the URL to be used for history and B/F list.
@@ -682,14 +665,9 @@ KURL DocumentLoader::urlForHistory() const
     return m_originalRequestCopy.url();
 }
 
-void DocumentLoader::loadFromCachedPage(PassRefPtr<CachedPage> cachedPage)
+bool DocumentLoader::urlForHistoryReflectsFailure() const
 {
-    LOG(PageCache, "WebCorePageCache: DocumentLoader %p loading from cached page %p", this, cachedPage.get());
-    
-    prepareForLoadStart();
-    setLoadingFromCachedPage(true);
-    setCommitted(true);
-    frameLoader()->commitProvisionalLoad(cachedPage);
+    return m_substituteData.isValid() || m_response.httpStatusCode() >= 400;
 }
 
 const KURL& DocumentLoader::originalURL() const
@@ -791,7 +769,7 @@ bool DocumentLoader::startLoadingMainResource(unsigned long identifier)
 
     // FIXME: Is there any way the extra fields could have not been added by now?
     // If not, it would be great to remove this line of code.
-    frameLoader()->addExtraFieldsToRequest(m_request, true, false);
+    frameLoader()->addExtraFieldsToMainResourceRequest(m_request);
 
     if (!m_mainResourceLoader->load(m_request, m_substituteData)) {
         // FIXME: If this should really be caught, we should just ASSERT this doesn't happen;
@@ -818,93 +796,33 @@ void DocumentLoader::subresourceLoaderFinishedLoadingOnePart(ResourceLoader* loa
         frame->loader()->checkLoadComplete();    
 }
 
+void DocumentLoader::transferLoadingResourcesFromPage(Page* oldPage)
+{
+    ASSERT(oldPage != m_frame->page());
+
+    FrameLoader* loader = frameLoader();
+    ASSERT(loader);
+
+    const ResourceRequest& request = originalRequest();
+    if (isLoadingMainResource()) {
+        loader->dispatchTransferLoadingResourceFromPage(
+            m_mainResourceLoader->identifier(), this, request, oldPage);
+    }
+
+    if (isLoadingSubresources()) {
+        ResourceLoaderSet::const_iterator it = m_subresourceLoaders.begin();
+        ResourceLoaderSet::const_iterator end = m_subresourceLoaders.end();
+        for (; it != end; ++it) {
+            loader->dispatchTransferLoadingResourceFromPage(
+                (*it)->identifier(), this, request, oldPage);
+        }
+    }
+}
+
 void DocumentLoader::iconLoadDecisionAvailable()
 {
     if (m_frame)
         m_frame->loader()->iconLoadDecisionAvailable();
 }
 
-#if ENABLE(OFFLINE_WEB_APPLICATIONS)
-void DocumentLoader::setCandidateApplicationCacheGroup(ApplicationCacheGroup* group)
-{
-    ASSERT(!m_applicationCache);
-    m_candidateApplicationCacheGroup = group;
-}
-    
-void DocumentLoader::setApplicationCache(PassRefPtr<ApplicationCache> applicationCache)
-{
-    if (m_candidateApplicationCacheGroup) {
-        ASSERT(!m_applicationCache);
-        m_candidateApplicationCacheGroup = 0;
-    }
-
-    m_applicationCache = applicationCache;
-}
-
-ApplicationCache* DocumentLoader::topLevelApplicationCache() const
-{
-    if (!m_frame)
-        return 0;
-    
-    if (m_applicationCache)
-        return m_applicationCache.get();
-    
-    if (Page* page = m_frame->page())
-        return page->mainFrame()->loader()->documentLoader()->applicationCache();
-    
-    return 0;
-}
-
-ApplicationCache* DocumentLoader::mainResourceApplicationCache() const
-{
-    if (m_mainResourceApplicationCache)
-        return m_mainResourceApplicationCache.get();
-    if (m_mainResourceLoader)
-        return m_mainResourceLoader->applicationCache();
-    return 0;
-}
-
-bool DocumentLoader::shouldLoadResourceFromApplicationCache(const ResourceRequest& request, ApplicationCacheResource*& resource)
-{
-    ApplicationCache* cache = topLevelApplicationCache();    
-    if (!cache)
-        return false;
-    
-    // If the resource is not a HTTP/HTTPS GET, then abort
-    if (!ApplicationCache::requestIsHTTPOrHTTPSGet(request))
-        return false;
-    
-    if (cache->isURLInOnlineWhitelist(request.url()))
-        return false;
-    
-    resource = cache->resourceForURL(request.url());
-    
-    // Don't load foreign resources.
-    if (resource && (resource->type() & ApplicationCacheResource::Foreign))
-        resource = 0;
-    
-    return true;
-}
-
-bool DocumentLoader::scheduleApplicationCacheLoad(ResourceLoader* loader, const ResourceRequest& request, const KURL& originalURL)
-{
-    if (!frameLoader()->frame()->settings() || !frameLoader()->frame()->settings()->offlineWebApplicationCacheEnabled())
-        return false;
-    
-    if (request.url() != originalURL)
-        return false;
-
-    ApplicationCacheResource* resource;
-    if (!shouldLoadResourceFromApplicationCache(request, resource))
-        // FIXME: Handle opportunistic caching namespaces
-        return false;
-    
-    m_pendingSubstituteResources.set(loader, resource);
-    deliverSubstituteResourcesAfterDelay();
-        
-    return true;
-}
-
-#endif // ENABLE(OFFLINE_WEB_APPLICATIONS)
-
 }
