@@ -32,13 +32,16 @@
 #define FrameLoader_h
 
 #include "CachePolicy.h"
+#include "DocumentWriter.h"
+#include "FrameLoaderStateMachine.h"
 #include "FrameLoaderTypes.h"
 #include "HistoryController.h"
+#include "NavigationScheduler.h"
 #include "PolicyCallback.h"
 #include "PolicyChecker.h"
-#include "RedirectScheduler.h"
 #include "ResourceLoadNotifier.h"
 #include "ResourceRequest.h"
+#include "SubframeLoader.h"
 #include "ThreadableLoader.h"
 #include "Timer.h"
 #include <wtf/Forward.h>
@@ -56,28 +59,27 @@ class DocumentLoader;
 class Event;
 class FormData;
 class FormState;
+class FormSubmission;
 class Frame;
 class FrameLoaderClient;
+class FrameNetworkingContext;
 class HistoryItem;
-class HTMLAppletElement;
 class HTMLFormElement;
-class HTMLFrameOwnerElement;
 class IconLoader;
-class IntSize;
 class NavigationAction;
-class RenderPart;
+class NetworkingContext;
+class Page;
+class ProtectionSpace;
 class ResourceError;
 class ResourceLoader;
 class ResourceResponse;
 class ScriptSourceCode;
-class ScriptString;
 class ScriptValue;
 class SecurityOrigin;
 class SerializedScriptValue;
 class SharedBuffer;
 class SubstituteData;
 class TextResourceDecoder;
-class Widget;
 
 struct FrameLoadRequest;
 struct WindowFeatures;
@@ -96,6 +98,8 @@ public:
     PolicyChecker* policyChecker() const { return &m_policyChecker; }
     HistoryController* history() const { return &m_history; }
     ResourceLoadNotifier* notifier() const { return &m_notifer; }
+    DocumentWriter* writer() const { return &m_writer; }
+    SubframeLoader* subframeLoader() const { return &m_subframeLoader; }
 
     // FIXME: This is not cool, people. There are too many different functions that all start loads.
     // We should aim to consolidate these into a smaller set of functions, and try to reuse more of
@@ -118,9 +122,6 @@ public:
 
     static void reportLocalLoadFailed(Frame*, const String& url);
 
-    // Called by createWindow in JSDOMWindowBase.cpp, e.g. to fulfill a modal dialog creation
-    Frame* createWindow(FrameLoader* frameLoaderForFrameLookup, const FrameLoadRequest&, const WindowFeatures&, bool& created);
-
     unsigned long loadResourceSynchronously(const ResourceRequest&, StoredCredentials, ResourceError&, ResourceResponse&, Vector<char>& data);
 
     bool canHandleRequest(const ResourceRequest&);
@@ -132,6 +133,8 @@ public:
     bool isLoadingMainResource() const { return m_isLoadingMainResource; }
     bool isLoading() const;
     bool frameHasLoaded() const;
+    void transferLoadingResourcesFromPage(Page*);
+    void dispatchTransferLoadingResourceFromPage(unsigned long, DocumentLoader*, const ResourceRequest&, Page*);
 
     int numPendingOrLoadingRequests(bool recurse) const;
     String referrer() const;
@@ -146,10 +149,12 @@ public:
     static double timeOfLastCompletedLoad();
 
     bool shouldUseCredentialStorage(ResourceLoader*);
+#if USE(PROTECTION_SPACE_AUTH_CALLBACK)
+    bool canAuthenticateAgainstProtectionSpace(ResourceLoader* loader, const ProtectionSpace& protectionSpace);
+#endif
     const ResourceRequest& originalRequest() const;
     const ResourceRequest& initialRequest() const;
     void receivedMainResourceError(const ResourceError&, bool isComplete);
-    void receivedData(const char*, int);
 
     bool willLoadMediaElementURL(KURL&);
 
@@ -162,7 +167,7 @@ public:
     ResourceError fileDoesNotExistError(const ResourceResponse&) const;
     ResourceError blockedError(const ResourceRequest&) const;
     ResourceError cannotShowURLError(const ResourceRequest&) const;
-    ResourceError interruptionForPolicyChangeError(const ResourceRequest&);
+    ResourceError interruptionForPolicyChangeError(const ResourceRequest&) const;
 
     bool isHostedByObjectElement() const;
     bool isLoadingMainFrame() const;
@@ -175,7 +180,6 @@ public:
 
     void didReceiveServerRedirectForProvisionalLoadForFrame();
     void finishedLoadingDocument(DocumentLoader*);
-    void committedLoad(DocumentLoader*, const char*, int);
     bool isReplacing() const;
     void setReplacing();
     void revertToProvisional(DocumentLoader*);
@@ -184,12 +188,13 @@ public:
     bool subframeIsLoading() const;
     void willChangeTitle(DocumentLoader*);
     void didChangeTitle(DocumentLoader*);
+    void didChangeIcons(DocumentLoader*);
 
     FrameLoadType loadType() const;
+
     CachePolicy subresourceCachePolicy() const;
 
     void didFirstLayout();
-    bool firstLayoutDone() const;
 
     void didFirstVisuallyNonEmptyLayout();
 
@@ -209,13 +214,10 @@ public:
 
     void setDefersLoading(bool);
 
-    void changeLocation(const KURL&, const String& referrer, bool lockHistory = true, bool lockBackForwardList = true, bool userGesture = false, bool refresh = false);
-    void urlSelected(const ResourceRequest&, const String& target, PassRefPtr<Event>, bool lockHistory, bool lockBackForwardList, bool userGesture, ReferrerPolicy);
-    bool requestFrame(HTMLFrameOwnerElement*, const String& url, const AtomicString& frameName);
+    void changeLocation(PassRefPtr<SecurityOrigin>, const KURL&, const String& referrer, bool lockHistory = true, bool lockBackForwardList = true, bool refresh = false);
+    void urlSelected(const KURL&, const String& target, PassRefPtr<Event>, bool lockHistory, bool lockBackForwardList, ReferrerPolicy);
 
-    void submitForm(const char* action, const String& url,
-        PassRefPtr<FormData>, const String& target, const String& contentType, const String& boundary,
-        bool lockHistory, PassRefPtr<Event>, PassRefPtr<FormState>);
+    void submitForm(PassRefPtr<FormSubmission>);
 
     void stop();
     void stopLoading(UnloadEventPolicy, DatabasePolicy = DatabasePolicyStop);
@@ -223,31 +225,19 @@ public:
 
     void didExplicitOpen();
 
+    // Callbacks from DocumentWriter
+    void didBeginDocument(bool dispatchWindowObjectAvailable);
+    void didEndDocument();
+    void willSetEncoding();
+
     KURL iconURL();
     void commitIconURLToIconDatabase(const KURL&);
 
     KURL baseURL() const;
 
-    void replaceDocument(const String&);
-
-    void begin();
-    void begin(const KURL&, bool dispatchWindowObjectAvailable = true, SecurityOrigin* forcedSecurityOrigin = 0);
-
-    void write(const char* string, int length = -1, bool flush = false);
-    void write(const String&);
-    void end();
-    void endIfNotLoadingMainResource();
-
-    void setEncoding(const String& encoding, bool userChosen);
-    String encoding() const;
-
-    void tokenizerProcessedData();
-
     void handledOnloadEvents();
     String userAgent(const KURL&) const;
 
-    PassRefPtr<Widget> createJavaAppletWidget(const IntSize&, HTMLAppletElement*, const HashMap<String, String>& args);
-
     void dispatchDidClearWindowObjectInWorld(DOMWrapperWorld*);
     void dispatchDidClearWindowObjectsInAllWorlds();
     void dispatchDocumentElementAvailable();
@@ -258,7 +248,7 @@ public:
     SandboxFlags sandboxFlags() const { return m_sandboxFlags; }
     // The following sandbox flags will be forced, regardless of changes to
     // the sandbox attribute of any parent frames.
-    void setForceSandboxFlags(SandboxFlags flags) { m_forceSandboxFlags = flags; m_sandboxFlags |= flags; }
+    void setForcedSandboxFlags(SandboxFlags flags) { m_forcedSandboxFlags = flags; m_sandboxFlags |= flags; }
 
     // Mixed content related functions.
     static bool isMixedContent(SecurityOrigin* context, const KURL&);
@@ -272,19 +262,14 @@ public:
 
     void resetMultipleFormSubmissionProtection();
 
-    void addData(const char* bytes, int length);
-
     void checkCallImplicitClose();
 
     void frameDetached();
 
     const KURL& url() const { return m_URL; }
 
-    void setResponseMIMEType(const String&);
-    const String& responseMIMEType() const;
-
-    bool allowPlugins(ReasonForCallingAllowPlugins);
-    bool containsPlugins() const;
+    // setURL is a low-level setter and does not trigger loading.
+    void setURL(const KURL&);
 
     void loadDone();
     void finishedParsing();
@@ -294,21 +279,17 @@ public:
 
     bool isComplete() const;
 
-    bool requestObject(RenderPart* frame, const String& url, const AtomicString& frameName,
-        const String& serviceType, const Vector<String>& paramNames, const Vector<String>& paramValues);
-
     KURL completeURL(const String& url);
 
     void cancelAndClear();
 
     void setTitle(const String&);
+    void setIconURL(const String&);
 
-    void commitProvisionalLoad(PassRefPtr<CachedPage>);
+    void commitProvisionalLoad();
     bool isLoadingFromCachedPage() const { return m_loadingFromCachedPage; }
 
-    bool committingFirstRealLoad() const { return !m_creatingInitialEmptyDocument && !m_committedFirstRealDocumentLoad; }
-    bool committedFirstRealDocumentLoad() const { return m_committedFirstRealDocumentLoad; }
-    bool creatingInitialEmptyDocument() const { return m_creatingInitialEmptyDocument; }
+    FrameLoaderStateMachine* stateMachine() const { return &m_stateMachine; }
 
     void iconLoadDecisionAvailable();
 
@@ -340,27 +321,25 @@ public:
 
     static ObjectContentType defaultObjectContentType(const KURL& url, const String& mimeType);
 
-private:
-    bool canCachePageContainingThisFrame();
-#ifndef NDEBUG
-    void logCanCachePageDecision();
-    bool logCanCacheFrameDecision(int indentLevel);
-#endif
+    void clear(bool clearWindowProperties = true, bool clearScriptObjects = true, bool clearFrameView = true);
 
-    void checkTimerFired(Timer<FrameLoader>*);
+    bool quickRedirectComing() const { return m_quickRedirectComing; }
 
+    bool shouldClose();
+    
     void started();
 
-    bool shouldUsePlugin(const KURL&, const String& mimeType, bool hasFallback, bool& useFallback);
-    bool loadPlugin(RenderPart*, const KURL&, const String& mimeType,
-    const Vector<String>& paramNames, const Vector<String>& paramValues, bool useFallback);
+    bool pageDismissalEventBeingDispatched() const { return m_pageDismissalEventBeingDispatched; }
+
+    NetworkingContext* networkingContext() const;
+
+private:
+    void checkTimerFired(Timer<FrameLoader>*);
     
-    void navigateWithinDocument(HistoryItem*);
-    void navigateToDifferentDocument(HistoryItem*, FrameLoadType);
+    void loadSameDocumentItem(HistoryItem*);
+    void loadDifferentDocumentItem(HistoryItem*, FrameLoadType);
     
-    bool loadProvisionalItemFromCachedPage();
-    void cachePageForHistoryItem(HistoryItem*);
-    void pageHidden();
+    void loadProvisionalItemFromCachedPage();
 
     void receivedFirstData();
 
@@ -379,14 +358,12 @@ private:
 
     void mainReceivedError(const ResourceError&, bool isComplete);
 
-    void setLoadType(FrameLoadType);
-
     static void callContinueLoadAfterNavigationPolicy(void*, const ResourceRequest&, PassRefPtr<FormState>, bool shouldContinue);
-    static void callContinueLoadAfterNewWindowPolicy(void*, const ResourceRequest&, PassRefPtr<FormState>, const String& frameName, bool shouldContinue);
+    static void callContinueLoadAfterNewWindowPolicy(void*, const ResourceRequest&, PassRefPtr<FormState>, const String& frameName, const NavigationAction&, bool shouldContinue);
     static void callContinueFragmentScrollAfterNavigationPolicy(void*, const ResourceRequest&, PassRefPtr<FormState>, bool shouldContinue);
 
     void continueLoadAfterNavigationPolicy(const ResourceRequest&, PassRefPtr<FormState>, bool shouldContinue);
-    void continueLoadAfterNewWindowPolicy(const ResourceRequest&, PassRefPtr<FormState>, const String& frameName, bool shouldContinue);
+    void continueLoadAfterNewWindowPolicy(const ResourceRequest&, PassRefPtr<FormState>, const String& frameName, const NavigationAction&, bool shouldContinue);
     void continueFragmentScrollAfterNavigationPolicy(const ResourceRequest&, bool shouldContinue);
 
     bool shouldScrollToAnchor(bool isFormSubmission, FrameLoadType, const KURL&);
@@ -400,16 +377,14 @@ private:
     void setState(FrameState);
 
     void closeOldDataSources();
-    void open(CachedPage&);
-
-    void updateHistoryAfterClientRedirect();
-
-    void clear(bool clearWindowProperties = true, bool clearScriptObjects = true, bool clearFrameView = true);
+    void prepareForCachedPageRestore();
 
     bool shouldReloadToHandleUnreachableURL(DocumentLoader*);
 
     void dispatchDidCommitLoad();
 
+    void urlSelected(const FrameLoadRequest&, PassRefPtr<Event>, bool lockHistory, bool lockBackForwardList, ReferrerPolicy, ShouldReplaceDocumentIfJavaScriptURL);
+
     void loadWithDocumentLoader(DocumentLoader*, FrameLoadType, PassRefPtr<FormState>); // Calls continueLoadAfterNavigationPolicy
     void load(DocumentLoader*);                                                         // Calls loadWithDocumentLoader   
 
@@ -430,14 +405,10 @@ private:
     void detachChildren();
     void closeAndRemoveChild(Frame*);
 
-    Frame* loadSubframe(HTMLFrameOwnerElement*, const KURL&, const String& name, const String& referrer);
-
     void loadInSameDocument(const KURL&, SerializedScriptValue* stateObject, bool isNewNavigation);
 
     void provisionalLoadStarted();
 
-    bool canCachePage();
-
     bool didOpenURL(const KURL&);
 
     void scheduleCheckCompleted();
@@ -449,8 +420,6 @@ private:
     bool shouldTreatURLAsSameAsCurrent(const KURL&) const;
 
     void updateSandboxFlags();
-    // FIXME: isDocumentSandboxed should eventually replace isSandboxed.
-    bool isDocumentSandboxed(SandboxFlags) const;
 
     Frame* m_frame;
     FrameLoaderClient* m_client;
@@ -458,6 +427,9 @@ private:
     mutable PolicyChecker m_policyChecker;
     mutable HistoryController m_history;
     mutable ResourceLoadNotifier m_notifer;
+    mutable DocumentWriter m_writer;
+    mutable SubframeLoader m_subframeLoader;
+    mutable FrameLoaderStateMachine m_stateMachine;
 
     FrameState m_state;
     FrameLoadType m_loadType;
@@ -472,7 +444,6 @@ private:
 
     bool m_delegateIsHandlingProvisionalLoadError;
 
-    bool m_firstLayoutDone;
     bool m_quickRedirectComing;
     bool m_sentRedirectNotification;
     bool m_inStopAllLoaders;
@@ -481,11 +452,9 @@ private:
 
     bool m_isExecutingJavaScriptFormAction;
 
-    String m_responseMIMEType;
-
     bool m_didCallImplicitClose;
     bool m_wasUnloadEventEmitted;
-    bool m_unloadEventBeingDispatched;
+    bool m_pageDismissalEventBeingDispatched;
     bool m_isComplete;
     bool m_isLoadingMainResource;
 
@@ -497,16 +466,7 @@ private:
     OwnPtr<IconLoader> m_iconLoader;
     bool m_mayLoadIconLater;
 
-    bool m_cancellingWithLoadInProgress;
-
     bool m_needsClear;
-    bool m_receivedData;
-
-    bool m_encodingWasChosenByUser;
-    String m_encoding;
-    RefPtr<TextResourceDecoder> m_decoder;
-
-    bool m_containsPlugIns;
 
     KURL m_submittedFormURL;
 
@@ -517,22 +477,25 @@ private:
     Frame* m_opener;
     HashSet<Frame*> m_openedFrames;
 
-    bool m_creatingInitialEmptyDocument;
-    bool m_isDisplayingInitialEmptyDocument;
-    bool m_committedFirstRealDocumentLoad;
-
     bool m_didPerformFirstNavigation;
     bool m_loadingFromCachedPage;
     bool m_suppressOpenerInNewFrame;
     
     SandboxFlags m_sandboxFlags;
-    SandboxFlags m_forceSandboxFlags;
+    SandboxFlags m_forcedSandboxFlags;
 
-#ifndef NDEBUG
-    bool m_didDispatchDidCommitLoad;
-#endif
+    RefPtr<FrameNetworkingContext> m_networkingContext;
 };
 
+// This function is called by createWindow() in JSDOMWindowBase.cpp, for example, for
+// modal dialog creation.  The lookupFrame is for looking up the frame name in case
+// the frame name references a frame different from the openerFrame, e.g. when it is
+// "_self" or "_parent".
+//
+// FIXME: Consider making this function part of an appropriate class (not FrameLoader)
+// and moving it to a more appropriate location.
+Frame* createWindow(Frame* openerFrame, Frame* lookupFrame, const FrameLoadRequest&, const WindowFeatures&, bool& created);
+
 } // namespace WebCore
 
 #endif // FrameLoader_h
