@@ -40,10 +40,11 @@
 
 namespace WebCore {
 
-    class ApplicationCache;
+    class ApplicationCacheHost;
     class DocumentLoader;
     class Frame;
     class FrameLoader;
+    class ProtectionSpace;
     class ResourceHandle;
     class SharedBuffer;
     
@@ -53,7 +54,7 @@ namespace WebCore {
 
         void cancel();
 
-        virtual bool load(const ResourceRequest&);
+        virtual void init(const ResourceRequest&);
 
         FrameLoader* frameLoader() const;
         DocumentLoader* documentLoader() const { return m_documentLoader.get(); }
@@ -79,13 +80,17 @@ namespace WebCore {
         virtual void didSendData(unsigned long long bytesSent, unsigned long long totalBytesToBeSent);
         virtual void didReceiveResponse(const ResourceResponse&);
         virtual void didReceiveData(const char*, int, long long lengthReceived, bool allAtOnce);
+        virtual void didReceiveCachedMetadata(const char*, int) { }
         void willStopBufferingData(const char*, int);
-        virtual void didFinishLoading();
+        virtual void didFinishLoading(double finishTime);
         virtual void didFail(const ResourceError&);
 
         virtual bool shouldUseCredentialStorage();
         virtual void didReceiveAuthenticationChallenge(const AuthenticationChallenge&);
         void didCancelAuthenticationChallenge(const AuthenticationChallenge&);
+#if USE(PROTECTION_SPACE_AUTH_CALLBACK)
+        virtual bool canAuthenticateAgainstProtectionSpace(const ProtectionSpace&);
+#endif
         virtual void receivedCancellation(const AuthenticationChallenge&);
 
         // ResourceHandleClient
@@ -93,7 +98,8 @@ namespace WebCore {
         virtual void didSendData(ResourceHandle*, unsigned long long bytesSent, unsigned long long totalBytesToBeSent);
         virtual void didReceiveResponse(ResourceHandle*, const ResourceResponse&);
         virtual void didReceiveData(ResourceHandle*, const char*, int, int lengthReceived);
-        virtual void didFinishLoading(ResourceHandle*);
+        virtual void didReceiveCachedMetadata(ResourceHandle*, const char* data, int length) { didReceiveCachedMetadata(data, length); }
+        virtual void didFinishLoading(ResourceHandle*, double finishTime);
         virtual void didFail(ResourceHandle*, const ResourceError&);
         virtual void wasBlocked(ResourceHandle*);
         virtual void cannotShowURL(ResourceHandle*);
@@ -101,6 +107,9 @@ namespace WebCore {
         virtual bool shouldUseCredentialStorage(ResourceHandle*) { return shouldUseCredentialStorage(); }
         virtual void didReceiveAuthenticationChallenge(ResourceHandle*, const AuthenticationChallenge& challenge) { didReceiveAuthenticationChallenge(challenge); } 
         virtual void didCancelAuthenticationChallenge(ResourceHandle*, const AuthenticationChallenge& challenge) { didCancelAuthenticationChallenge(challenge); } 
+#if USE(PROTECTION_SPACE_AUTH_CALLBACK)
+        virtual bool canAuthenticateAgainstProtectionSpace(ResourceHandle*, const ProtectionSpace& protectionSpace) { return canAuthenticateAgainstProtectionSpace(protectionSpace); }
+#endif
         virtual void receivedCancellation(ResourceHandle*, const AuthenticationChallenge& challenge) { receivedCancellation(challenge); }
         virtual void willCacheResponse(ResourceHandle*, CacheStoragePolicy&);
 #if PLATFORM(MAC)
@@ -109,37 +118,46 @@ namespace WebCore {
 #if USE(CFNETWORK)
         virtual bool shouldCacheResponse(ResourceHandle*, CFCachedURLResponseRef);
 #endif
+#if ENABLE(BLOB)
+        virtual AsyncFileStream* createAsyncFileStream(FileStreamClient*);
+#endif
 
+        const KURL& url() const { return m_request.url(); } 
         ResourceHandle* handle() const { return m_handle.get(); }
         bool sendResourceLoadCallbacks() const { return m_sendResourceLoadCallbacks; }
 
+        bool reachedTerminalState() const { return m_reachedTerminalState; }
+
         void setShouldBufferData(bool shouldBufferData);
 
     protected:
         ResourceLoader(Frame*, bool sendResourceLoadCallbacks, bool shouldContentSniff);
 
 #if ENABLE(OFFLINE_WEB_APPLICATIONS)
-        bool scheduleLoadFallbackResourceFromApplicationCache(ApplicationCache* = 0);
+        friend class ApplicationCacheHost;  // for access to request()
 #endif
-
+        friend class ResourceLoadScheduler; // for access to start()
+        // start() actually sends the load to the network (unless the load is being 
+        // deferred) and should only be called by ResourceLoadScheduler or setDefersLoading().
+        void start();
+        
         virtual void didCancel(const ResourceError&);
-        void didFinishLoadingOnePart();
+        void didFinishLoadingOnePart(double finishTime);
 
         const ResourceRequest& request() const { return m_request; }
-        bool reachedTerminalState() const { return m_reachedTerminalState; }
         bool cancelled() const { return m_cancelled; }
         bool defersLoading() const { return m_defersLoading; }
 
         RefPtr<ResourceHandle> m_handle;
         RefPtr<Frame> m_frame;
         RefPtr<DocumentLoader> m_documentLoader;
-        ResourceResponse m_response;        
+        ResourceResponse m_response;
         
     private:
         ResourceRequest m_request;
         RefPtr<SharedBuffer> m_resourceData;
         
-        unsigned long m_identifier;        
+        unsigned long m_identifier;
 
         bool m_reachedTerminalState;
         bool m_cancelled;
