@@ -40,11 +40,14 @@
 #include "Frame.h"
 #include "FrameTree.h"
 #include "FrameView.h"
-#include "HTMLFrameOwnerElement.h"
+#include "HTMLAreaElement.h"
+#include "HTMLImageElement.h"
 #include "HTMLNames.h"
+#include "HitTestResult.h"
 #include "KeyboardEvent.h"
 #include "Page.h"
 #include "Range.h"
+#include "RenderLayer.h"
 #include "RenderObject.h"
 #include "RenderWidget.h"
 #include "SelectionController.h"
@@ -86,6 +89,7 @@ FocusController::FocusController(Page* page)
 
 void FocusController::setFocusedFrame(PassRefPtr<Frame> frame)
 {
+    ASSERT(!frame || frame->page() == m_page);
     if (m_focusedFrame == frame || m_isChangingFocusedFrame)
         return;
 
@@ -107,10 +111,12 @@ void FocusController::setFocusedFrame(PassRefPtr<Frame> frame)
         newFrame->document()->dispatchWindowEvent(Event::create(eventNames().focusEvent, false, false));
     }
 
+    m_page->chrome()->focusedFrameChanged(newFrame.get());
+
     m_isChangingFocusedFrame = false;
 }
 
-Frame* FocusController::focusedOrMainFrame()
+Frame* FocusController::focusedOrMainFrame() const
 {
     if (Frame* frame = focusedFrame())
         return frame;
@@ -123,8 +129,14 @@ void FocusController::setFocused(bool focused)
         return;
     
     m_isFocused = focused;
-    
-    if (m_focusedFrame && m_focusedFrame->view()) {
+
+    if (!m_isFocused)
+        focusedOrMainFrame()->eventHandler()->stopAutoscrollTimer();
+
+    if (!m_focusedFrame)
+        setFocusedFrame(m_page->mainFrame());
+
+    if (m_focusedFrame->view()) {
         m_focusedFrame->selection()->setFocused(focused);
         dispatchEventsOnWindowAndFocusedNode(m_focusedFrame->document(), focused);
     }
@@ -275,7 +287,7 @@ bool FocusController::advanceFocusInDocumentOrder(FocusDirection direction, Keyb
 
     if (caretBrowsing) {
         VisibleSelection newSelection(Position(node, 0), Position(node, 0), DOWNSTREAM);
-        if (frame->shouldChangeSelection(newSelection))
+        if (frame->selection()->shouldChangeSelection(newSelection))
             frame->selection()->setSelection(newSelection);
     }
 
@@ -283,141 +295,6 @@ bool FocusController::advanceFocusInDocumentOrder(FocusDirection direction, Keyb
     return true;
 }
 
-bool FocusController::advanceFocusDirectionally(FocusDirection direction, KeyboardEvent* event)
-{
-    Frame* frame = focusedOrMainFrame();
-    ASSERT(frame);
-    Document* focusedDocument = frame->document();
-    if (!focusedDocument)
-        return false;
-
-    Node* focusedNode = focusedDocument->focusedNode();
-    if (!focusedNode) {
-        // Just move to the first focusable node.
-        FocusDirection tabDirection = (direction == FocusDirectionUp || direction == FocusDirectionLeft) ?
-                                       FocusDirectionForward : FocusDirectionBackward;
-        // 'initialFocus' is set to true so the chrome is not focused.
-        return advanceFocusInDocumentOrder(tabDirection, event, true);
-    }
-
-    // Move up in the chain of nested frames.
-    frame = frame->tree()->top();
-
-    FocusCandidate focusCandidate;
-    findFocusableNodeInDirection(frame->document(), focusedNode, direction, event, focusCandidate);
-
-    Node* node = focusCandidate.node;
-    if (!node || !node->isElementNode()) {
-        // FIXME: May need a way to focus a document here.
-        Frame* frame = focusedOrMainFrame();
-        scrollInDirection(frame, direction);
-        return false;
-    }
-
-    // In order to avoid crazy jump between links that are either far away from each other,
-    // or just not currently visible, lets do a scroll in the given direction and bail out
-    // if |node| element is not in the viewport.
-    if (hasOffscreenRect(node)) {
-        Frame* frame = node->document()->view()->frame();
-        scrollInDirection(frame, direction);
-        return true;
-    }
-
-    Document* newDocument = node->document();
-
-    if (newDocument != focusedDocument) {
-        // Focus is going away from the originally focused document, so clear the focused node.
-        focusedDocument->setFocusedNode(0);
-    }
-
-    if (newDocument)
-        setFocusedFrame(newDocument->frame());
-
-    Element* element = static_cast<Element*>(node);
-    ASSERT(element);
-
-    scrollIntoView(element);
-    element->focus(false);
-    return true;
-}
-
-static void updateFocusCandidateIfCloser(Node* focusedNode, Node* candidate, long long distance, FocusCandidate& closestFocusCandidate)
-{
-    // Bail out if |distance| is bigger than the current closest candidate.
-    if (distance >= closestFocusCandidate.distance)
-        return;
-
-    // If |focusedNode| and |candidate| are in the same document AND
-    // current |closestFocusCandidadte| is not in an {i}frame that is
-    // preferable to get focused.
-    if (focusedNode->document() == candidate->document()
-        && distance < closestFocusCandidate.parentDistance) {
-        closestFocusCandidate.node = candidate;
-        closestFocusCandidate.distance = distance;
-        closestFocusCandidate.parentDistance = maxDistance();
-    } else if (focusedNode->document() != candidate->document()) {
-        // If the |focusedNode| is in an inner document and the |candidate| is
-        // in a different document, we only consider to change focus if there is
-        // not another already good focusable candidate in the same document as
-        // |focusedNode|.
-        if (!((isInRootDocument(candidate) && !isInRootDocument(focusedNode))
-            && closestFocusCandidate.node
-            && focusedNode->document() == closestFocusCandidate.node->document())) {
-            closestFocusCandidate.node = candidate;
-            closestFocusCandidate.distance = distance;
-        }
-    }
-}
-
-void FocusController::findFocusableNodeInDirection(Document* document, Node* focusedNode, FocusDirection direction, KeyboardEvent* event, FocusCandidate& closestFocusCandidate)
-{
-    ASSERT(document);
-
-    // Walk all the child nodes and update focusCandidate if we find a nearer node.
-    for (Node* candidate = document->firstChild(); candidate; candidate = candidate->traverseNextNode()) {
-        // Inner documents case.
-        if (candidate->isFrameOwnerElement())
-            deepFindFocusableNodeInDirection(focusedNode, candidate, direction, event, closestFocusCandidate);
-        else if (candidate != focusedNode && candidate->isKeyboardFocusable(event)) {
-            long long distance = distanceInDirection(focusedNode, candidate,
-                                                     direction, closestFocusCandidate);
-            updateFocusCandidateIfCloser(focusedNode, candidate, distance, closestFocusCandidate);
-        }
-    }
-}
-
-void FocusController::deepFindFocusableNodeInDirection(Node* focusedNode, Node* candidate, FocusDirection direction, KeyboardEvent* event, FocusCandidate& closestFocusCandidate)
-{
-    HTMLFrameOwnerElement* owner = static_cast<HTMLFrameOwnerElement*>(candidate);
-    if (!owner->contentFrame())
-        return;
-
-    Document* innerDocument = owner->contentFrame()->document();
-    if (!innerDocument)
-        return;
-
-    if (innerDocument == focusedNode->document())
-        findFocusableNodeInDirection(innerDocument, focusedNode, direction, event, closestFocusCandidate);
-    else {
-        // Check if the current {i}frame element itself is a good candidate
-        // to move focus to. If it is, then we traverse its inner nodes.
-        // Lets pass a copy of the best candidate, to not get fooled by a
-        // frame without focusable elements.
-        FocusCandidate focusCandidateCopy = closestFocusCandidate;
-        long long distance = distanceInDirection(focusedNode, candidate, direction, focusCandidateCopy);
-        if (distance < focusCandidateCopy.distance) {
-            focusCandidateCopy.parentAlignment = focusCandidateCopy.alignment;
-            focusCandidateCopy.parentDistance = distance;
-
-            findFocusableNodeInDirection(innerDocument, focusedNode, direction, event, focusCandidateCopy);
-
-            // If we really have an inner closer focus candidate node, take it.
-            if (closestFocusCandidate.node != focusCandidateCopy.node)
-                closestFocusCandidate = focusCandidateCopy;
-        }
-    }
-}
-
 static bool relinquishesEditingFocus(Node *node)
 {
     ASSERT(node);
@@ -451,14 +328,19 @@ static void clearSelectionIfNeeded(Frame* oldFocusedFrame, Frame* newFocusedFram
     if (selectionStartNode == newFocusedNode || selectionStartNode->isDescendantOf(newFocusedNode) || selectionStartNode->shadowAncestorNode() == newFocusedNode)
         return;
         
-    if (Node* mousePressNode = newFocusedFrame->eventHandler()->mousePressNode())
-        if (mousePressNode->renderer() && !mousePressNode->canStartSelection())
-            if (Node* root = s->rootEditableElement())
-                if (Node* shadowAncestorNode = root->shadowAncestorNode())
-                    // Don't do this for textareas and text fields, when they lose focus their selections should be cleared
-                    // and then restored when they regain focus, to match other browsers.
-                    if (!shadowAncestorNode->hasTagName(inputTag) && !shadowAncestorNode->hasTagName(textareaTag))
-                        return;
+    if (Node* mousePressNode = newFocusedFrame->eventHandler()->mousePressNode()) {
+        if (mousePressNode->renderer() && !mousePressNode->canStartSelection()) {
+            // Don't clear the selection for contentEditable elements, but do clear it for input and textarea. See bug 38696.
+            Node * root = s->rootEditableElement();
+            if (!root)
+                return;
+
+            if (Node* shadowAncestorNode = root->shadowAncestorNode()) {
+                if (!shadowAncestorNode->hasTagName(inputTag) && !shadowAncestorNode->hasTagName(textareaTag))
+                    return;
+            }
+        }
+    }
     
     s->clear();
 }
@@ -475,9 +357,11 @@ bool FocusController::setFocusedNode(Node* node, PassRefPtr<Frame> newFocusedFra
     // FIXME: Might want to disable this check for caretBrowsing
     if (oldFocusedNode && oldFocusedNode->rootEditableElement() == oldFocusedNode && !relinquishesEditingFocus(oldFocusedNode))
         return false;
-        
+
+    m_page->editorClient()->willSetInputMethodState();
+
     clearSelectionIfNeeded(oldFocusedFrame.get(), newFocusedFrame.get(), node);
-    
+
     if (!node) {
         if (oldDocument)
             oldDocument->setFocusedNode(0);
@@ -499,8 +383,11 @@ bool FocusController::setFocusedNode(Node* node, PassRefPtr<Frame> newFocusedFra
 
     // Setting the focused node can result in losing our last reft to node when JS event handlers fire.
     RefPtr<Node> protect = node;
-    if (newDocument)
-        newDocument->setFocusedNode(node);
+    if (newDocument) {
+        bool successfullyFocused = newDocument->setFocusedNode(node);
+        if (!successfullyFocused)
+            return false;
+    }
 
     if (newDocument->focusedNode() == node)
         m_page->editorClient()->setInputMethodState(node->shouldUseInputMethod());
@@ -517,7 +404,7 @@ void FocusController::setActive(bool active)
 
     if (FrameView* view = m_page->mainFrame()->view()) {
         if (!view->platformWidget()) {
-            view->layoutIfNeededRecursive();
+            view->updateLayoutAndStyleIfNeededRecursive();
             view->updateControlTints();
         }
     }
@@ -528,4 +415,183 @@ void FocusController::setActive(bool active)
         dispatchEventsOnWindowAndFocusedNode(m_focusedFrame->document(), active);
 }
 
+static void updateFocusCandidateIfNeeded(FocusDirection direction, const IntRect& startingRect, FocusCandidate& candidate, FocusCandidate& closest)
+{
+    if (!candidate.visibleNode->isElementNode() || !candidate.visibleNode->renderer())
+        return;
+
+    // Ignore iframes that don't have a src attribute
+    if (frameOwnerElement(candidate) && (!frameOwnerElement(candidate)->contentFrame() || candidate.rect.isEmpty()))
+        return;
+
+    // Ignore off screen child nodes of containers that do not scroll (overflow:hidden)
+    if (candidate.isOffscreen && !canBeScrolledIntoView(direction, candidate))
+        return;
+
+    FocusCandidate current;
+    current.rect = startingRect;
+    distanceDataForNode(direction, current, candidate);
+    if (candidate.distance == maxDistance())
+        return;
+
+    if (candidate.isOffscreenAfterScrolling && candidate.alignment < Full)
+        return;
+
+    if (closest.isNull()) {
+        closest = candidate;
+        return;
+    }
+
+    IntRect intersectionRect = intersection(candidate.rect, closest.rect);
+    if (!intersectionRect.isEmpty()) {
+        // If 2 nodes are intersecting, do hit test to find which node in on top.
+        int x = intersectionRect.x() + intersectionRect.width() / 2;
+        int y = intersectionRect.y() + intersectionRect.height() / 2;
+        HitTestResult result = candidate.visibleNode->document()->page()->mainFrame()->eventHandler()->hitTestResultAtPoint(IntPoint(x, y), false, true);
+        if (candidate.visibleNode->contains(result.innerNode())) {
+            closest = candidate;
+            return;
+        }
+        if (closest.visibleNode->contains(result.innerNode()))
+            return;
+    }
+
+    if (candidate.alignment == closest.alignment) {
+        if (candidate.distance < closest.distance)
+            closest = candidate;
+        return;
+    }
+
+    if (candidate.alignment > closest.alignment)
+        closest = candidate;
+}
+
+void FocusController::findFocusCandidateInContainer(Node* container, const IntRect& startingRect, FocusDirection direction, KeyboardEvent* event, FocusCandidate& closest)
+{
+    ASSERT(container);
+    Node* focusedNode = (focusedFrame() && focusedFrame()->document()) ? focusedFrame()->document()->focusedNode() : 0;
+
+    Node* node = container->firstChild();
+    for (; node; node = (node->isFrameOwnerElement() || canScrollInDirection(direction, node)) ? node->traverseNextSibling(container) : node->traverseNextNode(container)) {
+        if (node == focusedNode)
+            continue;
+
+        if (!node->isKeyboardFocusable(event) && !node->isFrameOwnerElement() && !canScrollInDirection(direction, node))
+            continue;
+
+        FocusCandidate candidate = FocusCandidate(node, direction);
+        if (candidate.isNull())
+            continue;
+
+        candidate.enclosingScrollableBox = container;
+        updateFocusCandidateIfNeeded(direction, startingRect, candidate, closest);
+    }
+}
+
+bool FocusController::advanceFocusDirectionallyInContainer(Node* container, const IntRect& startingRect, FocusDirection direction, KeyboardEvent* event)
+{
+    if (!container || !container->document())
+        return false;
+
+    IntRect newStartingRect = startingRect;
+
+    if (startingRect.isEmpty())
+        newStartingRect = virtualRectForDirection(direction, nodeRectInAbsoluteCoordinates(container));
+
+    // Find the closest node within current container in the direction of the navigation.
+    FocusCandidate focusCandidate;
+    findFocusCandidateInContainer(container, newStartingRect, direction, event, focusCandidate);
+
+    if (focusCandidate.isNull()) {
+        // Nothing to focus, scroll if possible.
+        // NOTE: If no scrolling is performed (i.e. scrollInDirection returns false), the
+        // spatial navigation algorithm will skip this container.
+        return scrollInDirection(container, direction);
+    }
+
+    if (HTMLFrameOwnerElement* frameElement = frameOwnerElement(focusCandidate)) {
+        // If we have an iframe without the src attribute, it will not have a contentFrame().
+        // We ASSERT here to make sure that
+        // updateFocusCandidateIfNeeded() will never consider such an iframe as a candidate.
+        ASSERT(frameElement->contentFrame());
+
+        if (focusCandidate.isOffscreenAfterScrolling) {
+            scrollInDirection(focusCandidate.visibleNode->document(), direction);
+            return true;
+        }
+        // Navigate into a new frame.
+        IntRect rect;
+        Node* focusedNode = focusedOrMainFrame()->document()->focusedNode();
+        if (focusedNode && !hasOffscreenRect(focusedNode))
+            rect = nodeRectInAbsoluteCoordinates(focusedNode, true /* ignore border */);
+        frameElement->contentFrame()->document()->updateLayoutIgnorePendingStylesheets();
+        if (!advanceFocusDirectionallyInContainer(frameElement->contentFrame()->document(), rect, direction, event)) {
+            // The new frame had nothing interesting, need to find another candidate.
+            return advanceFocusDirectionallyInContainer(container, nodeRectInAbsoluteCoordinates(focusCandidate.visibleNode, true), direction, event);
+        }
+        return true;
+    }
+    if (canScrollInDirection(direction, focusCandidate.visibleNode)) {
+        if (focusCandidate.isOffscreenAfterScrolling) {
+            scrollInDirection(focusCandidate.visibleNode, direction);
+            return true;
+        }
+        // Navigate into a new scrollable container.
+        IntRect startingRect;
+        Node* focusedNode = focusedOrMainFrame()->document()->focusedNode();
+        if (focusedNode && !hasOffscreenRect(focusedNode))
+            startingRect = nodeRectInAbsoluteCoordinates(focusedNode, true);
+        return advanceFocusDirectionallyInContainer(focusCandidate.visibleNode, startingRect, direction, event);
+    }
+    if (focusCandidate.isOffscreenAfterScrolling) {
+        Node* container = focusCandidate.enclosingScrollableBox;
+        scrollInDirection(container, direction);
+        return true;
+    }
+
+    // We found a new focus node, navigate to it.
+    Element* element = toElement(focusCandidate.focusableNode);
+    ASSERT(element);
+
+    element->focus(false);
+    return true;
+}
+
+bool FocusController::advanceFocusDirectionally(FocusDirection direction, KeyboardEvent* event)
+{
+    Frame* curFrame = focusedOrMainFrame();
+    ASSERT(curFrame);
+
+    Document* focusedDocument = curFrame->document();
+    if (!focusedDocument)
+        return false;
+
+    Node* focusedNode = focusedDocument->focusedNode();
+    Node* container = focusedDocument;
+
+    // Figure out the starting rect.
+    IntRect startingRect;
+    if (focusedNode) {
+        if (!hasOffscreenRect(focusedNode)) {
+            container = scrollableEnclosingBoxOrParentFrameForNodeInDirection(direction, focusedNode);
+            startingRect = nodeRectInAbsoluteCoordinates(focusedNode, true /* ignore border */);
+        } else if (focusedNode->hasTagName(areaTag)) {
+            HTMLAreaElement* area = static_cast<HTMLAreaElement*>(focusedNode);
+            container = scrollableEnclosingBoxOrParentFrameForNodeInDirection(direction, area->imageElement());
+            startingRect = virtualRectForAreaElementAndDirection(direction, area);
+        }
+    }
+
+    bool consumed = false;
+    do {
+        if (container->isDocumentNode())
+            static_cast<Document*>(container)->updateLayoutIgnorePendingStylesheets();
+        consumed = advanceFocusDirectionallyInContainer(container, startingRect, direction, event);
+        startingRect = nodeRectInAbsoluteCoordinates(container, true /* ignore border */);
+        container = scrollableEnclosingBoxOrParentFrameForNodeInDirection(direction, container);
+    } while (!consumed && container);
+
+    return consumed;
+}
+
 } // namespace WebCore
