@@ -28,7 +28,6 @@
 #define Geolocation_h
 
 #include "GeolocationPositionCache.h"
-#include "GeolocationService.h"
 #include "Geoposition.h"
 #include "PositionCallback.h"
 #include "PositionError.h"
@@ -36,6 +35,10 @@
 #include "PositionOptions.h"
 #include "Timer.h"
 
+#if !ENABLE(CLIENT_BASED_GEOLOCATION)
+#include "GeolocationService.h"
+#endif
+
 namespace WebCore {
 
 class Frame;
@@ -46,7 +49,7 @@ class GeolocationError;
 #endif
 
 class Geolocation : public RefCounted<Geolocation>
-#if !ENABLE(CLIENT_BASED_GEOLOCATION)
+#if !ENABLE(CLIENT_BASED_GEOLOCATION) && ENABLE(GEOLOCATION)
     , public GeolocationServiceClient
 #endif
 {
@@ -61,10 +64,15 @@ public:
     int watchPosition(PassRefPtr<PositionCallback>, PassRefPtr<PositionErrorCallback>, PassRefPtr<PositionOptions>);
     void clearWatch(int watchId);
 
+    // These methods are used by Android.
+    void suspend();
+    void resume();
+
     void setIsAllowed(bool);
+    Frame* frame() const { return m_frame; }
 
 #if ENABLE(CLIENT_BASED_GEOLOCATION)
-    void setPosition(GeolocationPosition*);
+    void positionChanged();
     void setError(GeolocationError*);
 #else
     GeolocationService* getGeolocationService() const { return m_service.get(); }
@@ -101,6 +109,9 @@ private:
         GeoNotifier(Geolocation*, PassRefPtr<PositionCallback>, PassRefPtr<PositionErrorCallback>, PassRefPtr<PositionOptions>);
     };
 
+    typedef Vector<RefPtr<GeoNotifier> > GeoNotifierVector;
+    typedef HashSet<RefPtr<GeoNotifier> > GeoNotifierSet;
+
     class Watchers {
     public:
         void set(int id, PassRefPtr<GeoNotifier>);
@@ -109,7 +120,7 @@ private:
         bool contains(GeoNotifier*) const;
         void clear();
         bool isEmpty() const;
-        void getNotifiersVector(Vector<RefPtr<GeoNotifier> >&) const;
+        void getNotifiersVector(GeoNotifierVector&) const;
     private:
         typedef HashMap<int, RefPtr<GeoNotifier> > IdToNotifierMap;
         typedef HashMap<RefPtr<GeoNotifier>, int> NotifierToIdMap;
@@ -117,17 +128,40 @@ private:
         NotifierToIdMap m_notifierToIdMap;
     };
 
+    class PositionCacheWrapper {
+    public:
+        PositionCacheWrapper()
+            : m_cache(GeolocationPositionCache::instance())
+        {
+            m_cache->addUser();
+        }
+        ~PositionCacheWrapper()
+        {
+            m_cache->removeUser();
+        }
+        void setCachedPosition(Geoposition* cachedPosition) { m_cache->setCachedPosition(cachedPosition); }
+        Geoposition* cachedPosition() { return m_cache->cachedPosition(); }
+    private:
+        GeolocationPositionCache* m_cache;
+    };
+
     bool hasListeners() const { return !m_oneShots.isEmpty() || !m_watchers.isEmpty(); }
 
-    void sendError(Vector<RefPtr<GeoNotifier> >&, PositionError*);
-    void sendPosition(Vector<RefPtr<GeoNotifier> >&, Geoposition*);
-    
-    static void stopTimer(Vector<RefPtr<GeoNotifier> >&);
+    void sendError(GeoNotifierVector&, PositionError*);
+    void sendPosition(GeoNotifierVector&, Geoposition*);
+
+    static void extractNotifiersWithCachedPosition(GeoNotifierVector& notifiers, GeoNotifierVector* cached);
+    static void copyToSet(const GeoNotifierVector&, GeoNotifierSet&);
+
+    static void stopTimer(GeoNotifierVector&);
     void stopTimersForOneShots();
     void stopTimersForWatchers();
     void stopTimers();
-    
-    void positionChanged(PassRefPtr<Geoposition>);
+
+    void cancelRequests(GeoNotifierVector&);
+    void cancelAllRequests();
+
+    void positionChangedInternal();
     void makeSuccessCallbacks();
     void handleError(PositionError*);
 
@@ -136,7 +170,11 @@ private:
     bool startUpdating(GeoNotifier*);
     void stopUpdating();
 
-#if !ENABLE(CLIENT_BASED_GEOLOCATION)
+#if USE(PREEMPT_GEOLOCATION_PERMISSION)
+    void handlePendingPermissionNotifiers();
+#endif
+
+#if !ENABLE(CLIENT_BASED_GEOLOCATION) && ENABLE(GEOLOCATION)
     // GeolocationServiceClient
     virtual void geolocationServicePositionChanged(GeolocationService*);
     virtual void geolocationServiceErrorOccurred(GeolocationService*);
@@ -150,18 +188,16 @@ private:
     bool haveSuitableCachedPosition(PositionOptions*);
     void makeCachedPositionCallbacks();
 
-    typedef HashSet<RefPtr<GeoNotifier> > GeoNotifierSet;
-    
     GeoNotifierSet m_oneShots;
     Watchers m_watchers;
     Frame* m_frame;
 #if !ENABLE(CLIENT_BASED_GEOLOCATION)
     OwnPtr<GeolocationService> m_service;
-#else
-    RefPtr<GeoNotifier> m_startRequestPermissionNotifier;
+#endif
+#if USE(PREEMPT_GEOLOCATION_PERMISSION)
+    GeoNotifierSet m_pendingForPermissionNotifiers;
 #endif
     RefPtr<Geoposition> m_lastPosition;
-    RefPtr<Geoposition> m_currentPosition;
 
     enum {
         Unknown,
@@ -169,9 +205,10 @@ private:
         Yes,
         No
     } m_allowGeolocation;
-    bool m_shouldClearCache;
 
-    OwnPtr<GeolocationPositionCache> m_positionCache;
+#if ENABLE(GEOLOCATION)
+    PositionCacheWrapper m_positionCache;
+#endif
     GeoNotifierSet m_requestsAwaitingCachedPosition;
 };
     
