@@ -6,7 +6,6 @@
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
- *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -21,6 +20,7 @@
 #include "config.h"
 #include "Page.h"
 
+#include "BackForwardController.h"
 #include "BackForwardList.h"
 #include "Base64.h"
 #include "CSSStyleSelector.h"
@@ -29,6 +29,8 @@
 #include "ContextMenuClient.h"
 #include "ContextMenuController.h"
 #include "DOMWindow.h"
+#include "DeviceMotionController.h"
+#include "DeviceOrientationController.h"
 #include "DragController.h"
 #include "EditorClient.h"
 #include "Event.h"
@@ -44,7 +46,6 @@
 #include "HTMLElement.h"
 #include "HistoryItem.h"
 #include "InspectorController.h"
-#include "InspectorTimelineAgent.h"
 #include "Logging.h"
 #include "MediaCanStartListener.h"
 #include "Navigator.h"
@@ -53,19 +54,27 @@
 #include "PluginData.h"
 #include "PluginHalter.h"
 #include "PluginView.h"
+#include "PluginViewBase.h"
 #include "ProgressTracker.h"
 #include "RenderTheme.h"
 #include "RenderWidget.h"
+#include "RuntimeEnabledFeatures.h"
 #include "ScriptController.h"
 #include "SelectionController.h"
 #include "Settings.h"
 #include "SharedBuffer.h"
-#include "StringHash.h"
+#include "SpeechInput.h"
+#include "SpeechInputClient.h"
 #include "TextResourceDecoder.h"
 #include "Widget.h"
 #include <wtf/HashMap.h>
 #include <wtf/RefCountedLeakCounter.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/text/StringHash.h>
+
+#if ENABLE(ACCELERATED_2D_CANVAS)
+#include "SharedGraphicsContext3D.h"
+#endif
 
 #if ENABLE(DOM_STORAGE)
 #include "StorageArea.h"
@@ -84,6 +93,10 @@
 #include "GeolocationController.h"
 #endif
 
+#if ENABLE(INSPECTOR) && ENABLE(OFFLINE_WEB_APPLICATIONS)
+#include "InspectorApplicationCacheAgent.h"
+#endif
+
 namespace WebCore {
 
 static HashSet<Page*>* allPages;
@@ -96,11 +109,19 @@ static void networkStateChanged()
 {
     Vector<RefPtr<Frame> > frames;
     
+#if ENABLE(INSPECTOR) && ENABLE(OFFLINE_WEB_APPLICATIONS)
+    bool isNowOnline = networkStateNotifier().onLine();
+#endif
+
     // Get all the frames of all the pages in all the page groups
     HashSet<Page*>::iterator end = allPages->end();
     for (HashSet<Page*>::iterator it = allPages->begin(); it != end; ++it) {
         for (Frame* frame = (*it)->mainFrame(); frame; frame = frame->tree()->traverseNext())
             frames.append(frame);
+#if ENABLE(INSPECTOR) && ENABLE(OFFLINE_WEB_APPLICATIONS)
+        if (InspectorApplicationCacheAgent* applicationCacheAgent = (*it)->inspectorController()->applicationCacheAgent())
+            applicationCacheAgent->updateNetworkState(isNowOnline);
+#endif
     }
 
     AtomicString eventName = networkStateNotifier().onLine() ? eventNames().onlineEvent : eventNames().offlineEvent;
@@ -108,27 +129,34 @@ static void networkStateChanged()
         frames[i]->document()->dispatchWindowEvent(Event::create(eventName, false, false));
 }
 
-Page::Page(ChromeClient* chromeClient, ContextMenuClient* contextMenuClient, EditorClient* editorClient, DragClient* dragClient, InspectorClient* inspectorClient, PluginHalterClient* pluginHalterClient, GeolocationControllerClient* geolocationControllerClient)
-    : m_chrome(new Chrome(this, chromeClient))
-    , m_dragCaretController(new SelectionController(0, true))
+Page::Page(const PageClients& pageClients)
+    : m_chrome(adoptPtr(new Chrome(this, pageClients.chromeClient)))
+    , m_dragCaretController(adoptPtr(new SelectionController(0, true)))
 #if ENABLE(DRAG_SUPPORT)
-    , m_dragController(new DragController(this, dragClient))
+    , m_dragController(adoptPtr(new DragController(this, pageClients.dragClient)))
 #endif
-    , m_focusController(new FocusController(this))
+    , m_focusController(adoptPtr(new FocusController(this)))
 #if ENABLE(CONTEXT_MENUS)
-    , m_contextMenuController(new ContextMenuController(this, contextMenuClient))
+    , m_contextMenuController(adoptPtr(new ContextMenuController(this, pageClients.contextMenuClient)))
 #endif
 #if ENABLE(INSPECTOR)
-    , m_inspectorController(new InspectorController(this, inspectorClient))
+    , m_inspectorController(adoptPtr(new InspectorController(this, pageClients.inspectorClient)))
 #endif
 #if ENABLE(CLIENT_BASED_GEOLOCATION)
-    , m_geolocationController(new GeolocationController(this, geolocationControllerClient))
+    , m_geolocationController(adoptPtr(new GeolocationController(this, pageClients.geolocationClient)))
 #endif
-    , m_settings(new Settings(this))
-    , m_progress(new ProgressTracker)
-    , m_backForwardList(BackForwardList::create(this))
+#if ENABLE(DEVICE_ORIENTATION)
+    , m_deviceMotionController(RuntimeEnabledFeatures::deviceMotionEnabled() ? new DeviceMotionController(pageClients.deviceMotionClient) : 0)
+    , m_deviceOrientationController(RuntimeEnabledFeatures::deviceOrientationEnabled() ? new DeviceOrientationController(this, pageClients.deviceOrientationClient) : 0)
+#endif
+#if ENABLE(INPUT_SPEECH)
+    , m_speechInputClient(pageClients.speechInputClient)
+#endif
+    , m_settings(adoptPtr(new Settings(this)))
+    , m_progress(adoptPtr(new ProgressTracker))
+    , m_backForwardController(adoptPtr(new BackForwardController(this, pageClients.backForwardClient)))
     , m_theme(RenderTheme::themeForPage(this))
-    , m_editorClient(editorClient)
+    , m_editorClient(pageClients.editorClient)
     , m_frameCount(0)
     , m_openedByDOM(false)
     , m_tabKeyCyclesThroughElements(true)
@@ -145,20 +173,8 @@ Page::Page(ChromeClient* chromeClient, ContextMenuClient* contextMenuClient, Edi
     , m_customHTMLTokenizerTimeDelay(-1)
     , m_customHTMLTokenizerChunkSize(-1)
     , m_canStartMedia(true)
+    , m_viewMode(ViewModeWindowed)
 {
-#if !ENABLE(CONTEXT_MENUS)
-    UNUSED_PARAM(contextMenuClient);
-#endif
-#if !ENABLE(DRAG_SUPPORT)
-    UNUSED_PARAM(dragClient);
-#endif
-#if !ENABLE(INSPECTOR)
-    UNUSED_PARAM(inspectorClient);
-#endif
-#if !ENABLE(CLIENT_BASED_GEOLOCATION)
-    UNUSED_PARAM(geolocationControllerClient);
-#endif
-
     if (!allPages) {
         allPages = new HashSet<Page*>;
         
@@ -168,8 +184,8 @@ Page::Page(ChromeClient* chromeClient, ContextMenuClient* contextMenuClient, Edi
     ASSERT(!allPages->contains(this));
     allPages->add(this);
 
-    if (pluginHalterClient) {
-        m_pluginHalter.set(new PluginHalter(pluginHalterClient));
+    if (pageClients.pluginHalterClient) {
+        m_pluginHalter.set(new PluginHalter(pageClients.pluginHalterClient));
         m_pluginHalter->setPluginAllowedRunTime(m_settings->pluginAllowedRunTime());
     }
 
@@ -199,7 +215,7 @@ Page::~Page()
     m_inspectorController->inspectedPageDestroyed();
 #endif
 
-    m_backForwardList->close();
+    backForward()->close();
 
 #ifndef NDEBUG
     pageCounter.decrement();
@@ -211,6 +227,45 @@ Page::~Page()
 #endif
 }
 
+struct ViewModeInfo {
+    const char* name;
+    Page::ViewMode type;
+};
+static const int viewModeMapSize = 5;
+static ViewModeInfo viewModeMap[viewModeMapSize] = {
+    {"windowed", Page::ViewModeWindowed},
+    {"floating", Page::ViewModeFloating},
+    {"fullscreen", Page::ViewModeFullscreen},
+    {"maximized", Page::ViewModeMaximized},
+    {"minimized", Page::ViewModeMinimized}
+};
+
+Page::ViewMode Page::stringToViewMode(const String& text)
+{
+    for (int i = 0; i < viewModeMapSize; ++i) {
+        if (text == viewModeMap[i].name)
+            return viewModeMap[i].type;
+    }
+    return Page::ViewModeInvalid;
+}
+
+void Page::setViewMode(ViewMode viewMode)
+{
+    if (viewMode == m_viewMode || viewMode == ViewModeInvalid)
+        return;
+
+    m_viewMode = viewMode;
+
+    if (!m_mainFrame)
+        return;
+
+    if (m_mainFrame->view())
+        m_mainFrame->view()->forceLayout();
+
+    if (m_mainFrame->document())
+        m_mainFrame->document()->styleSelectorChanged(RecalcStyleImmediately);
+}
+
 void Page::setMainFrame(PassRefPtr<Frame> mainFrame)
 {
     ASSERT(!m_mainFrame); // Should only be called during initialization
@@ -227,14 +282,14 @@ void Page::setOpenedByDOM()
     m_openedByDOM = true;
 }
 
-BackForwardList* Page::backForwardList()
+BackForwardList* Page::backForwardList() const
 {
-    return m_backForwardList.get();
+    return m_backForwardController->client();
 }
 
 bool Page::goBack()
 {
-    HistoryItem* item = m_backForwardList->backItem();
+    HistoryItem* item = backForward()->backItem();
     
     if (item) {
         goToItem(item, FrameLoadTypeBack);
@@ -245,7 +300,7 @@ bool Page::goBack()
 
 bool Page::goForward()
 {
-    HistoryItem* item = m_backForwardList->forwardItem();
+    HistoryItem* item = backForward()->forwardItem();
     
     if (item) {
         goToItem(item, FrameLoadTypeForward);
@@ -258,9 +313,9 @@ bool Page::canGoBackOrForward(int distance) const
 {
     if (distance == 0)
         return true;
-    if (distance > 0 && distance <= m_backForwardList->forwardListCount())
+    if (distance > 0 && distance <= backForward()->forwardCount())
         return true;
-    if (distance < 0 && -distance <= m_backForwardList->backListCount())
+    if (distance < 0 && -distance <= backForward()->backCount())
         return true;
     return false;
 }
@@ -270,26 +325,33 @@ void Page::goBackOrForward(int distance)
     if (distance == 0)
         return;
 
-    HistoryItem* item = m_backForwardList->itemAtIndex(distance);
+    HistoryItem* item = backForward()->itemAtIndex(distance);
     if (!item) {
         if (distance > 0) {
-            int forwardListCount = m_backForwardList->forwardListCount();
-            if (forwardListCount > 0) 
-                item = m_backForwardList->itemAtIndex(forwardListCount);
+            if (int forwardCount = backForward()->forwardCount()) 
+                item = backForward()->itemAtIndex(forwardCount);
         } else {
-            int backListCount = m_backForwardList->backListCount();
-            if (backListCount > 0)
-                item = m_backForwardList->itemAtIndex(-backListCount);
+            if (int backCount = backForward()->backCount())
+                item = backForward()->itemAtIndex(-backCount);
         }
     }
 
-    ASSERT(item); // we should not reach this line with an empty back/forward list
-    if (item)
-        goToItem(item, FrameLoadTypeIndexedBackForward);
+    ASSERT(item);
+    if (!item)
+        return;
+
+    goToItem(item, FrameLoadTypeIndexedBackForward);
 }
 
 void Page::goToItem(HistoryItem* item, FrameLoadType type)
 {
+    if (defersLoading())
+        return;
+
+    // stopAllLoaders may end up running onload handlers, which could cause further history traversals that may lead to the passed in HistoryItem
+    // being deref()-ed. Make sure we can still use it with HistoryController::goToItem later.
+    RefPtr<HistoryItem> protector(item);
+    
     // Abort any current load unless we're navigating the current document to a new state object
     HistoryItem* currentItem = m_mainFrame->loader()->history()->currentItem();
     if (!item->stateObject() || !currentItem || item->documentSequenceNumber() != currentItem->documentSequenceNumber() || item == currentItem) {
@@ -313,7 +375,7 @@ void Page::goToItem(HistoryItem* item, FrameLoadType type)
 
 int Page::getHistoryLength()
 {
-    return m_backForwardList->backListCount() + 1 + m_backForwardList->forwardListCount();
+    return backForward()->backCount() + 1 + backForward()->forwardCount();
 }
 
 void Page::setGlobalHistoryItem(HistoryItem* item)
@@ -352,14 +414,23 @@ void Page::initGroup()
     m_group = m_singlePageGroup.get();
 }
 
-void Page::setNeedsReapplyStyles()
+void Page::scheduleForcedStyleRecalcForAllPages()
 {
     if (!allPages)
         return;
     HashSet<Page*>::iterator end = allPages->end();
     for (HashSet<Page*>::iterator it = allPages->begin(); it != end; ++it)
         for (Frame* frame = (*it)->mainFrame(); frame; frame = frame->tree()->traverseNext())
-            frame->setNeedsReapplyStyles();
+            frame->document()->scheduleForcedStyleRecalc();
+}
+
+void Page::updateViewportArguments()
+{
+    if (!mainFrame() || !mainFrame()->document() || mainFrame()->document()->viewportArguments() == m_viewportArguments)
+        return;
+
+    m_viewportArguments = mainFrame()->document()->viewportArguments();
+    chrome()->dispatchViewportDataDidChange(m_viewportArguments);
 }
 
 void Page::refreshPlugins(bool reload)
@@ -383,9 +454,9 @@ void Page::refreshPlugins(bool reload)
 
         if (!reload)
             continue;
-
+        
         for (Frame* frame = (*it)->mainFrame(); frame; frame = frame->tree()->traverseNext()) {
-            if (frame->loader()->containsPlugins())
+            if (frame->loader()->subframeLoader()->containsPlugins())
                 framesNeedingReload.append(frame);
         }
     }
@@ -396,25 +467,20 @@ void Page::refreshPlugins(bool reload)
 
 PluginData* Page::pluginData() const
 {
-    if (!mainFrame()->loader()->allowPlugins(NotAboutToInstantiatePlugin))
+    if (!mainFrame()->loader()->subframeLoader()->allowPlugins(NotAboutToInstantiatePlugin))
         return 0;
     if (!m_pluginData)
         m_pluginData = PluginData::create(this);
     return m_pluginData.get();
 }
 
-void Page::addMediaCanStartListener(MediaCanStartListener* listener)
+inline MediaCanStartListener* Page::takeAnyMediaCanStartListener()
 {
-    ASSERT(!m_canStartMedia);
-    ASSERT(!m_mediaCanStartListeners.contains(listener));
-    m_mediaCanStartListeners.add(listener);
-}
-
-void Page::removeMediaCanStartListener(MediaCanStartListener* listener)
-{
-    ASSERT(!m_canStartMedia);
-    ASSERT(m_mediaCanStartListeners.contains(listener));
-    m_mediaCanStartListeners.remove(listener);
+    for (Frame* frame = mainFrame(); frame; frame = frame->tree()->traverseNext()) {
+        if (MediaCanStartListener* listener = frame->document()->takeAnyMediaCanStartListener())
+            return listener;
+    }
+    return 0;
 }
 
 void Page::setCanStartMedia(bool canStartMedia)
@@ -424,16 +490,12 @@ void Page::setCanStartMedia(bool canStartMedia)
 
     m_canStartMedia = canStartMedia;
 
-    if (!m_canStartMedia || m_mediaCanStartListeners.isEmpty())
-        return;
-
-    Vector<MediaCanStartListener*> listeners;
-    copyToVector(m_mediaCanStartListeners, listeners);
-    m_mediaCanStartListeners.clear();
-
-    size_t size = listeners.size();
-    for (size_t i = 0; i < size; ++i)
-        listeners[i]->mediaCanStart();
+    while (m_canStartMedia) {
+        MediaCanStartListener* listener = takeAnyMediaCanStartListener();
+        if (!listener)
+            break;
+        listener->mediaCanStart();
+    }
 }
 
 static Frame* incrementFrame(Frame* curr, bool forward, bool wrapFlag)
@@ -444,26 +506,32 @@ static Frame* incrementFrame(Frame* curr, bool forward, bool wrapFlag)
 }
 
 bool Page::findString(const String& target, TextCaseSensitivity caseSensitivity, FindDirection direction, bool shouldWrap)
+{
+    return findString(target, (caseSensitivity == TextCaseInsensitive ? CaseInsensitive : 0) | (direction == FindDirectionBackward ? Backwards : 0) | (shouldWrap ? WrapAround : 0));
+}
+
+bool Page::findString(const String& target, FindOptions options)
 {
     if (target.isEmpty() || !mainFrame())
         return false;
 
+    bool shouldWrap = options & WrapAround;
     Frame* frame = focusController()->focusedOrMainFrame();
     Frame* startFrame = frame;
     do {
-        if (frame->findString(target, direction == FindDirectionForward, caseSensitivity == TextCaseSensitive, false, true)) {
+        if (frame->editor()->findString(target, (options & ~WrapAround) | StartInSelection)) {
             if (frame != startFrame)
                 startFrame->selection()->clear();
             focusController()->setFocusedFrame(frame);
             return true;
         }
-        frame = incrementFrame(frame, direction == FindDirectionForward, shouldWrap);
+        frame = incrementFrame(frame, !(options & Backwards), shouldWrap);
     } while (frame && frame != startFrame);
 
     // Search contents of startFrame, on the other side of the selection that we did earlier.
     // We cheat a bit and just research with wrap on
     if (shouldWrap && !startFrame->selection()->isNone()) {
-        bool found = startFrame->findString(target, direction == FindDirectionForward, caseSensitivity == TextCaseSensitive, true, true);
+        bool found = startFrame->editor()->findString(target, options | WrapAround | StartInSelection);
         focusController()->setFocusedFrame(frame);
         return found;
     }
@@ -472,6 +540,11 @@ bool Page::findString(const String& target, TextCaseSensitivity caseSensitivity,
 }
 
 unsigned int Page::markAllMatchesForText(const String& target, TextCaseSensitivity caseSensitivity, bool shouldHighlight, unsigned limit)
+{
+    return markAllMatchesForText(target, caseSensitivity == TextCaseInsensitive ? CaseInsensitive : 0, shouldHighlight, limit);
+}
+
+unsigned int Page::markAllMatchesForText(const String& target, FindOptions options, bool shouldHighlight, unsigned limit)
 {
     if (target.isEmpty() || !mainFrame())
         return 0;
@@ -480,8 +553,8 @@ unsigned int Page::markAllMatchesForText(const String& target, TextCaseSensitivi
 
     Frame* frame = mainFrame();
     do {
-        frame->setMarkedTextMatchesAreHighlighted(shouldHighlight);
-        matches += frame->markAllMatchesForText(target, caseSensitivity == TextCaseSensitive, (limit == 0) ? 0 : (limit - matches));
+        frame->editor()->setMarkedTextMatchesAreHighlighted(shouldHighlight);
+        matches += frame->editor()->countMatchesForText(target, options, limit ? (limit - matches) : 0, true);
         frame = incrementFrame(frame, true, false);
     } while (frame);
 
@@ -495,7 +568,7 @@ void Page::unmarkAllTextMatches()
 
     Frame* frame = mainFrame();
     do {
-        frame->document()->removeMarkers(DocumentMarker::TextMatch);
+        frame->document()->markers()->removeMarkers(DocumentMarker::TextMatch);
         frame = incrementFrame(frame, true, false);
     } while (frame);
 }
@@ -576,25 +649,20 @@ void Page::userStyleSheetLocationChanged()
     m_didLoadUserStyleSheet = false;
     m_userStyleSheet = String();
     m_userStyleSheetModificationTime = 0;
-    
+
     // Data URLs with base64-encoded UTF-8 style sheets are common. We can process them
     // synchronously and avoid using a loader. 
-    if (url.protocolIs("data") && url.string().startsWith("data:text/css;charset=utf-8;base64,")) {
+    if (url.protocolIsData() && url.string().startsWith("data:text/css;charset=utf-8;base64,")) {
         m_didLoadUserStyleSheet = true;
-        
-        const unsigned prefixLength = 35;
-        Vector<char> encodedData(url.string().length() - prefixLength);
-        for (unsigned i = prefixLength; i < url.string().length(); ++i)
-            encodedData[i - prefixLength] = static_cast<char>(url.string()[i]);
 
         Vector<char> styleSheetAsUTF8;
-        if (base64Decode(encodedData, styleSheetAsUTF8))
+        if (base64Decode(decodeURLEscapeSequences(url.string().substring(35)), styleSheetAsUTF8, IgnoreWhitespace))
             m_userStyleSheet = String::fromUTF8(styleSheetAsUTF8.data(), styleSheetAsUTF8.size());
     }
-    
+
     for (Frame* frame = mainFrame(); frame; frame = frame->tree()->traverseNext()) {
         if (frame->document())
-            frame->document()->clearPageUserSheet();
+            frame->document()->updatePageUserSheet();
     }
 }
 
@@ -710,11 +778,23 @@ void Page::setDebugger(JSC::Debugger* debugger)
         frame->script()->attachDebugger(m_debugger);
 }
 
+SharedGraphicsContext3D* Page::sharedGraphicsContext3D()
+{
+#if ENABLE(ACCELERATED_2D_CANVAS)
+    if (!m_sharedGraphicsContext3D)
+        m_sharedGraphicsContext3D = SharedGraphicsContext3D::create(chrome());
+
+    return m_sharedGraphicsContext3D.get();
+#else
+    return 0;
+#endif
+}
+
 #if ENABLE(DOM_STORAGE)
 StorageNamespace* Page::sessionStorage(bool optionalCreate)
 {
     if (!m_sessionStorage && optionalCreate)
-        m_sessionStorage = StorageNamespace::sessionStorageNamespace(this);
+        m_sessionStorage = StorageNamespace::sessionStorageNamespace(this, m_settings->sessionStorageQuota());
 
     return m_sessionStorage.get();
 }
@@ -775,21 +855,29 @@ bool Page::javaScriptURLsAreAllowed() const
     return m_javaScriptURLsAreAllowed;
 }
 
-#if ENABLE(INSPECTOR)
-InspectorTimelineAgent* Page::inspectorTimelineAgent() const
+#if ENABLE(INPUT_SPEECH)
+SpeechInput* Page::speechInput()
 {
-    return m_inspectorController->timelineAgent();
+    ASSERT(m_speechInputClient);
+    if (!m_speechInput.get())
+        m_speechInput.set(new SpeechInput(m_speechInputClient));
+    return m_speechInput.get();
 }
 #endif
 
+void Page::dnsPrefetchingStateChanged()
+{
+    for (Frame* frame = mainFrame(); frame; frame = frame->tree()->traverseNext())
+        frame->document()->initDNSPrefetch();
+}
+
 void Page::privateBrowsingStateChanged()
 {
     bool privateBrowsingEnabled = m_settings->privateBrowsingEnabled();
 
     // Collect the PluginViews in to a vector to ensure that action the plug-in takes
     // from below privateBrowsingStateChanged does not affect their lifetime.
-
-    Vector<RefPtr<PluginView>, 32> pluginViews;
+    Vector<RefPtr<PluginViewBase>, 32> pluginViewBases;
     for (Frame* frame = mainFrame(); frame; frame = frame->tree()->traverseNext()) {
         FrameView* view = frame->view();
         if (!view)
@@ -801,14 +889,13 @@ void Page::privateBrowsingStateChanged()
         HashSet<RefPtr<Widget> >::const_iterator end = children->end();
         for (HashSet<RefPtr<Widget> >::const_iterator it = children->begin(); it != end; ++it) {
             Widget* widget = (*it).get();
-            if (!widget->isPluginView())
-                continue;
-            pluginViews.append(static_cast<PluginView*>(widget));
+            if (widget->isPluginViewBase())
+                pluginViewBases.append(static_cast<PluginViewBase*>(widget));
         }
     }
 
-    for (size_t i = 0; i < pluginViews.size(); i++)
-        pluginViews[i]->privateBrowsingStateChanged(privateBrowsingEnabled);
+    for (size_t i = 0; i < pluginViewBases.size(); ++i)
+        pluginViewBases[i]->privateBrowsingStateChanged(privateBrowsingEnabled);
 }
 
 void Page::pluginAllowedRunTimeChanged()
@@ -841,4 +928,23 @@ void Page::checkFrameCountConsistency() const
     ASSERT(m_frameCount + 1 == frameCount);
 }
 #endif
+
+Page::PageClients::PageClients()
+    : chromeClient(0)
+    , contextMenuClient(0)
+    , editorClient(0)
+    , dragClient(0)
+    , inspectorClient(0)
+    , pluginHalterClient(0)
+    , geolocationClient(0)
+    , deviceMotionClient(0)
+    , deviceOrientationClient(0)
+    , speechInputClient(0)
+{
+}
+
+Page::PageClients::~PageClients()
+{
+}
+
 } // namespace WebCore
