@@ -31,6 +31,8 @@
 
 namespace WebCore {
 
+class Cursor;
+
 class HostWindow : public Noncopyable {
 public:
     virtual ~HostWindow() { }
@@ -46,7 +48,12 @@ public:
 
     // Requests the host invalidate the contents, not the window.  This is the slow path for scrolling.
     virtual void invalidateContentsForSlowScroll(const IntRect& updateRect, bool immediate) = 0;
-    
+
+#if ENABLE(TILED_BACKING_STORE)
+    // Requests the host to do the actual scrolling. This is only used in combination with a tiled backing store.
+    virtual void delegatedScrollRequested(const IntSize& scrollDelta) = 0;
+#endif
+
     // Methods for doing coordinate conversions to and from screen coordinates.
     virtual IntPoint screenToWindow(const IntPoint&) const = 0;
     virtual IntRect windowToScreen(const IntRect&) const = 0;
@@ -54,12 +61,11 @@ public:
     // Method for retrieving the native client of the page.
     virtual PlatformPageClient platformPageClient() const = 0;
     
-    // For scrolling a rect into view recursively.  Useful in the cases where a WebView is embedded inside some containing
-    // platform-specific ScrollView.
-    virtual void scrollRectIntoView(const IntRect&, const ScrollView*) const = 0;
-
     // To notify WebKit of scrollbar mode changes.
     virtual void scrollbarsModeDidChange() const = 0;
+
+    // Request that the cursor change.
+    virtual void setCursor(const Cursor&) = 0;
 };
 
 } // namespace WebCore
