@@ -29,11 +29,11 @@
 
 #include "KURL.h"
 
-#include "CString.h"
-#include "StringHash.h"
 #include "TextEncoding.h"
+#include <wtf/text/CString.h>
 #include <wtf/HashMap.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/text/StringHash.h>
 
 #if USE(ICU_UNICODE)
 #include <unicode/uidna.h>
@@ -551,12 +551,12 @@ String KURL::lastPathComponent() const
     if (!hasPath())
         return String();
 
-    int end = m_pathEnd - 1;
+    unsigned end = m_pathEnd - 1;
     if (m_string[end] == '/')
         --end;
 
-    int start = m_string.reverseFind('/', end);
-    if (start < m_portEnd)
+    size_t start = m_string.reverseFind('/', end);
+    if (start < static_cast<unsigned>(m_portEnd))
         return String();
     ++start;
 
@@ -616,6 +616,33 @@ bool KURL::hasFragmentIdentifier() const
     return m_fragmentEnd != m_queryEnd;
 }
 
+void KURL::copyParsedQueryTo(ParsedURLParameters& parameters) const
+{
+    const UChar* pos = m_string.characters() + m_pathEnd + 1;
+    const UChar* end = m_string.characters() + m_queryEnd;
+    while (pos < end) {
+        const UChar* parameterStart = pos;
+        while (pos < end && *pos != '&')
+            ++pos;
+        const UChar* parameterEnd = pos;
+        if (pos < end) {
+            ASSERT(*pos == '&');
+            ++pos;
+        }
+        if (parameterStart == parameterEnd)
+            continue;
+        const UChar* nameStart = parameterStart;
+        const UChar* equalSign = parameterStart;
+        while (equalSign < parameterEnd && *equalSign != '=')
+            ++equalSign;
+        if (equalSign == nameStart)
+            continue;
+        String name(nameStart, equalSign - nameStart);
+        String value = equalSign == parameterEnd ? String() : String(equalSign + 1, parameterEnd - equalSign - 1);
+        parameters.set(name, value);
+    }
+}
+
 String KURL::baseAsString() const
 {
     return m_string.left(m_pathAfterLastSlash);
@@ -675,7 +702,7 @@ String KURL::path() const
 bool KURL::setProtocol(const String& s)
 {
     // Firefox and IE remove everything after the first ':'.
-    int separatorPosition = s.find(':');
+    size_t separatorPosition = s.find(':');
     String newProtocol = s.substring(0, separatorPosition);
 
     if (!isValidProtocol(newProtocol))
@@ -882,13 +909,13 @@ String decodeURLEscapeSequences(const String& str, const TextEncoding& encoding)
 
     CharBuffer buffer;
 
-    int length = str.length();
-    int decodedPosition = 0;
-    int searchPosition = 0;
-    int encodedRunPosition;
-    while ((encodedRunPosition = str.find('%', searchPosition)) >= 0) {
+    unsigned length = str.length();
+    unsigned decodedPosition = 0;
+    unsigned searchPosition = 0;
+    size_t encodedRunPosition;
+    while ((encodedRunPosition = str.find('%', searchPosition)) != notFound) {
         // Find the sequence of %-escape codes.
-        int encodedRunEnd = encodedRunPosition;
+        unsigned encodedRunEnd = encodedRunPosition;
         while (length - encodedRunEnd >= 3
                 && str[encodedRunEnd] == '%'
                 && isASCIIHexDigit(str[encodedRunEnd + 1])
@@ -1353,27 +1380,29 @@ bool protocolHostAndPortAreEqual(const KURL& a, const KURL& b)
 {
     if (a.m_schemeEnd != b.m_schemeEnd)
         return false;
+
     int hostStartA = a.hostStart();
+    int hostLengthA = a.hostEnd() - hostStartA;
     int hostStartB = b.hostStart();
-    if (a.m_hostEnd - hostStartA != b.m_hostEnd - hostStartB)
+    int hostLengthB = b.hostEnd() - b.hostStart();
+    if (hostLengthA != hostLengthB)
         return false;
 
     // Check the scheme
     for (int i = 0; i < a.m_schemeEnd; ++i)
         if (a.string()[i] != b.string()[i])
             return false;
-    
+
     // And the host
-    for (int i = hostStartA; i < a.m_hostEnd; ++i)
-        if (a.string()[i] != b.string()[i])
+    for (int i = 0; i < hostLengthA; ++i)
+        if (a.string()[hostStartA + i] != b.string()[hostStartB + i])
             return false;
-    
+
     if (a.port() != b.port())
         return false;
 
     return true;
 }
-    
 
 String encodeWithURLEscapeSequences(const String& notEncodedString)
 {
@@ -1612,13 +1641,13 @@ static void encodeRelativeString(const String& rel, const TextEncoding& encoding
 
 static String substituteBackslashes(const String& string)
 {
-    int questionPos = string.find('?');
-    int hashPos = string.find('#');
-    int pathEnd;
+    size_t questionPos = string.find('?');
+    size_t hashPos = string.find('#');
+    unsigned pathEnd;
 
-    if (hashPos >= 0 && (questionPos < 0 || questionPos > hashPos))
+    if (hashPos != notFound && (questionPos == notFound || questionPos > hashPos))
         pathEnd = hashPos;
-    else if (questionPos >= 0)
+    else if (questionPos != notFound)
         pathEnd = questionPos;
     else
         pathEnd = string.length();
@@ -1767,7 +1796,7 @@ bool portAllowed(const KURL& url)
         6669, // Alternate IRC [Apple addition]
         invalidPortNumber, // Used to block all invalid port numbers
     };
-    const unsigned short* const blockedPortListEnd = blockedPortList + sizeof(blockedPortList) / sizeof(blockedPortList[0]);
+    const unsigned short* const blockedPortListEnd = blockedPortList + WTF_ARRAY_LENGTH(blockedPortList);
 
 #ifndef NDEBUG
     // The port list must be sorted for binary_search to work.
@@ -1797,13 +1826,12 @@ bool portAllowed(const KURL& url)
 String mimeTypeFromDataURL(const String& url)
 {
     ASSERT(protocolIs(url, "data"));
-    int index = url.find(';');
-    if (index == -1)
+    size_t index = url.find(';');
+    if (index == notFound)
         index = url.find(',');
-    if (index != -1) {
-        int len = index - 5;
-        if (len > 0)
-            return url.substring(5, len);
+    if (index != notFound) {
+        if (index > 5)
+            return url.substring(5, index - 5);
         return "text/plain"; // Data URLs with no MIME type are considered text/plain.
     }
     return "";
