@@ -29,10 +29,10 @@
 
 #include "ArchiveFactory.h"
 #include "MediaPlayer.h"
-#include "StringHash.h"
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/text/StringHash.h>
 
 #if PLATFORM(CG)
 #include "ImageSourceCG.h"
@@ -52,8 +52,9 @@ static HashSet<String>* supportedImageMIMETypesForEncoding;
 static HashSet<String>* supportedJavaScriptMIMETypes;
 static HashSet<String>* supportedNonImageMIMETypes;
 static HashSet<String>* supportedMediaMIMETypes;
-static HashMap<String, String, CaseFoldingHash>* mediaMIMETypeForExtensionMap;
 
+typedef HashMap<String, Vector<String>*, CaseFoldingHash> MediaMIMETypeMap;
+    
 static void initializeSupportedImageMIMETypes()
 {
 #if PLATFORM(CG)
@@ -101,12 +102,11 @@ static void initializeSupportedImageMIMETypes()
             continue;
 #endif
         String mimeType = MIMETypeRegistry::getMIMETypeForExtension(formats.at(i).constData());
-        supportedImageMIMETypes->add(mimeType);
-        supportedImageResourceMIMETypes->add(mimeType);
+        if (!mimeType.isEmpty()) {
+            supportedImageMIMETypes->add(mimeType);
+            supportedImageResourceMIMETypes->add(mimeType);
+        }
     }
-
-    supportedImageMIMETypes->remove("application/octet-stream");
-    supportedImageResourceMIMETypes->remove("application/octet-stream");
 #else
     // assume that all implementations at least support the following standard
     // image types:
@@ -119,7 +119,7 @@ static void initializeSupportedImageMIMETypes()
         "image/x-icon",    // ico
         "image/x-xbitmap"  // xbm
     };
-    for (size_t i = 0; i < sizeof(types) / sizeof(types[0]); ++i) {
+    for (size_t i = 0; i < WTF_ARRAY_LENGTH(types); ++i) {
         supportedImageMIMETypes->add(types[i]);
         supportedImageResourceMIMETypes->add(types[i]);
     }
@@ -151,10 +151,15 @@ static void initializeSupportedImageMIMETypesForEncoding()
     QList<QByteArray> formats = QImageWriter::supportedImageFormats();
     for (int i = 0; i < formats.size(); ++i) {
         String mimeType = MIMETypeRegistry::getMIMETypeForExtension(formats.at(i).constData());
-        supportedImageMIMETypesForEncoding->add(mimeType);
+        if (!mimeType.isEmpty())
+            supportedImageMIMETypesForEncoding->add(mimeType);
     }
-
-    supportedImageMIMETypesForEncoding->remove("application/octet-stream");
+#elif PLATFORM(GTK)
+    supportedImageMIMETypesForEncoding->add("image/png");
+    supportedImageMIMETypesForEncoding->add("image/jpeg");
+    supportedImageMIMETypesForEncoding->add("image/tiff");
+    supportedImageMIMETypesForEncoding->add("image/bmp");
+    supportedImageMIMETypesForEncoding->add("image/ico");
 #elif PLATFORM(CAIRO)
     supportedImageMIMETypesForEncoding->add("image/png");
 #endif
@@ -182,7 +187,7 @@ static void initializeSupportedJavaScriptMIMETypes()
         "text/jscript",
         "text/livescript",
     };
-    for (size_t i = 0; i < sizeof(types) / sizeof(types[0]); ++i)
+    for (size_t i = 0; i < WTF_ARRAY_LENGTH(types); ++i)
       supportedJavaScriptMIMETypes->add(types[i]);
 }
 
@@ -200,9 +205,7 @@ static void initializeSupportedNonImageMimeTypes()
         "text/",
         "application/xml",
         "application/xhtml+xml",
-#if ENABLE(XHTMLMP)
         "application/vnd.wap.xhtml+xml",
-#endif
         "application/rss+xml",
         "application/atom+xml",
         "application/json",
@@ -219,13 +222,13 @@ static void initializeSupportedNonImageMimeTypes()
     COMPILE_ASSERT(sizeof(types) / sizeof(types[0]) <= 16,
                    nonimage_mime_types_must_be_less_than_or_equal_to_16);
 
-    for (size_t i = 0; i < sizeof(types)/sizeof(types[0]); ++i)
+    for (size_t i = 0; i < WTF_ARRAY_LENGTH(types); ++i)
         supportedNonImageMIMETypes->add(types[i]);
 
     ArchiveFactory::registerKnownArchiveMIMETypes();
 }
 
-static void initializeMediaTypeMaps()
+static MediaMIMETypeMap& mediaMIMETypeMap()
 {
     struct TypeExtensionPair {
         const char* type;
@@ -233,11 +236,7 @@ static void initializeMediaTypeMaps()
     };
 
     // A table of common media MIME types and file extenstions used when a platform's
-    // specific MIME type lookup doens't have a match for a media file extension. While some
-    // file extensions are claimed by multiple MIME types, this table only includes one 
-    // for each because it is currently only used by getMediaMIMETypeForExtension. If we
-    // ever add a MIME type -> file extension mapping, the alternate MIME types will need
-    // to be added.
+    // specific MIME type lookup doesn't have a match for a media file extension.
     static const TypeExtensionPair pairs[] = {
     
         // Ogg
@@ -252,6 +251,10 @@ static void initializeMediaTypeMaps()
         { "video/annodex", "axv" },
         { "audio/speex", "spx" },
 
+        // WebM
+        { "video/webm", "webm" },
+        { "audio/webm", "webm" },
+
         // MPEG
         { "audio/mpeg", "m1a" },
         { "audio/mpeg", "m2a" },
@@ -268,17 +271,25 @@ static void initializeMediaTypeMaps()
         { "video/mpeg", "mpv" },
 
         // MPEG playlist
-        { "audio/x-mpegurl", "m3url" },
+        { "application/vnd.apple.mpegurl", "m3u8" },
+        { "application/mpegurl", "m3u8" },
         { "application/x-mpegurl", "m3u8" },
+        { "audio/mpegurl", "m3url" },
+        { "audio/x-mpegurl", "m3url" },
+        { "audio/mpegurl", "m3u" },
+        { "audio/x-mpegurl", "m3u" },
 
         // MPEG-4
         { "video/x-m4v", "m4v" },
         { "audio/x-m4a", "m4a" },
         { "audio/x-m4b", "m4b" },
         { "audio/x-m4p", "m4p" },
+        { "audio/mp4", "m4a" },
  
         // MP3
         { "audio/mp3", "mp3" },
+        { "audio/x-mp3", "mp3" },
+        { "audio/x-mpeg", "mp3" },
 
         // MPEG-2
         { "video/x-mpeg2", "mp2" },
@@ -300,26 +311,69 @@ static void initializeMediaTypeMaps()
 
         // CoreAudio File
         { "audio/x-caf", "caf" },
-        { "audio/x-gsm", "gsm" }
+        { "audio/x-gsm", "gsm" },
+
+        // ADPCM
+        { "audio/x-wav", "wav" }
     };
 
-    mediaMIMETypeForExtensionMap = new HashMap<String, String, CaseFoldingHash>;
+    DEFINE_STATIC_LOCAL(MediaMIMETypeMap, mediaMIMETypeForExtensionMap, ());
+
+    if (!mediaMIMETypeForExtensionMap.isEmpty())
+        return mediaMIMETypeForExtensionMap;
+
     const unsigned numPairs = sizeof(pairs) / sizeof(pairs[0]);
-    for (unsigned ndx = 0; ndx < numPairs; ++ndx)
-        mediaMIMETypeForExtensionMap->set(pairs[ndx].extension, pairs[ndx].type);
+    for (unsigned ndx = 0; ndx < numPairs; ++ndx) {
+
+        if (mediaMIMETypeForExtensionMap.contains(pairs[ndx].extension))
+            mediaMIMETypeForExtensionMap.get(pairs[ndx].extension)->append(pairs[ndx].type);
+        else {
+            Vector<String>* synonyms = new Vector<String>;
+
+            // If there is a system specific type for this extension, add it as the first type so
+            // getMediaMIMETypeForExtension will always return it.
+            String systemType = MIMETypeRegistry::getMIMETypeForExtension(pairs[ndx].extension);
+            if (!systemType.isEmpty() && pairs[ndx].type != systemType)
+                synonyms->append(systemType);
+            synonyms->append(pairs[ndx].type);
+            mediaMIMETypeForExtensionMap.add(pairs[ndx].extension, synonyms);
+        }
+    }
+
+    return mediaMIMETypeForExtensionMap;
 }
 
 String MIMETypeRegistry::getMediaMIMETypeForExtension(const String& ext)
 {
-    // Check with system specific implementation first.
-    String mimeType = getMIMETypeForExtension(ext);
-    if (!mimeType.isEmpty())
-        return mimeType;
+    // Look in the system-specific registry first.
+    String type = getMIMETypeForExtension(ext);
+    if (!type.isEmpty())
+        return type;
 
-    // No match, look in the static mapping.
-    if (!mediaMIMETypeForExtensionMap)
-        initializeMediaTypeMaps();
-    return mediaMIMETypeForExtensionMap->get(ext);
+    Vector<String>* typeList = mediaMIMETypeMap().get(ext);
+    if (typeList)
+        return (*typeList)[0];
+    
+    return String();
+}
+    
+Vector<String> MIMETypeRegistry::getMediaMIMETypesForExtension(const String& ext)
+{
+    Vector<String>* typeList = mediaMIMETypeMap().get(ext);
+    if (typeList)
+        return *typeList;
+
+    // Only need to look in the system-specific registry if mediaMIMETypeMap() doesn't contain
+    // the extension at all, because it always contains the system-specific type if the
+    // extension is in the static mapping table.
+    String type = getMIMETypeForExtension(ext);
+    if (!type.isEmpty()) {
+        Vector<String> typeList;
+        typeList.append(type);
+        return typeList;
+    }
+    
+    return Vector<String>();
 }
 
 static void initializeSupportedMediaMIMETypes()
@@ -345,8 +399,8 @@ static void initializeMIMETypeRegistry()
 
 String MIMETypeRegistry::getMIMETypeForPath(const String& path)
 {
-    int pos = path.reverseFind('.');
-    if (pos >= 0) {
+    size_t pos = path.reverseFind('.');
+    if (pos != notFound) {
         String extension = path.substring(pos + 1);
         String result = getMIMETypeForExtension(extension);
         if (result.length())
@@ -375,6 +429,8 @@ bool MIMETypeRegistry::isSupportedImageResourceMIMEType(const String& mimeType)
 
 bool MIMETypeRegistry::isSupportedImageMIMETypeForEncoding(const String& mimeType)
 {
+    ASSERT(isMainThread());
+
     if (mimeType.isEmpty())
         return false;
     if (!supportedImageMIMETypesForEncoding)
