@@ -34,6 +34,7 @@
 #include "ScrollbarTheme.h"
 #include <wtf/StdLibExtras.h>
 
+
 using std::max;
 
 namespace WebCore {
@@ -41,6 +42,8 @@ namespace WebCore {
 ScrollView::ScrollView()
     : m_horizontalScrollbarMode(ScrollbarAuto)
     , m_verticalScrollbarMode(ScrollbarAuto)
+    , m_horizontalScrollbarLock(false)
+    , m_verticalScrollbarLock(false)
     , m_prohibitsScrolling(false)
     , m_canBlitOnScroll(true)
     , m_scrollbarsAvoidingResizer(0)
@@ -50,6 +53,7 @@ ScrollView::ScrollView()
     , m_drawPanScrollIcon(false)
     , m_useFixedLayout(false)
     , m_paintsEntireContents(false)
+    , m_delegatesScrolling(false)
 {
     platformInit();
 }
@@ -80,6 +84,9 @@ void ScrollView::removeChild(Widget* child)
 
 void ScrollView::setHasHorizontalScrollbar(bool hasBar)
 {
+    if (hasBar && avoidScrollbarCreation())
+        return;
+
     if (hasBar && !m_horizontalScrollbar) {
         m_horizontalScrollbar = createScrollbar(HorizontalScrollbar);
         addChild(m_horizontalScrollbar.get());
@@ -92,6 +99,9 @@ void ScrollView::setHasHorizontalScrollbar(bool hasBar)
 
 void ScrollView::setHasVerticalScrollbar(bool hasBar)
 {
+    if (hasBar && avoidScrollbarCreation())
+        return;
+
     if (hasBar && !m_verticalScrollbar) {
         m_verticalScrollbar = createScrollbar(VerticalScrollbar);
         addChild(m_verticalScrollbar.get());
@@ -108,12 +118,30 @@ PassRefPtr<Scrollbar> ScrollView::createScrollbar(ScrollbarOrientation orientati
     return Scrollbar::createNativeScrollbar(this, orientation, RegularScrollbar);
 }
 
-void ScrollView::setScrollbarModes(ScrollbarMode horizontalMode, ScrollbarMode verticalMode)
+void ScrollView::setScrollbarModes(ScrollbarMode horizontalMode, ScrollbarMode verticalMode,
+                                   bool horizontalLock, bool verticalLock)
 {
-    if (horizontalMode == horizontalScrollbarMode() && verticalMode == verticalScrollbarMode())
+    bool needsUpdate = false;
+
+    if (horizontalMode != horizontalScrollbarMode() && !m_horizontalScrollbarLock) {
+        m_horizontalScrollbarMode = horizontalMode;
+        needsUpdate = true;
+    }
+
+    if (verticalMode != verticalScrollbarMode() && !m_verticalScrollbarLock) {
+        m_verticalScrollbarMode = verticalMode;
+        needsUpdate = true;
+    }
+
+    if (horizontalLock)
+        setHorizontalScrollbarLock();
+
+    if (verticalLock)
+        setVerticalScrollbarLock();
+
+    if (!needsUpdate)
         return;
-    m_horizontalScrollbarMode = horizontalMode;
-    m_verticalScrollbarMode = verticalMode;
+
     if (platformWidget())
         platformSetScrollbarModes();
     else
@@ -174,11 +202,20 @@ void ScrollView::setPaintsEntireContents(bool paintsEntireContents)
     m_paintsEntireContents = paintsEntireContents;
 }
 
+void ScrollView::setDelegatesScrolling(bool delegatesScrolling)
+{
+    m_delegatesScrolling = delegatesScrolling;
+}
+
 #if !PLATFORM(GTK)
 IntRect ScrollView::visibleContentRect(bool includeScrollbars) const
 {
     if (platformWidget())
         return platformVisibleContentRect(includeScrollbars);
+
+    if (paintsEntireContents())
+        return IntRect(IntPoint(0, 0), contentsSize());
+
     return IntRect(IntPoint(m_scrollOffset.width(), m_scrollOffset.height()),
                    IntSize(max(0, width() - (verticalScrollbar() && !includeScrollbars ? verticalScrollbar()->width() : 0)), 
                            max(0, height() - (horizontalScrollbar() && !includeScrollbars ? horizontalScrollbar()->height() : 0))));
@@ -241,9 +278,35 @@ void ScrollView::setContentsSize(const IntSize& newSize)
 
 IntPoint ScrollView::maximumScrollPosition() const
 {
-    IntSize maximumOffset = contentsSize() - visibleContentRect().size();
+    IntPoint maximumOffset(contentsWidth() - visibleWidth() - m_scrollOrigin.x(), contentsHeight() - visibleHeight() - m_scrollOrigin.y());
     maximumOffset.clampNegativeToZero();
-    return IntPoint(maximumOffset.width(), maximumOffset.height());
+    return maximumOffset;
+}
+
+IntPoint ScrollView::minimumScrollPosition() const
+{
+    return IntPoint(-m_scrollOrigin.x(), -m_scrollOrigin.y());
+}
+
+IntPoint ScrollView::adjustScrollPositionWithinRange(const IntPoint& scrollPoint) const
+{
+    IntPoint newScrollPosition = scrollPoint.shrunkTo(maximumScrollPosition());
+    newScrollPosition = newScrollPosition.expandedTo(minimumScrollPosition());
+    return newScrollPosition;
+}
+
+int ScrollView::scrollSize(ScrollbarOrientation orientation) const
+{
+    Scrollbar* scrollbar = ((orientation == HorizontalScrollbar) ? m_horizontalScrollbar : m_verticalScrollbar).get();
+    return scrollbar ? (scrollbar->totalSize() - scrollbar->visibleSize()) : 0;
+}
+
+void ScrollView::setScrollOffsetFromAnimation(const IntPoint& offset)
+{
+    if (m_horizontalScrollbar)
+        m_horizontalScrollbar->setValue(offset.x(), Scrollbar::FromScrollAnimator);
+    if (m_verticalScrollbar)
+        m_verticalScrollbar->setValue(offset.y(), Scrollbar::FromScrollAnimator);
 }
 
 void ScrollView::valueChanged(Scrollbar* scrollbar)
@@ -252,9 +315,9 @@ void ScrollView::valueChanged(Scrollbar* scrollbar)
     IntSize newOffset = m_scrollOffset;
     if (scrollbar) {
         if (scrollbar->orientation() == HorizontalScrollbar)
-            newOffset.setWidth(scrollbar->value());
+            newOffset.setWidth(scrollbar->value() - m_scrollOrigin.x());
         else if (scrollbar->orientation() == VerticalScrollbar)
-            newOffset.setHeight(scrollbar->value());
+            newOffset.setHeight(scrollbar->value() - m_scrollOrigin.y());
     }
 
     IntSize scrollDelta = newOffset - m_scrollOffset;
@@ -265,33 +328,17 @@ void ScrollView::valueChanged(Scrollbar* scrollbar)
     if (scrollbarsSuppressed())
         return;
 
+    repaintFixedElementsAfterScrolling();
     scrollContents(scrollDelta);
 }
 
-void ScrollView::scrollRectIntoViewRecursively(const IntRect& r)
+void ScrollView::valueChanged(const IntSize& scrollDelta)
 {
-    // FIXME: This method is not transform-aware.  It should just be moved to FrameView so that an accurate
-    // position for the child view can be determined.
-    IntRect rect = r;
-    ScrollView* view = this;
-    while (view) {
-        if (view->prohibitsScrolling()) // Allow the views to scroll into view recursively until we hit one that prohibits scrolling.
-            return;
-        view->setScrollPosition(rect.location());
-        rect.move(view->x() - view->scrollOffset().width(), view->y() - view->scrollOffset().height());
-        if (view->parent())
-            rect.intersect(view->frameRect());
-        view = view->parent();
-    }
-    
-    // We may be embedded inside some containing platform scroll view that we don't manage.  This is the case
-    // in Mail.app on OS X, for example, where the WebKit view for message bodies is inside a Cocoa NSScrollView
-    // that contains both it and message headers.  Let the HostWindow know about this scroll so that it can pass the message
-    // on up the view chain.
-    // This rect is not clamped, since Mail actually relies on receiving an unclamped rect with negative coordinates in order to
-    // expose the headers.
-    if (hostWindow())
-        hostWindow()->scrollRectIntoView(rect, this);
+    if (scrollbarsSuppressed())
+        return;
+
+    repaintFixedElementsAfterScrolling();
+    scrollContents(scrollDelta);
 }
 
 void ScrollView::setScrollPosition(const IntPoint& scrollPoint)
@@ -304,8 +351,16 @@ void ScrollView::setScrollPosition(const IntPoint& scrollPoint)
         return;
     }
 
-    IntPoint newScrollPosition = scrollPoint.shrunkTo(maximumScrollPosition());
-    newScrollPosition.clampNegativeToZero();
+#if ENABLE(TILED_BACKING_STORE)
+    if (delegatesScrolling()) {
+        hostWindow()->delegatedScrollRequested(IntSize(scrollPoint.x(), scrollPoint.y()));
+        if (!m_actualVisibleContentRect.isEmpty())
+            m_actualVisibleContentRect.setLocation(scrollPoint);
+        return;
+    }
+#endif
+
+    IntPoint newScrollPosition = adjustScrollPositionWithinRange(scrollPoint);
 
     if (newScrollPosition == scrollPosition())
         return;
@@ -317,7 +372,7 @@ bool ScrollView::scroll(ScrollDirection direction, ScrollGranularity granularity
 {
     if (platformWidget())
         return platformScroll(direction, granularity);
-
+    
     if (direction == ScrollUp || direction == ScrollDown) {
         if (m_verticalScrollbar)
             return m_verticalScrollbar->scroll(direction, granularity);
@@ -328,11 +383,24 @@ bool ScrollView::scroll(ScrollDirection direction, ScrollGranularity granularity
     return false;
 }
 
+bool ScrollView::logicalScroll(ScrollLogicalDirection direction, ScrollGranularity granularity)
+{
+    return scroll(logicalToPhysical(direction, isVerticalDocument(), isFlippedDocument()), granularity);
+}
+
+void ScrollView::windowResizerRectChanged()
+{
+    if (platformWidget())
+        return;
+
+    updateScrollbars(scrollOffset());
+}
+
 static const unsigned cMaxUpdateScrollbarsPass = 2;
 
 void ScrollView::updateScrollbars(const IntSize& desiredOffset)
 {
-    if (m_inUpdateScrollbars || prohibitsScrolling() || platformWidget())
+    if (m_inUpdateScrollbars || prohibitsScrolling() || delegatesScrolling() || platformWidget())
         return;
 
     // If we came in here with the view already needing a layout, then go ahead and do that
@@ -388,11 +456,15 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
             newHasHorizontalScrollbar = false;
 
         if (hasHorizontalScrollbar != newHasHorizontalScrollbar) {
+            if (m_scrollOrigin.y() && !newHasHorizontalScrollbar)
+                m_scrollOrigin.setY(m_scrollOrigin.y() - m_horizontalScrollbar->height());
             setHasHorizontalScrollbar(newHasHorizontalScrollbar);
             sendContentResizedNotification = true;
         }
 
         if (hasVerticalScrollbar != newHasVerticalScrollbar) {
+            if (m_scrollOrigin.x() && !newHasVerticalScrollbar)
+                m_scrollOrigin.setX(m_scrollOrigin.x() - m_verticalScrollbar->width());
             setHasVerticalScrollbar(newHasVerticalScrollbar);
             sendContentResizedNotification = true;
         }
@@ -418,10 +490,10 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
         return;
 
     m_inUpdateScrollbars = true;
-    IntSize maxScrollPosition(contentsWidth() - visibleWidth(), contentsHeight() - visibleHeight());
-    IntSize scroll = desiredOffset.shrunkTo(maxScrollPosition);
-    scroll.clampNegativeToZero();
- 
+
+    IntPoint scrollPoint = adjustScrollPositionWithinRange(IntPoint(desiredOffset.width(), desiredOffset.height()));
+    IntSize scroll(scrollPoint.x(), scrollPoint.y());
+
     if (m_horizontalScrollbar) {
         int clientWidth = visibleWidth();
         m_horizontalScrollbar->setEnabled(contentsWidth() > clientWidth);
@@ -439,7 +511,7 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
             m_horizontalScrollbar->setSuppressInvalidation(true);
         m_horizontalScrollbar->setSteps(Scrollbar::pixelsPerLineStep(), pageStep);
         m_horizontalScrollbar->setProportion(clientWidth, contentsWidth());
-        m_horizontalScrollbar->setValue(scroll.width());
+        m_horizontalScrollbar->setValue(scroll.width() + m_scrollOrigin.x(), Scrollbar::NotFromScrollAnimator);
         if (m_scrollbarsSuppressed)
             m_horizontalScrollbar->setSuppressInvalidation(false); 
     } 
@@ -461,7 +533,7 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
             m_verticalScrollbar->setSuppressInvalidation(true);
         m_verticalScrollbar->setSteps(Scrollbar::pixelsPerLineStep(), pageStep);
         m_verticalScrollbar->setProportion(clientHeight, contentsHeight());
-        m_verticalScrollbar->setValue(scroll.height());
+        m_verticalScrollbar->setValue(scroll.height() + m_scrollOrigin.y(), Scrollbar::NotFromScrollAnimator);
         if (m_scrollbarsSuppressed)
             m_verticalScrollbar->setSuppressInvalidation(false);
     }
@@ -478,7 +550,7 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
     IntSize scrollDelta = scroll - m_scrollOffset;
     if (scrollDelta != IntSize()) {
        m_scrollOffset = scroll;
-       scrollContents(scrollDelta);
+       valueChanged(scrollDelta);
     }
 
     m_inUpdateScrollbars = false;
@@ -502,6 +574,8 @@ void ScrollView::scrollContents(const IntSize& scrollDelta)
     hostWindow()->invalidateWindow(updateRect, false /*immediate*/);
 
     if (m_drawPanScrollIcon) {
+        // FIXME: the pan icon is broken when accelerated compositing is on, since it will draw under the compositing layers.
+        // https://bugs.webkit.org/show_bug.cgi?id=47837
         int panIconDirtySquareSizeLength = 2 * (panIconSizeLength + max(abs(scrollDelta.width()), abs(scrollDelta.height()))); // We only want to repaint what's necessary
         IntPoint panIconDirtySquareLocation = IntPoint(m_panScrollIconPoint.x() - (panIconDirtySquareSizeLength / 2), m_panScrollIconPoint.y() - (panIconDirtySquareSizeLength / 2));
         IntRect panScrollIconDirtyRect = IntRect(panIconDirtySquareLocation , IntSize(panIconDirtySquareSizeLength, panIconDirtySquareSizeLength));
@@ -512,11 +586,11 @@ void ScrollView::scrollContents(const IntSize& scrollDelta)
     if (canBlitOnScroll()) { // The main frame can just blit the WebView window
         // FIXME: Find a way to scroll subframes with this faster path
         if (!scrollContentsFastPath(-scrollDelta, scrollViewRect, clipRect))
-            hostWindow()->invalidateContentsForSlowScroll(updateRect, false);
+            scrollContentsSlowPath(updateRect);
     } else { 
        // We need to go ahead and repaint the entire backing store.  Do it now before moving the
        // windowed plugins.
-       hostWindow()->invalidateContentsForSlowScroll(updateRect, false);
+       scrollContentsSlowPath(updateRect);
     }
 
     // This call will move children with native widgets (plugins) and invalidate them as well.
@@ -532,6 +606,11 @@ bool ScrollView::scrollContentsFastPath(const IntSize& scrollDelta, const IntRec
     return true;
 }
 
+void ScrollView::scrollContentsSlowPath(const IntRect& updateRect)
+{
+    hostWindow()->invalidateContentsForSlowScroll(updateRect, false);
+}
+
 IntPoint ScrollView::windowToContents(const IntPoint& windowPoint) const
 {
     IntPoint viewPoint = convertFromContainingWindow(windowPoint);
@@ -655,23 +734,29 @@ void ScrollView::wheelEvent(PlatformWheelEvent& e)
         return;
     }
 
-    // Determine how much we want to scroll.  If we can move at all, we will accept the event.
-    IntSize maxScrollDelta = maximumScrollPosition() - scrollPosition();
-    if ((e.deltaX() < 0 && maxScrollDelta.width() > 0) ||
-        (e.deltaX() > 0 && scrollOffset().width() > 0) ||
-        (e.deltaY() < 0 && maxScrollDelta.height() > 0) ||
-        (e.deltaY() > 0 && scrollOffset().height() > 0)) {
+    // Accept the event if we have a scrollbar in that direction and can still
+    // scroll any further.
+    float deltaX = m_horizontalScrollbar ? e.deltaX() : 0;
+    float deltaY = m_verticalScrollbar ? e.deltaY() : 0;
+    IntSize maxForwardScrollDelta = maximumScrollPosition() - scrollPosition();
+    IntSize maxBackwardScrollDelta = scrollPosition() - minimumScrollPosition();
+    if ((deltaX < 0 && maxForwardScrollDelta.width() > 0)
+        || (deltaX > 0 && maxBackwardScrollDelta.width() >0)
+        || (deltaY < 0 && maxForwardScrollDelta.height() > 0)
+        || (deltaY > 0 && maxBackwardScrollDelta.height() > 0)) {
         e.accept();
-        float deltaX = e.deltaX();
-        float deltaY = e.deltaY();
         if (e.granularity() == ScrollByPageWheelEvent) {
-            ASSERT(deltaX == 0);
+            ASSERT(!e.deltaX());
             bool negative = deltaY < 0;
-            deltaY = max(max<int>(visibleHeight() * Scrollbar::minFractionToStepWhenPaging(), visibleHeight() - Scrollbar::maxOverlapBetweenPages()), 1);
+            deltaY = max(max(static_cast<float>(visibleHeight()) * Scrollbar::minFractionToStepWhenPaging(), static_cast<float>(visibleHeight() - Scrollbar::maxOverlapBetweenPages())), 1.0f);
             if (negative)
                 deltaY = -deltaY;
         }
-        scrollBy(IntSize(-deltaX, -deltaY));
+
+        if (deltaY)
+            m_verticalScrollbar->scroll(ScrollUp, ScrollByPixel, deltaY);
+        if (deltaX)
+            m_horizontalScrollbar->scroll(ScrollLeft, ScrollByPixel, deltaX);
     }
 }
 
@@ -689,7 +774,8 @@ void ScrollView::setFrameRect(const IntRect& newRect)
     
     if (newRect.width() != oldRect.width() || newRect.height() != oldRect.height()) {
         updateScrollbars(m_scrollOffset);
-        contentsResized();
+        if (!m_useFixedLayout)
+            contentsResized();
     }
 
     frameRectsChanged();
@@ -764,8 +850,8 @@ void ScrollView::paintScrollbars(GraphicsContext* context, const IntRect& rect)
 
 void ScrollView::paintPanScrollIcon(GraphicsContext* context)
 {
-    DEFINE_STATIC_LOCAL(Image*, panScrollIcon, (Image::loadPlatformResource("panIcon").releaseRef()));
-    context->drawImage(panScrollIcon, DeviceColorSpace, m_panScrollIconPoint);
+    static Image* panScrollIcon = Image::loadPlatformResource("panIcon").releaseRef();
+    context->drawImage(panScrollIcon, ColorSpaceDeviceRGB, m_panScrollIconPoint);
 }
 
 void ScrollView::paint(GraphicsContext* context, const IntRect& rect)
@@ -786,10 +872,12 @@ void ScrollView::paint(GraphicsContext* context, const IntRect& rect)
     context->translate(x(), y());
     documentDirtyRect.move(-x(), -y());
 
-    context->translate(-scrollX(), -scrollY());
-    documentDirtyRect.move(scrollX(), scrollY());
+    if (!paintsEntireContents()) {
+        context->translate(-scrollX(), -scrollY());
+        documentDirtyRect.move(scrollX(), scrollY());
 
-    context->clip(visibleContentRect());
+        context->clip(visibleContentRect());
+    }
 
     paintContents(context, documentDirtyRect);
 
@@ -948,7 +1036,24 @@ void ScrollView::removePanScrollIcon()
     hostWindow()->invalidateContentsAndWindow(IntRect(m_panScrollIconPoint, IntSize(panIconSizeLength, panIconSizeLength)), true /*immediate*/);
 }
 
-#if !PLATFORM(WX) && !PLATFORM(GTK) && !PLATFORM(QT)
+void ScrollView::setScrollOrigin(const IntPoint& origin, bool updatePosition)
+{
+    if (m_scrollOrigin == origin)
+        return;
+
+    m_scrollOrigin = origin;
+
+    if (platformWidget()) {
+        platformSetScrollOrigin(origin, updatePosition);
+        return;
+    }
+    
+    // Update if the scroll origin changes, since our position will be different if the content size did not change.
+    if (updatePosition)
+        updateScrollbars(scrollOffset());
+}
+
+#if !PLATFORM(WX) && !PLATFORM(GTK) && !PLATFORM(EFL)
 
 void ScrollView::platformInit()
 {
@@ -978,6 +1083,10 @@ void ScrollView::platformSetScrollbarsSuppressed(bool)
 {
 }
 
+void ScrollView::platformSetScrollOrigin(const IntPoint&, bool updatePosition)
+{
+}
+
 #endif
 
 #if !PLATFORM(MAC) && !PLATFORM(WX)
