@@ -96,7 +96,14 @@ void Gradient::addColorStop(float value, const Color& color)
     m_stops.append(ColorStop(value, r, g, b, a));
 
     m_stopsSorted = false;
+    platformDestroy();
+}
 
+void Gradient::addColorStop(const Gradient::ColorStop& stop)
+{
+    m_stops.append(stop);
+
+    m_stopsSorted = false;
     platformDestroy();
 }
 
@@ -110,9 +117,16 @@ void Gradient::sortStopsIfNecessary()
     if (m_stopsSorted)
         return;
 
-    if (m_stops.size())
-        std::stable_sort(m_stops.begin(), m_stops.end(), compareStops);
     m_stopsSorted = true;
+
+    if (!m_stops.size())
+        return;
+
+    // Shortcut for the ideal case (ordered 2-stop gradient)
+    if (m_stops.size() == 2 && compareStops(*m_stops.begin(), *m_stops.end()))
+        return;
+
+    std::stable_sort(m_stops.begin(), m_stops.end(), compareStops);
 }
 
 void Gradient::getColor(float value, float* r, float* g, float* b, float* a) const
@@ -182,6 +196,16 @@ int Gradient::findStop(float value) const
     return m_lastStop;
 }
 
+bool Gradient::hasAlpha() const
+{
+    for (size_t i = 0; i < m_stops.size(); i++) {
+        if (m_stops[i].alpha < 1)
+            return true;
+    }
+
+    return false;
+}
+
 void Gradient::setSpreadMethod(GradientSpreadMethod spreadMethod)
 {
     // FIXME: Should it become necessary, allow calls to this method after m_gradient has been set.
@@ -189,14 +213,14 @@ void Gradient::setSpreadMethod(GradientSpreadMethod spreadMethod)
     m_spreadMethod = spreadMethod;
 }
 
-void Gradient::setGradientSpaceTransform(const TransformationMatrix& gradientSpaceTransformation)
+void Gradient::setGradientSpaceTransform(const AffineTransform& gradientSpaceTransformation)
 { 
     m_gradientSpaceTransformation = gradientSpaceTransformation;
     setPlatformGradientSpaceTransform(gradientSpaceTransformation);
 }
 
-#if !PLATFORM(SKIA)
-void Gradient::setPlatformGradientSpaceTransform(const TransformationMatrix&)
+#if !PLATFORM(SKIA) && !PLATFORM(CAIRO)
+void Gradient::setPlatformGradientSpaceTransform(const AffineTransform&)
 {
 }
 #endif
