@@ -29,7 +29,7 @@
 #include "BidiResolver.h"
 #include "Font.h"
 #include "Generator.h"
-#include "GraphicsContextPrivate.h"
+#include "ImageBuffer.h"
 
 using namespace std;
 
@@ -73,14 +73,15 @@ private:
     int m_offset;
 };
 
-GraphicsContextPrivate* GraphicsContext::createGraphicsContextPrivate()
+GraphicsContext::GraphicsContext(PlatformGraphicsContext* platformGraphicsContext)
+    : m_updatingControlTints(false)
 {
-    return new GraphicsContextPrivate;
+    platformInit(platformGraphicsContext);
 }
 
-void GraphicsContext::destroyGraphicsContextPrivate(GraphicsContextPrivate* deleteMe)
+GraphicsContext::~GraphicsContext()
 {
-    delete deleteMe;
+    platformDestroy();
 }
 
 void GraphicsContext::save()
@@ -88,7 +89,7 @@ void GraphicsContext::save()
     if (paintingDisabled())
         return;
 
-    m_common->stack.append(m_common->state);
+    m_stack.append(m_state);
 
     savePlatformState();
 }
@@ -98,214 +99,230 @@ void GraphicsContext::restore()
     if (paintingDisabled())
         return;
 
-    if (m_common->stack.isEmpty()) {
+    if (m_stack.isEmpty()) {
         LOG_ERROR("ERROR void GraphicsContext::restore() stack is empty");
         return;
     }
-    m_common->state = m_common->stack.last();
-    m_common->stack.removeLast();
+    m_state = m_stack.last();
+    m_stack.removeLast();
 
     restorePlatformState();
 }
 
 void GraphicsContext::setStrokeThickness(float thickness)
 {
-    m_common->state.strokeThickness = thickness;
+    m_state.strokeThickness = thickness;
     setPlatformStrokeThickness(thickness);
 }
 
 void GraphicsContext::setStrokeStyle(const StrokeStyle& style)
 {
-    m_common->state.strokeStyle = style;
+    m_state.strokeStyle = style;
     setPlatformStrokeStyle(style);
 }
 
 void GraphicsContext::setStrokeColor(const Color& color, ColorSpace colorSpace)
 {
-    m_common->state.strokeColor = color;
-    m_common->state.strokeColorSpace = colorSpace;
-    m_common->state.strokeGradient.clear();
-    m_common->state.strokePattern.clear();
+    m_state.strokeColor = color;
+    m_state.strokeColorSpace = colorSpace;
+    m_state.strokeGradient.clear();
+    m_state.strokePattern.clear();
     setPlatformStrokeColor(color, colorSpace);
 }
 
-void GraphicsContext::setShadow(const IntSize& size, int blur, const Color& color, ColorSpace colorSpace)
+void GraphicsContext::setShadow(const FloatSize& offset, float blur, const Color& color, ColorSpace colorSpace)
 {
-    m_common->state.shadowSize = size;
-    m_common->state.shadowBlur = blur;
-    m_common->state.shadowColor = color;
-    setPlatformShadow(size, blur, color, colorSpace);
+    m_state.shadowOffset = offset;
+    m_state.shadowBlur = blur;
+    m_state.shadowColor = color;
+    setPlatformShadow(offset, blur, color, colorSpace);
 }
 
 void GraphicsContext::clearShadow()
 {
-    m_common->state.shadowSize = IntSize();
-    m_common->state.shadowBlur = 0;
-    m_common->state.shadowColor = Color();
+    m_state.shadowOffset = FloatSize();
+    m_state.shadowBlur = 0;
+    m_state.shadowColor = Color();
     clearPlatformShadow();
 }
 
-bool GraphicsContext::getShadow(IntSize& size, int& blur, Color& color) const
+bool GraphicsContext::getShadow(FloatSize& offset, float& blur, Color& color) const
 {
-    size = m_common->state.shadowSize;
-    blur = m_common->state.shadowBlur;
-    color = m_common->state.shadowColor;
+    offset = m_state.shadowOffset;
+    blur = m_state.shadowBlur;
+    color = m_state.shadowColor;
 
-    return color.isValid() && color.alpha() && (blur || size.width() || size.height());
+    return color.isValid() && color.alpha() && (blur || offset.width() || offset.height());
 }
 
 float GraphicsContext::strokeThickness() const
 {
-    return m_common->state.strokeThickness;
+    return m_state.strokeThickness;
 }
 
 StrokeStyle GraphicsContext::strokeStyle() const
 {
-    return m_common->state.strokeStyle;
+    return m_state.strokeStyle;
 }
 
 Color GraphicsContext::strokeColor() const
 {
-    return m_common->state.strokeColor;
+    return m_state.strokeColor;
 }
 
 ColorSpace GraphicsContext::strokeColorSpace() const
 {
-    return m_common->state.strokeColorSpace;
+    return m_state.strokeColorSpace;
 }
 
 WindRule GraphicsContext::fillRule() const
 {
-    return m_common->state.fillRule;
+    return m_state.fillRule;
 }
 
 void GraphicsContext::setFillRule(WindRule fillRule)
 {
-    m_common->state.fillRule = fillRule;
+    m_state.fillRule = fillRule;
 }
 
 void GraphicsContext::setFillColor(const Color& color, ColorSpace colorSpace)
 {
-    m_common->state.fillColor = color;
-    m_common->state.fillColorSpace = colorSpace;
-    m_common->state.fillGradient.clear();
-    m_common->state.fillPattern.clear();
+    m_state.fillColor = color;
+    m_state.fillColorSpace = colorSpace;
+    m_state.fillGradient.clear();
+    m_state.fillPattern.clear();
     setPlatformFillColor(color, colorSpace);
 }
 
 Color GraphicsContext::fillColor() const
 {
-    return m_common->state.fillColor;
+    return m_state.fillColor;
 }
 
 ColorSpace GraphicsContext::fillColorSpace() const
 {
-    return m_common->state.fillColorSpace;
+    return m_state.fillColorSpace;
 }
 
 void GraphicsContext::setShouldAntialias(bool b)
 {
-    m_common->state.shouldAntialias = b;
+    m_state.shouldAntialias = b;
     setPlatformShouldAntialias(b);
 }
 
 bool GraphicsContext::shouldAntialias() const
 {
-    return m_common->state.shouldAntialias;
+    return m_state.shouldAntialias;
+}
+
+void GraphicsContext::setShouldSmoothFonts(bool b)
+{
+    m_state.shouldSmoothFonts = b;
+    setPlatformShouldSmoothFonts(b);
+}
+
+bool GraphicsContext::shouldSmoothFonts() const
+{
+    return m_state.shouldSmoothFonts;
+}
+
+const GraphicsContextState& GraphicsContext::state() const
+{
+    return m_state;
 }
 
 void GraphicsContext::setStrokePattern(PassRefPtr<Pattern> pattern)
 {
     ASSERT(pattern);
     if (!pattern) {
-        setStrokeColor(Color::black, DeviceColorSpace);
+        setStrokeColor(Color::black, ColorSpaceDeviceRGB);
         return;
     }
-    m_common->state.strokeGradient.clear();
-    m_common->state.strokePattern = pattern;
-    setPlatformStrokePattern(m_common->state.strokePattern.get());
+    m_state.strokeGradient.clear();
+    m_state.strokePattern = pattern;
+    setPlatformStrokePattern(m_state.strokePattern.get());
 }
 
 void GraphicsContext::setFillPattern(PassRefPtr<Pattern> pattern)
 {
     ASSERT(pattern);
     if (!pattern) {
-        setFillColor(Color::black, DeviceColorSpace);
+        setFillColor(Color::black, ColorSpaceDeviceRGB);
         return;
     }
-    m_common->state.fillGradient.clear();
-    m_common->state.fillPattern = pattern;
-    setPlatformFillPattern(m_common->state.fillPattern.get());
+    m_state.fillGradient.clear();
+    m_state.fillPattern = pattern;
+    setPlatformFillPattern(m_state.fillPattern.get());
 }
 
 void GraphicsContext::setStrokeGradient(PassRefPtr<Gradient> gradient)
 {
     ASSERT(gradient);
     if (!gradient) {
-        setStrokeColor(Color::black, DeviceColorSpace);
+        setStrokeColor(Color::black, ColorSpaceDeviceRGB);
         return;
     }
-    m_common->state.strokeGradient = gradient;
-    m_common->state.strokePattern.clear();
-    setPlatformStrokeGradient(m_common->state.strokeGradient.get());
+    m_state.strokeGradient = gradient;
+    m_state.strokePattern.clear();
+    setPlatformStrokeGradient(m_state.strokeGradient.get());
 }
 
 void GraphicsContext::setFillGradient(PassRefPtr<Gradient> gradient)
 {
     ASSERT(gradient);
     if (!gradient) {
-        setFillColor(Color::black, DeviceColorSpace);
+        setFillColor(Color::black, ColorSpaceDeviceRGB);
         return;
     }
-    m_common->state.fillGradient = gradient;
-    m_common->state.fillPattern.clear();
-    setPlatformFillGradient(m_common->state.fillGradient.get());
+    m_state.fillGradient = gradient;
+    m_state.fillPattern.clear();
+    setPlatformFillGradient(m_state.fillGradient.get());
 }
 
 Gradient* GraphicsContext::fillGradient() const
 {
-    return m_common->state.fillGradient.get();
+    return m_state.fillGradient.get();
 }
 
 Gradient* GraphicsContext::strokeGradient() const
 {
-    return m_common->state.strokeGradient.get();
+    return m_state.strokeGradient.get();
 }
 
 Pattern* GraphicsContext::fillPattern() const
 {
-    return m_common->state.fillPattern.get();
+    return m_state.fillPattern.get();
 }
 
 Pattern* GraphicsContext::strokePattern() const
 {
-    return m_common->state.strokePattern.get();
+    return m_state.strokePattern.get();
 }
 
 void GraphicsContext::setShadowsIgnoreTransforms(bool ignoreTransforms)
 {
-    m_common->state.shadowsIgnoreTransforms = ignoreTransforms;
+    m_state.shadowsIgnoreTransforms = ignoreTransforms;
 }
 
 bool GraphicsContext::updatingControlTints() const
 {
-    return m_common->m_updatingControlTints;
+    return m_updatingControlTints;
 }
 
 void GraphicsContext::setUpdatingControlTints(bool b)
 {
     setPaintingDisabled(b);
-    m_common->m_updatingControlTints = b;
+    m_updatingControlTints = b;
 }
 
 void GraphicsContext::setPaintingDisabled(bool f)
 {
-    m_common->state.paintingDisabled = f;
+    m_state.paintingDisabled = f;
 }
 
 bool GraphicsContext::paintingDisabled() const
 {
-    return m_common->state.paintingDisabled;
+    return m_state.paintingDisabled;
 }
 
 void GraphicsContext::drawImage(Image* image, ColorSpace styleColorSpace, const IntPoint& p, CompositeOperator op)
@@ -338,6 +355,14 @@ void GraphicsContext::drawText(const Font& font, const TextRun& run, const IntPo
 }
 #endif
 
+void GraphicsContext::drawEmphasisMarks(const Font& font, const TextRun& run, const AtomicString& mark, const IntPoint& point, int from, int to)
+{
+    if (paintingDisabled())
+        return;
+
+    font.drawEmphasisMarks(this, run, mark, point, from, to);
+}
+
 void GraphicsContext::drawBidiText(const Font& font, const TextRun& run, const FloatPoint& point)
 {
     if (paintingDisabled())
@@ -403,25 +428,27 @@ void GraphicsContext::drawImage(Image* image, ColorSpace styleColorSpace, const
         th = image->height();
 
     if (useLowQualityScale) {
-        save();
+        InterpolationQuality previousInterpolationQuality = imageInterpolationQuality();
+        // FIXME: Should be InterpolationLow
         setImageInterpolationQuality(InterpolationNone);
-    }
-    image->draw(this, FloatRect(dest.location(), FloatSize(tw, th)), FloatRect(src.location(), FloatSize(tsw, tsh)), styleColorSpace, op);
-    if (useLowQualityScale)
-        restore();
+        image->draw(this, FloatRect(dest.location(), FloatSize(tw, th)), FloatRect(src.location(), FloatSize(tsw, tsh)), styleColorSpace, op);
+        setImageInterpolationQuality(previousInterpolationQuality);
+    } else
+        image->draw(this, FloatRect(dest.location(), FloatSize(tw, th)), FloatRect(src.location(), FloatSize(tsw, tsh)), styleColorSpace, op);
 }
 
 void GraphicsContext::drawTiledImage(Image* image, ColorSpace styleColorSpace, const IntRect& rect, const IntPoint& srcPoint, const IntSize& tileSize, CompositeOperator op, bool useLowQualityScale)
 {
     if (paintingDisabled() || !image)
         return;
+
     if (useLowQualityScale) {
-        save();
+        InterpolationQuality previousInterpolationQuality = imageInterpolationQuality();
         setImageInterpolationQuality(InterpolationLow);
-    }
-    image->drawTiled(this, rect, srcPoint, tileSize, styleColorSpace, op);
-    if (useLowQualityScale)
-        restore();
+        image->drawTiled(this, rect, srcPoint, tileSize, styleColorSpace, op);
+        setImageInterpolationQuality(previousInterpolationQuality);
+    } else
+        image->drawTiled(this, rect, srcPoint, tileSize, styleColorSpace, op);
 }
 
 void GraphicsContext::drawTiledImage(Image* image, ColorSpace styleColorSpace, const IntRect& dest, const IntRect& srcRect, Image::TileRule hRule, Image::TileRule vRule, CompositeOperator op, bool useLowQualityScale)
@@ -429,17 +456,69 @@ void GraphicsContext::drawTiledImage(Image* image, ColorSpace styleColorSpace, c
     if (paintingDisabled() || !image)
         return;
 
-    if (useLowQualityScale) {
-        save();
-        setImageInterpolationQuality(InterpolationLow);
-    }
-    if (hRule == Image::StretchTile && vRule == Image::StretchTile)
+    if (hRule == Image::StretchTile && vRule == Image::StretchTile) {
         // Just do a scale.
         drawImage(image, styleColorSpace, dest, srcRect, op);
-    else
+        return;
+    }
+
+    if (useLowQualityScale) {
+        InterpolationQuality previousInterpolationQuality = imageInterpolationQuality();
+        setImageInterpolationQuality(InterpolationLow);
         image->drawTiled(this, dest, srcRect, hRule, vRule, styleColorSpace, op);
-    if (useLowQualityScale)
-        restore();
+        setImageInterpolationQuality(previousInterpolationQuality);
+    } else
+        image->drawTiled(this, dest, srcRect, hRule, vRule, styleColorSpace, op);
+}
+
+void GraphicsContext::drawImageBuffer(ImageBuffer* image, ColorSpace styleColorSpace, const IntPoint& p, CompositeOperator op)
+{
+    drawImageBuffer(image, styleColorSpace, p, IntRect(0, 0, -1, -1), op);
+}
+
+void GraphicsContext::drawImageBuffer(ImageBuffer* image, ColorSpace styleColorSpace, const IntRect& r, CompositeOperator op, bool useLowQualityScale)
+{
+    drawImageBuffer(image, styleColorSpace, r, IntRect(0, 0, -1, -1), op, useLowQualityScale);
+}
+
+void GraphicsContext::drawImageBuffer(ImageBuffer* image, ColorSpace styleColorSpace, const IntPoint& dest, const IntRect& srcRect, CompositeOperator op)
+{
+    drawImageBuffer(image, styleColorSpace, IntRect(dest, srcRect.size()), srcRect, op);
+}
+
+void GraphicsContext::drawImageBuffer(ImageBuffer* image, ColorSpace styleColorSpace, const IntRect& dest, const IntRect& srcRect, CompositeOperator op, bool useLowQualityScale)
+{
+    drawImageBuffer(image, styleColorSpace, FloatRect(dest), srcRect, op, useLowQualityScale);
+}
+
+void GraphicsContext::drawImageBuffer(ImageBuffer* image, ColorSpace styleColorSpace, const FloatRect& dest, const FloatRect& src, CompositeOperator op, bool useLowQualityScale)
+{
+    if (paintingDisabled() || !image)
+        return;
+
+    float tsw = src.width();
+    float tsh = src.height();
+    float tw = dest.width();
+    float th = dest.height();
+
+    if (tsw == -1)
+        tsw = image->width();
+    if (tsh == -1)
+        tsh = image->height();
+
+    if (tw == -1)
+        tw = image->width();
+    if (th == -1)
+        th = image->height();
+
+    if (useLowQualityScale) {
+        InterpolationQuality previousInterpolationQuality = imageInterpolationQuality();
+        // FIXME: Should be InterpolationLow
+        setImageInterpolationQuality(InterpolationNone);
+        image->draw(this, styleColorSpace, dest, src, op, useLowQualityScale);
+        setImageInterpolationQuality(previousInterpolationQuality);
+    } else
+        image->draw(this, styleColorSpace, dest, src, op, useLowQualityScale);
 }
 
 void GraphicsContext::addRoundedRectClip(const IntRect& rect, const IntSize& topLeft, const IntSize& topRight,
@@ -448,7 +527,9 @@ void GraphicsContext::addRoundedRectClip(const IntRect& rect, const IntSize& top
     if (paintingDisabled())
         return;
 
-    clip(Path::createRoundedRectangle(rect, topLeft, topRight, bottomLeft, bottomRight));
+    Path path;
+    path.addRoundedRect(rect, topLeft, topRight, bottomLeft, bottomRight);
+    clip(path);
 }
 
 void GraphicsContext::clipOutRoundedRect(const IntRect& rect, const IntSize& topLeft, const IntSize& topRight,
@@ -457,17 +538,26 @@ void GraphicsContext::clipOutRoundedRect(const IntRect& rect, const IntSize& top
     if (paintingDisabled())
         return;
 
-    clipOut(Path::createRoundedRectangle(rect, topLeft, topRight, bottomLeft, bottomRight));
+    Path path;
+    path.addRoundedRect(rect, topLeft, topRight, bottomLeft, bottomRight);
+    clipOut(path);
 }
 
-int GraphicsContext::textDrawingMode()
+void GraphicsContext::clipToImageBuffer(ImageBuffer* buffer, const FloatRect& rect)
 {
-    return m_common->state.textDrawingMode;
+    if (paintingDisabled())
+        return;
+    buffer->clip(this, rect);
 }
 
-void GraphicsContext::setTextDrawingMode(int mode)
+TextDrawingModeFlags GraphicsContext::textDrawingMode() const
 {
-    m_common->state.textDrawingMode = mode;
+    return m_state.textDrawingMode;
+}
+
+void GraphicsContext::setTextDrawingMode(TextDrawingModeFlags mode)
+{
+    m_state.textDrawingMode = mode;
     if (paintingDisabled())
         return;
     setPlatformTextDrawingMode(mode);
@@ -480,6 +570,17 @@ void GraphicsContext::fillRect(const FloatRect& rect, Generator& generator)
     generator.fill(this, rect);
 }
 
+void GraphicsContext::setCompositeOperation(CompositeOperator compositeOperation)
+{
+    m_state.compositeOperator = compositeOperation;
+    setPlatformCompositeOperation(compositeOperation);
+}
+
+CompositeOperator GraphicsContext::compositeOperation() const
+{
+    return m_state.compositeOperator;
+}
+
 #if !PLATFORM(SKIA)
 void GraphicsContext::setPlatformFillGradient(Gradient*)
 {
@@ -501,7 +602,7 @@ void GraphicsContext::setPlatformStrokePattern(Pattern*)
 #if !PLATFORM(CG) && !PLATFORM(SKIA)
 // Implement this if you want to go ahead and push the drawing mode into your native context
 // immediately.
-void GraphicsContext::setPlatformTextDrawingMode(int mode)
+void GraphicsContext::setPlatformTextDrawingMode(TextDrawingModeFlags mode)
 {
 }
 #endif
@@ -512,6 +613,27 @@ void GraphicsContext::setPlatformStrokeStyle(const StrokeStyle&)
 }
 #endif
 
+#if !PLATFORM(CG)
+void GraphicsContext::setPlatformShouldSmoothFonts(bool)
+{
+}
+#endif
+
+#if !PLATFORM(SKIA)
+void GraphicsContext::setSharedGraphicsContext3D(SharedGraphicsContext3D*, DrawingBuffer*, const IntSize&)
+{
+}
+
+void GraphicsContext::syncSoftwareCanvas()
+{
+}
+
+void GraphicsContext::markDirtyRect(const IntRect&)
+{
+}
+#endif
+
+
 void GraphicsContext::adjustLineToPixelBoundaries(FloatPoint& p1, FloatPoint& p2, float strokeWidth, const StrokeStyle& penStyle)
 {
     // For odd widths, we add in 0.5 to the appropriate x/y so that the float arithmetic
