@@ -4,6 +4,8 @@
  * Copyright (C) 2008, 2009 Dirk Schulze <krit@webkit.org>
  * Copyright (C) 2008 Nuanti Ltd.
  * Copyright (C) 2009 Brent Fulgham <bfulgham@webkit.org>
+ * Copyright (C) 2010 Igalia S.L.
+ * Copyright (C) Research In Motion Limited 2010. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,18 +36,18 @@
 
 #include "AffineTransform.h"
 #include "CairoPath.h"
-#include "FEGaussianBlur.h"
+#include "CairoUtilities.h"
+#include "ContextShadow.h"
 #include "FloatRect.h"
 #include "Font.h"
-#include "ImageBuffer.h"
-#include "ImageBufferFilter.h"
+#include "GraphicsContextPlatformPrivateCairo.h"
+#include "OwnPtrCairo.h"
 #include "IntRect.h"
 #include "NotImplemented.h"
 #include "Path.h"
 #include "Pattern.h"
+#include "RefPtrCairo.h"
 #include "SimpleFontData.h"
-#include "SourceGraphic.h"
-
 #include <cairo.h>
 #include <math.h>
 #include <stdio.h>
@@ -57,8 +59,8 @@
 #elif PLATFORM(WIN)
 #include <cairo-win32.h>
 #endif
-#include "GraphicsContextPlatformPrivateCairo.h"
-#include "GraphicsContextPrivate.h"
+
+using namespace std;
 
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
@@ -66,49 +68,46 @@
 
 namespace WebCore {
 
-static inline void setColor(cairo_t* cr, const Color& col)
-{
-    float red, green, blue, alpha;
-    col.getRGBA(red, green, blue, alpha);
-    cairo_set_source_rgba(cr, red, green, blue, alpha);
-}
-
-static inline void setPlatformFill(GraphicsContext* context, cairo_t* cr, GraphicsContextPrivate* gcp)
+static inline void setPlatformFill(GraphicsContext* context, cairo_t* cr)
 {
     cairo_pattern_t* pattern = 0;
     cairo_save(cr);
-    if (gcp->state.fillPattern) {
+    
+    const GraphicsContextState& state = context->state();
+    if (state.fillPattern) {
         AffineTransform affine;
-        pattern = gcp->state.fillPattern->createPlatformPattern(affine);
+        pattern = state.fillPattern->createPlatformPattern(affine);
         cairo_set_source(cr, pattern);
-    } else if (gcp->state.fillGradient)
-        cairo_set_source(cr, gcp->state.fillGradient->platformGradient());
+    } else if (state.fillGradient)
+        cairo_set_source(cr, state.fillGradient->platformGradient());
     else
-        setColor(cr, context->fillColor());
+        setSourceRGBAFromColor(cr, context->fillColor());
     cairo_clip_preserve(cr);
-    cairo_paint_with_alpha(cr, gcp->state.globalAlpha);
+    cairo_paint_with_alpha(cr, state.globalAlpha);
     cairo_restore(cr);
     if (pattern)
         cairo_pattern_destroy(pattern);
 }
 
-static inline void setPlatformStroke(GraphicsContext* context, cairo_t* cr, GraphicsContextPrivate* gcp)
+static inline void setPlatformStroke(GraphicsContext* context, cairo_t* cr)
 {
     cairo_pattern_t* pattern = 0;
     cairo_save(cr);
-    if (gcp->state.strokePattern) {
+    
+    const GraphicsContextState& state = context->state();
+    if (state.strokePattern) {
         AffineTransform affine;
-        pattern = gcp->state.strokePattern->createPlatformPattern(affine);
+        pattern = state.strokePattern->createPlatformPattern(affine);
         cairo_set_source(cr, pattern);
-    } else if (gcp->state.strokeGradient)
-        cairo_set_source(cr, gcp->state.strokeGradient->platformGradient());
+    } else if (state.strokeGradient)
+        cairo_set_source(cr, state.strokeGradient->platformGradient());
     else  {
-        Color strokeColor = colorWithOverrideAlpha(context->strokeColor().rgb(), context->strokeColor().alpha() / 255.f * gcp->state.globalAlpha);
-        setColor(cr, strokeColor);
+        Color strokeColor = colorWithOverrideAlpha(context->strokeColor().rgb(), context->strokeColor().alpha() / 255.f * state.globalAlpha);
+        setSourceRGBAFromColor(cr, strokeColor);
     }
-    if (gcp->state.globalAlpha < 1.0f && (gcp->state.strokePattern || gcp->state.strokeGradient)) {
+    if (state.globalAlpha < 1.0f && (state.strokePattern || state.strokeGradient)) {
         cairo_push_group(cr);
-        cairo_paint_with_alpha(cr, gcp->state.globalAlpha);
+        cairo_paint_with_alpha(cr, state.globalAlpha);
         cairo_pop_group_to_source(cr);
     }
     cairo_stroke_preserve(cr);
@@ -120,99 +119,95 @@ static inline void setPlatformStroke(GraphicsContext* context, cairo_t* cr, Grap
 // A fillRect helper
 static inline void fillRectSourceOver(cairo_t* cr, const FloatRect& rect, const Color& col)
 {
-    setColor(cr, col);
+    setSourceRGBAFromColor(cr, col);
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
     cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
     cairo_fill(cr);
 }
 
-static inline void copyContextProperties(cairo_t* srcCr, cairo_t* dstCr)
+static void addConvexPolygonToContext(cairo_t* context, size_t numPoints, const FloatPoint* points)
 {
-    cairo_set_antialias(dstCr, cairo_get_antialias(srcCr));
-
-    size_t dashCount = cairo_get_dash_count(srcCr);
-    Vector<double> dashes(dashCount);
-
-    double offset;
-    cairo_get_dash(srcCr, dashes.data(), &offset);
-    cairo_set_dash(dstCr, dashes.data(), dashCount, offset);
-    cairo_set_line_cap(dstCr, cairo_get_line_cap(srcCr));
-    cairo_set_line_join(dstCr, cairo_get_line_join(srcCr));
-    cairo_set_line_width(dstCr, cairo_get_line_width(srcCr));
-    cairo_set_miter_limit(dstCr, cairo_get_miter_limit(srcCr));
-    cairo_set_fill_rule(dstCr, cairo_get_fill_rule(srcCr));
+    cairo_move_to(context, points[0].x(), points[0].y());
+    for (size_t i = 1; i < numPoints; i++)
+        cairo_line_to(context, points[i].x(), points[i].y());
+    cairo_close_path(context);
 }
 
-void GraphicsContext::calculateShadowBufferDimensions(IntSize& shadowBufferSize, FloatRect& shadowRect, float& kernelSize, const FloatRect& sourceRect, const IntSize& shadowSize, int shadowBlur)
+enum PathDrawingStyle { 
+    Fill = 1,
+    Stroke = 2,
+    FillAndStroke = Fill + Stroke
+};
+
+static inline void drawPathShadow(GraphicsContext* context, PathDrawingStyle drawingStyle)
 {
-#if ENABLE(FILTERS)
-    // calculate the kernel size according to the HTML5 canvas shadow specification
-    kernelSize = (shadowBlur < 8 ? shadowBlur / 2.f : sqrt(shadowBlur * 2.f));
-    int blurRadius = ceil(kernelSize);
-
-    shadowBufferSize = IntSize(sourceRect.width() + blurRadius * 2, sourceRect.height() + blurRadius * 2);
-
-    // determine dimensions of shadow rect
-    shadowRect = FloatRect(sourceRect.location(), shadowBufferSize);
-    shadowRect.move(shadowSize.width() - kernelSize, shadowSize.height() - kernelSize);
-#endif
-}
-
-static inline void drawPathShadow(GraphicsContext* context, GraphicsContextPrivate* gcp, bool fillShadow, bool strokeShadow)
-{
-#if ENABLE(FILTERS)
-    IntSize shadowSize;
-    int shadowBlur;
-    Color shadowColor;
-    if (!context->getShadow(shadowSize, shadowBlur, shadowColor))
+    ContextShadow* shadow = context->contextShadow();
+    ASSERT(shadow);
+    if (shadow->m_type == ContextShadow::NoShadow)
         return;
-    
-    // Calculate filter values to create appropriate shadow.
-    cairo_t* cr = context->platformContext();
-    cairo_path_t* path = cairo_copy_path(cr);
-    double x0, x1, y0, y1;
-    if (strokeShadow)
-        cairo_stroke_extents(cr, &x0, &y0, &x1, &y1);
-    else
-        cairo_fill_extents(cr, &x0, &y0, &x1, &y1);
-    FloatRect rect(x0, y0, x1 - x0, y1 - y0);
 
-    IntSize shadowBufferSize;
-    FloatRect shadowRect;
-    float kernelSize = 0;
-    GraphicsContext::calculateShadowBufferDimensions(shadowBufferSize, shadowRect, kernelSize, rect, shadowSize, shadowBlur);
+    // Calculate the extents of the rendered solid paths.
+    cairo_t* cairoContext = context->platformContext();
+    OwnPtr<cairo_path_t> path(cairo_copy_path(cairoContext));
 
-    // Create suitably-sized ImageBuffer to hold the shadow.
-    OwnPtr<ImageBuffer> shadowBuffer = ImageBuffer::create(shadowBufferSize);
+    FloatRect solidFigureExtents;
+    double x0 = 0;
+    double x1 = 0;
+    double y0 = 0;
+    double y1 = 0;
+    if (drawingStyle & Stroke) {
+        cairo_stroke_extents(cairoContext, &x0, &y0, &x1, &y1);
+        solidFigureExtents = FloatRect(x0, y0, x1 - x0, y1 - y0);
+    }
+    if (drawingStyle & Fill) {
+        cairo_fill_extents(cairoContext, &x0, &y0, &x1, &y1);
+        FloatRect fillExtents(x0, y0, x1 - x0, y1 - y0);
+        solidFigureExtents.unite(fillExtents);
+    }
 
-    // Draw shadow into a new ImageBuffer.
-    cairo_t* shadowContext = shadowBuffer->context()->platformContext();
-    copyContextProperties(cr, shadowContext);
-    cairo_translate(shadowContext, -rect.x() + kernelSize, -rect.y() + kernelSize);
-    cairo_new_path(shadowContext);
-    cairo_append_path(shadowContext, path);
+    cairo_t* shadowContext = shadow->beginShadowLayer(cairoContext, solidFigureExtents);
+    if (!shadowContext)
+        return;
 
-    if (fillShadow)
-        setPlatformFill(context, shadowContext, gcp);
-    if (strokeShadow)
-        setPlatformStroke(context, shadowContext, gcp);
+    // It's important to copy the context properties to the new shadow
+    // context to preserve things such as the fill rule and stroke width.
+    copyContextProperties(cairoContext, shadowContext);
+    cairo_append_path(shadowContext, path.get());
 
-    context->createPlatformShadow(shadowBuffer.release(), shadowColor, shadowRect, kernelSize);
-#endif
+    if (drawingStyle & Fill)
+        setPlatformFill(context, shadowContext);
+    if (drawingStyle & Stroke)
+        setPlatformStroke(context, shadowContext);
+
+    shadow->endShadowLayer(cairoContext);
 }
 
-GraphicsContext::GraphicsContext(PlatformGraphicsContext* cr)
-    : m_common(createGraphicsContextPrivate())
-    , m_data(new GraphicsContextPlatformPrivate)
+static void fillCurrentCairoPath(GraphicsContext* context, cairo_t* cairoContext)
 {
+    cairo_set_fill_rule(cairoContext, context->fillRule() == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
+    drawPathShadow(context, Fill);
+
+    setPlatformFill(context, cairoContext);
+    cairo_new_path(cairoContext);
+}
+
+static void strokeCurrentCairoPath(GraphicsContext* context,  cairo_t* cairoContext)
+{
+    drawPathShadow(context, Stroke);
+    setPlatformStroke(context, cairoContext);
+    cairo_new_path(cairoContext);
+}
+
+void GraphicsContext::platformInit(PlatformGraphicsContext* cr)
+{
+    m_data = new GraphicsContextPlatformPrivate;
     m_data->cr = cairo_reference(cr);
     m_data->syncContext(cr);
     setPaintingDisabled(!cr);
 }
 
-GraphicsContext::~GraphicsContext()
+void GraphicsContext::platformDestroy()
 {
-    destroyGraphicsContextPrivate(m_common);
     delete m_data;
 }
 
@@ -233,12 +228,20 @@ void GraphicsContext::savePlatformState()
 {
     cairo_save(m_data->cr);
     m_data->save();
+    m_data->shadowStack.append(m_data->shadow);
 }
 
 void GraphicsContext::restorePlatformState()
 {
     cairo_restore(m_data->cr);
     m_data->restore();
+
+    if (m_data->shadowStack.isEmpty())
+        m_data->shadow = ContextShadow();
+    else {
+        m_data->shadow = m_data->shadowStack.last();
+        m_data->shadowStack.removeLast();
+    }
 }
 
 // Draws a filled rectangle with a stroked border.
@@ -254,7 +257,7 @@ void GraphicsContext::drawRect(const IntRect& rect)
         fillRectSourceOver(cr, rect, fillColor());
 
     if (strokeStyle() != NoStroke) {
-        setColor(cr, strokeColor());
+        setSourceRGBAFromColor(cr, strokeColor());
         FloatRect r(rect);
         r.inflate(-.5f);
         cairo_rectangle(cr, r.x(), r.y(), r.width(), r.height());
@@ -302,7 +305,7 @@ void GraphicsContext::drawLine(const IntPoint& point1, const IntPoint& point2)
         break;
     }
 
-    setColor(cr, strokeColor());
+    setSourceRGBAFromColor(cr, strokeColor());
 
     cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
 
@@ -372,17 +375,16 @@ void GraphicsContext::drawEllipse(const IntRect& rect)
     cairo_restore(cr);
 
     if (fillColor().alpha()) {
-        setColor(cr, fillColor());
+        setSourceRGBAFromColor(cr, fillColor());
         cairo_fill_preserve(cr);
     }
 
     if (strokeStyle() != NoStroke) {
-        setColor(cr, strokeColor());
+        setSourceRGBAFromColor(cr, strokeColor());
         cairo_set_line_width(cr, strokeThickness());
         cairo_stroke(cr);
-    }
-
-    cairo_new_path(cr);
+    } else
+        cairo_new_path(cr);
 }
 
 void GraphicsContext::strokeArc(const IntRect& rect, int startAngle, int angleSpan)
@@ -427,7 +429,7 @@ void GraphicsContext::strokeArc(const IntRect& rect, int startAngle, int angleSp
         break;
     }
 
-    setColor(cr, strokeColor());
+    setSourceRGBAFromColor(cr, strokeColor());
 
     if (patWidth) {
         // Example: 80 pixels with a width of 30 pixels.
@@ -483,67 +485,65 @@ void GraphicsContext::drawConvexPolygon(size_t npoints, const FloatPoint* points
 
     cairo_save(cr);
     cairo_set_antialias(cr, shouldAntialias ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
-    cairo_move_to(cr, points[0].x(), points[0].y());
-    for (size_t i = 1; i < npoints; i++)
-        cairo_line_to(cr, points[i].x(), points[i].y());
-    cairo_close_path(cr);
+    addConvexPolygonToContext(cr, npoints, points);
 
     if (fillColor().alpha()) {
-        setColor(cr, fillColor());
+        setSourceRGBAFromColor(cr, fillColor());
         cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
         cairo_fill_preserve(cr);
     }
 
     if (strokeStyle() != NoStroke) {
-        setColor(cr, strokeColor());
+        setSourceRGBAFromColor(cr, strokeColor());
         cairo_set_line_width(cr, strokeThickness());
         cairo_stroke(cr);
-    }
+    } else
+        cairo_new_path(cr);
 
-    cairo_new_path(cr);
     cairo_restore(cr);
 }
 
-void GraphicsContext::fillPath()
+void GraphicsContext::clipConvexPolygon(size_t numPoints, const FloatPoint* points, bool antialiased)
 {
     if (paintingDisabled())
         return;
 
+    if (numPoints <= 1)
+        return;
+
     cairo_t* cr = m_data->cr;
 
-    cairo_set_fill_rule(cr, fillRule() == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
-    drawPathShadow(this, m_common, true, false);
-
-    setPlatformFill(this, cr, m_common);
     cairo_new_path(cr);
+    cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
+    cairo_antialias_t savedAntialiasRule = cairo_get_antialias(cr);
+
+    cairo_set_antialias(cr, antialiased ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
+    cairo_set_fill_rule(cr, CAIRO_FILL_RULE_WINDING);
+    addConvexPolygonToContext(cr, numPoints, points);
+    cairo_clip(cr);
+
+    cairo_set_antialias(cr, savedAntialiasRule);
+    cairo_set_fill_rule(cr, savedFillRule);
 }
 
-void GraphicsContext::strokePath()
+void GraphicsContext::fillPath(const Path& path)
 {
     if (paintingDisabled())
         return;
 
     cairo_t* cr = m_data->cr;
-    drawPathShadow(this, m_common, false, true);
-
-    setPlatformStroke(this, cr, m_common);
-    cairo_new_path(cr);
-
+    setPathOnCairoContext(cr, path.platformPath()->context());
+    fillCurrentCairoPath(this, cr);
 }
 
-void GraphicsContext::drawPath()
+void GraphicsContext::strokePath(const Path& path)
 {
     if (paintingDisabled())
         return;
 
     cairo_t* cr = m_data->cr;
-
-    cairo_set_fill_rule(cr, fillRule() == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
-    drawPathShadow(this, m_common, true, true);
-
-    setPlatformFill(this, cr, m_common);
-    setPlatformStroke(this, cr, m_common);
-    cairo_new_path(cr);
+    setPathOnCairoContext(cr, path.platformPath()->context());
+    strokeCurrentCairoPath(this, cr);
 }
 
 void GraphicsContext::fillRect(const FloatRect& rect)
@@ -552,40 +552,20 @@ void GraphicsContext::fillRect(const FloatRect& rect)
         return;
 
     cairo_t* cr = m_data->cr;
+    cairo_save(cr);
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
-    fillPath();
+    fillCurrentCairoPath(this, cr);
+    cairo_restore(cr);
 }
 
-static void drawBorderlessRectShadow(GraphicsContext* context, const FloatRect& rect, const Color& rectColor)
-{
-#if ENABLE(FILTERS)
-    IntSize shadowSize;
-    int shadowBlur;
-    Color shadowColor;
-
-    if (!context->getShadow(shadowSize, shadowBlur, shadowColor))
-        return;
-
-    IntSize shadowBufferSize;
-    FloatRect shadowRect;
-    float kernelSize = 0;
-    GraphicsContext::calculateShadowBufferDimensions(shadowBufferSize, shadowRect, kernelSize, rect, shadowSize, shadowBlur);
-
-    // Draw shadow into a new ImageBuffer
-    OwnPtr<ImageBuffer> shadowBuffer = ImageBuffer::create(shadowBufferSize);
-    GraphicsContext* shadowContext = shadowBuffer->context();
-    shadowContext->fillRect(FloatRect(FloatPoint(kernelSize, kernelSize), rect.size()), rectColor, DeviceColorSpace);
-
-    context->createPlatformShadow(shadowBuffer.release(), shadowColor, shadowRect, kernelSize);
-#endif
-}
-
-void GraphicsContext::fillRect(const FloatRect& rect, const Color& color, ColorSpace colorSpace)
+void GraphicsContext::fillRect(const FloatRect& rect, const Color& color, ColorSpace)
 {
     if (paintingDisabled())
         return;
 
-    drawBorderlessRectShadow(this, rect, color);
+    if (m_data->hasShadow())
+        m_data->shadow.drawRectShadow(this, enclosingIntRect(rect));
+
     if (color.alpha())
         fillRectSourceOver(m_data->cr, rect, color);
 }
@@ -604,19 +584,57 @@ void GraphicsContext::clip(const FloatRect& rect)
     m_data->clip(rect);
 }
 
-void GraphicsContext::clipPath(WindRule clipRule)
+void GraphicsContext::clipPath(const Path& path, WindRule clipRule)
 {
     if (paintingDisabled())
         return;
 
     cairo_t* cr = m_data->cr;
+    setPathOnCairoContext(cr, path.platformPath()->context());
     cairo_set_fill_rule(cr, clipRule == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
     cairo_clip(cr);
 }
 
-void GraphicsContext::drawFocusRing(const Vector<Path>& paths, int width, int offset, const Color& color)
+static inline void adjustFocusRingColor(Color& color)
 {
-    // FIXME: implement
+#if !PLATFORM(GTK)
+    // Force the alpha to 50%.  This matches what the Mac does with outline rings.
+    color.setRGB(makeRGBA(color.red(), color.green(), color.blue(), 127));
+#endif
+}
+
+static inline void adjustFocusRingLineWidth(int& width)
+{
+#if PLATFORM(GTK)
+    width = 2;
+#endif
+}
+
+static inline StrokeStyle focusRingStrokeStyle()
+{
+#if PLATFORM(GTK)
+    return DottedStroke;
+#else
+    return SolidStroke;
+#endif
+}
+
+void GraphicsContext::drawFocusRing(const Path& path, int width, int /* offset */, const Color& color)
+{
+    // FIXME: We should draw paths that describe a rectangle with rounded corners
+    // so as to be consistent with how we draw rectangular focus rings.
+    Color ringColor = color;
+    adjustFocusRingColor(ringColor);
+    adjustFocusRingLineWidth(width);
+
+    cairo_t* cr = m_data->cr;
+    cairo_save(cr);
+    appendWebCorePathToCairoContext(cr, path);
+    setSourceRGBAFromColor(cr, ringColor);
+    cairo_set_line_width(cr, width);
+    setPlatformStrokeStyle(focusRingStrokeStyle());
+    cairo_stroke(cr);
+    cairo_restore(cr);
 }
 
 void GraphicsContext::drawFocusRing(const Vector<IntRect>& rects, int width, int /* offset */, const Color& color)
@@ -632,28 +650,43 @@ void GraphicsContext::drawFocusRing(const Vector<IntRect>& rects, int width, int
     cairo_new_path(cr);
 
 #if PLATFORM(GTK)
+#ifdef GTK_API_VERSION_2
     GdkRegion* reg = gdk_region_new();
+#else
+    cairo_region_t* reg = cairo_region_create();
+#endif
+
     for (unsigned i = 0; i < rectCount; i++) {
+#ifdef GTK_API_VERSION_2
         GdkRectangle rect = rects[i];
         gdk_region_union_with_rect(reg, &rect);
+#else
+        cairo_rectangle_int_t rect = rects[i];
+        cairo_region_union_rectangle(reg, &rect);
+#endif
     }
     gdk_cairo_region(cr, reg);
+#ifdef GTK_API_VERSION_2
     gdk_region_destroy(reg);
-
-    setColor(cr, color);
-    cairo_set_line_width(cr, 2.0f);
-    setPlatformStrokeStyle(DottedStroke);
+#else
+    cairo_region_destroy(reg);
+#endif
 #else
     int radius = (width - 1) / 2;
-    for (unsigned i = 0; i < rectCount; i++)
-        addPath(Path::createRoundedRectangle(rects[i], FloatSize(radius, radius)));
-
-    // Force the alpha to 50%.  This matches what the Mac does with outline rings.
-    Color ringColor(color.red(), color.green(), color.blue(), 127);
-    setColor(cr, ringColor);
-    cairo_set_line_width(cr, width);
-    setPlatformStrokeStyle(SolidStroke);
+    Path path;
+    for (unsigned i = 0; i < rectCount; ++i) {
+        if (i > 0)
+            path.clear();
+        path.addRoundedRect(rects[i], FloatSize(radius, radius));
+        appendWebCorePathToCairoContext(cr, path);
+    }
 #endif
+    Color ringColor = color;
+    adjustFocusRingColor(ringColor);
+    adjustFocusRingLineWidth(width);
+    setSourceRGBAFromColor(cr, ringColor);
+    cairo_set_line_width(cr, width);
+    setPlatformStrokeStyle(focusRingStrokeStyle());
 
     cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
     cairo_stroke_preserve(cr);
@@ -673,17 +706,15 @@ void GraphicsContext::drawLineForText(const IntPoint& origin, int width, bool pr
     if (paintingDisabled())
         return;
 
-    // This is a workaround for http://bugs.webkit.org/show_bug.cgi?id=15659
-    StrokeStyle savedStrokeStyle = strokeStyle();
-    setStrokeStyle(SolidStroke);
-
     IntPoint endPoint = origin + IntSize(width, 0);
     drawLine(origin, endPoint);
-
-    setStrokeStyle(savedStrokeStyle);
 }
 
-void GraphicsContext::drawLineForMisspellingOrBadGrammar(const IntPoint& origin, int width, bool grammar)
+#if !PLATFORM(GTK)
+#include "DrawErrorUnderline.h"
+#endif
+
+void GraphicsContext::drawLineForTextChecking(const IntPoint& origin, int width, TextCheckingLineStyle style)
 {
     if (paintingDisabled())
         return;
@@ -691,18 +722,23 @@ void GraphicsContext::drawLineForMisspellingOrBadGrammar(const IntPoint& origin,
     cairo_t* cr = m_data->cr;
     cairo_save(cr);
 
-    // Convention is green for grammar, red for spelling
-    // These need to become configurable
-    if (grammar)
-        cairo_set_source_rgb(cr, 0, 1, 0);
-    else
+    switch (style) {
+    case TextCheckingSpellingLineStyle:
         cairo_set_source_rgb(cr, 1, 0, 0);
+        break;
+    case TextCheckingGrammarLineStyle:
+        cairo_set_source_rgb(cr, 0, 1, 0);
+        break;
+    default:
+        cairo_restore(cr);
+        return;
+    }
 
 #if PLATFORM(GTK)
     // We ignore most of the provided constants in favour of the platform style
     pango_cairo_show_error_underline(cr, origin.x(), origin.y(), width, cMisspellingLineThickness);
 #else
-    notImplemented();
+    drawErrorUnderline(cr, origin.x(), origin.y(), width, cMisspellingLineThickness);
 #endif
 
     cairo_restore(cr);
@@ -741,14 +777,6 @@ void GraphicsContext::translate(float x, float y)
     m_data->translate(x, y);
 }
 
-IntPoint GraphicsContext::origin()
-{
-    cairo_matrix_t matrix;
-    cairo_t* cr = m_data->cr;
-    cairo_get_matrix(cr, &matrix);
-    return IntPoint(static_cast<int>(matrix.x0), static_cast<int>(matrix.y0));
-}
-
 void GraphicsContext::setPlatformFillColor(const Color& col, ColorSpace colorSpace)
 {
     // Cairo contexts can't hold separate fill and stroke colors
@@ -815,6 +843,7 @@ void GraphicsContext::addInnerRoundedRectClip(const IntRect& rect, int thickness
     if (paintingDisabled())
         return;
 
+    cairo_t* cr = m_data->cr;
     clip(rect);
 
     Path p;
@@ -824,70 +853,36 @@ void GraphicsContext::addInnerRoundedRectClip(const IntRect& rect, int thickness
     // Add inner ellipse
     r.inflate(-thickness);
     p.addEllipse(r);
-    addPath(p);
+    appendWebCorePathToCairoContext(cr, p);
 
-    cairo_t* cr = m_data->cr;
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
     cairo_clip(cr);
     cairo_set_fill_rule(cr, savedFillRule);
 }
 
-void GraphicsContext::clipToImageBuffer(const FloatRect& rect, const ImageBuffer* imageBuffer)
+void GraphicsContext::setPlatformShadow(FloatSize const& size, float blur, Color const& color, ColorSpace)
 {
-    if (paintingDisabled())
-        return;
-
-    notImplemented();
-}
-
-void GraphicsContext::setPlatformShadow(IntSize const& size, int, Color const&, ColorSpace)
-{
-    // Cairo doesn't support shadows natively, they are drawn manually in the draw*
-    // functions
-
-    if (m_common->state.shadowsIgnoreTransforms) {
+    // Cairo doesn't support shadows natively, they are drawn manually in the draw* functions
+    if (m_state.shadowsIgnoreTransforms) {
         // Meaning that this graphics context is associated with a CanvasRenderingContext
         // We flip the height since CG and HTML5 Canvas have opposite Y axis
-        m_common->state.shadowSize = IntSize(size.width(), -size.height());
-    }
+        m_state.shadowOffset = FloatSize(size.width(), -size.height());
+        m_data->shadow = ContextShadow(color, blur, FloatSize(size.width(), -size.height()));
+    } else
+        m_data->shadow = ContextShadow(color, blur, FloatSize(size.width(), size.height()));
+
+    m_data->shadow.setShadowsIgnoreTransforms(m_state.shadowsIgnoreTransforms);
 }
 
-void GraphicsContext::createPlatformShadow(PassOwnPtr<ImageBuffer> buffer, const Color& shadowColor, const FloatRect& shadowRect, float kernelSize)
+ContextShadow* GraphicsContext::contextShadow()
 {
-#if ENABLE(FILTERS)
-    cairo_t* cr = m_data->cr;
-
-    // draw the shadow without blurring, if kernelSize is zero
-    if (!kernelSize) {
-        setColor(cr, shadowColor);
-        cairo_mask_surface(cr, buffer->image()->nativeImageForCurrentFrame(), shadowRect.x(), shadowRect.y());
-        return;
-    }
-
-    // limit kernel size to 1000, this is what CG is doing.
-    kernelSize = std::min(1000.f, kernelSize);
-
-    // create filter
-    RefPtr<Filter> filter = ImageBufferFilter::create();
-    filter->setSourceImage(buffer.release());
-    RefPtr<FilterEffect> source = SourceGraphic::create();
-    source->setScaledSubRegion(FloatRect(FloatPoint(), shadowRect.size()));
-    source->setIsAlphaImage(true);
-    RefPtr<FilterEffect> blur = FEGaussianBlur::create(source.get(), kernelSize, kernelSize);
-    blur->setScaledSubRegion(FloatRect(FloatPoint(), shadowRect.size()));
-    blur->apply(filter.get());
-
-    // Mask the filter with the shadow color and draw it to the context.
-    // Masking makes it possible to just blur the alpha channel.
-    setColor(cr, shadowColor);
-    cairo_mask_surface(cr, blur->resultImage()->image()->nativeImageForCurrentFrame(), shadowRect.x(), shadowRect.y());
-#endif
+    return &m_data->shadow;
 }
 
 void GraphicsContext::clearPlatformShadow()
 {
-    notImplemented();
+    m_data->shadow.clear();
 }
 
 void GraphicsContext::beginTransparencyLayer(float opacity)
@@ -937,7 +932,7 @@ void GraphicsContext::strokeRect(const FloatRect& rect, float width)
     cairo_save(cr);
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
     cairo_set_line_width(cr, width);
-    strokePath();
+    strokeCurrentCairoPath(this, cr);
     cairo_restore(cr);
 }
 
@@ -996,52 +991,15 @@ void GraphicsContext::setMiterLimit(float miter)
 
 void GraphicsContext::setAlpha(float alpha)
 {
-    m_common->state.globalAlpha = alpha;
+    m_state.globalAlpha = alpha;
 }
 
 float GraphicsContext::getAlpha()
 {
-    return m_common->state.globalAlpha;
+    return m_state.globalAlpha;
 }
 
-static inline cairo_operator_t toCairoOperator(CompositeOperator op)
-{
-    switch (op) {
-    case CompositeClear:
-        return CAIRO_OPERATOR_CLEAR;
-    case CompositeCopy:
-        return CAIRO_OPERATOR_SOURCE;
-    case CompositeSourceOver:
-        return CAIRO_OPERATOR_OVER;
-    case CompositeSourceIn:
-        return CAIRO_OPERATOR_IN;
-    case CompositeSourceOut:
-        return CAIRO_OPERATOR_OUT;
-    case CompositeSourceAtop:
-        return CAIRO_OPERATOR_ATOP;
-    case CompositeDestinationOver:
-        return CAIRO_OPERATOR_DEST_OVER;
-    case CompositeDestinationIn:
-        return CAIRO_OPERATOR_DEST_IN;
-    case CompositeDestinationOut:
-        return CAIRO_OPERATOR_DEST_OUT;
-    case CompositeDestinationAtop:
-        return CAIRO_OPERATOR_DEST_ATOP;
-    case CompositeXOR:
-        return CAIRO_OPERATOR_XOR;
-    case CompositePlusDarker:
-        return CAIRO_OPERATOR_SATURATE;
-    case CompositeHighlight:
-        // There is no Cairo equivalent for CompositeHighlight.
-        return CAIRO_OPERATOR_OVER;
-    case CompositePlusLighter:
-        return CAIRO_OPERATOR_ADD;
-    default:
-        return CAIRO_OPERATOR_SOURCE;
-    }
-}
-
-void GraphicsContext::setCompositeOperation(CompositeOperator op)
+void GraphicsContext::setPlatformCompositeOperation(CompositeOperator op)
 {
     if (paintingDisabled())
         return;
@@ -1049,35 +1007,14 @@ void GraphicsContext::setCompositeOperation(CompositeOperator op)
     cairo_set_operator(m_data->cr, toCairoOperator(op));
 }
 
-void GraphicsContext::beginPath()
-{
-    if (paintingDisabled())
-        return;
-
-    cairo_t* cr = m_data->cr;
-    cairo_new_path(cr);
-}
-
-void GraphicsContext::addPath(const Path& path)
-{
-    if (paintingDisabled())
-        return;
-
-    cairo_t* cr = m_data->cr;
-    cairo_path_t* p = cairo_copy_path(path.platformPath()->m_cr);
-    cairo_append_path(cr, p);
-    cairo_path_destroy(p);
-}
-
 void GraphicsContext::clip(const Path& path)
 {
     if (paintingDisabled())
         return;
 
     cairo_t* cr = m_data->cr;
-    cairo_path_t* p = cairo_copy_path(path.platformPath()->m_cr);
-    cairo_append_path(cr, p);
-    cairo_path_destroy(p);
+    OwnPtr<cairo_path_t> p(cairo_copy_path(path.platformPath()->context()));
+    cairo_append_path(cr, p.get());
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_WINDING);
     cairo_clip(cr);
@@ -1099,7 +1036,7 @@ void GraphicsContext::clipOut(const Path& path)
     double x1, y1, x2, y2;
     cairo_clip_extents(cr, &x1, &y1, &x2, &y2);
     cairo_rectangle(cr, x1, y1, x2 - x1, y2 - y1);
-    addPath(path);
+    appendWebCorePathToCairoContext(cr, path);
 
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
@@ -1141,14 +1078,12 @@ void GraphicsContext::clipOut(const IntRect& r)
     cairo_set_fill_rule(cr, savedFillRule);
 }
 
-void GraphicsContext::clipOutEllipseInRect(const IntRect& r)
+static inline FloatPoint getPhase(const FloatRect& dest, const FloatRect& tile)
 {
-    if (paintingDisabled())
-        return;
+    FloatPoint phase = dest.location();
+    phase.move(-tile.x(), -tile.y());
 
-    Path p;
-    p.addEllipse(r);
-    clipOut(p);
+    return phase;
 }
 
 void GraphicsContext::fillRoundedRect(const IntRect& r, const IntSize& topLeft, const IntSize& topRight, const IntSize& bottomLeft, const IntSize& bottomRight, const Color& color, ColorSpace colorSpace)
@@ -1156,12 +1091,15 @@ void GraphicsContext::fillRoundedRect(const IntRect& r, const IntSize& topLeft,
     if (paintingDisabled())
         return;
 
+    if (m_data->hasShadow())
+        m_data->shadow.drawRectShadow(this, r, topLeft, topRight, bottomLeft, bottomRight);
+
     cairo_t* cr = m_data->cr;
     cairo_save(cr);
-    beginPath();
-    addPath(Path::createRoundedRectangle(r, topLeft, topRight, bottomLeft, bottomRight));
-    setColor(cr, color);
-    drawPathShadow(this, m_common, true, false);
+    Path path;
+    path.addRoundedRect(r, topLeft, topRight, bottomLeft, bottomRight);
+    appendWebCorePathToCairoContext(cr, path);
+    setSourceRGBAFromColor(cr, color);
     cairo_fill(cr);
     cairo_restore(cr);
 }
@@ -1177,12 +1115,12 @@ GdkEventExpose* GraphicsContext::gdkExposeEvent() const
     return m_data->expose;
 }
 
-GdkDrawable* GraphicsContext::gdkDrawable() const
+GdkWindow* GraphicsContext::gdkWindow() const
 {
     if (!m_data->expose)
         return 0;
 
-    return GDK_DRAWABLE(m_data->expose->window);
+    return m_data->expose->window;
 }
 #endif
 
