@@ -27,6 +27,7 @@
 
 #include "GraphicsContext.h"
 
+#include "ContextShadow.h"
 #include <cairo.h>
 #include <math.h>
 #include <stdio.h>
@@ -96,6 +97,10 @@ public:
     cairo_t* cr;
     Vector<float> layers;
 
+    ContextShadow shadow;
+    Vector<ContextShadow> shadowStack;
+    bool hasShadow() const { return shadow.m_type != ContextShadow::NoShadow; }
+
 #if PLATFORM(GTK)
     GdkEventExpose* expose;
 #elif PLATFORM(WIN)
