@@ -18,17 +18,19 @@
 
 #include "config.h"
 #include "WebKitWebSourceGStreamer.h"
+#if USE(GSTREAMER)
 
-#include "CString.h"
 #include "Document.h"
 #include "GOwnPtr.h"
 #include "GRefPtr.h"
+#include "NetworkingContext.h"
 #include "Noncopyable.h"
 #include "NotImplemented.h"
 #include "ResourceHandleClient.h"
 #include "ResourceHandleInternal.h"
 #include "ResourceRequest.h"
 #include "ResourceResponse.h"
+#include <wtf/text/CString.h>
 #include <gst/app/gstappsrc.h>
 #include <gst/pbutils/missing-plugins.h>
 
@@ -42,7 +44,7 @@ class StreamingClient : public Noncopyable, public ResourceHandleClient {
         virtual void willSendRequest(ResourceHandle*, ResourceRequest&, const ResourceResponse&);
         virtual void didReceiveResponse(ResourceHandle*, const ResourceResponse&);
         virtual void didReceiveData(ResourceHandle*, const char*, int, int);
-        virtual void didFinishLoading(ResourceHandle*);
+        virtual void didFinishLoading(ResourceHandle*, double /*finishTime*/);
         virtual void didFail(ResourceHandle*, const ResourceError&);
         virtual void wasBlocked(ResourceHandle*);
         virtual void cannotShowURL(ResourceHandle*);
@@ -108,6 +110,7 @@ static void webKitWebSrcFinalize(GObject* object);
 static void webKitWebSrcSetProperty(GObject* object, guint propID, const GValue* value, GParamSpec* pspec);
 static void webKitWebSrcGetProperty(GObject* object, guint propID, GValue* value, GParamSpec* pspec);
 static GstStateChangeReturn webKitWebSrcChangeState(GstElement* element, GstStateChange transition);
+static gboolean webKitWebSrcQuery(GstPad* pad, GstQuery* query);
 
 static void webKitWebSrcNeedDataCb(GstAppSrc* appsrc, guint length, gpointer userData);
 static void webKitWebSrcEnoughDataCb(GstAppSrc* appsrc, gpointer userData);
@@ -219,6 +222,7 @@ static void webkit_web_src_init(WebKitWebSrc* src,
                                                              padTemplate);
 
     gst_element_add_pad(GST_ELEMENT(src), priv->srcpad);
+    gst_pad_set_query_function(priv->srcpad, webKitWebSrcQuery);
 
     priv->appsrc = GST_APP_SRC(gst_element_factory_make("appsrc", 0));
     if (!priv->appsrc) {
@@ -255,6 +259,7 @@ static void webkit_web_src_init(WebKitWebSrc* src,
     // already if the queue is 20% empty, it's much more
     // likely that libsoup already provides new data before
     // the queue is really empty.
+    // This might need tweaking for ports not using libsoup.
     if (priv->haveAppSrc27)
         g_object_set(priv->appsrc, "min-percent", 20, NULL);
 
@@ -329,6 +334,7 @@ static void webKitWebSrcStop(WebKitWebSrc* src, bool seeking)
     if (priv->frame)
         priv->frame.release();
 
+    GST_OBJECT_LOCK(src);
     if (priv->needDataID)
         g_source_remove(priv->needDataID);
     priv->needDataID = 0;
@@ -342,6 +348,7 @@ static void webKitWebSrcStop(WebKitWebSrc* src, bool seeking)
     priv->seekID = 0;
 
     priv->paused = FALSE;
+    GST_OBJECT_UNLOCK(src);
 
     g_free(priv->iradioName);
     priv->iradioName = 0;
@@ -387,20 +394,24 @@ static bool webKitWebSrcStart(WebKitWebSrc* src)
     request.setTargetType(ResourceRequestBase::TargetIsMedia);
     request.setAllowCookies(true);
 
-    // Let Apple web servers know we want to access their nice movie trailers.
-    if (!g_ascii_strcasecmp("movies.apple.com", url.host().utf8().data()))
-        request.setHTTPUserAgent("Quicktime/7.2.0");
-
+    NetworkingContext* context = 0;
     if (priv->frame) {
         Document* document = priv->frame->document();
         if (document)
             request.setHTTPReferrer(document->documentURI());
 
         FrameLoader* loader = priv->frame->loader();
-        if (loader)
+        if (loader) {
             loader->addExtraFieldsToSubresourceRequest(request);
+            context = loader->networkingContext();
+        }
     }
 
+    // Let Apple web servers know we want to access their nice movie trailers.
+    if (!g_ascii_strcasecmp("movies.apple.com", url.host().utf8().data())
+        || !g_ascii_strcasecmp("trailers.apple.com", url.host().utf8().data()))
+        request.setHTTPUserAgent("Quicktime/7.6.6");
+
     if (priv->requestedOffset) {
         GOwnPtr<gchar> val;
 
@@ -414,7 +425,7 @@ static bool webKitWebSrcStart(WebKitWebSrc* src)
     // Needed to use DLNA streaming servers
     request.setHTTPHeaderField("transferMode.dlna", "Streaming");
 
-    priv->resourceHandle = ResourceHandle::create(request, priv->client, 0, false, false);
+    priv->resourceHandle = ResourceHandle::create(context, request, priv->client, false, false);
     if (!priv->resourceHandle) {
         GST_ERROR_OBJECT(src, "Failed to create ResourceHandle");
         return false;
@@ -467,6 +478,36 @@ static GstStateChangeReturn webKitWebSrcChangeState(GstElement* element, GstStat
     return ret;
 }
 
+static gboolean webKitWebSrcQuery(GstPad* pad, GstQuery* query)
+{
+    WebKitWebSrc* src = WEBKIT_WEB_SRC(gst_pad_get_parent(pad));
+    gboolean result = FALSE;
+
+    switch (GST_QUERY_TYPE(query)) {
+    case GST_QUERY_DURATION:
+    {
+        GstFormat format;
+
+        gst_query_parse_duration(query, &format, NULL);
+
+        GST_DEBUG_OBJECT(src, "duration query in format %s", gst_format_get_name(format));
+        if ((format == GST_FORMAT_BYTES) && (src->priv->size > 0)) {
+            gst_query_set_duration(query, format, src->priv->size);
+            result = TRUE;
+        }
+        break;
+    }
+    default:
+        break;
+    }
+
+    if (!result)
+        result = gst_pad_query_default(pad, query);
+
+    gst_object_unref(src);
+    return result;
+}
+
 // uri handler interface
 
 static GstURIType webKitWebSrcUriGetType(void)
@@ -505,17 +546,14 @@ static gboolean webKitWebSrcSetUri(GstURIHandler* handler, const gchar* uri)
     if (!uri)
         return TRUE;
 
-    SoupURI* soupUri = soup_uri_new(uri);
+    KURL url(KURL(), uri);
 
-    if (!soupUri || !SOUP_URI_VALID_FOR_HTTP(soupUri)) {
+    if (!url.isValid() || !url.protocolInHTTPFamily()) {
         GST_ERROR_OBJECT(src, "Invalid URI '%s'", uri);
-        soup_uri_free(soupUri);
         return FALSE;
     }
 
-    priv->uri = soup_uri_to_string(soupUri, FALSE);
-    soup_uri_free(soupUri);
-
+    priv->uri = g_strdup(url.string().utf8().data());
     return TRUE;
 }
 
@@ -535,12 +573,12 @@ static gboolean webKitWebSrcNeedDataMainCb(WebKitWebSrc* src)
 {
     WebKitWebSrcPrivate* priv = src->priv;
 
-    ResourceHandleInternal* d = priv->resourceHandle->getInternal();
-    if (d->m_msg)
-        soup_session_unpause_message(ResourceHandle::defaultSession(), d->m_msg);
+    priv->resourceHandle->setDefersLoading(false);
 
+    GST_OBJECT_LOCK(src);
     priv->paused = FALSE;
     priv->needDataID = 0;
+    GST_OBJECT_UNLOCK(src);
     return FALSE;
 }
 
@@ -550,21 +588,28 @@ static void webKitWebSrcNeedDataCb(GstAppSrc* appsrc, guint length, gpointer use
     WebKitWebSrcPrivate* priv = src->priv;
 
     GST_DEBUG_OBJECT(src, "Need more data: %u", length);
-    if (priv->needDataID || !priv->paused)
+
+    GST_OBJECT_LOCK(src);
+    if (priv->needDataID || !priv->paused) {
+        GST_OBJECT_UNLOCK(src);
         return;
+    }
 
     priv->needDataID = g_timeout_add_full(G_PRIORITY_DEFAULT, 0, (GSourceFunc) webKitWebSrcNeedDataMainCb, gst_object_ref(src), (GDestroyNotify) gst_object_unref);
+    GST_OBJECT_UNLOCK(src);
 }
 
 static gboolean webKitWebSrcEnoughDataMainCb(WebKitWebSrc* src)
 {
     WebKitWebSrcPrivate* priv = src->priv;
 
-    ResourceHandleInternal* d = priv->resourceHandle->getInternal();
-    soup_session_pause_message(ResourceHandle::defaultSession(), d->m_msg);
-    
+    priv->resourceHandle->setDefersLoading(true);
+
+    GST_OBJECT_LOCK(src);
     priv->paused = TRUE;
     priv->enoughDataID = 0;
+    GST_OBJECT_UNLOCK(src);
+
     return FALSE;
 }
 
@@ -574,10 +619,15 @@ static void webKitWebSrcEnoughDataCb(GstAppSrc* appsrc, gpointer userData)
     WebKitWebSrcPrivate* priv = src->priv;
 
     GST_DEBUG_OBJECT(src, "Have enough data");
-    if (priv->enoughDataID || priv->paused)
+
+    GST_OBJECT_LOCK(src);
+    if (priv->enoughDataID || priv->paused) {
+        GST_OBJECT_UNLOCK(src);
         return;
+    }
 
     priv->enoughDataID = g_timeout_add_full(G_PRIORITY_DEFAULT, 0, (GSourceFunc) webKitWebSrcEnoughDataMainCb, gst_object_ref(src), (GDestroyNotify) gst_object_unref);
+    GST_OBJECT_UNLOCK(src);
 }
 
 static gboolean webKitWebSrcSeekMainCb(WebKitWebSrc* src)
@@ -604,9 +654,12 @@ static gboolean webKitWebSrcSeekDataCb(GstAppSrc* appsrc, guint64 offset, gpoint
 
     GST_DEBUG_OBJECT(src, "Doing range-request seek");
     priv->requestedOffset = offset;
+
+    GST_OBJECT_LOCK(src);
     if (priv->seekID)
         g_source_remove(priv->seekID);
     priv->seekID = g_timeout_add_full(G_PRIORITY_DEFAULT, 0, (GSourceFunc) webKitWebSrcSeekMainCb, gst_object_ref(src), (GDestroyNotify) gst_object_unref);
+    GST_OBJECT_UNLOCK(src);
     
     return TRUE;
 }
@@ -734,10 +787,14 @@ void StreamingClient::didReceiveData(ResourceHandle* handle, const char* data, i
         GST_ELEMENT_ERROR(m_src, CORE, FAILED, (0), (0));
 }
 
-void StreamingClient::didFinishLoading(ResourceHandle*)
+void StreamingClient::didFinishLoading(ResourceHandle*, double)
 {
+    WebKitWebSrcPrivate* priv = m_src->priv;
+
     GST_DEBUG_OBJECT(m_src, "Have EOS");
-    gst_app_src_end_of_stream(m_src->priv->appsrc);
+
+    if (!priv->seekID)
+        gst_app_src_end_of_stream(m_src->priv->appsrc);
 }
 
 void StreamingClient::didFail(ResourceHandle*, const ResourceError& error)
@@ -749,9 +806,15 @@ void StreamingClient::didFail(ResourceHandle*, const ResourceError& error)
 
 void StreamingClient::wasBlocked(ResourceHandle*)
 {
+    GST_ERROR_OBJECT(m_src, "Request was blocked");
+    GST_ELEMENT_ERROR(m_src, RESOURCE, OPEN_READ, ("Access to \"%s\" was blocked", m_src->priv->uri), (0));
 }
 
 void StreamingClient::cannotShowURL(ResourceHandle*)
 {
+    GST_ERROR_OBJECT(m_src, "Cannot show URL");
+    GST_ELEMENT_ERROR(m_src, RESOURCE, OPEN_READ, ("Can't show \"%s\"", m_src->priv->uri), (0));
 }
 
+#endif // USE(GSTREAMER)
+
