@@ -27,14 +27,13 @@
 #include "config.h"
 #include "FileChooser.h"
 
-#include "CString.h"
 #include "FileSystem.h"
 #include "Icon.h"
 #include "LocalizedStrings.h"
 #include "StringTruncator.h"
+#include <wtf/text/CString.h>
 
 #include <glib.h>
-#include <gtk/gtk.h>
 
 namespace WebCore {
 
@@ -57,9 +56,8 @@ String FileChooser::basenameForWidth(const Font& font, int width) const
     String string = fileButtonNoFileSelectedLabel();
 
     if (m_filenames.size() == 1) {
-        gchar* systemFilename = filenameFromString(m_filenames[0]);
-        gchar* systemBasename = g_path_get_basename(systemFilename);
-        g_free(systemFilename);
+        CString systemFilename = fileSystemRepresentation(m_filenames[0]);
+        gchar* systemBasename = g_path_get_basename(systemFilename.data());
         stringByAdoptingFileSystemRepresentation(systemBasename, string);
     } else if (m_filenames.size() > 1)
         return StringTruncator::rightTruncate(multipleFileUploadText(m_filenames.size()), width, font, false);
