@@ -36,12 +36,13 @@
 #include "Frame.h"
 #include "FrameView.h"
 #include "GraphicsContext.h"
+#include "GtkVersioning.h"
 #include "HostWindow.h"
 #include "IntRect.h"
+#include "MainFrameScrollbarGtk.h"
 #include "Page.h"
 #include "PlatformMouseEvent.h"
 #include "PlatformWheelEvent.h"
-#include "ScrollbarGtk.h"
 #include "ScrollbarTheme.h"
 
 #include <gtk/gtk.h>
@@ -52,8 +53,6 @@ namespace WebCore {
 
 void ScrollView::platformInit()
 {
-    m_horizontalAdjustment = 0;
-    m_verticalAdjustment = 0;
 }
 
 void ScrollView::platformDestroy()
@@ -64,54 +63,70 @@ void ScrollView::platformDestroy()
 
 PassRefPtr<Scrollbar> ScrollView::createScrollbar(ScrollbarOrientation orientation)
 {
-    if (orientation == HorizontalScrollbar && m_horizontalAdjustment)
-        return ScrollbarGtk::createScrollbar(this, orientation, m_horizontalAdjustment);
-    else if (orientation == VerticalScrollbar && m_verticalAdjustment)
-        return ScrollbarGtk::createScrollbar(this, orientation, m_verticalAdjustment);
-    else
+    // If this is an interior frame scrollbar, we want to create a totally fake
+    // scrollbar with no GtkAdjustment backing it.
+    if (parent())
         return Scrollbar::createNativeScrollbar(this, orientation, RegularScrollbar);
+
+    // If this is the main frame, we want to create a Scrollbar that does no  painting
+    // and defers to our GtkAdjustment for all of its state. Note that the GtkAdjustment
+    // may be null here.
+    if (orientation == HorizontalScrollbar)
+        return MainFrameScrollbarGtk::create(this, orientation, m_horizontalAdjustment.get());
+
+    // VerticalScrollbar
+    return MainFrameScrollbarGtk::create(this, orientation, m_verticalAdjustment.get());
 }
 
-#if !GTK_CHECK_VERSION(2, 14, 0)
-#define gtk_adjustment_configure AdjustmentConfigure
-
-static void AdjustmentConfigure(GtkAdjustment* adjustment, gdouble value, gdouble lower, gdouble upper,
-                                gdouble stepIncrement, gdouble pageIncrement, gdouble pageSize)
+void ScrollView::setHorizontalAdjustment(GtkAdjustment* hadj, bool resetValues)
 {
-    g_object_freeze_notify(G_OBJECT(adjustment));
-
-    g_object_set(adjustment,
-                 "lower", lower,
-                 "upper", upper,
-                 "step-increment", stepIncrement,
-                 "page-increment", pageIncrement,
-                 "page-size", pageSize,
-                 NULL);
-
-    g_object_thaw_notify(G_OBJECT(adjustment));
-
-    gtk_adjustment_changed(adjustment);
-    gtk_adjustment_value_changed(adjustment);
-}
-#endif
-
-/*
- * The following is assumed:
- *   (hadj && vadj) || (!hadj && !vadj)
- */
-void ScrollView::setGtkAdjustments(GtkAdjustment* hadj, GtkAdjustment* vadj, bool resetValues)
-{
-    ASSERT(!hadj == !vadj);
+    ASSERT(!parent() || !hadj);
+    if (parent())
+        return;
 
     m_horizontalAdjustment = hadj;
-    m_verticalAdjustment = vadj;
 
     if (!m_horizontalAdjustment) {
-        ScrollbarGtk* hScrollbar = reinterpret_cast<ScrollbarGtk*>(horizontalScrollbar());
+        MainFrameScrollbarGtk* hScrollbar = reinterpret_cast<MainFrameScrollbarGtk*>(horizontalScrollbar());
         if (hScrollbar)
             hScrollbar->detachAdjustment();
 
-        ScrollbarGtk* vScrollbar = reinterpret_cast<ScrollbarGtk*>(verticalScrollbar());
+        return;
+    }
+
+    // We may be lacking scrollbars when returning to a cached
+    // page, this kicks the page to recreate the scrollbars.
+    setHasHorizontalScrollbar(true);
+
+    MainFrameScrollbarGtk* hScrollbar = reinterpret_cast<MainFrameScrollbarGtk*>(horizontalScrollbar());
+    hScrollbar->attachAdjustment(m_horizontalAdjustment.get());
+
+    // We used to reset everything to 0 here, but when page cache
+    // is enabled we reuse FrameViews that are cached. Since their
+    // size is not going to change when being restored, (which is
+    // what would cause the upper limit in the adjusments to be
+    // set in the normal case), we make sure they are up-to-date
+    // here. This is needed for the parent scrolling widget to be
+    // able to report correct values.
+    int horizontalPageStep = max(max<int>(frameRect().width() * Scrollbar::minFractionToStepWhenPaging(), frameRect().width() - Scrollbar::maxOverlapBetweenPages()), 1);
+    gtk_adjustment_configure(m_horizontalAdjustment.get(),
+                             resetValues ? 0 : scrollOffset().width(), 0,
+                             resetValues ? 0 : contentsSize().width(),
+                             resetValues ? 0 : Scrollbar::pixelsPerLineStep(),
+                             resetValues ? 0 : horizontalPageStep,
+                             resetValues ? 0 : frameRect().width());
+}
+
+void ScrollView::setVerticalAdjustment(GtkAdjustment* vadj, bool resetValues)
+{
+    ASSERT(!parent() || !vadj);
+    if (parent())
+        return;
+
+    m_verticalAdjustment = vadj;
+
+    if (!m_verticalAdjustment) {
+        MainFrameScrollbarGtk* vScrollbar = reinterpret_cast<MainFrameScrollbarGtk*>(verticalScrollbar());
         if (vScrollbar)
             vScrollbar->detachAdjustment();
 
@@ -121,13 +136,9 @@ void ScrollView::setGtkAdjustments(GtkAdjustment* hadj, GtkAdjustment* vadj, boo
     // We may be lacking scrollbars when returning to a cached
     // page, this kicks the page to recreate the scrollbars.
     setHasVerticalScrollbar(true);
-    setHasHorizontalScrollbar(true);
 
-    ScrollbarGtk* hScrollbar = reinterpret_cast<ScrollbarGtk*>(horizontalScrollbar());
-    hScrollbar->attachAdjustment(m_horizontalAdjustment);
-
-    ScrollbarGtk* vScrollbar = reinterpret_cast<ScrollbarGtk*>(verticalScrollbar());
-    vScrollbar->attachAdjustment(m_verticalAdjustment);
+    MainFrameScrollbarGtk* vScrollbar = reinterpret_cast<MainFrameScrollbarGtk*>(verticalScrollbar());
+    vScrollbar->attachAdjustment(m_verticalAdjustment.get());
 
     // We used to reset everything to 0 here, but when page cache
     // is enabled we reuse FrameViews that are cached. Since their
@@ -136,17 +147,8 @@ void ScrollView::setGtkAdjustments(GtkAdjustment* hadj, GtkAdjustment* vadj, boo
     // set in the normal case), we make sure they are up-to-date
     // here. This is needed for the parent scrolling widget to be
     // able to report correct values.
-
-    int horizontalPageStep = max(max<int>(frameRect().width() * Scrollbar::minFractionToStepWhenPaging(), frameRect().width() - Scrollbar::maxOverlapBetweenPages()), 1);
-    gtk_adjustment_configure(m_horizontalAdjustment,
-                             resetValues ? 0 : scrollOffset().width(), 0,
-                             resetValues ? 0 : contentsSize().width(),
-                             resetValues ? 0 : Scrollbar::pixelsPerLineStep(),
-                             resetValues ? 0 : horizontalPageStep,
-                             resetValues ? 0 : frameRect().width());
-
-    int verticalPageStep = max(max<int>(frameRect().height() * Scrollbar::minFractionToStepWhenPaging(), frameRect().height() - Scrollbar::maxOverlapBetweenPages()), 1);
-    gtk_adjustment_configure(m_verticalAdjustment,
+    int verticalPageStep = max(max<int>(frameRect().width() * Scrollbar::minFractionToStepWhenPaging(), frameRect().width() - Scrollbar::maxOverlapBetweenPages()), 1);
+    gtk_adjustment_configure(m_verticalAdjustment.get(),
                              resetValues ? 0 : scrollOffset().height(), 0,
                              resetValues ? 0 : contentsSize().height(),
                              resetValues ? 0 : Scrollbar::pixelsPerLineStep(),
@@ -154,69 +156,81 @@ void ScrollView::setGtkAdjustments(GtkAdjustment* hadj, GtkAdjustment* vadj, boo
                              resetValues ? 0 : frameRect().height());
 }
 
+void ScrollView::setGtkAdjustments(GtkAdjustment* hadj, GtkAdjustment* vadj, bool resetValues)
+{
+    setHorizontalAdjustment(hadj, resetValues);
+    setVerticalAdjustment(vadj, resetValues);
+}
+
 void ScrollView::platformAddChild(Widget* child)
 {
-    if (!GTK_IS_SOCKET(child->platformWidget()))
-        gtk_container_add(GTK_CONTAINER(hostWindow()->platformPageClient()), child->platformWidget());
 }
 
 void ScrollView::platformRemoveChild(Widget* child)
 {
-    GtkWidget* parent;
-
-    // HostWindow can be NULL here. If that's the case
-    // let's grab the child's parent instead.
-    if (hostWindow())
-        parent = GTK_WIDGET(hostWindow()->platformPageClient());
-    else
-        parent = GTK_WIDGET(child->platformWidget()->parent);
-
-    if (GTK_IS_CONTAINER(parent) && parent == child->platformWidget()->parent)
-        gtk_container_remove(GTK_CONTAINER(parent), child->platformWidget());
 }
 
 IntRect ScrollView::visibleContentRect(bool includeScrollbars) const
 {
-    if (!m_horizontalAdjustment)
+    // If we are an interior frame scrollbar or are in some sort of transition
+    // state, just calculate our size based on what the GTK+ theme says the
+    // scrollbar width should be.
+    if (parent() || !hostWindow() || !hostWindow()->platformPageClient()) {
         return IntRect(IntPoint(m_scrollOffset.width(), m_scrollOffset.height()),
                        IntSize(max(0, width() - (verticalScrollbar() && !includeScrollbars ? verticalScrollbar()->width() : 0)),
                                max(0, height() - (horizontalScrollbar() && !includeScrollbars ? horizontalScrollbar()->height() : 0))));
+    }
 
-    // Main frame.
+    // We don't have a parent, so we are the main frame and thus have
+    // a parent widget which we can use to measure the visible region.
     GtkWidget* measuredWidget = hostWindow()->platformPageClient();
-    GtkWidget* parent = gtk_widget_get_parent(measuredWidget);
+    GtkWidget* parentWidget = gtk_widget_get_parent(measuredWidget);
 
     // We may not be in a widget that displays scrollbars, but we may
     // have other kinds of decoration that make us smaller.
-    if (parent && includeScrollbars)
-        measuredWidget = parent;
+    if (parentWidget && includeScrollbars)
+        measuredWidget = parentWidget;
 
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(measuredWidget, &allocation);
     return IntRect(IntPoint(m_scrollOffset.width(), m_scrollOffset.height()),
-                   IntSize(measuredWidget->allocation.width,
-                           measuredWidget->allocation.height));
+                   IntSize(allocation.width, allocation.height));
 }
 
-void ScrollView::setScrollbarModes(ScrollbarMode horizontalMode, ScrollbarMode verticalMode)
+void ScrollView::setScrollbarModes(ScrollbarMode horizontalMode, ScrollbarMode verticalMode, bool horizontalLock, bool verticalLock)
 {
-    if (horizontalMode == m_horizontalScrollbarMode && verticalMode == m_verticalScrollbarMode)
-        return;
+    // FIXME: Restructure the ScrollView abstraction so that we do not have to
+    // copy this verbatim from ScrollView.cpp. Until then, we should make sure this
+    // is kept in sync.
+    bool needsUpdate = false;
 
-    m_horizontalScrollbarMode = horizontalMode;
-    m_verticalScrollbarMode = verticalMode;
-
-    // We don't really care about reporting policy changes on frames
-    // that have no adjustments attached to them.
-    if (!m_horizontalAdjustment) {
-        updateScrollbars(scrollOffset());
-        return;
+    if (horizontalMode != horizontalScrollbarMode() && !m_horizontalScrollbarLock) {
+        m_horizontalScrollbarMode = horizontalMode;
+        needsUpdate = true;
     }
 
-    if (!isFrameView())
+    if (verticalMode != verticalScrollbarMode() && !m_verticalScrollbarLock) {
+        m_verticalScrollbarMode = verticalMode;
+        needsUpdate = true;
+    }
+
+    if (horizontalLock)
+        setHorizontalScrollbarLock();
+
+    if (verticalLock)
+        setVerticalScrollbarLock();
+
+    if (needsUpdate)
+        updateScrollbars(scrollOffset());
+
+    // We don't need to report policy changes on ScrollView's unless this
+    // one has an adjustment attached and it is a main frame.
+    if (!m_horizontalAdjustment || parent() || !isFrameView())
         return;
 
     // For frames that do have adjustments attached, we want to report
     // policy changes, so that they may be applied to the widget to
-    // which the WebView has been added, for instance.
+    // which the WebView's container (e.g. GtkScrolledWindow).
     if (hostWindow())
         hostWindow()->scrollbarsModeDidChange();
 }
