@@ -48,7 +48,6 @@
 #ifndef _GTK_DRAWING_H_
 #define _GTK_DRAWING_H_
 
-#include <gdk/gdk.h>
 #include <gtk/gtk.h>
 
 #ifdef __cplusplus
@@ -76,32 +75,43 @@ typedef struct {
   gint stepper_size;
   gint stepper_spacing;
   gint min_slider_size;
+  gboolean trough_under_steppers;
+  gboolean has_secondary_forward_stepper;
+  gboolean has_secondary_backward_stepper;
 } MozGtkScrollbarMetrics;
 
+typedef struct _GtkThemeParts {
+#ifdef GTK_API_VERSION_2
+    GdkColormap* colormap;
+#endif // GTK_API_VERSION_2
+    GtkWidget* protoWindow;
+    GtkWidget* protoLayout;
+    GtkWidget* buttonWidget;
+    GtkWidget* toggleButtonWidget;
+    GtkWidget* buttonArrowWidget;
+    GtkWidget* checkboxWidget;
+    GtkWidget* radiobuttonWidget;
+    GtkWidget* horizScrollbarWidget;
+    GtkWidget* vertScrollbarWidget;
+    GtkWidget* entryWidget;
+    GtkWidget* comboBoxWidget;
+    GtkWidget* comboBoxButtonWidget;
+    GtkWidget* comboBoxArrowWidget;
+    GtkWidget* comboBoxSeparatorWidget;
+    GtkWidget* comboBoxEntryWidget;
+    GtkWidget* comboBoxEntryTextareaWidget;
+    GtkWidget* comboBoxEntryButtonWidget;
+    GtkWidget* comboBoxEntryArrowWidget;
+    GtkWidget* progresWidget;
+    GtkWidget* scrolledWindowWidget;
+} GtkThemeParts;
+
 typedef enum {
   MOZ_GTK_STEPPER_DOWN        = 1 << 0,
   MOZ_GTK_STEPPER_BOTTOM      = 1 << 1,
   MOZ_GTK_STEPPER_VERTICAL    = 1 << 2
 } GtkScrollbarButtonFlags;
 
-/** flags for tab state **/
-typedef enum {
-  /* first eight bits are used to pass a margin */
-  MOZ_GTK_TAB_MARGIN_MASK     = 0xFF,
-  /* bottom tabs */
-  MOZ_GTK_TAB_BOTTOM          = 1 << 8,
-  /* the first tab in the group */
-  MOZ_GTK_TAB_FIRST           = 1 << 9,
-  /* the selected tab */
-  MOZ_GTK_TAB_SELECTED        = 1 << 10
-} GtkTabFlags;
-
-/** flags for menuitems **/
-typedef enum {
-  /* menuitem is part of the menubar */
-  MOZ_TOPLEVEL_MENU_ITEM      = 1 << 0
-} GtkMenuItemFlags;
-
 /* function type for moz_gtk_enable_style_props */
 typedef gint (*style_prop_t)(GtkStyle*, const gchar*, gint);
 
@@ -133,86 +143,15 @@ typedef enum {
   /* Paints the slider (thumb) of a GtkScrollbar. */
   MOZ_GTK_SCROLLBAR_THUMB_HORIZONTAL,
   MOZ_GTK_SCROLLBAR_THUMB_VERTICAL,
-  /* Paints a GtkScale. */
-  MOZ_GTK_SCALE_HORIZONTAL,
-  MOZ_GTK_SCALE_VERTICAL,
-  /* Paints a GtkScale thumb. */
-  MOZ_GTK_SCALE_THUMB_HORIZONTAL,
-  MOZ_GTK_SCALE_THUMB_VERTICAL,
-  /* Paints a GtkSpinButton */
-  MOZ_GTK_SPINBUTTON,
-  MOZ_GTK_SPINBUTTON_UP,
-  MOZ_GTK_SPINBUTTON_DOWN,
-  MOZ_GTK_SPINBUTTON_ENTRY,
-  /* Paints the gripper of a GtkHandleBox. */
-  MOZ_GTK_GRIPPER,
-  /* Paints a GtkEntry. */
+  /* Paints the background of a scrolled window */
+  MOZ_GTK_SCROLLED_WINDOW,
   MOZ_GTK_ENTRY,
-  /* Paints the native caret (or in GTK-speak: insertion cursor) */
-  MOZ_GTK_ENTRY_CARET,
   /* Paints a GtkOptionMenu. */
   MOZ_GTK_DROPDOWN,
-  /* Paints a dropdown arrow (a GtkButton containing a down GtkArrow). */
-  MOZ_GTK_DROPDOWN_ARROW,
-  /* Paints an entry in an editable option menu */
-  MOZ_GTK_DROPDOWN_ENTRY,
-  /* Paints the container part of a GtkCheckButton. */
-  MOZ_GTK_CHECKBUTTON_CONTAINER,
-  /* Paints the container part of a GtkRadioButton. */
-  MOZ_GTK_RADIOBUTTON_CONTAINER,
-  /* Paints the label of a GtkCheckButton (focus outline) */
-  MOZ_GTK_CHECKBUTTON_LABEL,
-  /* Paints the label of a GtkRadioButton (focus outline) */
-  MOZ_GTK_RADIOBUTTON_LABEL,
-  /* Paints the background of a GtkHandleBox. */
-  MOZ_GTK_TOOLBAR,
-  /* Paints a toolbar separator */
-  MOZ_GTK_TOOLBAR_SEPARATOR,
-  /* Paints a GtkToolTip */
-  MOZ_GTK_TOOLTIP,
-  /* Paints a GtkFrame (e.g. a status bar panel). */
-  MOZ_GTK_FRAME,
-  /* Paints a resize grip for a GtkWindow */
-  MOZ_GTK_RESIZER,
   /* Paints a GtkProgressBar. */
   MOZ_GTK_PROGRESSBAR,
   /* Paints a progress chunk of a GtkProgressBar. */
-  MOZ_GTK_PROGRESS_CHUNK,
-  /* Paints a tab of a GtkNotebook. flags is a GtkTabFlags, defined above. */
-  MOZ_GTK_TAB,
-  /* Paints the background and border of a GtkNotebook. */
-  MOZ_GTK_TABPANELS,
-  /* Paints a GtkArrow for a GtkNotebook. flags is a GtkArrowType. */
-  MOZ_GTK_TAB_SCROLLARROW,
-  /* Paints the background and border of a GtkTreeView */
-  MOZ_GTK_TREEVIEW,
-  /* Paints treeheader cells */
-  MOZ_GTK_TREE_HEADER_CELL,
-  /* Paints sort arrows in treeheader cells */
-  MOZ_GTK_TREE_HEADER_SORTARROW,
-  /* Paints an expander for a GtkTreeView */
-  MOZ_GTK_TREEVIEW_EXPANDER,
-  /* Paints a GtkExpander */
-  MOZ_GTK_EXPANDER,
-  /* Paints the background of the menu bar. */
-  MOZ_GTK_MENUBAR,
-  /* Paints the background of menus, context menus. */
-  MOZ_GTK_MENUPOPUP,
-  /* Paints the arrow of menuitems that contain submenus */
-  MOZ_GTK_MENUARROW,
-  /* Paints an arrow that points down */
-  MOZ_GTK_TOOLBARBUTTON_ARROW,
-  /* Paints items of menubar and popups. */
-  MOZ_GTK_MENUITEM,
-  MOZ_GTK_CHECKMENUITEM,
-  MOZ_GTK_RADIOMENUITEM,
-  MOZ_GTK_MENUSEPARATOR,
-  /* Paints a GtkVPaned separator */
-  MOZ_GTK_SPLITTER_HORIZONTAL,
-  /* Paints a GtkHPaned separator */
-  MOZ_GTK_SPLITTER_VERTICAL,
-  /* Paints the background of a window, dialog or page. */
-  MOZ_GTK_WINDOW
+  MOZ_GTK_PROGRESS_CHUNK
 } GtkThemeWidgetType;
 
 /*** General library functions ***/
@@ -225,6 +164,14 @@ typedef enum {
  */
 gint moz_gtk_init();
 
+/**
+ * Instruct the drawing library to do all rendering based on
+ * the given collection of theme parts. If any members of the
+ * GtkThemeParts struct are NULL, they will be created lazily.
+ */
+void
+moz_gtk_use_theme_parts(GtkThemeParts* parts);
+
 /**
  * Enable GTK+ 1.2.9+ theme enhancements. You must provide a pointer
  * to the GTK+ 1.2.9+ function "gtk_style_get_prop_experimental".
@@ -242,6 +189,11 @@ gint moz_gtk_enable_style_props(style_prop_t styleGetProp);
  */
 gint moz_gtk_shutdown();
 
+/**
+ * Destroy the widgets in the given GtkThemeParts, which should
+ * be destroyed before the GtkThemeParts can be freed.
+ */
+void moz_gtk_destroy_theme_parts_widgets(GtkThemeParts* parts);
 
 /*** Widget drawing ***/
 /**
@@ -253,12 +205,18 @@ gint moz_gtk_shutdown();
  * flags:     widget-dependant flags; see the GtkThemeWidgetType definition.
  * direction: the text direction, to draw the widget correctly LTR and RTL.
  */
+#ifdef GTK_API_VERSION_2
 gint
 moz_gtk_widget_paint(GtkThemeWidgetType widget, GdkDrawable* drawable,
                      GdkRectangle* rect, GdkRectangle* cliprect,
                      GtkWidgetState* state, gint flags,
                      GtkTextDirection direction);
-
+#else
+gint
+moz_gtk_widget_paint(GtkThemeWidgetType widget, cairo_t* cr,
+                     GdkRectangle* rect, GtkWidgetState* state,
+                     gint flags, GtkTextDirection direction);
+#endif
 
 /*** Widget metrics ***/
 /**
@@ -297,16 +255,6 @@ moz_gtk_checkbox_get_metrics(gint* indicator_size, gint* indicator_spacing);
 gint
 moz_gtk_radio_get_metrics(gint* indicator_size, gint* indicator_spacing);
 
-/**
- * Get the inner-border value for a GtkButton widget (button or tree header)
- * widget:             [IN]  the widget to get the border value for 
- * inner_border:       [OUT] the inner border
- *
- * returns:   MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint
-moz_gtk_button_get_inner_border(GtkWidget* widget, GtkBorder* inner_border);
-
 /** Get the focus metrics for a treeheadercell, button, checkbox, or radio button.
  * widget:             [IN]  the widget to get the focus metrics for    
  * interior_focus:     [OUT] whether the focus is drawn around the
@@ -320,17 +268,6 @@ gint
 moz_gtk_widget_get_focus(GtkWidget* widget, gboolean* interior_focus,
                          gint* focus_width, gint* focus_pad);
 
-/**
- * Get the desired size of a GtkScale thumb
- * orient:           [IN] the scale orientation
- * thumb_length:     [OUT] the length of the thumb
- * thumb_height:     [OUT] the height of the thumb
- *
- * returns:    MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint
-moz_gtk_get_scalethumb_metrics(GtkOrientation orient, gint* thumb_length, gint* thumb_height);
-
 /**
  * Get the desired metrics for a GtkScrollbar
  * metrics:          [IN]  struct which will contain the metrics
@@ -340,74 +277,6 @@ moz_gtk_get_scalethumb_metrics(GtkOrientation orient, gint* thumb_length, gint*
 gint
 moz_gtk_get_scrollbar_metrics(MozGtkScrollbarMetrics* metrics);
 
-/**
- * Get the desired size of a dropdown arrow button
- * width:   [OUT] the desired width
- * height:  [OUT] the desired height
- *
- * returns:    MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_get_combo_box_entry_button_size(gint* width, gint* height);
-
-/**
- * Get the desired size of a scroll arrow widget
- * width:   [OUT] the desired width
- * height:  [OUT] the desired height
- *
- * returns:    MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_get_tab_scroll_arrow_size(gint* width, gint* height);
-
-/**
- * Get the desired size of a toolbar button dropdown arrow
- * width:   [OUT] the desired width
- * height:  [OUT] the desired height
- *
- * returns:    MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_get_downarrow_size(gint* width, gint* height);
-
-/**
- * Get the desired size of a toolbar separator
- * size:    [OUT] the desired width
- *
- * returns: MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_get_toolbar_separator_width(gint* size);
-
-/**
- * Get the size of a regular GTK expander that shows/hides content
- * size:    [OUT] the size of the GTK expander, size = width = height.
- *
- * returns:    MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_get_expander_size(gint* size);
-
-/**
- * Get the size of a treeview's expander (we call them twisties)
- * size:    [OUT] the size of the GTK expander, size = width = height.
- *
- * returns:    MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_get_treeview_expander_size(gint* size);
-
-/**
- * Get the desired height of a menu separator
- * size:    [OUT] the desired height
- *
- * returns: MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_get_menu_separator_height(gint* size);
-
-/**
- * Get the desired size of a splitter
- * orientation:   [IN]  GTK_ORIENTATION_HORIZONTAL or GTK_ORIENTATION_VERTICAL
- * size:          [OUT] width or height of the splitter handle
- *
- * returns:    MOZ_GTK_SUCCESS if there was no error, an error code otherwise
- */
-gint moz_gtk_splitter_get_metrics(gint orientation, gint* size);
-
 /**
  * Retrieve an actual GTK scrollbar widget for style analysis. It will not
  * be modified.
@@ -415,15 +284,10 @@ gint moz_gtk_splitter_get_metrics(gint orientation, gint* size);
 GtkWidget* moz_gtk_get_scrollbar_widget(void);
 
 /**
- * Get the YTHICKNESS of a tab (notebook extension).
+ * Retrieve an actual GTK progress bar widget for style analysis. It will not
+ * be modified.
  */
-gint moz_gtk_get_tab_thickness(void);
-
-/**
- * Get a boolean which indicates whether or not to use images in menus.
- * If TRUE, use images in menus.
- */
-gboolean moz_gtk_images_in_menus(void);
+GtkWidget* moz_gtk_get_progress_widget(void);
 
 #ifdef __cplusplus
 }
