@@ -40,8 +40,10 @@ namespace WebCore {
 // don't pack).
 static const size_t sizeOfFileHeader = 14;
 
-BMPImageDecoder::BMPImageDecoder()
-    : m_decodedOffset(0)
+BMPImageDecoder::BMPImageDecoder(ImageSource::AlphaOption alphaOption,
+                                 ImageSource::GammaAndColorProfileOption gammaAndColorProfileOption)
+    : ImageDecoder(alphaOption, gammaAndColorProfileOption)
+    , m_decodedOffset(0)
 {
 }
 
@@ -68,8 +70,10 @@ RGBA32Buffer* BMPImageDecoder::frameBufferAtIndex(size_t index)
     if (index)
         return 0;
 
-    if (m_frameBufferCache.isEmpty())
+    if (m_frameBufferCache.isEmpty()) {
         m_frameBufferCache.resize(1);
+        m_frameBufferCache.first().setPremultiplyAlpha(m_premultiplyAlpha);
+    }
 
     RGBA32Buffer* buffer = &m_frameBufferCache.first();
     if (buffer->status() != RGBA32Buffer::FrameComplete)
@@ -77,6 +81,12 @@ RGBA32Buffer* BMPImageDecoder::frameBufferAtIndex(size_t index)
     return buffer;
 }
 
+bool BMPImageDecoder::setFailed()
+{
+    m_reader.clear();
+    return ImageDecoder::setFailed();
+}
+
 void BMPImageDecoder::decode(bool onlySize)
 {
     if (failed())
@@ -86,6 +96,10 @@ void BMPImageDecoder::decode(bool onlySize)
     // has failed.
     if (!decodeHelper(onlySize) && isAllDataReceived())
         setFailed();
+    // If we're done decoding the image, we don't need the BMPImageReader
+    // anymore.  (If we failed, |m_reader| has already been cleared.)
+    else if (!m_frameBufferCache.isEmpty() && (m_frameBufferCache.first().status() == RGBA32Buffer::FrameComplete))
+        m_reader.clear();
 }
 
 bool BMPImageDecoder::decodeHelper(bool onlySize)
