@@ -29,8 +29,10 @@
 
 namespace WebCore {
 
-GIFImageDecoder::GIFImageDecoder()
-    : m_alreadyScannedThisDataForFrameCount(true)
+GIFImageDecoder::GIFImageDecoder(ImageSource::AlphaOption alphaOption,
+                                 ImageSource::GammaAndColorProfileOption gammaAndColorProfileOption)
+    : ImageDecoder(alphaOption, gammaAndColorProfileOption)
+    , m_alreadyScannedThisDataForFrameCount(true)
     , m_repetitionCount(cAnimationLoopOnce)
     , m_readOffset(0)
 {
@@ -59,8 +61,11 @@ bool GIFImageDecoder::isSizeAvailable()
     return ImageDecoder::isSizeAvailable();
 }
 
-bool GIFImageDecoder::setSize(unsigned width, unsigned height)
+bool GIFImageDecoder::setSize(int width, int height)
 {
+    if (ImageDecoder::isSizeAvailable() && size().width() == width && size().height() == height)
+        return true;
+
     if (!ImageDecoder::setSize(width, height))
         return false;
 
@@ -80,6 +85,8 @@ size_t GIFImageDecoder::frameCount()
         reader.read((const unsigned char*)m_data->data(), m_data->size(), GIFFrameCountQuery, static_cast<unsigned>(-1));
         m_alreadyScannedThisDataForFrameCount = true;
         m_frameBufferCache.resize(reader.images_count);
+        for (int i = 0; i < reader.images_count; ++i)
+            m_frameBufferCache[i].setPremultiplyAlpha(m_premultiplyAlpha);
     }
 
     return m_frameBufferCache.size();
@@ -90,21 +97,24 @@ int GIFImageDecoder::repetitionCount() const
     // This value can arrive at any point in the image data stream.  Most GIFs
     // in the wild declare it near the beginning of the file, so it usually is
     // set by the time we've decoded the size, but (depending on the GIF and the
-    // packets sent back by the webserver) not always.  Our caller is
-    // responsible for waiting until image decoding has finished to ask this if
-    // it needs an authoritative answer.  In the meantime, we should default to
-    // "loop once".
-    if (m_reader) {
-        // Added wrinkle: ImageSource::clear() may destroy the reader, making
-        // the result from the reader _less_ authoritative on future calls.  To
-        // detect this, the reader returns cLoopCountNotSeen (-2) instead of
-        // cAnimationLoopOnce (-1) when its current incarnation hasn't actually
-        // seen a loop count yet; in this case we return our previously-cached
-        // value.
-        const int repetitionCount = m_reader->loop_count;
-        if (repetitionCount != cLoopCountNotSeen)
-            m_repetitionCount = repetitionCount;
-    }
+    // packets sent back by the webserver) not always.  If the reader hasn't
+    // seen a loop count yet, it will return cLoopCountNotSeen, in which case we
+    // should default to looping once (the initial value for
+    // |m_repetitionCount|).
+    //
+    // There are two additional wrinkles here.  First, ImageSource::clear() may
+    // destroy the reader, making the result from the reader _less_
+    // authoritative on future calls if the recreated reader hasn't seen the
+    // loop count.  We don't need to special-case this because in this case the
+    // new reader will once again return cLoopCountNotSeen, and we won't
+    // overwrite the cached correct value.
+    //
+    // Second, a GIF might never set a loop count at all, in which case we
+    // should continue to treat it as a "loop once" animation.  We don't need
+    // special code here either, because in this case we'll never change
+    // |m_repetitionCount| from its default value.
+    if (m_reader && (m_reader->loop_count != cLoopCountNotSeen))
+        m_repetitionCount = m_reader->loop_count;
     return m_repetitionCount;
 }
 
@@ -119,6 +129,12 @@ RGBA32Buffer* GIFImageDecoder::frameBufferAtIndex(size_t index)
     return &frame;
 }
 
+bool GIFImageDecoder::setFailed()
+{
+    m_reader.clear();
+    return ImageDecoder::setFailed();
+}
+
 void GIFImageDecoder::clearFrameBufferCache(size_t clearBeforeFrame)
 {
     // In some cases, like if the decoder was destroyed while animating, we
@@ -185,8 +201,8 @@ bool GIFImageDecoder::haveDecodedRow(unsigned frameIndex, unsigned char* rowBuff
     // row's X-coordinates.
     int xBegin = upperBoundScaledX(frameReader->x_offset);
     int yBegin = upperBoundScaledY(frameReader->y_offset + rowNumber);
-    int xEnd = lowerBoundScaledX(std::min(xBegin + static_cast<int>(rowEnd - rowBuffer), size().width()) - 1, xBegin + 1) + 1;
-    int yEnd = lowerBoundScaledY(std::min(yBegin + static_cast<int>(repeatCount), size().height()) - 1, yBegin + 1) + 1;
+    int xEnd = lowerBoundScaledX(std::min(static_cast<int>(frameReader->x_offset + (rowEnd - rowBuffer)), size().width()) - 1, xBegin + 1) + 1;
+    int yEnd = lowerBoundScaledY(std::min(static_cast<int>(frameReader->y_offset + rowNumber + repeatCount), size().height()) - 1, yBegin + 1) + 1;
     if (!rowBuffer || (xBegin < 0) || (yBegin < 0) || (xEnd <= xBegin) || (yEnd <= yBegin))
         return true;
 
@@ -235,13 +251,13 @@ bool GIFImageDecoder::haveDecodedRow(unsigned frameIndex, unsigned char* rowBuff
     return true;
 }
 
-void GIFImageDecoder::frameComplete(unsigned frameIndex, unsigned frameDuration, RGBA32Buffer::FrameDisposalMethod disposalMethod)
+bool GIFImageDecoder::frameComplete(unsigned frameIndex, unsigned frameDuration, RGBA32Buffer::FrameDisposalMethod disposalMethod)
 {
     // Initialize the frame if necessary.  Some GIFs insert do-nothing frames,
     // in which case we never reach haveDecodedRow() before getting here.
     RGBA32Buffer& buffer = m_frameBufferCache[frameIndex];
     if ((buffer.status() == RGBA32Buffer::FrameEmpty) && !initFrameBuffer(frameIndex))
-        return;
+        return false; // initFrameBuffer() has already called setFailed().
 
     buffer.setStatus(RGBA32Buffer::FrameComplete);
     buffer.setDuration(frameDuration);
@@ -277,12 +293,16 @@ void GIFImageDecoder::frameComplete(unsigned frameIndex, unsigned frameDuration,
                 buffer.setHasAlpha(false);
         }
     }
+
+    return true;
 }
 
 void GIFImageDecoder::gifComplete()
 {
-    if (m_reader)
-        m_repetitionCount = m_reader->loop_count;
+    // Cache the repetition count, which is now as authoritative as it's ever
+    // going to be.
+    repetitionCount();
+
     m_reader.clear();
 }
 
@@ -294,11 +314,10 @@ void GIFImageDecoder::decode(unsigned haltAtFrame, GIFQuery query)
     if (!m_reader)
         m_reader.set(new GIFImageReader(this));
 
-    if (!m_reader->read((const unsigned char*)m_data->data() + m_readOffset, m_data->size() - m_readOffset, query, haltAtFrame))
+    // If we couldn't decode the image but we've received all the data, decoding
+    // has failed.
+    if (!m_reader->read((const unsigned char*)m_data->data() + m_readOffset, m_data->size() - m_readOffset, query, haltAtFrame) && isAllDataReceived())
         setFailed();
-
-    if (failed())
-        m_reader.clear();
 }
 
 bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
@@ -343,7 +362,7 @@ bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
 
         if ((prevMethod == RGBA32Buffer::DisposeNotSpecified) || (prevMethod == RGBA32Buffer::DisposeKeep)) {
             // Preserve the last frame as the starting state for this frame.
-            if (!buffer->copyBitmapData(*prevBuffer));
+            if (!buffer->copyBitmapData(*prevBuffer))
                 return setFailed();
         } else {
             // We want to clear the previous frame to transparent, without
@@ -357,7 +376,7 @@ bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
                     return setFailed();
             } else {
               // Copy the whole previous buffer, then clear just its frame.
-              if (!buffer->copyBitmapData(*prevBuffer));
+              if (!buffer->copyBitmapData(*prevBuffer))
                   return setFailed();
               for (int y = prevRect.y(); y < prevRect.bottom(); ++y) {
                   for (int x = prevRect.x(); x < prevRect.right(); ++x)
