@@ -40,7 +40,7 @@ namespace WebCore {
     // This class decodes the ICO and CUR image formats.
     class ICOImageDecoder : public ImageDecoder {
     public:
-        ICOImageDecoder();
+        ICOImageDecoder(ImageSource::AlphaOption, ImageSource::GammaAndColorProfileOption);
         virtual ~ICOImageDecoder();
 
         // ImageDecoder
@@ -52,6 +52,10 @@ namespace WebCore {
         virtual bool setSize(unsigned width, unsigned height);
         virtual size_t frameCount();
         virtual RGBA32Buffer* frameBufferAtIndex(size_t);
+        // CAUTION: setFailed() deletes all readers and decoders.  Be careful to
+        // avoid accessing deleted memory, especially when calling this from
+        // inside BMPImageReader!
+        virtual bool setFailed();
 
     private:
         enum ImageType {
