@@ -51,7 +51,13 @@
 #endif
 
 extern "C" {
+
 #include "jpeglib.h"
+
+#if USE(ICCJPEG)
+#include "iccjpeg.h"
+#endif
+
 }
 
 #include <setjmp.h>
@@ -69,7 +75,6 @@ enum jstate {
     JPEG_DECOMPRESS_PROGRESSIVE, // Output progressive pixels
     JPEG_DECOMPRESS_SEQUENTIAL,  // Output sequential pixels
     JPEG_DONE,
-    JPEG_SINK_NON_JPEG_TRAILER,  // Some image files have a non-JPEG trailer
     JPEG_ERROR    
 };
 
@@ -87,6 +92,24 @@ struct decoder_source_mgr {
     JPEGImageReader* decoder;
 };
 
+static ColorProfile readColorProfile(jpeg_decompress_struct* info)
+{
+#if USE(ICCJPEG)
+    JOCTET* profile;
+    unsigned int profileLength;
+
+    if (!read_icc_profile(info, &profile, &profileLength))
+        return ColorProfile();
+
+    ColorProfile colorProfile;
+    colorProfile.append(reinterpret_cast<char*>(profile), profileLength);
+    free(profile);
+    return colorProfile;
+#else
+    return ColorProfile();
+#endif
+}
+
 class JPEGImageReader
 {
 public:
@@ -124,6 +147,11 @@ public:
         src->pub.resync_to_restart = jpeg_resync_to_restart;
         src->pub.term_source = term_source;
         src->decoder = this;
+
+        // Enable these markers for the ICC color profile.
+        // Apparently there are 16 of these markers.  I don't see anywhere in the header with this constant.
+        for (unsigned i = 0; i < 0xF; ++i)
+            jpeg_save_markers(&m_info, JPEG_APP0 + i, 0xFFFF);
     }
 
     ~JPEGImageReader()
@@ -168,23 +196,25 @@ public:
         m_bufferLength = data.size();
         
         // We need to do the setjmp here. Otherwise bad things will happen
-        if (setjmp(m_err.setjmp_buffer)) {
-            m_state = JPEG_SINK_NON_JPEG_TRAILER;
-            close();
-            return false;
-        }
+        if (setjmp(m_err.setjmp_buffer))
+            return m_decoder->setFailed();
 
         switch (m_state) {
         case JPEG_HEADER:
             // Read file parameters with jpeg_read_header().
             if (jpeg_read_header(&m_info, true) == JPEG_SUSPENDED)
-                return true; // I/O suspension.
+                return false; // I/O suspension.
 
             // Let libjpeg take care of gray->RGB and YCbCr->RGB conversions.
             switch (m_info.jpeg_color_space) {
             case JCS_GRAYSCALE:
-            case JCS_RGB:
             case JCS_YCbCr:
+                // Grayscale images get "upsampled" by libjpeg.  If we use
+                // their color profile, CoreGraphics will "upsample" them
+                // again, resulting in horizontal distortions.
+                m_decoder->setIgnoreGammaAndColorProfile(true);
+                // Note fall-through!
+            case JCS_RGB:
                 m_info.out_color_space = JCS_RGB;
                 break;
             case JCS_CMYK:
@@ -194,7 +224,7 @@ public:
                 m_info.out_color_space = JCS_CMYK;
                 break;
             default:
-                return false;
+                return m_decoder->setFailed();
             }
 
             // Don't allocate a giant and superfluous memory buffer when the
@@ -216,6 +246,9 @@ public:
             if (!m_decoder->setSize(m_info.image_width, m_info.image_height))
                 return false;
 
+            if (!m_decoder->ignoresGammaAndColorProfile())
+                m_decoder->setColorProfile(readColorProfile(info()));
+
             if (m_decodingSizeOnly) {
                 // We can stop here.  Reduce our buffer length and available
                 // data.
@@ -237,7 +270,7 @@ public:
 
             // Start decompressor.
             if (!jpeg_start_decompress(&m_info))
-                return true; // I/O suspension.
+                return false; // I/O suspension.
 
             // If this is a progressive JPEG ...
             m_state = (m_info.buffered_image) ? JPEG_DECOMPRESS_PROGRESSIVE : JPEG_DECOMPRESS_SEQUENTIAL;
@@ -247,7 +280,7 @@ public:
             if (m_state == JPEG_DECOMPRESS_SEQUENTIAL) {
   
                 if (!m_decoder->outputScanlines())
-                    return true; // I/O suspension.
+                    return false; // I/O suspension.
   
                 // If we've completed image output...
                 ASSERT(m_info.output_scanline == m_info.output_height);
@@ -273,7 +306,7 @@ public:
                             --scan;
 
                         if (!jpeg_start_output(&m_info, scan))
-                            return true; // I/O suspension.
+                            return false; // I/O suspension.
                     }
 
                     if (m_info.output_scanline == 0xffffff)
@@ -285,12 +318,12 @@ public:
                             // don't call jpeg_start_output() multiple times for
                             // the same scan.
                             m_info.output_scanline = 0xffffff;
-                        return true; // I/O suspension.
+                        return false; // I/O suspension.
                     }
 
                     if (m_info.output_scanline == m_info.output_height) {
                         if (!jpeg_finish_output(&m_info))
-                            return true; // I/O suspension.
+                            return false; // I/O suspension.
 
                         if (jpeg_input_complete(&m_info) && (m_info.input_scan_number == m_info.output_scan_number))
                             break;
@@ -305,14 +338,7 @@ public:
 
         case JPEG_DONE:
             // Finish decompression.
-            if (!jpeg_finish_decompress(&m_info))
-                return true; // I/O suspension.
-
-            m_state = JPEG_SINK_NON_JPEG_TRAILER;
-            break;
-        
-        case JPEG_SINK_NON_JPEG_TRAILER:
-            break;
+            return jpeg_finish_decompress(&m_info);
 
         case JPEG_ERROR:
             // We can get here if the constructor failed.
@@ -372,7 +398,9 @@ void term_source(j_decompress_ptr jd)
     src->decoder->decoder()->jpegComplete();
 }
 
-JPEGImageDecoder::JPEGImageDecoder()
+JPEGImageDecoder::JPEGImageDecoder(ImageSource::AlphaOption alphaOption,
+                                   ImageSource::GammaAndColorProfileOption gammaAndColorProfileOption)
+    : ImageDecoder(alphaOption, gammaAndColorProfileOption)
 {
 }
 
@@ -402,8 +430,10 @@ RGBA32Buffer* JPEGImageDecoder::frameBufferAtIndex(size_t index)
     if (index)
         return 0;
 
-    if (m_frameBufferCache.isEmpty())
+    if (m_frameBufferCache.isEmpty()) {
         m_frameBufferCache.resize(1);
+        m_frameBufferCache[0].setPremultiplyAlpha(m_premultiplyAlpha);
+    }
 
     RGBA32Buffer& frame = m_frameBufferCache[0];
     if (frame.status() != RGBA32Buffer::FrameComplete)
@@ -411,6 +441,12 @@ RGBA32Buffer* JPEGImageDecoder::frameBufferAtIndex(size_t index)
     return &frame;
 }
 
+bool JPEGImageDecoder::setFailed()
+{
+    m_reader.clear();
+    return ImageDecoder::setFailed();
+}
+
 bool JPEGImageDecoder::outputScanlines()
 {
     if (m_frameBufferCache.isEmpty())
@@ -423,6 +459,7 @@ bool JPEGImageDecoder::outputScanlines()
             return setFailed();
         buffer.setStatus(RGBA32Buffer::FramePartial);
         buffer.setHasAlpha(false);
+        buffer.setColorProfile(m_colorProfile);
 
         // For JPEGs, the frame always fills the entire image.
         buffer.setRect(IntRect(IntPoint(), size()));
@@ -487,10 +524,13 @@ void JPEGImageDecoder::decode(bool onlySize)
     if (!m_reader)
         m_reader.set(new JPEGImageReader(this));
 
-    if (!m_reader->decode(m_data->buffer(), onlySize))
+    // If we couldn't decode the image but we've received all the data, decoding
+    // has failed.
+    if (!m_reader->decode(m_data->buffer(), onlySize) && isAllDataReceived())
         setFailed();
-
-    if (failed() || (!m_frameBufferCache.isEmpty() && m_frameBufferCache[0].status() == RGBA32Buffer::FrameComplete))
+    // If we're done decoding the image, we don't need the JPEGImageReader
+    // anymore.  (If we failed, |m_reader| has already been cleared.)
+    else if (!m_frameBufferCache.isEmpty() && (m_frameBufferCache[0].status() == RGBA32Buffer::FrameComplete))
         m_reader.clear();
 }
 
