@@ -37,7 +37,7 @@ namespace WebCore {
     // This class decodes the JPEG image format.
     class JPEGImageDecoder : public ImageDecoder {
     public:
-        JPEGImageDecoder();
+        JPEGImageDecoder(ImageSource::AlphaOption, ImageSource::GammaAndColorProfileOption);
         virtual ~JPEGImageDecoder();
 
         // ImageDecoder
@@ -46,13 +46,20 @@ namespace WebCore {
         virtual bool setSize(unsigned width, unsigned height);
         virtual RGBA32Buffer* frameBufferAtIndex(size_t index);
         virtual bool supportsAlpha() const { return false; }
+        // CAUTION: setFailed() deletes |m_reader|.  Be careful to avoid
+        // accessing deleted memory, especially when calling this from inside
+        // JPEGImageReader!
+        virtual bool setFailed();
 
         bool outputScanlines();
         void jpegComplete();
 
+        void setColorProfile(const ColorProfile& colorProfile) { m_colorProfile = colorProfile; }
+
     private:
         // Decodes the image.  If |onlySize| is true, stops decoding after
-        // calculating the image size.
+        // calculating the image size.  If decoding fails but there is no more
+        // data coming, sets the "decode failure" flag.
         void decode(bool onlySize);
 
         OwnPtr<JPEGImageReader> m_reader;
