@@ -36,7 +36,7 @@ namespace WebCore {
     // This class decodes the PNG image format.
     class PNGImageDecoder : public ImageDecoder {
     public:
-        PNGImageDecoder();
+        PNGImageDecoder(ImageSource::AlphaOption, ImageSource::GammaAndColorProfileOption);
         virtual ~PNGImageDecoder();
 
         // ImageDecoder
@@ -44,6 +44,10 @@ namespace WebCore {
         virtual bool isSizeAvailable();
         virtual bool setSize(unsigned width, unsigned height);
         virtual RGBA32Buffer* frameBufferAtIndex(size_t index);
+        // CAUTION: setFailed() deletes |m_reader|.  Be careful to avoid
+        // accessing deleted memory, especially when calling this from inside
+        // PNGImageReader!
+        virtual bool setFailed();
 
         // Callbacks from libpng
         void headerAvailable();
@@ -57,10 +61,12 @@ namespace WebCore {
 
     private:
         // Decodes the image.  If |onlySize| is true, stops decoding after
-        // calculating the image size.
+        // calculating the image size.  If decoding fails but there is no more
+        // data coming, sets the "decode failure" flag.
         void decode(bool onlySize);
 
         OwnPtr<PNGImageReader> m_reader;
+        bool m_doNothingOnFailure;
     };
 
 } // namespace WebCore
