@@ -46,12 +46,13 @@
 #endif
 
 #if USE(SOUP)
+#include "soup-requester.h"
+#include <GRefPtr.h>
 #include <libsoup/soup.h>
 class Frame;
 #endif
 
 #if PLATFORM(QT)
-class QWebFrame;
 class QWebNetworkJob;
 namespace WebCore {
 class QNetworkReplyHandler;
@@ -68,6 +69,10 @@ class NSURLConnection;
 #endif
 #endif
 
+#if PLATFORM(ANDROID)
+#include "ResourceLoaderAndroid.h"
+#endif
+
 // The allocations and releases in ResourceHandleInternal are
 // Cocoa-exception-free (either simple Foundation classes or
 // WebCoreResourceLoaderImp which avoids doing work in dealloc).
@@ -77,29 +82,25 @@ namespace WebCore {
 
     class ResourceHandleInternal : public Noncopyable {
     public:
-        ResourceHandleInternal(ResourceHandle* loader, const ResourceRequest& request, ResourceHandleClient* c, bool defersLoading, bool shouldContentSniff, bool mightDownloadFromHandle)
+        ResourceHandleInternal(ResourceHandle* loader, const ResourceRequest& request, ResourceHandleClient* c, bool defersLoading, bool shouldContentSniff)
             : m_client(c)
-            , m_request(request)
+            , m_firstRequest(request)
+            , m_lastHTTPMethod(request.httpMethod())
             , status(0)
             , m_defersLoading(defersLoading)
             , m_shouldContentSniff(shouldContentSniff)
-            , m_mightDownloadFromHandle(mightDownloadFromHandle)
 #if USE(CFNETWORK)
             , m_connection(0)
 #endif
 #if USE(WININET)
-            , m_fileHandle(INVALID_HANDLE_VALUE)
             , m_fileLoadTimer(loader, &ResourceHandle::fileLoadTimer)
-            , m_resourceHandle(0)
-            , m_secondaryHandle(0)
-            , m_jobId(0)
-            , m_threadId(0)
-            , m_writing(false)
-            , m_formDataString(0)
-            , m_formDataLength(0)
+            , m_internetHandle(0)
+            , m_connectHandle(0)
+            , m_requestHandle(0)
+            , m_sentEndRequest(false)
             , m_bytesRemainingToWrite(0)
+            , m_loadSynchronously(false)
             , m_hasReceivedResponse(false)
-            , m_resend(false)
 #endif
 #if USE(CURL)
             , m_handle(0)
@@ -109,34 +110,30 @@ namespace WebCore {
             , m_formDataStream(loader)
 #endif
 #if USE(SOUP)
-            , m_msg(0)
             , m_cancelled(false)
-            , m_gfile(0)
-            , m_inputStream(0)
-            , m_cancellable(0)
             , m_buffer(0)
-            , m_bufferSize(0)
             , m_total(0)
             , m_idleHandler(0)
-            , m_frame(0)
+            , m_gotChunkHandler(0)
 #endif
 #if PLATFORM(QT)
             , m_job(0)
-            , m_frame(0)
 #endif
 #if PLATFORM(MAC)
             , m_startWhenScheduled(false)
             , m_needsSiteSpecificQuirks(false)
             , m_currentMacChallenge(nil)
-#elif USE(CFNETWORK)
-            , m_currentCFChallenge(0)
 #endif
+            , m_scheduledFailureType(ResourceHandle::NoFailure)
             , m_failureTimer(loader, &ResourceHandle::fireFailure)
         {
-            const KURL& url = m_request.url();
+            const KURL& url = m_firstRequest.url();
             m_user = url.user();
             m_pass = url.pass();
-            m_request.removeCredentials();
+            m_firstRequest.removeCredentials();
+#if USE(SOUP)
+            m_requester = adoptPlatformRef(webkit_soup_requester_new());
+#endif
         }
         
         ~ResourceHandleInternal();
@@ -144,17 +141,19 @@ namespace WebCore {
         ResourceHandleClient* client() { return m_client; }
         ResourceHandleClient* m_client;
         
-        ResourceRequest m_request;
+        ResourceRequest m_firstRequest;
+        String m_lastHTTPMethod;
 
         // Suggested credentials for the current redirection step.
         String m_user;
         String m_pass;
         
+        Credential m_initialCredential;
+        
         int status;
 
         bool m_defersLoading;
         bool m_shouldContentSniff;
-        bool m_mightDownloadFromHandle;
 #if USE(CFNETWORK)
         RetainPtr<CFURLConnectionRef> m_connection;
 #elif PLATFORM(MAC)
@@ -165,19 +164,16 @@ namespace WebCore {
         bool m_needsSiteSpecificQuirks;
 #endif
 #if USE(WININET)
-        HANDLE m_fileHandle;
         Timer<ResourceHandle> m_fileLoadTimer;
-        HINTERNET m_resourceHandle;
-        HINTERNET m_secondaryHandle;
-        unsigned m_jobId;
-        DWORD m_threadId;
-        bool m_writing;
-        char* m_formDataString;
-        int m_formDataLength;
-        int m_bytesRemainingToWrite;
-        String m_postReferrer;
+        HINTERNET m_internetHandle;
+        HINTERNET m_connectHandle;
+        HINTERNET m_requestHandle;
+        bool m_sentEndRequest;
+        Vector<char> m_formData;
+        size_t m_bytesRemainingToWrite;
+        bool m_loadSynchronously;
         bool m_hasReceivedResponse;
-        bool m_resend;
+        String m_redirectUrl;
 #endif
 #if USE(CURL)
         CURL* m_handle;
@@ -190,36 +186,35 @@ namespace WebCore {
         Vector<char> m_postBytes;
 #endif
 #if USE(SOUP)
-        SoupMessage* m_msg;
+        PlatformRefPtr<SoupMessage> m_soupMessage;
         ResourceResponse m_response;
         bool m_cancelled;
-        GFile* m_gfile;
-        GInputStream* m_inputStream;
-        GCancellable* m_cancellable;
+        PlatformRefPtr<WebKitSoupRequest> m_soupRequest;
+        PlatformRefPtr<WebKitSoupRequester> m_requester;
+        PlatformRefPtr<GInputStream> m_inputStream;
+        PlatformRefPtr<GCancellable> m_cancellable;
         char* m_buffer;
-        gsize m_bufferSize, m_total;
+        gsize m_total;
         guint m_idleHandler;
-        Frame* m_frame;
+        RefPtr<NetworkingContext> m_context;
+        gulong m_gotChunkHandler;
 #endif
 #if PLATFORM(QT)
-#if QT_VERSION < 0x040400
-        QWebNetworkJob* m_job;
-#else
         QNetworkReplyHandler* m_job;
-#endif
-        QWebFrame* m_frame;
+        RefPtr<NetworkingContext> m_context;
 #endif
 
-        // FIXME: The platform challenge is almost identical to the one stored in m_currentWebChallenge, but it has a different sender. We only need to store a sender reference here.
 #if PLATFORM(MAC)
+        // We need to keep a reference to the original challenge to be able to cancel it.
+        // It is almost identical to m_currentWebChallenge.nsURLAuthenticationChallenge(), but has a different sender.
         NSURLAuthenticationChallenge *m_currentMacChallenge;
 #endif
-#if USE(CFNETWORK)
-        CFURLAuthChallengeRef m_currentCFChallenge;
+#if PLATFORM(ANDROID)
+        RefPtr<ResourceLoaderAndroid> m_loader;
 #endif
         AuthenticationChallenge m_currentWebChallenge;
 
-        ResourceHandle::FailureType m_failureType;
+        ResourceHandle::FailureType m_scheduledFailureType;
         Timer<ResourceHandle> m_failureTimer;
     };
 
