@@ -29,8 +29,11 @@
 
 #include "HTTPHeaderMap.h"
 #include "KURL.h"
+#include "ResourceLoadInfo.h"
+#include "ResourceLoadTiming.h"
 
 #include <wtf/PassOwnPtr.h>
+#include <wtf/RefPtr.h>
 
 namespace WebCore {
 
@@ -95,6 +98,21 @@ public:
     double expires() const;
     double lastModified() const;
 
+    unsigned connectionID() const;
+    void setConnectionID(unsigned);
+
+    bool connectionReused() const;
+    void setConnectionReused(bool);
+
+    bool wasCached() const;
+    void setWasCached(bool);
+
+    ResourceLoadTiming* resourceLoadTiming() const;
+    void setResourceLoadTiming(PassRefPtr<ResourceLoadTiming>);
+
+    PassRefPtr<ResourceLoadInfo> resourceLoadInfo() const;
+    void setResourceLoadInfo(PassRefPtr<ResourceLoadInfo>);
+
     // The ResourceResponse subclass may "shadow" this method to provide platform-specific memory usage information
     unsigned memoryUsage() const
     {
@@ -102,7 +120,7 @@ public:
         return 1280;
     }
 
-    static bool compare(const ResourceResponse& a, const ResourceResponse& b);
+    static bool compare(const ResourceResponse&, const ResourceResponse&);
 
 protected:
     ResourceResponseBase();
@@ -125,10 +143,16 @@ protected:
     String m_httpStatusText;
     HTTPHeaderMap m_httpHeaderFields;
     time_t m_lastModifiedDate;
+    bool m_wasCached : 1;
+    unsigned m_connectionID;
+    bool m_connectionReused : 1;
+    RefPtr<ResourceLoadTiming> m_resourceLoadTiming;
+    RefPtr<ResourceLoadInfo> m_resourceLoadInfo;
 
     bool m_isNull : 1;
     
 private:
+    const ResourceResponse& asResourceResponse() const;
     void parseCacheControlDirectives() const;
 
     mutable bool m_haveParsedCacheControlHeader : 1;
@@ -151,7 +175,7 @@ private:
 inline bool operator==(const ResourceResponse& a, const ResourceResponse& b) { return ResourceResponseBase::compare(a, b); }
 inline bool operator!=(const ResourceResponse& a, const ResourceResponse& b) { return !(a == b); }
 
-struct CrossThreadResourceResponseData : Noncopyable {
+struct CrossThreadResourceResponseDataBase : Noncopyable {
     KURL m_url;
     String m_mimeType;
     long long m_expectedContentLength;
@@ -161,6 +185,7 @@ struct CrossThreadResourceResponseData : Noncopyable {
     String m_httpStatusText;
     OwnPtr<CrossThreadHTTPHeaderMapData> m_httpHeaders;
     time_t m_lastModifiedDate;
+    RefPtr<ResourceLoadTiming> m_resourceLoadTiming;
 };
 
 } // namespace WebCore
