@@ -31,8 +31,10 @@
 #include "Logging.h"
 #include "SQLiteFileSystem.h"
 #include "SQLiteStatement.h"
-
 #include <sqlite3.h>
+#include <wtf/Threading.h>
+#include <wtf/text/CString.h>
+#include <wtf/text/StringConcatenate.h>
 
 namespace WebCore {
 
@@ -42,13 +44,15 @@ const int SQLResultOk = SQLITE_OK;
 const int SQLResultRow = SQLITE_ROW;
 const int SQLResultSchema = SQLITE_SCHEMA;
 const int SQLResultFull = SQLITE_FULL;
-
+const int SQLResultInterrupt = SQLITE_INTERRUPT;
 
 SQLiteDatabase::SQLiteDatabase()
     : m_db(0)
     , m_pageSize(-1)
     , m_transactionInProgress(false)
+    , m_sharable(false)
     , m_openingThread(0)
+    , m_interrupted(false)
 {
 }
 
@@ -57,22 +61,27 @@ SQLiteDatabase::~SQLiteDatabase()
     close();
 }
 
-bool SQLiteDatabase::open(const String& filename)
+bool SQLiteDatabase::open(const String& filename, bool forWebSQLDatabase)
 {
     close();
-    
-    m_lastError = SQLiteFileSystem::openDatabase(filename, &m_db);
-    if (m_lastError != SQLITE_OK) {
+
+    if (SQLiteFileSystem::openDatabase(filename, &m_db, forWebSQLDatabase) != SQLITE_OK) {
         LOG_ERROR("SQLite database failed to load from %s\nCause - %s", filename.ascii().data(),
             sqlite3_errmsg(m_db));
         sqlite3_close(m_db);
         m_db = 0;
         return false;
     }
+    if (sqlite3_extended_result_codes(m_db, 1) != SQLITE_OK) {
+        LOG_ERROR("SQLite database error when enabling extended errors - %s", sqlite3_errmsg(m_db));
+        sqlite3_close(m_db);
+        m_db = 0;
+        return false;
+    }
 
     if (isOpen())
         m_openingThread = currentThread();
-    
+
     if (!SQLiteStatement(*this, "PRAGMA temp_store = MEMORY;").executeCommand())
         LOG_ERROR("SQLite database could not set temp_store to memory");
 
@@ -84,13 +93,39 @@ void SQLiteDatabase::close()
     if (m_db) {
         // FIXME: This is being called on themain thread during JS GC. <rdar://problem/5739818>
         // ASSERT(currentThread() == m_openingThread);
-        sqlite3_close(m_db);
-        m_db = 0;
+        sqlite3* db = m_db;
+        {
+            MutexLocker locker(m_databaseClosingMutex);
+            m_db = 0;
+        }
+        sqlite3_close(db);
     }
 
     m_openingThread = 0;
 }
 
+void SQLiteDatabase::interrupt()
+{
+#if !ENABLE(SINGLE_THREADED)
+    m_interrupted = true;
+    while (!m_lockingMutex.tryLock()) {
+        MutexLocker locker(m_databaseClosingMutex);
+        if (!m_db)
+            return;
+        sqlite3_interrupt(m_db);
+        yield();
+    }
+
+    m_lockingMutex.unlock();
+#endif
+}
+
+bool SQLiteDatabase::isInterrupted()
+{
+    ASSERT(!m_lockingMutex.tryLock());
+    return m_interrupted;
+}
+
 void SQLiteDatabase::setFullsync(bool fsync) 
 {
     if (fsync) 
@@ -101,14 +136,17 @@ void SQLiteDatabase::setFullsync(bool fsync)
 
 int64_t SQLiteDatabase::maximumSize()
 {
-    MutexLocker locker(m_authorizerLock);
-    enableAuthorizer(false);
-    
-    SQLiteStatement statement(*this, "PRAGMA max_page_count");
-    int64_t size = statement.getColumnInt64(0) * pageSize();
-    
-    enableAuthorizer(true);
-    return size;
+    int64_t maxPageCount = 0;
+
+    {
+        MutexLocker locker(m_authorizerLock);
+        enableAuthorizer(false);
+        SQLiteStatement statement(*this, "PRAGMA max_page_count");
+        maxPageCount = statement.getColumnInt64(0);
+        enableAuthorizer(true);
+    }
+
+    return maxPageCount * pageSize();
 }
 
 void SQLiteDatabase::setMaximumSize(int64_t size)
@@ -127,7 +165,11 @@ void SQLiteDatabase::setMaximumSize(int64_t size)
     SQLiteStatement statement(*this, "PRAGMA max_page_count = " + String::number(newMaxPageCount));
     statement.prepare();
     if (statement.step() != SQLResultRow)
-        LOG_ERROR("Failed to set maximum size of database to %lli bytes", size);
+#if OS(WINDOWS)
+        LOG_ERROR("Failed to set maximum size of database to %I64i bytes", static_cast<long long>(size));
+#else
+        LOG_ERROR("Failed to set maximum size of database to %lli bytes", static_cast<long long>(size));
+#endif
 
     enableAuthorizer(true);
 
@@ -152,19 +194,38 @@ int SQLiteDatabase::pageSize()
 
 int64_t SQLiteDatabase::freeSpaceSize()
 {
-    MutexLocker locker(m_authorizerLock);
-    enableAuthorizer(false);
-    // Note: freelist_count was added in SQLite 3.4.1.
-    SQLiteStatement statement(*this, "PRAGMA freelist_count");
-    int64_t size = statement.getColumnInt64(0) * pageSize();
+    int64_t freelistCount = 0;
 
-    enableAuthorizer(true);
-    return size;
+    {
+        MutexLocker locker(m_authorizerLock);
+        enableAuthorizer(false);
+        // Note: freelist_count was added in SQLite 3.4.1.
+        SQLiteStatement statement(*this, "PRAGMA freelist_count");
+        freelistCount = statement.getColumnInt64(0);
+        enableAuthorizer(true);
+    }
+
+    return freelistCount * pageSize();
+}
+
+int64_t SQLiteDatabase::totalSize()
+{
+    int64_t pageCount = 0;
+
+    {
+        MutexLocker locker(m_authorizerLock);
+        enableAuthorizer(false);
+        SQLiteStatement statement(*this, "PRAGMA page_count");
+        pageCount = statement.getColumnInt64(0);
+        enableAuthorizer(true);
+    }
+
+    return pageCount * pageSize();
 }
 
 void SQLiteDatabase::setSynchronous(SynchronousPragma sync)
 {
-    executeCommand(String::format("PRAGMA synchronous = %i", sync));
+    executeCommand(makeString("PRAGMA synchronous = ", String::number(sync)));
 }
 
 void SQLiteDatabase::setBusyTimeout(int ms)
@@ -228,6 +289,17 @@ void SQLiteDatabase::runVacuumCommand()
         LOG(SQLDatabase, "Unable to vacuum database - %s", lastErrorMsg());
 }
 
+void SQLiteDatabase::runIncrementalVacuumCommand()
+{
+    MutexLocker locker(m_authorizerLock);
+    enableAuthorizer(false);
+
+    if (!executeCommand("PRAGMA incremental_vacuum"))
+        LOG(SQLDatabase, "Unable to run incremental vacuum - %s", lastErrorMsg());
+
+    enableAuthorizer(true);
+}
+
 int64_t SQLiteDatabase::lastInsertRowID()
 {
     if (!m_db)
@@ -252,6 +324,18 @@ const char* SQLiteDatabase::lastErrorMsg()
     return sqlite3_errmsg(m_db);
 }
 
+#ifndef NDEBUG
+void SQLiteDatabase::disableThreadingChecks()
+{
+    // This doesn't guarantee that SQList was compiled with -DTHREADSAFE, or that you haven't turned off the mutexes.
+#if SQLITE_VERSION_NUMBER >= 3003001
+    m_sharable = true;
+#else
+    ASSERT(0); // Your SQLite doesn't support sharing handles across threads.
+#endif
+}
+#endif
+
 int SQLiteDatabase::authorizerFunction(void* userData, int actionCode, const char* parameter1, const char* parameter2, const char* /*databaseName*/, const char* /*trigger_or_view*/)
 {
     DatabaseAuthorizer* auth = static_cast<DatabaseAuthorizer*>(userData);
@@ -351,19 +435,39 @@ void SQLiteDatabase::enableAuthorizer(bool enable)
         sqlite3_set_authorizer(m_db, NULL, 0);
 }
 
-void SQLiteDatabase::lock()
-{
-    m_lockingMutex.lock();
-}
-
-void SQLiteDatabase::unlock()
-{
-    m_lockingMutex.unlock();
-}
-
 bool SQLiteDatabase::isAutoCommitOn() const
 {
     return sqlite3_get_autocommit(m_db);
 }
 
+bool SQLiteDatabase::turnOnIncrementalAutoVacuum()
+{
+    SQLiteStatement statement(*this, "PRAGMA auto_vacuum");
+    int autoVacuumMode = statement.getColumnInt(0);
+    int error = lastError();
+
+    // Check if we got an error while trying to get the value of the auto_vacuum flag.
+    // If we got a SQLITE_BUSY error, then there's probably another transaction in
+    // progress on this database. In this case, keep the current value of the
+    // auto_vacuum flag and try to set it to INCREMENTAL the next time we open this
+    // database. If the error is not SQLITE_BUSY, then we probably ran into a more
+    // serious problem and should return false (to log an error message).
+    if (error != SQLITE_ROW)
+        return false;
+
+    switch (autoVacuumMode) {
+    case AutoVacuumIncremental:
+        return true;
+    case AutoVacuumFull:
+        return executeCommand("PRAGMA auto_vacuum = 2");
+    case AutoVacuumNone:
+    default:
+        if (!executeCommand("PRAGMA auto_vacuum = 2"))
+            return false;
+        runVacuumCommand();
+        error = lastError();
+        return (error == SQLITE_OK);
+    }
+}
+
 } // namespace WebCore
