@@ -201,6 +201,7 @@ protected:
 private:
     void raiseExplicitEmbeddingLevel(WTF::Unicode::Direction from, WTF::Unicode::Direction to);
     void lowerExplicitEmbeddingLevel(WTF::Unicode::Direction from);
+    void checkDirectionInLowerRaiseEmbeddingLevel();
 
     Vector<WTF::Unicode::Direction, 8> m_currentExplicitEmbeddingSequence;
 };
@@ -307,24 +308,40 @@ void BidiResolver<Iterator, Run>::embed(WTF::Unicode::Direction d)
     m_currentExplicitEmbeddingSequence.append(d);
 }
 
+template <class Iterator, class Run>
+void BidiResolver<Iterator, Run>::checkDirectionInLowerRaiseEmbeddingLevel()
+{
+    using namespace WTF::Unicode;
+
+    ASSERT(m_status.eor != OtherNeutral || eor.atEnd());
+    // bidi.sor ... bidi.eor ... bidi.last eor; need to append the bidi.sor-bidi.eor run or extend it through bidi.last
+    // Bidi control characters are included into BidiRun, so last direction
+    // could be one of the bidi embeddings when there are nested embeddings.
+    // For example:  "&#x202a;&#x202b;....."
+    ASSERT(m_status.last == EuropeanNumberSeparator
+        || m_status.last == EuropeanNumberTerminator
+        || m_status.last == CommonNumberSeparator
+        || m_status.last == BoundaryNeutral
+        || m_status.last == BlockSeparator
+        || m_status.last == SegmentSeparator
+        || m_status.last == WhiteSpaceNeutral
+        || m_status.last == OtherNeutral
+        || m_status.last == RightToLeftEmbedding
+        || m_status.last == LeftToRightEmbedding
+        || m_status.last == RightToLeftOverride
+        || m_status.last == LeftToRightOverride
+        || m_status.last == PopDirectionalFormat);
+    if (m_direction == OtherNeutral)
+        m_direction = m_status.lastStrong == LeftToRight ? LeftToRight : RightToLeft;
+}
+
 template <class Iterator, class Run>
 void BidiResolver<Iterator, Run>::lowerExplicitEmbeddingLevel(WTF::Unicode::Direction from)
 {
     using namespace WTF::Unicode;
 
     if (!emptyRun && eor != last) {
-        ASSERT(m_status.eor != OtherNeutral || eor.atEnd());
-        // bidi.sor ... bidi.eor ... bidi.last eor; need to append the bidi.sor-bidi.eor run or extend it through bidi.last
-        ASSERT(m_status.last == EuropeanNumberSeparator
-            || m_status.last == EuropeanNumberTerminator
-            || m_status.last == CommonNumberSeparator
-            || m_status.last == BoundaryNeutral
-            || m_status.last == BlockSeparator
-            || m_status.last == SegmentSeparator
-            || m_status.last == WhiteSpaceNeutral
-            || m_status.last == OtherNeutral);
-        if (m_direction == OtherNeutral)
-            m_direction = m_status.lastStrong == LeftToRight ? LeftToRight : RightToLeft;
+        checkDirectionInLowerRaiseEmbeddingLevel();
         if (from == LeftToRight) {
             // bidi.sor ... bidi.eor ... bidi.last L
             if (m_status.eor == EuropeanNumber) {
@@ -359,18 +376,7 @@ void BidiResolver<Iterator, Run>::raiseExplicitEmbeddingLevel(WTF::Unicode::Dire
     using namespace WTF::Unicode;
 
     if (!emptyRun && eor != last) {
-        ASSERT(m_status.eor != OtherNeutral || eor.atEnd());
-        // bidi.sor ... bidi.eor ... bidi.last eor; need to append the bidi.sor-bidi.eor run or extend it through bidi.last
-        ASSERT(m_status.last == EuropeanNumberSeparator
-            || m_status.last == EuropeanNumberTerminator
-            || m_status.last == CommonNumberSeparator
-            || m_status.last == BoundaryNeutral
-            || m_status.last == BlockSeparator
-            || m_status.last == SegmentSeparator
-            || m_status.last == WhiteSpaceNeutral
-            || m_status.last == OtherNeutral);
-        if (m_direction == OtherNeutral)
-            m_direction = m_status.lastStrong == LeftToRight ? LeftToRight : RightToLeft;
+        checkDirectionInLowerRaiseEmbeddingLevel();
         if (to == LeftToRight) {
             // bidi.sor ... bidi.eor ... bidi.last L
             if (m_status.eor == EuropeanNumber) {
@@ -568,7 +574,7 @@ void BidiResolver<Iterator, Run>::createBidiRunsForLine(const Iterator& end, boo
             commitExplicitEmbedding();
             break;
 
-            // strong types
+        // strong types
         case LeftToRight:
             switch(m_status.last) {
                 case RightToLeft:
@@ -860,11 +866,6 @@ void BidiResolver<Iterator, Run>::createBidiRunsForLine(const Iterator& end, boo
                 break;
             case NonSpacingMark:
             case BoundaryNeutral:
-            case RightToLeftEmbedding:
-            case LeftToRightEmbedding:
-            case RightToLeftOverride:
-            case LeftToRightOverride:
-            case PopDirectionalFormat:
                 // ignore these
                 break;
             case EuropeanNumber:
@@ -875,11 +876,7 @@ void BidiResolver<Iterator, Run>::createBidiRunsForLine(const Iterator& end, boo
 
         last = current;
 
-        if (emptyRun && !(dirCurrent == RightToLeftEmbedding
-                || dirCurrent == LeftToRightEmbedding
-                || dirCurrent == RightToLeftOverride
-                || dirCurrent == LeftToRightOverride
-                || dirCurrent == PopDirectionalFormat)) {
+        if (emptyRun) {
             sor = current;
             emptyRun = false;
         }
@@ -901,17 +898,6 @@ void BidiResolver<Iterator, Run>::createBidiRunsForLine(const Iterator& end, boo
             }
         }
 
-        if (emptyRun && (dirCurrent == RightToLeftEmbedding
-                || dirCurrent == LeftToRightEmbedding
-                || dirCurrent == RightToLeftOverride
-                || dirCurrent == LeftToRightOverride
-                || dirCurrent == PopDirectionalFormat)) {
-            // exclude the embedding char itself from the new run so that ATSUI will never see it
-            eor = Iterator();
-            last = current;
-            sor = current;
-        }
-
         if (!pastEnd && (current == end || current.atEnd())) {
             if (emptyRun)
                 break;
