@@ -28,22 +28,25 @@
 #ifndef PluginView_h
 #define PluginView_h
 
-#include "CString.h"
 #include "FrameLoadRequest.h"
 #include "HaltablePlugin.h"
 #include "IntRect.h"
 #include "MediaCanStartListener.h"
-#include "PluginStream.h"
+#include "PluginViewBase.h"
 #include "ResourceRequest.h"
 #include "Timer.h"
-#include "Widget.h"
-#include "npruntime_internal.h"
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/OwnPtr.h>
 #include <wtf/PassRefPtr.h>
 #include <wtf/RefPtr.h>
 #include <wtf/Vector.h>
+#include <wtf/text/CString.h>
+
+#if ENABLE(NETSCAPE_PLUGIN_API)
+#include "PluginStream.h"
+#include "npruntime_internal.h"
+#endif
 
 #if OS(WINDOWS) && (PLATFORM(QT) || PLATFORM(WX))
 typedef struct HWND__* HWND;
@@ -54,12 +57,19 @@ typedef PlatformWidget PlatformPluginWidget;
 #include <QPixmap>
 #endif
 #endif
+#if PLATFORM(QT)
+#include <QGraphicsItem>
+#include <QImage>
+class QPainter;
+#endif
 
+#if USE(JSC)
 namespace JSC {
     namespace Bindings {
         class Instance;
     }
 }
+#endif
 
 namespace WebCore {
     class Element;
@@ -111,41 +121,60 @@ namespace WebCore {
         virtual void didFail(const ResourceError&) = 0;
     };
 
-    class PluginView : public Widget, private PluginStreamClient, public PluginManualLoader, private HaltablePlugin, private MediaCanStartListener {
+    class PluginView : public PluginViewBase
+#if ENABLE(NETSCAPE_PLUGIN_API)
+                     , private PluginStreamClient
+#endif
+                     , public PluginManualLoader
+                     , private HaltablePlugin
+                     , private MediaCanStartListener {
     public:
         static PassRefPtr<PluginView> create(Frame* parentFrame, const IntSize&, Element*, const KURL&, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually);
         virtual ~PluginView();
 
         PluginPackage* plugin() const { return m_plugin.get(); }
+#if ENABLE(NETSCAPE_PLUGIN_API)
         NPP instance() const { return m_instance; }
+#endif
 
         void setNPWindowRect(const IntRect&);
         static PluginView* currentPluginView();
 
+#if ENABLE(NETSCAPE_PLUGIN_API)
+        NPObject* npObject();
+#endif
+#if USE(JSC)
         PassRefPtr<JSC::Bindings::Instance> bindingInstance();
+#endif
 
         PluginStatus status() const { return m_status; }
 
+#if ENABLE(NETSCAPE_PLUGIN_API)
         // NPN functions
         NPError getURLNotify(const char* url, const char* target, void* notifyData);
         NPError getURL(const char* url, const char* target);
-        NPError postURLNotify(const char* url, const char* target, uint32 len, const char* but, NPBool file, void* notifyData);
-        NPError postURL(const char* url, const char* target, uint32 len, const char* but, NPBool file);
+        NPError postURLNotify(const char* url, const char* target, uint32_t len, const char* but, NPBool file, void* notifyData);
+        NPError postURL(const char* url, const char* target, uint32_t len, const char* but, NPBool file);
         NPError newStream(NPMIMEType type, const char* target, NPStream** stream);
-        int32 write(NPStream* stream, int32 len, void* buffer);
+        int32_t write(NPStream* stream, int32_t len, void* buffer);
         NPError destroyStream(NPStream* stream, NPReason reason);
+#endif
         const char* userAgent();
 #if ENABLE(NETSCAPE_PLUGIN_API)
         static const char* userAgentStatic();
 #endif
         void status(const char* message);
-        NPError getValue(NPNVariable variable, void* value);
+        
 #if ENABLE(NETSCAPE_PLUGIN_API)
+        NPError getValue(NPNVariable variable, void* value);
         static NPError getValueStatic(NPNVariable variable, void* value);
-#endif
         NPError setValue(NPPVariable variable, void* value);
+        NPError getValueForURL(NPNURLVariable variable, const char* url, char** value, uint32_t* len);
+        NPError setValueForURL(NPNURLVariable variable, const char* url, const char* value, uint32_t len);
+        NPError getAuthenticationInfo(const char* protocol, const char* host, int32_t port, const char* scheme, const char* realm, char** username, uint32_t* ulen, char** password, uint32_t* plen);
         void invalidateRect(NPRect*);
         void invalidateRegion(NPRegion);
+#endif
         void forceRedraw();
         void pushPopupsEnabledState(bool state);
         void popPopupsEnabledState();
@@ -164,7 +193,7 @@ namespace WebCore {
         // Widget functions
         virtual void setFrameRect(const IntRect&);
         virtual void frameRectsChanged();
-        virtual void setFocus();
+        virtual void setFocus(bool);
         virtual void show();
         virtual void hide();
         virtual void paint(GraphicsContext*, const IntRect&);
@@ -219,20 +248,29 @@ namespace WebCore {
 #endif
         void keepAlive();
 
+#if USE(ACCELERATED_COMPOSITING)
+#if defined(XP_UNIX) && ENABLE(NETSCAPE_PLUGIN_API) && PLATFORM(QT)
+        virtual PlatformLayer* platformLayer() const;
+#else
+        virtual PlatformLayer* platformLayer() const { return 0; }
+#endif
+#endif
+
     private:
         PluginView(Frame* parentFrame, const IntSize&, PluginPackage*, Element*, const KURL&, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually);
 
         void setParameters(const Vector<String>& paramNames, const Vector<String>& paramValues);
         bool startOrAddToUnstartedList();
-        void removeFromUnstartedListIfNecessary();
         void init();
         bool platformStart();
         void stop();
         void platformDestroy();
         static void setCurrentPluginView(PluginView*);
+#if ENABLE(NETSCAPE_PLUGIN_API)
         NPError load(const FrameLoadRequest&, bool sendNotification, void* notifyData);
-        NPError handlePost(const char* url, const char* target, uint32 len, const char* buf, bool file, void* notifyData, bool sendNotification, bool allowHeaders);
-        NPError handlePostReadFile(Vector<char>& buffer, uint32 len, const char* buf);
+        NPError handlePost(const char* url, const char* target, uint32_t len, const char* buf, bool file, void* notifyData, bool sendNotification, bool allowHeaders);
+        NPError handlePostReadFile(Vector<char>& buffer, uint32_t len, const char* buf);
+#endif
         static void freeStringArray(char** stringArray, int length);
         void setCallingPlugin(bool) const;
 
@@ -246,8 +284,10 @@ namespace WebCore {
         static BOOL WINAPI hookedEndPaint(HWND, const PAINTSTRUCT*);
 #endif
 
+#if ENABLE(NETSCAPE_PLUGIN_API)
         static bool platformGetValueStatic(NPNVariable variable, void* value, NPError* result);
         bool platformGetValue(NPNVariable variable, void* value, NPError* result);
+#endif
 
         RefPtr<Frame> m_parentFrame;
         RefPtr<PluginPackage> m_plugin;
@@ -272,7 +312,9 @@ namespace WebCore {
         Timer<PluginView> m_lifeSupportTimer;
 
 #ifndef NP_NO_CARBON
+#if ENABLE(NETSCAPE_PLUGIN_API)
         bool dispatchNPEvent(NPEvent&);
+#endif // ENABLE(NETSCAPE_PLUGIN_API)
 #endif
         void updatePluginWidget();
         void paintMissingPluginIcon(GraphicsContext*, const IntRect&);
@@ -296,11 +338,13 @@ namespace WebCore {
         String m_pluginsPage;
 
         String m_mimeType;
-        CString m_userAgent;
+        WTF::CString m_userAgent;
 
+#if ENABLE(NETSCAPE_PLUGIN_API)
         NPP m_instance;
         NPP_t m_instanceStruct;
         NPWindow m_npWindow;
+#endif
 
         Vector<bool, 4> m_popupStateStack;
 
@@ -370,6 +414,21 @@ private:
         void initXEvent(XEvent* event);
 #endif
 
+#if PLATFORM(QT) 
+#if defined(MOZ_PLATFORM_MAEMO) && (MOZ_PLATFORM_MAEMO >= 5)
+        QImage m_image;
+        bool m_renderToImage;
+        void paintUsingImageSurfaceExtension(QPainter* painter, const IntRect& exposedRect);
+#endif
+#if defined(XP_UNIX) && ENABLE(NETSCAPE_PLUGIN_API)
+        void paintUsingXPixmap(QPainter* painter, const QRect &exposedRect);
+#if USE(ACCELERATED_COMPOSITING)
+        OwnPtr<PlatformLayer> m_platformLayer;
+        friend class PluginGraphicsLayerQt;
+#endif // USE(ACCELERATED_COMPOSITING)
+#endif
+#endif // PLATFORM(QT)
+
         IntRect m_clipRect; // The clip rect to apply to a windowed plug-in
         IntRect m_windowRect; // Our window rect.
 
@@ -381,6 +440,8 @@ private:
         bool m_isHalted;
         bool m_hasBeenHalted;
 
+        bool m_haveCalledSetWindow;
+
         static PluginView* s_currentPluginView;
     };
 
