@@ -72,8 +72,7 @@ static void            gtk_xtbin_class_init (GtkXtBinClass *klass);
 static void            gtk_xtbin_init       (GtkXtBin      *xtbin);
 static void            gtk_xtbin_realize    (GtkWidget      *widget);
 static void            gtk_xtbin_unrealize    (GtkWidget      *widget);
-static void            gtk_xtbin_destroy    (GtkObject      *object);
-static void            gtk_xtbin_shutdown   (GtkObject      *object);
+static void            gtk_xtbin_dispose    (GObject      *object);
 
 /* Xt aware XEmbed */
 static void       xt_client_init      (XtClient * xtclient, 
@@ -159,7 +158,6 @@ xt_event_dispatch (GSource*  source_data,
                     GSourceFunc call_back,
                     gpointer  user_data)
 {
-  XEvent event;
   XtAppContext ac;
   int i = 0;
 
@@ -181,11 +179,13 @@ xt_event_dispatch (GSource*  source_data,
   return TRUE;  
 }
 
+typedef void (*GSourceFuncsFinalize) (GSource* source);
+
 static GSourceFuncs xt_event_funcs = {
   xt_event_prepare,
   xt_event_check,
   xt_event_dispatch,
-  g_free,
+  (GSourceFuncsFinalize)g_free,
   (GSourceFunc)NULL,
   (GSourceDummyMarshal)NULL
 };
@@ -230,11 +230,12 @@ gtk_xtbin_get_type (void)
         sizeof (GtkXtBin),
         0,
         (GInstanceInitFunc)gtk_xtbin_init,
+        NULL
       };
       xtbin_type = g_type_register_static (GTK_TYPE_SOCKET,
                                            "GtkXtBin",
-					   &xtbin_info,
-					   0);
+                                           &xtbin_info,
+                                           0);
   }
   return xtbin_type;
 }
@@ -243,7 +244,7 @@ static void
 gtk_xtbin_class_init (GtkXtBinClass *klass)
 {
   GtkWidgetClass *widget_class;
-  GtkObjectClass *object_class;
+  GObjectClass   *object_class;
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -251,8 +252,8 @@ gtk_xtbin_class_init (GtkXtBinClass *klass)
   widget_class->realize = gtk_xtbin_realize;
   widget_class->unrealize = gtk_xtbin_unrealize;
 
-  object_class = GTK_OBJECT_CLASS (klass);
-  object_class->destroy = gtk_xtbin_destroy;
+  object_class = G_OBJECT_CLASS (klass);
+  object_class->dispose = gtk_xtbin_dispose;
 }
 
 static void
@@ -273,7 +274,6 @@ gtk_xtbin_realize (GtkWidget *widget)
 #if GTK_CHECK_VERSION(2, 18, 0)
   GtkAllocation widget_allocation;
 #endif
-  gint  x, y, w, h, d; /* geometry of window */
 
 #ifdef DEBUG_XTBIN
   printf("gtk_xtbin_realize()\n");
@@ -284,9 +284,15 @@ gtk_xtbin_realize (GtkWidget *widget)
   xtbin = GTK_XTBIN (widget);
 
   /* caculate the allocation before realize */
+#if GTK_CHECK_VERSION(2, 24, 0)
+  allocation.width = gdk_window_get_width(xtbin->parent_window);
+  allocation.height = gdk_window_get_height(xtbin->parent_window);
+#else
+  gint  x, y, w, h, d; /* geometry of window */
   gdk_window_get_geometry(xtbin->parent_window, &x, &y, &w, &h, &d);
   allocation.width = w;
   allocation.height = h;
+#endif
   gtk_widget_size_allocate (widget, &allocation);
 
 #ifdef DEBUG_XTBIN
@@ -321,10 +327,14 @@ gtk_xtbin_realize (GtkWidget *widget)
 
 
 GtkWidget*
-gtk_xtbin_new (GdkWindow *parent_window, String * f)
+gtk_xtbin_new (GtkWidget *parent_widget, String *f)
 {
   GtkXtBin *xtbin;
   gpointer user_data;
+  GdkScreen *screen;
+  GdkVisual* visual;
+  Colormap colormap;
+  GdkWindow* parent_window = gtk_widget_get_window(parent_widget);
 
   assert(parent_window != NULL);
   xtbin = g_object_new (GTK_TYPE_XTBIN, NULL);
@@ -338,10 +348,16 @@ gtk_xtbin_new (GdkWindow *parent_window, String * f)
   /* Initialize the Xt toolkit */
   xtbin->parent_window = parent_window;
 
+  screen = gtk_widget_get_screen(parent_widget);
+  visual = gdk_screen_get_system_visual(screen);
+  colormap = XCreateColormap(GDK_DISPLAY_XDISPLAY(gdk_screen_get_display(screen)),
+                             GDK_WINDOW_XWINDOW(gdk_screen_get_root_window(screen)),
+                             GDK_VISUAL_XVISUAL(visual), AllocNone);
+
   xt_client_init(&(xtbin->xtclient), 
-      GDK_VISUAL_XVISUAL(gdk_rgb_get_visual()),
-      GDK_COLORMAP_XCOLORMAP(gdk_rgb_get_colormap()),
-      gdk_visual_get_depth(gdk_rgb_get_visual()));
+                 GDK_VISUAL_XVISUAL(visual),
+                 colormap,
+                 gdk_visual_get_depth(visual));
 
   if (!xtbin->xtclient.xtdisplay) {
     /* If XtOpenDisplay failed, we can't go any further.
@@ -474,7 +490,7 @@ gtk_xtbin_unrealize (GtkWidget *object)
 }
 
 static void
-gtk_xtbin_destroy (GtkObject *object)
+gtk_xtbin_dispose (GObject *object)
 {
   GtkXtBin *xtbin;
 
@@ -508,7 +524,7 @@ gtk_xtbin_destroy (GtkObject *object)
     }
   }
 
-  GTK_OBJECT_CLASS(parent_class)->destroy(object);
+  G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
 /*
@@ -892,7 +908,6 @@ xt_add_focus_listener( Widget w, XtPointer user_data)
   XWindowAttributes attr;
   long eventmask;
   XtClient *xtclient = user_data;
-  int errorcode;
 
   trap_errors ();
   XGetWindowAttributes(XtDisplay(w), XtWindow(w), &attr);
@@ -912,8 +927,6 @@ xt_add_focus_listener( Widget w, XtPointer user_data)
 static void
 xt_remove_focus_listener(Widget w, XtPointer user_data)
 {
-  int errorcode;
-
   trap_errors ();
   XtRemoveEventHandler(w, SubstructureNotifyMask | ButtonReleaseMask, TRUE, 
                       (XtEventHandler)xt_client_focus_listener, user_data);
