@@ -23,7 +23,7 @@
 #include "HitTestResult.h"
 #include "InlineFlowBox.h"
 #include "RenderArena.h"
-#include "RenderBox.h"
+#include "RenderBlock.h"
 #include "RootInlineBox.h"
 
 using namespace std;
@@ -85,31 +85,24 @@ void InlineBox::showTreeForThis() const
 }
 #endif
 
-int InlineBox::height() const
+int InlineBox::logicalHeight() const
 {
 #if ENABLE(SVG)
-    if (hasVirtualHeight())
-        return virtualHeight();
+    if (hasVirtualLogicalHeight())
+        return virtualLogicalHeight();
 #endif
     
     if (renderer()->isText())
         return m_isText ? renderer()->style(m_firstLine)->font().height() : 0;
     if (renderer()->isBox() && parent())
-        return toRenderBox(m_renderer)->height();
+        return isHorizontal() ? toRenderBox(m_renderer)->height() : toRenderBox(m_renderer)->width();
 
     ASSERT(isInlineFlowBox());
-    const InlineFlowBox* flowBox = static_cast<const InlineFlowBox*>(this);
     RenderBoxModelObject* flowObject = boxModelObject();
     const Font& font = renderer()->style(m_firstLine)->font();
     int result = font.height();
-    bool strictMode = renderer()->document()->inStrictMode();
     if (parent())
-        result += flowObject->borderTop() + flowObject->paddingTop() + flowObject->borderBottom() + flowObject->paddingBottom();
-    if (strictMode || flowBox->hasTextChildren() || flowObject->hasHorizontalBordersOrPadding())
-        return result;
-    int bottom = root()->bottomOverflow();
-    if (y() + result > bottom)
-        result = bottom - y();
+        result += flowObject->borderAndPaddingLogicalHeight();
     return result;
 }
 
@@ -166,27 +159,31 @@ void InlineBox::adjustPosition(int dx, int dy)
     }
 }
 
-void InlineBox::paint(RenderObject::PaintInfo& paintInfo, int tx, int ty)
+void InlineBox::paint(PaintInfo& paintInfo, int tx, int ty)
 {
-    if (!renderer()->shouldPaintWithinRoot(paintInfo) || (paintInfo.phase != PaintPhaseForeground && paintInfo.phase != PaintPhaseSelection))
+    if (!paintInfo.shouldPaintWithinRoot(renderer()) || (paintInfo.phase != PaintPhaseForeground && paintInfo.phase != PaintPhaseSelection))
         return;
 
+    IntPoint childPoint = IntPoint(tx, ty);
+    if (parent()->renderer()->style()->isFlippedBlocksWritingMode()) // Faster than calling containingBlock().
+        childPoint = renderer()->containingBlock()->flipForWritingMode(toRenderBox(renderer()), childPoint, RenderBox::ParentToChildFlippingAdjustment);
+    
     // Paint all phases of replaced elements atomically, as though the replaced element established its
     // own stacking context.  (See Appendix E.2, section 6.4 on inline block/table elements in the CSS2.1
     // specification.)
     bool preservePhase = paintInfo.phase == PaintPhaseSelection || paintInfo.phase == PaintPhaseTextClip;
-    RenderObject::PaintInfo info(paintInfo);
+    PaintInfo info(paintInfo);
     info.phase = preservePhase ? paintInfo.phase : PaintPhaseBlockBackground;
-    renderer()->paint(info, tx, ty);
+    renderer()->paint(info, childPoint.x(), childPoint.y());
     if (!preservePhase) {
         info.phase = PaintPhaseChildBlockBackgrounds;
-        renderer()->paint(info, tx, ty);
+        renderer()->paint(info, childPoint.x(), childPoint.y());
         info.phase = PaintPhaseFloat;
-        renderer()->paint(info, tx, ty);
+        renderer()->paint(info, childPoint.x(), childPoint.y());
         info.phase = PaintPhaseForeground;
-        renderer()->paint(info, tx, ty);
+        renderer()->paint(info, childPoint.x(), childPoint.y());
         info.phase = PaintPhaseOutline;
-        renderer()->paint(info, tx, ty);
+        renderer()->paint(info, childPoint.x(), childPoint.y());
     }
 }
 
@@ -275,7 +272,7 @@ bool InlineBox::canAccommodateEllipsis(bool ltr, int blockEdge, int ellipsisWidt
     if (!m_renderer || !m_renderer->isReplaced())
         return true;
     
-    IntRect boxRect(m_x, 0, m_width, 10);
+    IntRect boxRect(m_x, 0, m_logicalWidth, 10);
     IntRect ellipsisRect(ltr ? blockEdge - ellipsisWidth : blockEdge, 0, ellipsisWidth, 10);
     return !(boxRect.intersects(ellipsisRect));
 }
@@ -286,6 +283,31 @@ int InlineBox::placeEllipsisBox(bool, int, int, int, bool&)
     return -1;
 }
 
+IntPoint InlineBox::locationIncludingFlipping()
+{
+    if (!renderer()->style()->isFlippedBlocksWritingMode())
+        return IntPoint(x(), y());
+    RenderBlock* block = root()->block();
+    if (block->style()->isHorizontalWritingMode())
+        return IntPoint(x(), block->height() - height() - y());
+    else
+        return IntPoint(block->width() - width() - x(), y());
+}
+
+void InlineBox::flipForWritingMode(IntRect& rect)
+{
+    if (!renderer()->style()->isFlippedBlocksWritingMode())
+        return;
+    root()->block()->flipForWritingMode(rect);
+}
+
+IntPoint InlineBox::flipForWritingMode(const IntPoint& point)
+{
+    if (!renderer()->style()->isFlippedBlocksWritingMode())
+        return point;
+    return root()->block()->flipForWritingMode(point);
+}
+
 } // namespace WebCore
 
 #ifndef NDEBUG
