@@ -21,6 +21,7 @@
 #ifndef InlineBox_h
 #define InlineBox_h
 
+#include "RenderBR.h"
 #include "RenderBoxModelObject.h"
 #include "TextDirection.h"
 
@@ -30,7 +31,6 @@ class HitTestRequest;
 class HitTestResult;
 class RootInlineBox;
 
-
 // InlineBox represents a rectangle that occurs on a line.  It corresponds to
 // some RenderObject (i.e., it represents a portion of that RenderObject).
 class InlineBox {
@@ -42,18 +42,19 @@ public:
         , m_renderer(obj)
         , m_x(0)
         , m_y(0)
-        , m_width(0)
+        , m_logicalWidth(0)
         , m_firstLine(false)
         , m_constructed(false)
         , m_bidiEmbeddingLevel(0)
         , m_dirty(false)
         , m_extracted(false)
 #if ENABLE(SVG)
-        , m_hasVirtualHeight(false)
+        , m_hasVirtualLogicalHeight(false)
 #endif
+        , m_isHorizontal(true)
         , m_endsWithBreak(false)
         , m_hasSelectedChildren(false)
-        , m_hasEllipsisBox(false)
+        , m_hasEllipsisBoxOrHyphen(false)
         , m_dirOverride(false)
         , m_isText(false)
         , m_determinedIfNextOnLineExists(false)
@@ -67,26 +68,27 @@ public:
     {
     }
 
-    InlineBox(RenderObject* obj, int x, int y, int width, bool firstLine, bool constructed,
-              bool dirty, bool extracted, InlineBox* next, InlineBox* prev, InlineFlowBox* parent)
+    InlineBox(RenderObject* obj, int x, int y, int logicalWidth, bool firstLine, bool constructed,
+              bool dirty, bool extracted, bool isHorizontal, InlineBox* next, InlineBox* prev, InlineFlowBox* parent)
         : m_next(next)
         , m_prev(prev)
         , m_parent(parent)
         , m_renderer(obj)
         , m_x(x)
         , m_y(y)
-        , m_width(width)
+        , m_logicalWidth(logicalWidth)
         , m_firstLine(firstLine)
         , m_constructed(constructed)
         , m_bidiEmbeddingLevel(0)
         , m_dirty(dirty)
         , m_extracted(extracted)
 #if ENABLE(SVG)
-        , m_hasVirtualHeight(false)
+        , m_hasVirtualLogicalHeight(false)
 #endif
+        , m_isHorizontal(isHorizontal)
         , m_endsWithBreak(false)
         , m_hasSelectedChildren(false)   
-        , m_hasEllipsisBox(false)
+        , m_hasEllipsisBoxOrHyphen(false)
         , m_dirOverride(false)
         , m_isText(false)
         , m_determinedIfNextOnLineExists(false)
@@ -111,10 +113,26 @@ public:
     virtual bool isLineBreak() const { return false; }
 
     virtual void adjustPosition(int dx, int dy);
+    void adjustLineDirectionPosition(int delta)
+    {
+        if (isHorizontal())
+            adjustPosition(delta, 0);
+        else
+            adjustPosition(0, delta);
+    }
+    void adjustBlockDirectionPosition(int delta)
+    {
+        if (isHorizontal())
+            adjustPosition(0, delta);
+        else
+            adjustPosition(delta, 0);
+    }
 
-    virtual void paint(RenderObject::PaintInfo&, int tx, int ty);
+    virtual void paint(PaintInfo&, int tx, int ty);
     virtual bool nodeAtPoint(const HitTestRequest&, HitTestResult&, int x, int y, int tx, int ty);
 
+    InlineBox* next() const { return m_next; }
+
     // Overloaded new operator.
     void* operator new(size_t, RenderArena*) throw();
 
@@ -133,26 +151,35 @@ public:
     bool isText() const { return m_isText; }
     void setIsText(bool b) { m_isText = b; }
  
-    virtual bool isInlineBox() { return false; }
     virtual bool isInlineFlowBox() const { return false; }
-    virtual bool isInlineTextBox() { return false; }
+    virtual bool isInlineTextBox() const { return false; }
     virtual bool isRootInlineBox() const { return false; }
-#if ENABLE(SVG) 
-    virtual bool isSVGRootInlineBox() { return false; }
-
-    bool hasVirtualHeight() const { return m_hasVirtualHeight; }
-    void setHasVirtualHeight() { m_hasVirtualHeight = true; }
-    virtual int virtualHeight() const { ASSERT_NOT_REACHED(); return 0; }
+#if ENABLE(SVG)
+    virtual bool isSVGInlineTextBox() const { return false; }
+    virtual bool isSVGInlineFlowBox() const { return false; }
+    virtual bool isSVGRootInlineBox() const { return false; }
 #endif
-    
-    bool isConstructed() { return m_constructed; }
-    virtual void setConstructed()
+
+    bool hasVirtualLogicalHeight() const { return m_hasVirtualLogicalHeight; }
+    void setHasVirtualLogicalHeight() { m_hasVirtualLogicalHeight = true; }
+    virtual int virtualLogicalHeight() const
     {
-        m_constructed = true;
-        if (m_next)
-            m_next->setConstructed();
+        ASSERT_NOT_REACHED();
+        return 0;
     }
 
+    bool isHorizontal() const { return m_isHorizontal; }
+    void setIsHorizontal(bool horizontal) { m_isHorizontal = horizontal; }
+
+    virtual IntRect calculateBoundaries() const
+    {
+        ASSERT_NOT_REACHED();
+        return IntRect();
+    }
+
+    bool isConstructed() { return m_constructed; }
+    virtual void setConstructed() { m_constructed = true; }
+
     void setExtracted(bool b = true) { m_extracted = b; }
     
     void setFirstLineStyleBit(bool f) { m_firstLine = f; }
@@ -192,27 +219,51 @@ public:
     const RootInlineBox* root() const;
     RootInlineBox* root();
 
-    void setWidth(int w) { m_width = w; }
-    int width() const { return m_width; }
-
-    // x() is the left side of the box in the parent's coordinate system.
+    // x() is the left side of the box in the containing block's coordinate system.
     void setX(int x) { m_x = x; }
     int x() const { return m_x; }
 
-    // y() is the top of the box in the parent's coordinate system.
+    // y() is the top side of the box in the containing block's coordinate system.
     void setY(int y) { m_y = y; }
     int y() const { return m_y; }
 
-    int height() const;
+    int width() const { return isHorizontal() ? logicalWidth() : logicalHeight(); }
+    int height() const { return isHorizontal() ? logicalHeight() : logicalWidth(); }
 
-    inline int baselinePosition(bool isRootLineBox) const { return renderer()->baselinePosition(m_firstLine, isRootLineBox); }
-    inline int lineHeight(bool isRootLineBox) const { return renderer()->lineHeight(m_firstLine, isRootLineBox); }
+    IntRect frameRect() const { return IntRect(x(), y(), width(), height()); }
 
-    virtual int topOverflow() const { return y(); }
-    virtual int bottomOverflow() const { return y() + height(); }
-    virtual int leftOverflow() const { return x(); }
-    virtual int rightOverflow() const { return x() + width(); }
+    // The logicalLeft position is the left edge of the line box in a horizontal line and the top edge in a vertical line.
+    int logicalLeft() const { return isHorizontal() ? m_x : m_y; }
+    int logicalRight() const { return logicalLeft() + logicalWidth(); }
+    void setLogicalLeft(int left)
+    {
+        if (isHorizontal())
+            m_x = left;
+        else
+            m_y = left;
+    }
 
+    // The logicalTop[ position is the top edge of the line box in a horizontal line and the left edge in a vertical line.
+    int logicalTop() const { return isHorizontal() ? m_y : m_x; }
+    int logicalBottom() const { return logicalTop() + logicalHeight(); }
+    void setLogicalTop(int top)
+    {
+        if (isHorizontal())
+            m_y = top;
+        else
+            m_x = top;
+    }
+
+    // The logical width is our extent in the line's overall inline direction, i.e., width for horizontal text and height for vertical text.
+    void setLogicalWidth(int w) { m_logicalWidth = w; }
+    int logicalWidth() const { return m_logicalWidth; }
+
+    // The logical height is our extent in the block flow direction, i.e., height for horizontal text and width for vertical text.
+    int logicalHeight() const;
+
+    virtual int baselinePosition(FontBaseline baselineType) const { return boxModelObject()->baselinePosition(baselineType, m_firstLine, isHorizontal() ? HorizontalLine : VerticalLine, PositionOnContainingLine); }
+    virtual int lineHeight() const { return boxModelObject()->lineHeight(m_firstLine, isHorizontal() ? HorizontalLine : VerticalLine, PositionOnContainingLine); }
+    
     virtual int caretMinOffset() const;
     virtual int caretMaxOffset() const;
     virtual unsigned caretMaxRenderedOffset() const;
@@ -220,8 +271,9 @@ public:
     unsigned char bidiLevel() const { return m_bidiEmbeddingLevel; }
     void setBidiLevel(unsigned char level) { m_bidiEmbeddingLevel = level; }
     TextDirection direction() const { return m_bidiEmbeddingLevel % 2 ? RTL : LTR; }
-    int caretLeftmostOffset() const { return direction() == LTR ? caretMinOffset() : caretMaxOffset(); }
-    int caretRightmostOffset() const { return direction() == LTR ? caretMaxOffset() : caretMinOffset(); }
+    bool isLeftToRightDirection() const { return direction() == LTR; }
+    int caretLeftmostOffset() const { return isLeftToRightDirection() ? caretMinOffset() : caretMaxOffset(); }
+    int caretRightmostOffset() const { return isLeftToRightDirection() ? caretMaxOffset() : caretMinOffset(); }
 
     virtual void clearTruncation() { }
 
@@ -246,10 +298,14 @@ public:
     RenderBoxModelObject* boxModelObject() const
     { 
         if (!m_renderer->isText())
-            return static_cast<RenderBoxModelObject*>(m_renderer);
+            return toRenderBoxModelObject(m_renderer);
         return 0;
     }
 
+    IntPoint locationIncludingFlipping();
+    void flipForWritingMode(IntRect&);
+    IntPoint flipForWritingMode(const IntPoint&);
+
 private:
     InlineBox* m_next; // The next element on the same line as us.
     InlineBox* m_prev; // The previous element on the same line as us.
@@ -261,7 +317,7 @@ public:
 
     int m_x;
     int m_y;
-    int m_width;
+    int m_logicalWidth;
     
     // Some of these bits are actually for subclasses and moved here to compact the structures.
 
@@ -274,12 +330,14 @@ private:
 protected:
     bool m_dirty : 1;
     bool m_extracted : 1;
-    bool m_hasVirtualHeight : 1;
+    bool m_hasVirtualLogicalHeight : 1;
+
+    bool m_isHorizontal : 1;
 
     // for RootInlineBox
     bool m_endsWithBreak : 1;  // Whether the line ends with a <br>.
     bool m_hasSelectedChildren : 1; // Whether we have any children selected (this bit will also be set if the <br> that terminates our line is selected).
-    bool m_hasEllipsisBox : 1; 
+    bool m_hasEllipsisBoxOrHyphen : 1; 
 
     // for InlineTextBox
 public:
@@ -290,7 +348,7 @@ protected:
     mutable bool m_determinedIfPrevOnLineExists : 1;
     mutable bool m_nextOnLineExists : 1;
     mutable bool m_prevOnLineExists : 1;
-    int m_toAdd : 12; // for justified text
+    int m_toAdd : 11; // for justified text
 
 #ifndef NDEBUG
 private:
