@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,50 +38,79 @@
 #include "HTMLNames.h"
 #include "LocalizedStrings.h"
 #include "MouseEvent.h"
+#include "Page.h"
 #include "RenderMedia.h"
 #include "RenderSlider.h"
 #include "RenderTheme.h"
+#include "Settings.h"
 
 namespace WebCore {
 
 using namespace HTMLNames;
 
+HTMLMediaElement* toParentMediaElement(RenderObject* o)
+{
+    Node* node = o->node();
+    Node* mediaNode = node ? node->shadowAncestorNode() : 0;
+    if (!mediaNode || (!mediaNode->hasTagName(HTMLNames::videoTag) && !mediaNode->hasTagName(HTMLNames::audioTag)))
+        return 0;
+
+    return static_cast<HTMLMediaElement*>(mediaNode);
+}
+
 // FIXME: These constants may need to be tweaked to better match the seeking in the QuickTime plug-in.
 static const float cSeekRepeatDelay = 0.1f;
 static const float cStepTime = 0.07f;
 static const float cSeekTime = 0.2f;
 
-MediaControlShadowRootElement::MediaControlShadowRootElement(Document* document, HTMLMediaElement* mediaElement)
-    : HTMLDivElement(divTag, document)
-    , m_mediaElement(mediaElement) 
+inline MediaControlShadowRootElement::MediaControlShadowRootElement(HTMLMediaElement* mediaElement)
+    : HTMLDivElement(divTag, mediaElement->document())
 {
+    setShadowHost(mediaElement);
+}
+
+PassRefPtr<MediaControlShadowRootElement> MediaControlShadowRootElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlShadowRootElement> element = adoptRef(new MediaControlShadowRootElement(mediaElement));
+
     RefPtr<RenderStyle> rootStyle = RenderStyle::create();
     rootStyle->inheritFrom(mediaElement->renderer()->style());
     rootStyle->setDisplay(BLOCK);
     rootStyle->setPosition(RelativePosition);
-    RenderMediaControlShadowRoot* renderer = new (mediaElement->renderer()->renderArena()) RenderMediaControlShadowRoot(this);
+
+    RenderMediaControlShadowRoot* renderer = new (mediaElement->renderer()->renderArena()) RenderMediaControlShadowRoot(element.get());
     renderer->setStyle(rootStyle.release());
-    setRenderer(renderer);
-    setAttached();
-    setInDocument(true);
+
+    element->setRenderer(renderer);
+    element->setAttached();
+    element->setInDocument();
+
+    return element.release();
 }
 
 void MediaControlShadowRootElement::updateStyle()
 {
     if (renderer()) {
-        RenderStyle* timelineContainerStyle = m_mediaElement->renderer()->getCachedPseudoStyle(MEDIA_CONTROLS_TIMELINE_CONTAINER);
+        RenderStyle* timelineContainerStyle = shadowHost()->renderer()->getCachedPseudoStyle(MEDIA_CONTROLS_TIMELINE_CONTAINER);
         renderer()->setStyle(timelineContainerStyle);
     }
 }
 
+void MediaControlShadowRootElement::detach()
+{
+    HTMLDivElement::detach();
+    // FIXME: Remove once shadow DOM uses Element::setShadowRoot().
+    setShadowHost(0);
+}
+
 // ----------------------------
 
-MediaControlElement::MediaControlElement(Document* document, PseudoId pseudo, HTMLMediaElement* mediaElement)
-    : HTMLDivElement(divTag, document)
+MediaControlElement::MediaControlElement(HTMLMediaElement* mediaElement, PseudoId pseudo)
+    : HTMLDivElement(divTag, mediaElement->document())
     , m_mediaElement(mediaElement)
     , m_pseudoStyleId(pseudo)
 {
-    setInDocument(true);
+    setInDocument();
     switch (pseudo) {
     case MEDIA_CONTROLS_CURRENT_TIME_DISPLAY:
         m_displayType = MediaCurrentTimeDisplay;
@@ -107,9 +136,16 @@ MediaControlElement::MediaControlElement(Document* document, PseudoId pseudo, HT
     }
 }
 
+PassRefPtr<MediaControlElement> MediaControlElement::create(HTMLMediaElement* mediaElement, PseudoId pseudoStyleId)
+{
+    return adoptRef(new MediaControlElement(mediaElement, pseudoStyleId));
+}
+
 void MediaControlElement::attachToParent(Element* parent)
 {
-    parent->addChild(this);
+    // FIXME: This code seems very wrong.  Why are we magically adding |this| to the DOM here?
+    //        We shouldn't be calling parser API methods outside of the parser!
+    parent->parserAddChild(this);
 }
 
 void MediaControlElement::update()
@@ -135,7 +171,10 @@ PassRefPtr<RenderStyle> MediaControlElement::styleForElement()
 
 bool MediaControlElement::rendererIsNeeded(RenderStyle* style)
 {
-    return HTMLDivElement::rendererIsNeeded(style) && parent() && parent()->renderer();
+    ASSERT(document()->page());
+
+    return HTMLDivElement::rendererIsNeeded(style) && parentNode() && parentNode()->renderer()
+        && (!style->hasAppearance() || document()->page()->theme()->shouldRenderMediaControlPart(style->appearance(), m_mediaElement));
 }
     
 void MediaControlElement::attach()
@@ -151,12 +190,12 @@ void MediaControlElement::attach()
         return;
     renderer->setStyle(style.get());
     setRenderer(renderer);
-    if (parent() && parent()->renderer()) {
+    if (parentNode() && parentNode()->renderer()) {
         // Find next sibling with a renderer to determine where to insert.
         Node* sibling = nextSibling();
         while (sibling && !sibling->renderer())
             sibling = sibling->nextSibling();
-        parent()->renderer()->addChild(renderer, sibling ? sibling->renderer() : 0);
+        parentNode()->renderer()->addChild(renderer, sibling ? sibling->renderer() : 0);
     }
     ContainerNode::attach();
 }
@@ -170,7 +209,7 @@ void MediaControlElement::updateStyle()
     if (!style)
         return;
 
-    bool needsRenderer = rendererIsNeeded(style.get()) && parent() && parent()->renderer();
+    bool needsRenderer = rendererIsNeeded(style.get()) && parentNode() && parentNode()->renderer();
     if (renderer() && !needsRenderer)
         detach();
     else if (!renderer() && needsRenderer)
@@ -186,11 +225,16 @@ void MediaControlElement::updateStyle()
 
 // ----------------------------
 
-MediaControlTimelineContainerElement::MediaControlTimelineContainerElement(Document* document, HTMLMediaElement* element)
-    : MediaControlElement(document, MEDIA_CONTROLS_TIMELINE_CONTAINER, element)
+inline MediaControlTimelineContainerElement::MediaControlTimelineContainerElement(HTMLMediaElement* mediaElement)
+    : MediaControlElement(mediaElement, MEDIA_CONTROLS_TIMELINE_CONTAINER)
 {
 }
 
+PassRefPtr<MediaControlTimelineContainerElement> MediaControlTimelineContainerElement::create(HTMLMediaElement* mediaElement)
+{
+    return adoptRef(new MediaControlTimelineContainerElement(mediaElement));
+}
+
 bool MediaControlTimelineContainerElement::rendererIsNeeded(RenderStyle* style)
 {
     if (!MediaControlElement::rendererIsNeeded(style))
@@ -199,23 +243,28 @@ bool MediaControlTimelineContainerElement::rendererIsNeeded(RenderStyle* style)
     // This is for MediaControllerThemeClassic:
     // If there is no style for MediaControlStatusDisplayElement style, don't hide
     // the timeline.
-    if (!m_mediaElement->renderer()->getCachedPseudoStyle(MEDIA_CONTROLS_STATUS_DISPLAY))
+    if (!mediaElement()->renderer()->getCachedPseudoStyle(MEDIA_CONTROLS_STATUS_DISPLAY))
         return true;
 
-    float duration = m_mediaElement->duration();
+    float duration = mediaElement()->duration();
     return !isnan(duration) && !isinf(duration);
 }
 
 // ----------------------------
 
-MediaControlVolumeSliderContainerElement::MediaControlVolumeSliderContainerElement(Document* doc, HTMLMediaElement* element)
-    : MediaControlElement(doc, MEDIA_CONTROLS_VOLUME_SLIDER_CONTAINER, element)
+inline MediaControlVolumeSliderContainerElement::MediaControlVolumeSliderContainerElement(HTMLMediaElement* mediaElement)
+    : MediaControlElement(mediaElement, MEDIA_CONTROLS_VOLUME_SLIDER_CONTAINER)
     , m_isVisible(false)
     , m_x(0)
     , m_y(0)
 {
 }
 
+PassRefPtr<MediaControlVolumeSliderContainerElement> MediaControlVolumeSliderContainerElement::create(HTMLMediaElement* mediaElement)
+{
+    return adoptRef(new MediaControlVolumeSliderContainerElement(mediaElement));
+}
+
 PassRefPtr<RenderStyle> MediaControlVolumeSliderContainerElement::styleForElement()
 {
     RefPtr<RenderStyle> style = MediaControlElement::styleForElement();
@@ -251,12 +300,17 @@ bool MediaControlVolumeSliderContainerElement::hitTest(const IntPoint& absPoint)
 
 // ----------------------------
 
-MediaControlStatusDisplayElement::MediaControlStatusDisplayElement(Document* document, HTMLMediaElement* element)
-    : MediaControlElement(document, MEDIA_CONTROLS_STATUS_DISPLAY, element)
+inline MediaControlStatusDisplayElement::MediaControlStatusDisplayElement(HTMLMediaElement* mediaElement)
+    : MediaControlElement(mediaElement, MEDIA_CONTROLS_STATUS_DISPLAY)
     , m_stateBeingDisplayed(Nothing)
 {
 }
 
+PassRefPtr<MediaControlStatusDisplayElement> MediaControlStatusDisplayElement::create(HTMLMediaElement* mediaElement)
+{
+    return adoptRef(new MediaControlStatusDisplayElement(mediaElement));
+}
+
 void MediaControlStatusDisplayElement::update()
 {
     MediaControlElement::update();
@@ -264,9 +318,9 @@ void MediaControlStatusDisplayElement::update()
     // Get the new state that we'll have to display.
     StateBeingDisplayed newStateToDisplay = Nothing;
 
-    if (m_mediaElement->readyState() != HTMLMediaElement::HAVE_ENOUGH_DATA && !m_mediaElement->currentSrc().isEmpty())
+    if (mediaElement()->readyState() != HTMLMediaElement::HAVE_ENOUGH_DATA && !mediaElement()->currentSrc().isEmpty())
         newStateToDisplay = Loading;
-    else if (m_mediaElement->movieLoadType() == MediaPlayer::LiveStream)
+    else if (mediaElement()->movieLoadType() == MediaPlayer::LiveStream)
         newStateToDisplay = LiveBroadcast;
 
     // Propagate only if needed.
@@ -292,19 +346,18 @@ bool MediaControlStatusDisplayElement::rendererIsNeeded(RenderStyle* style)
 {
     if (!MediaControlElement::rendererIsNeeded(style))
         return false;
-    float duration = m_mediaElement->duration();
+    float duration = mediaElement()->duration();
     return (isnan(duration) || isinf(duration));
 }
 
 // ----------------------------
     
-MediaControlInputElement::MediaControlInputElement(Document* document, PseudoId pseudo, const String& type, HTMLMediaElement* mediaElement)
-    : HTMLInputElement(inputTag, document)
+MediaControlInputElement::MediaControlInputElement(HTMLMediaElement* mediaElement, PseudoId pseudo)
+    : HTMLInputElement(inputTag, mediaElement->document())
     , m_mediaElement(mediaElement)
     , m_pseudoStyleId(pseudo)
 {
-    setInputType(type);
-    setInDocument(true);
+    setInDocument();
 
     switch (pseudo) {
     case MEDIA_CONTROLS_MUTE_BUTTON:
@@ -334,6 +387,12 @@ MediaControlInputElement::MediaControlInputElement(Document* document, PseudoId
     case MEDIA_CONTROLS_VOLUME_SLIDER:
         m_displayType = MediaVolumeSlider;
         break;
+    case MEDIA_CONTROLS_VOLUME_SLIDER_MUTE_BUTTON:
+        m_displayType = MediaVolumeSliderMuteButton;
+        break;
+    case MEDIA_CONTROLS_TOGGLE_CLOSED_CAPTIONS_BUTTON:
+        m_displayType = MediaShowClosedCaptionsButton;
+        break;
     default:
         ASSERT_NOT_REACHED();
         break;
@@ -342,7 +401,9 @@ MediaControlInputElement::MediaControlInputElement(Document* document, PseudoId
 
 void MediaControlInputElement::attachToParent(Element* parent)
 {
-    parent->addChild(this);
+    // FIXME: This code seems very wrong.  Why are we magically adding |this| to the DOM here?
+    //        We shouldn't be calling parser API methods outside of the parser!
+    parent->parserAddChild(this);
 }
 
 void MediaControlInputElement::update()
@@ -355,12 +416,15 @@ void MediaControlInputElement::update()
 
 PassRefPtr<RenderStyle> MediaControlInputElement::styleForElement()
 {
-    return m_mediaElement->renderer()->getCachedPseudoStyle(m_pseudoStyleId);
+    return mediaElement()->renderer()->getCachedPseudoStyle(m_pseudoStyleId);
 }
 
 bool MediaControlInputElement::rendererIsNeeded(RenderStyle* style)
 {
-    return HTMLInputElement::rendererIsNeeded(style) && parent() && parent()->renderer();
+    ASSERT(document()->page());
+
+    return HTMLInputElement::rendererIsNeeded(style) && parentNode() && parentNode()->renderer()
+        && (!style->hasAppearance() || document()->page()->theme()->shouldRenderMediaControlPart(style->appearance(), mediaElement()));
 }
 
 void MediaControlInputElement::attach()
@@ -372,31 +436,31 @@ void MediaControlInputElement::attach()
     bool needsRenderer = rendererIsNeeded(style.get());
     if (!needsRenderer)
         return;
-    RenderObject* renderer = createRenderer(m_mediaElement->renderer()->renderArena(), style.get());
+    RenderObject* renderer = createRenderer(mediaElement()->renderer()->renderArena(), style.get());
     if (!renderer)
         return;
     renderer->setStyle(style.get());
     setRenderer(renderer);
-    if (parent() && parent()->renderer()) {
+    if (parentNode() && parentNode()->renderer()) {
         // Find next sibling with a renderer to determine where to insert.
         Node* sibling = nextSibling();
         while (sibling && !sibling->renderer())
             sibling = sibling->nextSibling();
-        parent()->renderer()->addChild(renderer, sibling ? sibling->renderer() : 0);
+        parentNode()->renderer()->addChild(renderer, sibling ? sibling->renderer() : 0);
     }  
     ContainerNode::attach();
 }
 
 void MediaControlInputElement::updateStyle()
 {
-    if (!m_mediaElement || !m_mediaElement->renderer())
+    if (!mediaElement() || !mediaElement()->renderer())
         return;
     
     RefPtr<RenderStyle> style = styleForElement();
     if (!style)
         return;
     
-    bool needsRenderer = rendererIsNeeded(style.get()) && parent() && parent()->renderer();
+    bool needsRenderer = rendererIsNeeded(style.get()) && parentNode() && parentNode()->renderer();
     if (renderer() && !needsRenderer)
         detach();
     else if (!renderer() && needsRenderer)
@@ -425,15 +489,22 @@ void MediaControlInputElement::setDisplayType(MediaControlElementType displayTyp
 
 // ----------------------------
 
-MediaControlMuteButtonElement::MediaControlMuteButtonElement(Document* document, HTMLMediaElement* element)
-    : MediaControlInputElement(document, MEDIA_CONTROLS_MUTE_BUTTON, "button", element)
+inline MediaControlMuteButtonElement::MediaControlMuteButtonElement(HTMLMediaElement* mediaElement, ButtonLocation location)
+    : MediaControlInputElement(mediaElement, location == Controller ? MEDIA_CONTROLS_MUTE_BUTTON : MEDIA_CONTROLS_VOLUME_SLIDER_MUTE_BUTTON)
 {
 }
 
+PassRefPtr<MediaControlMuteButtonElement> MediaControlMuteButtonElement::create(HTMLMediaElement* mediaElement, ButtonLocation location)
+{
+    RefPtr<MediaControlMuteButtonElement> button = adoptRef(new MediaControlMuteButtonElement(mediaElement, location));
+    button->setType("button");
+    return button.release();
+}
+
 void MediaControlMuteButtonElement::defaultEventHandler(Event* event)
 {
     if (event->type() == eventNames().clickEvent) {
-        m_mediaElement->setMuted(!m_mediaElement->muted());
+        mediaElement()->setMuted(!mediaElement()->muted());
         event->setDefaultHandled();
     }
     HTMLInputElement::defaultEventHandler(event);
@@ -441,30 +512,27 @@ void MediaControlMuteButtonElement::defaultEventHandler(Event* event)
 
 void MediaControlMuteButtonElement::updateDisplayType()
 {
-    setDisplayType(m_mediaElement->muted() ? MediaUnMuteButton : MediaMuteButton);
-}
-
-bool MediaControlMuteButtonElement::disabled() const
-{
-    return !m_mediaElement->hasAudio();
-}
-
-bool MediaControlMuteButtonElement::rendererIsNeeded(RenderStyle* style)
-{
-    return MediaControlInputElement::rendererIsNeeded(style) && !disabled();
+    setDisplayType(mediaElement()->muted() ? MediaUnMuteButton : MediaMuteButton);
 }
 
 // ----------------------------
 
-MediaControlPlayButtonElement::MediaControlPlayButtonElement(Document* document, HTMLMediaElement* element)
-    : MediaControlInputElement(document, MEDIA_CONTROLS_PLAY_BUTTON, "button", element)
+inline MediaControlPlayButtonElement::MediaControlPlayButtonElement(HTMLMediaElement* mediaElement)
+    : MediaControlInputElement(mediaElement, MEDIA_CONTROLS_PLAY_BUTTON)
 {
 }
 
+PassRefPtr<MediaControlPlayButtonElement> MediaControlPlayButtonElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlPlayButtonElement> button = adoptRef(new MediaControlPlayButtonElement(mediaElement));
+    button->setType("button");
+    return button.release();
+}
+
 void MediaControlPlayButtonElement::defaultEventHandler(Event* event)
 {
     if (event->type() == eventNames().clickEvent) {
-        m_mediaElement->togglePlayState();
+        mediaElement()->togglePlayState();
         event->setDefaultHandled();
     }
     HTMLInputElement::defaultEventHandler(event);
@@ -472,21 +540,31 @@ void MediaControlPlayButtonElement::defaultEventHandler(Event* event)
 
 void MediaControlPlayButtonElement::updateDisplayType()
 {
-    setDisplayType(m_mediaElement->canPlay() ? MediaPlayButton : MediaPauseButton);
+    setDisplayType(mediaElement()->canPlay() ? MediaPlayButton : MediaPauseButton);
 }
 
 // ----------------------------
 
-MediaControlSeekButtonElement::MediaControlSeekButtonElement(Document* document, HTMLMediaElement* element, bool forward)
-    : MediaControlInputElement(document, forward ? MEDIA_CONTROLS_SEEK_FORWARD_BUTTON : MEDIA_CONTROLS_SEEK_BACK_BUTTON,
-                               "button", element)
-    , m_forward(forward)
+inline MediaControlSeekButtonElement::MediaControlSeekButtonElement(HTMLMediaElement* mediaElement, PseudoId pseudoId)
+    : MediaControlInputElement(mediaElement, pseudoId)
     , m_seeking(false)
     , m_capturing(false)
     , m_seekTimer(this, &MediaControlSeekButtonElement::seekTimerFired)
 {
 }
 
+PassRefPtr<MediaControlSeekButtonElement> MediaControlSeekButtonElement::create(HTMLMediaElement* mediaElement, PseudoId pseudoStyleId)
+{
+    RefPtr<MediaControlSeekButtonElement> button = adoptRef(new MediaControlSeekButtonElement(mediaElement, pseudoStyleId));
+    button->setType("button");
+    return button.release();
+}
+
+inline bool MediaControlSeekButtonElement::isForwardButton() const
+{
+    return pseudoStyleId() == MEDIA_CONTROLS_SEEK_FORWARD_BUTTON;
+}
+
 void MediaControlSeekButtonElement::defaultEventHandler(Event* event)
 {
     if (event->type() == eventNames().mousedownEvent) {
@@ -494,7 +572,7 @@ void MediaControlSeekButtonElement::defaultEventHandler(Event* event)
             m_capturing = true;
             frame->eventHandler()->setCapturingMouseEventsNode(this);
         }
-        m_mediaElement->pause();
+        mediaElement()->pause(event->fromUserGesture());
         m_seekTimer.startRepeating(cSeekRepeatDelay);
         event->setDefaultHandled();
     } else if (event->type() == eventNames().mouseupEvent) {
@@ -506,8 +584,8 @@ void MediaControlSeekButtonElement::defaultEventHandler(Event* event)
         ExceptionCode ec;
         if (m_seeking || m_seekTimer.isActive()) {
             if (!m_seeking) {
-                float stepTime = m_forward ? cStepTime : -cStepTime;
-                m_mediaElement->setCurrentTime(m_mediaElement->currentTime() + stepTime, ec);
+                float stepTime = isForwardButton() ? cStepTime : -cStepTime;
+                mediaElement()->setCurrentTime(mediaElement()->currentTime() + stepTime, ec);
             }
             m_seekTimer.stop();
             m_seeking = false;
@@ -521,8 +599,8 @@ void MediaControlSeekButtonElement::seekTimerFired(Timer<MediaControlSeekButtonE
 {
     ExceptionCode ec;
     m_seeking = true;
-    float seekTime = m_forward ? cSeekTime : -cSeekTime;
-    m_mediaElement->setCurrentTime(m_mediaElement->currentTime() + seekTime, ec);
+    float seekTime = isForwardButton() ? cSeekTime : -cSeekTime;
+    mediaElement()->setCurrentTime(mediaElement()->currentTime() + seekTime, ec);
 }
 
 void MediaControlSeekButtonElement::detach()
@@ -534,65 +612,107 @@ void MediaControlSeekButtonElement::detach()
     MediaControlInputElement::detach();
 }
 
-
 // ----------------------------
 
-MediaControlRewindButtonElement::MediaControlRewindButtonElement(Document* document, HTMLMediaElement* element)
-    : MediaControlInputElement(document, MEDIA_CONTROLS_REWIND_BUTTON, "button", element)
+inline MediaControlRewindButtonElement::MediaControlRewindButtonElement(HTMLMediaElement* element)
+    : MediaControlInputElement(element, MEDIA_CONTROLS_REWIND_BUTTON)
 {
 }
 
+PassRefPtr<MediaControlRewindButtonElement> MediaControlRewindButtonElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlRewindButtonElement> button = adoptRef(new MediaControlRewindButtonElement(mediaElement));
+    button->setType("button");
+    return button.release();
+}
+
 void MediaControlRewindButtonElement::defaultEventHandler(Event* event)
 {
     if (event->type() == eventNames().clickEvent) {
-        m_mediaElement->rewind(30);
+        mediaElement()->rewind(30);
         event->setDefaultHandled();
     }    
     HTMLInputElement::defaultEventHandler(event);
 }
 
-bool MediaControlRewindButtonElement::rendererIsNeeded(RenderStyle* style)
-{
-    return MediaControlInputElement::rendererIsNeeded(style) && m_mediaElement->movieLoadType() != MediaPlayer::LiveStream;
-}
-
-
 // ----------------------------
 
-MediaControlReturnToRealtimeButtonElement::MediaControlReturnToRealtimeButtonElement(Document* document, HTMLMediaElement* element)
-    : MediaControlInputElement(document, MEDIA_CONTROLS_RETURN_TO_REALTIME_BUTTON, "button", element)
+inline MediaControlReturnToRealtimeButtonElement::MediaControlReturnToRealtimeButtonElement(HTMLMediaElement* mediaElement)
+    : MediaControlInputElement(mediaElement, MEDIA_CONTROLS_RETURN_TO_REALTIME_BUTTON)
 {
 }
 
+PassRefPtr<MediaControlReturnToRealtimeButtonElement> MediaControlReturnToRealtimeButtonElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlReturnToRealtimeButtonElement> button = adoptRef(new MediaControlReturnToRealtimeButtonElement(mediaElement));
+    button->setType("button");
+    return button.release();
+}
+
 void MediaControlReturnToRealtimeButtonElement::defaultEventHandler(Event* event)
 {
     if (event->type() == eventNames().clickEvent) {
-        m_mediaElement->returnToRealtime();
+        mediaElement()->returnToRealtime();
         event->setDefaultHandled();
     }
     HTMLInputElement::defaultEventHandler(event);
 }
 
-bool MediaControlReturnToRealtimeButtonElement::rendererIsNeeded(RenderStyle* style)
+
+// ----------------------------
+
+inline MediaControlToggleClosedCaptionsButtonElement::MediaControlToggleClosedCaptionsButtonElement(HTMLMediaElement* mediaElement)
+    : MediaControlInputElement(mediaElement, MEDIA_CONTROLS_TOGGLE_CLOSED_CAPTIONS_BUTTON)
 {
-    return MediaControlInputElement::rendererIsNeeded(style) && m_mediaElement->movieLoadType() == MediaPlayer::LiveStream;
+}
+
+PassRefPtr<MediaControlToggleClosedCaptionsButtonElement> MediaControlToggleClosedCaptionsButtonElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlToggleClosedCaptionsButtonElement> button = adoptRef(new MediaControlToggleClosedCaptionsButtonElement(mediaElement));
+    button->setType("button");
+    return button.release();
+}
+
+void MediaControlToggleClosedCaptionsButtonElement::defaultEventHandler(Event* event)
+{
+    if (event->type() == eventNames().clickEvent) {
+        mediaElement()->setClosedCaptionsVisible(!mediaElement()->closedCaptionsVisible());
+        setChecked(mediaElement()->closedCaptionsVisible());
+        event->setDefaultHandled();
+    }
+    HTMLInputElement::defaultEventHandler(event);
+}
+
+void MediaControlToggleClosedCaptionsButtonElement::updateDisplayType()
+{
+    setDisplayType(mediaElement()->closedCaptionsVisible() ? MediaHideClosedCaptionsButton : MediaShowClosedCaptionsButton);
 }
 
 // ----------------------------
 
-MediaControlTimelineElement::MediaControlTimelineElement(Document* document, HTMLMediaElement* element)
-    : MediaControlInputElement(document, MEDIA_CONTROLS_TIMELINE, "range", element)
+MediaControlTimelineElement::MediaControlTimelineElement(HTMLMediaElement* mediaElement)
+    : MediaControlInputElement(mediaElement, MEDIA_CONTROLS_TIMELINE)
 {
 }
 
+PassRefPtr<MediaControlTimelineElement> MediaControlTimelineElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlTimelineElement> timeline = adoptRef(new MediaControlTimelineElement(mediaElement));
+    timeline->setType("range");
+    return timeline.release();
+}
+
 void MediaControlTimelineElement::defaultEventHandler(Event* event)
 {
     // Left button is 0. Rejects mouse events not from left button.
     if (event->isMouseEvent() && static_cast<MouseEvent*>(event)->button())
         return;
 
+    if (!attached())
+        return;
+
     if (event->type() == eventNames().mousedownEvent)
-        m_mediaElement->beginScrubbing();
+        mediaElement()->beginScrubbing();
 
     MediaControlInputElement::defaultEventHandler(event);
 
@@ -600,85 +720,123 @@ void MediaControlTimelineElement::defaultEventHandler(Event* event)
         return;
 
     float time = narrowPrecisionToFloat(value().toDouble());
-    if (time != m_mediaElement->currentTime()) {
+    if (time != mediaElement()->currentTime()) {
         ExceptionCode ec;
-        m_mediaElement->setCurrentTime(time, ec);
+        mediaElement()->setCurrentTime(time, ec);
     }
 
     RenderSlider* slider = toRenderSlider(renderer());
     if (slider && slider->inDragMode())
-        toRenderMedia(m_mediaElement->renderer())->updateTimeDisplay();
+        toRenderMedia(mediaElement()->renderer())->updateTimeDisplay();
 
     if (event->type() == eventNames().mouseupEvent)
-        m_mediaElement->endScrubbing();
+        mediaElement()->endScrubbing();
 }
 
 void MediaControlTimelineElement::update(bool updateDuration) 
 {
     if (updateDuration) {
-        float dur = m_mediaElement->duration();
-        setAttribute(maxAttr, String::number(isfinite(dur) ? dur : 0));
+        float duration = mediaElement()->duration();
+        setAttribute(maxAttr, String::number(isfinite(duration) ? duration : 0));
     }
-    setValue(String::number(m_mediaElement->currentTime()));
+    setValue(String::number(mediaElement()->currentTime()));
     MediaControlInputElement::update();
 }
 
 // ----------------------------
 
-MediaControlVolumeSliderElement::MediaControlVolumeSliderElement(Document* document, HTMLMediaElement* element)
-    : MediaControlInputElement(document, MEDIA_CONTROLS_VOLUME_SLIDER, "range", element)
+inline MediaControlVolumeSliderElement::MediaControlVolumeSliderElement(HTMLMediaElement* mediaElement)
+    : MediaControlInputElement(mediaElement, MEDIA_CONTROLS_VOLUME_SLIDER)
 {
 }
 
+PassRefPtr<MediaControlVolumeSliderElement> MediaControlVolumeSliderElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlVolumeSliderElement> slider = adoptRef(new MediaControlVolumeSliderElement(mediaElement));
+    slider->setType("range");
+    return slider.release();
+}
+
 void MediaControlVolumeSliderElement::defaultEventHandler(Event* event)
 {
     // Left button is 0. Rejects mouse events not from left button.
     if (event->isMouseEvent() && static_cast<MouseEvent*>(event)->button())
         return;
 
+    if (!attached())
+        return;
+
     MediaControlInputElement::defaultEventHandler(event);
 
     if (event->type() == eventNames().mouseoverEvent || event->type() == eventNames().mouseoutEvent || event->type() == eventNames().mousemoveEvent)
         return;
 
     float volume = narrowPrecisionToFloat(value().toDouble());
-    if (volume != m_mediaElement->volume()) {
+    if (volume != mediaElement()->volume()) {
         ExceptionCode ec = 0;
-        m_mediaElement->setVolume(volume, ec);
+        mediaElement()->setVolume(volume, ec);
         ASSERT(!ec);
     }
 }
 
+void MediaControlVolumeSliderElement::update()
+{
+    float volume = mediaElement()->volume();
+    if (value().toFloat() != volume)
+        setValue(String::number(volume));
+    MediaControlInputElement::update();
+}
+
 // ----------------------------
 
-MediaControlFullscreenButtonElement::MediaControlFullscreenButtonElement(Document* document, HTMLMediaElement* element)
-    : MediaControlInputElement(document, MEDIA_CONTROLS_FULLSCREEN_BUTTON, "button", element)
+inline MediaControlFullscreenButtonElement::MediaControlFullscreenButtonElement(HTMLMediaElement* mediaElement)
+    : MediaControlInputElement(mediaElement, MEDIA_CONTROLS_FULLSCREEN_BUTTON)
 {
 }
 
+PassRefPtr<MediaControlFullscreenButtonElement> MediaControlFullscreenButtonElement::create(HTMLMediaElement* mediaElement)
+{
+    RefPtr<MediaControlFullscreenButtonElement> button = adoptRef(new MediaControlFullscreenButtonElement(mediaElement));
+    button->setType("button");
+    return button.release();
+}
+
 void MediaControlFullscreenButtonElement::defaultEventHandler(Event* event)
 {
     if (event->type() == eventNames().clickEvent) {
+#if ENABLE(FULLSCREEN_API)
+        // Only use the new full screen API if the fullScreenEnabled setting has 
+        // been explicitly enabled.  Otherwise, use the old fullscreen API.  This
+        // allows apps which embed a WebView to retain the existing full screen
+        // video implementation without requiring them to implement their own full 
+        // screen behavior.
+        if (document()->settings() && document()->settings()->fullScreenEnabled()) {
+            if (document()->webkitFullScreen() && document()->webkitCurrentFullScreenElement() == mediaElement())
+                document()->webkitCancelFullScreen();
+            else
+                mediaElement()->webkitRequestFullScreen(0);
+        } else
+#endif
+            mediaElement()->enterFullscreen();
         event->setDefaultHandled();
     }
     HTMLInputElement::defaultEventHandler(event);
 }
 
-bool MediaControlFullscreenButtonElement::rendererIsNeeded(RenderStyle* style)
-{
-    return MediaControlInputElement::rendererIsNeeded(style) && m_mediaElement->supportsFullscreen();
-}
-
-
 // ----------------------------
 
-MediaControlTimeDisplayElement::MediaControlTimeDisplayElement(Document* document, PseudoId pseudo, HTMLMediaElement* element)
-    : MediaControlElement(document, pseudo, element)
+inline MediaControlTimeDisplayElement::MediaControlTimeDisplayElement(HTMLMediaElement* mediaElement, PseudoId pseudo)
+    : MediaControlElement(mediaElement, pseudo)
     , m_currentValue(0)
     , m_isVisible(true)
 {
 }
 
+PassRefPtr<MediaControlTimeDisplayElement> MediaControlTimeDisplayElement::create(HTMLMediaElement* mediaElement, PseudoId pseudoStyleId)
+{
+    return adoptRef(new MediaControlTimeDisplayElement(mediaElement, pseudoStyleId));
+}
+
 PassRefPtr<RenderStyle> MediaControlTimeDisplayElement::styleForElement()
 {
     RefPtr<RenderStyle> style = MediaControlElement::styleForElement();
@@ -704,32 +862,11 @@ void MediaControlTimeDisplayElement::setVisible(bool visible)
     renderer()->setStyle(style.get());
 }
 
-String MediaControlTimeDisplayElement::formatTime(float time)
-{
-    if (!isfinite(time))
-        time = 0;
-    int seconds = (int)fabsf(time);
-    int hours = seconds / (60 * 60);
-    int minutes = (seconds / 60) % 60;
-    seconds %= 60;
-    if (hours) {
-        if (hours > 9)
-            return String::format("%s%02d:%02d:%02d", (time < 0 ? "-" : ""), hours, minutes, seconds);
-
-        return String::format("%s%01d:%02d:%02d", (time < 0 ? "-" : ""), hours, minutes, seconds);
-    }
-
-    return String::format("%s%02d:%02d", (time < 0 ? "-" : ""), minutes, seconds);
-}
-
 void MediaControlTimeDisplayElement::setCurrentValue(float time)
 {
     m_currentValue = time;
-
-    ExceptionCode ec;
-    setInnerText(formatTime(m_currentValue), ec);
 }
 
+} // namespace WebCore
 
-} //namespace WebCore
-#endif // enable(video)
+#endif // ENABLE(VIDEO)
