@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -54,6 +54,8 @@ enum MediaControlElementType {
     MediaSliderThumb,
     MediaRewindButton,
     MediaReturnToRealtimeButton,
+    MediaShowClosedCaptionsButton,
+    MediaHideClosedCaptionsButton,
     MediaUnMuteButton,
     MediaPauseButton,
     MediaTimelineContainer,
@@ -63,41 +65,48 @@ enum MediaControlElementType {
     MediaControlsPanel,
     MediaVolumeSliderContainer,
     MediaVolumeSlider,
-    MediaVolumeSliderThumb
+    MediaVolumeSliderThumb,
+    MediaVolumeSliderMuteButton,
 };
 
+HTMLMediaElement* toParentMediaElement(RenderObject*);
+
 class MediaControlShadowRootElement : public HTMLDivElement {
 public:
-    MediaControlShadowRootElement(Document*, HTMLMediaElement*);
-    
-    virtual bool isShadowNode() const { return true; }
-    virtual Node* shadowParentNode() { return m_mediaElement; }
+    static PassRefPtr<MediaControlShadowRootElement> create(HTMLMediaElement*);
 
     void updateStyle();
+    virtual void detach();
     
 private:
-    HTMLMediaElement* m_mediaElement;    
+    MediaControlShadowRootElement(HTMLMediaElement*);
 };
 
 // ----------------------------
 
 class MediaControlElement : public HTMLDivElement {
 public:
-    MediaControlElement(Document*, PseudoId, HTMLMediaElement*);
-    virtual void attach();
-    virtual bool rendererIsNeeded(RenderStyle*);
+    static PassRefPtr<MediaControlElement> create(HTMLMediaElement*, PseudoId);
 
-    virtual PassRefPtr<RenderStyle> styleForElement();
+    virtual void attach();
     void attachToParent(Element*);
     void update();
-    virtual void updateStyle();
+    void updateStyle();
 
     MediaControlElementType displayType() const { return m_displayType; }
 
     HTMLMediaElement* mediaElement() const { return m_mediaElement; }
-    virtual bool isMediaControlElement() const { return true; }
 
 protected:
+    MediaControlElement(HTMLMediaElement*, PseudoId);
+
+    virtual bool rendererIsNeeded(RenderStyle*);
+
+    virtual PassRefPtr<RenderStyle> styleForElement();
+
+private:
+    virtual bool isMediaControlElement() const { return true; }
+
     HTMLMediaElement* m_mediaElement;   
     PseudoId m_pseudoStyleId;
     MediaControlElementType m_displayType;  // some elements can show multiple types (e.g. play/pause)
@@ -107,7 +116,10 @@ protected:
 
 class MediaControlTimelineContainerElement : public MediaControlElement {
 public:
-    MediaControlTimelineContainerElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlTimelineContainerElement> create(HTMLMediaElement*);
+
+private:
+    MediaControlTimelineContainerElement(HTMLMediaElement*);
     virtual bool rendererIsNeeded(RenderStyle*);
 };
 
@@ -115,7 +127,8 @@ public:
 
 class MediaControlVolumeSliderContainerElement : public MediaControlElement {
 public:
-    MediaControlVolumeSliderContainerElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlVolumeSliderContainerElement> create(HTMLMediaElement*);
+
     virtual PassRefPtr<RenderStyle> styleForElement();
     void setVisible(bool);
     bool isVisible() { return m_isVisible; }
@@ -123,6 +136,8 @@ public:
     bool hitTest(const IntPoint& absPoint);
 
 private:
+    MediaControlVolumeSliderContainerElement(HTMLMediaElement*);
+
     bool m_isVisible;
     int m_x, m_y;
 };
@@ -131,10 +146,15 @@ private:
 
 class MediaControlStatusDisplayElement : public MediaControlElement {
 public:
-    MediaControlStatusDisplayElement(Document*, HTMLMediaElement*);
-    virtual void update();
-    virtual bool rendererIsNeeded(RenderStyle*);
+    static PassRefPtr<MediaControlStatusDisplayElement> create(HTMLMediaElement*);
+
+    void update();
+
 private:
+    MediaControlStatusDisplayElement(HTMLMediaElement*);
+
+    virtual bool rendererIsNeeded(RenderStyle*);
+
     enum StateBeingDisplayed { Nothing, Loading, LiveBroadcast };
     StateBeingDisplayed m_stateBeingDisplayed;
 };
@@ -143,28 +163,33 @@ private:
 
 class MediaControlInputElement : public HTMLInputElement {
 public:
-    MediaControlInputElement(Document*, PseudoId, const String& type, HTMLMediaElement*);
-    virtual void attach();
-    virtual bool rendererIsNeeded(RenderStyle*);
-
-    virtual PassRefPtr<RenderStyle> styleForElement();
     void attachToParent(Element*);
     void update();
     void updateStyle();
 
     bool hitTest(const IntPoint& absPoint);
+
     MediaControlElementType displayType() const { return m_displayType; }
 
-    // Some elements are disabled by movie state (eg. mute if no audio).
-    virtual bool disabled() const  { return false; }
-
     HTMLMediaElement* mediaElement() const { return m_mediaElement; }
-    virtual bool isMediaControlElement() const { return true; }
 
 protected:
-    virtual void updateDisplayType() { }
+    MediaControlInputElement(HTMLMediaElement*, PseudoId);
+
     void setDisplayType(MediaControlElementType);
 
+    PseudoId pseudoStyleId() const { return m_pseudoStyleId; }
+
+private:
+    virtual void attach();
+    virtual bool rendererIsNeeded(RenderStyle*);
+
+    virtual PassRefPtr<RenderStyle> styleForElement();
+
+    virtual bool isMediaControlElement() const { return true; }
+
+    virtual void updateDisplayType() { }
+
     HTMLMediaElement* m_mediaElement;   
     PseudoId m_pseudoStyleId;
     MediaControlElementType m_displayType;
@@ -174,19 +199,28 @@ protected:
 
 class MediaControlMuteButtonElement : public MediaControlInputElement {
 public:
-    MediaControlMuteButtonElement(Document*, HTMLMediaElement*);
+    enum ButtonLocation { Controller, VolumeSlider };
+    static PassRefPtr<MediaControlMuteButtonElement> create(HTMLMediaElement*, ButtonLocation);
+
     virtual void defaultEventHandler(Event*);
+
+private:
+    MediaControlMuteButtonElement(HTMLMediaElement*, ButtonLocation);
+
     virtual void updateDisplayType();
-    virtual bool disabled() const;
-    virtual bool rendererIsNeeded(RenderStyle*);
 };
 
 // ----------------------------
 
 class MediaControlPlayButtonElement : public MediaControlInputElement {
 public:
-    MediaControlPlayButtonElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlPlayButtonElement> create(HTMLMediaElement*);
+
     virtual void defaultEventHandler(Event*);
+
+private:
+    MediaControlPlayButtonElement(HTMLMediaElement*);
+
     virtual void updateDisplayType();
 };
 
@@ -194,13 +228,18 @@ public:
 
 class MediaControlSeekButtonElement : public MediaControlInputElement {
 public:
-    MediaControlSeekButtonElement(Document*, HTMLMediaElement*, bool forward);
+    static PassRefPtr<MediaControlSeekButtonElement> create(HTMLMediaElement*, PseudoId);
+
     virtual void defaultEventHandler(Event*);
+
+private:
+    MediaControlSeekButtonElement(HTMLMediaElement*, PseudoId);
+
+    bool isForwardButton() const;
+
     virtual void detach();
     void seekTimerFired(Timer<MediaControlSeekButtonElement>*);
 
-private:
-    bool m_forward;
     bool m_seeking;
     bool m_capturing;
     Timer<MediaControlSeekButtonElement> m_seekTimer;
@@ -210,60 +249,93 @@ private:
 
 class MediaControlRewindButtonElement : public MediaControlInputElement {
 public:
-    MediaControlRewindButtonElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlRewindButtonElement> create(HTMLMediaElement*);
+
     virtual void defaultEventHandler(Event*);
-    virtual bool rendererIsNeeded(RenderStyle*);
+
+private:
+    MediaControlRewindButtonElement(HTMLMediaElement*);
 };
 
 // ----------------------------
 
 class MediaControlReturnToRealtimeButtonElement : public MediaControlInputElement {
 public:
-    MediaControlReturnToRealtimeButtonElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlReturnToRealtimeButtonElement> create(HTMLMediaElement*);
+
     virtual void defaultEventHandler(Event*);
-    virtual bool rendererIsNeeded(RenderStyle*);
+
+private:
+    MediaControlReturnToRealtimeButtonElement(HTMLMediaElement*);
+};    
+
+// ----------------------------
+
+class MediaControlToggleClosedCaptionsButtonElement : public MediaControlInputElement {
+public:
+    static PassRefPtr<MediaControlToggleClosedCaptionsButtonElement> create(HTMLMediaElement*);
+
+    virtual void defaultEventHandler(Event*);
+
+private:
+    MediaControlToggleClosedCaptionsButtonElement(HTMLMediaElement*);
+
+    virtual void updateDisplayType();
 };    
 
 // ----------------------------
 
 class MediaControlTimelineElement : public MediaControlInputElement {
 public:
-    MediaControlTimelineElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlTimelineElement> create(HTMLMediaElement*);
+
     virtual void defaultEventHandler(Event*);
     void update(bool updateDuration = true);
+
+private:
+    MediaControlTimelineElement(HTMLMediaElement*);
 };
 
 // ----------------------------
 
 class MediaControlVolumeSliderElement : public MediaControlInputElement {
 public:
-    MediaControlVolumeSliderElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlVolumeSliderElement> create(HTMLMediaElement*);
+
     virtual void defaultEventHandler(Event*);
+    void update();
+
+private:
+    MediaControlVolumeSliderElement(HTMLMediaElement*);
 };
 
 // ----------------------------
 
 class MediaControlFullscreenButtonElement : public MediaControlInputElement {
 public:
-    MediaControlFullscreenButtonElement(Document*, HTMLMediaElement*);
+    static PassRefPtr<MediaControlFullscreenButtonElement> create(HTMLMediaElement*);
+
     virtual void defaultEventHandler(Event*);
-    virtual bool rendererIsNeeded(RenderStyle*);
+
+private:
+    MediaControlFullscreenButtonElement(HTMLMediaElement*);
 };
 
 // ----------------------------
 
 class MediaControlTimeDisplayElement : public MediaControlElement {
 public:
-    MediaControlTimeDisplayElement(Document*, PseudoId, HTMLMediaElement*);
+    static PassRefPtr<MediaControlTimeDisplayElement> create(HTMLMediaElement*, PseudoId);
+
     void setVisible(bool);
-    virtual PassRefPtr<RenderStyle> styleForElement();
 
     void setCurrentValue(float);
     float currentValue() const { return m_currentValue; }
 
 private:
-    String formatTime(float time);
+    MediaControlTimeDisplayElement(HTMLMediaElement*, PseudoId);
 
+    virtual PassRefPtr<RenderStyle> styleForElement();
     float m_currentValue;
     bool m_isVisible;
 };
@@ -278,7 +350,8 @@ public:
     
 // ----------------------------
 
-
 } //namespace WebCore
-#endif // enable(video)
+
+#endif // ENABLE(VIDEO)
+
 #endif // MediaControlElements_h
