@@ -39,7 +39,7 @@ public:
     FlexBoxIterator(RenderFlexibleBox* parent)
     {
         box = parent;
-        if (box->style()->boxOrient() == HORIZONTAL && box->style()->direction() == RTL)
+        if (box->style()->boxOrient() == HORIZONTAL && !box->style()->isLeftToRightDirection())
             forward = box->style()->boxDirection() != BNORMAL;
         else
             forward = box->style()->boxDirection() == BNORMAL;
@@ -132,8 +132,8 @@ void RenderFlexibleBox::calcHorizontalPrefWidths()
             marginRight += mr.value();
         margin = marginLeft + marginRight;
 
-        m_minPrefWidth += child->minPrefWidth() + margin;
-        m_maxPrefWidth += child->maxPrefWidth() + margin;
+        m_minPreferredLogicalWidth += child->minPreferredLogicalWidth() + margin;
+        m_maxPreferredLogicalWidth += child->maxPreferredLogicalWidth() + margin;
     }    
 }
 
@@ -155,53 +155,56 @@ void RenderFlexibleBox::calcVerticalPrefWidths()
         if (mr.isFixed())
             margin += mr.value();
         
-        int w = child->minPrefWidth() + margin;
-        m_minPrefWidth = max(w, m_minPrefWidth);
+        int w = child->minPreferredLogicalWidth() + margin;
+        m_minPreferredLogicalWidth = max(w, m_minPreferredLogicalWidth);
         
-        w = child->maxPrefWidth() + margin;
-        m_maxPrefWidth = max(w, m_maxPrefWidth);
+        w = child->maxPreferredLogicalWidth() + margin;
+        m_maxPreferredLogicalWidth = max(w, m_maxPreferredLogicalWidth);
     }    
 }
 
-void RenderFlexibleBox::calcPrefWidths()
+void RenderFlexibleBox::computePreferredLogicalWidths()
 {
-    ASSERT(prefWidthsDirty());
+    ASSERT(preferredLogicalWidthsDirty());
 
     if (style()->width().isFixed() && style()->width().value() > 0)
-        m_minPrefWidth = m_maxPrefWidth = calcContentBoxWidth(style()->width().value());
+        m_minPreferredLogicalWidth = m_maxPreferredLogicalWidth = computeContentBoxLogicalWidth(style()->width().value());
     else {
-        m_minPrefWidth = m_maxPrefWidth = 0;
+        m_minPreferredLogicalWidth = m_maxPreferredLogicalWidth = 0;
 
         if (hasMultipleLines() || isVertical())
             calcVerticalPrefWidths();
         else
             calcHorizontalPrefWidths();
 
-        m_maxPrefWidth = max(m_minPrefWidth, m_maxPrefWidth);
+        m_maxPreferredLogicalWidth = max(m_minPreferredLogicalWidth, m_maxPreferredLogicalWidth);
+    }
+
+    if (hasOverflowClip() && style()->overflowY() == OSCROLL) {
+        layer()->setHasVerticalScrollbar(true);
+        int scrollbarWidth = verticalScrollbarWidth();
+        m_maxPreferredLogicalWidth += scrollbarWidth;
+        m_minPreferredLogicalWidth += scrollbarWidth;
     }
 
     if (style()->minWidth().isFixed() && style()->minWidth().value() > 0) {
-        m_maxPrefWidth = max(m_maxPrefWidth, calcContentBoxWidth(style()->minWidth().value()));
-        m_minPrefWidth = max(m_minPrefWidth, calcContentBoxWidth(style()->minWidth().value()));
+        m_maxPreferredLogicalWidth = max(m_maxPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->minWidth().value()));
+        m_minPreferredLogicalWidth = max(m_minPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->minWidth().value()));
     }
     
     if (style()->maxWidth().isFixed() && style()->maxWidth().value() != undefinedLength) {
-        m_maxPrefWidth = min(m_maxPrefWidth, calcContentBoxWidth(style()->maxWidth().value()));
-        m_minPrefWidth = min(m_minPrefWidth, calcContentBoxWidth(style()->maxWidth().value()));
+        m_maxPreferredLogicalWidth = min(m_maxPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->maxWidth().value()));
+        m_minPreferredLogicalWidth = min(m_minPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->maxWidth().value()));
     }
 
-    int toAdd = borderLeft() + borderRight() + paddingLeft() + paddingRight();
-    
-    if (hasOverflowClip() && style()->overflowY() == OSCROLL)
-        toAdd += verticalScrollbarWidth();
+    int borderAndPadding = borderAndPaddingLogicalWidth();
+    m_minPreferredLogicalWidth += borderAndPadding;
+    m_maxPreferredLogicalWidth += borderAndPadding;
 
-    m_minPrefWidth += toAdd;
-    m_maxPrefWidth += toAdd;
-
-    setPrefWidthsDirty(false);
+    setPreferredLogicalWidthsDirty(false);
 }
 
-void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
+void RenderFlexibleBox::layoutBlock(bool relayoutChildren, int /*pageHeight FIXME: Implement */)
 {
     ASSERT(needsLayout());
 
@@ -209,14 +212,15 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
         return;
 
     LayoutRepainter repainter(*this, checkForRepaintDuringLayout());
-    LayoutStateMaintainer statePusher(view(), this, IntSize(x(), y()), hasTransform() || hasReflection());
+    LayoutStateMaintainer statePusher(view(), this, IntSize(x(), y()), hasTransform() || hasReflection() || style()->isFlippedBlocksWritingMode());
 
     int previousWidth = width();
     int previousHeight = height();
     
-    calcWidth();
-    calcHeight();
-    m_overflowWidth = width();
+    computeLogicalWidth();
+    computeLogicalHeight();
+    
+    m_overflow.clear();
 
     if (previousWidth != width() || previousHeight != height() ||
         (parent()->isFlexibleBox() && parent()->style()->boxOrient() == HORIZONTAL &&
@@ -224,7 +228,7 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
         relayoutChildren = true;
 
     setHeight(0);
-    m_overflowHeight = 0;
+
     m_flexingChildren = m_stretchingChildren = false;
 
     initMaxMarginValues();
@@ -242,15 +246,9 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
     else
         layoutVerticalBox(relayoutChildren);
 
-    int oldHeight = height();
-    calcHeight();
-    if (oldHeight != height()) {
-        // If the block got expanded in size, then increase our overflowheight to match.
-        if (m_overflowHeight > height())
-            m_overflowHeight -= (borderBottom() + paddingBottom() + horizontalScrollbarHeight());
-        if (m_overflowHeight < height())
-            m_overflowHeight = height();
-    }
+    int oldClientAfterEdge = clientLogicalBottom();
+    computeLogicalHeight();
+
     if (previousHeight != height())
         relayoutChildren = true;
 
@@ -264,39 +262,25 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
         // bottom margin max values to 0.  This way we don't factor in the values
         // twice when we collapse with our previous vertically adjacent and
         // following vertically adjacent blocks.
-        int pos = maxTopPosMargin();
-        int neg = maxTopNegMargin();
-        if (maxBottomPosMargin() > pos)
-            pos = maxBottomPosMargin();
-        if (maxBottomNegMargin() > neg)
-            neg = maxBottomNegMargin();
-        setMaxTopMargins(pos, neg);
-        setMaxBottomMargins(0, 0);
-    }
-
-    // Always ensure our overflow width is at least as large as our width.
-    if (m_overflowWidth < width())
-        m_overflowWidth = width();
-
-    if (!hasOverflowClip()) {
-        for (ShadowData* boxShadow = style()->boxShadow(); boxShadow; boxShadow = boxShadow->next) {
-            m_overflowLeft = min(m_overflowLeft, boxShadow->x - boxShadow->blur - boxShadow->spread);
-            m_overflowWidth = max(m_overflowWidth, width() + boxShadow->x + boxShadow->blur + boxShadow->spread);
-            m_overflowTop = min(m_overflowTop, boxShadow->y - boxShadow->blur - boxShadow->spread);
-            m_overflowHeight = max(m_overflowHeight, height() + boxShadow->y + boxShadow->blur + boxShadow->spread);
-        }
-        
-        if (hasReflection()) {
-            IntRect reflection(reflectionBox());
-            m_overflowTop = min(m_overflowTop, reflection.y());
-            m_overflowHeight = max(m_overflowHeight, reflection.bottom());
-            m_overflowLeft = min(m_overflowLeft, reflection.x());
-            m_overflowHeight = max(m_overflowWidth, reflection.right());
-        }
+        int pos = maxPositiveMarginBefore();
+        int neg = maxNegativeMarginBefore();
+        if (maxPositiveMarginAfter() > pos)
+            pos = maxPositiveMarginAfter();
+        if (maxNegativeMarginAfter() > neg)
+            neg = maxNegativeMarginAfter();
+        setMaxMarginBeforeValues(pos, neg);
+        setMaxMarginAfterValues(0, 0);
     }
+    
+    computeOverflow(oldClientAfterEdge);
 
     statePusher.pop();
 
+    updateLayerTransform();
+
+    if (view()->layoutState()->pageLogicalHeight())
+        setPageLogicalOffset(view()->layoutState()->pageLogicalOffset(y()));
+
     // Update our scrollbars if we're overflow:auto/scroll/hidden now that we know if
     // we overflow or not.
     if (hasOverflowClip())
@@ -308,23 +292,10 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
     setNeedsLayout(false);
 }
 
-void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
+// The first walk over our kids is to find out if we have any flexible children.
+static void gatherFlexChildrenInfo(FlexBoxIterator& iterator, bool relayoutChildren, unsigned int& highestFlexGroup, unsigned int& lowestFlexGroup, bool& haveFlex)
 {
-    int toAdd = borderBottom() + paddingBottom() + horizontalScrollbarHeight();
-    int yPos = borderTop() + paddingTop();
-    int xPos = borderLeft() + paddingLeft();
-    bool heightSpecified = false;
-    int oldHeight = 0;
-    
-    unsigned int highestFlexGroup = 0;
-    unsigned int lowestFlexGroup = 0;
-    bool haveFlex = false;
-    int remainingSpace = 0;
-    m_overflowHeight = height();
-
-    // The first walk over our kids is to find out if we have any flexible children.
-    FlexBoxIterator iterator(this);
-    RenderBox* child = iterator.next();
+    RenderBox* child = iterator.first();
     while (child) {
         // Check to see if this child flexes.
         if (!child->isPositioned() && child->style()->boxFlex() > 0.0f) {
@@ -344,6 +315,26 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
         }
         child = iterator.next();
     }
+}
+
+void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
+{
+    int toAdd = borderBottom() + paddingBottom() + horizontalScrollbarHeight();
+    int yPos = borderTop() + paddingTop();
+    int xPos = borderLeft() + paddingLeft();
+    bool heightSpecified = false;
+    int oldHeight = 0;
+
+    int remainingSpace = 0;
+
+
+    FlexBoxIterator iterator(this);
+    unsigned int highestFlexGroup = 0;
+    unsigned int lowestFlexGroup = 0;
+    bool haveFlex = false;
+    gatherFlexChildrenInfo(iterator, relayoutChildren, highestFlexGroup, lowestFlexGroup, haveFlex);
+
+    RenderBox* child;
 
     RenderBlock::startDelayUpdateScrollInfo();
 
@@ -352,7 +343,7 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
     do {
         // Reset our height.
         setHeight(yPos);
-        m_overflowHeight = height();
+
         xPos = borderLeft() + paddingLeft();
                 
         // Our first pass is done without flexing.  We simply lay the children
@@ -371,8 +362,11 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
             }
     
             // Compute the child's vertical margins.
-            child->calcVerticalMargins();
+            child->computeBlockDirectionMargins(this);
     
+            if (!child->needsLayout())
+                child->markForPaginationRelayoutIfNeeded();
+
             // Now do the layout.
             child->layoutIfNeeded();
     
@@ -400,16 +394,12 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
         }
         
         if (!iterator.first() && hasLineIfEmpty())
-            setHeight(height() + lineHeight(true, true));
+            setHeight(height() + lineHeight(true, style()->isHorizontalWritingMode() ? HorizontalLine : VerticalLine, PositionOfInteriorLineBoxes));
         
         setHeight(height() + toAdd);
-
-        // Always make sure our overflowheight is at least our height.
-        if (m_overflowHeight < height())
-            m_overflowHeight = height();
         
         oldHeight = height();
-        calcHeight();
+        computeLogicalHeight();
 
         relayoutChildren = false;
         if (oldHeight != height())
@@ -422,12 +412,17 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
             if (child->isPositioned()) {
                 child->containingBlock()->insertPositionedObject(child);
                 if (child->style()->hasStaticX()) {
-                    if (style()->direction() == LTR)
+                    if (style()->isLeftToRightDirection())
                         child->layer()->setStaticX(xPos);
                     else child->layer()->setStaticX(width() - xPos);
                 }
-                if (child->style()->hasStaticY())
-                    child->layer()->setStaticY(yPos);
+                if (child->style()->hasStaticY()) {
+                    RenderLayer* childLayer = child->layer();
+                    if (childLayer->staticY() != yPos) {
+                        child->layer()->setStaticY(yPos);
+                        child->setChildNeedsLayout(true, false);
+                    }
+                }
                 child = iterator.next();
                 continue;
             }
@@ -436,9 +431,13 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
             // fill the height of a containing box by default.
             // Now do a layout.
             int oldChildHeight = child->height();
-            child->calcHeight();
+            child->computeLogicalHeight();
             if (oldChildHeight != child->height())
                 child->setChildNeedsLayout(true, false);
+                
+            if (!child->needsLayout())
+                child->markForPaginationRelayoutIfNeeded();
+
             child->layoutIfNeeded();
     
             // We can place the child now, using our value of box-align.
@@ -465,12 +464,6 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
             }
 
             placeChild(child, xPos, childY);
-
-            if (child->isRenderBlock())
-                toRenderBlock(child)->addVisualOverflow(toRenderBlock(child)->floatRect());
-
-            m_overflowHeight = max(m_overflowHeight, childY + child->overflowHeight(false));
-            m_overflowTop = min(m_overflowTop, child->y() + child->overflowTop(false));
             
             xPos += child->width() + child->marginRight();
     
@@ -574,8 +567,8 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
 
     RenderBlock::finishDelayUpdateScrollInfo();
 
-    if (remainingSpace > 0 && ((style()->direction() == LTR && style()->boxPack() != BSTART) ||
-                               (style()->direction() == RTL && style()->boxPack() != BEND))) {
+    if (remainingSpace > 0 && ((style()->isLeftToRightDirection() && style()->boxPack() != BSTART)
+        || (!style()->isLeftToRightDirection() && style()->boxPack() != BEND))) {
         // Children must be repositioned.
         int offset = 0;
         if (style()->boxPack() == BJUSTIFY) {
@@ -634,23 +627,7 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
         }
     }
     
-    child = iterator.first();
-    while (child && child->isPositioned()) {
-        child = iterator.next();
-    }
-    
-    if (child) {
-        m_overflowLeft = min(child->x() + child->overflowLeft(false), m_overflowLeft);
-
-        RenderBox* lastChild = child;
-        while ((child = iterator.next())) {
-            if (!child->isPositioned())
-                lastChild = child;
-        }
-        m_overflowWidth = max(lastChild->x() + lastChild->overflowWidth(false), m_overflowWidth);
-    }
-    
-    // So that the calcHeight in layoutBlock() knows to relayout positioned objects because of
+    // So that the computeLogicalHeight in layoutBlock() knows to relayout positioned objects because of
     // a height change, we revert our height back to the intrinsic height before returning.
     if (heightSpecified)
         setHeight(oldHeight);
@@ -660,146 +637,27 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
 {
     int xPos = borderLeft() + paddingLeft();
     int yPos = borderTop() + paddingTop();
-    if (style()->direction() == RTL)
+    if (!style()->isLeftToRightDirection())
         xPos = width() - paddingRight() - borderRight();
     int toAdd = borderBottom() + paddingBottom() + horizontalScrollbarHeight();
     bool heightSpecified = false;
     int oldHeight = 0;
-    
+
+    int remainingSpace = 0;
+
+    FlexBoxIterator iterator(this);
     unsigned int highestFlexGroup = 0;
     unsigned int lowestFlexGroup = 0;
     bool haveFlex = false;
-    int remainingSpace = 0;
-    
-    // The first walk over our kids is to find out if we have any flexible children.
-    FlexBoxIterator iterator(this);
-    RenderBox* child = iterator.next();
-    while (child) {
-        // Check to see if this child flexes.
-        if (!child->isPositioned() && child->style()->boxFlex() > 0.0f) {
-            // We always have to lay out flexible objects again, since the flex distribution
-            // may have changed, and we need to reallocate space.
-            child->setOverrideSize(-1);
-            if (!relayoutChildren)
-                child->setChildNeedsLayout(true, false);
-            haveFlex = true;
-            unsigned int flexGroup = child->style()->boxFlexGroup();
-            if (lowestFlexGroup == 0)
-                lowestFlexGroup = flexGroup;
-            if (flexGroup < lowestFlexGroup)
-                lowestFlexGroup = flexGroup;
-            if (flexGroup > highestFlexGroup)
-                highestFlexGroup = flexGroup;
-        }
-        child = iterator.next();
-    }
+    gatherFlexChildrenInfo(iterator, relayoutChildren, highestFlexGroup, lowestFlexGroup, haveFlex);
+
+    RenderBox* child;
 
     // We confine the line clamp ugliness to vertical flexible boxes (thus keeping it out of
     // mainstream block layout); this is not really part of the XUL box model.
-    bool haveLineClamp = style()->lineClamp() >= 0 && style()->lineClamp() <= 100;
-    if (haveLineClamp) {
-        int maxLineCount = 0;
-        child = iterator.first();
-        while (child) {
-            if (!child->isPositioned()) {
-                if (relayoutChildren || (child->isReplaced() && (child->style()->width().isPercent() || child->style()->height().isPercent())) ||
-                    (child->style()->height().isAuto() && child->isBlockFlow() && !child->needsLayout())) {
-                    child->setChildNeedsLayout(true, false);
-                    
-                    // Dirty all the positioned objects.
-                    if (child->isRenderBlock()) {
-                        toRenderBlock(child)->markPositionedObjectsForLayout();
-                        toRenderBlock(child)->clearTruncation();
-                    }
-                }
-                child->layoutIfNeeded();
-                if (child->style()->height().isAuto() && child->isBlockFlow())
-                    maxLineCount = max(maxLineCount, toRenderBlock(child)->lineCount());
-            }
-            child = iterator.next();
-        }
-        
-        // Get the # of lines and then alter all block flow children with auto height to use the
-        // specified height. We always try to leave room for at least one line.
-        int numVisibleLines = max(1, static_cast<int>((maxLineCount + 1) * style()->lineClamp() / 100.0));
-        if (numVisibleLines < maxLineCount) {
-            for (child = iterator.first(); child; child = iterator.next()) {
-                if (child->isPositioned() || !child->style()->height().isAuto() || !child->isBlockFlow())
-                    continue;
-                
-                RenderBlock* blockChild = toRenderBlock(child);
-                int lineCount = blockChild->lineCount();
-                if (lineCount <= numVisibleLines)
-                    continue;
-                
-                int newHeight = blockChild->heightForLineCount(numVisibleLines);
-                if (newHeight == child->height())
-                    continue;
-                
-                child->setChildNeedsLayout(true, false);
-                child->setOverrideSize(newHeight);
-                m_flexingChildren = true;
-                child->layoutIfNeeded();
-                m_flexingChildren = false;
-                child->setOverrideSize(-1);
-                
-                // FIXME: For now don't support RTL.
-                if (style()->direction() != LTR)
-                    continue;
-                
-                // Get the last line
-                RootInlineBox* lastLine = blockChild->lineAtIndex(lineCount-1);
-                if (!lastLine)
-                    continue;
-                
-                // See if the last item is an anchor
-                InlineBox* anchorBox = lastLine->lastChild();
-                if (!anchorBox)
-                    continue;
-                if (!anchorBox->renderer()->node())
-                    continue;
-                if (!anchorBox->renderer()->node()->isLink())
-                    continue;
-                
-                RootInlineBox* lastVisibleLine = blockChild->lineAtIndex(numVisibleLines-1);
-                if (!lastVisibleLine)
-                    continue;
-
-                const UChar ellipsisAndSpace[2] = { horizontalEllipsis, ' ' };
-                DEFINE_STATIC_LOCAL(AtomicString, ellipsisAndSpaceStr, (ellipsisAndSpace, 2));
-
-                const Font& font = style(numVisibleLines == 1)->font();
-                int ellipsisAndSpaceWidth = font.width(TextRun(ellipsisAndSpace, 2));
-
-                // Get ellipsis width + " " + anchor width
-                int totalWidth = ellipsisAndSpaceWidth + anchorBox->width();
-                
-                // See if this width can be accommodated on the last visible line
-                RenderBlock* destBlock = toRenderBlock(lastVisibleLine->renderer());
-                RenderBlock* srcBlock = toRenderBlock(lastLine->renderer());
-                
-                // FIXME: Directions of src/destBlock could be different from our direction and from one another.
-                if (srcBlock->style()->direction() != LTR)
-                    continue;
-                if (destBlock->style()->direction() != LTR)
-                    continue;
-                int ltr = true;
-
-                int blockRightEdge = destBlock->rightOffset(lastVisibleLine->y(), false);
-                int blockLeftEdge = destBlock->leftOffset(lastVisibleLine->y(), false);
-
-                int blockEdge = ltr ? blockRightEdge : blockLeftEdge;
-                if (!lastVisibleLine->canAccommodateEllipsis(ltr, blockEdge,
-                                                             lastVisibleLine->x() + lastVisibleLine->width(),
-                                                             totalWidth))
-                    continue;
-
-                // Let the truncation code kick in.
-                lastVisibleLine->placeEllipsis(ellipsisAndSpaceStr, ltr, blockLeftEdge, blockRightEdge, totalWidth, anchorBox);
-                destBlock->setHasMarkupTruncation(true);
-            }
-        }
-    }
+    bool haveLineClamp = !style()->lineClamp().isNone();
+    if (haveLineClamp)
+        applyLineClamp(iterator, relayoutChildren);
 
     RenderBlock::startDelayUpdateScrollInfo();
 
@@ -810,7 +668,6 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
     do {
         setHeight(borderTop() + paddingTop());
         int minHeight = height() + toAdd;
-        m_overflowHeight = height();
 
         child = iterator.first();
         while (child) {
@@ -818,27 +675,34 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
             if (!haveLineClamp && (relayoutChildren || (child->isReplaced() && (child->style()->width().isPercent() || child->style()->height().isPercent()))))
                 child->setChildNeedsLayout(true, false);
     
-            if (child->isPositioned())
-            {
+            if (child->isPositioned()) {
                 child->containingBlock()->insertPositionedObject(child);
                 if (child->style()->hasStaticX()) {
-                    if (style()->direction() == LTR)
+                    if (style()->isLeftToRightDirection())
                         child->layer()->setStaticX(borderLeft()+paddingLeft());
                     else
                         child->layer()->setStaticX(borderRight()+paddingRight());
                 }
-                if (child->style()->hasStaticY())
-                    child->layer()->setStaticY(height());
+                if (child->style()->hasStaticY()) {
+                    RenderLayer* childLayer = child->layer();
+                    if (childLayer->staticY() != height()) {
+                        child->layer()->setStaticY(height());
+                        child->setChildNeedsLayout(true, false);
+                    }
+                }
                 child = iterator.next();
                 continue;
             } 
     
             // Compute the child's vertical margins.
-            child->calcVerticalMargins();
+            child->computeBlockDirectionMargins(this);
     
             // Add in the child's marginTop to our height.
             setHeight(height() + child->marginTop());
     
+            if (!child->needsLayout())
+                child->markForPaginationRelayoutIfNeeded();
+
             // Now do a layout.
             child->layoutIfNeeded();
     
@@ -850,13 +714,13 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                     childX += child->marginLeft() + max(0, (contentWidth() - (child->width() + child->marginLeft() + child->marginRight()))/2);
                     break;
                 case BEND:
-                    if (style()->direction() == RTL)
+                    if (!style()->isLeftToRightDirection())
                         childX += child->marginLeft();
                     else
                         childX += contentWidth() - child->marginRight() - child->width();
                     break;
                 default: // BSTART/BSTRETCH
-                    if (style()->direction() == LTR)
+                    if (style()->isLeftToRightDirection())
                         childX += child->marginLeft();
                     else
                         childX += contentWidth() - child->marginRight() - child->width();
@@ -866,21 +730,14 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
             // Place the child.
             placeChild(child, childX, height());
             setHeight(height() + child->height() + child->marginBottom());
-    
-            if (child->isRenderBlock())
-                toRenderBlock(child)->addVisualOverflow(toRenderBlock(child)->floatRect());
 
-            // See if this child has made our overflow need to grow.
-            m_overflowWidth = max(child->x() + child->overflowWidth(false), m_overflowWidth);
-            m_overflowLeft = min(child->x() + child->overflowLeft(false), m_overflowLeft);
-            
             child = iterator.next();
         }
 
         yPos = height();
         
         if (!iterator.first() && hasLineIfEmpty())
-            setHeight(height() + lineHeight(true, true));
+            setHeight(height() + lineHeight(true, style()->isHorizontalWritingMode() ? HorizontalLine : VerticalLine, PositionOfInteriorLineBoxes));
     
         setHeight(height() + toAdd);
 
@@ -889,13 +746,9 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
         if (height() < minHeight)
             setHeight(minHeight);
 
-        // Always make sure our overflowheight is at least our height.
-        if (m_overflowHeight < height())
-            m_overflowHeight = height();
-
         // Now we have to calc our height, so we know how much space we have remaining.
         oldHeight = height();
-        calcHeight();
+        computeLogicalHeight();
         if (oldHeight != height())
             heightSpecified = true;
 
@@ -1050,29 +903,115 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
             }
         }
     }
-    
-    child = iterator.first();
-    while (child && child->isPositioned()) {
-        child = iterator.next();
-    }
-    
-    if (child) {
-        m_overflowTop = min(child->y() + child->overflowTop(false), m_overflowTop);
 
-        RenderBox* lastChild = child;
-        while ((child = iterator.next())) {
-            if (!child->isPositioned())
-                lastChild = child;
-        }
-        m_overflowHeight = max(lastChild->y() + lastChild->overflowHeight(false), m_overflowHeight);
-    }
-
-    // So that the calcHeight in layoutBlock() knows to relayout positioned objects because of
+    // So that the computeLogicalHeight in layoutBlock() knows to relayout positioned objects because of
     // a height change, we revert our height back to the intrinsic height before returning.
     if (heightSpecified)
         setHeight(oldHeight); 
 }
 
+void RenderFlexibleBox::applyLineClamp(FlexBoxIterator& iterator, bool relayoutChildren)
+{
+    int maxLineCount = 0;
+    for (RenderBox* child = iterator.first(); child; child = iterator.next()) {
+        if (child->isPositioned())
+            continue;
+
+        if (relayoutChildren || (child->isReplaced() && (child->style()->width().isPercent() || child->style()->height().isPercent()))
+            || (child->style()->height().isAuto() && child->isBlockFlow())) {
+            child->setChildNeedsLayout(true, false);
+
+            // Dirty all the positioned objects.
+            if (child->isRenderBlock()) {
+                toRenderBlock(child)->markPositionedObjectsForLayout();
+                toRenderBlock(child)->clearTruncation();
+            }
+        }
+        child->layoutIfNeeded();
+        if (child->style()->height().isAuto() && child->isBlockFlow())
+            maxLineCount = max(maxLineCount, toRenderBlock(child)->lineCount());
+    }
+
+    // Get the number of lines and then alter all block flow children with auto height to use the
+    // specified height. We always try to leave room for at least one line.
+    LineClampValue lineClamp = style()->lineClamp();
+    int numVisibleLines = lineClamp.isPercentage() ? max(1, (maxLineCount + 1) * lineClamp.value() / 100) : lineClamp.value();
+    if (numVisibleLines >= maxLineCount)
+        return;
+
+    for (RenderBox* child = iterator.first(); child; child = iterator.next()) {
+        if (child->isPositioned() || !child->style()->height().isAuto() || !child->isBlockFlow())
+            continue;
+        
+        RenderBlock* blockChild = toRenderBlock(child);
+        int lineCount = blockChild->lineCount();
+        if (lineCount <= numVisibleLines)
+            continue;
+        
+        int newHeight = blockChild->heightForLineCount(numVisibleLines);
+        if (newHeight == child->height())
+            continue;
+        
+        child->setChildNeedsLayout(true, false);
+        child->setOverrideSize(newHeight);
+        m_flexingChildren = true;
+        child->layoutIfNeeded();
+        m_flexingChildren = false;
+        child->setOverrideSize(-1);
+
+        // FIXME: For now don't support RTL.
+        if (style()->direction() != LTR)
+            continue;
+
+        // Get the last line
+        RootInlineBox* lastLine = blockChild->lineAtIndex(lineCount-1);
+        if (!lastLine)
+            continue;
+
+        RootInlineBox* lastVisibleLine = blockChild->lineAtIndex(numVisibleLines-1);
+        if (!lastVisibleLine)
+            continue;
+
+        const UChar ellipsisAndSpace[2] = { horizontalEllipsis, ' ' };
+        DEFINE_STATIC_LOCAL(AtomicString, ellipsisAndSpaceStr, (ellipsisAndSpace, 2));
+        DEFINE_STATIC_LOCAL(AtomicString, ellipsisStr, (&horizontalEllipsis, 1));
+        const Font& font = style(numVisibleLines == 1)->font();
+
+        // Get ellipsis width, and if the last child is an anchor, it will go after the ellipsis, so add in a space and the anchor width too 
+        int totalWidth;
+        InlineBox* anchorBox = lastLine->lastChild();
+        if (anchorBox && anchorBox->renderer()->node() && anchorBox->renderer()->node()->isLink())
+            totalWidth = anchorBox->logicalWidth() + font.width(TextRun(ellipsisAndSpace, 2));
+        else {
+            anchorBox = 0;
+            totalWidth = font.width(TextRun(&horizontalEllipsis, 1));
+        }
+
+        // See if this width can be accommodated on the last visible line
+        RenderBlock* destBlock = toRenderBlock(lastVisibleLine->renderer());
+        RenderBlock* srcBlock = toRenderBlock(lastLine->renderer());
+        
+        // FIXME: Directions of src/destBlock could be different from our direction and from one another.
+        if (!srcBlock->style()->isLeftToRightDirection())
+            continue;
+
+        bool leftToRight = destBlock->style()->isLeftToRightDirection();
+        if (!leftToRight)
+            continue;
+
+        int blockRightEdge = destBlock->logicalRightOffsetForLine(lastVisibleLine->y(), false);
+        int blockLeftEdge = destBlock->logicalLeftOffsetForLine(lastVisibleLine->y(), false);
+
+        int blockEdge = leftToRight ? blockRightEdge : blockLeftEdge;
+        if (!lastVisibleLine->canAccommodateEllipsis(leftToRight, blockEdge, lastVisibleLine->x() + lastVisibleLine->logicalWidth(), totalWidth))
+            continue;
+
+        // Let the truncation code kick in.
+        lastVisibleLine->placeEllipsis(anchorBox ? ellipsisAndSpaceStr : ellipsisStr, leftToRight, blockLeftEdge, blockRightEdge, totalWidth, anchorBox);
+        destBlock->setHasMarkupTruncation(true);
+    }
+}
+
 void RenderFlexibleBox::placeChild(RenderBox* child, int x, int y)
 {
     IntRect oldRect(child->x(), child->y() , child->width(), child->height());
@@ -1096,21 +1035,21 @@ int RenderFlexibleBox::allowedChildFlex(RenderBox* child, bool expanding, unsign
         if (isHorizontal()) {
             // FIXME: For now just handle fixed values.
             int maxW = INT_MAX;
-            int w = child->overrideWidth() - (child->borderLeft() + child->borderRight() + child->paddingLeft() + child->paddingRight());
+            int w = child->overrideWidth() - child->borderAndPaddingWidth();
             if (!child->style()->maxWidth().isUndefined() &&
                 child->style()->maxWidth().isFixed())
                 maxW = child->style()->maxWidth().value();
             else if (child->style()->maxWidth().type() == Intrinsic)
-                maxW = child->maxPrefWidth();
+                maxW = child->maxPreferredLogicalWidth();
             else if (child->style()->maxWidth().type() == MinIntrinsic)
-                maxW = child->minPrefWidth();
+                maxW = child->minPreferredLogicalWidth();
             if (maxW == INT_MAX)
                 return maxW;
             return max(0, maxW - w);
         } else {
             // FIXME: For now just handle fixed values.
             int maxH = INT_MAX;
-            int h = child->overrideHeight() - (child->borderTop() + child->borderBottom() + child->paddingTop() + child->paddingBottom());
+            int h = child->overrideHeight() - child->borderAndPaddingHeight();
             if (!child->style()->maxHeight().isUndefined() &&
                 child->style()->maxHeight().isFixed())
                 maxH = child->style()->maxHeight().value();
@@ -1122,21 +1061,21 @@ int RenderFlexibleBox::allowedChildFlex(RenderBox* child, bool expanding, unsign
 
     // FIXME: For now just handle fixed values.
     if (isHorizontal()) {
-        int minW = child->minPrefWidth();
-        int w = child->overrideWidth() - (child->borderLeft() + child->borderRight() + child->paddingLeft() + child->paddingRight());
+        int minW = child->minPreferredLogicalWidth();
+        int w = child->overrideWidth() - child->borderAndPaddingWidth();
         if (child->style()->minWidth().isFixed())
             minW = child->style()->minWidth().value();
         else if (child->style()->minWidth().type() == Intrinsic)
-            minW = child->maxPrefWidth();
+            minW = child->maxPreferredLogicalWidth();
         else if (child->style()->minWidth().type() == MinIntrinsic)
-            minW = child->minPrefWidth();
+            minW = child->minPreferredLogicalWidth();
             
         int allowedShrinkage = min(0, minW - w);
         return allowedShrinkage;
     } else {
         if (child->style()->minHeight().isFixed()) {
             int minH = child->style()->minHeight().value();
-            int h = child->overrideHeight() - (child->borderLeft() + child->borderRight() + child->paddingLeft() + child->paddingRight());
+            int h = child->overrideHeight() - child->borderAndPaddingHeight();
             int allowedShrinkage = min(0, minH - h);
             return allowedShrinkage;
         }
@@ -1151,6 +1090,8 @@ const char *RenderFlexibleBox::renderName() const
         return "RenderFlexibleBox (floating)";
     if (isPositioned())
         return "RenderFlexibleBox (positioned)";
+    if (isAnonymous())
+        return "RenderFlexibleBox (generated)";
     if (isRelPositioned())
         return "RenderFlexibleBox (relative positioned)";
     return "RenderFlexibleBox";
