@@ -47,22 +47,17 @@
 #include "RenderBox.h"
 #include "ScrollBehavior.h"
 #include "ScrollbarClient.h"
-#include "Timer.h"
 #include <wtf/OwnPtr.h>
 
 namespace WebCore {
 
-class CachedResource;
 class HitTestRequest;
 class HitTestResult;
 class HitTestingTransformState;
-class RenderFrameSet;
 class RenderMarquee;
 class RenderReplica;
 class RenderScrollbarPart;
 class RenderStyle;
-class RenderTable;
-class RenderText;
 class RenderView;
 class Scrollbar;
 class TransformationMatrix;
@@ -145,8 +140,6 @@ public:
         m_fixed = other.fixed();
         return *this;
     }
-    
-    static IntRect infiniteRect() { return IntRect(INT_MIN/2, INT_MIN/2, INT_MAX, INT_MAX); }
 
 private:
     // The normal operator new is disallowed on all render objects.
@@ -240,14 +233,14 @@ public:
     // Scrolling methods for layers that can scroll their overflow.
     void scrollByRecursively(int xDelta, int yDelta);
 
-    IntSize scrolledContentOffset() const { return IntSize(scrollXOffset() + m_scrollLeftOverflow, scrollYOffset()); }
+    IntSize scrolledContentOffset() const { return IntSize(scrollXOffset() + m_scrollLeftOverflow, scrollYOffset() + m_scrollTopOverflow); }
 
-    int scrollXOffset() const { return m_scrollX + m_scrollOriginX; }
-    int scrollYOffset() const { return m_scrollY; }
+    int scrollXOffset() const { return m_scrollX + m_scrollOrigin.x(); }
+    int scrollYOffset() const { return m_scrollY + m_scrollOrigin.y(); }
 
     void scrollToOffset(int x, int y, bool updateScrollbars = true, bool repaint = true);
-    void scrollToXOffset(int x) { scrollToOffset(x, m_scrollY); }
-    void scrollToYOffset(int y) { scrollToOffset(m_scrollX + m_scrollOriginX, y); }
+    void scrollToXOffset(int x) { scrollToOffset(x, m_scrollY + m_scrollOrigin.y()); }
+    void scrollToYOffset(int y) { scrollToOffset(m_scrollX + m_scrollOrigin.x(), y); }
     void scrollRectToVisible(const IntRect&, bool scrollToAnchor = false, const ScrollAlignment& alignX = ScrollAlignment::alignCenterIfNeeded, const ScrollAlignment& alignY = ScrollAlignment::alignCenterIfNeeded);
 
     IntRect getRectToExpose(const IntRect& visibleRect, const IntRect& exposeRect, const ScrollAlignment& alignX, const ScrollAlignment& alignY);
@@ -276,7 +269,7 @@ public:
 
     void updateScrollInfoAfterLayout();
 
-    bool scroll(ScrollDirection, ScrollGranularity, float multiplier = 1.0f);
+    bool scroll(ScrollDirection, ScrollGranularity, float multiplier = 1);
     void autoscroll();
 
     void resize(const PlatformMouseEvent&, const IntSize&);
@@ -290,12 +283,15 @@ public:
     
     // Notification from the renderer that its content changed (e.g. current frame of image changed).
     // Allows updates of layer content without repainting.
-    void rendererContentChanged();
+    enum ContentChangeType { ImageChanged, MaskImageChanged, CanvasChanged, VideoChanged };
+    void contentChanged(ContentChangeType);
 #endif
 
     // Returns true if the accelerated compositing is enabled
     bool hasAcceleratedCompositing() const;
-    
+
+    bool canRender3DTransforms() const;
+
     void updateLayerPosition();
     
     enum UpdateLayerPositionsFlag {
@@ -303,9 +299,10 @@ public:
         CheckForRepaint = 1 << 1,
         IsCompositingUpdateRoot = 1 << 2,
         UpdateCompositingLayers = 1 << 3,
+        UpdatePagination = 1 << 4
     };
     typedef unsigned UpdateLayerPositionsFlags;
-    void updateLayerPositions(UpdateLayerPositionsFlags = DoFullRepaint | IsCompositingUpdateRoot | UpdateCompositingLayers);
+    void updateLayerPositions(UpdateLayerPositionsFlags = DoFullRepaint | IsCompositingUpdateRoot | UpdateCompositingLayers, IntPoint* cachedOffset = 0);
 
     void updateTransform();
 
@@ -394,7 +391,9 @@ public:
 
     // Return a cached repaint rect, computed relative to the layer renderer's containerForRepaint.
     IntRect repaintRect() const { return m_repaintRect; }
+    IntRect repaintRectIncludingDescendants() const;
     void computeRepaintRects();
+    void updateRepaintRectsAfterScroll(bool fixed = false);
     void setNeedsFullRepaint(bool f = true) { m_needsFullRepaint = f; }
     
     int staticX() const { return m_staticX; }
@@ -476,13 +475,36 @@ private:
     typedef unsigned PaintLayerFlags;
 
     void paintLayer(RenderLayer* rootLayer, GraphicsContext*, const IntRect& paintDirtyRect,
-                    PaintBehavior, RenderObject* paintingRoot, RenderObject::OverlapTestRequestMap* = 0,
-                    PaintLayerFlags paintFlags = 0);
+                    PaintBehavior, RenderObject* paintingRoot, OverlapTestRequestMap* = 0,
+                    PaintLayerFlags = 0);
+    void paintList(Vector<RenderLayer*>*, RenderLayer* rootLayer, GraphicsContext* p,
+                   const IntRect& paintDirtyRect, PaintBehavior,
+                   RenderObject* paintingRoot, OverlapTestRequestMap*,
+                   PaintLayerFlags);
+    void paintPaginatedChildLayer(RenderLayer* childLayer, RenderLayer* rootLayer, GraphicsContext*,
+                                  const IntRect& paintDirtyRect, PaintBehavior,
+                                  RenderObject* paintingRoot, OverlapTestRequestMap*,
+                                  PaintLayerFlags);
+    void paintChildLayerIntoColumns(RenderLayer* childLayer, RenderLayer* rootLayer, GraphicsContext*,
+                                    const IntRect& paintDirtyRect, PaintBehavior,
+                                    RenderObject* paintingRoot, OverlapTestRequestMap*,
+                                    PaintLayerFlags, const Vector<RenderLayer*>& columnLayers, size_t columnIndex);
 
     RenderLayer* hitTestLayer(RenderLayer* rootLayer, RenderLayer* containerLayer, const HitTestRequest& request, HitTestResult& result,
-                            const IntRect& hitTestRect, const IntPoint& hitTestPoint, bool appliedTransform,
-                            const HitTestingTransformState* transformState = 0, double* zOffset = 0);
-
+                              const IntRect& hitTestRect, const IntPoint& hitTestPoint, bool appliedTransform,
+                              const HitTestingTransformState* transformState = 0, double* zOffset = 0);
+    RenderLayer* hitTestList(Vector<RenderLayer*>*, RenderLayer* rootLayer, const HitTestRequest& request, HitTestResult& result,
+                             const IntRect& hitTestRect, const IntPoint& hitTestPoint,
+                             const HitTestingTransformState* transformState, double* zOffsetForDescendants, double* zOffset,
+                             const HitTestingTransformState* unflattenedTransformState, bool depthSortDescendants);
+    RenderLayer* hitTestPaginatedChildLayer(RenderLayer* childLayer, RenderLayer* rootLayer, const HitTestRequest& request, HitTestResult& result,
+                                            const IntRect& hitTestRect, const IntPoint& hitTestPoint,
+                                            const HitTestingTransformState* transformState, double* zOffset);
+    RenderLayer* hitTestChildLayerColumns(RenderLayer* childLayer, RenderLayer* rootLayer, const HitTestRequest& request, HitTestResult& result,
+                                          const IntRect& hitTestRect, const IntPoint& hitTestPoint,
+                                          const HitTestingTransformState* transformState, double* zOffset,
+                                          const Vector<RenderLayer*>& columnLayers, size_t columnIndex);
+                                    
     PassRefPtr<HitTestingTransformState> createLocalTransformState(RenderLayer* rootLayer, RenderLayer* containerLayer,
                             const IntRect& hitTestRect, const IntPoint& hitTestPoint,
                             const HitTestingTransformState* containerTransformState) const;
@@ -494,6 +516,8 @@ private:
     bool shouldBeNormalFlowOnly() const; 
 
     // ScrollBarClient interface
+    virtual int scrollSize(ScrollbarOrientation orientation) const;
+    virtual void setScrollOffsetFromAnimation(const IntPoint&);
     virtual void valueChanged(Scrollbar*);
     virtual void invalidateScrollbarRect(Scrollbar*, const IntRect&);
     virtual bool isActive() const;
@@ -539,6 +563,9 @@ private:
     void updateScrollCornerStyle();
     void updateResizerStyle();
 
+    void updatePagination();
+    bool isPaginated() const { return m_isPaginated; }
+
 #if USE(ACCELERATED_COMPOSITING)    
     bool hasCompositingDescendant() const { return m_hasCompositingDescendant; }
     void setHasCompositingDescendant(bool b)  { m_hasCompositingDescendant = b; }
@@ -555,6 +582,11 @@ private:
     // Only safe to call from RenderBoxModelObject::destroyLayer(RenderArena*)
     void destroy(RenderArena*);
 
+    int overflowTop() const;
+    int overflowBottom() const;
+    int overflowLeft() const;
+    int overflowRight() const;
+
 protected:
     RenderBoxModelObject* m_renderer;
 
@@ -582,9 +614,22 @@ protected:
     // Our scroll offsets if the view is scrolled.
     int m_scrollX;
     int m_scrollY;
-    int m_scrollOriginX;        // only non-zero for rtl content
-    int m_scrollLeftOverflow;   // only non-zero for rtl content
-
+    
+    // There are 8 possible combinations of writing mode and direction.  Scroll origin (and its corresponding left/top overflow)
+    // will be non-zero in the x or y axis if there is any reversed direction or writing-mode.  The combinations are:
+    // writing-mode / direction     scrollOrigin.x() set    scrollOrigin.y() set
+    // horizontal-tb / ltr          NO                      NO
+    // horizontal-tb / rtl          YES                     NO
+    // horizontal-bt / ltr          NO                      YES
+    // horizontal-bt / rtl          YES                     YES
+    // vertical-lr / ltr            NO                      NO
+    // vertical-lr / rtl            NO                      YES
+    // vertical-rl / ltr            YES                     NO
+    // vertical-rl / rtl            YES                     YES
+    IntPoint m_scrollOrigin;
+    int m_scrollLeftOverflow;
+    int m_scrollTopOverflow;
+    
     // The width/height of our scrolled area.
     int m_scrollWidth;
     int m_scrollHeight;
@@ -632,6 +677,8 @@ protected:
     bool m_visibleDescendantStatusDirty : 1;
     bool m_hasVisibleDescendant : 1;
 
+    bool m_isPaginated : 1; // If we think this layer is split by a multi-column ancestor, then this bit will be set.
+
     bool m_3DTransformedDescendantStatusDirty : 1;
     bool m_has3DTransformedDescendant : 1;  // Set on a stacking context layer that has 3D descendants anywhere
                                             // in a preserves3D hierarchy. Hint to do 3D-aware hit testing.
@@ -667,7 +714,8 @@ private:
 
 #ifndef NDEBUG
 // Outside the WebCore namespace for ease of invocation from gdb.
-void showLayerTree(const WebCore::RenderLayer* layer);
+void showLayerTree(const WebCore::RenderLayer*);
+void showLayerTree(const WebCore::RenderObject*);
 #endif
 
 #endif // RenderLayer_h
