@@ -33,16 +33,14 @@ class SVGElement;
 
 class RenderSVGContainer : public RenderSVGModelObject {
 public:
-    RenderSVGContainer(SVGStyledElement*);
+    explicit RenderSVGContainer(SVGStyledElement*);
+    virtual ~RenderSVGContainer();
 
     const RenderObjectChildList* children() const { return &m_children; }
     RenderObjectChildList* children() { return &m_children; }
 
-    // <marker> uses these methods to only allow drawing children during a special marker draw time
-    void setDrawsContents(bool);
-    bool drawsContents() const;
-
     virtual void paint(PaintInfo&, int parentX, int parentY);
+    virtual void setNeedsBoundariesUpdate() { m_needsBoundariesUpdate = true; }
 
 protected:
     virtual RenderObjectChildList* virtualChildren() { return children(); }
@@ -55,25 +53,29 @@ protected:
 
     virtual void addFocusRingRects(Vector<IntRect>&, int tx, int ty);
 
-    virtual FloatRect objectBoundingBox() const;
-    virtual FloatRect strokeBoundingBox() const;
-    virtual FloatRect repaintRectInLocalCoordinates() const;
+    virtual FloatRect objectBoundingBox() const { return m_objectBoundingBox; }
+    virtual FloatRect strokeBoundingBox() const { return m_strokeBoundingBox; }
+    virtual FloatRect repaintRectInLocalCoordinates() const { return m_repaintBoundingBox; }
 
     virtual bool nodeAtFloatPoint(const HitTestRequest&, HitTestResult&, const FloatPoint& pointInParent, HitTestAction);
 
     // Allow RenderSVGTransformableContainer to hook in at the right time in layout()
-    virtual void calculateLocalTransform() { }
+    virtual bool calculateLocalTransform() { return false; }
 
     // Allow RenderSVGViewportContainer to hook in at the right times in layout(), paint() and nodeAtFloatPoint()
     virtual void calcViewport() { }
     virtual void applyViewportClip(PaintInfo&) { }
     virtual bool pointIsInsideViewportClip(const FloatPoint& /*pointInParent*/) { return true; }
 
-    bool selfWillPaint() const;
+    bool selfWillPaint();
+    void updateCachedBoundaries();
 
 private:
     RenderObjectChildList m_children;
-    bool m_drawsContents : 1;
+    FloatRect m_objectBoundingBox;
+    FloatRect m_strokeBoundingBox;
+    FloatRect m_repaintBoundingBox;
+    bool m_needsBoundariesUpdate : 1;
 };
   
 inline RenderSVGContainer* toRenderSVGContainer(RenderObject* object)
@@ -97,5 +99,3 @@ void toRenderSVGContainer(const RenderSVGContainer*);
 
 #endif // ENABLE(SVG)
 #endif // RenderSVGContainer_h
-
-// vim:ts=4:noet
