@@ -24,7 +24,6 @@
 #define RenderSVGHiddenContainer_h
 
 #if ENABLE(SVG)
-
 #include "RenderSVGContainer.h"
 
 namespace WebCore {
@@ -35,29 +34,22 @@ namespace WebCore {
     // <defs>, <linearGradient>, <radialGradient> are all good examples
     class RenderSVGHiddenContainer : public RenderSVGContainer {
     public:
-        RenderSVGHiddenContainer(SVGStyledElement*);
-        virtual ~RenderSVGHiddenContainer();
-
-        virtual bool isSVGContainer() const { return true; }
-        virtual bool isSVGHiddenContainer() const { return true; }
+        explicit RenderSVGHiddenContainer(SVGStyledElement*);
 
         virtual const char* renderName() const { return "RenderSVGHiddenContainer"; }
 
+    protected:
+        virtual void layout();
+
+    private:
+        virtual bool isSVGHiddenContainer() const { return true; }
         virtual bool requiresLayer() const { return false; }
 
-        virtual void layout();
         virtual void paint(PaintInfo&, int parentX, int parentY);
         
-        virtual IntRect clippedOverflowRectForRepaint(RenderBoxModelObject* repaintContainer);
-        virtual void absoluteRects(Vector<IntRect>& rects, int tx, int ty);
+        virtual IntRect clippedOverflowRectForRepaint(RenderBoxModelObject*) { return IntRect(); }
         virtual void absoluteQuads(Vector<FloatQuad>&);
 
-        // FIXME: This override only exists to match existing LayoutTest results.
-        virtual TransformationMatrix absoluteTransform() const { return TransformationMatrix(); }
-
-        virtual FloatRect objectBoundingBox() const;
-        virtual FloatRect repaintRectInLocalCoordinates() const;
-
         virtual bool nodeAtFloatPoint(const HitTestRequest&, HitTestResult&, const FloatPoint& pointInParent, HitTestAction);
     };
 }
