@@ -3,6 +3,7 @@
     Copyright (C) 2006, 2009 Apple Inc. All rights reserved.
     Copyright (C) 2007 Rob Buis <buis@kde.org>
     Copyright (C) 2009 Google, Inc.
+    Copyright (C) 2010 Patrick Gansterer <paroga@paroga.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -26,60 +27,73 @@
 #if ENABLE(SVG)
 #include "AffineTransform.h"
 #include "FloatRect.h"
-#include "RenderImage.h"
+#include "RenderSVGModelObject.h"
 #include "SVGPreserveAspectRatio.h"
 #include "SVGRenderSupport.h"
 
 namespace WebCore {
 
+class RenderImageResource;
 class SVGImageElement;
 
-class RenderSVGImage : public RenderImage, protected SVGRenderBase {
+class RenderSVGImage : public RenderSVGModelObject {
 public:
     RenderSVGImage(SVGImageElement*);
+    virtual ~RenderSVGImage();
+
+    virtual void setNeedsTransformUpdate() { m_needsTransformUpdate = true; }
+    virtual void updateFromElement();
+
+    RenderImageResource* imageResource() { return m_imageResource.get(); }
+    const RenderImageResource* imageResource() const { return m_imageResource.get(); }
 
 private:
-    virtual const SVGRenderBase* toSVGRenderBase() const { return this; }
     virtual const char* renderName() const { return "RenderSVGImage"; }
     virtual bool isSVGImage() const { return true; }
 
     virtual const AffineTransform& localToParentTransform() const { return m_localTransform; }
 
-    virtual FloatRect objectBoundingBox() const;
-    virtual FloatRect strokeBoundingBox() const { return m_localBounds; }
-    virtual FloatRect repaintRectInLocalCoordinates() const;
+    virtual FloatRect objectBoundingBox() const { return m_objectBoundingBox; }
+    virtual FloatRect strokeBoundingBox() const { return m_objectBoundingBox; }
+    virtual FloatRect repaintRectInLocalCoordinates() const { return m_repaintBoundingBox; }
 
-    virtual IntRect clippedOverflowRectForRepaint(RenderBoxModelObject* repaintContainer);
-    virtual void computeRectForRepaint(RenderBoxModelObject* repaintContainer, IntRect&, bool fixed = false);
-
-    virtual void mapLocalToContainer(RenderBoxModelObject* repaintContainer, bool useTransforms, bool fixed, TransformState&) const;
-
-    virtual void absoluteRects(Vector<IntRect>&, int tx, int ty);
-    virtual void absoluteQuads(Vector<FloatQuad>&);
     virtual void addFocusRingRects(Vector<IntRect>&, int tx, int ty);
 
     virtual void imageChanged(WrappedImagePtr, const IntRect* = 0);
-    
+
     virtual void layout();
     virtual void paint(PaintInfo&, int parentX, int parentY);
 
-    virtual void destroy();
-
     virtual bool requiresLayer() const { return false; }
 
     virtual bool nodeAtFloatPoint(const HitTestRequest&, HitTestResult&, const FloatPoint& pointInParent, HitTestAction);
-    virtual bool nodeAtPoint(const HitTestRequest&, HitTestResult&, int x, int y, int tx, int ty, HitTestAction);
 
     virtual AffineTransform localTransform() const { return m_localTransform; }
 
+    bool m_updateCachedRepaintRect : 1;
+    bool m_needsTransformUpdate : 1;
     AffineTransform m_localTransform;
-    FloatRect m_localBounds;
-    mutable FloatRect m_cachedLocalRepaintRect;
+    FloatRect m_objectBoundingBox;
+    FloatRect m_repaintBoundingBox;
+    OwnPtr<RenderImageResource> m_imageResource;
 };
 
+inline RenderSVGImage* toRenderSVGImage(RenderObject* object)
+{
+    ASSERT(!object || object->isSVGImage());
+    return static_cast<RenderSVGImage*>(object);
+}
+
+inline const RenderSVGImage* toRenderSVGImage(const RenderObject* object)
+{
+    ASSERT(!object || object->isSVGImage());
+    return static_cast<const RenderSVGImage*>(object);
+}
+
+// This will catch anyone doing an unnecessary cast.
+void toRenderSVGImage(const RenderSVGImage*);
+
 } // namespace WebCore
 
 #endif // ENABLE(SVG)
 #endif // RenderSVGImage_h
-
-// vim:ts=4:noet
