@@ -29,6 +29,7 @@
 #include "CharacterNames.h"
 #include "EllipsisBox.h"
 #include "FloatQuad.h"
+#include "FontTranscoder.h"
 #include "FrameView.h"
 #include "InlineTextBox.h"
 #include "Range.h"
@@ -38,9 +39,11 @@
 #include "RenderView.h"
 #include "Text.h"
 #include "TextBreakIterator.h"
+#include "TextResourceDecoder.h"
 #include "VisiblePosition.h"
 #include "break_lines.h"
 #include <wtf/AlwaysInline.h>
+#include <wtf/text/StringBuffer.h>
 
 using namespace std;
 using namespace WTF;
@@ -48,10 +51,49 @@ using namespace Unicode;
 
 namespace WebCore {
 
+static void makeCapitalized(String* string, UChar previous)
+{
+    if (string->isNull())
+        return;
+
+    unsigned length = string->length();
+    const UChar* characters = string->characters();
+
+    if (length >= numeric_limits<unsigned>::max())
+        CRASH();
+
+    StringBuffer stringWithPrevious(length + 1);
+    stringWithPrevious[0] = previous == noBreakSpace ? ' ' : previous;
+    for (unsigned i = 1; i < length + 1; i++) {
+        // Replace &nbsp with a real space since ICU no longer treats &nbsp as a word separator.
+        if (characters[i - 1] == noBreakSpace)
+            stringWithPrevious[i] = ' ';
+        else
+            stringWithPrevious[i] = characters[i - 1];
+    }
+
+    TextBreakIterator* boundary = wordBreakIterator(stringWithPrevious.characters(), length + 1);
+    if (!boundary)
+        return;
+
+    StringBuffer data(length);
+
+    int32_t endOfWord;
+    int32_t startOfWord = textBreakFirst(boundary);
+    for (endOfWord = textBreakNext(boundary); endOfWord != TextBreakDone; startOfWord = endOfWord, endOfWord = textBreakNext(boundary)) {
+        if (startOfWord != 0) // Ignore first char of previous string
+            data[startOfWord - 1] = characters[startOfWord - 1] == noBreakSpace ? noBreakSpace : toTitleCase(stringWithPrevious[startOfWord]);
+        for (int i = startOfWord + 1; i < endOfWord; i++)
+            data[i - 1] = characters[i - 1];
+    }
+
+    *string = String::adopt(data);
+}
+
 RenderText::RenderText(Node* node, PassRefPtr<StringImpl> str)
      : RenderObject(node)
      , m_minWidth(-1)
-     , m_text(document()->displayStringModifiedByEncoding(str))
+     , m_text(str)
      , m_firstTextBox(0)
      , m_lastTextBox(0)
      , m_maxWidth(-1)
@@ -61,7 +103,8 @@ RenderText::RenderText(Node* node, PassRefPtr<StringImpl> str)
      , m_linesDirty(false)
      , m_containsReversedText(false)
      , m_isAllASCII(m_text.containsOnlyASCII())
-     , m_knownNotToUseFallbackFonts(false)
+     , m_knownToHaveNoOverflowAndNoFallbackFonts(false)
+     , m_needsTranscoding(false)
 {
     ASSERT(m_text);
 
@@ -98,6 +141,12 @@ bool RenderText::isWordBreak() const
     return false;
 }
 
+void RenderText::updateNeedsTranscoding()
+{
+    const TextEncoding* encoding = document()->decoder() ? &document()->decoder()->encoding() : 0;
+    m_needsTranscoding = fontTranscoder().needsTranscoding(style()->font().fontDescription(), encoding);
+}
+
 void RenderText::styleDidChange(StyleDifference diff, const RenderStyle* oldStyle)
 {
     // There is no need to ever schedule repaints from a style change of a text run, since
@@ -106,13 +155,21 @@ void RenderText::styleDidChange(StyleDifference diff, const RenderStyle* oldStyl
     // need to relayout.
     if (diff == StyleDifferenceLayout) {
         setNeedsLayoutAndPrefWidthsRecalc();
-        m_knownNotToUseFallbackFonts = false;
+        m_knownToHaveNoOverflowAndNoFallbackFonts = false;
+    }
+
+    bool needsResetText = false;
+    if (!oldStyle) {
+        updateNeedsTranscoding();
+        needsResetText = m_needsTranscoding;
+    } else if (oldStyle->font().needsTranscoding() != style()->font().needsTranscoding() || (style()->font().needsTranscoding() && oldStyle->font().family().family() != style()->font().family().family())) {
+        updateNeedsTranscoding();
+        needsResetText = true;
     }
 
     ETextTransform oldTransform = oldStyle ? oldStyle->textTransform() : TTNONE;
     ETextSecurity oldSecurity = oldStyle ? oldStyle->textSecurity() : TSNONE;
-
-    if (oldTransform != style()->textTransform() || oldSecurity != style()->textSecurity()) {
+    if (needsResetText || oldTransform != style()->textTransform() || oldSecurity != style()->textSecurity()) {
         if (RefPtr<StringImpl> textToTransform = originalText())
             setText(textToTransform.release(), true);
     }
@@ -209,7 +266,7 @@ PassRefPtr<StringImpl> RenderText::originalText() const
 void RenderText::absoluteRects(Vector<IntRect>& rects, int tx, int ty)
 {
     for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox())
-        rects.append(IntRect(tx + box->x(), ty + box->y(), box->width(), box->height()));
+        rects.append(IntRect(tx + box->x(), ty + box->y(), box->logicalWidth(), box->logicalHeight()));
 }
 
 void RenderText::absoluteRectsForRange(Vector<IntRect>& rects, unsigned start, unsigned end, bool useSelectionHeight)
@@ -227,7 +284,7 @@ void RenderText::absoluteRectsForRange(Vector<IntRect>& rects, unsigned start, u
     for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
         // Note: box->end() returns the index of the last character, not the index past it
         if (start <= box->start() && box->end() < end) {
-            IntRect r = IntRect(box->x(), box->y(), box->width(), box->height());
+            IntRect r = IntRect(box->x(), box->y(), box->logicalWidth(), box->logicalHeight());
             if (useSelectionHeight) {
                 IntRect selectionRect = box->selectionRect(0, 0, start, end);
                 r.setHeight(selectionRect.height());
@@ -243,7 +300,7 @@ void RenderText::absoluteRectsForRange(Vector<IntRect>& rects, unsigned start, u
             if (!r.isEmpty()) {
                 if (!useSelectionHeight) {
                     // change the height and y position because selectionRect uses selection-specific values
-                    r.setHeight(box->height());
+                    r.setHeight(box->logicalHeight());
                     r.setY(box->y());
                 }
                 FloatPoint origin = localToAbsolute(r.location());
@@ -256,10 +313,46 @@ void RenderText::absoluteRectsForRange(Vector<IntRect>& rects, unsigned start, u
     }
 }
 
+static IntRect ellipsisRectForBox(InlineTextBox* box, unsigned startPos, unsigned endPos)
+{
+    if (!box)
+        return IntRect();
+    
+    unsigned short truncation = box->truncation();
+    if (truncation == cNoTruncation)
+        return IntRect();
+    
+    IntRect rect;
+    if (EllipsisBox* ellipsis = box->root()->ellipsisBox()) {
+        int ellipsisStartPosition = max<int>(startPos - box->start(), 0);
+        int ellipsisEndPosition = min<int>(endPos - box->start(), box->len());
+        
+        // The ellipsis should be considered to be selected if the end of
+        // the selection is past the beginning of the truncation and the
+        // beginning of the selection is before or at the beginning of the truncation.
+        if (ellipsisEndPosition >= truncation && ellipsisStartPosition <= truncation)
+            return ellipsis->selectionRect(0, 0);
+    }
+    
+    return IntRect();
+}
+    
+void RenderText::absoluteQuads(Vector<FloatQuad>& quads, ClippingOption option)
+{
+    for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
+        IntRect boundaries = box->calculateBoundaries();
+
+        // Shorten the width of this text box if it ends in an ellipsis.
+        IntRect ellipsisRect = (option == ClipToEllipsis) ? ellipsisRectForBox(box, 0, textLength()) : IntRect();
+        if (!ellipsisRect.isEmpty())
+            boundaries.setWidth(ellipsisRect.right() - boundaries.x());
+        quads.append(localToAbsoluteQuad(FloatRect(boundaries)));
+    }
+}
+    
 void RenderText::absoluteQuads(Vector<FloatQuad>& quads)
 {
-    for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox())
-        quads.append(localToAbsoluteQuad(FloatRect(box->x(), box->y(), box->width(), box->height())));
+    absoluteQuads(quads, NoClipping);
 }
 
 void RenderText::absoluteQuadsForRange(Vector<FloatQuad>& quads, unsigned start, unsigned end, bool useSelectionHeight)
@@ -277,7 +370,7 @@ void RenderText::absoluteQuadsForRange(Vector<FloatQuad>& quads, unsigned start,
     for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
         // Note: box->end() returns the index of the last character, not the index past it
         if (start <= box->start() && box->end() < end) {
-            IntRect r = IntRect(box->x(), box->y(), box->width(), box->height());
+            IntRect r(box->calculateBoundaries());
             if (useSelectionHeight) {
                 IntRect selectionRect = box->selectionRect(0, 0, start, end);
                 r.setHeight(selectionRect.height());
@@ -290,7 +383,7 @@ void RenderText::absoluteQuadsForRange(Vector<FloatQuad>& quads, unsigned start,
             if (r.height()) {
                 if (!useSelectionHeight) {
                     // change the height and y position because selectionRect uses selection-specific values
-                    r.setHeight(box->height());
+                    r.setHeight(box->logicalHeight());
                     r.setY(box->y());
                 }
                 quads.append(localToAbsoluteQuad(FloatRect(r)));
@@ -328,38 +421,44 @@ VisiblePosition RenderText::positionForPoint(const IntPoint& point)
     // Get the offset for the position, since this will take rtl text into account.
     int offset;
 
+    int pointLineDirection = firstTextBox()->isHorizontal() ? point.x() : point.y();
+    int pointBlockDirection = firstTextBox()->isHorizontal() ? point.y() : point.x();
+    
     // FIXME: We should be able to roll these special cases into the general cases in the loop below.
-    if (firstTextBox() && point.y() <  firstTextBox()->root()->lineBottom() && point.x() < firstTextBox()->m_x) {
+    if (firstTextBox() && pointBlockDirection <  firstTextBox()->root()->selectionBottom() && pointLineDirection < firstTextBox()->logicalLeft()) {
         // at the y coordinate of the first line or above
         // and the x coordinate is to the left of the first text box left edge
-        offset = firstTextBox()->offsetForPosition(point.x());
-        return createVisiblePosition(offset + firstTextBox()->start(), DOWNSTREAM);
+        offset = firstTextBox()->offsetForPosition(pointLineDirection);
+        return createVisiblePosition(offset + firstTextBox()->start(), offset > 0 ? VP_UPSTREAM_IF_POSSIBLE : DOWNSTREAM);
     }
-    if (lastTextBox() && point.y() >= lastTextBox()->root()->lineTop() && point.x() >= lastTextBox()->m_x + lastTextBox()->m_width) {
+    if (lastTextBox() && pointBlockDirection >= lastTextBox()->root()->selectionTop() && pointLineDirection >= lastTextBox()->logicalRight()) {
         // at the y coordinate of the last line or below
         // and the x coordinate is to the right of the last text box right edge
-        offset = lastTextBox()->offsetForPosition(point.x());
+        offset = lastTextBox()->offsetForPosition(pointLineDirection);
         return createVisiblePosition(offset + lastTextBox()->start(), VP_UPSTREAM_IF_POSSIBLE);
     }
 
     InlineTextBox* lastBoxAbove = 0;
     for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
-        if (point.y() >= box->root()->lineTop()) {
-            int bottom = box->root()->nextRootBox() ? box->root()->nextRootBox()->lineTop() : box->root()->lineBottom();
-            if (point.y() < bottom) {
-                offset = box->offsetForPosition(point.x());
+        RootInlineBox* rootBox = box->root();
+        if (pointBlockDirection >= rootBox->selectionTop()) {
+            int bottom = rootBox->selectionBottom();
+            if (rootBox->nextRootBox())
+                bottom = min(bottom, rootBox->nextRootBox()->lineTop());
+            if (pointBlockDirection < bottom) {
+                offset = box->offsetForPosition(pointLineDirection);
 
-                if (point.x() == box->m_x)
+                if (pointLineDirection == box->logicalLeft())
                     // the x coordinate is equal to the left edge of this box
                     // the affinity must be downstream so the position doesn't jump back to the previous line
                     return createVisiblePosition(offset + box->start(), DOWNSTREAM);
 
-                if (point.x() < box->m_x + box->m_width)
+                if (pointLineDirection < box->logicalRight())
                     // and the x coordinate is to the left of the right edge of this box
                     // check to see if position goes in this box
                     return createVisiblePosition(offset + box->start(), offset > 0 ? VP_UPSTREAM_IF_POSSIBLE : DOWNSTREAM);
 
-                if (!box->prevOnLine() && point.x() < box->m_x)
+                if (!box->prevOnLine() && pointLineDirection < box->logicalLeft())
                     // box is first on line
                     // and the x coordinate is to the left of the first text box left edge
                     return createVisiblePosition(offset + box->start(), DOWNSTREAM);
@@ -388,8 +487,8 @@ IntRect RenderText::localCaretRect(InlineBox* inlineBox, int caretOffset, int* e
 
     InlineTextBox* box = static_cast<InlineTextBox*>(inlineBox);
 
-    int height = box->root()->lineBottom() - box->root()->lineTop();
-    int top = box->root()->lineTop();
+    int height = box->root()->selectionHeight();
+    int top = box->root()->selectionTop();
 
     int left = box->positionForOffset(caretOffset);
 
@@ -398,37 +497,54 @@ IntRect RenderText::localCaretRect(InlineBox* inlineBox, int caretOffset, int* e
     left -= caretWidthLeftOfOffset;
     int caretWidthRightOfOffset = caretWidth - caretWidthLeftOfOffset;
 
-    int rootLeft = box->root()->x();
-    int rootRight = rootLeft + box->root()->width();
+    int rootLeft = box->root()->logicalLeft();
+    int rootRight = rootLeft + box->root()->logicalWidth();
     // FIXME: should we use the width of the root inline box or the
     // width of the containing block for this?
     if (extraWidthToEndOfLine)
-        *extraWidthToEndOfLine = (box->root()->width() + rootLeft) - (left + 1);
+        *extraWidthToEndOfLine = (box->root()->logicalWidth() + rootLeft) - (left + 1);
 
     RenderBlock* cb = containingBlock();
+    RenderStyle* cbStyle = cb->style();
+    int leftEdge;
+    int rightEdge;
     if (style()->autoWrap()) {
-        int availableWidth = cb->lineWidth(top, false);
-        if (box->direction() == LTR)
-            left = min(left, rootLeft + availableWidth - caretWidthRightOfOffset);
-        else
-            left = max(left, cb->x());
+        leftEdge = cb->logicalLeft();
+        rightEdge = cb->logicalRight();
     } else {
-        // If there is no wrapping, the caret can leave its containing block, but not its root line box.
-        if (cb->style()->direction() == LTR) {
-            int rightEdge = max(cb->width(), rootRight);
-            left = min(left, rightEdge - caretWidthRightOfOffset);
-            left = max(left, rootLeft);
-        } else {
-            int leftEdge = min(cb->x(), rootLeft);
-            left = max(left, leftEdge);
-            left = min(left, rootRight - caretWidth);
-        }
+        leftEdge = min(cb->logicalLeft(), rootLeft);
+        rightEdge = max(cb->logicalRight(), rootRight);
     }
 
-    return IntRect(left, top, caretWidth, height);
+    bool rightAligned = false;
+    switch (cbStyle->textAlign()) {
+    case TAAUTO:
+    case JUSTIFY:
+        rightAligned = !cbStyle->isLeftToRightDirection();
+        break;
+    case RIGHT:
+    case WEBKIT_RIGHT:
+        rightAligned = true;
+        break;
+    case LEFT:
+    case WEBKIT_LEFT:
+    case CENTER:
+    case WEBKIT_CENTER:
+        break;
+    }
+
+    if (rightAligned) {
+        left = max(left, leftEdge);
+        left = min(left, rootRight - caretWidth);
+    } else {
+        left = min(left, rightEdge - caretWidthRightOfOffset);
+        left = max(left, rootLeft);
+    }
+
+    return style()->isHorizontalWritingMode() ? IntRect(left, top, caretWidth, height) : IntRect(top, left, height, caretWidth);
 }
 
-ALWAYS_INLINE int RenderText::widthFromCache(const Font& f, int start, int len, int xPos, HashSet<const SimpleFontData*>* fallbackFonts) const
+ALWAYS_INLINE int RenderText::widthFromCache(const Font& f, int start, int len, int xPos, HashSet<const SimpleFontData*>* fallbackFonts, GlyphOverflow* glyphOverflow) const
 {
     if (f.isFixedPitch() && !f.isSmallCaps() && m_isAllASCII) {
         int monospaceCharacterWidth = f.spaceWidth();
@@ -460,7 +576,7 @@ ALWAYS_INLINE int RenderText::widthFromCache(const Font& f, int start, int len,
         return w;
     }
 
-    return f.width(TextRun(text()->characters() + start, len, allowTabs(), xPos), fallbackFonts);
+    return f.width(TextRun(text()->characters() + start, len, allowTabs(), xPos), fallbackFonts, glyphOverflow);
 }
 
 void RenderText::trimmedPrefWidths(int leadWidth,
@@ -474,8 +590,8 @@ void RenderText::trimmedPrefWidths(int leadWidth,
     if (!collapseWhiteSpace)
         stripFrontSpaces = false;
 
-    if (m_hasTab || prefWidthsDirty())
-        calcPrefWidths(leadWidth);
+    if (m_hasTab || preferredLogicalWidthsDirty())
+        computePreferredLogicalWidths(leadWidth);
 
     beginWS = !stripFrontSpaces && m_hasBeginWS;
     endWS = m_hasEndWS;
@@ -531,7 +647,7 @@ void RenderText::trimmedPrefWidths(int leadWidth,
                 linelen++;
 
             if (linelen) {
-                endMaxW = widthFromCache(f, i, linelen, leadWidth + endMaxW, 0);
+                endMaxW = widthFromCache(f, i, linelen, leadWidth + endMaxW, 0, 0);
                 if (firstLine) {
                     firstLine = false;
                     leadWidth = 0;
@@ -557,33 +673,34 @@ static inline bool isSpaceAccordingToStyle(UChar c, RenderStyle* style)
     return c == ' ' || (c == noBreakSpace && style->nbspMode() == SPACE);
 }
 
-int RenderText::minPrefWidth() const
+int RenderText::minPreferredLogicalWidth() const
 {
-    if (prefWidthsDirty())
-        const_cast<RenderText*>(this)->calcPrefWidths(0);
+    if (preferredLogicalWidthsDirty())
+        const_cast<RenderText*>(this)->computePreferredLogicalWidths(0);
         
     return m_minWidth;
 }
 
-int RenderText::maxPrefWidth() const
+int RenderText::maxPreferredLogicalWidth() const
 {
-    if (prefWidthsDirty())
-        const_cast<RenderText*>(this)->calcPrefWidths(0);
+    if (preferredLogicalWidthsDirty())
+        const_cast<RenderText*>(this)->computePreferredLogicalWidths(0);
         
     return m_maxWidth;
 }
 
-void RenderText::calcPrefWidths(int leadWidth)
+void RenderText::computePreferredLogicalWidths(int leadWidth)
 {
     HashSet<const SimpleFontData*> fallbackFonts;
-    calcPrefWidths(leadWidth, fallbackFonts);
-    if (fallbackFonts.isEmpty())
-        m_knownNotToUseFallbackFonts = true;
+    GlyphOverflow glyphOverflow;
+    computePreferredLogicalWidths(leadWidth, fallbackFonts, glyphOverflow);
+    if (fallbackFonts.isEmpty() && !glyphOverflow.left && !glyphOverflow.right && !glyphOverflow.top && !glyphOverflow.bottom)
+        m_knownToHaveNoOverflowAndNoFallbackFonts = true;
 }
 
-void RenderText::calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& fallbackFonts)
+void RenderText::computePreferredLogicalWidths(int leadWidth, HashSet<const SimpleFontData*>& fallbackFonts, GlyphOverflow& glyphOverflow)
 {
-    ASSERT(m_hasTab || prefWidthsDirty() || !m_knownNotToUseFallbackFonts);
+    ASSERT(m_hasTab || preferredLogicalWidthsDirty() || !m_knownToHaveNoOverflowAndNoFallbackFonts);
 
     m_minWidth = 0;
     m_beginMinWidth = 0;
@@ -613,6 +730,8 @@ void RenderText::calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& f
     int nextBreakable = -1;
     int lastWordBoundary = 0;
 
+    int firstGlyphLeftOverflow = -1;
+
     bool breakNBSP = style()->autoWrap() && style()->nbspMode() == SPACE;
     bool breakAll = (style()->wordBreak() == BreakAllWordBreak || style()->wordBreak() == BreakWordBreak) && style()->autoWrap();
 
@@ -655,7 +774,9 @@ void RenderText::calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& f
             lastWordBoundary++;
             continue;
         } else if (c == softHyphen) {
-            currMaxWidth += widthFromCache(f, lastWordBoundary, i - lastWordBoundary, leadWidth + currMaxWidth, &fallbackFonts);
+            currMaxWidth += widthFromCache(f, lastWordBoundary, i - lastWordBoundary, leadWidth + currMaxWidth, &fallbackFonts, &glyphOverflow);
+            if (firstGlyphLeftOverflow < 0)
+                firstGlyphLeftOverflow = glyphOverflow.left;
             lastWordBoundary = i + 1;
             continue;
         }
@@ -678,13 +799,15 @@ void RenderText::calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& f
 
         int wordLen = j - i;
         if (wordLen) {
-            int w = widthFromCache(f, i, wordLen, leadWidth + currMaxWidth, &fallbackFonts);
+            int w = widthFromCache(f, i, wordLen, leadWidth + currMaxWidth, &fallbackFonts, &glyphOverflow);
+            if (firstGlyphLeftOverflow < 0)
+                firstGlyphLeftOverflow = glyphOverflow.left;
             currMinWidth += w;
             if (betweenWords) {
                 if (lastWordBoundary == i)
                     currMaxWidth += w;
                 else
-                    currMaxWidth += widthFromCache(f, lastWordBoundary, j - lastWordBoundary, leadWidth + currMaxWidth, &fallbackFonts);
+                    currMaxWidth += widthFromCache(f, lastWordBoundary, j - lastWordBoundary, leadWidth + currMaxWidth, &fallbackFonts, &glyphOverflow);
                 lastWordBoundary = j;
             }
 
@@ -737,6 +860,7 @@ void RenderText::calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& f
                 currMaxWidth = 0;
             } else {
                 currMaxWidth += f.width(TextRun(txt + i, 1, allowTabs(), leadWidth + currMaxWidth));
+                glyphOverflow.right = 0;
                 needsWordSpacing = isSpace && !previousCharacterIsSpace && i == len - 1;
             }
             ASSERT(lastWordBoundary == i);
@@ -744,6 +868,9 @@ void RenderText::calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& f
         }
     }
 
+    if (firstGlyphLeftOverflow > 0)
+        glyphOverflow.left = firstGlyphLeftOverflow;
+
     if ((needsWordSpacing && len > 1) || (ignoringSpaces && !firstWord))
         currMaxWidth += wordSpacing;
 
@@ -759,7 +886,7 @@ void RenderText::calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& f
         m_endMinWidth = currMaxWidth;
     }
 
-    setPrefWidthsDirty(false);
+    setPreferredLogicalWidthsDirty(false);
 }
 
 bool RenderText::isAllCollapsibleWhitespace()
@@ -912,7 +1039,7 @@ void RenderText::setTextWithOffset(PassRefPtr<StringImpl> text, unsigned offset,
     setText(text, force);
 }
 
-static inline bool isInlineFlowOrEmptyText(RenderObject* o)
+static inline bool isInlineFlowOrEmptyText(const RenderObject* o)
 {
     if (o->isRenderInline())
         return true;
@@ -924,10 +1051,10 @@ static inline bool isInlineFlowOrEmptyText(RenderObject* o)
     return !text->length();
 }
 
-UChar RenderText::previousCharacter()
+UChar RenderText::previousCharacter() const
 {
     // find previous text renderer if one exists
-    RenderObject* previousText = this;
+    const RenderObject* previousText = this;
     while ((previousText = previousText->previousInPreOrder()))
         if (!isInlineFlowOrEmptyText(previousText))
             break;
@@ -938,69 +1065,50 @@ UChar RenderText::previousCharacter()
     return prev;
 }
 
+void RenderText::transformText(String& text) const
+{
+    ASSERT(style());
+    switch (style()->textTransform()) {
+    case TTNONE:
+        break;
+    case CAPITALIZE:
+        makeCapitalized(&text, previousCharacter());
+        break;
+    case UPPERCASE:
+        text.makeUpper();
+        break;
+    case LOWERCASE:
+        text.makeLower();
+        break;
+    }
+}
+
 void RenderText::setTextInternal(PassRefPtr<StringImpl> text)
 {
     ASSERT(text);
-    m_text = document()->displayStringModifiedByEncoding(text);
+    m_text = text;
+    if (m_needsTranscoding) {
+        const TextEncoding* encoding = document()->decoder() ? &document()->decoder()->encoding() : 0;
+        fontTranscoder().convert(m_text, style()->font().fontDescription(), encoding);
+    }
     ASSERT(m_text);
 
-#if ENABLE(SVG)
-    if (isSVGText()) {
-        if (style() && style()->whiteSpace() == PRE) {
-            // Spec: When xml:space="preserve", the SVG user agent will do the following using a
-            // copy of the original character data content. It will convert all newline and tab
-            // characters into space characters. Then, it will draw all space characters, including
-            // leading, trailing and multiple contiguous space characters.
-
-            m_text.replace('\n', ' ');
-
-            // If xml:space="preserve" is set, white-space is set to "pre", which
-            // preserves leading, trailing & contiguous space character for us.
-       } else {
-            // Spec: When xml:space="default", the SVG user agent will do the following using a
-            // copy of the original character data content. First, it will remove all newline
-            // characters. Then it will convert all tab characters into space characters.
-            // Then, it will strip off all leading and trailing space characters.
-            // Then, all contiguous space characters will be consolidated.    
-
-           m_text.replace('\n', StringImpl::empty());
-
-           // If xml:space="default" is set, white-space is set to "nowrap", which handles
-           // leading, trailing & contiguous space character removal for us.
-        }
-
-        m_text.replace('\t', ' ');
-    }
-#endif
-
     if (style()) {
-        switch (style()->textTransform()) {
-            case TTNONE:
-                break;
-            case CAPITALIZE:
-                m_text.makeCapitalized(previousCharacter());
-                break;
-            case UPPERCASE:
-                m_text.makeUpper();
-                break;
-            case LOWERCASE:
-                m_text.makeLower();
-                break;
-        }
+        transformText(m_text);
 
         // We use the same characters here as for list markers.
         // See the listMarkerText function in RenderListMarker.cpp.
         switch (style()->textSecurity()) {
-            case TSNONE:
-                break;
-            case TSCIRCLE:
-                m_text.makeSecure(whiteBullet);
-                break;
-            case TSDISC:
-                m_text.makeSecure(bullet);
-                break;
-            case TSSQUARE:
-                m_text.makeSecure(blackSquare);
+        case TSNONE:
+            break;
+        case TSCIRCLE:
+            m_text.makeSecure(whiteBullet);
+            break;
+        case TSDISC:
+            m_text.makeSecure(bullet);
+            break;
+        case TSSQUARE:
+            m_text.makeSecure(blackSquare);
         }
     }
 
@@ -1019,17 +1127,25 @@ void RenderText::setText(PassRefPtr<StringImpl> text, bool force)
 
     setTextInternal(text);
     setNeedsLayoutAndPrefWidthsRecalc();
-    m_knownNotToUseFallbackFonts = false;
+    m_knownToHaveNoOverflowAndNoFallbackFonts = false;
     
     AXObjectCache* axObjectCache = document()->axObjectCache();
     if (axObjectCache->accessibilityEnabled())
         axObjectCache->contentChanged(this);
 }
 
-int RenderText::lineHeight(bool firstLine, bool) const
+String RenderText::textWithoutTranscoding() const
 {
-    // Always use the interior line height of the parent (e.g., if our parent is an inline block).
-    return parent()->lineHeight(firstLine, true);
+    // If m_text isn't transcoded or is secure, we can just return the modified text.
+    if (!m_needsTranscoding || style()->textSecurity() != TSNONE)
+        return text();
+
+    // Otherwise, we should use original text. If text-transform is
+    // specified, we should transform the text on the fly.
+    String text = originalText();
+    if (style())
+        transformText(text);
+    return text;
 }
 
 void RenderText::dirtyLineBoxes(bool fullLayout)
@@ -1082,10 +1198,10 @@ void RenderText::positionLineBox(InlineBox* box)
         return;
     }
 
-    m_containsReversedText |= s->direction() == RTL;
+    m_containsReversedText |= !s->isLeftToRightDirection();
 }
 
-unsigned RenderText::width(unsigned from, unsigned len, int xPos, bool firstLine, HashSet<const SimpleFontData*>* fallbackFonts) const
+unsigned RenderText::width(unsigned from, unsigned len, int xPos, bool firstLine, HashSet<const SimpleFontData*>* fallbackFonts, GlyphOverflow* glyphOverflow) const
 {
     if (from >= textLength())
         return 0;
@@ -1093,10 +1209,10 @@ unsigned RenderText::width(unsigned from, unsigned len, int xPos, bool firstLine
     if (from + len > textLength())
         len = textLength() - from;
 
-    return width(from, len, style(firstLine)->font(), xPos, fallbackFonts);
+    return width(from, len, style(firstLine)->font(), xPos, fallbackFonts, glyphOverflow);
 }
 
-unsigned RenderText::width(unsigned from, unsigned len, const Font& f, int xPos, HashSet<const SimpleFontData*>* fallbackFonts) const
+unsigned RenderText::width(unsigned from, unsigned len, const Font& f, int xPos, HashSet<const SimpleFontData*>* fallbackFonts, GlyphOverflow* glyphOverflow) const
 {
     ASSERT(from + len <= textLength());
     if (!characters())
@@ -1106,18 +1222,19 @@ unsigned RenderText::width(unsigned from, unsigned len, const Font& f, int xPos,
     if (&f == &style()->font()) {
         if (!style()->preserveNewline() && !from && len == textLength()) {
             if (fallbackFonts) {
-                if (prefWidthsDirty() || !m_knownNotToUseFallbackFonts) {
-                    const_cast<RenderText*>(this)->calcPrefWidths(0, *fallbackFonts);
-                    if (fallbackFonts->isEmpty())
-                        m_knownNotToUseFallbackFonts = true;
+                ASSERT(glyphOverflow);
+                if (preferredLogicalWidthsDirty() || !m_knownToHaveNoOverflowAndNoFallbackFonts) {
+                    const_cast<RenderText*>(this)->computePreferredLogicalWidths(0, *fallbackFonts, *glyphOverflow);
+                    if (fallbackFonts->isEmpty() && !glyphOverflow->left && !glyphOverflow->right && !glyphOverflow->top && !glyphOverflow->bottom)
+                        m_knownToHaveNoOverflowAndNoFallbackFonts = true;
                 }
                 w = m_maxWidth;
             } else
-                w = maxPrefWidth();
+                w = maxPreferredLogicalWidth();
         } else
-            w = widthFromCache(f, from, len, xPos, fallbackFonts);
+            w = widthFromCache(f, from, len, xPos, fallbackFonts, glyphOverflow);
     } else
-        w = f.width(TextRun(text()->characters() + from, len, allowTabs(), xPos), fallbackFonts);
+        w = f.width(TextRun(text()->characters() + from, len, allowTabs(), xPos), fallbackFonts, glyphOverflow);
 
     return w;
 }
@@ -1129,18 +1246,22 @@ IntRect RenderText::linesBoundingBox() const
     ASSERT(!firstTextBox() == !lastTextBox());  // Either both are null or both exist.
     if (firstTextBox() && lastTextBox()) {
         // Return the width of the minimal left side and the maximal right side.
-        int leftSide = 0;
-        int rightSide = 0;
+        int logicalLeftSide = 0;
+        int logicalRightSide = 0;
         for (InlineTextBox* curr = firstTextBox(); curr; curr = curr->nextTextBox()) {
-            if (curr == firstTextBox() || curr->x() < leftSide)
-                leftSide = curr->x();
-            if (curr == firstTextBox() || curr->x() + curr->width() > rightSide)
-                rightSide = curr->x() + curr->width();
+            if (curr == firstTextBox() || curr->logicalLeft() < logicalLeftSide)
+                logicalLeftSide = curr->logicalLeft();
+            if (curr == firstTextBox() || curr->logicalRight() > logicalRightSide)
+                logicalRightSide = curr->logicalRight();
         }
-        result.setWidth(rightSide - leftSide);
-        result.setX(leftSide);
-        result.setHeight(lastTextBox()->y() + lastTextBox()->height() - firstTextBox()->y());
-        result.setY(firstTextBox()->y());
+        
+        bool isHorizontal = style()->isHorizontalWritingMode();
+        
+        int x = isHorizontal ? logicalLeftSide : firstTextBox()->x();
+        int y = isHorizontal ? firstTextBox()->y() : logicalLeftSide;
+        int width = isHorizontal ? logicalRightSide - logicalLeftSide : lastTextBox()->logicalBottom() - x;
+        int height = isHorizontal ? lastTextBox()->logicalBottom() - y : logicalRightSide - logicalLeftSide;
+        result = IntRect(x, y, width, height);
     }
 
     return result;
@@ -1149,6 +1270,11 @@ IntRect RenderText::linesBoundingBox() const
 IntRect RenderText::clippedOverflowRectForRepaint(RenderBoxModelObject* repaintContainer)
 {
     RenderObject* cb = containingBlock();
+    // The containing block may be an ancestor of repaintContainer, but we need to do a repaintContainer-relative repaint.
+    if (repaintContainer && repaintContainer != cb) {
+        if (!cb->isDescendantOf(repaintContainer))
+            return repaintContainer->clippedOverflowRectForRepaint(repaintContainer);
+    }
     return cb->clippedOverflowRectForRepaint(repaintContainer);
 }
 
@@ -1158,7 +1284,7 @@ IntRect RenderText::selectionRectForRepaint(RenderBoxModelObject* repaintContain
 
     if (selectionState() == SelectionNone)
         return IntRect();
-    RenderBlock* cb =  containingBlock();
+    RenderBlock* cb = containingBlock();
     if (!cb)
         return IntRect();
 
@@ -1183,21 +1309,7 @@ IntRect RenderText::selectionRectForRepaint(RenderBoxModelObject* repaintContain
     IntRect rect;
     for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
         rect.unite(box->selectionRect(0, 0, startPos, endPos));
-
-        // Check if there are ellipsis which fall within the selection.
-        unsigned short truncation = box->truncation();
-        if (truncation != cNoTruncation) {
-            if (EllipsisBox* ellipsis = box->root()->ellipsisBox()) {
-                int ePos = min<int>(endPos - box->start(), box->len());
-                int sPos = max<int>(startPos - box->start(), 0);
-                // The ellipsis should be considered to be selected if the end of
-                // the selection is past the beginning of the truncation and the
-                // beginning of the selection is before or at the beginning of the
-                // truncation.
-                if (ePos >= truncation && sPos <= truncation)
-                    rect.unite(ellipsis->selectionRect(0, 0));
-            }
-        }
+        rect.unite(ellipsisRectForBox(box, startPos, endPos));
     }
 
     if (clipToVisibleContent)
