@@ -24,11 +24,11 @@
 #define RenderText_h
 
 #include "RenderObject.h"
+#include <wtf/Forward.h>
 
 namespace WebCore {
 
 class InlineTextBox;
-class StringImpl;
 
 class RenderText : public RenderObject {
 public:
@@ -51,6 +51,7 @@ public:
     virtual void destroy();
 
     StringImpl* text() const { return m_text.impl(); }
+    String textWithoutTranscoding() const;
 
     InlineTextBox* createInlineTextBox();
     void dirtyLineBoxes(bool fullLayout);
@@ -61,19 +62,20 @@ public:
     virtual void absoluteQuads(Vector<FloatQuad>&);
     void absoluteQuadsForRange(Vector<FloatQuad>&, unsigned startOffset = 0, unsigned endOffset = UINT_MAX, bool useSelectionHeight = false);
 
+    enum ClippingOption { NoClipping, ClipToEllipsis };
+    void absoluteQuads(Vector<FloatQuad>&, ClippingOption option = NoClipping);
+
     virtual VisiblePosition positionForPoint(const IntPoint&);
 
     const UChar* characters() const { return m_text.characters(); }
     unsigned textLength() const { return m_text.length(); } // non virtual implementation of length()
     void positionLineBox(InlineBox*);
 
-    virtual unsigned width(unsigned from, unsigned len, const Font&, int xPos, HashSet<const SimpleFontData*>* fallbackFonts = 0) const;
-    virtual unsigned width(unsigned from, unsigned len, int xPos, bool firstLine = false, HashSet<const SimpleFontData*>* fallbackFonts = 0) const;
+    virtual unsigned width(unsigned from, unsigned len, const Font&, int xPos, HashSet<const SimpleFontData*>* fallbackFonts = 0, GlyphOverflow* = 0) const;
+    virtual unsigned width(unsigned from, unsigned len, int xPos, bool firstLine = false, HashSet<const SimpleFontData*>* fallbackFonts = 0, GlyphOverflow* = 0) const;
 
-    virtual int lineHeight(bool firstLine, bool isRootLineBox = false) const;
-
-    virtual int minPrefWidth() const;
-    virtual int maxPrefWidth() const;
+    virtual int minPreferredLogicalWidth() const;
+    virtual int maxPreferredLogicalWidth() const;
 
     void trimmedPrefWidths(int leadWidth,
                            int& beginMinW, bool& beginWS,
@@ -82,7 +84,7 @@ public:
                            int& beginMaxW, int& endMaxW,
                            int& minW, int& maxW, bool& stripFrontSpaces);
 
-    IntRect linesBoundingBox() const;
+    virtual IntRect linesBoundingBox() const;
 
     IntPoint firstRunOrigin() const;
     int firstRunX() const;
@@ -120,7 +122,7 @@ public:
 
     void checkConsistency() const;
 
-    virtual void calcPrefWidths(int leadWidth);
+    virtual void computePreferredLogicalWidths(int leadWidth);
     bool isAllCollapsibleWhitespace();
     
 protected:
@@ -128,12 +130,12 @@ protected:
     virtual void styleDidChange(StyleDifference, const RenderStyle* oldStyle);
 
     virtual void setTextInternal(PassRefPtr<StringImpl>);
-    virtual UChar previousCharacter();
+    virtual UChar previousCharacter() const;
     
     virtual InlineTextBox* createTextBox(); // Subclassed by SVG.
 
 private:
-    void calcPrefWidths(int leadWidth, HashSet<const SimpleFontData*>& fallbackFonts);
+    void computePreferredLogicalWidths(int leadWidth, HashSet<const SimpleFontData*>& fallbackFonts, GlyphOverflow&);
 
     // Make length() private so that callers that have a RenderText*
     // will use the more efficient textLength() instead, while
@@ -146,8 +148,11 @@ private:
 
     void deleteTextBoxes();
     bool containsOnlyWhitespace(unsigned from, unsigned len) const;
-    int widthFromCache(const Font&, int start, int len, int xPos, HashSet<const SimpleFontData*>* fallbackFonts) const;
+    int widthFromCache(const Font&, int start, int len, int xPos, HashSet<const SimpleFontData*>* fallbackFonts, GlyphOverflow*) const;
     bool isAllASCII() const { return m_isAllASCII; }
+    void updateNeedsTranscoding();
+
+    inline void transformText(String&) const;
 
     int m_minWidth; // here to minimize padding in 64-bit.
 
@@ -171,7 +176,8 @@ private:
                            // or removed).
     bool m_containsReversedText : 1;
     bool m_isAllASCII : 1;
-    mutable bool m_knownNotToUseFallbackFonts : 1;
+    mutable bool m_knownToHaveNoOverflowAndNoFallbackFonts : 1;
+    bool m_needsTranscoding : 1;
 };
 
 inline RenderText* toRenderText(RenderObject* object)
