@@ -29,6 +29,8 @@
 #include "EventNames.h"
 #include "Frame.h"
 #include "HTMLBRElement.h"
+#include "HTMLFormControlElement.h"
+#include "HTMLInputElement.h"
 #include "HTMLNames.h"
 #include "HitTestResult.h"
 #include "RenderLayer.h"
@@ -134,7 +136,7 @@ void RenderTextControl::adjustInnerTextStyle(const RenderStyle* startStyle, Rend
 
     bool disabled = updateUserModifyProperty(node(), textBlockStyle);
     if (disabled)
-        textBlockStyle->setColor(disabledTextColor(textBlockStyle->color(), startStyle->backgroundColor()));
+        textBlockStyle->setColor(disabledTextColor(textBlockStyle->visitedDependentColor(CSSPropertyColor), startStyle->visitedDependentColor(CSSPropertyBackgroundColor)));
 }
 
 void RenderTextControl::createSubtreeIfNeeded(TextControlInnerElement* innerBlock)
@@ -144,20 +146,19 @@ void RenderTextControl::createSubtreeIfNeeded(TextControlInnerElement* innerBloc
         // For non-search fields, there is no intermediate innerBlock as the shadow node.
         // m_innerText will be the shadow node in that case.        
         RenderStyle* parentStyle = innerBlock ? innerBlock->renderer()->style() : style();
-        m_innerText = new TextControlInnerTextElement(document(), innerBlock ? 0 : node());
+        m_innerText = TextControlInnerTextElement::create(document(), innerBlock ? 0 : static_cast<HTMLElement*>(node()));
         m_innerText->attachInnerElement(innerBlock ? innerBlock : node(), createInnerTextStyle(parentStyle), renderArena());
     }
 }
 
 int RenderTextControl::textBlockHeight() const
 {
-    return height() - paddingTop() - paddingBottom() - borderTop() - borderBottom();
+    return height() - borderAndPaddingHeight();
 }
 
 int RenderTextControl::textBlockWidth() const
 {
-    return width() - paddingLeft() - paddingRight() - borderLeft() - borderRight()
-           - m_innerText->renderBox()->paddingLeft() - m_innerText->renderBox()->paddingRight();
+    return width() - borderAndPaddingWidth() - m_innerText->renderBox()->paddingLeft() - m_innerText->renderBox()->paddingRight();
 }
 
 void RenderTextControl::updateFromElement()
@@ -167,18 +168,10 @@ void RenderTextControl::updateFromElement()
 
 void RenderTextControl::setInnerTextValue(const String& innerTextValue)
 {
-    String value;
-
-    if (innerTextValue.isNull())
-        value = "";
-    else {
-        value = innerTextValue; 
-        value = document()->displayStringModifiedByEncoding(value);
-    }
-
+    String value = innerTextValue;
     if (value != text() || !m_innerText->hasChildNodes()) {
         if (value != text()) {
-            if (Frame* frame = document()->frame()) {
+            if (Frame* frame = this->frame()) {
                 frame->editor()->clearUndoRedoOperations();
                 
                 if (AXObjectCache::accessibilityEnabled())
@@ -191,7 +184,7 @@ void RenderTextControl::setInnerTextValue(const String& innerTextValue)
         ASSERT(!ec);
 
         if (value.endsWith("\n") || value.endsWith("\r")) {
-            m_innerText->appendChild(new HTMLBRElement(brTag, document()), ec);
+            m_innerText->appendChild(HTMLBRElement::create(document()), ec);
             ASSERT(!ec);
         }
 
@@ -208,78 +201,87 @@ void RenderTextControl::setLastChangeWasUserEdit(bool lastChangeWasUserEdit)
     document()->setIgnoreAutofocus(lastChangeWasUserEdit);
 }
 
-int RenderTextControl::selectionStart()
+int RenderTextControl::selectionStart() const
 {
-    Frame* frame = document()->frame();
+    Frame* frame = this->frame();
     if (!frame)
         return 0;
     return indexForVisiblePosition(frame->selection()->start());
 }
 
-int RenderTextControl::selectionEnd()
+int RenderTextControl::selectionEnd() const
 {
-    Frame* frame = document()->frame();
+    Frame* frame = this->frame();
     if (!frame)
         return 0;
     return indexForVisiblePosition(frame->selection()->end());
 }
 
-void RenderTextControl::setSelectionStart(int start)
+bool RenderTextControl::hasVisibleTextArea() const
 {
-    setSelectionRange(start, max(start, selectionEnd()));
+    return style()->visibility() == HIDDEN || !m_innerText || !m_innerText->renderer() || !m_innerText->renderBox()->height();
 }
 
-void RenderTextControl::setSelectionEnd(int end)
+void setSelectionRange(Node* node, int start, int end)
 {
-    setSelectionRange(min(end, selectionStart()), end);
-}
+    ASSERT(node);
+    node->document()->updateLayoutIgnorePendingStylesheets();
 
-void RenderTextControl::select()
-{
-    setSelectionRange(0, text().length());
-}
+    if (!node->renderer() || !node->renderer()->isTextControl())
+        return;
 
-void RenderTextControl::setSelectionRange(int start, int end)
-{
     end = max(end, 0);
     start = min(max(start, 0), end);
 
-    ASSERT(!document()->childNeedsAndNotInStyleRecalc());
+    RenderTextControl* control = toRenderTextControl(node->renderer());
 
-    if (style()->visibility() == HIDDEN || !m_innerText || !m_innerText->renderer() || !m_innerText->renderBox()->height()) {
-        cacheSelection(start, end);
+    if (control->hasVisibleTextArea()) {
+        control->cacheSelection(start, end);
         return;
     }
-    VisiblePosition startPosition = visiblePositionForIndex(start);
+    VisiblePosition startPosition = control->visiblePositionForIndex(start);
     VisiblePosition endPosition;
     if (start == end)
         endPosition = startPosition;
     else
-        endPosition = visiblePositionForIndex(end);
+        endPosition = control->visiblePositionForIndex(end);
 
     // startPosition and endPosition can be null position for example when
     // "-webkit-user-select: none" style attribute is specified.
     if (startPosition.isNotNull() && endPosition.isNotNull()) {
-        ASSERT(startPosition.deepEquivalent().node()->shadowAncestorNode() == node() && endPosition.deepEquivalent().node()->shadowAncestorNode() == node());
+        ASSERT(startPosition.deepEquivalent().node()->shadowAncestorNode() == node && endPosition.deepEquivalent().node()->shadowAncestorNode() == node);
     }
     VisibleSelection newSelection = VisibleSelection(startPosition, endPosition);
 
-    if (Frame* frame = document()->frame())
+    if (Frame* frame = node->document()->frame())
         frame->selection()->setSelection(newSelection);
-
-    // FIXME: Granularity is stored separately on the frame, but also in the selection controller.
-    // The granularity in the selection controller should be used, and then this line of code would not be needed.
-    if (Frame* frame = document()->frame())
-        frame->setSelectionGranularity(CharacterGranularity);
 }
 
-VisibleSelection RenderTextControl::selection(int start, int end) const
+bool RenderTextControl::isSelectableElement(Node* node) const
 {
-    return VisibleSelection(VisiblePosition(m_innerText.get(), start, VP_DEFAULT_AFFINITY),
-                            VisiblePosition(m_innerText.get(), end, VP_DEFAULT_AFFINITY));
+    if (!node || !m_innerText)
+        return false;
+    
+    if (node->rootEditableElement() == m_innerText)
+        return true;
+    
+    if (!m_innerText->contains(node))
+        return false;
+    
+    Node* shadowAncestor = node->shadowAncestorNode();
+    return shadowAncestor && (shadowAncestor->hasTagName(textareaTag)
+        || (shadowAncestor->hasTagName(inputTag) && static_cast<HTMLInputElement*>(shadowAncestor)->isTextField()));
 }
 
-VisiblePosition RenderTextControl::visiblePositionForIndex(int index)
+PassRefPtr<Range> RenderTextControl::selection(int start, int end) const
+{
+    if (!m_innerText)
+        return 0;
+
+    return Range::create(document(), m_innerText, start, m_innerText, end);
+}
+
+VisiblePosition RenderTextControl::visiblePositionForIndex(int index) const
 {
     if (index <= 0)
         return VisiblePosition(m_innerText.get(), 0, DOWNSTREAM);
@@ -296,10 +298,10 @@ VisiblePosition RenderTextControl::visiblePositionForIndex(int index)
     return VisiblePosition(endContainer, endOffset, UPSTREAM);
 }
 
-int RenderTextControl::indexForVisiblePosition(const VisiblePosition& pos)
+int RenderTextControl::indexForVisiblePosition(const VisiblePosition& pos) const
 {
     Position indexPosition = pos.deepEquivalent();
-    if (!indexPosition.node() || indexPosition.node()->rootEditableElement() != m_innerText)
+    if (!isSelectableElement(indexPosition.node()))
         return 0;
     ExceptionCode ec = 0;
     RefPtr<Range> range = Range::create(document());
@@ -323,9 +325,6 @@ String RenderTextControl::finishText(Vector<UChar>& result) const
     if (size && result[size - 1] == '\n')
         result.shrink(--size);
 
-    // Convert backslash to currency symbol.
-    document()->displayBufferModifiedByEncoding(result.data(), result.size());
-    
     return String::adopt(result);
 }
 
@@ -369,26 +368,22 @@ String RenderTextControl::textWithHardLineBreaks()
 {
     if (!m_innerText)
         return "";
-    Node* firstChild = m_innerText->firstChild();
-    if (!firstChild)
-        return "";
 
-    RenderObject* renderer = firstChild->renderer();
+    RenderBlock* renderer = toRenderBlock(m_innerText->renderer());
     if (!renderer)
         return "";
 
-    InlineBox* box = renderer->isText() ? toRenderText(renderer)->firstTextBox() : toRenderBox(renderer)->inlineBoxWrapper();
-    if (!box)
-        return "";
-
     Node* breakNode;
     unsigned breakOffset;
-    RootInlineBox* line = box->root();
+    RootInlineBox* line = renderer->firstRootBox();
+    if (!line)
+        return "";
+
     getNextSoftBreak(line, breakNode, breakOffset);
 
     Vector<UChar> result;
 
-    for (Node* n = firstChild; n; n = n->traverseNextNode(m_innerText.get())) {
+    for (Node* n = m_innerText->firstChild(); n; n = n->traverseNextNode(m_innerText.get())) {
         if (n->hasTagName(brTag))
             result.append(&newlineCharacter, 1);
         else if (n->isTextNode()) {
@@ -419,20 +414,20 @@ int RenderTextControl::scrollbarThickness() const
     return ScrollbarTheme::nativeTheme()->scrollbarThickness();
 }
 
-void RenderTextControl::calcHeight()
+void RenderTextControl::computeLogicalHeight()
 {
     setHeight(m_innerText->renderBox()->borderTop() + m_innerText->renderBox()->borderBottom() +
               m_innerText->renderBox()->paddingTop() + m_innerText->renderBox()->paddingBottom() +
               m_innerText->renderBox()->marginTop() + m_innerText->renderBox()->marginBottom());
 
-    adjustControlHeightBasedOnLineHeight(m_innerText->renderer()->lineHeight(true, true));
-    setHeight(height() + paddingTop() + paddingBottom() + borderTop() + borderBottom());
+    adjustControlHeightBasedOnLineHeight(m_innerText->renderBox()->lineHeight(true, HorizontalLine, PositionOfInteriorLineBoxes));
+    setHeight(height() + borderAndPaddingHeight());
 
     // We are able to have a horizontal scrollbar if the overflow style is scroll, or if its auto and there's no word wrap.
     if (style()->overflowX() == OSCROLL ||  (style()->overflowX() == OAUTO && m_innerText->renderer()->style()->wordWrap() == NormalWordWrap))
         setHeight(height() + scrollbarThickness());
 
-    RenderBlock::calcHeight();
+    RenderBlock::computeLogicalHeight();
 }
 
 void RenderTextControl::hitInnerTextElement(HitTestResult& result, int xPos, int yPos, int tx, int ty)
@@ -450,13 +445,6 @@ void RenderTextControl::forwardEvent(Event* event)
     m_innerText->defaultEventHandler(event);
 }
 
-IntRect RenderTextControl::controlClipRect(int tx, int ty) const
-{
-    IntRect clipRect = contentBoxRect();
-    clipRect.move(tx, ty);
-    return clipRect;
-}
-
 static const char* fontFamiliesWithInvalidCharWidth[] = {
     "American Typewriter",
     "Arial Hebrew",
@@ -502,11 +490,11 @@ static const char* fontFamiliesWithInvalidCharWidth[] = {
 bool RenderTextControl::hasValidAvgCharWidth(AtomicString family)
 {
     static HashSet<AtomicString>* fontFamiliesWithInvalidCharWidthMap = 0;
-    
+
     if (!fontFamiliesWithInvalidCharWidthMap) {
         fontFamiliesWithInvalidCharWidthMap = new HashSet<AtomicString>;
-        
-        for (unsigned i = 0; i < sizeof(fontFamiliesWithInvalidCharWidth) / sizeof(fontFamiliesWithInvalidCharWidth[0]); i++)
+
+        for (size_t i = 0; i < WTF_ARRAY_LENGTH(fontFamiliesWithInvalidCharWidth); ++i)
             fontFamiliesWithInvalidCharWidthMap->add(AtomicString(fontFamiliesWithInvalidCharWidth[i]));
     }
 
@@ -529,47 +517,47 @@ float RenderTextControl::scaleEmToUnits(int x) const
     return roundf(style()->font().size() * x / unitsPerEm);
 }
 
-void RenderTextControl::calcPrefWidths()
+void RenderTextControl::computePreferredLogicalWidths()
 {
-    ASSERT(prefWidthsDirty());
+    ASSERT(preferredLogicalWidthsDirty());
 
-    m_minPrefWidth = 0;
-    m_maxPrefWidth = 0;
+    m_minPreferredLogicalWidth = 0;
+    m_maxPreferredLogicalWidth = 0;
 
     if (style()->width().isFixed() && style()->width().value() > 0)
-        m_minPrefWidth = m_maxPrefWidth = calcContentBoxWidth(style()->width().value());
+        m_minPreferredLogicalWidth = m_maxPreferredLogicalWidth = computeContentBoxLogicalWidth(style()->width().value());
     else {
         // Use average character width. Matches IE.
         AtomicString family = style()->font().family().family();
-        m_maxPrefWidth = preferredContentWidth(getAvgCharWidth(family)) + m_innerText->renderBox()->paddingLeft() + m_innerText->renderBox()->paddingRight();
+        m_maxPreferredLogicalWidth = preferredContentWidth(getAvgCharWidth(family)) + m_innerText->renderBox()->paddingLeft() + m_innerText->renderBox()->paddingRight();
     }
 
     if (style()->minWidth().isFixed() && style()->minWidth().value() > 0) {
-        m_maxPrefWidth = max(m_maxPrefWidth, calcContentBoxWidth(style()->minWidth().value()));
-        m_minPrefWidth = max(m_minPrefWidth, calcContentBoxWidth(style()->minWidth().value()));
+        m_maxPreferredLogicalWidth = max(m_maxPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->minWidth().value()));
+        m_minPreferredLogicalWidth = max(m_minPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->minWidth().value()));
     } else if (style()->width().isPercent() || (style()->width().isAuto() && style()->height().isPercent()))
-        m_minPrefWidth = 0;
+        m_minPreferredLogicalWidth = 0;
     else
-        m_minPrefWidth = m_maxPrefWidth;
+        m_minPreferredLogicalWidth = m_maxPreferredLogicalWidth;
 
     if (style()->maxWidth().isFixed() && style()->maxWidth().value() != undefinedLength) {
-        m_maxPrefWidth = min(m_maxPrefWidth, calcContentBoxWidth(style()->maxWidth().value()));
-        m_minPrefWidth = min(m_minPrefWidth, calcContentBoxWidth(style()->maxWidth().value()));
+        m_maxPreferredLogicalWidth = min(m_maxPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->maxWidth().value()));
+        m_minPreferredLogicalWidth = min(m_minPreferredLogicalWidth, computeContentBoxLogicalWidth(style()->maxWidth().value()));
     }
 
-    int toAdd = paddingLeft() + paddingRight() + borderLeft() + borderRight();
+    int toAdd = borderAndPaddingWidth();
 
-    m_minPrefWidth += toAdd;
-    m_maxPrefWidth += toAdd;
+    m_minPreferredLogicalWidth += toAdd;
+    m_maxPreferredLogicalWidth += toAdd;
 
-    setPrefWidthsDirty(false);
+    setPreferredLogicalWidthsDirty(false);
 }
 
 void RenderTextControl::selectionChanged(bool userTriggered)
 {
     cacheSelection(selectionStart(), selectionEnd());
 
-    if (Frame* frame = document()->frame()) {
+    if (Frame* frame = this->frame()) {
         if (frame->selection()->isRange() && userTriggered)
             node()->dispatchEvent(Event::create(eventNames().selectEvent, true, false));
     }
@@ -590,14 +578,52 @@ void RenderTextControl::updatePlaceholderVisibility(bool placeholderShouldBeVisi
 {
     bool oldPlaceholderVisible = m_placeholderVisible;
     m_placeholderVisible = placeholderShouldBeVisible;
-    if (oldPlaceholderVisible != m_placeholderVisible || placeholderValueChanged) {
-        // Sets the inner text style to the normal style or :placeholder style.
-        setInnerTextStyle(createInnerTextStyle(textBaseStyle()));
+    if (oldPlaceholderVisible != m_placeholderVisible || placeholderValueChanged)
+        repaint();
+}
 
-        // updateFromElement() of the subclasses updates the text content
-        // to the element's value(), placeholder(), or the empty string.
-        updateFromElement();
+void RenderTextControl::paintPlaceholder(PaintInfo& paintInfo, int tx, int ty)
+{
+    if (style()->visibility() != VISIBLE)
+        return;
+    
+    IntRect clipRect(tx + borderLeft(), ty + borderTop(), width() - borderLeft() - borderRight(), height() - borderBottom() - borderTop());
+    if (clipRect.isEmpty())
+        return;
+    
+    paintInfo.context->save();
+    
+    paintInfo.context->clip(clipRect);
+    
+    RefPtr<RenderStyle> placeholderStyle = getCachedPseudoStyle(INPUT_PLACEHOLDER);
+    if (!placeholderStyle)
+        placeholderStyle = style();
+    
+    paintInfo.context->setFillColor(placeholderStyle->visitedDependentColor(CSSPropertyColor), placeholderStyle->colorSpace());
+    
+    String placeholderText = static_cast<HTMLTextFormControlElement*>(node())->strippedPlaceholder();
+    TextRun textRun(placeholderText.characters(), placeholderText.length(), 0, 0, 0, !placeholderStyle->isLeftToRightDirection(), placeholderStyle->unicodeBidi() == Override, false, false);
+    
+    RenderBox* textRenderer = innerTextElement() ? innerTextElement()->renderBox() : 0;
+    if (textRenderer) {
+        IntPoint textPoint;
+        textPoint.setY(ty + textBlockInsetTop() + placeholderStyle->font().ascent());
+        if (placeholderStyle->isLeftToRightDirection())
+            textPoint.setX(tx + textBlockInsetLeft());
+        else
+            textPoint.setX(tx + width() - textBlockInsetRight() - style()->font().width(textRun));
+        
+        paintInfo.context->drawBidiText(placeholderStyle->font(), textRun, textPoint);
     }
+    paintInfo.context->restore();    
+}
+
+void RenderTextControl::paintObject(PaintInfo& paintInfo, int tx, int ty)
+{    
+    if (m_placeholderVisible && paintInfo.phase == PaintPhaseForeground)
+        paintPlaceholder(paintInfo, tx, ty);
+    
+    RenderBlock::paintObject(paintInfo, tx, ty);
 }
 
 } // namespace WebCore
