@@ -31,16 +31,15 @@
 #if ENABLE(DATABASE)
 
 #include "PlatformString.h"
-#include "StringHash.h"
-
 #include <wtf/HashMap.h>
-#include <wtf/HashSet.h>
+#include <wtf/HashSet.h> 
+#include <wtf/text/StringHash.h>
 
 namespace WebCore {
 
 // Objects of this class can be used from multiple threads with external synchronization.
 // String arguments are also supposed to be deeply copied by the caller when necessary.
-class OriginUsageRecord {
+class OriginUsageRecord : public Noncopyable {
 public:
     OriginUsageRecord();
 
@@ -68,4 +67,4 @@ private:
 
 #endif
 
-#endif 
+#endif
