@@ -31,26 +31,26 @@
 
 #if ENABLE(DATABASE)
 
-#include "ChromeClient.h"
 #include "Database.h"
 #include "DatabaseAuthorizer.h"
-#include "DatabaseDetails.h"
-#include "Document.h"
-#include "ExceptionCode.h"
+#include "DatabaseThread.h"
 #include "Logging.h"
-#include "Page.h"
 #include "PlatformString.h"
-#include "SecurityOrigin.h"
-#include "Settings.h"
+#include "ScriptExecutionContext.h"
 #include "SQLError.h"
 #include "SQLiteTransaction.h"
-#include "SQLResultSet.h"
 #include "SQLStatement.h"
 #include "SQLStatementCallback.h"
 #include "SQLStatementErrorCallback.h"
+#include "SQLTransactionCallback.h"
 #include "SQLTransactionClient.h"
 #include "SQLTransactionCoordinator.h"
+#include "SQLTransactionErrorCallback.h"
 #include "SQLValue.h"
+#include "VoidCallback.h"
+#include <wtf/OwnPtr.h>
+#include <wtf/PassRefPtr.h>
+#include <wtf/RefPtr.h>
 
 // There's no way of knowing exactly how much more space will be required when a statement hits the quota limit.
 // For now, we'll arbitrarily choose currentQuota + 1mb.
@@ -65,8 +65,8 @@ PassRefPtr<SQLTransaction> SQLTransaction::create(Database* db, PassRefPtr<SQLTr
     return adoptRef(new SQLTransaction(db, callback, errorCallback, successCallback, wrapper, readOnly));
 }
 
-SQLTransaction::SQLTransaction(Database* db, PassRefPtr<SQLTransactionCallback> callback, PassRefPtr<SQLTransactionErrorCallback> errorCallback, PassRefPtr<VoidCallback> successCallback,
-                               PassRefPtr<SQLTransactionWrapper> wrapper, bool readOnly)
+SQLTransaction::SQLTransaction(Database* db, PassRefPtr<SQLTransactionCallback> callback, PassRefPtr<SQLTransactionErrorCallback> errorCallback,
+                               PassRefPtr<VoidCallback> successCallback, PassRefPtr<SQLTransactionWrapper> wrapper, bool readOnly)
     : m_nextStep(&SQLTransaction::acquireLock)
     , m_executeSqlAllowed(false)
     , m_database(db)
@@ -84,23 +84,23 @@ SQLTransaction::SQLTransaction(Database* db, PassRefPtr<SQLTransactionCallback>
 
 SQLTransaction::~SQLTransaction()
 {
+    ASSERT(!m_sqliteTransaction);
 }
 
 void SQLTransaction::executeSQL(const String& sqlStatement, const Vector<SQLValue>& arguments, PassRefPtr<SQLStatementCallback> callback, PassRefPtr<SQLStatementErrorCallback> callbackError, ExceptionCode& e)
 {
-    if (!m_executeSqlAllowed || m_database->stopped()) {
+    if (!m_executeSqlAllowed || !m_database->opened()) {
         e = INVALID_STATE_ERR;
         return;
     }
 
-    bool readOnlyMode = m_readOnly;
-    if (!readOnlyMode) {
-        Page* page = m_database->document()->page();
-        if (!page || page->settings()->privateBrowsingEnabled())
-            readOnlyMode = true;
-    }
+    int permissions = DatabaseAuthorizer::ReadWriteMask;
+    if (!m_database->scriptExecutionContext()->allowDatabaseAccess())
+        permissions |= DatabaseAuthorizer::NoAccessMask;
+    else if (m_readOnly)
+        permissions |= DatabaseAuthorizer::ReadOnlyMask;
 
-    RefPtr<SQLStatement> statement = SQLStatement::create(sqlStatement, arguments, callback, callbackError, readOnlyMode);
+    RefPtr<SQLStatement> statement = SQLStatement::create(sqlStatement, arguments, callback, callbackError, permissions);
 
     if (m_database->deleted())
         statement->setDatabaseDeletedError();
@@ -147,17 +147,26 @@ const char* SQLTransaction::debugStepName(SQLTransaction::TransactionStepMethod
 }
 #endif
 
-void SQLTransaction::checkAndHandleClosedDatabase()
+void SQLTransaction::checkAndHandleClosedOrInterruptedDatabase()
 {
-    if (!m_database->stopped())
+    if (m_database->opened() && !m_database->isInterrupted())
         return;
 
     // If the database was stopped, don't do anything and cancel queued work
-    LOG(StorageAPI, "Database was stopped - cancelling work for this transaction");
+    LOG(StorageAPI, "Database was stopped or interrupted - cancelling work for this transaction");
     MutexLocker locker(m_statementMutex);
     m_statementQueue.clear();
     m_nextStep = 0;
 
+    // Release the unneeded callbacks, to break reference cycles.
+    m_callback = 0;
+    m_successCallback = 0;
+    m_errorCallback = 0;
+
+    // The next steps should be executed only if we're on the DB thread.
+    if (currentThread() != database()->scriptExecutionContext()->databaseThread()->getThreadID())
+        return;
+
     // The current SQLite transaction should be stopped, as well
     if (m_sqliteTransaction) {
         m_sqliteTransaction->stop();
@@ -180,7 +189,7 @@ bool SQLTransaction::performNextStep()
            m_nextStep == &SQLTransaction::cleanupAfterSuccessCallback ||
            m_nextStep == &SQLTransaction::cleanupAfterTransactionErrorCallback);
 
-    checkAndHandleClosedDatabase();
+    checkAndHandleClosedOrInterruptedDatabase();
 
     if (m_nextStep)
         (this->*m_nextStep)();
@@ -199,15 +208,25 @@ void SQLTransaction::performPendingCallback()
            m_nextStep == &SQLTransaction::deliverQuotaIncreaseCallback ||
            m_nextStep == &SQLTransaction::deliverSuccessCallback);
 
-    checkAndHandleClosedDatabase();
+    checkAndHandleClosedOrInterruptedDatabase();
 
     if (m_nextStep)
         (this->*m_nextStep)();
 }
 
+void SQLTransaction::notifyDatabaseThreadIsShuttingDown()
+{
+    ASSERT(currentThread() == database()->scriptExecutionContext()->databaseThread()->getThreadID());
+
+    // If the transaction is in progress, we should roll it back here, since this is our last
+    // oportunity to do something related to this transaction on the DB thread.
+    // Clearing m_sqliteTransaction invokes SQLiteTransaction's destructor which does just that.
+    m_sqliteTransaction.clear();
+}
+
 void SQLTransaction::acquireLock()
 {
-    m_database->transactionCoordinator()->acquireLock(this, m_readOnly);
+    m_database->transactionCoordinator()->acquireLock(this);
 }
 
 void SQLTransaction::lockAcquired()
@@ -220,44 +239,45 @@ void SQLTransaction::lockAcquired()
 
 void SQLTransaction::openTransactionAndPreflight()
 {
-    ASSERT(!m_database->m_sqliteDatabase.transactionInProgress());
+    ASSERT(!m_database->sqliteDatabase().transactionInProgress());
     ASSERT(m_lockAcquired);
 
     LOG(StorageAPI, "Opening and preflighting transaction %p", this);
 
     // If the database was deleted, jump to the error callback
     if (m_database->deleted()) {
-        m_transactionError = SQLError::create(0, "unable to open a transaction, because the user deleted the database");
+        m_transactionError = SQLError::create(SQLError::UNKNOWN_ERR, "unable to open a transaction, because the user deleted the database");
         handleTransactionError(false);
         return;
     }
 
-    // Set the maximum usage for this transaction
-    m_database->m_sqliteDatabase.setMaximumSize(m_database->maximumSize());
+    // Set the maximum usage for this transaction if this transactions is not read-only
+    if (!m_readOnly)
+        m_database->sqliteDatabase().setMaximumSize(m_database->maximumSize());
 
     ASSERT(!m_sqliteTransaction);
-    m_sqliteTransaction.set(new SQLiteTransaction(m_database->m_sqliteDatabase));
+    m_sqliteTransaction = adoptPtr(new SQLiteTransaction(m_database->sqliteDatabase(), m_readOnly));
 
-    m_database->m_databaseAuthorizer->disable();
+    m_database->resetDeletes();
+    m_database->disableAuthorizer();
     m_sqliteTransaction->begin();
-    m_database->m_databaseAuthorizer->enable();
+    m_database->enableAuthorizer();
 
     // Transaction Steps 1+2 - Open a transaction to the database, jumping to the error callback if that fails
     if (!m_sqliteTransaction->inProgress()) {
-        ASSERT(!m_database->m_sqliteDatabase.transactionInProgress());
+        ASSERT(!m_database->sqliteDatabase().transactionInProgress());
         m_sqliteTransaction.clear();
-        m_transactionError = SQLError::create(0, "unable to open a transaction to the database");
+        m_transactionError = SQLError::create(SQLError::DATABASE_ERR, "unable to open a transaction to the database");
         handleTransactionError(false);
         return;
     }
 
     // Transaction Steps 3 - Peform preflight steps, jumping to the error callback if they fail
     if (m_wrapper && !m_wrapper->performPreflight(this)) {
-        ASSERT(!m_database->m_sqliteDatabase.transactionInProgress());
         m_sqliteTransaction.clear();
         m_transactionError = m_wrapper->sqlError();
         if (!m_transactionError)
-            m_transactionError = SQLError::create(0, "unknown error occured setting up transaction");
+            m_transactionError = SQLError::create(SQLError::UNKNOWN_ERR, "unknown error occured setting up transaction");
 
         handleTransactionError(false);
         return;
@@ -275,14 +295,14 @@ void SQLTransaction::deliverTransactionCallback()
 
     if (m_callback) {
         m_executeSqlAllowed = true;
-        m_callback->handleEvent(this, shouldDeliverErrorCallback);
+        shouldDeliverErrorCallback = !m_callback->handleEvent(this);
         m_executeSqlAllowed = false;
-    } else
-        shouldDeliverErrorCallback = true;
+        m_callback = 0;
+    }
 
     // Transaction Step 5 - If the transaction callback was null or raised an exception, jump to the error callback
     if (shouldDeliverErrorCallback) {
-        m_transactionError = SQLError::create(0, "the SQLTransactionCallback was null or threw an exception");
+        m_transactionError = SQLError::create(SQLError::UNKNOWN_ERR, "the SQLTransactionCallback was null or threw an exception");
         deliverTransactionErrorCallback();
     } else
         scheduleToRunStatements();
@@ -302,13 +322,16 @@ void SQLTransaction::runStatements()
     // If there is a series of statements queued up that are all successful and have no associated
     // SQLStatementCallback objects, then we can burn through the queue
     do {
-        if (m_shouldRetryCurrentStatement) {
+        if (m_shouldRetryCurrentStatement && !m_sqliteTransaction->wasRolledBackBySqlite()) {
             m_shouldRetryCurrentStatement = false;
             // FIXME - Another place that needs fixing up after <rdar://problem/5628468> is addressed.
             // See ::openTransactionAndPreflight() for discussion
 
-            // Reset the maximum size here, as it was increased to allow us to retry this statement
-            m_database->m_sqliteDatabase.setMaximumSize(m_database->maximumSize());
+            // Reset the maximum size here, as it was increased to allow us to retry this statement.
+            // m_shouldRetryCurrentStatement is set to true only when a statement exceeds
+            // the quota, which can happen only in a read-write transaction. Therefore, there
+            // is no need to check here if the transaction is read-write.
+            m_database->sqliteDatabase().setMaximumSize(m_database->maximumSize());
         } else {
             // If the current statement has already been run, failed due to quota constraints, and we're not retrying it,
             // that means it ended in an error.  Handle it now
@@ -335,8 +358,7 @@ void SQLTransaction::getNextStatement()
 
     MutexLocker locker(m_statementMutex);
     if (!m_statementQueue.isEmpty()) {
-        m_currentStatement = m_statementQueue.first();
-        m_statementQueue.removeFirst();
+        m_currentStatement = m_statementQueue.takeFirst();
     }
 }
 
@@ -345,14 +367,14 @@ bool SQLTransaction::runCurrentStatement()
     if (!m_currentStatement)
         return false;
 
-    m_database->m_databaseAuthorizer->reset();
+    m_database->resetAuthorizer();
 
     if (m_currentStatement->execute(m_database.get())) {
-        if (m_database->m_databaseAuthorizer->lastActionChangedDatabase()) {
+        if (m_database->lastActionChangedDatabase()) {
             // Flag this transaction as having changed the database for later delegate notification
             m_modifiedDatabase = true;
             // Also dirty the size of this database file for calculating quota usage
-            m_database->transactionClient()->didExecuteStatement(this);
+            m_database->transactionClient()->didExecuteStatement(database());
         }
 
         if (m_currentStatement->hasStatementCallback()) {
@@ -379,15 +401,15 @@ bool SQLTransaction::runCurrentStatement()
 void SQLTransaction::handleCurrentStatementError()
 {
     // Transaction Steps 6.error - Call the statement's error callback, but if there was no error callback,
-    // jump to the transaction error callback
-    if (m_currentStatement->hasStatementErrorCallback()) {
+    // or the transaction was rolled back, jump to the transaction error callback
+    if (m_currentStatement->hasStatementErrorCallback() && !m_sqliteTransaction->wasRolledBackBySqlite()) {
         m_nextStep = &SQLTransaction::deliverStatementCallback;
         LOG(StorageAPI, "Scheduling deliverStatementCallback for transaction %p\n", this);
         m_database->scheduleTransactionCallback(this);
     } else {
         m_transactionError = m_currentStatement->sqlError();
         if (!m_transactionError)
-            m_transactionError = SQLError::create(1, "the statement failed to execute");
+            m_transactionError = SQLError::create(SQLError::DATABASE_ERR, "the statement failed to execute");
         handleTransactionError(false);
     }
 }
@@ -403,7 +425,7 @@ void SQLTransaction::deliverStatementCallback()
     m_executeSqlAllowed = false;
 
     if (result) {
-        m_transactionError = SQLError::create(0, "the statement callback raised an exception or statement error callback did not return false");
+      m_transactionError = SQLError::create(SQLError::UNKNOWN_ERR, "the statement callback raised an exception or statement error callback did not return false");
         handleTransactionError(true);
     } else
         scheduleToRunStatements();
@@ -414,7 +436,7 @@ void SQLTransaction::deliverQuotaIncreaseCallback()
     ASSERT(m_currentStatement);
     ASSERT(!m_shouldRetryCurrentStatement);
 
-    m_shouldRetryCurrentStatement = m_database->transactionClient()->didExceedQuota(this);
+    m_shouldRetryCurrentStatement = m_database->transactionClient()->didExceedQuota(database());
 
     m_nextStep = &SQLTransaction::runStatements;
     LOG(StorageAPI, "Scheduling runStatements for transaction %p\n", this);
@@ -429,7 +451,7 @@ void SQLTransaction::postflightAndCommit()
     if (m_wrapper && !m_wrapper->performPostflight(this)) {
         m_transactionError = m_wrapper->sqlError();
         if (!m_transactionError)
-            m_transactionError = SQLError::create(0, "unknown error occured setting up transaction");
+            m_transactionError = SQLError::create(SQLError::UNKNOWN_ERR, "unknown error occured setting up transaction");
         handleTransactionError(false);
         return;
     }
@@ -437,23 +459,27 @@ void SQLTransaction::postflightAndCommit()
     // Transacton Step 8+9 - Commit the transaction, jumping to the error callback if that fails
     ASSERT(m_sqliteTransaction);
 
-    m_database->m_databaseAuthorizer->disable();
+    m_database->disableAuthorizer();
     m_sqliteTransaction->commit();
-    m_database->m_databaseAuthorizer->enable();
+    m_database->enableAuthorizer();
 
     // If the commit failed, the transaction will still be marked as "in progress"
     if (m_sqliteTransaction->inProgress()) {
-        m_transactionError = SQLError::create(0, "failed to commit the transaction");
+        m_successCallback = 0;
+        m_transactionError = SQLError::create(SQLError::DATABASE_ERR, "failed to commit the transaction");
         handleTransactionError(false);
         return;
     }
 
-    // The commit was successful, notify the delegates if the transaction modified this database
+    // Vacuum the database if anything was deleted.
+    if (m_database->hadDeletes())
+        m_database->incrementalVacuumIfNeeded();
+
+    // The commit was successful. If the transaction modified this database, notify the delegates.
     if (m_modifiedDatabase)
-        m_database->transactionClient()->didCommitTransaction(this);
+        m_database->transactionClient()->didCommitWriteTransaction(database());
 
     // Now release our unneeded callbacks, to break reference cycles.
-    m_callback = 0;
     m_errorCallback = 0;
 
     // Transaction Step 10 - Deliver success callback, if there is one
@@ -488,7 +514,8 @@ void SQLTransaction::cleanupAfterSuccessCallback()
     // Transaction Step 11 - End transaction steps
     // There is no next step
     LOG(StorageAPI, "Transaction %p is complete\n", this);
-    ASSERT(!m_database->m_sqliteDatabase.transactionInProgress());
+    ASSERT(!m_database->sqliteDatabase().transactionInProgress());
+    m_sqliteTransaction.clear();
     m_nextStep = 0;
 
     // Release the lock on this database
@@ -525,8 +552,10 @@ void SQLTransaction::deliverTransactionErrorCallback()
 
     // Transaction Step 12 - If exists, invoke error callback with the last
     // error to have occurred in this transaction.
-    if (m_errorCallback)
+    if (m_errorCallback) {
         m_errorCallback->handleEvent(m_transactionError.get());
+        m_errorCallback = 0;
+    }
 
     m_nextStep = &SQLTransaction::cleanupAfterTransactionErrorCallback;
     LOG(StorageAPI, "Scheduling cleanupAfterTransactionErrorCallback for transaction %p\n", this);
@@ -537,15 +566,15 @@ void SQLTransaction::cleanupAfterTransactionErrorCallback()
 {
     ASSERT(m_lockAcquired);
 
-    m_database->m_databaseAuthorizer->disable();
+    m_database->disableAuthorizer();
     if (m_sqliteTransaction) {
         // Transaction Step 12 - Rollback the transaction.
         m_sqliteTransaction->rollback();
 
-        ASSERT(!m_database->m_sqliteDatabase.transactionInProgress());
+        ASSERT(!m_database->sqliteDatabase().transactionInProgress());
         m_sqliteTransaction.clear();
     }
-    m_database->m_databaseAuthorizer->enable();
+    m_database->enableAuthorizer();
 
     // Transaction Step 12 - Any still-pending statements in the transaction are discarded.
     {
@@ -555,13 +584,9 @@ void SQLTransaction::cleanupAfterTransactionErrorCallback()
 
     // Transaction is complete!  There is no next step
     LOG(StorageAPI, "Transaction %p is complete with an error\n", this);
-    ASSERT(!m_database->m_sqliteDatabase.transactionInProgress());
+    ASSERT(!m_database->sqliteDatabase().transactionInProgress());
     m_nextStep = 0;
 
-    // Now release our callbacks, to break reference cycles.
-    m_callback = 0;
-    m_errorCallback = 0;
-
     // Now release the lock on this database
     m_database->transactionCoordinator()->releaseLock(this);
 }
