@@ -1,48 +1,57 @@
 /*
-    Copyright (C) 2004, 2005 Nikolas Zimmermann <wildfox@kde.org>
-                  2004, 2005, 2006 Rob Buis <buis@kde.org>
-    Copyright (C) 2008 Apple Inc. All rights reserved.
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2006 Rob Buis <buis@kde.org>
+ * Copyright (C) 2008 Apple Inc. All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #include "config.h"
+
 #if ENABLE(SVG) && ENABLE(SVG_ANIMATION)
 #include "SVGAnimateElement.h"
 
 #include "ColorDistance.h"
 #include "FloatConversion.h"
 #include "SVGColor.h"
+#include "SVGNames.h"
 #include "SVGParserUtilities.h"
+#include "SVGPathParserFactory.h"
 #include "SVGPathSegList.h"
-#include <math.h>
+#include "SVGPointList.h"
 
 using namespace std;
 
 namespace WebCore {
 
-SVGAnimateElement::SVGAnimateElement(const QualifiedName& tagName, Document* doc)
-    : SVGAnimationElement(tagName, doc)
+SVGAnimateElement::SVGAnimateElement(const QualifiedName& tagName, Document* document)
+    : SVGAnimationElement(tagName, document)
     , m_propertyType(StringProperty)
     , m_fromNumber(0)
     , m_toNumber(0)
     , m_animatedNumber(numeric_limits<double>::infinity())
+    , m_animatedPathPointer(0)
 {
 }
 
+PassRefPtr<SVGAnimateElement> SVGAnimateElement::create(const QualifiedName& tagName, Document* document)
+{
+    return adoptRef(new SVGAnimateElement(tagName, document));
+}
+
 SVGAnimateElement::~SVGAnimateElement()
 {
 }
@@ -80,6 +89,8 @@ SVGAnimateElement::PropertyType SVGAnimateElement::determinePropertyType(const S
         return ColorProperty;
     if (attribute == "d")
         return PathProperty;
+    if (attribute == "points")
+        return PointsProperty;
     if (attribute == "color" || attribute == "fill" || attribute == "stroke")
         return ColorProperty;
     return NumberProperty;
@@ -127,19 +138,47 @@ void SVGAnimateElement::calculateAnimatedValue(float percentage, unsigned repeat
     }
     AnimationMode animationMode = this->animationMode();
     if (m_propertyType == PathProperty) {
-        if (percentage == 0)
-            results->m_animatedPath = m_fromPath;
-        else if (percentage == 1.f)
-            results->m_animatedPath = m_toPath;
-        else {
-            if (m_fromPath && m_toPath)
-                results->m_animatedPath = SVGPathSegList::createAnimated(m_fromPath.get(), m_toPath.get(), percentage);
-            else
-                results->m_animatedPath.clear();
+        if (!percentage) {
+            ASSERT(m_fromPath);
+            ASSERT(percentage >= 0);
+            results->m_animatedPathPointer = m_fromPath.get();
+        } else if (percentage == 1.f) {
+            ASSERT(m_toPath);
+            results->m_animatedPathPointer = m_toPath.get();
+        } else {
+            if (m_fromPath && m_toPath) {
+                SVGPathParserFactory* factory = SVGPathParserFactory::self();
+                if (!factory->buildAnimatedSVGPathByteStream(m_fromPath.get(), m_toPath.get(), results->m_animatedPath, percentage)) {
+                    results->m_animatedPath.clear();
+                    results->m_animatedPathPointer = 0;
+                } else
+                    results->m_animatedPathPointer = results->m_animatedPath.get();
+            } else
+                results->m_animatedPathPointer = 0;
             // Fall back to discrete animation if the paths are not compatible
-            if (!results->m_animatedPath)
-                results->m_animatedPath = ((animationMode == FromToAnimation && percentage > 0.5f) || animationMode == ToAnimation || percentage == 1.0f) 
-                    ? m_toPath : m_fromPath;
+            if (!results->m_animatedPathPointer) {
+                ASSERT(m_fromPath);
+                ASSERT(m_toPath);
+                ASSERT(!results->m_animatedPath);
+                results->m_animatedPathPointer = ((animationMode == FromToAnimation && percentage > 0.5f) || animationMode == ToAnimation || percentage == 1.0f) 
+                    ? m_toPath.get() : m_fromPath.get();
+            }
+        }
+        return;
+    } else if (m_propertyType == PointsProperty) {
+        if (percentage == 0)
+            results->m_animatedPoints = m_fromPoints;
+        else if (percentage == 1.f)
+            results->m_animatedPoints = m_toPoints;
+        else {
+            if (!m_fromPoints.isEmpty() && !m_toPoints.isEmpty())
+                SVGPointList::createAnimated(m_fromPoints, m_toPoints, results->m_animatedPoints, percentage);
+            else
+                results->m_animatedPoints.clear();
+            // Fall back to discrete animation if the points are not compatible
+            if (results->m_animatedPoints.isEmpty())
+                results->m_animatedPoints = ((animationMode == FromToAnimation && percentage > 0.5f) || animationMode == ToAnimation || percentage == 1.0f) 
+                    ? m_toPoints : m_fromPoints;
         }
         return;
     }
@@ -159,7 +198,7 @@ bool SVGAnimateElement::calculateFromAndToValues(const String& fromString, const
     if (m_propertyType == ColorProperty) {
         m_fromColor = SVGColor::colorFromRGBColorString(fromString);
         m_toColor = SVGColor::colorFromRGBColorString(toString);
-        if (m_fromColor.isValid() && m_toColor.isValid())
+        if ((m_fromColor.isValid() && m_toColor.isValid()) || (m_toColor.isValid() && animationMode() == ToAnimation))
             return true;
     } else if (m_propertyType == NumberProperty) {
         m_numberUnit = String();
@@ -169,14 +208,20 @@ bool SVGAnimateElement::calculateFromAndToValues(const String& fromString, const
                 return true;
         }
     } else if (m_propertyType == PathProperty) {
-        m_fromPath = SVGPathSegList::create(SVGNames::dAttr);
-        if (pathSegListFromSVGData(m_fromPath.get(), fromString)) {
-            m_toPath = SVGPathSegList::create(SVGNames::dAttr);
-            if (pathSegListFromSVGData(m_toPath.get(), toString))
+        SVGPathParserFactory* factory = SVGPathParserFactory::self();
+        if (factory->buildSVGPathByteStreamFromString(fromString, m_fromPath, UnalteredParsing)) {
+            if (factory->buildSVGPathByteStreamFromString(toString, m_toPath, UnalteredParsing))
                 return true;
         }
         m_fromPath.clear();
         m_toPath.clear();
+    } else if (m_propertyType == PointsProperty) {
+        m_fromPoints.clear();
+        if (pointsListFromSVGData(m_fromPoints, fromString)) {
+            m_toPoints.clear();
+            if (pointsListFromSVGData(m_toPoints, toString))
+                return true;
+        }
     }
     m_fromString = fromString;
     m_toString = toString;
@@ -208,11 +253,14 @@ bool SVGAnimateElement::calculateFromAndByValues(const String& fromString, const
 void SVGAnimateElement::resetToBaseValue(const String& baseString)
 {
     m_animatedString = baseString;
+    PropertyType lastType = m_propertyType;
     m_propertyType = determinePropertyType(attributeName());
     if (m_propertyType == ColorProperty) {
         m_animatedColor = baseString.isEmpty() ? Color() : SVGColor::colorFromRGBColorString(baseString);
-        if (m_animatedColor.isValid())
+        if (isContributing(elapsed())) {
+            m_propertyType = lastType;
             return;
+        }
     } else if (m_propertyType == NumberProperty) {
         if (baseString.isEmpty()) {
             m_animatedNumber = 0;
@@ -223,6 +271,10 @@ void SVGAnimateElement::resetToBaseValue(const String& baseString)
             return;
     } else if (m_propertyType == PathProperty) {
         m_animatedPath.clear();
+        m_animatedPathPointer = 0;
+        return;
+    } else if (m_propertyType == PointsProperty) {
+        m_animatedPoints.clear();
         return;
     }
     m_propertyType = StringProperty;
@@ -236,21 +288,19 @@ void SVGAnimateElement::applyResultsToTarget()
     else if (m_propertyType == NumberProperty)
         valueToApply = String::number(m_animatedNumber) + m_numberUnit;
     else if (m_propertyType == PathProperty) {
-        if (!m_animatedPath || !m_animatedPath->numberOfItems())
+        if (!m_animatedPathPointer || m_animatedPathPointer->isEmpty())
             valueToApply = m_animatedString;
         else {
             // We need to keep going to string and back because we are currently only able to paint
             // "processed" paths where complex shapes are replaced with simpler ones. Path 
             // morphing needs to be done with unprocessed paths.
             // FIXME: This could be optimized if paths were not processed at parse time.
-            unsigned itemCount = m_animatedPath->numberOfItems();
-            ExceptionCode ec;
-            for (unsigned n = 0; n < itemCount; ++n) {
-                RefPtr<SVGPathSeg> segment = m_animatedPath->getItem(n, ec);
-                valueToApply.append(segment->toString() + " ");
-            }
+            SVGPathParserFactory* factory = SVGPathParserFactory::self();
+            factory->buildStringFromByteStream(m_animatedPathPointer, valueToApply, UnalteredParsing);
         }
-    } else
+    } else if (m_propertyType == PointsProperty)
+        valueToApply = m_animatedPoints.isEmpty() ? m_animatedString : m_animatedPoints.valueAsString();
+    else
         valueToApply = m_animatedString;
     
     setTargetAttributeAnimatedValue(valueToApply);
