@@ -1,66 +1,65 @@
 /*
-    Copyright (C) 2004, 2005 Nikolas Zimmermann <wildfox@kde.org>
-                  2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
-    Copyright (C) 2007 Eric Seidel <eric@webkit.org>
-    Copyright (C) 2008 Apple Inc. All rights reserved.
-    Copyright (C) 2009 Cameron McCormack <cam@mcc.id.au>
-
-    This file is part of the KDE project
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
+ * Copyright (C) 2007 Eric Seidel <eric@webkit.org>
+ * Copyright (C) 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2009 Cameron McCormack <cam@mcc.id.au>
+ * Copyright (C) Research In Motion Limited 2010. All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #include "config.h"
 
 #if ENABLE(SVG_ANIMATION)
 #include "SVGAnimationElement.h"
 
+#include "Attribute.h"
 #include "CSSComputedStyleDeclaration.h"
 #include "CSSParser.h"
 #include "CSSPropertyNames.h"
+#include "Color.h"
 #include "Document.h"
 #include "Event.h"
 #include "EventListener.h"
 #include "FloatConversion.h"
 #include "HTMLNames.h"
-#include "MappedAttribute.h"
+#include "PlatformString.h"
+#include "RenderObject.h"
 #include "SVGElementInstance.h"
 #include "SVGNames.h"
+#include "SVGParserUtilities.h"
 #include "SVGURIReference.h"
 #include "SVGUseElement.h"
 #include "XLinkNames.h"
-#include <math.h>
 #include <wtf/StdLibExtras.h>
 
 using namespace std;
 
 namespace WebCore {
-    
-SVGAnimationElement::SVGAnimationElement(const QualifiedName& tagName, Document* doc)
-    : SVGSMILElement(tagName, doc)
-    , SVGTests()
-    , SVGExternalResourcesRequired()
+
+// Animated property definitions
+DEFINE_ANIMATED_BOOLEAN(SVGAnimationElement, SVGNames::externalResourcesRequiredAttr, ExternalResourcesRequired, externalResourcesRequired)
+
+SVGAnimationElement::SVGAnimationElement(const QualifiedName& tagName, Document* document)
+    : SVGSMILElement(tagName, document)
     , m_animationValid(false)
 {
 }
 
-SVGAnimationElement::~SVGAnimationElement()
-{
-}
-    
 static void parseKeyTimes(const String& parse, Vector<float>& result, bool verifyOrder)
 {
     result.clear();
@@ -89,32 +88,55 @@ fail:
 static void parseKeySplines(const String& parse, Vector<UnitBezier>& result)
 {
     result.clear();
-    Vector<String> parseList;
-    parse.split(';', parseList);
-    for (unsigned n = 0; n < parseList.size(); ++n) {
-        Vector<String> parseSpline;
-        parseList[n].split(',', parseSpline);
-        // The spec says the sepator is a space, all tests use commas. Weird.
-        if (parseSpline.size() == 1) 
-            parseList[n].split(' ', parseSpline);
-        if (parseSpline.size() != 4)
-            goto fail;
-        double curveValues[4];
-        for (unsigned i = 0; i < 4; ++i) {
-            String parseNumber = parseSpline[i]; 
-            bool ok;
-            curveValues[i] = parseNumber.toDouble(&ok);
-            if (!ok || curveValues[i] < 0.0 || curveValues[i] > 1.0)
-                goto fail;
+    if (parse.isEmpty())
+        return;
+    const UChar* cur = parse.characters();
+    const UChar* end = cur + parse.length();
+
+    skipOptionalSpaces(cur, end);
+
+    bool delimParsed = false;
+    while (cur < end) {
+        delimParsed = false;
+        float posA = 0.0f;
+        if (!parseNumber(cur, end, posA)) {
+            result.clear();
+            return;
         }
-        result.append(UnitBezier(curveValues[0], curveValues[1], curveValues[2], curveValues[3]));
+
+        float posB = 0.0f;
+        if (!parseNumber(cur, end, posB)) {
+            result.clear();
+            return;
+        }
+
+        float posC = 0.0f;
+        if (!parseNumber(cur, end, posC)) {
+            result.clear();
+            return;
+        }
+
+        float posD = 0.0f;
+        if (!parseNumber(cur, end, posD, false)) {
+            result.clear();
+            return;
+        }
+
+        skipOptionalSpaces(cur, end);
+
+        if (cur < end && *cur == ';') {
+            delimParsed = true;
+            cur++;
+        }
+        skipOptionalSpaces(cur, end);
+
+        result.append(UnitBezier(posA, posB, posC, posD));
     }
-    return;
-fail:
-    result.clear();
+    if (!(cur == end && !delimParsed))
+        result.clear();
 }
 
-void SVGAnimationElement::parseMappedAttribute(MappedAttribute* attr)
+void SVGAnimationElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == SVGNames::valuesAttr)
         attr->value().string().split(';', m_values);
@@ -142,6 +164,22 @@ void SVGAnimationElement::attributeChanged(Attribute* attr, bool preserveDecls)
     SVGSMILElement::attributeChanged(attr, preserveDecls);
 }
 
+void SVGAnimationElement::synchronizeProperty(const QualifiedName& attrName)
+{
+    SVGSMILElement::synchronizeProperty(attrName);
+
+    if (attrName == anyQName()) {
+        synchronizeExternalResourcesRequired();
+        SVGTests::synchronizeProperties(this, attrName);
+        return;
+    }
+
+    if (SVGExternalResourcesRequired::isKnownAttribute(attrName))
+        synchronizeExternalResourcesRequired();
+    else if (SVGTests::isKnownAttribute(attrName))
+        SVGTests::synchronizeProperties(this, attrName);
+}
+
 float SVGAnimationElement::getStartTime() const
 {
     return narrowPrecisionToFloat(intervalBegin().value());
@@ -310,9 +348,9 @@ void SVGAnimationElement::setTargetAttributeAnimatedValue(const String& value)
         static_cast<SVGStyledElement*>(target)->setInstanceUpdatesBlocked(false);
     
     // If the target element is used in an <use> instance tree, update that as well.
-    HashSet<SVGElementInstance*> instances = target->instancesForElement();
-    HashSet<SVGElementInstance*>::iterator end = instances.end();
-    for (HashSet<SVGElementInstance*>::iterator it = instances.begin(); it != end; ++it) {
+    const HashSet<SVGElementInstance*>& instances = target->instancesForElement();
+    const HashSet<SVGElementInstance*>::const_iterator end = instances.end();
+    for (HashSet<SVGElementInstance*>::const_iterator it = instances.begin(); it != end; ++it) {
         SVGElement* shadowTreeElement = (*it)->shadowTreeElement();
         ASSERT(shadowTreeElement);
         if (isCSS)
@@ -354,7 +392,18 @@ void SVGAnimationElement::calculateKeyTimesForCalcModePaced()
 }
 
 static inline double solveEpsilon(double duration) { return 1. / (200. * duration); }
-    
+
+unsigned SVGAnimationElement::calculateKeyTimesIndex(float percent) const
+{
+    unsigned index;
+    unsigned keyTimesCount = m_keyTimes.size();
+    for (index = 1; index < keyTimesCount; ++index) {
+        if (m_keyTimes[index] >= percent)
+            break;
+    }
+    return --index;
+}
+
 float SVGAnimationElement::calculatePercentForSpline(float percent, unsigned splineIndex) const
 {
     ASSERT(calcMode() == CalcModeSpline);
@@ -370,17 +419,10 @@ float SVGAnimationElement::calculatePercentFromKeyPoints(float percent) const
 {
     ASSERT(!m_keyPoints.isEmpty());
     ASSERT(calcMode() != CalcModePaced);
-    unsigned keyTimesCount = m_keyTimes.size();
-    ASSERT(keyTimesCount > 1);
-    ASSERT(m_keyPoints.size() == keyTimesCount);
-
-    unsigned index;
-    for (index = 1; index < keyTimesCount; ++index) {
-        if (m_keyTimes[index] >= percent)
-            break;
-    }
-    --index;
+    ASSERT(m_keyTimes.size() > 1);
+    ASSERT(m_keyPoints.size() == m_keyTimes.size());
 
+    unsigned index = calculateKeyTimesIndex(percent);
     float fromPercent = m_keyTimes[index];
     float toPercent = m_keyTimes[index + 1];
     float fromKeyPoint = m_keyPoints[index];
@@ -423,13 +465,7 @@ void SVGAnimationElement::currentValuesForValuesAnimation(float percent, float&
     ASSERT(!keyTimesCount || valuesCount == keyTimesCount);
     ASSERT(!keyTimesCount || (keyTimesCount > 1 && m_keyTimes[0] == 0));
 
-    unsigned index;
-    for (index = 1; index < keyTimesCount; ++index) {
-        if (m_keyTimes[index] >= percent)
-            break;
-    }
-    --index;
-    
+    unsigned index = calculateKeyTimesIndex(percent);
     if (calcMode == CalcModeDiscrete) {
         if (!keyTimesCount) 
             index = percent == 1.0f ? valuesCount - 1 : static_cast<unsigned>(percent * valuesCount);
@@ -462,6 +498,14 @@ void SVGAnimationElement::currentValuesForValuesAnimation(float percent, float&
         effectivePercent = calculatePercentForSpline(effectivePercent, index);
     }
 }
+static inline void adjustForCurrentColor(String& value, SVGElement* target)
+{
+    if (!target || !target->isStyled() || value != "currentColor")
+        return;
+
+    if (RenderObject* targetRenderer = target->renderer())
+        value = targetRenderer->style()->visitedDependentColor(CSSPropertyColor).name();
+}
     
 void SVGAnimationElement::startedActiveInterval()
 {
@@ -470,30 +514,51 @@ void SVGAnimationElement::startedActiveInterval()
     if (!hasValidTarget())
         return;
 
+    // These validations are appropriate for all animation modes.
+    if (hasAttribute(SVGNames::keyPointsAttr) && m_keyPoints.size() != m_keyTimes.size())
+        return;
+
     AnimationMode animationMode = this->animationMode();
+    CalcMode calcMode = this->calcMode();
+    if (calcMode == CalcModeSpline) {
+        unsigned splinesCount = m_keySplines.size() + 1;
+        if ((hasAttribute(SVGNames::keyPointsAttr) && m_keyPoints.size() != splinesCount)
+            || (animationMode == ValuesAnimation && m_values.size() != splinesCount))
+            return;
+    }
+
+    String from = fromValue();
+    String to = toValue();
+    String by = byValue();
+    SVGElement* target = targetElement();
     if (animationMode == NoAnimation)
         return;
-    if (animationMode == FromToAnimation)
-        m_animationValid = calculateFromAndToValues(fromValue(), toValue());
-    else if (animationMode == ToAnimation) {
+    if (animationMode == FromToAnimation) {
+        adjustForCurrentColor(from, target);
+        adjustForCurrentColor(to, target);
+        m_animationValid = calculateFromAndToValues(from, to);
+    } else if (animationMode == ToAnimation) {
         // For to-animations the from value is the current accumulated value from lower priority animations.
         // The value is not static and is determined during the animation.
-        m_animationValid = calculateFromAndToValues(String(), toValue());
-    } else if (animationMode == FromByAnimation)
-        m_animationValid = calculateFromAndByValues(fromValue(), byValue());
-    else if (animationMode == ByAnimation)
-        m_animationValid = calculateFromAndByValues(String(), byValue());
-    else if (animationMode == ValuesAnimation) {
-        CalcMode calcMode = this->calcMode();
+        adjustForCurrentColor(to, target);
+        m_animationValid = calculateFromAndToValues(String(), to);
+    } else if (animationMode == FromByAnimation) {
+        adjustForCurrentColor(from, target);
+        adjustForCurrentColor(by, target);
+        m_animationValid = calculateFromAndByValues(from, by);
+    } else if (animationMode == ByAnimation) {
+        adjustForCurrentColor(by, target);
+        m_animationValid = calculateFromAndByValues(String(), by);
+    } else if (animationMode == ValuesAnimation) {
         m_animationValid = m_values.size() > 1
             && (calcMode == CalcModePaced || !hasAttribute(SVGNames::keyTimesAttr) || hasAttribute(SVGNames::keyPointsAttr) || (m_values.size() == m_keyTimes.size()))
             && (calcMode == CalcModeDiscrete || !m_keyTimes.size() || m_keyTimes.last() == 1.0)
-            && (calcMode != CalcModeSpline || (m_keySplines.size() && (m_keySplines.size() == m_values.size() - 1) || m_keySplines.size() == m_keyPoints.size() - 1))
+            && (calcMode != CalcModeSpline || ((m_keySplines.size() && (m_keySplines.size() == m_values.size() - 1)) || m_keySplines.size() == m_keyPoints.size() - 1))
             && (!hasAttribute(SVGNames::keyPointsAttr) || (m_keyTimes.size() > 1 && m_keyTimes.size() == m_keyPoints.size()));
         if (calcMode == CalcModePaced && m_animationValid)
             calculateKeyTimesForCalcModePaced();
     } else if (animationMode == PathAnimation)
-        m_animationValid = calcMode() == CalcModePaced || !hasAttribute(SVGNames::keyPointsAttr) || (m_keyTimes.size() > 1 && m_keyTimes.size() == m_keyPoints.size());
+        m_animationValid = calcMode == CalcModePaced || !hasAttribute(SVGNames::keyPointsAttr) || (m_keyTimes.size() > 1 && m_keyTimes.size() == m_keyPoints.size());
 }
     
 void SVGAnimationElement::updateAnimation(float percent, unsigned repeat, SVGSMILElement* resultElement)
@@ -502,6 +567,7 @@ void SVGAnimationElement::updateAnimation(float percent, unsigned repeat, SVGSMI
         return;
     
     float effectivePercent;
+    CalcMode mode = calcMode();
     if (animationMode() == ValuesAnimation) {
         String from;
         String to;
@@ -513,9 +579,11 @@ void SVGAnimationElement::updateAnimation(float percent, unsigned repeat, SVGSMI
             m_lastValuesAnimationFrom = from;
             m_lastValuesAnimationTo = to;
         }
-    } else if (!m_keyPoints.isEmpty() && calcMode() != CalcModePaced)
+    } else if (!m_keyPoints.isEmpty() && mode != CalcModePaced)
         effectivePercent = calculatePercentFromKeyPoints(percent);
-    else 
+    else if (m_keyPoints.isEmpty() && mode == CalcModeSpline && m_keyTimes.size() > 1)
+        effectivePercent = calculatePercentForSpline(percent, calculateKeyTimesIndex(percent));
+    else
         effectivePercent = percent;
 
     calculateAnimatedValue(effectivePercent, repeat, resultElement);
