@@ -1,22 +1,22 @@
 /*
-    Copyright (C) 2004, 2005 Nikolas Zimmermann <wildfox@kde.org>
-                  2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #ifndef SVGColor_h
 #define SVGColor_h
@@ -45,8 +45,6 @@ namespace WebCore {
             return adoptRef(new SVGColor(SVG_COLORTYPE_CURRENTCOLOR));
         }
 
-        virtual ~SVGColor();
-
         enum SVGColorType {
             SVG_COLORTYPE_UNKNOWN                   = 0,
             SVG_COLORTYPE_RGBCOLOR                  = 1,
@@ -55,7 +53,7 @@ namespace WebCore {
         };
 
         // 'SVGColor' functions
-        unsigned short colorType() const;
+        unsigned short colorType() const { return m_colorType; }
 
         PassRefPtr<RGBColor> rgbColor() const;
 
@@ -66,16 +64,15 @@ namespace WebCore {
         void setRGBColorICCColor(const String& rgbColor, const String& iccColor, ExceptionCode&);
         void setColor(unsigned short colorType, const String& rgbColor, const String& iccColor, ExceptionCode&);
 
-        virtual String cssText() const;
-
-        // Helpers
-        const Color& color() const;
+        const Color& color() const { return m_color; }
 
     protected:
         SVGColor();
         SVGColor(const String& color);
         SVGColor(const Color&);
 
+        virtual String cssText() const;
+
     private:
         SVGColor(SVGColorType);
 
