@@ -1,31 +1,32 @@
 /*
-    Copyright (C) 2004, 2005, 2006, 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
-                  2004, 2005, 2006, 2008 Rob Buis <buis@kde.org>
-    Copyright (C) 2008 Apple Inc. All rights reserved.
-    Copyright (C) 2008 Alp Toker <alp@atoker.com>
-    Copyright (C) 2009 Cameron McCormack <cam@mcc.id.au>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2006, 2008 Rob Buis <buis@kde.org>
+ * Copyright (C) 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2008 Alp Toker <alp@atoker.com>
+ * Copyright (C) 2009 Cameron McCormack <cam@mcc.id.au>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #include "config.h"
 
 #if ENABLE(SVG)
 #include "SVGElement.h"
 
+#include "Attribute.h"
 #include "CSSCursorImageValue.h"
 #include "DOMImplementation.h"
 #include "Document.h"
@@ -34,15 +35,15 @@
 #include "EventNames.h"
 #include "FrameView.h"
 #include "HTMLNames.h"
-#include "MappedAttribute.h"
 #include "RegisteredEventListener.h"
 #include "RenderObject.h"
 #include "SVGCursorElement.h"
 #include "SVGElementInstance.h"
 #include "SVGElementRareData.h"
 #include "SVGNames.h"
-#include "SVGResource.h"
 #include "SVGSVGElement.h"
+#include "SVGStyledLocatableElement.h"
+#include "SVGTextElement.h"
 #include "SVGURIReference.h"
 #include "SVGUseElement.h"
 #include "ScriptEventListener.h"
@@ -53,13 +54,13 @@ namespace WebCore {
 using namespace HTMLNames;
 
 SVGElement::SVGElement(const QualifiedName& tagName, Document* document)
-    : StyledElement(tagName, document, CreateElementZeroRefCount)
+    : StyledElement(tagName, document, CreateSVGElement)
 {
 }
 
 PassRefPtr<SVGElement> SVGElement::create(const QualifiedName& tagName, Document* document)
 {
-    return new SVGElement(tagName, document);
+    return adoptRef(new SVGElement(tagName, document));
 }
 
 SVGElement::~SVGElement()
@@ -96,7 +97,7 @@ SVGElementRareData* SVGElement::ensureRareSVGData()
     ASSERT(!SVGElementRareData::rareDataMap().contains(this));
     SVGElementRareData* data = new SVGElementRareData;
     SVGElementRareData::rareDataMap().set(this, data);
-    m_hasRareSVGData = true;
+    setHasRareSVGData();
     return data;
 }
 
@@ -117,12 +118,12 @@ void SVGElement::setXmlbase(const String& value, ExceptionCode&)
 
 SVGSVGElement* SVGElement::ownerSVGElement() const
 {
-    Node* n = isShadowNode() ? const_cast<SVGElement*>(this)->shadowParentNode() : parentNode();
+    ContainerNode* n = parentOrHostNode();
     while (n) {
         if (n->hasTagName(SVGNames::svgTag))
             return static_cast<SVGSVGElement*>(n);
 
-        n = n->isShadowNode() ? n->shadowParentNode() : n->parentNode();
+        n = n->parentOrHostNode();
     }
 
     return 0;
@@ -132,12 +133,12 @@ SVGElement* SVGElement::viewportElement() const
 {
     // This function needs shadow tree support - as RenderSVGContainer uses this function
     // to determine the "overflow" property. <use> on <symbol> wouldn't work otherwhise.
-    Node* n = isShadowNode() ? const_cast<SVGElement*>(this)->shadowParentNode() : parentNode();
+    ContainerNode* n = parentOrHostNode();
     while (n) {
         if (n->hasTagName(SVGNames::svgTag) || n->hasTagName(SVGNames::imageTag) || n->hasTagName(SVGNames::symbolTag))
             return static_cast<SVGElement*>(n);
 
-        n = n->isShadowNode() ? n->shadowParentNode() : n->parentNode();
+        n = n->parentOrHostNode();
     }
 
     return 0;
@@ -180,6 +181,19 @@ const HashSet<SVGElementInstance*>& SVGElement::instancesForElement() const
     return rareSVGData()->elementInstances();
 }
 
+bool SVGElement::boundingBox(FloatRect& rect, SVGLocatable::StyleUpdateStrategy styleUpdateStrategy) const
+{
+    if (isStyledLocatable()) {
+        rect = static_cast<const SVGStyledLocatableElement*>(this)->getBBox(styleUpdateStrategy);
+        return true;
+    }
+    if (hasTagName(SVGNames::textTag)) {
+        rect = static_cast<const SVGTextElement*>(this)->getBBox(styleUpdateStrategy);
+        return true;
+    }
+    return false;
+}
+
 void SVGElement::setCursorElement(SVGCursorElement* cursorElement)
 {
     SVGElementRareData* rareData = ensureRareSVGData();
@@ -208,13 +222,13 @@ void SVGElement::setCursorImageValue(CSSCursorImageValue* cursorImageValue)
     rareData->setCursorImageValue(cursorImageValue);
 }
 
-void SVGElement::cursorImageValueRemoved()
+void SVGElement::cursorImageElementRemoved()
 {
     ASSERT(hasRareSVGData());
     rareSVGData()->setCursorImageValue(0);
- }
+}
 
-void SVGElement::parseMappedAttribute(MappedAttribute* attr)
+void SVGElement::parseMappedAttribute(Attribute* attr)
 {
     // standard events
     if (attr->name() == onloadAttr)
@@ -232,9 +246,9 @@ void SVGElement::parseMappedAttribute(MappedAttribute* attr)
     else if (attr->name() == onmouseupAttr)
         setAttributeEventListener(eventNames().mouseupEvent, createAttributeEventListener(this, attr));
     else if (attr->name() == SVGNames::onfocusinAttr)
-        setAttributeEventListener(eventNames().DOMFocusInEvent, createAttributeEventListener(this, attr));
+        setAttributeEventListener(eventNames().focusinEvent, createAttributeEventListener(this, attr));
     else if (attr->name() == SVGNames::onfocusoutAttr)
-        setAttributeEventListener(eventNames().DOMFocusOutEvent, createAttributeEventListener(this, attr));
+        setAttributeEventListener(eventNames().focusoutEvent, createAttributeEventListener(this, attr));
     else if (attr->name() == SVGNames::onactivateAttr)
         setAttributeEventListener(eventNames().DOMActivateEvent, createAttributeEventListener(this, attr));
     else
@@ -303,22 +317,22 @@ bool SVGElement::childShouldCreateRenderer(Node* child) const
 void SVGElement::insertedIntoDocument()
 {
     StyledElement::insertedIntoDocument();
+
+    if (!needsPendingResourceHandling())
+        return;
+
     SVGDocumentExtensions* extensions = document()->accessSVGExtensions();
+    String resourceId = getIdAttribute();
+    if (!extensions->isPendingResource(resourceId))
+        return;
+    
+    OwnPtr<SVGDocumentExtensions::SVGPendingElements> clients(extensions->removePendingResource(resourceId));
+    if (clients->isEmpty())
+        return;
 
-    String resourceId = SVGURIReference::getTarget(getAttribute(idAttributeName()));
-    if (extensions->isPendingResource(resourceId)) {
-        OwnPtr<HashSet<SVGStyledElement*> > clients(extensions->removePendingResource(resourceId));
-        if (clients->isEmpty())
-            return;
-
-        HashSet<SVGStyledElement*>::const_iterator it = clients->begin();
-        const HashSet<SVGStyledElement*>::const_iterator end = clients->end();
-
-        for (; it != end; ++it)
-            (*it)->buildPendingResource();
-
-        SVGResource::invalidateClients(*clients);
-    }
+    const SVGDocumentExtensions::SVGPendingElements::const_iterator end = clients->end();
+    for (SVGDocumentExtensions::SVGPendingElements::const_iterator it = clients->begin(); it != end; ++it)
+        (*it)->buildPendingResource();
 }
 
 void SVGElement::attributeChanged(Attribute* attr, bool preserveDecls)
@@ -334,35 +348,27 @@ void SVGElement::attributeChanged(Attribute* attr, bool preserveDecls)
     // SVGAnimatedPropertySynchronizer may call NamedNodeMap::removeAttribute(), which in turn calls attributeChanged().
     // At this point we're not allowed to call svgAttributeChanged() again - it may lead to extra work being done, or crashes
     // see bug https://bugs.webkit.org/show_bug.cgi?id=40994.
-    if (m_synchronizingSVGAttributes)
+    if (isSynchronizingSVGAttributes())
         return;
 
-    svgAttributeChanged(attr->name());
+    // Changes to the style attribute are processed lazily (see Element::getAttribute() and related methods),
+    // so we don't want changes to the style attribute to result in extra work here.
+    if (attr->name() != HTMLNames::styleAttr)
+        svgAttributeChanged(attr->name());
 }
 
 void SVGElement::updateAnimatedSVGAttribute(const QualifiedName& name) const
 {
-    ASSERT(!m_areSVGAttributesValid);
-
-    if (m_synchronizingSVGAttributes)
+    if (isSynchronizingSVGAttributes() || areSVGAttributesValid())
         return;
 
-    m_synchronizingSVGAttributes = true;
+    setIsSynchronizingSVGAttributes();
 
     const_cast<SVGElement*>(this)->synchronizeProperty(name);
     if (name == anyQName())
-        m_areSVGAttributesValid = true;
+        setAreSVGAttributesValid();
 
-    m_synchronizingSVGAttributes = false;
-}
-
-ContainerNode* SVGElement::eventParentNode()
-{
-    if (Node* shadowParent = shadowParentNode()) {
-        ASSERT(shadowParent->isContainerNode());
-        return static_cast<ContainerNode*>(shadowParent);
-    }
-    return StyledElement::eventParentNode();
+    clearIsSynchronizingSVGAttributes();
 }
 
 }
