@@ -1,29 +1,30 @@
 /*
-    Copyright (C) 2004, 2005, 2006, 2008 Nikolas Zimmermann <zimmermann@kde.org>
-                  2004, 2005, 2006 Rob Buis <buis@kde.org>
-    Copyright (C) 2009 Apple Inc. All rights reserved.
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005, 2006, 2008 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2006 Rob Buis <buis@kde.org>
+ * Copyright (C) 2009 Apple Inc. All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #ifndef SVGElement_h
 #define SVGElement_h
 
 #if ENABLE(SVG)
 #include "SVGDocumentExtensions.h"
+#include "SVGLocatable.h"
 #include "StyledElement.h"
 
 namespace WebCore {
@@ -49,8 +50,6 @@ namespace WebCore {
 
         SVGDocumentExtensions* accessDocumentSVGExtensions() const;
 
-        virtual void parseMappedAttribute(MappedAttribute*);
-
         virtual bool isStyled() const { return false; }
         virtual bool isStyledTransformable() const { return false; }
         virtual bool isStyledLocatable() const { return false; }
@@ -62,9 +61,6 @@ namespace WebCore {
         // For SVGTests
         virtual bool isValid() const { return true; }
 
-        virtual bool rendererIsNeeded(RenderStyle*) { return false; }
-        virtual bool childShouldCreateRenderer(Node*) const;
-
         virtual void svgAttributeChanged(const QualifiedName&) { }
         virtual void synchronizeProperty(const QualifiedName&) { }
 
@@ -72,22 +68,28 @@ namespace WebCore {
         
         virtual AffineTransform* supplementalTransform() { return 0; }
 
-        void setSynchronizedSVGAttributes(bool value) { m_areSVGAttributesValid = value; }
+        void invalidateSVGAttributes() { clearAreSVGAttributesValid(); }
 
         const HashSet<SVGElementInstance*>& instancesForElement() const;
 
+        bool boundingBox(FloatRect&, SVGLocatable::StyleUpdateStrategy = SVGLocatable::AllowStyleUpdate) const;
+
         void setCursorElement(SVGCursorElement*);
-        void cursorElementRemoved(); 
+        void cursorElementRemoved();
         void setCursorImageValue(CSSCursorImageValue*);
-        void cursorImageValueRemoved(); 
+        void cursorImageElementRemoved();
+
+        virtual void updateAnimatedSVGAttribute(const QualifiedName&) const;
 
     protected:
         SVGElement(const QualifiedName&, Document*);
 
+        virtual void parseMappedAttribute(Attribute*);
+
         virtual void finishParsingChildren();
         virtual void insertedIntoDocument();
         virtual void attributeChanged(Attribute*, bool preserveDecls = false);
-        virtual void updateAnimatedSVGAttribute(const QualifiedName&) const;
+        virtual bool childShouldCreateRenderer(Node*) const;
 
         SVGElementRareData* rareSVGData() const;
         SVGElementRareData* ensureRareSVGData();
@@ -95,10 +97,11 @@ namespace WebCore {
     private:
         friend class SVGElementInstance;
 
-        virtual bool isSVGElement() const { return true; }
+        virtual bool rendererIsNeeded(RenderStyle*) { return false; }
+
         virtual bool isSupported(StringImpl* feature, StringImpl* version) const;
 
-        virtual ContainerNode* eventParentNode();
+        virtual bool needsPendingResourceHandling() const { return true; }
         virtual void buildPendingResource() { }
 
         void mapInstanceToElement(SVGElementInstance*);
@@ -109,8 +112,5 @@ namespace WebCore {
 
 }
 
-// This file needs to be included after the SVGElement declaration
-#include "SVGAnimatedProperty.h"
-
 #endif
 #endif
