@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
  * Copyright (C) Research In Motion Limited 2010. All rights reserved.
  *
@@ -16,7 +16,6 @@
  * along with this library; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
- *
  */
 
 #include "config.h"
@@ -27,6 +26,7 @@
 #include "CSSFontSelector.h"
 #include "GraphicsContext.h"
 #include "RenderObject.h"
+#include "RenderSVGResourceSolidColor.h"
 #include "SimpleFontData.h"
 #include "SVGAltGlyphElement.h"
 #include "SVGFontData.h"
@@ -35,8 +35,7 @@
 #include "SVGFontElement.h"
 #include "SVGFontFaceElement.h"
 #include "SVGMissingGlyphElement.h"
-#include "SVGPaintServer.h"
-#include "SVGPaintServerSolid.h"
+#include "SVGNames.h"
 #include "XMLNames.h"
 
 using namespace WTF::Unicode;
@@ -184,8 +183,8 @@ static inline bool isCompatibleGlyph(const SVGGlyphIdentifier& identifier, bool
         // Split subcode from language, if existant.
         String languagePrefix;
 
-        int subCodeSeparator = language.find('-');
-        if (subCodeSeparator != -1)
+        size_t subCodeSeparator = language.find('-');
+        if (subCodeSeparator != notFound)
             languagePrefix = language.left(subCodeSeparator);
 
         Vector<String>::const_iterator it = identifier.languages.begin();
@@ -414,7 +413,7 @@ static float floatWidthOfSubStringUsingSVGFont(const Font* font, const TextRun&
         }
 
         SVGTextRunWalker<SVGTextRunWalkerMeasuredLengthData> runWalker(fontData, fontElement, data, floatWidthUsingSVGFontCallback, floatWidthMissingGlyphCallback);
-        runWalker.walk(run, isVerticalText, language, 0, run.length());
+        runWalker.walk(run, isVerticalText, language, from, to);
         charsConsumed = data.charsConsumed;
         glyphName = data.glyphName;
         return data.length;
@@ -471,20 +470,20 @@ void Font::drawTextUsingSVGFont(GraphicsContext* context, const TextRun& run,
         FloatPoint currentPoint = point;
         float scale = convertEmUnitToPixel(size(), fontFaceElement->unitsPerEm(), 1.0f);
 
-        SVGPaintServer* activePaintServer = run.activePaintServer();
+        RenderSVGResource* activePaintingResource = run.activePaintingResource();
 
         // If renderObject is not set, we're dealing for HTML text rendered using SVG Fonts.
         if (!run.referencingRenderObject()) {
-            ASSERT(!activePaintServer);
+            ASSERT(!activePaintingResource);
 
             // TODO: We're only supporting simple filled HTML text so far.
-            SVGPaintServerSolid* solidPaintServer = SVGPaintServer::sharedSolidPaintServer();
-            solidPaintServer->setColor(context->fillColor());
+            RenderSVGResourceSolidColor* solidPaintingResource = RenderSVGResource::sharedSolidPaintingResource();
+            solidPaintingResource->setColor(context->fillColor());
 
-            activePaintServer = solidPaintServer;
+            activePaintingResource = solidPaintingResource;
         }
 
-        ASSERT(activePaintServer);
+        ASSERT(activePaintingResource);
 
         int charsConsumed;
         String glyphName;
@@ -513,7 +512,7 @@ void Font::drawTextUsingSVGFont(GraphicsContext* context, const TextRun& run,
         SVGTextRunWalker<SVGTextRunWalkerDrawTextData> runWalker(fontData, fontElement, data, drawTextUsingSVGFontCallback, drawTextMissingGlyphCallback);
         runWalker.walk(run, isVerticalText, language, from, to);
 
-        SVGPaintTargetType targetType = context->textDrawingMode() == cTextStroke ? ApplyToStrokeTargetType : ApplyToFillTargetType;
+        RenderSVGResourceMode resourceMode = context->textDrawingMode() == TextModeStroke ? ApplyToStrokeMode : ApplyToFillMode;
 
         unsigned numGlyphs = data.glyphIdentifiers.size();
         unsigned fallbackCharacterIndex = 0;
@@ -536,12 +535,9 @@ void Font::drawTextUsingSVGFont(GraphicsContext* context, const TextRun& run,
                     Path glyphPath = identifier.pathData;
                     glyphPath.transform(glyphPathTransform);
 
-                    context->beginPath();
-                    context->addPath(glyphPath);
-                    if (activePaintServer->setup(context, run.referencingRenderObject(), targetType)) {
-                        activePaintServer->renderPath(context, run.referencingRenderObject(), targetType);
-                        activePaintServer->teardown(context, run.referencingRenderObject(), targetType);
-                    }
+                    RenderStyle* style = run.referencingRenderObject() ? run.referencingRenderObject()->style() : 0;
+                    if (activePaintingResource->applyResource(run.referencingRenderObject(), style, context, resourceMode))
+                        activePaintingResource->postApplyResource(run.referencingRenderObject(), context, resourceMode, &glyphPath);
 
                     context->restore();
                 }
@@ -572,7 +568,7 @@ void Font::drawTextUsingSVGFont(GraphicsContext* context, const TextRun& run,
     }
 }
 
-FloatRect Font::selectionRectForTextUsingSVGFont(const TextRun& run, const IntPoint& point, int height, int from, int to) const
+FloatRect Font::selectionRectForTextUsingSVGFont(const TextRun& run, const FloatPoint& point, int height, int from, int to) const
 {
     int charsConsumed;
     String glyphName;
@@ -581,7 +577,7 @@ FloatRect Font::selectionRectForTextUsingSVGFont(const TextRun& run, const IntPo
                      point.y(), floatWidthOfSubStringUsingSVGFont(this, run, 0, from, to, charsConsumed, glyphName), height);
 }
 
-int Font::offsetForPositionForTextUsingSVGFont(const TextRun&, int, bool) const
+int Font::offsetForPositionForTextUsingSVGFont(const TextRun&, float, bool) const
 {
     // TODO: Fix text selection when HTML text is drawn using a SVG Font
     // We need to integrate the SVG text selection code in the offsetForPosition() framework.
