@@ -1,22 +1,22 @@
 /*
-    Copyright (C) 2004, 2005, 2006, 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
-                  2004, 2005, 2007, 2008 Rob Buis <buis@kde.org>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2007, 2008 Rob Buis <buis@kde.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #include "config.h"
 
@@ -26,26 +26,29 @@
 #include "Attr.h"
 #include "CSSParser.h"
 #include "CSSStyleSelector.h"
-#include "CString.h"
 #include "Document.h"
 #include "HTMLNames.h"
-#include "MappedAttribute.h"
 #include "PlatformString.h"
 #include "RenderObject.h"
 #include "RenderSVGResource.h"
 #include "RenderSVGResourceClipper.h"
+#include "RenderSVGResourceFilter.h"
 #include "RenderSVGResourceMasker.h"
 #include "SVGElement.h"
 #include "SVGElementInstance.h"
 #include "SVGElementRareData.h"
 #include "SVGNames.h"
 #include "SVGRenderStyle.h"
-#include "SVGResourceFilter.h"
+#include "SVGRenderSupport.h"
 #include "SVGSVGElement.h"
+#include "SVGUseElement.h"
 #include <wtf/Assertions.h>
 
 namespace WebCore {
 
+// Animated property definitions
+DEFINE_ANIMATED_STRING(SVGStyledElement, HTMLNames::classAttr, ClassName, className)
+
 using namespace SVGNames;
 
 void mapAttributeToCSSProperty(HashMap<AtomicStringImpl*, int>* propertyNameToIdMap, const QualifiedName& attrName)
@@ -55,14 +58,60 @@ void mapAttributeToCSSProperty(HashMap<AtomicStringImpl*, int>* propertyNameToId
     propertyNameToIdMap->set(attrName.localName().impl(), propertyId);
 }
 
-SVGStyledElement::SVGStyledElement(const QualifiedName& tagName, Document* doc)
-    : SVGElement(tagName, doc)
+SVGStyledElement::SVGStyledElement(const QualifiedName& tagName, Document* document)
+    : SVGElement(tagName, document)
 {
 }
 
 SVGStyledElement::~SVGStyledElement()
 {
-    SVGResource::removeClient(this);
+}
+
+String SVGStyledElement::title() const
+{
+    // According to spec, we should not return titles when hovering over root <svg> elements (those
+    // <title> elements are the title of the document, not a tooltip) so we instantly return.
+    if (hasTagName(SVGNames::svgTag)) {
+        const SVGSVGElement* svg = static_cast<const SVGSVGElement*>(this);
+        if (svg->isOutermostSVG())
+            return String();
+    }
+    
+    // Walk up the tree, to find out whether we're inside a <use> shadow tree, to find the right title.
+    Node* parent = const_cast<SVGStyledElement*>(this);
+    while (parent) {
+        if (!parent->isShadowNode()) {
+            parent = parent->parentNodeGuaranteedHostFree();
+            continue;
+        }
+        
+        // Get the <use> element.
+        ContainerNode* shadowParent = parent->shadowParentNode();
+        if (shadowParent && shadowParent->isSVGElement() && shadowParent->hasTagName(SVGNames::useTag)) {
+            SVGUseElement* useElement = static_cast<SVGUseElement*>(shadowParent);
+            // If the <use> title is not empty we found the title to use.
+            String useTitle(useElement->title());
+            if (useTitle.isEmpty())
+                break;
+            return useTitle;
+        }
+        parent = parent->parentNode();
+    }
+    
+    // If we aren't an instance in a <use> or the <use> title was not found, then find the first
+    // <title> child of this element.
+    Element* titleElement = firstElementChild();
+    for (; titleElement; titleElement = titleElement->nextElementSibling()) {
+        if (titleElement->hasTagName(SVGNames::titleTag) && titleElement->isSVGElement())
+            break;
+    }
+
+    // If a title child was found, return the text contents.
+    if (titleElement)
+        return titleElement->innerText();
+    
+    // Otherwise return a null/empty string.
+    return String();
 }
 
 bool SVGStyledElement::rendererIsNeeded(RenderStyle* style)
@@ -142,6 +191,7 @@ int SVGStyledElement::cssPropertyIdForSVGAttributeName(const QualifiedName& attr
         mapAttributeToCSSProperty(propertyNameToIdMap, text_decorationAttr);
         mapAttributeToCSSProperty(propertyNameToIdMap, text_renderingAttr);
         mapAttributeToCSSProperty(propertyNameToIdMap, unicode_bidiAttr);
+        mapAttributeToCSSProperty(propertyNameToIdMap, vector_effectAttr);
         mapAttributeToCSSProperty(propertyNameToIdMap, visibilityAttr);
         mapAttributeToCSSProperty(propertyNameToIdMap, word_spacingAttr);
         mapAttributeToCSSProperty(propertyNameToIdMap, writing_modeAttr);
@@ -159,7 +209,7 @@ bool SVGStyledElement::mapToEntry(const QualifiedName& attrName, MappedAttribute
     return SVGElement::mapToEntry(attrName, result);
 }
 
-void SVGStyledElement::parseMappedAttribute(MappedAttribute* attr)
+void SVGStyledElement::parseMappedAttribute(Attribute* attr)
 {
     const QualifiedName& attrName = attr->name();
     // NOTE: Any subclass which overrides parseMappedAttribute for a property handled by
@@ -184,12 +234,7 @@ void SVGStyledElement::parseMappedAttribute(MappedAttribute* attr)
 
 bool SVGStyledElement::isKnownAttribute(const QualifiedName& attrName)
 {
-    // Recognize all style related SVG CSS properties
-    int propId = SVGStyledElement::cssPropertyIdForSVGAttributeName(attrName);
-    if (propId > 0)
-        return true;
-
-    return (attrName == idAttributeName() || attrName == HTMLNames::styleAttr); 
+    return isIdAttributeName(attrName);
 }
 
 void SVGStyledElement::svgAttributeChanged(const QualifiedName& attrName)
@@ -199,11 +244,13 @@ void SVGStyledElement::svgAttributeChanged(const QualifiedName& attrName)
     if (attrName.matches(HTMLNames::classAttr))
         classAttributeChanged(className());
 
-    // If we're the child of a resource element, be sure to invalidate it.
-    invalidateResourcesInAncestorChain();
+    RenderObject* object = renderer();
 
-    // If the element is using resources, invalidate them.
-    invalidateResources();
+    if (isIdAttributeName(attrName)) {
+        // Notify resources about id changes, this is important as we cache resources by id in SVGDocumentExtensions
+        if (object && object->isSVGResourceContainer())
+            object->toRenderSVGResourceContainer()->idChanged();
+    }
 
     // Invalidate all SVGElementInstances associated with us
     SVGElementInstance::invalidateAllInstancesOfElement(this);
@@ -217,58 +264,24 @@ void SVGStyledElement::synchronizeProperty(const QualifiedName& attrName)
         synchronizeClassName();
 }
 
-void SVGStyledElement::invalidateResources()
+void SVGStyledElement::attach()
 {
-    RenderObject* object = renderer();
-    if (!object)
-        return;
+    SVGElement::attach();
 
-    const SVGRenderStyle* svgStyle = object->style()->svgStyle();
-    Document* document = this->document();
-
-    if (document->parsing())
-        return;
-
-#if ENABLE(FILTERS)
-    SVGResourceFilter* filter = getFilterById(document, svgStyle->filter(), object);
-    if (filter)
-        filter->invalidate();
-#endif
-
-    if (RenderSVGResourceMasker* masker = getRenderSVGResourceById<RenderSVGResourceMasker>(document, svgStyle->maskElement()))
-        masker->invalidateClient(object);
-
-    if (RenderSVGResourceClipper* clipper = getRenderSVGResourceById<RenderSVGResourceClipper>(document, svgStyle->clipPath()))
-        clipper->invalidateClient(object);
+    if (RenderObject* object = renderer())
+        object->updateFromElement();
 }
 
-void SVGStyledElement::invalidateResourcesInAncestorChain() const
+void SVGStyledElement::insertedIntoDocument()
 {
-    Node* node = parentNode();
-    while (node) {
-        if (!node->isSVGElement())
-            break;
-
-        SVGElement* element = static_cast<SVGElement*>(node);
-        if (SVGStyledElement* styledElement = static_cast<SVGStyledElement*>(element->isStyled() ? element : 0))
-            styledElement->invalidateCanvasResources();
-
-        node = node->parentNode();
-    }
+    SVGElement::insertedIntoDocument();
+    updateRelativeLengthsInformation();
 }
 
-void SVGStyledElement::invalidateCanvasResources()
+void SVGStyledElement::removedFromDocument()
 {
-    RenderObject* object = renderer();
-    if (!object)
-        return;
-
-    if (object->isSVGResource())
-        object->toRenderSVGResource()->invalidateClients();
-
-    // The following lines will be removed soon, once all resources are handled by renderers.
-    if (SVGResource* resource = canvasResource(object))
-        resource->invalidate();
+    updateRelativeLengthsInformation(false, this);
+    SVGElement::removedFromDocument();
 }
 
 void SVGStyledElement::childrenChanged(bool changedByParser, Node* beforeChange, Node* afterChange, int childCountDelta)
@@ -276,7 +289,8 @@ void SVGStyledElement::childrenChanged(bool changedByParser, Node* beforeChange,
     SVGElement::childrenChanged(changedByParser, beforeChange, afterChange, childCountDelta);
 
     // Invalidate all SVGElementInstances associated with us
-    SVGElementInstance::invalidateAllInstancesOfElement(this);
+    if (!changedByParser)
+        SVGElementInstance::invalidateAllInstancesOfElement(this);
 }
 
 PassRefPtr<RenderStyle> SVGStyledElement::resolveStyle(RenderStyle* parentStyle)
@@ -288,15 +302,15 @@ PassRefPtr<RenderStyle> SVGStyledElement::resolveStyle(RenderStyle* parentStyle)
 
 PassRefPtr<CSSValue> SVGStyledElement::getPresentationAttribute(const String& name)
 {
-    if (!mappedAttributes())
+    if (!attributeMap())
         return 0;
 
     QualifiedName attributeName(nullAtom, name, nullAtom);
-    Attribute* attr = mappedAttributes()->getAttributeItem(attributeName);
+    Attribute* attr = attributeMap()->getAttributeItem(attributeName);
     if (!attr || !attr->isMappedAttribute() || !attr->style())
         return 0;
 
-    MappedAttribute* cssSVGAttr = static_cast<MappedAttribute*>(attr);
+    Attribute* cssSVGAttr = attr;
     // This function returns a pointer to a CSSValue which can be mutated from JavaScript.
     // If the associated MappedAttribute uses the same CSSMappedAttributeDeclaration
     // as StyledElement's mappedAttributeDecls cache, create a new CSSMappedAttributeDeclaration
@@ -311,12 +325,6 @@ PassRefPtr<CSSValue> SVGStyledElement::getPresentationAttribute(const String& na
     return cssSVGAttr->style()->getPropertyCSSValue(name);
 }
 
-void SVGStyledElement::detach()
-{
-    SVGResource::removeClient(this);
-    SVGElement::detach();
-}
-
 bool SVGStyledElement::instanceUpdatesBlocked() const
 {
     return hasRareSVGData() && rareSVGData()->instanceUpdatesBlocked();
@@ -328,6 +336,52 @@ void SVGStyledElement::setInstanceUpdatesBlocked(bool value)
         rareSVGData()->setInstanceUpdatesBlocked(value);
 }
 
+AffineTransform SVGStyledElement::localCoordinateSpaceTransform(SVGLocatable::CTMScope) const
+{
+    // To be overriden by SVGStyledLocatableElement/SVGStyledTransformableElement (or as special case SVGTextElement)
+    ASSERT_NOT_REACHED();
+    return AffineTransform();
+}
+
+void SVGStyledElement::updateRelativeLengthsInformation(bool hasRelativeLengths, SVGStyledElement* element)
+{
+    // If we're not yet in a document, this function will be called again from insertedIntoDocument(). Do nothing now.
+    if (!inDocument())
+        return;
+
+    // An element wants to notify us that its own relative lengths state changed.
+    // Register it in the relative length map, and register us in the parent relative length map.
+    // Register the parent in the grandparents map, etc. Repeat procedure until the root of the SVG tree.
+
+    if (hasRelativeLengths)
+        m_elementsWithRelativeLengths.add(element);
+    else {
+        if (!m_elementsWithRelativeLengths.contains(element)) {
+            // We were never registered. Do nothing.
+            return;
+        }
+
+        m_elementsWithRelativeLengths.remove(element);
+    }
+
+    // Find first styled parent node, and notify it that we've changed our relative length state.
+    ContainerNode* node = parentNode();
+    while (node) {
+        if (!node->isSVGElement())
+            break;
+
+        SVGElement* element = static_cast<SVGElement*>(node);
+        if (!element->isStyled()) {
+            node = node->parentNode();
+            continue;
+        }
+
+        // Register us in the parent element map.
+        static_cast<SVGStyledElement*>(element)->updateRelativeLengthsInformation(hasRelativeLengths, this);
+        break;
+    }
+}
+
 }
 
 #endif // ENABLE(SVG)
