@@ -1,32 +1,30 @@
 /*
-    Copyright (C) 2004, 2005, 2008 Nikolas Zimmermann <zimmermann@kde.org>
-                  2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
-                  2007 Eric Seidel <eric@webkit.org>
-
-    This file is part of the WebKit project
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005, 2008 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
+ * Copyright (C) 2007 Eric Seidel <eric@webkit.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #include "config.h"
 
 #if ENABLE(SVG)
 #include "SVGTransformable.h"
 
-#include "TransformationMatrix.h"
+#include "AffineTransform.h"
 #include "FloatConversion.h"
 #include "SVGNames.h"
 #include "SVGParserUtilities.h"
@@ -35,26 +33,6 @@
 
 namespace WebCore {
 
-SVGTransformable::SVGTransformable() : SVGLocatable()
-{
-}
-
-SVGTransformable::~SVGTransformable()
-{
-}
-
-TransformationMatrix SVGTransformable::getCTM(const SVGElement* element) const
-{
-    TransformationMatrix ctm = SVGLocatable::getCTM(element);
-    return animatedLocalTransform() * ctm;
-}
-
-TransformationMatrix SVGTransformable::getScreenCTM(const SVGElement* element) const
-{
-    TransformationMatrix ctm = SVGLocatable::getScreenCTM(element);
-    return animatedLocalTransform() * ctm;
-}
-
 static int parseTransformParamList(const UChar*& ptr, const UChar* end, float* values, int required, int optional)
 {
     int optionalParams = 0, requiredParams = 0;
@@ -111,7 +89,7 @@ static int parseTransformParamList(const UChar*& ptr, const UChar* end, float* v
 static const int requiredValuesForType[] =  {0, 6, 1, 1, 1, 1, 1};
 static const int optionalValuesForType[] =  {0, 0, 1, 1, 2, 0, 0};
 
-bool SVGTransformable::parseTransformValue(unsigned type, const UChar*& ptr, const UChar* end, SVGTransform& t)
+bool SVGTransformable::parseTransformValue(unsigned type, const UChar*& ptr, const UChar* end, SVGTransform& transform)
 {
     if (type == SVGTransform::SVG_TRANSFORM_UNKNOWN)
         return false;
@@ -122,33 +100,33 @@ bool SVGTransformable::parseTransformValue(unsigned type, const UChar*& ptr, con
         return false;
 
     switch (type) {
-        case SVGTransform::SVG_TRANSFORM_SKEWX:
-           t.setSkewX(values[0]);
-            break;
-        case SVGTransform::SVG_TRANSFORM_SKEWY:
-               t.setSkewY(values[0]);
-            break;
-        case SVGTransform::SVG_TRANSFORM_SCALE:
-              if (valueCount == 1) // Spec: if only one param given, assume uniform scaling
-                  t.setScale(values[0], values[0]);
-              else
-                  t.setScale(values[0], values[1]);
-            break;
-        case SVGTransform::SVG_TRANSFORM_TRANSLATE:
-              if (valueCount == 1) // Spec: if only one param given, assume 2nd param to be 0
-                  t.setTranslate(values[0], 0);
-              else
-                  t.setTranslate(values[0], values[1]);
-            break;
-        case SVGTransform::SVG_TRANSFORM_ROTATE:
-              if (valueCount == 1)
-                  t.setRotate(values[0], 0, 0);
-              else
-                  t.setRotate(values[0], values[1], values[2]);
-            break;
-        case SVGTransform::SVG_TRANSFORM_MATRIX:
-            t.setMatrix(TransformationMatrix(values[0], values[1], values[2], values[3], values[4], values[5]));
-            break;
+    case SVGTransform::SVG_TRANSFORM_SKEWX:
+        transform.setSkewX(values[0]);
+        break;
+    case SVGTransform::SVG_TRANSFORM_SKEWY:
+        transform.setSkewY(values[0]);
+        break;
+    case SVGTransform::SVG_TRANSFORM_SCALE:
+        if (valueCount == 1) // Spec: if only one param given, assume uniform scaling
+            transform.setScale(values[0], values[0]);
+        else
+            transform.setScale(values[0], values[1]);
+        break;
+    case SVGTransform::SVG_TRANSFORM_TRANSLATE:
+        if (valueCount == 1) // Spec: if only one param given, assume 2nd param to be 0
+            transform.setTranslate(values[0], 0);
+        else
+            transform.setTranslate(values[0], values[1]);
+        break;
+    case SVGTransform::SVG_TRANSFORM_ROTATE:
+        if (valueCount == 1)
+            transform.setRotate(values[0], 0, 0);
+        else
+            transform.setRotate(values[0], values[1], values[2]);
+        break;
+    case SVGTransform::SVG_TRANSFORM_MATRIX:
+        transform.setMatrix(AffineTransform(values[0], values[1], values[2], values[3], values[4], values[5]));
+        break;
     }
 
     return true;
@@ -165,41 +143,38 @@ static inline bool parseAndSkipType(const UChar*& currTransform, const UChar* en
 {
     if (currTransform >= end)
         return false;
-    
+
     if (*currTransform == 's') {
-        if (skipString(currTransform, end, skewXDesc, sizeof(skewXDesc) / sizeof(UChar)))
+        if (skipString(currTransform, end, skewXDesc, WTF_ARRAY_LENGTH(skewXDesc)))
             type = SVGTransform::SVG_TRANSFORM_SKEWX;
-        else if (skipString(currTransform, end, skewYDesc, sizeof(skewYDesc) / sizeof(UChar)))
+        else if (skipString(currTransform, end, skewYDesc, WTF_ARRAY_LENGTH(skewYDesc)))
             type = SVGTransform::SVG_TRANSFORM_SKEWY;
-        else if (skipString(currTransform, end, scaleDesc, sizeof(scaleDesc) / sizeof(UChar)))
+        else if (skipString(currTransform, end, scaleDesc, WTF_ARRAY_LENGTH(scaleDesc)))
             type = SVGTransform::SVG_TRANSFORM_SCALE;
         else
             return false;
-    } else if (skipString(currTransform, end, translateDesc, sizeof(translateDesc) / sizeof(UChar)))
+    } else if (skipString(currTransform, end, translateDesc, WTF_ARRAY_LENGTH(translateDesc)))
         type = SVGTransform::SVG_TRANSFORM_TRANSLATE;
-    else if (skipString(currTransform, end, rotateDesc, sizeof(rotateDesc) / sizeof(UChar)))
+    else if (skipString(currTransform, end, rotateDesc, WTF_ARRAY_LENGTH(rotateDesc)))
         type = SVGTransform::SVG_TRANSFORM_ROTATE;
-    else if (skipString(currTransform, end, matrixDesc, sizeof(matrixDesc) / sizeof(UChar)))
+    else if (skipString(currTransform, end, matrixDesc, WTF_ARRAY_LENGTH(matrixDesc)))
         type = SVGTransform::SVG_TRANSFORM_MATRIX;
-    else 
+    else
         return false;
-    
+
     return true;
 }
 
-bool SVGTransformable::parseTransformAttribute(SVGTransformList* list, const AtomicString& transform)
+bool SVGTransformable::parseTransformAttribute(SVGTransformList& list, const AtomicString& transform)
 {
     const UChar* start = transform.characters();
     return parseTransformAttribute(list, start, start + transform.length());
 }
 
-bool SVGTransformable::parseTransformAttribute(SVGTransformList* list, const UChar*& currTransform, const UChar* end, TransformParsingMode mode)
+bool SVGTransformable::parseTransformAttribute(SVGTransformList& list, const UChar*& currTransform, const UChar* end, TransformParsingMode mode)
 {
-    ExceptionCode ec = 0;
-    if (mode == ClearList) {
-        list->clear(ec);
-        ASSERT(!ec);
-    }
+    if (mode == ClearList)
+        list.clear();
 
     bool delimParsed = false;
     while (currTransform < end) {
@@ -210,11 +185,11 @@ bool SVGTransformable::parseTransformAttribute(SVGTransformList* list, const UCh
         if (!parseAndSkipType(currTransform, end, type))
             return false;
 
-        SVGTransform t;
-        if (!parseTransformValue(type, currTransform, end, t))
+        SVGTransform transform;
+        if (!parseTransformValue(type, currTransform, end, transform))
             return false;
 
-        list->appendItem(t, ec);
+        list.append(transform);
         skipOptionalSpaces(currTransform, end);
         if (currTransform < end && *currTransform == ',') {
             delimParsed = true;
