@@ -1,51 +1,53 @@
 /*
-    Copyright (C) 2004, 2005, 2006, 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
-                  2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
-    Copyright (C) Research In Motion Limited 2009-2010. All rights reserved.
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
-*/
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
+ * Copyright (C) Research In Motion Limited 2009-2010. All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #include "config.h"
 
 #if ENABLE(SVG)
 #include "SVGUseElement.h"
 
+#include "Attribute.h"
 #include "CSSStyleSelector.h"
-#include "CString.h"
 #include "Document.h"
 #include "Event.h"
 #include "EventListener.h"
 #include "HTMLNames.h"
-#include "MappedAttribute.h"
 #include "NodeRenderStyle.h"
 #include "RegisteredEventListener.h"
+#include "RenderSVGResource.h"
 #include "RenderSVGShadowTreeRootContainer.h"
 #include "SVGElementInstance.h"
 #include "SVGElementInstanceList.h"
 #include "SVGGElement.h"
-#include "SVGLength.h"
-#include "SVGPreserveAspectRatio.h"
-#include "SVGShadowTreeElements.h"
+#include "SVGNames.h"
 #include "SVGSMILElement.h"
 #include "SVGSVGElement.h"
+#include "SVGShadowTreeElements.h"
 #include "SVGSymbolElement.h"
 #include "XLinkNames.h"
+#include "XMLDocumentParser.h"
 #include "XMLSerializer.h"
 
+#include <wtf/text/StringConcatenate.h>
+
 // Dump SVGElementInstance object tree - useful to debug instanceRoot problems
 // #define DUMP_INSTANCE_TREE
 
@@ -54,23 +56,29 @@
 
 namespace WebCore {
 
-SVGUseElement::SVGUseElement(const QualifiedName& tagName, Document* doc)
-    : SVGStyledTransformableElement(tagName, doc)
-    , SVGTests()
-    , SVGLangSpace()
-    , SVGExternalResourcesRequired()
-    , SVGURIReference()
+// Animated property definitions
+DEFINE_ANIMATED_LENGTH(SVGUseElement, SVGNames::xAttr, X, x)
+DEFINE_ANIMATED_LENGTH(SVGUseElement, SVGNames::yAttr, Y, y)
+DEFINE_ANIMATED_LENGTH(SVGUseElement, SVGNames::widthAttr, Width, width)
+DEFINE_ANIMATED_LENGTH(SVGUseElement, SVGNames::heightAttr, Height, height)
+DEFINE_ANIMATED_STRING(SVGUseElement, XLinkNames::hrefAttr, Href, href)
+DEFINE_ANIMATED_BOOLEAN(SVGUseElement, SVGNames::externalResourcesRequiredAttr, ExternalResourcesRequired, externalResourcesRequired)
+
+inline SVGUseElement::SVGUseElement(const QualifiedName& tagName, Document* document)
+    : SVGStyledTransformableElement(tagName, document)
     , m_x(LengthModeWidth)
     , m_y(LengthModeHeight)
     , m_width(LengthModeWidth)
     , m_height(LengthModeHeight)
+    , m_updatesBlocked(false)
     , m_isPendingResource(false)
     , m_needsShadowTreeRecreation(false)
 {
 }
 
-SVGUseElement::~SVGUseElement()
+PassRefPtr<SVGUseElement> SVGUseElement::create(const QualifiedName& tagName, Document* document)
 {
+    return adoptRef(new SVGUseElement(tagName, document));
 }
 
 SVGElementInstance* SVGUseElement::instanceRoot() const
@@ -91,7 +99,7 @@ SVGElementInstance* SVGUseElement::animatedInstanceRoot() const
     return 0;
 }
  
-void SVGUseElement::parseMappedAttribute(MappedAttribute* attr)
+void SVGUseElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == SVGNames::xAttr)
         setXBaseValue(SVGLength(LengthModeWidth, attr->value()));
@@ -121,22 +129,32 @@ void SVGUseElement::parseMappedAttribute(MappedAttribute* attr)
 void SVGUseElement::insertedIntoDocument()
 {
     // This functions exists to assure assumptions made in the code regarding SVGElementInstance creation/destruction are satisfied.
-    SVGElement::insertedIntoDocument();
-    ASSERT(!m_targetElementInstance);
+    SVGStyledTransformableElement::insertedIntoDocument();
+    ASSERT(!m_targetElementInstance || ((document()->isSVGDocument() || document()->isXHTMLDocument()) && !static_cast<XMLDocumentParser*>(document()->parser())->wellFormed()));
     ASSERT(!m_isPendingResource);
 }
 
 void SVGUseElement::removedFromDocument()
 {
+    SVGStyledTransformableElement::removedFromDocument();
     m_targetElementInstance = 0;
-    SVGElement::removedFromDocument();
 }
 
 void SVGUseElement::svgAttributeChanged(const QualifiedName& attrName)
 {
     SVGStyledTransformableElement::svgAttributeChanged(attrName);
 
-    if (!renderer())
+    bool isXYAttribute = attrName == SVGNames::xAttr || attrName == SVGNames::yAttr;
+    bool isWidthHeightAttribute = attrName == SVGNames::widthAttr || attrName == SVGNames::heightAttr;
+
+    if (isXYAttribute || isWidthHeightAttribute)
+        updateRelativeLengthsInformation();
+
+    if (SVGTests::handleAttributeChange(this, attrName))
+        return;
+
+    RenderObject* object = renderer();
+    if (!object)
         return;
 
     if (SVGURIReference::isKnownAttribute(attrName)) {
@@ -150,12 +168,12 @@ void SVGUseElement::svgAttributeChanged(const QualifiedName& attrName)
         return;
     }
 
-    if (attrName == SVGNames::xAttr || attrName == SVGNames::yAttr) {
+    if (isXYAttribute) {
         updateContainerOffsets();
         return;
     }
 
-    if (attrName == SVGNames::widthAttr || attrName == SVGNames::heightAttr) {
+    if (isWidthHeightAttribute) {
         updateContainerSizes();
         return;
     }
@@ -166,12 +184,15 @@ void SVGUseElement::svgAttributeChanged(const QualifiedName& attrName)
         return;
     }
 
-    if (SVGTests::isKnownAttribute(attrName)
-        || SVGLangSpace::isKnownAttribute(attrName)
-        || SVGExternalResourcesRequired::isKnownAttribute(attrName)
-        || SVGStyledTransformableElement::isKnownAttribute(attrName)) {
-        invalidateShadowTree();
+    if (SVGStyledTransformableElement::isKnownAttribute(attrName)) {
+        object->setNeedsTransformUpdate();
+        RenderSVGResource::markForLayoutAndParentResourceInvalidation(object);
+        return;
     }
+
+    if (SVGLangSpace::isKnownAttribute(attrName)
+        || SVGExternalResourcesRequired::isKnownAttribute(attrName))
+        invalidateShadowTree();
 }
 
 void SVGUseElement::synchronizeProperty(const QualifiedName& attrName)
@@ -185,6 +206,7 @@ void SVGUseElement::synchronizeProperty(const QualifiedName& attrName)
         synchronizeHeight();
         synchronizeExternalResourcesRequired();
         synchronizeHref();
+        SVGTests::synchronizeProperties(this, attrName);
         return;
     }
 
@@ -200,6 +222,8 @@ void SVGUseElement::synchronizeProperty(const QualifiedName& attrName)
         synchronizeExternalResourcesRequired();
     else if (SVGURIReference::isKnownAttribute(attrName))
         synchronizeHref();
+    else if (SVGTests::isKnownAttribute(attrName))
+        SVGTests::synchronizeProperties(this, attrName);
 }
 
 static void updateContainerSize(SVGUseElement* useElement, SVGElementInstance* targetInstance)
@@ -246,8 +270,8 @@ void SVGUseElement::updateContainerSizes()
     // Update whole subtree, scanning for shadow container elements, that correspond to <svg>/<symbol> tags
     updateContainerSize(this, m_targetElementInstance.get());
 
-    if (renderer())
-        renderer()->setNeedsLayout(true);
+    if (RenderObject* object = renderer())
+        RenderSVGResource::markForLayoutAndParentResourceInvalidation(object);
 }
 
 static void updateContainerOffset(SVGElementInstance* targetInstance)
@@ -286,7 +310,7 @@ void SVGUseElement::updateContainerOffsets()
     SVGElement* shadowRoot = m_targetElementInstance->shadowTreeElement();
     ASSERT(shadowRoot);
 
-    Node* parentNode = shadowRoot->parentNode();
+    ContainerNode* parentNode = shadowRoot->parentNode();
     ASSERT(parentNode);
     ASSERT(parentNode->isSVGElement());
     ASSERT(parentNode->hasTagName(SVGNames::gTag));
@@ -298,29 +322,33 @@ void SVGUseElement::updateContainerOffsets()
     // Update whole subtree, scanning for shadow container elements, marking a cloned use subtree
     updateContainerOffset(m_targetElementInstance.get());
 
-    if (renderer())
-        renderer()->setNeedsLayout(true);
+    if (RenderObject* object = renderer())
+        RenderSVGResource::markForLayoutAndParentResourceInvalidation(object);
 }
 
 void SVGUseElement::recalcStyle(StyleChange change)
 {
     // Eventually mark shadow root element needing style recalc
-    if (needsStyleRecalc() && m_targetElementInstance) {
+    if (needsStyleRecalc() && m_targetElementInstance && !m_updatesBlocked) {
         if (SVGElement* shadowRoot = m_targetElementInstance->shadowTreeElement())
             shadowRoot->setNeedsStyleRecalc();
     }
 
     SVGStyledTransformableElement::recalcStyle(change);
 
-    bool needsStyleUpdate = !m_needsShadowTreeRecreation;
-    if (m_needsShadowTreeRecreation) {
-        static_cast<RenderSVGShadowTreeRootContainer*>(renderer())->markShadowTreeForRecreation();
-        m_needsShadowTreeRecreation = false;
-    }
+    // Assure that the shadow tree has not been marked for recreation, while we're building it.
+    if (m_updatesBlocked)
+        ASSERT(!m_needsShadowTreeRecreation);
 
     RenderSVGShadowTreeRootContainer* shadowRoot = static_cast<RenderSVGShadowTreeRootContainer*>(renderer());
     if (!shadowRoot)
         return;
+    
+    bool needsStyleUpdate = !m_needsShadowTreeRecreation;
+    if (m_needsShadowTreeRecreation) {
+        shadowRoot->markShadowTreeForRecreation();
+        m_needsShadowTreeRecreation = false;
+    }
 
     shadowRoot->updateFromElement();
 
@@ -339,7 +367,7 @@ void dumpInstanceTree(unsigned int& depth, String& text, SVGElementInstance* tar
     SVGElement* shadowTreeElement = targetInstance->shadowTreeElement();
     ASSERT(shadowTreeElement);
 
-    String elementId = element->getIDAttribute();
+    String elementId = element->getIdAttribute();
     String elementNodeName = element->nodeName();
     String shadowTreeElementNodeName = shadowTreeElement->nodeName();
     String parentNodeName = element->parentNode() ? element->parentNode()->nodeName() : "null";
@@ -356,7 +384,7 @@ void dumpInstanceTree(unsigned int& depth, String& text, SVGElementInstance* tar
         text += "  ";
 
     const HashSet<SVGElementInstance*>& elementInstances = element->instancesForElement();
-    text += String::format("Corresponding element is associated with %i instance(s):\n", elementInstances.size());
+    text += makeString("Corresponding element is associated with ", String::number(elementInstances.size()), " instance(s):\n");
 
     const HashSet<SVGElementInstance*>::const_iterator end = elementInstances.end();
     for (HashSet<SVGElementInstance*>::const_iterator it = elementInstances.begin(); it != end; ++it) {
@@ -432,6 +460,26 @@ void SVGUseElement::buildPendingResource()
 
 void SVGUseElement::buildShadowAndInstanceTree(SVGShadowTreeRootElement* shadowRoot)
 {
+    struct ShadowTreeUpdateBlocker {
+        ShadowTreeUpdateBlocker(SVGUseElement* currentUseElement)
+            : useElement(currentUseElement)
+        {
+            useElement->setUpdatesBlocked(true);
+        }
+
+        ~ShadowTreeUpdateBlocker()
+        {
+            useElement->setUpdatesBlocked(false);
+        }
+
+        SVGUseElement* useElement;
+    };
+
+    // When cloning the target nodes, they may decide to synchronize style and/or animated SVG attributes.
+    // That causes calls to SVGElementInstance::updateAllInstancesOfElement(), which mark the shadow tree for recreation.
+    // Solution: block any updates to the shadow tree while we're building it.
+    ShadowTreeUpdateBlocker blocker(this);
+
     String id = SVGURIReference::getTarget(href());
     Element* targetElement = document()->getElementById(id);
     if (!targetElement) {
@@ -443,12 +491,12 @@ void SVGUseElement::buildShadowAndInstanceTree(SVGShadowTreeRootElement* shadowR
 
     // Do not build the shadow/instance tree for <use> elements living in a shadow tree.
     // The will be expanded soon anyway - see expandUseElementsInShadowTree().
-    Node* parent = parentNode();
+    ContainerNode* parent = parentNode();
     while (parent) {
         if (parent->isShadowNode())
             return;
 
-        parent = parent->parentNode();
+        parent = parent->parentNodeGuaranteedHostFree();
     }
  
     SVGElement* target = 0;
@@ -549,6 +597,9 @@ void SVGUseElement::buildShadowAndInstanceTree(SVGShadowTreeRootElement* shadowR
     // Update container offset/size
     updateContainerOffsets();
     updateContainerSizes();
+
+    // Update relative length information
+    updateRelativeLengthsInformation();
 }
 
 RenderObject* SVGUseElement::createRenderer(RenderArena* arena, RenderStyle*)
@@ -572,8 +623,8 @@ void SVGUseElement::attach()
 
 void SVGUseElement::detach()
 {
-    m_targetElementInstance = 0;
     SVGStyledTransformableElement::detach();
+    m_targetElementInstance = 0;
 }
 
 static bool isDirectReference(Node* n)
@@ -587,25 +638,36 @@ static bool isDirectReference(Node* n)
            n->hasTagName(SVGNames::textTag);
 }
 
-Path SVGUseElement::toClipPath() const
+void SVGUseElement::toClipPath(Path& path) const
 {
+    ASSERT(path.isEmpty());
+
     Node* n = m_targetElementInstance ? m_targetElementInstance->shadowTreeElement() : 0;
     if (!n)
-        return Path();
+        return;
 
     if (n->isSVGElement() && static_cast<SVGElement*>(n)->isStyledTransformable()) {
         if (!isDirectReference(n))
             // Spec: Indirect references are an error (14.3.5)
             document()->accessSVGExtensions()->reportError("Not allowed to use indirect reference in <clip-path>");
         else {
-            Path clipPath = static_cast<SVGStyledTransformableElement*>(n)->toClipPath();
-            clipPath.translate(FloatSize(x().value(this), y().value(this)));
-            clipPath.transform(animatedLocalTransform());
-            return clipPath;
+            static_cast<SVGStyledTransformableElement*>(n)->toClipPath(path);
+            path.translate(FloatSize(x().value(this), y().value(this)));
+            path.transform(animatedLocalTransform());
         }
     }
+}
 
-    return Path();
+RenderObject* SVGUseElement::rendererClipChild() const
+{
+    Node* n = m_targetElementInstance ? m_targetElementInstance->shadowTreeElement() : 0;
+    if (!n)
+        return 0;
+
+    if (n->isSVGElement() && isDirectReference(n))
+        return static_cast<SVGElement*>(n)->renderer();
+
+    return 0;
 }
 
 void SVGUseElement::buildInstanceTree(SVGElement* target, SVGElementInstance* targetInstance, bool& foundProblem)
@@ -666,7 +728,8 @@ void SVGUseElement::handleDeepUseReferencing(SVGUseElement* use, SVGElementInsta
     while (instance) {
         SVGElement* element = instance->correspondingElement();
 
-        if (element->getIDAttribute() == id) {
+        // FIXME: This should probably be using getIdAttribute instead of idForStyleResolution.
+        if (element->hasID() && element->idForStyleResolution() == id) {
             foundProblem = true;
             return;
         }
@@ -692,7 +755,7 @@ void SVGUseElement::removeDisallowedElementsFromSubtree(Node* subtree)
         if (isDisallowedElement(node)) {
             Node* next = node->traverseNextSibling(subtree);
             // The subtree is not in document so this won't generate events that could mutate the tree.
-            node->parent()->removeChild(node, ec);
+            node->parentNode()->removeChild(node, ec);
             node = next;
         } else
             node = node->traverseNextNode(subtree);
@@ -746,7 +809,7 @@ void SVGUseElement::expandUseElementsInShadowTree(SVGShadowTreeRootElement* shad
 
         // Don't ASSERT(target) here, it may be "pending", too.
         // Setup sub-shadow tree root node
-        RefPtr<SVGShadowTreeContainerElement> cloneParent = new SVGShadowTreeContainerElement(document());
+        RefPtr<SVGShadowTreeContainerElement> cloneParent = SVGShadowTreeContainerElement::create(document());
         use->cloneChildNodes(cloneParent.get());
 
         // Spec: In the generated content, the 'use' will be replaced by 'g', where all attributes from the
@@ -797,7 +860,7 @@ void SVGUseElement::expandSymbolElementsInShadowTree(SVGShadowTreeRootElement* s
         // height are provided on the 'use' element, then these attributes will be transferred to
         // the generated 'svg'. If attributes width and/or height are not specified, the generated
         // 'svg' element will use values of 100% for these attributes.
-        RefPtr<SVGSVGElement> svgElement = new SVGSVGElement(SVGNames::svgTag, document());
+        RefPtr<SVGSVGElement> svgElement = SVGSVGElement::create(SVGNames::svgTag, document());
 
         // Transfer all attributes from <symbol> to the new <svg> element
         svgElement->attributes()->setAttributes(*element->attributes());
@@ -897,6 +960,9 @@ void SVGUseElement::associateInstancesWithShadowTreeElements(Node* target, SVGEl
         while (node && !node->isSVGElement())
            node = node->nextSibling();
 
+        if (!node)
+            break;
+
         associateInstancesWithShadowTreeElements(node, instance);
         node = node->nextSibling();
     }
@@ -935,6 +1001,10 @@ SVGElementInstance* SVGUseElement::instanceForShadowTreeElement(Node* element, S
 
 void SVGUseElement::invalidateShadowTree()
 {
+    // Don't mutate the shadow tree while we're building it.
+    if (m_updatesBlocked)
+        return;
+
     m_needsShadowTreeRecreation = true;
     setNeedsStyleRecalc();
 }
@@ -964,9 +1034,22 @@ void SVGUseElement::transferUseAttributesToReplacedElement(SVGElement* from, SVG
     ASSERT(!ec);
 }
 
-bool SVGUseElement::hasRelativeValues() const
+bool SVGUseElement::selfHasRelativeLengths() const
 {
-    return x().isRelative() || y().isRelative() || width().isRelative() || height().isRelative();
+    if (x().isRelative()
+     || y().isRelative()
+     || width().isRelative()
+     || height().isRelative())
+        return true;
+
+    if (!m_targetElementInstance)
+        return false;
+
+    SVGElement* element = m_targetElementInstance->correspondingElement();
+    if (!element || !element->isStyled())
+        return false;
+
+    return static_cast<SVGStyledElement*>(element)->hasRelativeLengths();
 }
 
 }
