@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2008 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,6 +28,7 @@
 #if ENABLE(SVG_ANIMATION)
 #include "SVGSMILElement.h"
 
+#include "Attribute.h"
 #include "CSSPropertyNames.h"
 #include "Document.h"
 #include "Event.h"
@@ -35,7 +36,6 @@
 #include "FloatConversion.h"
 #include "FrameView.h"
 #include "HTMLNames.h"
-#include "MappedAttribute.h"
 #include "SMILTimeContainer.h"
 #include "SVGNames.h"
 #include "SVGParserUtilities.h"
@@ -56,9 +56,9 @@ static const double invalidCachedTime = -1.;
     
 class ConditionEventListener : public EventListener {
 public:
-    static PassRefPtr<ConditionEventListener> create(SVGSMILElement* animation, Element* eventBase, SVGSMILElement::Condition* condition)
+    static PassRefPtr<ConditionEventListener> create(SVGSMILElement* animation, SVGSMILElement::Condition* condition)
     {
-        return adoptRef(new ConditionEventListener(animation, eventBase, condition));
+        return adoptRef(new ConditionEventListener(animation, condition));
     }
 
     static const ConditionEventListener* cast(const EventListener* listener)
@@ -69,42 +69,38 @@ public:
     }
 
     virtual bool operator==(const EventListener& other);
-
-    void unregister()
+    
+    void disconnectAnimation()
     {
-        // If this has only one ref then the event base is dead already and we don't need to remove ourself.
-        if (!hasOneRef())
-            m_eventBase->removeEventListener(m_condition->m_name, this, false);
+        m_animation = 0;
     }
 
 private:
-    ConditionEventListener(SVGSMILElement* animation, Element* eventBase, SVGSMILElement::Condition* condition) 
+    ConditionEventListener(SVGSMILElement* animation, SVGSMILElement::Condition* condition) 
         : EventListener(ConditionEventListenerType)
         , m_animation(animation)
         , m_condition(condition) 
-        , m_eventBase(eventBase)
     {
-        m_eventBase->addEventListener(m_condition->m_name, this, false);
     }
 
-    virtual void handleEvent(Event*, bool);
+    virtual void handleEvent(ScriptExecutionContext*, Event*);
 
     SVGSMILElement* m_animation;
     SVGSMILElement::Condition* m_condition;
-    Element* m_eventBase;
 };
 
 bool ConditionEventListener::operator==(const EventListener& listener)
 {
     if (const ConditionEventListener* conditionEventListener = ConditionEventListener::cast(&listener))
-        return m_animation == conditionEventListener->m_animation
-               && m_condition == conditionEventListener->m_condition
-               && m_eventBase == conditionEventListener->m_eventBase;
+        return m_animation == conditionEventListener->m_animation &&
+               m_condition == conditionEventListener->m_condition;
     return false;
 }
 
-void ConditionEventListener::handleEvent(Event* event, bool) 
+void ConditionEventListener::handleEvent(ScriptExecutionContext*, Event* event) 
 {
+    if (!m_animation)
+        return;
     m_animation->handleConditionEvent(event, m_condition);
 }
 
@@ -151,7 +147,7 @@ void SVGSMILElement::insertedIntoDocument()
     SVGElement::insertedIntoDocument();
 #ifndef NDEBUG
     // Verify we are not in <use> instance tree.
-    for (Node* n = this; n; n = n->parent())
+    for (ContainerNode* n = this; n; n = n->parentNode())
         ASSERT(!n->isShadowNode());
 #endif
     SVGSVGElement* owner = ownerSVGElement();
@@ -223,8 +219,8 @@ SMILTime SVGSMILElement::parseClockValue(const String& data)
 
     double result = 0;
     bool ok;
-    int doublePointOne = parse.find(':');
-    int doublePointTwo = parse.find(':', doublePointOne + 1);
+    size_t doublePointOne = parse.find(':');
+    size_t doublePointTwo = parse.find(':', doublePointOne + 1);
     if (doublePointOne == 2 && doublePointTwo == 5 && parse.length() >= 8) { 
         result += parse.substring(0, 2).toUIntStrict(&ok) * 60 * 60;
         if (!ok)
@@ -233,7 +229,7 @@ SMILTime SVGSMILElement::parseClockValue(const String& data)
         if (!ok)
             return SMILTime::unresolved();
         result += parse.substring(6).toDouble(&ok);
-    } else if (doublePointOne == 2 && doublePointTwo == -1 && parse.length() >= 5) { 
+    } else if (doublePointOne == 2 && doublePointTwo == notFound && parse.length() >= 5) { 
         result += parse.substring(0, 2).toUIntStrict(&ok) * 60;
         if (!ok)
             return SMILTime::unresolved();
@@ -257,15 +253,15 @@ bool SVGSMILElement::parseCondition(const String& value, BeginOrEnd beginOrEnd)
     
     double sign = 1.;
     bool ok;
-    int pos = parseString.find('+');
-    if (pos == -1) {
+    size_t pos = parseString.find('+');
+    if (pos == notFound) {
         pos = parseString.find('-');
-        if (pos != -1)
+        if (pos != notFound)
             sign = -1.;
     }
     String conditionString;
     SMILTime offset = 0;
-    if (pos == -1)
+    if (pos == notFound)
         conditionString = parseString;
     else {
         conditionString = parseString.left(pos).stripWhiteSpace();
@@ -281,7 +277,7 @@ bool SVGSMILElement::parseCondition(const String& value, BeginOrEnd beginOrEnd)
     
     String baseID;
     String nameString;
-    if (pos == -1)
+    if (pos == notFound)
         nameString = conditionString;
     else {
         baseID = conditionString.left(pos);
@@ -346,7 +342,7 @@ void SVGSMILElement::parseBeginOrEnd(const String& parseString, BeginOrEnd begin
     sortTimeList(timeList);
 }
 
-void SVGSMILElement::parseMappedAttribute(MappedAttribute* attr)
+void SVGSMILElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == SVGNames::beginAttr) {
         if (!m_conditions.isEmpty()) {
@@ -394,6 +390,11 @@ void SVGSMILElement::attributeChanged(Attribute* attr, bool preserveDecls)
     }
 }
 
+inline Element* SVGSMILElement::eventBaseFor(const Condition& condition) const
+{
+    return condition.m_baseID.isEmpty() ? targetElement() : document()->getElementById(condition.m_baseID);
+}
+
 void SVGSMILElement::connectConditions()
 {
     if (m_conditionsConnected)
@@ -403,11 +404,12 @@ void SVGSMILElement::connectConditions()
         Condition& condition = m_conditions[n];
         if (condition.m_type == Condition::EventBase) {
             ASSERT(!condition.m_syncbase);
-            Element* eventBase = condition.m_baseID.isEmpty() ? targetElement() : document()->getElementById(condition.m_baseID);
+            Element* eventBase = eventBaseFor(condition);
             if (!eventBase)
                 continue;
             ASSERT(!condition.m_eventListener);
-            condition.m_eventListener = ConditionEventListener::create(this, eventBase, &condition);
+            condition.m_eventListener = ConditionEventListener::create(this, &condition);
+            eventBase->addEventListener(condition.m_name, condition.m_eventListener, false);
         } else if (condition.m_type == Condition::Syncbase) {
             ASSERT(!condition.m_baseID.isEmpty());
             condition.m_syncbase = document()->getElementById(condition.m_baseID);
@@ -430,10 +432,18 @@ void SVGSMILElement::disconnectConditions()
         Condition& condition = m_conditions[n];
         if (condition.m_type == Condition::EventBase) {
             ASSERT(!condition.m_syncbase);
-            if (condition.m_eventListener) {
-                condition.m_eventListener->unregister();
-                condition.m_eventListener = 0;
-            }
+            if (!condition.m_eventListener)
+                continue;
+            // Note: It's a memory optimization to try to remove our condition
+            // event listener, but it's not guaranteed to work, since we have
+            // no guarantee that eventBaseFor() will be able to find our condition's
+            // original eventBase. So, we also have to disconnect ourselves from
+            // our condition event listener, in case it later fires.
+            Element* eventBase = eventBaseFor(condition);
+            if (eventBase)
+                eventBase->removeEventListener(condition.m_name, condition.m_eventListener.get(), false);
+            condition.m_eventListener->disconnectAnimation();
+            condition.m_eventListener = 0;
         } else if (condition.m_type == Condition::Syncbase) {
             if (condition.m_syncbase) {
                 ASSERT(isSMILElement(condition.m_syncbase.get()));
@@ -453,7 +463,7 @@ void SVGSMILElement::reschedule()
 SVGElement* SVGSMILElement::targetElement() const
 {
     String href = xlinkHref();
-    Node* target = href.isEmpty() ? parentNode() : document()->getElementById(SVGURIReference::getTarget(href));
+    ContainerNode* target = href.isEmpty() ? parentNode() : document()->getElementById(SVGURIReference::getTarget(href));
     if (target && target->isSVGElement())
         return static_cast<SVGElement*>(target);
     return 0;
@@ -518,7 +528,8 @@ SMILTime SVGSMILElement::repeatDur() const
         return m_cachedRepeatDur;
     const AtomicString& value = getAttribute(SVGNames::repeatDurAttr);
     SMILTime clockValue = parseClockValue(value);
-    return m_cachedRepeatDur = clockValue < 0 ? SMILTime::unresolved() : clockValue;
+    m_cachedRepeatDur = clockValue <= 0 ? SMILTime::unresolved() : clockValue;
+    return m_cachedRepeatDur;
 }
     
 // So a count is not really a time but let just all pretend we did not notice.
