@@ -34,18 +34,22 @@
 
 #include "WebSocketChannel.h"
 
-#include "CString.h"
 #include "CookieJar.h"
 #include "Document.h"
+#include "InspectorInstrumentation.h"
 #include "Logging.h"
+#include "Page.h"
 #include "PlatformString.h"
+#include "ProgressTracker.h"
 #include "ScriptExecutionContext.h"
 #include "SocketStreamError.h"
 #include "SocketStreamHandle.h"
-#include "StringHash.h"
 #include "WebSocketChannelClient.h"
 #include "WebSocketHandshake.h"
 
+#include <wtf/text/CString.h>
+#include <wtf/text/StringConcatenate.h>
+#include <wtf/text/StringHash.h>
 #include <wtf/Deque.h>
 #include <wtf/FastMalloc.h>
 #include <wtf/HashMap.h>
@@ -58,7 +62,19 @@ WebSocketChannel::WebSocketChannel(ScriptExecutionContext* context, WebSocketCha
     , m_handshake(url, protocol, context)
     , m_buffer(0)
     , m_bufferSize(0)
+    , m_resumeTimer(this, &WebSocketChannel::resumeTimerFired)
+    , m_suspended(false)
+    , m_closed(false)
+    , m_shouldDiscardReceivedData(false)
+    , m_unhandledBufferedAmount(0)
+    , m_identifier(0)
 {
+    if (m_context->isDocument())
+        if (Page* page = static_cast<Document*>(m_context)->page())
+            m_identifier = page->progress()->createUniqueIdentifier();
+
+    if (m_identifier)
+        InspectorInstrumentation::didCreateWebSocket(m_context, m_identifier, url, m_context->url());
 }
 
 WebSocketChannel::~WebSocketChannel()
@@ -70,6 +86,7 @@ void WebSocketChannel::connect()
 {
     LOG(Network, "WebSocketChannel %p connect", this);
     ASSERT(!m_handle);
+    ASSERT(!m_suspended);
     m_handshake.reset();
     ref();
     m_handle = SocketStreamHandle::create(m_handshake.url(), this);
@@ -79,9 +96,11 @@ bool WebSocketChannel::send(const String& msg)
 {
     LOG(Network, "WebSocketChannel %p send %s", this, msg.utf8().data());
     ASSERT(m_handle);
+    ASSERT(!m_suspended);
     Vector<char> buf;
     buf.append('\0');  // frame type
-    buf.append(msg.utf8().data(), msg.utf8().length());
+    CString utf8 = msg.utf8();
+    buf.append(utf8.data(), utf8.length());
     buf.append('\xff');  // frame end
     return m_handle->send(buf.data(), buf.size());
 }
@@ -90,12 +109,14 @@ unsigned long WebSocketChannel::bufferedAmount() const
 {
     LOG(Network, "WebSocketChannel %p bufferedAmount", this);
     ASSERT(m_handle);
+    ASSERT(!m_suspended);
     return m_handle->bufferedAmount();
 }
 
 void WebSocketChannel::close()
 {
     LOG(Network, "WebSocketChannel %p close", this);
+    ASSERT(!m_suspended);
     if (m_handle)
         m_handle->close();  // will call didClose()
 }
@@ -103,6 +124,8 @@ void WebSocketChannel::close()
 void WebSocketChannel::disconnect()
 {
     LOG(Network, "WebSocketChannel %p disconnect", this);
+    if (m_identifier && m_context)
+        InspectorInstrumentation::didCloseWebSocket(m_context, m_identifier);
     m_handshake.clearScriptExecutionContext();
     m_client = 0;
     m_context = 0;
@@ -110,15 +133,29 @@ void WebSocketChannel::disconnect()
         m_handle->close();
 }
 
+void WebSocketChannel::suspend()
+{
+    m_suspended = true;
+}
+
+void WebSocketChannel::resume()
+{
+    m_suspended = false;
+    if ((m_buffer || m_closed) && m_client && !m_resumeTimer.isActive())
+        m_resumeTimer.startOneShot(0);
+}
+
 void WebSocketChannel::didOpen(SocketStreamHandle* handle)
 {
     LOG(Network, "WebSocketChannel %p didOpen", this);
     ASSERT(handle == m_handle);
     if (!m_context)
         return;
-    const CString& handshakeMessage = m_handshake.clientHandshakeMessage();
+    if (m_identifier)
+        InspectorInstrumentation::willSendWebSocketHandshakeRequest(m_context, m_identifier, m_handshake.clientHandshakeRequest());
+    CString handshakeMessage = m_handshake.clientHandshakeMessage();
     if (!handle->send(handshakeMessage.data(), handshakeMessage.length())) {
-        m_context->addMessage(ConsoleDestination, JSMessageSource, LogMessageType, ErrorMessageLevel, "Error sending handshake message.", 0, m_handshake.clientOrigin());
+        m_context->addMessage(JSMessageSource, LogMessageType, ErrorMessageLevel, "Error sending handshake message.", 0, m_handshake.clientOrigin());
         handle->close();
     }
 }
@@ -126,15 +163,20 @@ void WebSocketChannel::didOpen(SocketStreamHandle* handle)
 void WebSocketChannel::didClose(SocketStreamHandle* handle)
 {
     LOG(Network, "WebSocketChannel %p didClose", this);
+    if (m_identifier && m_context)
+        InspectorInstrumentation::didCloseWebSocket(m_context, m_identifier);
     ASSERT_UNUSED(handle, handle == m_handle || !m_handle);
+    m_closed = true;
     if (m_handle) {
-        unsigned long unhandledBufferedAmount = m_handle->bufferedAmount();
+        m_unhandledBufferedAmount = m_handle->bufferedAmount();
+        if (m_suspended)
+            return;
         WebSocketChannelClient* client = m_client;
         m_client = 0;
         m_context = 0;
         m_handle = 0;
         if (client)
-            client->didClose(unhandledBufferedAmount);
+            client->didClose(m_unhandledBufferedAmount);
     }
     deref();
 }
@@ -148,100 +190,27 @@ void WebSocketChannel::didReceiveData(SocketStreamHandle* handle, const char* da
         return;
     }
     if (!m_client) {
+        m_shouldDiscardReceivedData = true;
         handle->close();
         return;
     }
+    if (m_shouldDiscardReceivedData)
+        return;
     if (!appendToBuffer(data, len)) {
+        m_shouldDiscardReceivedData = true;
         handle->close();
         return;
     }
-    if (m_handshake.mode() == WebSocketHandshake::Incomplete) {
-        int headerLength = m_handshake.readServerHandshake(m_buffer, m_bufferSize);
-        if (headerLength <= 0)
-            return;
-        switch (m_handshake.mode()) {
-        case WebSocketHandshake::Connected:
-            if (!m_handshake.serverSetCookie().isEmpty()) {
-                if (m_context->isDocument()) {
-                    Document* document = static_cast<Document*>(m_context);
-                    if (cookiesEnabled(document)) {
-                        ExceptionCode ec; // Exception (for sandboxed documents) ignored.
-                        document->setCookie(m_handshake.serverSetCookie(), ec);
-                    }
-                }
-            }
-            // FIXME: handle set-cookie2.
-            LOG(Network, "WebSocketChannel %p connected", this);
-            m_client->didConnect();
-            if (!m_client)
-                return;
+    while (!m_suspended && m_client && m_buffer)
+        if (!processBuffer())
             break;
-        default:
-            LOG(Network, "WebSocketChannel %p connection failed", this);
-            handle->close();
-            return;
-        }
-        skipBuffer(headerLength);
-        if (!m_buffer)
-            return;
-        LOG(Network, "remaining in read buf %ul", m_bufferSize);
-    }
-    if (m_handshake.mode() != WebSocketHandshake::Connected)
-        return;
-
-    const char* nextFrame = m_buffer;
-    const char* p = m_buffer;
-    const char* end = p + m_bufferSize;
-    while (p < end) {
-        unsigned char frameByte = static_cast<unsigned char>(*p++);
-        if ((frameByte & 0x80) == 0x80) {
-            int length = 0;
-            while (p < end) {
-                if (length > std::numeric_limits<int>::max() / 128) {
-                    LOG(Network, "frame length overflow %d", length);
-                    m_client->didReceiveMessageError();
-                    if (!m_client)
-                        return;
-                    handle->close();
-                    return;
-                }
-                char msgByte = *p;
-                length = length * 128 + (msgByte & 0x7f);
-                ++p;
-                if (!(msgByte & 0x80))
-                    break;
-            }
-            if (p + length < end) {
-                p += length;
-                nextFrame = p;
-                m_client->didReceiveMessageError();
-                if (!m_client)
-                    return;
-            } else
-                break;
-        } else {
-            const char* msgStart = p;
-            while (p < end && *p != '\xff')
-                ++p;
-            if (p < end && *p == '\xff') {
-                if (frameByte == 0x00)
-                    m_client->didReceiveMessage(String::fromUTF8(msgStart, p - msgStart));
-                else
-                    m_client->didReceiveMessageError();
-                if (!m_client)
-                    return;
-                ++p;
-                nextFrame = p;
-            }
-        }
-    }
-    skipBuffer(nextFrame - m_buffer);
 }
 
 void WebSocketChannel::didFail(SocketStreamHandle* handle, const SocketStreamError&)
 {
     LOG(Network, "WebSocketChannel %p didFail", this);
     ASSERT(handle == m_handle || !m_handle);
+    m_shouldDiscardReceivedData = true;
     handle->close();
 }
 
@@ -253,23 +222,28 @@ void WebSocketChannel::didCancelAuthenticationChallenge(SocketStreamHandle*, con
 {
 }
 
-bool WebSocketChannel::appendToBuffer(const char* data, int len)
+bool WebSocketChannel::appendToBuffer(const char* data, size_t len)
 {
+    size_t newBufferSize = m_bufferSize + len;
+    if (newBufferSize < m_bufferSize) {
+        LOG(Network, "WebSocket buffer overflow (%lu+%lu)", static_cast<unsigned long>(m_bufferSize), static_cast<unsigned long>(len));
+        return false;
+    }
     char* newBuffer = 0;
-    if (tryFastMalloc(m_bufferSize + len).getValue(newBuffer)) {
+    if (tryFastMalloc(newBufferSize).getValue(newBuffer)) {
         if (m_buffer)
             memcpy(newBuffer, m_buffer, m_bufferSize);
         memcpy(newBuffer + m_bufferSize, data, len);
         fastFree(m_buffer);
         m_buffer = newBuffer;
-        m_bufferSize += len;
+        m_bufferSize = newBufferSize;
         return true;
     }
-    m_context->addMessage(ConsoleDestination, JSMessageSource, LogMessageType, ErrorMessageLevel, String::format("WebSocket frame (at %d bytes) is too long.", m_bufferSize + len), 0, m_handshake.clientOrigin());
+    m_context->addMessage(JSMessageSource, LogMessageType, ErrorMessageLevel, makeString("WebSocket frame (at ", String::number(static_cast<unsigned long>(newBufferSize)), " bytes) is too long."), 0, m_handshake.clientOrigin());
     return false;
 }
 
-void WebSocketChannel::skipBuffer(int len)
+void WebSocketChannel::skipBuffer(size_t len)
 {
     ASSERT(len <= m_bufferSize);
     m_bufferSize -= len;
@@ -281,6 +255,138 @@ void WebSocketChannel::skipBuffer(int len)
     memmove(m_buffer, m_buffer + len, m_bufferSize);
 }
 
+bool WebSocketChannel::processBuffer()
+{
+    ASSERT(!m_suspended);
+    ASSERT(m_client);
+    ASSERT(m_buffer);
+    if (m_shouldDiscardReceivedData)
+        return false;
+
+    if (m_handshake.mode() == WebSocketHandshake::Incomplete) {
+        int headerLength = m_handshake.readServerHandshake(m_buffer, m_bufferSize);
+        if (headerLength <= 0)
+            return false;
+        if (m_handshake.mode() == WebSocketHandshake::Connected) {
+            if (m_identifier)
+                InspectorInstrumentation::didReceiveWebSocketHandshakeResponse(m_context, m_identifier, m_handshake.serverHandshakeResponse());
+            if (!m_handshake.serverSetCookie().isEmpty()) {
+                if (m_context->isDocument()) {
+                    Document* document = static_cast<Document*>(m_context);
+                    if (cookiesEnabled(document)) {
+                        ExceptionCode ec; // Exception (for sandboxed documents) ignored.
+                        document->setCookie(m_handshake.serverSetCookie(), ec);
+                    }
+                }
+            }
+            // FIXME: handle set-cookie2.
+            LOG(Network, "WebSocketChannel %p connected", this);
+            skipBuffer(headerLength);
+            m_client->didConnect();
+            LOG(Network, "remaining in read buf %lu", static_cast<unsigned long>(m_bufferSize));
+            return m_buffer;
+        }
+        LOG(Network, "WebSocketChannel %p connection failed", this);
+        skipBuffer(headerLength);
+        m_shouldDiscardReceivedData = true;
+        if (!m_closed)
+            m_handle->close();
+        return false;
+    }
+    if (m_handshake.mode() != WebSocketHandshake::Connected)
+        return false;
+
+    const char* nextFrame = m_buffer;
+    const char* p = m_buffer;
+    const char* end = p + m_bufferSize;
+
+    unsigned char frameByte = static_cast<unsigned char>(*p++);
+    if ((frameByte & 0x80) == 0x80) {
+        size_t length = 0;
+        bool errorFrame = false;
+        while (p < end) {
+            if (length > std::numeric_limits<size_t>::max() / 128) {
+                LOG(Network, "frame length overflow %lu", static_cast<unsigned long>(length));
+                errorFrame = true;
+                break;
+            }
+            size_t newLength = length * 128;
+            unsigned char msgByte = static_cast<unsigned char>(*p);
+            unsigned int lengthMsgByte = msgByte & 0x7f;
+            if (newLength > std::numeric_limits<size_t>::max() - lengthMsgByte) {
+                LOG(Network, "frame length overflow %lu+%u", static_cast<unsigned long>(newLength), lengthMsgByte);
+                errorFrame = true;
+                break;
+            }
+            newLength += lengthMsgByte;
+            if (newLength < length) { // sanity check
+                LOG(Network, "frame length integer wrap %lu->%lu", static_cast<unsigned long>(length), static_cast<unsigned long>(newLength));
+                errorFrame = true;
+                break;
+            }
+            length = newLength;
+            ++p;
+            if (!(msgByte & 0x80))
+                break;
+        }
+        if (p + length < p) {
+            LOG(Network, "frame buffer pointer wrap %p+%lu->%p", p, static_cast<unsigned long>(length), p + length);
+            errorFrame = true;
+        }
+        if (errorFrame) {
+            skipBuffer(m_bufferSize); // Save memory.
+            m_shouldDiscardReceivedData = true;
+            m_client->didReceiveMessageError();
+            if (!m_client)
+                return false;
+            if (!m_closed)
+                m_handle->close();
+            return false;
+        }
+        ASSERT(p + length >= p);
+        if (p + length < end) {
+            p += length;
+            nextFrame = p;
+            ASSERT(nextFrame > m_buffer);
+            skipBuffer(nextFrame - m_buffer);
+            m_client->didReceiveMessageError();
+            return m_buffer;
+        }
+        return false;
+    }
+
+    const char* msgStart = p;
+    while (p < end && *p != '\xff')
+        ++p;
+    if (p < end && *p == '\xff') {
+        int msgLength = p - msgStart;
+        ++p;
+        nextFrame = p;
+        if (frameByte == 0x00) {
+            String msg = String::fromUTF8(msgStart, msgLength);
+            skipBuffer(nextFrame - m_buffer);
+            m_client->didReceiveMessage(msg);
+        } else {
+            skipBuffer(nextFrame - m_buffer);
+            m_client->didReceiveMessageError();
+        }
+        return m_buffer;
+    }
+    return false;
+}
+
+void WebSocketChannel::resumeTimerFired(Timer<WebSocketChannel>* timer)
+{
+    ASSERT_UNUSED(timer, timer == &m_resumeTimer);
+
+    RefPtr<WebSocketChannel> protect(this); // The client can close the channel, potentially removing the last reference.
+    while (!m_suspended && m_client && m_buffer)
+        if (!processBuffer())
+            break;
+    if (!m_suspended && m_client && m_closed && m_handle)
+        didClose(m_handle.get());
+}
+
 }  // namespace WebCore
 
 #endif  // ENABLE(WEB_SOCKETS)
