@@ -23,12 +23,12 @@
 #if ENABLE(WML)
 #include "WMLInputElement.h"
 
+#include "Attribute.h"
 #include "EventNames.h"
 #include "FormDataList.h"
 #include "Frame.h"
 #include "HTMLNames.h"
 #include "KeyboardEvent.h"
-#include "MappedAttribute.h"
 #include "RenderTextControlSingleLine.h"
 #include "TextEvent.h"
 #include "WMLDocument.h"
@@ -45,6 +45,11 @@ WMLInputElement::WMLInputElement(const QualifiedName& tagName, Document* doc)
 {
 }
 
+PassRefPtr<WMLInputElement> WMLInputElement::create(const QualifiedName& tagName, Document* document)
+{
+    return adoptRef(new WMLInputElement(tagName, document));
+}
+
 WMLInputElement::~WMLInputElement()
 {
     if (m_isPasswordField)
@@ -123,6 +128,11 @@ const AtomicString& WMLInputElement::formControlName() const
     return m_data.name();
 }
 
+const String& WMLInputElement::suggestedValue() const
+{
+    return m_data.suggestedValue();
+}
+
 String WMLInputElement::value() const
 {
     String value = m_data.value();
@@ -132,9 +142,8 @@ String WMLInputElement::value() const
     return value;
 }
 
-void WMLInputElement::setValue(const String& value)
+void WMLInputElement::setValue(const String& value, bool sendChangeEvent)
 {
-    InputElement::updatePlaceholderVisibility(this, this);
     setFormControlValueMatchesRenderer(false);
     m_data.setValue(constrainValue(value));
     if (inDocument())
@@ -152,6 +161,13 @@ void WMLInputElement::setValue(const String& value)
     InputElement::notifyFormStateChanged(this);
 }
 
+void WMLInputElement::setValueForUser(const String& value)
+{
+    /* InputElement class defines pure virtual function 'setValueForUser', which 
+       will be useful only in HTMLInputElement. Do nothing in 'WMLInputElement'.
+     */
+}
+
 void WMLInputElement::setValueFromRenderer(const String& value)
 {
     InputElement::setValueFromRenderer(m_data, this, this, value);
@@ -175,7 +191,7 @@ void WMLInputElement::restoreFormControlState(const String& state)
 void WMLInputElement::select()
 {
     if (RenderTextControl* r = toRenderTextControl(renderer()))
-        r->select();
+        setSelectionRange(this, 0, r->text().length());
 }
 
 void WMLInputElement::accessKeyAction(bool)
@@ -184,7 +200,7 @@ void WMLInputElement::accessKeyAction(bool)
     focus(false);
 }
 
-void WMLInputElement::parseMappedAttribute(MappedAttribute* attr)
+void WMLInputElement::parseMappedAttribute(Attribute* attr)
 {
     if (attr->name() == HTMLNames::nameAttr)
         m_data.setName(parseValueForbiddingVariableReferences(attr->value()));
@@ -221,7 +237,7 @@ void WMLInputElement::copyNonAttributeProperties(const Element* source)
 
 RenderObject* WMLInputElement::createRenderer(RenderArena* arena, RenderStyle*)
 {
-    return new (arena) RenderTextControlSingleLine(this);
+    return new (arena) RenderTextControlSingleLine(this, false);
 }
 
 void WMLInputElement::detach()
@@ -258,7 +274,7 @@ void WMLInputElement::defaultEventHandler(Event* evt)
     }
 
     if (evt->type() == eventNames().keydownEvent && evt->isKeyboardEvent() && focused() && document()->frame()
-        && document()->frame()->doTextFieldCommandFromEvent(this, static_cast<KeyboardEvent*>(evt))) {
+        && document()->frame()->editor()->doTextFieldCommandFromEvent(this, static_cast<KeyboardEvent*>(evt))) {
         evt->setDefaultHandled();
         return;
     }
@@ -281,13 +297,13 @@ void WMLInputElement::defaultEventHandler(Event* evt)
     if (clickDefaultFormButton) {
         // Fire onChange for text fields.
         RenderObject* r = renderer();
-        if (r && toRenderTextControl(r)->isEdited()) {
-            dispatchEvent(eventNames().changeEvent, true, false);
+        if (r && toRenderTextControl(r)->wasChangedSinceLastChangeEvent()) {
+            dispatchEvent(Event::create(eventNames().changeEvent, true, false));
             
             // Refetch the renderer since arbitrary JS code run during onchange can do anything, including destroying it.
             r = renderer();
             if (r)
-                toRenderTextControl(r)->setEdited(false);
+                toRenderTextControl(r)->setChangedSinceLastChangeEvent(false);
         }
 
         evt->setDefaultHandled();
@@ -318,11 +334,6 @@ void WMLInputElement::documentDidBecomeActive()
     reset();
 }
 
-bool WMLInputElement::placeholderShouldBeVisible() const
-{
-    return InputElement::placeholderShouldBeVisible(this, this);
-}
-
 void WMLInputElement::willMoveToNewOwnerDocument()
 {
     // Always unregister for cache callbacks when leaving a document, even if we would otherwise like to be registered
