@@ -31,6 +31,8 @@
 #define DragClientGtk_h
 
 #include "DragClient.h"
+#include "GRefPtr.h"
+#include "RefPtrCairo.h"
 
 typedef struct _WebKitWebView WebKitWebView;
 
@@ -39,6 +41,7 @@ namespace WebKit {
     class DragClient : public WebCore::DragClient {
     public:
         DragClient(WebKitWebView*);
+        ~DragClient();
 
         virtual void willPerformDragDestinationAction(WebCore::DragDestinationAction, WebCore::DragData*);
         virtual void willPerformDragSourceAction(WebCore::DragSourceAction, const WebCore::IntPoint&, WebCore::Clipboard*);
@@ -47,13 +50,17 @@ namespace WebKit {
         virtual WebCore::DragSourceAction dragSourceActionMaskForPoint(const WebCore::IntPoint& windowPoint);
 
         virtual void startDrag(WebCore::DragImageRef dragImage, const WebCore::IntPoint& dragImageOrigin, const WebCore::IntPoint& eventPos, WebCore::Clipboard*, WebCore::Frame*, bool linkDrag = false);
-        virtual WebCore::DragImageRef createDragImageForLink(WebCore::KURL&, const WebCore::String& label, WebCore::Frame*);
+        virtual WebCore::DragImageRef createDragImageForLink(WebCore::KURL&, const WTF::String& label, WebCore::Frame*);
 
         virtual void dragControllerDestroyed();
 
+        void drawDragIconWindow(GtkWidget*, cairo_t*);
+
     private:
         WebKitWebView* m_webView;
         WebCore::IntPoint m_startPos;
+        PlatformRefPtr<GtkWidget> m_dragIconWindow;
+        RefPtr<cairo_surface_t> m_dragImage;
     };
 }
 
