@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2007 Alp Toker <alp@atoker.com>
+ * Copyright (C) 2008 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,11 +23,29 @@
 
 #include <webkit/webkitversion.h>
 #include <webkit/webkitdefines.h>
+#include <webkit/webkitdom.h>
+#include <webkit/webkitdownload.h>
+#include <webkit/webkitgeolocationpolicydecision.h>
+#include <webkit/webkithittestresult.h>
 #include <webkit/webkitnetworkrequest.h>
+#include <webkit/webkitnetworkresponse.h>
+#include <webkit/webkitsoupauthdialog.h>
+#include <webkit/webkitviewportattributes.h>
+#include <webkit/webkitwebdatasource.h>
 #include <webkit/webkitwebframe.h>
 #include <webkit/webkitwebsettings.h>
+#include <webkit/webkitwebinspector.h>
+#include <webkit/webkitwebwindowfeatures.h>
 #include <webkit/webkitwebview.h>
 #include <webkit/webkitwebbackforwardlist.h>
 #include <webkit/webkitwebhistoryitem.h>
+#include <webkit/webkitwebplugin.h>
+#include <webkit/webkitwebplugindatabase.h>
+#include <webkit/webkitwebpolicydecision.h>
+#include <webkit/webkitwebnavigationaction.h>
+#include <webkit/webkitwebresource.h>
+#include <webkit/webkitwebdatabase.h>
+#include <webkit/webkitsecurityorigin.h>
+#include <webkit/webkitenumtypes.h>
 
 #endif /* __WEBKIT_H__ */
