@@ -20,7 +20,6 @@
 
 #include "config.h"
 
-#include "CString.h"
 #include <glib/gi18n-lib.h>
 #include "GRefPtr.h"
 #include "Noncopyable.h"
@@ -30,10 +29,14 @@
 #include "ResourceRequest.h"
 #include "ResourceResponse.h"
 #include "webkitdownload.h"
+#include "webkitdownloadprivate.h"
 #include "webkitenumtypes.h"
 #include "webkitmarshal.h"
+#include "webkitnetworkrequestprivate.h"
 #include "webkitnetworkresponse.h"
+#include "webkitnetworkresponseprivate.h"
 #include "webkitprivate.h"
+#include <wtf/text/CString.h>
 
 #include <glib/gstdio.h>
 
@@ -60,7 +63,7 @@ class DownloadClient : public Noncopyable, public ResourceHandleClient {
 
         virtual void didReceiveResponse(ResourceHandle*, const ResourceResponse&);
         virtual void didReceiveData(ResourceHandle*, const char*, int, int);
-        virtual void didFinishLoading(ResourceHandle*);
+        virtual void didFinishLoading(ResourceHandle*, double);
         virtual void didFail(ResourceHandle*, const ResourceError&);
         virtual void wasBlocked(ResourceHandle*);
         virtual void cannotShowURL(ResourceHandle*);
@@ -69,8 +72,6 @@ class DownloadClient : public Noncopyable, public ResourceHandleClient {
         WebKitDownload* m_download;
 };
 
-#define WEBKIT_DOWNLOAD_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_DOWNLOAD, WebKitDownloadPrivate))
-
 struct _WebKitDownloadPrivate {
     gchar* destinationURI;
     gchar* suggestedFilename;
@@ -387,7 +388,7 @@ static void webkit_download_class_init(WebKitDownloadClass* downloadClass)
 
 static void webkit_download_init(WebKitDownload* download)
 {
-    WebKitDownloadPrivate* priv = WEBKIT_DOWNLOAD_GET_PRIVATE(download);
+    WebKitDownloadPrivate* priv = G_TYPE_INSTANCE_GET_PRIVATE(download, WEBKIT_TYPE_DOWNLOAD, WebKitDownloadPrivate);
     download->priv = priv;
 
     priv->downloadClient = new DownloadClient(download);
@@ -419,8 +420,8 @@ WebKitDownload* webkit_download_new_with_handle(WebKitNetworkRequest* request, W
     g_return_val_if_fail(request, NULL);
 
     ResourceHandleInternal* d = handle->getInternal();
-    if (d->m_msg)
-        soup_session_pause_message(webkit_get_default_session(), d->m_msg);
+    if (d->m_soupMessage)
+        soup_session_pause_message(webkit_get_default_session(), d->m_soupMessage.get());
 
     WebKitDownload* download = WEBKIT_DOWNLOAD(g_object_new(WEBKIT_TYPE_DOWNLOAD, "network-request", request, NULL));
     WebKitDownloadPrivate* priv = download->priv;
@@ -485,14 +486,15 @@ void webkit_download_start(WebKitDownload* download)
     g_return_if_fail(priv->status == WEBKIT_DOWNLOAD_STATUS_CREATED);
     g_return_if_fail(priv->timer == NULL);
 
+    // For GTK, when downloading a file NetworkingContext is null
     if (!priv->resourceHandle)
-        priv->resourceHandle = ResourceHandle::create(core(priv->networkRequest), priv->downloadClient, 0, false, false);
+        priv->resourceHandle = ResourceHandle::create(/* Null NetworkingContext */ NULL, core(priv->networkRequest), priv->downloadClient, false, false);
     else {
         priv->resourceHandle->setClient(priv->downloadClient);
 
         ResourceHandleInternal* d = priv->resourceHandle->getInternal();
-        if (d->m_msg)
-            soup_session_unpause_message(webkit_get_default_session(), d->m_msg);
+        if (d->m_soupMessage)
+            soup_session_unpause_message(webkit_get_default_session(), d->m_soupMessage.get());
     }
 
     priv->timer = g_timer_new();
@@ -558,7 +560,7 @@ const gchar* webkit_download_get_uri(WebKitDownload* download)
  * Retrieves the #WebKitNetworkRequest object that backs the download
  * process.
  *
- * Returns: the #WebKitNetworkRequest instance
+ * Returns: (transfer none): the #WebKitNetworkRequest instance
  *
  * Since: 1.1.2
  */
@@ -577,7 +579,7 @@ WebKitNetworkRequest* webkit_download_get_network_request(WebKitDownload* downlo
  * Retrieves the #WebKitNetworkResponse object that backs the download
  * process.
  *
- * Returns: the #WebKitNetworkResponse instance
+ * Returns: (transfer none): the #WebKitNetworkResponse instance
  *
  * Since: 1.1.16
  */
@@ -592,7 +594,7 @@ WebKitNetworkResponse* webkit_download_get_network_response(WebKitDownload* down
 static void webkit_download_set_response(WebKitDownload* download, const ResourceResponse& response)
 {
     WebKitDownloadPrivate* priv = download->priv;
-    priv->networkResponse = webkit_network_response_new_with_core_response(response);
+    priv->networkResponse = kitNew(response);
 
     if (!response.isNull() && !response.suggestedFilename().isEmpty())
         webkit_download_set_suggested_filename(download, response.suggestedFilename().utf8().data());
@@ -905,7 +907,7 @@ static void webkit_download_error(WebKitDownload* download, const ResourceError&
     webkit_download_close_stream(download);
 
     WebKitDownloadPrivate* priv = download->priv;
-    GRefPtr<WebKitDownload> protect(download);
+    PlatformRefPtr<WebKitDownload> protect(download);
 
     g_timer_stop(priv->timer);
     webkit_download_set_status(download, WEBKIT_DOWNLOAD_STATUS_ERROR);
@@ -929,7 +931,7 @@ void DownloadClient::didReceiveData(ResourceHandle*, const char* data, int lengt
     webkit_download_received_data(m_download, data, length);
 }
 
-void DownloadClient::didFinishLoading(ResourceHandle*)
+void DownloadClient::didFinishLoading(ResourceHandle*, double)
 {
     webkit_download_finished_loading(m_download);
 }
