@@ -19,16 +19,17 @@
  */
 
 #include "config.h"
-
 #include "webkitwebbackforwardlist.h"
-#include "webkitprivate.h"
-#include "webkitwebhistoryitem.h"
-#include "webkitwebview.h"
 
-#include <glib.h>
-
-#include "BackForwardList.h"
+#include "BackForwardListImpl.h"
 #include "HistoryItem.h"
+#include "webkitprivate.h"
+#include "webkitwebbackforwardlistprivate.h"
+#include "webkitwebhistoryitem.h"
+#include "webkitwebhistoryitemprivate.h"
+#include "webkitwebview.h"
+#include "webkitwebviewprivate.h"
+#include <glib.h>
 
 /**
  * SECTION:webkitwebbackforwardlist
@@ -53,18 +54,16 @@
 using namespace WebKit;
 
 struct _WebKitWebBackForwardListPrivate {
-    WebCore::BackForwardList* backForwardList;
+    WebCore::BackForwardListImpl* backForwardList;
     gboolean disposed;
 };
 
-#define WEBKIT_WEB_BACK_FORWARD_LIST_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_WEB_BACK_FORWARD_LIST, WebKitWebBackForwardListPrivate))
-
 G_DEFINE_TYPE(WebKitWebBackForwardList, webkit_web_back_forward_list, G_TYPE_OBJECT);
 
 static void webkit_web_back_forward_list_dispose(GObject* object)
 {
     WebKitWebBackForwardList* list = WEBKIT_WEB_BACK_FORWARD_LIST(object);
-    WebCore::BackForwardList* backForwardList = core(list);
+    WebCore::BackForwardListImpl* backForwardList = core(list);
     WebKitWebBackForwardListPrivate* priv = list->priv;
 
     if (!priv->disposed) {
@@ -92,16 +91,19 @@ static void webkit_web_back_forward_list_class_init(WebKitWebBackForwardListClas
 
 static void webkit_web_back_forward_list_init(WebKitWebBackForwardList* webBackForwardList)
 {
-    webBackForwardList->priv = WEBKIT_WEB_BACK_FORWARD_LIST_GET_PRIVATE(webBackForwardList);
+    webBackForwardList->priv = G_TYPE_INSTANCE_GET_PRIVATE(webBackForwardList, WEBKIT_TYPE_WEB_BACK_FORWARD_LIST, WebKitWebBackForwardListPrivate);
 }
 
 /**
- * webkit_web_back_forward_list_new_with_web_view:
+ * webkit_web_back_forward_list_new_with_web_view: (skip)
  * @web_view: the back forward list's #WebKitWebView
  *
  * Creates an instance of the back forward list with a controlling #WebKitWebView
  *
  * Return value: a #WebKitWebBackForwardList
+ *
+ * Deprecated: 1.3.4: Instances of #WebKitWebBackForwardList are
+ * created and owned by #WebKitWebView instances only.
  */
 WebKitWebBackForwardList* webkit_web_back_forward_list_new_with_web_view(WebKitWebView* webView)
 {
@@ -112,7 +114,7 @@ WebKitWebBackForwardList* webkit_web_back_forward_list_new_with_web_view(WebKitW
     webBackForwardList = WEBKIT_WEB_BACK_FORWARD_LIST(g_object_new(WEBKIT_TYPE_WEB_BACK_FORWARD_LIST, NULL));
     WebKitWebBackForwardListPrivate* priv = webBackForwardList->priv;
 
-    priv->backForwardList = core(webView)->backForwardList();
+    priv->backForwardList = static_cast<WebCore::BackForwardListImpl*>(core(webView)->backForwardList());
     priv->backForwardList->setEnabled(TRUE);
 
     return webBackForwardList;
@@ -128,7 +130,7 @@ void webkit_web_back_forward_list_go_forward(WebKitWebBackForwardList* webBackFo
 {
     g_return_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList));
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (backForwardList->enabled())
         backForwardList->goForward();
 }
@@ -143,7 +145,7 @@ void webkit_web_back_forward_list_go_back(WebKitWebBackForwardList* webBackForwa
 {
     g_return_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList));
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (backForwardList->enabled())
         backForwardList->goBack();
 }
@@ -151,11 +153,11 @@ void webkit_web_back_forward_list_go_back(WebKitWebBackForwardList* webBackForwa
 /**
  * webkit_web_back_forward_list_contains_item:
  * @web_back_forward_list: a #WebKitWebBackForwardList
- * @history_item: the #WebKitWebHistoryItem to check
+ * @history_item: (type WebKit.WebHistoryItem) (transfer none): the #WebKitWebHistoryItem to check
  *
  * Checks if @web_history_item is in the back forward list
  *
- * Return: %TRUE if @web_history_item is in the back forward list, %FALSE if it doesn't
+ * Return value: %TRUE if @web_history_item is in the back forward list, %FALSE if it doesn't
  */
 gboolean webkit_web_back_forward_list_contains_item(WebKitWebBackForwardList* webBackForwardList, WebKitWebHistoryItem* webHistoryItem)
 {
@@ -166,7 +168,7 @@ gboolean webkit_web_back_forward_list_contains_item(WebKitWebBackForwardList* we
 
     g_return_val_if_fail(historyItem != NULL, FALSE);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
 
     return (backForwardList->enabled() ? backForwardList->containsItem(historyItem) : FALSE);
 }
@@ -174,7 +176,7 @@ gboolean webkit_web_back_forward_list_contains_item(WebKitWebBackForwardList* we
 /**
  * webkit_web_back_forward_list_go_to_item:
  * @web_back_forward_list: a #WebKitWebBackForwardList
- * @history_item: the #WebKitWebHistoryItem to go to
+ * @history_item: (type WebKit.WebHistoryItem) (transfer none): the #WebKitWebHistoryItem to go to
  *
  * Go to the specified @web_history_item in the back forward list
  */
@@ -184,7 +186,7 @@ void webkit_web_back_forward_list_go_to_item(WebKitWebBackForwardList* webBackFo
     g_return_if_fail(WEBKIT_IS_WEB_HISTORY_ITEM(webHistoryItem));
 
     WebCore::HistoryItem* historyItem = core(webHistoryItem);
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
 
     if (backForwardList->enabled() && historyItem)
         backForwardList->goToItem(historyItem);
@@ -197,13 +199,13 @@ void webkit_web_back_forward_list_go_to_item(WebKitWebBackForwardList* webBackFo
  *
  * Returns a list of items that succeed the current item, limited by @limit
  *
- * Return value: a #GList of items succeeding the current item, limited by @limit
+ * Return value: (element-type WebKit.WebHistoryItem) (transfer container): a #GList of items succeeding the current item, limited by @limit
  */
 GList* webkit_web_back_forward_list_get_forward_list_with_limit(WebKitWebBackForwardList* webBackForwardList, gint limit)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), NULL);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return NULL;
 
@@ -227,13 +229,13 @@ GList* webkit_web_back_forward_list_get_forward_list_with_limit(WebKitWebBackFor
  *
  * Returns a list of items that precede the current item, limited by @limit
  *
- * Return value: a #GList of items preceding the current item, limited by @limit
+ * Return value: (element-type WebKit.WebHistoryItem) (transfer container): a #GList of items preceding the current item, limited by @limit
  */
 GList* webkit_web_back_forward_list_get_back_list_with_limit(WebKitWebBackForwardList* webBackForwardList, gint limit)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), NULL);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return NULL;
 
@@ -256,13 +258,13 @@ GList* webkit_web_back_forward_list_get_back_list_with_limit(WebKitWebBackForwar
  *
  * Returns the item that precedes the current item
  *
- * Return value: the #WebKitWebHistoryItem preceding the current item
+ * Return value: (type WebKit.WebHistoryItem) (transfer none): the #WebKitWebHistoryItem preceding the current item
  */
 WebKitWebHistoryItem* webkit_web_back_forward_list_get_back_item(WebKitWebBackForwardList* webBackForwardList)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), NULL);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return NULL;
 
@@ -279,13 +281,13 @@ WebKitWebHistoryItem* webkit_web_back_forward_list_get_back_item(WebKitWebBackFo
  *
  * Returns a NULL value if the back forward list is empty
  *
- * Return value: a #WebKitWebHistoryItem
+ * Return value: (type WebKit.WebHistoryItem) (transfer none): a #WebKitWebHistoryItem
  */
 WebKitWebHistoryItem* webkit_web_back_forward_list_get_current_item(WebKitWebBackForwardList* webBackForwardList)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), NULL);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return NULL;
 
@@ -302,13 +304,13 @@ WebKitWebHistoryItem* webkit_web_back_forward_list_get_current_item(WebKitWebBac
  *
  * Returns a NULL value if there nothing that succeeds the current item
  *
- * Return value: a #WebKitWebHistoryItem
+ * Return value: (type WebKit.WebHistoryItem) (transfer none): a #WebKitWebHistoryItem
  */
 WebKitWebHistoryItem* webkit_web_back_forward_list_get_forward_item(WebKitWebBackForwardList* webBackForwardList)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), NULL);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return NULL;
 
@@ -324,13 +326,13 @@ WebKitWebHistoryItem* webkit_web_back_forward_list_get_forward_item(WebKitWebBac
  *
  * Returns the item at a given index relative to the current item.
  *
- * Return value: the #WebKitWebHistoryItem located at the specified index relative to the current item
+ * Return value: (type WebKit.WebHistoryItem) (transfer none): the #WebKitWebHistoryItem located at the specified index relative to the current item
  */
 WebKitWebHistoryItem* webkit_web_back_forward_list_get_nth_item(WebKitWebBackForwardList* webBackForwardList, gint index)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), NULL);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList)
         return NULL;
 
@@ -351,7 +353,7 @@ gint webkit_web_back_forward_list_get_back_length(WebKitWebBackForwardList* webB
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), 0);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return 0;
 
@@ -370,7 +372,7 @@ gint webkit_web_back_forward_list_get_forward_length(WebKitWebBackForwardList* w
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), 0);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return 0;
 
@@ -389,7 +391,7 @@ gint webkit_web_back_forward_list_get_limit(WebKitWebBackForwardList* webBackFor
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), 0);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled())
         return 0;
 
@@ -409,7 +411,7 @@ void webkit_web_back_forward_list_set_limit(WebKitWebBackForwardList* webBackFor
 {
     g_return_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList));
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (backForwardList)
         backForwardList->setCapacity(limit);
 }
@@ -417,7 +419,7 @@ void webkit_web_back_forward_list_set_limit(WebKitWebBackForwardList* webBackFor
 /**
  * webkit_web_back_forward_list_add_item:
  * @web_back_forward_list: a #WebKitWebBackForwardList
- * @history_item: the #WebKitWebHistoryItem to add
+ * @history_item: (type WebKit.WebHistoryItem) (transfer none): the #WebKitWebHistoryItem to add
  *
  * Adds the item to the #WebKitWebBackForwardList.
  *
@@ -432,7 +434,7 @@ void webkit_web_back_forward_list_add_item(WebKitWebBackForwardList *webBackForw
 
     g_object_ref(webHistoryItem);
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     WebCore::HistoryItem* historyItem = core(webHistoryItem);
 
     backForwardList->addItem(historyItem);
@@ -440,18 +442,18 @@ void webkit_web_back_forward_list_add_item(WebKitWebBackForwardList *webBackForw
 
 /**
  * webkit_web_back_forward_list_clear:
- * @webBackForwardList:  a #WebKitWebBackForwardList
+ * @web_back_forward_list: the #WebKitWebBackForwardList to be cleared
  *
  * Clears the @webBackForwardList by removing all its elements. Note that not even
  * the current page is kept in list when cleared so you would have to add it later.
  *
- * Since: 1.1.30
+ * Since: 1.3.1
  **/
 void webkit_web_back_forward_list_clear(WebKitWebBackForwardList* webBackForwardList)
 {
     g_return_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList));
 
-    WebCore::BackForwardList* backForwardList = core(webBackForwardList);
+    WebCore::BackForwardListImpl* backForwardList = core(webBackForwardList);
     if (!backForwardList || !backForwardList->enabled() || !backForwardList->entries().size())
         return;
 
@@ -461,7 +463,7 @@ void webkit_web_back_forward_list_clear(WebKitWebBackForwardList* webBackForward
     backForwardList->setCapacity(capacity);
 }
 
-WebCore::BackForwardList* WebKit::core(WebKitWebBackForwardList* webBackForwardList)
+WebCore::BackForwardListImpl* WebKit::core(WebKitWebBackForwardList* webBackForwardList)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_BACK_FORWARD_LIST(webBackForwardList), NULL);
 
