@@ -20,13 +20,12 @@
 #include "config.h"
 #include "webkitwebdatabase.h"
 
-#include "webkitprivate.h"
-
-#include "CString.h"
 #include "DatabaseDetails.h"
 #include "DatabaseTracker.h"
-
+#include "webkitprivate.h"
+#include "webkitsecurityoriginprivate.h"
 #include <glib/gi18n-lib.h>
+#include <wtf/text/CString.h>
 
 /**
  * SECTION:webkitwebdatabase
@@ -78,8 +77,6 @@ struct _WebKitWebDatabasePrivate {
 static gchar* webkit_database_directory_path = NULL;
 static guint64 webkit_default_database_quota = 5 * 1024 * 1024;
 
-#define WEBKIT_WEB_DATABASE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_WEB_DATABASE, WebKitWebDatabasePrivate))
-
 static void webkit_web_database_set_security_origin(WebKitWebDatabase* webDatabase, WebKitSecurityOrigin* security_origin);
 
 static void webkit_web_database_set_name(WebKitWebDatabase* webDatabase, const gchar* name);
@@ -251,7 +248,7 @@ static void webkit_web_database_class_init(WebKitWebDatabaseClass* klass)
 
 static void webkit_web_database_init(WebKitWebDatabase* webDatabase)
 {
-    webDatabase->priv = WEBKIT_WEB_DATABASE_GET_PRIVATE(webDatabase);
+    webDatabase->priv = G_TYPE_INSTANCE_GET_PRIVATE(webDatabase, WEBKIT_TYPE_WEB_DATABASE, WebKitWebDatabasePrivate);
 }
 
 // Internal use only
@@ -280,11 +277,11 @@ static void webkit_web_database_set_name(WebKitWebDatabase* webDatabase, const g
 
 /**
  * webkit_web_database_get_security_origin:
- * @web_database: a #WebKitWebDatabase
+ * @webDatabase: a #WebKitWebDatabase
  *
  * Returns the security origin of the #WebKitWebDatabase.
  *
- * Returns: the security origin of the database
+ * Returns: (transfer none): the security origin of the database
  *
  * Since: 1.1.14
  **/
@@ -298,7 +295,7 @@ WebKitSecurityOrigin* webkit_web_database_get_security_origin(WebKitWebDatabase*
 
 /**
  * webkit_web_database_get_name:
- * @web_database: a #WebKitWebDatabase
+ * @webDatabase: a #WebKitWebDatabase
  *
  * Returns the canonical name of the #WebKitWebDatabase.
  *
@@ -316,7 +313,7 @@ G_CONST_RETURN gchar* webkit_web_database_get_name(WebKitWebDatabase* webDatabas
 
 /**
  * webkit_web_database_get_display_name:
- * @web_database: a #WebKitWebDatabase
+ * @webDatabase: a #WebKitWebDatabase
  *
  * Returns the name of the #WebKitWebDatabase as seen by the user.
  *
@@ -331,7 +328,7 @@ G_CONST_RETURN gchar* webkit_web_database_get_display_name(WebKitWebDatabase* we
 #if ENABLE(DATABASE)
     WebKitWebDatabasePrivate* priv = webDatabase->priv;
     WebCore::DatabaseDetails details = WebCore::DatabaseTracker::tracker().detailsForNameAndOrigin(priv->name, core(priv->origin));
-    WebCore::String displayName =  details.displayName();
+    WTF::String displayName =  details.displayName();
 
     if (displayName.isEmpty())
         return "";
@@ -346,7 +343,7 @@ G_CONST_RETURN gchar* webkit_web_database_get_display_name(WebKitWebDatabase* we
 
 /**
  * webkit_web_database_get_expected_size:
- * @web_database: a #WebKitWebDatabase
+ * @webDatabase: a #WebKitWebDatabase
  *
  * Returns the expected size of the #WebKitWebDatabase in bytes as defined by the
  * web author. The Web Database standard allows web authors to specify an expected
@@ -371,7 +368,7 @@ guint64 webkit_web_database_get_expected_size(WebKitWebDatabase* webDatabase)
 
 /**
  * webkit_web_database_get_size:
- * @web_database: a #WebKitWebDatabase
+ * @webDatabase: a #WebKitWebDatabase
  *
  * Returns the actual size of the #WebKitWebDatabase space on disk in bytes.
  *
@@ -394,7 +391,7 @@ guint64 webkit_web_database_get_size(WebKitWebDatabase* webDatabase)
 
 /**
  * webkit_web_database_get_filename:
- * @web_database: a #WebKitWebDatabase
+ * @webDatabase: a #WebKitWebDatabase
  *
  * Returns the absolute filename to the #WebKitWebDatabase file on disk.
  *
@@ -408,8 +405,8 @@ G_CONST_RETURN gchar* webkit_web_database_get_filename(WebKitWebDatabase* webDat
 
 #if ENABLE(DATABASE)
     WebKitWebDatabasePrivate* priv = webDatabase->priv;
-    WebCore::String coreName = WebCore::String::fromUTF8(priv->name);
-    WebCore::String corePath = WebCore::DatabaseTracker::tracker().fullPathForDatabase(core(priv->origin), coreName);
+    WTF::String coreName = WTF::String::fromUTF8(priv->name);
+    WTF::String corePath = WebCore::DatabaseTracker::tracker().fullPathForDatabase(core(priv->origin), coreName);
 
     if (corePath.isEmpty())
         return"";
@@ -425,7 +422,7 @@ G_CONST_RETURN gchar* webkit_web_database_get_filename(WebKitWebDatabase* webDat
 
 /**
  * webkit_web_database_remove:
- * @web_database: a #WebKitWebDatabase
+ * @webDatabase: a #WebKitWebDatabase
  *
  * Removes the #WebKitWebDatabase from its security origin and destroys all data
  * stored in the database.
@@ -470,7 +467,7 @@ void webkit_remove_all_web_databases()
 G_CONST_RETURN gchar* webkit_get_web_database_directory_path()
 {
 #if ENABLE(DATABASE)
-    WebCore::String path = WebCore::DatabaseTracker::tracker().databaseDirectoryPath();
+    WTF::String path = WebCore::DatabaseTracker::tracker().databaseDirectoryPath();
 
     if (path.isEmpty())
         return "";
@@ -495,7 +492,7 @@ G_CONST_RETURN gchar* webkit_get_web_database_directory_path()
 void webkit_set_web_database_directory_path(const gchar* path)
 {
 #if ENABLE(DATABASE)
-    WebCore::String corePath = WebCore::String::fromUTF8(path);
+    WTF::String corePath = WTF::String::fromUTF8(path);
     WebCore::DatabaseTracker::tracker().setDatabaseDirectoryPath(corePath);
 
     g_free(webkit_database_directory_path);
@@ -520,7 +517,7 @@ guint64 webkit_get_default_web_database_quota()
 
 /**
  * webkit_set_default_web_database_quota:
- * @default_quota: the new default database quota
+ * @defaultQuota: the new default database quota
  *
  * Sets the current path to the directory WebKit will write Web 
  * Database databases. 
