@@ -25,43 +25,49 @@
  */
 
 #include "config.h"
-
-#include "webkitenumtypes.h"
 #include "webkitwebframe.h"
-#include "webkitwebview.h"
-#include "webkitmarshal.h"
-#include "webkitprivate.h"
 
+#include "AXObjectCache.h"
 #include "AccessibilityObjectWrapperAtk.h"
 #include "AnimationController.h"
-#include "AXObjectCache.h"
-#include "CString.h"
+#include "DOMObjectCache.h"
 #include "DocumentLoader.h"
 #include "DocumentLoaderGtk.h"
 #include "FrameLoader.h"
 #include "FrameLoaderClientGtk.h"
 #include "FrameTree.h"
 #include "FrameView.h"
-#include <glib/gi18n-lib.h>
 #include "GCController.h"
 #include "GraphicsContext.h"
 #include "GtkVersioning.h"
 #include "HTMLFrameOwnerElement.h"
-#include "JSDOMWindow.h"
-#include "JSLock.h"
-#include "PrintContext.h"
-#include "RenderView.h"
-#include "RenderTreeAsText.h"
 #include "JSDOMBinding.h"
+#include "JSDOMWindow.h"
+#include "JSElement.h"
+#include "PrintContext.h"
+#include "RenderListItem.h"
+#include "RenderTreeAsText.h"
+#include "RenderView.h"
 #include "ScriptController.h"
 #include "SubstituteData.h"
+#include "webkitenumtypes.h"
+#include "webkitnetworkrequestprivate.h"
+#include "webkitnetworkresponseprivate.h"
+#include "webkitmarshal.h"
+#include "webkitprivate.h"
+#include "webkitsecurityoriginprivate.h"
+#include "webkitwebframeprivate.h"
+#include "webkitwebview.h"
+#include "webkitwebviewprivate.h"
+#include <JavaScriptCore/APICast.h>
+#include <atk/atk.h>
+#include <glib/gi18n-lib.h>
+#include <wtf/text/CString.h>
+
 #if ENABLE(SVG)
 #include "SVGSMILElement.h"
 #endif
 
-#include <atk/atk.h>
-#include <JavaScriptCore/APICast.h>
-
 /**
  * SECTION:webkitwebframe
  * @short_description: The content of a #WebKitWebView
@@ -144,7 +150,10 @@ static void webkit_web_frame_get_property(GObject* object, guint prop_id, GValue
 void webkit_web_frame_core_frame_gone(WebKitWebFrame* frame)
 {
     ASSERT(WEBKIT_IS_WEB_FRAME(frame));
-    frame->priv->coreFrame = 0;
+    WebKitWebFramePrivate* priv = frame->priv;
+    if (priv->coreFrame)
+        DOMObjectCache::clearByFrame(priv->coreFrame);
+    priv->coreFrame = 0;
 }
 
 static WebKitWebDataSource* webkit_web_frame_get_data_source_from_core_loader(WebCore::DocumentLoader* loader)
@@ -158,6 +167,7 @@ static void webkit_web_frame_finalize(GObject* object)
     WebKitWebFramePrivate* priv = frame->priv;
 
     if (priv->coreFrame) {
+        DOMObjectCache::clearByFrame(priv->coreFrame);
         priv->coreFrame->loader()->cancelAndClear();
         priv->coreFrame = 0;
     }
@@ -371,7 +381,7 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
 
 static void webkit_web_frame_init(WebKitWebFrame* frame)
 {
-    WebKitWebFramePrivate* priv = WEBKIT_WEB_FRAME_GET_PRIVATE(frame);
+    WebKitWebFramePrivate* priv = G_TYPE_INSTANCE_GET_PRIVATE(frame, WEBKIT_TYPE_WEB_FRAME, WebKitWebFramePrivate);
 
     // TODO: Move constructor code here.
     frame->priv = priv;
@@ -394,7 +404,7 @@ WebKitWebFrame* webkit_web_frame_new(WebKitWebView* webView)
 
     WebKitWebFrame* frame = WEBKIT_WEB_FRAME(g_object_new(WEBKIT_TYPE_WEB_FRAME, NULL));
     WebKitWebFramePrivate* priv = frame->priv;
-    WebKitWebViewPrivate* viewPriv = WEBKIT_WEB_VIEW_GET_PRIVATE(webView);
+    WebKitWebViewPrivate* viewPriv = webView->priv;
 
     priv->webView = webView;
     WebKit::FrameLoaderClient* client = new WebKit::FrameLoaderClient(frame);
@@ -406,21 +416,6 @@ WebKitWebFrame* webkit_web_frame_new(WebKitWebView* webView)
     return frame;
 }
 
-PassRefPtr<Frame> webkit_web_frame_init_with_web_view(WebKitWebView* webView, HTMLFrameOwnerElement* element)
-{
-    WebKitWebFrame* frame = WEBKIT_WEB_FRAME(g_object_new(WEBKIT_TYPE_WEB_FRAME, NULL));
-    WebKitWebFramePrivate* priv = frame->priv;
-    WebKitWebViewPrivate* viewPriv = WEBKIT_WEB_VIEW_GET_PRIVATE(webView);
-
-    priv->webView = webView;
-    WebKit::FrameLoaderClient* client = new WebKit::FrameLoaderClient(frame);
-
-    RefPtr<Frame> coreFrame = Frame::create(viewPriv->corePage, element, client);
-    priv->coreFrame = coreFrame.get();
-
-    return coreFrame.release();
-}
-
 /**
  * webkit_web_frame_get_title:
  * @frame: a #WebKitWebFrame
@@ -462,7 +457,7 @@ G_CONST_RETURN gchar* webkit_web_frame_get_uri(WebKitWebFrame* frame)
  * The #WebKitWebView returned manages the entire hierarchy of #WebKitWebFrame
  * objects that contains @frame.
  *
- * Return value: the #WebKitWebView that manages @frame
+ * Return value: (transfer none): the #WebKitWebView that manages @frame
  */
 WebKitWebView* webkit_web_frame_get_web_view(WebKitWebFrame* frame)
 {
@@ -493,7 +488,7 @@ G_CONST_RETURN gchar* webkit_web_frame_get_name(WebKitWebFrame* frame)
     if (!coreFrame)
         return "";
 
-    String string = coreFrame->tree()->name();
+    String string = coreFrame->tree()->uniqueName();
     priv->name = g_strdup(string.utf8().data());
     return priv->name;
 }
@@ -504,7 +499,7 @@ G_CONST_RETURN gchar* webkit_web_frame_get_name(WebKitWebFrame* frame)
  *
  * Returns the @frame's parent frame, or %NULL if it has none.
  *
- * Return value: the parent #WebKitWebFrame or %NULL in case there is none
+ * Return value: (transfer none): the parent #WebKitWebFrame or %NULL in case there is none
  */
 WebKitWebFrame* webkit_web_frame_get_parent(WebKitWebFrame* frame)
 {
@@ -675,7 +670,7 @@ void webkit_web_frame_reload(WebKitWebFrame* frame)
  * hierarchy, this function will search for a matching frame in other main
  * frame hierarchies. Returns %NULL if no match is found.
  *
- * Return value: the found #WebKitWebFrame or %NULL in case none is found
+ * Return value: (transfer none): the found #WebKitWebFrame or %NULL in case none is found
  */
 WebKitWebFrame* webkit_web_frame_find_frame(WebKitWebFrame* frame, const gchar* name)
 {
@@ -697,7 +692,7 @@ WebKitWebFrame* webkit_web_frame_find_frame(WebKitWebFrame* frame, const gchar*
  * Gets the global JavaScript execution context. Use this function to bridge
  * between the WebKit and JavaScriptCore APIs.
  *
- * Return value: the global JavaScript context
+ * Return value: (transfer none): the global JavaScript context
  */
 JSGlobalContextRef webkit_web_frame_get_global_context(WebKitWebFrame* frame)
 {
@@ -716,7 +711,7 @@ JSGlobalContextRef webkit_web_frame_get_global_context(WebKitWebFrame* frame)
  *
  * Returns the committed data source.
  *
- * Return value: the committed #WebKitWebDataSource.
+ * Return value: (transfer none): the committed #WebKitWebDataSource.
  *
  * Since: 1.1.14
  */
@@ -737,7 +732,7 @@ WebKitWebDataSource* webkit_web_frame_get_data_source(WebKitWebFrame* frame)
  * transition to a committed data source once any data has been received. Use
  * webkit_web_frame_get_data_source to get the committed data source.
  *
- * Return value: the provisional #WebKitWebDataSource or %NULL if a load
+ * Return value: (transfer none): the provisional #WebKitWebDataSource or %NULL if a load
  * request is not in progress.
  *
  * Since: 1.1.14
@@ -750,155 +745,6 @@ WebKitWebDataSource* webkit_web_frame_get_provisional_data_source(WebKitWebFrame
     return webkit_web_frame_get_data_source_from_core_loader(coreFrame->loader()->provisionalDocumentLoader());
 }
 
-/**
- * webkit_web_frame_get_children:
- * @frame: a #WebKitWebFrame
- *
- * Return value: child frames of @frame
- */
-GSList* webkit_web_frame_get_children(WebKitWebFrame* frame)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
-
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return NULL;
-
-    GSList* children = NULL;
-    for (Frame* child = coreFrame->tree()->firstChild(); child; child = child->tree()->nextSibling()) {
-        FrameLoader* loader = child->loader();
-        WebKit::FrameLoaderClient* client = static_cast<WebKit::FrameLoaderClient*>(loader->client());
-        if (client)
-          children = g_slist_append(children, client->webFrame());
-    }
-
-    return children;
-}
-
-/**
- * webkit_web_frame_get_inner_text:
- * @frame: a #WebKitWebFrame
- *
- * Return value: inner text of @frame
- */
-gchar* webkit_web_frame_get_inner_text(WebKitWebFrame* frame)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
-
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return g_strdup("");
-
-    FrameView* view = coreFrame->view();
-
-    if (view && view->layoutPending())
-        view->layout();
-
-    Element* documentElement = coreFrame->document()->documentElement();
-    String string =  documentElement->innerText();
-    return g_strdup(string.utf8().data());
-}
-
-/**
- * webkit_web_frame_dump_render_tree:
- * @frame: a #WebKitWebFrame
- *
- * Return value: Non-recursive render tree dump of @frame
- */
-gchar* webkit_web_frame_dump_render_tree(WebKitWebFrame* frame)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
-
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return g_strdup("");
-
-    FrameView* view = coreFrame->view();
-
-    if (view && view->layoutPending())
-        view->layout();
-
-    String string = externalRepresentation(coreFrame);
-    return g_strdup(string.utf8().data());
-}
-
-/**
- * webkit_web_frame_counter_value_for_element_by_id:
- * @frame: a #WebKitWebFrame
- * @id: an element ID string
- *
- * Return value: The counter value of element @id in @frame
- */
-gchar* webkit_web_frame_counter_value_for_element_by_id(WebKitWebFrame* frame, const gchar* id)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
-
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return 0;
-
-    Element* coreElement = coreFrame->document()->getElementById(AtomicString(id));
-    if (!coreElement)
-        return 0;
-    String counterValue = counterValueForElement(coreElement);
-    return g_strdup(counterValue.utf8().data());
-}
-
-/**
- * webkit_web_frame_page_number_for_element_by_id
- * @frame: a #WebKitWebFrame
- * @id: an element ID string
- * @pageWidth: width of a page
- * @pageHeight: height of a page
- *
- * Return value: The number of page where the specified element will be put
- */
-int webkit_web_frame_page_number_for_element_by_id(WebKitWebFrame* frame, const gchar* id, float pageWidth, float pageHeight)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
-
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return -1;
-
-    Element* coreElement = coreFrame->document()->getElementById(AtomicString(id));
-    if (!coreElement)
-        return -1;
-    return PrintContext::pageNumberForElement(coreElement, FloatSize(pageWidth, pageHeight));
-}
-
-/**
- * webkit_web_frame_number_of_pages
- * @frame: a #WebKitWebFrame
- * @pageWidth: width of a page
- * @pageHeight: height of a page
- *
- * Return value: The number of pages to be printed.
- */
-int webkit_web_frame_number_of_pages(WebKitWebFrame* frame, float pageWidth, float pageHeight)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
-
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return -1;
-
-    return PrintContext::numberOfPages(coreFrame, FloatSize(pageWidth, pageHeight));
-}
-
-/**
- * webkit_web_frame_get_pending_unload_event_count:
- * @frame: a #WebKitWebFrame
- *
- * Return value: number of pending unload events
- */
-guint webkit_web_frame_get_pending_unload_event_count(WebKitWebFrame* frame)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), 0);
-
-    return core(frame)->domWindow()->pendingUnloadEventListeners();
-}
-
 static void begin_print_callback(GtkPrintOperation* op, GtkPrintContext* context, gpointer user_data)
 {
     PrintContext* printContext = reinterpret_cast<PrintContext*>(user_data);
@@ -1009,53 +855,6 @@ void webkit_web_frame_print(WebKitWebFrame* frame)
     }
 }
 
-bool webkit_web_frame_pause_animation(WebKitWebFrame* frame, const gchar* name, double time, const gchar* element)
-{
-    ASSERT(core(frame));
-    Element* coreElement = core(frame)->document()->getElementById(AtomicString(element));
-    if (!coreElement || !coreElement->renderer())
-        return false;
-    return core(frame)->animation()->pauseAnimationAtTime(coreElement->renderer(), AtomicString(name), time);
-}
-
-bool webkit_web_frame_pause_transition(WebKitWebFrame* frame, const gchar* name, double time, const gchar* element)
-{
-    ASSERT(core(frame));
-    Element* coreElement = core(frame)->document()->getElementById(AtomicString(element));
-    if (!coreElement || !coreElement->renderer())
-        return false;
-    return core(frame)->animation()->pauseTransitionAtTime(coreElement->renderer(), AtomicString(name), time);
-}
-
-bool webkit_web_frame_pause_svg_animation(WebKitWebFrame* frame, const gchar* animationId, double time, const gchar* elementId)
-{
-    ASSERT(core(frame));
-#if ENABLE(SVG)
-    Document* document = core(frame)->document();
-    if (!document || !document->svgExtensions())
-        return false;
-    Element* coreElement = document->getElementById(AtomicString(animationId));
-    if (!coreElement || !SVGSMILElement::isSMILElement(coreElement))
-        return false;
-    return document->accessSVGExtensions()->sampleAnimationAtTime(elementId, static_cast<SVGSMILElement*>(coreElement), time);
-#else
-    return false;
-#endif
-}
-
-unsigned int webkit_web_frame_number_of_active_animations(WebKitWebFrame* frame)
-{
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return 0;
-
-    AnimationController* controller = coreFrame->animation();
-    if (!controller)
-        return 0;
-
-    return controller->numberOfActiveAnimations();
-}
-
 gchar* webkit_web_frame_get_response_mime_type(WebKitWebFrame* frame)
 {
     Frame* coreFrame = core(frame);
@@ -1080,56 +879,6 @@ WebKitLoadStatus webkit_web_frame_get_load_status(WebKitWebFrame* frame)
     return priv->loadStatus;
 }
 
-void webkit_web_frame_clear_main_frame_name(WebKitWebFrame* frame)
-{
-    g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
-
-    core(frame)->tree()->clearName();
-}
-
-void webkit_gc_collect_javascript_objects()
-{
-    gcController().garbageCollectNow();
-}
-
-void webkit_gc_collect_javascript_objects_on_alternate_thread(gboolean waitUntilDone)
-{
-    gcController().garbageCollectOnAlternateThreadForDebugging(waitUntilDone);
-}
-
-gsize webkit_gc_count_javascript_objects()
-{
-    JSC::JSLock lock(JSC::SilenceAssertionsOnly);
-    return JSDOMWindow::commonJSGlobalData()->heap.objectCount();
-
-}
-
-AtkObject* webkit_web_frame_get_focused_accessible_element(WebKitWebFrame* frame)
-{
-    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
-
-#if HAVE(ACCESSIBILITY)
-    if (!AXObjectCache::accessibilityEnabled())
-        AXObjectCache::enableAccessibility();
-
-    WebKitWebFramePrivate* priv = frame->priv;
-    if (!priv->coreFrame || !priv->coreFrame->document())
-        return NULL;
-
-    RenderView* root = toRenderView(priv->coreFrame->document()->renderer());
-    if (!root)
-        return NULL;
-
-    AtkObject* wrapper =  priv->coreFrame->document()->axObjectCache()->getOrCreate(root)->wrapper();
-    if (!wrapper)
-        return NULL;
-
-    return webkit_accessible_get_focused_element(WEBKIT_ACCESSIBLE(wrapper));
-#else
-    return NULL;
-#endif
-}
-
 GtkPolicyType webkit_web_frame_get_horizontal_scrollbar_policy(WebKitWebFrame* frame)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), GTK_POLICY_AUTOMATIC);
@@ -1176,7 +925,7 @@ GtkPolicyType webkit_web_frame_get_vertical_scrollbar_policy(WebKitWebFrame* fra
  *
  * Returns the @frame's security origin.
  *
- * Return value: the security origin of @frame
+ * Return value: (transfer none): the security origin of @frame
  *
  * Since: 1.1.14
  */
@@ -1196,19 +945,6 @@ WebKitSecurityOrigin* webkit_web_frame_get_security_origin(WebKitWebFrame* frame
     return priv->origin;
 }
 
-void webkit_web_frame_layout(WebKitWebFrame* frame)
-{
-    Frame* coreFrame = core(frame);
-    if (!coreFrame)
-        return;
-
-    FrameView* view = coreFrame->view();
-    if (!view)
-        return;
-
-    view->layout();
-}
-
 /**
  * webkit_web_frame_get_network_response:
  * @frame: a #WebKitWebFrame
@@ -1218,7 +954,7 @@ void webkit_web_frame_layout(WebKitWebFrame* frame)
  * frame was not created by a load. You must unref the object when you
  * are done with it.
  *
- * Return value: a #WebKitNetworkResponse object
+ * Return value: (transfer full): a #WebKitNetworkResponse object
  *
  * Since: 1.1.18
  */
@@ -1232,5 +968,34 @@ WebKitNetworkResponse* webkit_web_frame_get_network_response(WebKitWebFrame* fra
     if (!loader)
         return NULL;
 
-    return webkit_network_response_new_with_core_response(loader->response());
+    return kitNew(loader->response());
+}
+
+namespace WebKit {
+
+WebKitWebView* getViewFromFrame(WebKitWebFrame* frame)
+{
+    WebKitWebFramePrivate* priv = frame->priv;
+    return priv->webView;
+}
+
+WebCore::Frame* core(WebKitWebFrame* frame)
+{
+    if (!frame)
+        return 0;
+
+    WebKitWebFramePrivate* priv = frame->priv;
+    return priv ? priv->coreFrame : 0;
+}
+
+WebKitWebFrame* kit(WebCore::Frame* coreFrame)
+{
+    if (!coreFrame)
+        return 0;
+
+    ASSERT(coreFrame->loader());
+    WebKit::FrameLoaderClient* client = static_cast<WebKit::FrameLoaderClient*>(coreFrame->loader()->client());
+    return client ? client->webFrame() : 0;
+}
+
 }
