@@ -23,12 +23,12 @@
 #include "webkitwebhistoryitem.h"
 #include "webkitprivate.h"
 
-#include <glib.h>
-#include <glib/gi18n-lib.h>
-
-#include "CString.h"
 #include "HistoryItem.h"
 #include "PlatformString.h"
+#include "webkitwebhistoryitemprivate.h"
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+#include <wtf/text/CString.h>
 
 /**
  * SECTION:webkitwebhistoryitem
@@ -54,16 +54,14 @@ using namespace WebKit;
 struct _WebKitWebHistoryItemPrivate {
     WebCore::HistoryItem* historyItem;
 
-    WebCore::CString title;
-    WebCore::CString alternateTitle;
-    WebCore::CString uri;
-    WebCore::CString originalUri;
+    WTF::CString title;
+    WTF::CString alternateTitle;
+    WTF::CString uri;
+    WTF::CString originalUri;
 
     gboolean disposed;
 };
 
-#define WEBKIT_WEB_HISTORY_ITEM_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_WEB_HISTORY_ITEM, WebKitWebHistoryItemPrivate))
-
 enum {
     PROP_0,
 
@@ -113,10 +111,10 @@ static void webkit_web_history_item_finalize(GObject* object)
     WebKitWebHistoryItem* webHistoryItem = WEBKIT_WEB_HISTORY_ITEM(object);
     WebKitWebHistoryItemPrivate* priv = webHistoryItem->priv;
 
-    priv->title = WebCore::CString();
-    priv->alternateTitle = WebCore::CString();
-    priv->uri = WebCore::CString();
-    priv->originalUri = WebCore::CString();
+    priv->title = WTF::CString();
+    priv->alternateTitle = WTF::CString();
+    priv->uri = WTF::CString();
+    priv->originalUri = WTF::CString();
 
     G_OBJECT_CLASS(webkit_web_history_item_parent_class)->finalize(object);
 }
@@ -217,7 +215,7 @@ static void webkit_web_history_item_class_init(WebKitWebHistoryItemClass* klass)
 
 static void webkit_web_history_item_init(WebKitWebHistoryItem* webHistoryItem)
 {
-    webHistoryItem->priv = WEBKIT_WEB_HISTORY_ITEM_GET_PRIVATE(webHistoryItem);
+    webHistoryItem->priv = G_TYPE_INSTANCE_GET_PRIVATE(webHistoryItem, WEBKIT_TYPE_WEB_HISTORY_ITEM, WebKitWebHistoryItemPrivate);
 }
 
 static void webkit_web_history_item_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec)
@@ -301,7 +299,7 @@ WebKitWebHistoryItem* webkit_web_history_item_new_with_data(const gchar* uri, co
     WebKitWebHistoryItemPrivate* priv = webHistoryItem->priv;
 
     WebCore::KURL historyUri(WebCore::KURL(), uri);
-    WebCore::String historyTitle = WebCore::String::fromUTF8(title);
+    WTF::String historyTitle = WTF::String::fromUTF8(title);
     RefPtr<WebCore::HistoryItem> item = WebCore::HistoryItem::create(historyUri, historyTitle, 0);
     priv->historyItem = item.release().releaseRef();
     webkit_history_item_add(webHistoryItem, priv->historyItem);
@@ -365,7 +363,7 @@ void webkit_web_history_item_set_alternate_title(WebKitWebHistoryItem* webHistor
 
     WebCore::HistoryItem* item = core(webHistoryItem);
 
-    item->setAlternateTitle(WebCore::String::fromUTF8(title));
+    item->setAlternateTitle(WTF::String::fromUTF8(title));
     g_object_notify(G_OBJECT(webHistoryItem), "alternate-title");
 }
 
@@ -433,14 +431,14 @@ gdouble webkit_web_history_item_get_last_visited_time(WebKitWebHistoryItem* webH
 }
 
 /**
- * webkit_web_history_item_copy :
+ * webkit_web_history_item_copy:
  * @web_history_item: a #WebKitWebHistoryItem
  *
  * Makes a copy of the item for use with other WebView objects.
  *
  * Since: 1.1.18
  *
- * Return value: the new #WebKitWebHistoryItem.
+ * Return value: (transfer full): the new #WebKitWebHistoryItem.
  */
 WebKitWebHistoryItem* webkit_web_history_item_copy(WebKitWebHistoryItem* self)
 {
@@ -469,7 +467,7 @@ G_CONST_RETURN gchar* webkit_web_history_item_get_target(WebKitWebHistoryItem* w
 
     g_return_val_if_fail(item, NULL);
 
-    WebCore::CString t = item->target().utf8();
+    WTF::CString t = item->target().utf8();
     return g_strdup(t.data());
 }
 
