@@ -48,8 +48,6 @@ struct _WebKitWebNavigationActionPrivate {
     gchar* targetFrame;
 };
 
-#define WEBKIT_WEB_NAVIGATION_ACTION_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_WEB_NAVIGATION_ACTION, WebKitWebNavigationActionPrivate))
-
 enum  {
     PROP_0,
 
@@ -118,7 +116,7 @@ static void webkit_web_navigation_action_set_property(GObject* object, guint pro
 
 static void webkit_web_navigation_action_init(WebKitWebNavigationAction* navigationAction)
 {
-    navigationAction->priv = WEBKIT_WEB_NAVIGATION_ACTION_GET_PRIVATE(navigationAction);
+    navigationAction->priv = G_TYPE_INSTANCE_GET_PRIVATE(navigationAction, WEBKIT_TYPE_WEB_NAVIGATION_ACTION, WebKitWebNavigationActionPrivate);
 }
 
 static void webkit_web_navigation_action_finalize(GObject* obj)
@@ -177,8 +175,8 @@ static void webkit_web_navigation_action_class_init(WebKitWebNavigationActionCla
     /**
      * WebKitWebNavigationAction:button:
      *
-     * The DOM identifier for the mouse button used to click. DOM button values
-     * are 0, 1 and 2 for left, middle and right buttons. If the action was not
+     * The GTK+ identifier for the mouse button used to click. Notice that GTK+ button values
+     * are 1, 2 and 3 for left, middle and right buttons, so they are DOM button values +1. If the action was not
      * initiated by a mouse click the value will be -1.
      *
      * Since: 1.0.3
@@ -310,9 +308,9 @@ void webkit_web_navigation_action_set_original_uri(WebKitWebNavigationAction* na
  * webkit_web_navigation_action_get_button:
  * @navigationAction: a #WebKitWebNavigationAction
  *
- * Returns the DOM identifier for the mouse button used to click.
- * DOM button values are 0, 1 and 2 for left, middle and right buttons.
- * If the action was not initiated by a mouse click, returns -1.
+ * The GTK+ identifier for the mouse button used to click. Notice that GTK+ button values
+ * are 1, 2 and 3 for left, middle and right buttons, so they are DOM button values +1. If the action was not
+ * initiated by a mouse click the value will be -1.
  *
  * Return value: the mouse button used to click
  *
@@ -369,3 +367,17 @@ static void webkit_web_navigation_action_set_target_frame(WebKitWebNavigationAct
     navigationAction->priv->targetFrame = g_strdup(targetFrame);
     g_object_notify(G_OBJECT(navigationAction), "target-frame");
 }
+
+namespace WebKit {
+
+WebKitWebNavigationReason kit(WebCore::NavigationType type)
+{
+    return (WebKitWebNavigationReason)type;
+}
+
+WebCore::NavigationType core(WebKitWebNavigationReason type)
+{
+    return static_cast<WebCore::NavigationType>(type);
+}
+
+}
