@@ -26,16 +26,15 @@
 #include "config.h"
 #include "webkitwebsettings.h"
 
+#include "FileSystem.h"
+#include "Language.h"
+#include "PluginDatabase.h"
 #include "webkitenumtypes.h"
 #include "webkitprivate.h"
 #include "webkitversion.h"
-
-#include "CString.h"
-#include "FileSystem.h"
-#include "PluginDatabase.h"
-#include "Language.h"
-#include "PlatformString.h"
-
+#include "webkitwebsettingsprivate.h"
+#include <wtf/text/CString.h>
+#include <wtf/text/StringConcatenate.h>
 #include <glib/gi18n-lib.h>
 #if OS(UNIX)
 #include <sys/utsname.h>
@@ -45,9 +44,9 @@
  * SECTION:webkitwebsettings
  * @short_description: Control the behaviour of a #WebKitWebView
  *
- * #WebKitWebSettings can be applied to a #WebKitWebView to control
- * the to be used text encoding, color, font sizes, printing mode,
- * script support, loading of images and various other things.
+ * #WebKitWebSettings can be applied to a #WebKitWebView to control text encoding, 
+ * color, font sizes, printing mode, script support, loading of images and various other things. 
+ * After creation, a #WebKitWebSettings object contains default settings. 
  *
  * <informalexample><programlisting>
  * /<!-- -->* Create a new websettings and disable java script *<!-- -->/
@@ -94,6 +93,7 @@ struct _WebKitWebSettingsPrivate {
     gboolean enable_html5_local_storage;
     gboolean enable_xss_auditor;
     gboolean enable_spatial_navigation;
+    gboolean enable_frame_flattening;
     gchar* user_agent;
     gboolean javascript_can_open_windows_automatically;
     gboolean javascript_can_access_clipboard;
@@ -108,6 +108,8 @@ struct _WebKitWebSettingsPrivate {
     gboolean enable_page_cache;
     gboolean auto_resize_window;
     gboolean enable_java_applet;
+    gboolean enable_hyperlink_auditing;
+    gboolean enable_fullscreen;
 };
 
 #define WEBKIT_WEB_SETTINGS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_WEB_SETTINGS, WebKitWebSettingsPrivate))
@@ -144,6 +146,7 @@ enum {
     PROP_ENABLE_HTML5_LOCAL_STORAGE,
     PROP_ENABLE_XSS_AUDITOR,
     PROP_ENABLE_SPATIAL_NAVIGATION,
+    PROP_ENABLE_FRAME_FLATTENING,
     PROP_USER_AGENT,
     PROP_JAVASCRIPT_CAN_OPEN_WINDOWS_AUTOMATICALLY,
     PROP_JAVASCRIPT_CAN_ACCESS_CLIPBOARD,
@@ -157,62 +160,70 @@ enum {
     PROP_ENABLE_SITE_SPECIFIC_QUIRKS,
     PROP_ENABLE_PAGE_CACHE,
     PROP_AUTO_RESIZE_WINDOW,
-    PROP_ENABLE_JAVA_APPLET
+    PROP_ENABLE_JAVA_APPLET,
+    PROP_ENABLE_HYPERLINK_AUDITING,
+    PROP_ENABLE_FULLSCREEN
 };
 
 // Create a default user agent string
 // This is a liberal interpretation of http://www.mozilla.org/build/revised-user-agent-strings.html
 // See also http://developer.apple.com/internet/safari/faq.html#anchor2
-static String webkit_get_user_agent()
+static String webkitPlatform()
 {
-    gchar* platform;
-    gchar* osVersion;
-
 #if PLATFORM(X11)
-    platform = g_strdup("X11");
+    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("X11")));
 #elif OS(WINDOWS)
-    platform = g_strdup("Windows");
+    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("Windows")));
 #elif PLATFORM(MAC)
-    platform = g_strdup("Macintosh");
+    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("Macintosh")));
 #elif defined(GDK_WINDOWING_DIRECTFB)
-    platform = g_strdup("DirectFB");
+    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("DirectFB")));
 #else
-    platform = g_strdup("Unknown");
+    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("Unknown")));
 #endif
 
+    return uaPlatform;
+}
+
+static String webkitOSVersion()
+{
    // FIXME: platform/version detection can be shared.
 #if OS(DARWIN)
 
 #if CPU(X86)
-    osVersion = g_strdup("Intel Mac OS X");
+    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (String("Intel Mac OS X")));
 #else
-    osVersion = g_strdup("PPC Mac OS X");
+    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (String("PPC Mac OS X")));
 #endif
 
 #elif OS(UNIX)
+    DEFINE_STATIC_LOCAL(String, uaOSVersion, (String()));
+
+    if (!uaOSVersion.isEmpty())
+        return uaOSVersion;
+
     struct utsname name;
     if (uname(&name) != -1)
-        osVersion = g_strdup_printf("%s %s", name.sysname, name.machine);
+        uaOSVersion = makeString(name.sysname, ' ', name.machine);
     else
-        osVersion = g_strdup("Unknown");
-
+        uaOSVersion = String("Unknown");
 #elif OS(WINDOWS)
-    // FIXME: Compute the Windows version
-    osVersion = g_strdup("Windows");
-
+    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (String("Windows")));
 #else
-    osVersion = g_strdup("Unknown");
+    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (String("Unknown")));
 #endif
 
+    return uaOSVersion;
+}
+
+String webkitUserAgent()
+{
     // We mention Safari since many broken sites check for it (OmniWeb does this too)
     // We re-use the WebKit version, though it doesn't seem to matter much in practice
 
-    DEFINE_STATIC_LOCAL(const String, uaVersion, (String::format("%d.%d+", WEBKIT_USER_AGENT_MAJOR_VERSION, WEBKIT_USER_AGENT_MINOR_VERSION)));
-    DEFINE_STATIC_LOCAL(const String, staticUA, (String::format("Mozilla/5.0 (%s; U; %s; %s) AppleWebKit/%s (KHTML, like Gecko) Safari/%s",
-                                                                platform, osVersion, defaultLanguage().utf8().data(), uaVersion.utf8().data(), uaVersion.utf8().data())));
-
-    g_free(osVersion);
-    g_free(platform);
+    DEFINE_STATIC_LOCAL(const String, uaVersion, (makeString(String::number(WEBKIT_USER_AGENT_MAJOR_VERSION), '.', String::number(WEBKIT_USER_AGENT_MINOR_VERSION), '+')));
+    DEFINE_STATIC_LOCAL(const String, staticUA, (makeString("Mozilla/5.0 (", webkitPlatform(), "; U; ", webkitOSVersion(), "; ", defaultLanguage(), ") AppleWebKit/", uaVersion) +
+                                                 makeString(" (KHTML, like Gecko) Version/5.0 Safari/", uaVersion)));
 
     return staticUA;
 }
@@ -569,7 +580,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                     PROP_ENABLE_XSS_AUDITOR,
                                     g_param_spec_boolean("enable-xss-auditor",
                                                          _("Enable XSS Auditor"),
-                                                         _("Whether to enable teh XSS auditor"),
+                                                         _("Whether to enable the XSS auditor"),
                                                          TRUE,
                                                          flags));
     /**
@@ -591,6 +602,25 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          _("Whether to enable Spatial Navigation"),
                                                          FALSE,
                                                          flags));
+    /**
+    * WebKitWebSettings:enable-frame-flattening
+    *
+    * Whether to enable the Frame Flattening. With this setting each subframe is expanded
+    * to its contents, which will flatten all the frames to become one scrollable page.
+    * On touch devices, it is desired to not have any scrollable sub parts of the page as
+    * it results in a confusing user experience, with scrolling sometimes scrolling sub parts
+    * and at other times scrolling the page itself. For this reason iframes and framesets are
+    * barely usable on touch devices.
+    *
+    * Since: 1.3.5
+    */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_FRAME_FLATTENING,
+                                    g_param_spec_boolean("enable-frame-flattening",
+                                                         _("Enable Frame Flattening"),
+                                                         _("Whether to enable Frame Flattening"),
+                                                         FALSE,
+                                                         flags));
     /**
      * WebKitWebSettings:user-agent:
      *
@@ -607,7 +637,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                     g_param_spec_string("user-agent",
                                                         _("User Agent"),
                                                         _("The User-Agent string used by WebKitGtk"),
-                                                        webkit_get_user_agent().utf8().data(),
+                                                        webkitUserAgent().utf8().data(),
                                                         flags));
 
     /**
@@ -660,6 +690,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
 
     COMPILE_ASSERT(static_cast<int>(WEBKIT_EDITING_BEHAVIOR_MAC) == static_cast<int>(WebCore::EditingMacBehavior), editing_behavior_type_mac_match);
     COMPILE_ASSERT(static_cast<int>(WEBKIT_EDITING_BEHAVIOR_WINDOWS) == static_cast<int>(WebCore::EditingWindowsBehavior), editing_behavior_type_windows_match);
+    COMPILE_ASSERT(static_cast<int>(WEBKIT_EDITING_BEHAVIOR_UNIX) == static_cast<int>(WebCore::EditingUnixBehavior), editing_behavior_type_unix_match);
 
     /**
     * WebKitWebSettings:editing-behavior
@@ -684,7 +715,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                       _("Editing behavior"),
                                                       _("The behavior mode to use in editing mode"),
                                                       WEBKIT_TYPE_EDITING_BEHAVIOR,
-                                                      WEBKIT_EDITING_BEHAVIOR_MAC,
+                                                      WEBKIT_EDITING_BEHAVIOR_UNIX,
                                                       flags));
 
     /**
@@ -857,12 +888,36 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          TRUE,
                                                          flags));
 
+    /**
+    * WebKitWebSettings:enable-hyperlink-auditing:
+    *
+    * Enable or disable support for <a ping>.
+    *
+    * Since: 1.2.5
+    */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_HYPERLINK_AUDITING,
+                                    g_param_spec_boolean("enable-hyperlink-auditing",
+                                                         _("Enable Hyperlink Auditing"),
+                                                         _("Whether <a ping> should be able to send pings"),
+                                                         FALSE,
+                                                         flags));
+
+    /* Undocumented for now */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_FULLSCREEN,
+                                    g_param_spec_boolean("enable-fullscreen",
+                                                         _("Enable Fullscreen"),
+                                                         _("Whether the Mozilla style API should be enabled."),
+                                                         FALSE,
+                                                         flags));
+
     g_type_class_add_private(klass, sizeof(WebKitWebSettingsPrivate));
 }
 
 static void webkit_web_settings_init(WebKitWebSettings* web_settings)
 {
-    web_settings->priv = WEBKIT_WEB_SETTINGS_GET_PRIVATE(web_settings);
+    web_settings->priv = G_TYPE_INSTANCE_GET_PRIVATE(web_settings, WEBKIT_TYPE_WEB_SETTINGS, WebKitWebSettingsPrivate);
 }
 
 static EnchantBroker* get_enchant_broker()
@@ -905,6 +960,13 @@ static void webkit_web_settings_finalize(GObject* object)
     G_OBJECT_CLASS(webkit_web_settings_parent_class)->finalize(object);
 }
 
+static void getAvailableDictionariesCallback(const char* const languageTag, const char* const, const char* const, const char* const, void* data)
+{
+    Vector<CString>* dicts = static_cast<Vector<CString>*>(data);
+
+    dicts->append(languageTag);
+}
+
 static void webkit_web_settings_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec)
 {
     WebKitWebSettings* web_settings = WEBKIT_WEB_SETTINGS(object);
@@ -1019,6 +1081,14 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
             if (enchant_broker_dict_exists(broker, language)) {
                 dict = enchant_broker_request_dict(broker, language);
                 spellDictionaries = g_slist_append(spellDictionaries, dict);
+            } else {
+                // No dictionaries selected, we get one from the list
+                Vector<CString> allDictionaries;
+                enchant_broker_list_dicts(broker, getAvailableDictionariesCallback, &allDictionaries);
+                if (!allDictionaries.isEmpty()) {
+                    dict = enchant_broker_request_dict(broker, allDictionaries[0].data());
+                    spellDictionaries = g_slist_append(spellDictionaries, dict);
+                }
             }
         }
         g_slist_foreach(priv->enchant_dicts, free_spell_checking_language, 0);
@@ -1031,10 +1101,13 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
     case PROP_ENABLE_SPATIAL_NAVIGATION:
         priv->enable_spatial_navigation = g_value_get_boolean(value);
         break;
+    case PROP_ENABLE_FRAME_FLATTENING:
+        priv->enable_frame_flattening = g_value_get_boolean(value);
+        break;
     case PROP_USER_AGENT:
         g_free(priv->user_agent);
         if (!g_value_get_string(value) || !strlen(g_value_get_string(value)))
-            priv->user_agent = g_strdup(webkit_get_user_agent().utf8().data());
+            priv->user_agent = g_strdup(webkitUserAgent().utf8().data());
         else
             priv->user_agent = g_strdup(g_value_get_string(value));
         break;
@@ -1077,6 +1150,12 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
     case PROP_ENABLE_JAVA_APPLET:
         priv->enable_java_applet = g_value_get_boolean(value);
         break;
+    case PROP_ENABLE_HYPERLINK_AUDITING:
+        priv->enable_hyperlink_auditing = g_value_get_boolean(value);
+        break;
+    case PROP_ENABLE_FULLSCREEN:
+        priv->enable_fullscreen = g_value_get_boolean(value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -1176,6 +1255,9 @@ static void webkit_web_settings_get_property(GObject* object, guint prop_id, GVa
     case PROP_ENABLE_SPATIAL_NAVIGATION:
         g_value_set_boolean(value, priv->enable_spatial_navigation);
         break;
+    case PROP_ENABLE_FRAME_FLATTENING:
+        g_value_set_boolean(value, priv->enable_frame_flattening);
+        break;
     case PROP_USER_AGENT:
         g_value_set_string(value, priv->user_agent);
         break;
@@ -1218,6 +1300,12 @@ static void webkit_web_settings_get_property(GObject* object, guint prop_id, GVa
     case PROP_ENABLE_JAVA_APPLET:
         g_value_set_boolean(value, priv->enable_java_applet);
         break;
+    case PROP_ENABLE_HYPERLINK_AUDITING:
+        g_value_set_boolean(value, priv->enable_hyperlink_auditing);
+        break;
+    case PROP_ENABLE_FULLSCREEN:
+        g_value_set_boolean(value, priv->enable_fullscreen);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -1242,7 +1330,7 @@ WebKitWebSettings* webkit_web_settings_new()
  *
  * Copies an existing #WebKitWebSettings instance.
  *
- * Returns: a new #WebKitWebSettings instance
+ * Returns: (transfer full): a new #WebKitWebSettings instance
  **/
 WebKitWebSettings* webkit_web_settings_copy(WebKitWebSettings* web_settings)
 {
@@ -1277,6 +1365,7 @@ WebKitWebSettings* webkit_web_settings_copy(WebKitWebSettings* web_settings)
                  "enable-html5-local-storage", priv->enable_html5_local_storage,
                  "enable-xss-auditor", priv->enable_xss_auditor,
                  "enable-spatial-navigation", priv->enable_spatial_navigation,
+                 "enable-frame-flattening", priv->enable_frame_flattening,
                  "user-agent", webkit_web_settings_get_user_agent(web_settings),
                  "javascript-can-open-windows-automatically", priv->javascript_can_open_windows_automatically,
                  "javascript-can-access-clipboard", priv->javascript_can_access_clipboard,
@@ -1291,6 +1380,8 @@ WebKitWebSettings* webkit_web_settings_copy(WebKitWebSettings* web_settings)
                  "enable-page-cache", priv->enable_page_cache,
                  "auto-resize-window", priv->auto_resize_window,
                  "enable-java-applet", priv->enable_java_applet,
+                 "enable-hyperlink-auditing", priv->enable_hyperlink_auditing,
+                 "enable-fullscreen", priv->enable_fullscreen,
                  NULL));
 
     return copy;
