@@ -27,6 +27,7 @@
 #include <JavaScriptCore/JSBase.h>
 
 #include <webkit/webkitdefines.h>
+#include <webkit/webkitdom.h>
 #include <webkit/webkitwebbackforwardlist.h>
 #include <webkit/webkitwebframe.h>
 #include <webkit/webkitwebhistoryitem.h>
@@ -49,8 +50,21 @@ typedef enum {
     WEBKIT_NAVIGATION_RESPONSE_DOWNLOAD
 } WebKitNavigationResponse;
 
+/*  
+ * WebKitCacheModel:
+ * @WEBKIT_CACHE_MODEL_DEFAULT: The default cache model. This is
+ *   WEBKIT_CACHE_MODEL_WEB_BROWSER.
+ * @WEBKIT_CACHE_MODEL_DOCUMENT_VIEWER: Disable the cache completely, which 
+ *   substantially reduces memory usage. Useful for applications that only
+ *   access local files.
+ * @WEBKIT_CACHE_MODEL_WEB_BROWSER: Improve document load speed substantially
+ *   by caching previously viewed content.
+ * 
+ * Enum values used for determining the webview cache model.
+ */
 typedef enum {
-    WEBKIT_CACHE_MODEL_DOCUMENT_VIEWER = 1,
+    WEBKIT_CACHE_MODEL_DEFAULT,
+    WEBKIT_CACHE_MODEL_DOCUMENT_VIEWER,
     WEBKIT_CACHE_MODEL_WEB_BROWSER
 } WebKitCacheModel;
 
@@ -63,6 +77,28 @@ typedef enum
     WEBKIT_WEB_VIEW_TARGET_INFO_NETSCAPE_URL
 } WebKitWebViewTargetInfo;
 
+typedef enum
+{
+    WEBKIT_WEB_VIEW_VIEW_MODE_WINDOWED,
+    WEBKIT_WEB_VIEW_VIEW_MODE_FLOATING,
+    WEBKIT_WEB_VIEW_VIEW_MODE_FULLSCREEN,
+    WEBKIT_WEB_VIEW_VIEW_MODE_MAXIMIZED,
+    WEBKIT_WEB_VIEW_VIEW_MODE_MINIMIZED
+} WebKitWebViewViewMode;
+
+typedef enum
+{
+    WEBKIT_SELECTION_AFFINITY_UPSTREAM,
+    WEBKIT_SELECTION_AFFINITY_DOWNSTREAM,
+} WebKitSelectionAffinity;
+
+typedef enum
+{
+    WEBKIT_INSERT_ACTION_TYPED,
+    WEBKIT_INSERT_ACTION_PASTED,
+    WEBKIT_INSERT_ACTION_DROPPED,
+} WebKitInsertAction;
+
 struct _WebKitWebView {
     GtkContainer parent_instance;
 
@@ -127,12 +163,12 @@ struct _WebKitWebViewClass {
 
     void                       (* undo)                   (WebKitWebView        *web_view);
     void                       (* redo)                   (WebKitWebView        *web_view);
+    gboolean                   (* should_allow_editing_action) (WebKitWebView   *web_view);
 
     /* Padding for future expansion */
     void (*_webkit_reserved0) (void);
     void (*_webkit_reserved1) (void);
     void (*_webkit_reserved2) (void);
-    void (*_webkit_reserved3) (void);
 };
 
 WEBKIT_API GType
@@ -142,192 +178,195 @@ WEBKIT_API GtkWidget *
 webkit_web_view_new (void);
 
 WEBKIT_API G_CONST_RETURN gchar *
-webkit_web_view_get_title                       (WebKitWebView        *web_view);
+webkit_web_view_get_title                       (WebKitWebView        *webView);
 
 WEBKIT_API G_CONST_RETURN gchar *
-webkit_web_view_get_uri                         (WebKitWebView        *web_view);
+webkit_web_view_get_uri                         (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_set_maintains_back_forward_list (WebKitWebView        *web_view,
+webkit_web_view_set_maintains_back_forward_list (WebKitWebView        *webView,
                                                  gboolean              flag);
 
 WEBKIT_API WebKitWebBackForwardList *
-webkit_web_view_get_back_forward_list           (WebKitWebView        *web_view);
+webkit_web_view_get_back_forward_list           (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_go_to_back_forward_item         (WebKitWebView        *web_view,
+webkit_web_view_go_to_back_forward_item         (WebKitWebView        *webView,
                                                  WebKitWebHistoryItem *item);
 
 WEBKIT_API gboolean
-webkit_web_view_can_go_back                     (WebKitWebView        *web_view);
+webkit_web_view_can_go_back                     (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_can_go_back_or_forward          (WebKitWebView        *web_view,
+webkit_web_view_can_go_back_or_forward          (WebKitWebView        *webView,
                                                  gint                  steps);
 
 WEBKIT_API gboolean
-webkit_web_view_can_go_forward                  (WebKitWebView        *web_view);
+webkit_web_view_can_go_forward                  (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_go_back                         (WebKitWebView        *web_view);
+webkit_web_view_go_back                         (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_go_back_or_forward              (WebKitWebView        *web_view,
+webkit_web_view_go_back_or_forward              (WebKitWebView        *webView,
                                                  gint                  steps);
 
 WEBKIT_API void
-webkit_web_view_go_forward                      (WebKitWebView        *web_view);
+webkit_web_view_go_forward                      (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_stop_loading                    (WebKitWebView        *web_view);
+webkit_web_view_stop_loading                    (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_open                            (WebKitWebView        *web_view,
+webkit_web_view_open                            (WebKitWebView        *webView,
                                                  const gchar          *uri);
 
 WEBKIT_API void
-webkit_web_view_reload                          (WebKitWebView        *web_view);
+webkit_web_view_reload                          (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_reload_bypass_cache             (WebKitWebView        *web_view);
+webkit_web_view_reload_bypass_cache             (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_load_uri                        (WebKitWebView        *web_view,
+webkit_web_view_load_uri                        (WebKitWebView        *webView,
                                                  const gchar          *uri);
 
 WEBKIT_API void
-webkit_web_view_load_string                     (WebKitWebView        *web_view,
+webkit_web_view_load_string                     (WebKitWebView        *webView,
                                                  const gchar          *content,
                                                  const gchar          *mime_type,
                                                  const gchar          *encoding,
                                                  const gchar          *base_uri);
 
 WEBKIT_API void
-webkit_web_view_load_html_string                (WebKitWebView        *web_view,
+webkit_web_view_load_html_string                (WebKitWebView        *webView,
                                                  const gchar          *content,
                                                  const gchar          *base_uri);
 
 WEBKIT_API void
-webkit_web_view_load_request                    (WebKitWebView        *web_view,
+webkit_web_view_load_request                    (WebKitWebView        *webView,
                                                  WebKitNetworkRequest *request);
 
 WEBKIT_API gboolean
-webkit_web_view_search_text                     (WebKitWebView        *web_view,
+webkit_web_view_search_text                     (WebKitWebView        *webView,
                                                  const gchar          *text,
                                                  gboolean              case_sensitive,
                                                  gboolean              forward,
                                                  gboolean              wrap);
 
 WEBKIT_API guint
-webkit_web_view_mark_text_matches               (WebKitWebView        *web_view,
+webkit_web_view_mark_text_matches               (WebKitWebView        *webView,
                                                  const gchar          *string,
                                                  gboolean              case_sensitive,
                                                  guint                 limit);
 
 WEBKIT_API void
-webkit_web_view_set_highlight_text_matches      (WebKitWebView        *web_view,
+webkit_web_view_set_highlight_text_matches      (WebKitWebView        *webView,
                                                  gboolean              highlight);
 
 WEBKIT_API void
-webkit_web_view_unmark_text_matches             (WebKitWebView        *web_view);
+webkit_web_view_unmark_text_matches             (WebKitWebView        *webView);
 
 WEBKIT_API WebKitWebFrame *
-webkit_web_view_get_main_frame                  (WebKitWebView        *web_view);
+webkit_web_view_get_main_frame                  (WebKitWebView        *webView);
 
 WEBKIT_API WebKitWebFrame *
-webkit_web_view_get_focused_frame               (WebKitWebView        *web_view);
+webkit_web_view_get_focused_frame               (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_execute_script                  (WebKitWebView        *web_view,
+webkit_web_view_execute_script                  (WebKitWebView        *webView,
                                                  const gchar          *script);
 
 WEBKIT_API gboolean
-webkit_web_view_can_cut_clipboard               (WebKitWebView        *web_view);
+webkit_web_view_can_cut_clipboard               (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_can_copy_clipboard              (WebKitWebView        *web_view);
+webkit_web_view_can_copy_clipboard              (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_can_paste_clipboard             (WebKitWebView        *web_view);
+webkit_web_view_can_paste_clipboard             (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_cut_clipboard                   (WebKitWebView        *web_view);
+webkit_web_view_cut_clipboard                   (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_copy_clipboard                  (WebKitWebView        *web_view);
+webkit_web_view_copy_clipboard                  (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_paste_clipboard                 (WebKitWebView        *web_view);
+webkit_web_view_paste_clipboard                 (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_delete_selection                (WebKitWebView        *web_view);
+webkit_web_view_delete_selection                (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_has_selection                   (WebKitWebView        *web_view);
+webkit_web_view_has_selection                   (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_select_all                      (WebKitWebView        *web_view);
+webkit_web_view_select_all                      (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_get_editable                    (WebKitWebView        *web_view);
+webkit_web_view_get_editable                    (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_set_editable                    (WebKitWebView        *web_view,
+webkit_web_view_set_editable                    (WebKitWebView        *webView,
                                                  gboolean              flag);
 
 WEBKIT_API GtkTargetList *
-webkit_web_view_get_copy_target_list            (WebKitWebView        *web_view);
+webkit_web_view_get_copy_target_list            (WebKitWebView        *webView);
 
 WEBKIT_API GtkTargetList *
-webkit_web_view_get_paste_target_list           (WebKitWebView        *web_view);
+webkit_web_view_get_paste_target_list           (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_set_settings                    (WebKitWebView        *web_view,
+webkit_web_view_set_settings                    (WebKitWebView        *webView,
                                                  WebKitWebSettings    *settings);
 
 WEBKIT_API WebKitWebSettings *
-webkit_web_view_get_settings                    (WebKitWebView        *web_view);
+webkit_web_view_get_settings                    (WebKitWebView        *webView);
 
 WEBKIT_API WebKitWebInspector *
-webkit_web_view_get_inspector                   (WebKitWebView        *web_view);
+webkit_web_view_get_inspector                   (WebKitWebView        *webView);
 
 WEBKIT_API WebKitWebWindowFeatures*
-webkit_web_view_get_window_features             (WebKitWebView        *web_view);
+webkit_web_view_get_window_features             (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_can_show_mime_type              (WebKitWebView        *web_view,
+webkit_web_view_can_show_mime_type              (WebKitWebView        *webView,
                                                  const gchar          *mime_type);
 
 WEBKIT_API gboolean
-webkit_web_view_get_transparent                 (WebKitWebView        *web_view);
+webkit_web_view_get_transparent                 (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_set_transparent                 (WebKitWebView        *web_view,
+webkit_web_view_set_transparent                 (WebKitWebView        *webView,
                                                  gboolean              flag);
 
 WEBKIT_API gfloat
-webkit_web_view_get_zoom_level                  (WebKitWebView        *web_view);
+webkit_web_view_get_zoom_level                  (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_set_zoom_level                  (WebKitWebView        *web_view,
+webkit_web_view_set_zoom_level                  (WebKitWebView        *webView,
                                                  gfloat                zoom_level);
 
 WEBKIT_API void
-webkit_web_view_zoom_in                         (WebKitWebView        *web_view);
+webkit_web_view_zoom_in                         (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_zoom_out                        (WebKitWebView        *web_view);
+webkit_web_view_zoom_out                        (WebKitWebView        *webView);
 
 WEBKIT_API gboolean
-webkit_web_view_get_full_content_zoom           (WebKitWebView        *web_view);
+webkit_web_view_get_full_content_zoom           (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_set_full_content_zoom           (WebKitWebView        *web_view,
+webkit_web_view_set_full_content_zoom           (WebKitWebView        *webView,
                                                  gboolean              full_content_zoom);
 
 WEBKIT_API SoupSession*
 webkit_get_default_session                      (void);
 
+WEBKIT_API WebKitWebPluginDatabase *
+webkit_get_web_plugin_database                  (void);
+
 WEBKIT_API const gchar*
 webkit_web_view_get_encoding                    (WebKitWebView        * webView);
 
@@ -338,16 +377,23 @@ webkit_web_view_set_custom_encoding             (WebKitWebView        * webView,
 WEBKIT_API const char*
 webkit_web_view_get_custom_encoding             (WebKitWebView        * webView);
 
+WEBKIT_API void
+webkit_web_view_set_view_mode                   (WebKitWebView        *webView,
+                                                 WebKitWebViewViewMode mode);
+
+WEBKIT_API WebKitWebViewViewMode
+webkit_web_view_get_view_mode                   (WebKitWebView        *webView);
+
 WEBKIT_API void
 webkit_web_view_move_cursor                     (WebKitWebView        * webView,
                                                  GtkMovementStep        step,
                                                  gint                   count);
 
 WEBKIT_API WebKitLoadStatus
-webkit_web_view_get_load_status                 (WebKitWebView        *web_view);
+webkit_web_view_get_load_status                 (WebKitWebView        *webView);
 
 WEBKIT_API gdouble
-webkit_web_view_get_progress                    (WebKitWebView        *web_view);
+webkit_web_view_get_progress                    (WebKitWebView        *webView);
 
 WEBKIT_API void
 webkit_web_view_undo                            (WebKitWebView        *webView);
@@ -362,11 +408,11 @@ WEBKIT_API gboolean
 webkit_web_view_can_redo                        (WebKitWebView        *webView);
 
 WEBKIT_API void
-webkit_web_view_set_view_source_mode            (WebKitWebView        *web_view,
+webkit_web_view_set_view_source_mode            (WebKitWebView        *webView,
                                                  gboolean             view_source_mode);
 
 WEBKIT_API gboolean
-webkit_web_view_get_view_source_mode            (WebKitWebView        *web_view);
+webkit_web_view_get_view_source_mode            (WebKitWebView        *webView);
 
 WEBKIT_API WebKitHitTestResult*
 webkit_web_view_get_hit_test_result             (WebKitWebView        *webView,
@@ -381,6 +427,12 @@ webkit_set_cache_model                          (WebKitCacheModel     cache_mode
 WEBKIT_API WebKitCacheModel
 webkit_get_cache_model                          (void);
 
+WEBKIT_API WebKitDOMDocument *
+webkit_web_view_get_dom_document                (WebKitWebView        *webView);
+
+WEBKIT_API WebKitViewportAttributes*
+webkit_web_view_get_viewport_attributes         (WebKitWebView        *webView);
+
 G_END_DECLS
 
 #endif
