@@ -38,7 +38,7 @@
  * In the normal case one will use #webkit_web_view_get_window_features
  * to get the #WebKitWebWindowFeatures and then monitor the property
  * changes. Be aware that the #WebKitWebWindowFeatures might change
- * change before #WebKitWebView::web-view-ready signal is emitted.
+ * before #WebKitWebView::web-view-ready signal is emitted.
  * To be safe listen to the notify::window-features signal of the #WebKitWebView
  * and reconnect the signals whenever the #WebKitWebWindowFeatures of
  * a #WebKitWebView changes.
@@ -276,7 +276,7 @@ static void webkit_web_window_features_class_init(WebKitWebWindowFeaturesClass*
 
 static void webkit_web_window_features_init(WebKitWebWindowFeatures* web_window_features)
 {
-    web_window_features->priv = WEBKIT_WEB_WINDOW_FEATURES_GET_PRIVATE(web_window_features);
+    web_window_features->priv = G_TYPE_INSTANCE_GET_PRIVATE(web_window_features, WEBKIT_TYPE_WEB_WINDOW_FEATURES, WebKitWebWindowFeaturesPrivate);
 }
 
 static void webkit_web_window_features_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec)
